#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDbFromTemplate.sh,v 1.79 2005/05/09 22:34:45 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  15/10/96  created from ccdInstall.sh
#

#************************************************************************
#   NAME
#   ccdDbFromTemplate.sh - Store CCD configuration from template in DB
# 
#   SYNOPSIS
#   ccdDbFromTemplate.sh <DbFile> [<ins_root_dir>]
# 
#   DESCRIPTION
#   This shell script stores in the on-line database the values contained 
#   in a template configuration file.
#
#   <DbFile> Name of the .dbcfg template file.
#   <ins_root_dir> root directory for the instrument the CCD belongs to
#                  Default: $INS_ROOT (env. variable)
# 
#   FILES
#   $VLTROOT/config/ccd*.dbcfg 
#   Files containing on-line database initialisation values for CCD systems
#   known.
#   For technical CCDs there is one file for each system available. They
#   normally do not need to be changed by the user. The name is 
#   ccdTec<nn>.dbcfg whereby <nn> is the number of camera head. 
#   Example: head number 21 ---> ccdTec21.dbcfg
#   For scientific CCDs there is one file for each system to be delivered.
#   They normally have to be checked and possibly modified by experts of
#   the specific CCD system. The name is ccdSci<id>.dbcfg whereby <id> is
#   a symbolic name of the system.
#   Example: Fors CCD ---> ccdSciFors.dbcfg
#   
#
#   ENVIRONMENT
#   INTROOT  integration root directory
#   CCDNAME  CCD camera name
#   RTAPENV  CCD Rtap environment name
#   INS_ROOT default instrument root directory
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   > ccdDbFromTemplate.sh ccdTec21.dbcfg $INS_ROOT
#   Store in DB configuration for technical CCD head 21
#   > ccdDbFromTemplate.sh ccdSciFors.dbcfg
#   Store in DB configuration for Fors scientific CCD
#
#   SEE ALSO
#   ccdInstall.sh
#
#------------------------------------------------------------------------
if [ $# -lt 1 ]
then
    echo "Usage: ccdDbFromTemplate.sh <DbFile> [<ins_root_dir>]"
    exit 1
fi
template=$1

if [ $# -ge 2 ]
then
    if [ ! -d $2 ]
    then
	echo "$2 : Unknown file or directory"
	exit 1
    fi
    export INS_ROOT=$2
fi

#
# If database configuration file does not exist, create it from template
#
dir=$INS_ROOT/SYSTEM/COMMON/CONFIGFILES
if [ -d $dir ]
then
    cd $dir
    file=$CCDNAME.dbcfg
    if [ -f $file ]
    then
	mv -f $CCDNAME.dbcfg $CCDNAME.dbcfg.BAK
    fi
else
    echo "$dir : No such file or directory"
    exit 1
fi

relpath="SYSTEM/DETDATA"
imdir="/INS_ROOT/$relpath"
newimdir="$INS_ROOT/$relpath"
relpath="SYSTEM/COMMON/SETUPFILES/DET"
setdir="/INS_ROOT/$relpath"
newsetdir="$INS_ROOT/$relpath"
# remove comment lines, starting with ##
grep -v "##" $template > $file.tmp

# replace CWP line with the one needed for the camera used
# replace image and setup files directory according to INS_ROOT path
sed \
-e "s|@.*<alias>.*|@$RTAPENV<alias>$CCDNAME|g" \
-e "s|$setdir|$newsetdir|g" \
-e "s|$imdir|$newimdir|g" \
$file.tmp > $file.tmp1
mv -f $file.tmp1 $file
rm -f $file.tmp

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

#
# Copy contents of database backup file into database
#
dbRestore -f $dir/$file -co "<alias>$CCDNAME"
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___





