#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsKill.sh,v 4.7 2005/06/16 14:47:54 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  18/10/94  created
# alongino  10/04/97  UNIX kill command replaced with CCS KILL command
#

#************************************************************************
#   NAME
#   ccdDcsKill.sh - kills all programs belonging to CCD DCS
# 
#   SYNOPSIS
#   ccdDcsKill.sh [<camera>] [<WS env.>] [<LCU env.>]
# 
#   DESCRIPTION
#   This shell script kills all programs involved in CCD DCS module,
#   independently in which state they are or what they are doing.
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV).
#              If value is 0, no action on WS part of CCD sw is taken
#   <LCU env.> name of LCU environment (default env. var. CCDLENV)
#              If value is 0, no action on LCU part of CCD sw is taken
# 
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   This script should be used with care. By killing processes
#   'blindly', the system could remain in a dangerous state. To be used
#   only to recover when the system gets stuck.
#
#   EXAMPLES
#   > ccdDcsKill.sh ccdFors wte13 lte25
#         Kill all CCD DCS processes both at WS and LCU level
#         for camera "ccdFors", WS environment wte13, LCU environment lte25
#
#   > ccdDcsKill.sh ccdFors wte13 0
#         Kill all CCD DCS processes at WS level only
#         for camera "ccdFors", WS environment wte13
#
#   > ccdDcsKill.sh ccdFors 0 lte25
#         Kill all CCD DCS processes at LCU level
#         for camera "ccdFors", LCU environment lte25
#
#   SEE ALSO
#   ccdDcsStop.sh
#
#   BUGS
#------------------------------------------------------------------------
if [ $# -ge 1 ] 
then
    camera=$1
else
    camera=$CCDNAME
fi
if [ $# -ge 2 ]
then
    locEnv=$2
else
    locEnv=$RTAPENV
fi
if [ $# -ge 3 ]
then
    remEnv=$3
else
    remEnv=$CCDLENV
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

# log information that programs are going to be killed
if [ "$locEnv" != "" ]
then
    platform="WS"
    if [ "$remEnv" != "" ]
    then
	platform="WS and LCU"
    fi
elif [ "$remEnv" != "" ]
then
    platform="LCU"
else  
    platform=""
fi
logUserData "ccd" 0 "Killing all $platform processes for camera $camera"

#
# 1) kill all WS programs by sending a KILL command
#
if [ "$locEnv" != "" ]
then
#    echo ""
#    echo -n "Kill all CCD WS processes for camera $camera ... "
#    ps -aef | grep $camera | grep -v grep | grep -v .sh | awk '{print "kill -9",$2}' | sh
    process="ccdconCI_$camera"
    msgSend -n $locEnv $process "PING" "" >/dev/null  
    if [ $? = 0 ] 
    then
	msgSend -n $locEnv $process "EXIT" "" >/dev/null  
	if [ $? != 0 ] 
	then
	    echo "Kill CCD WS command interface process for camera $camera ... \c"
	    msgSend -n $locEnv $process "KILL" "" >/dev/null  
	    if [ $? != 0 ] 
	    then
		echo "FAILED"
	    else
		echo "done"
	    fi
	fi
    fi
    process="ccditWs_$camera"
    msgSend -n $locEnv $process "PING" "" >/dev/null  
    if [ $? = 0 ] 
    then
	echo "Kill CCD WS image transfer process for camera $camera ... \c"
	msgSend -n $locEnv $process "KILL" "" >/dev/null  
	if [ $? != 0 ]
	then
	    echo "FAILED"
	else
	    echo "done"
	fi
    fi
fi

#
# 2) kill all LCU programs by sending a KILL command
#
if [ "$remEnv" != "" ]
then
    process="ccdcon_$camera"
    msgSend -n $remEnv $process "PING" "" >/dev/null  
    if [ $? = 0 ] 
    then
	msgSend -n $remEnv $process "EXIT" "" >/dev/null
	if [ $? != 0 ]
	then
	    echo "Kill CCD LCU exposure coordination process for camera $camera ... \c"
	    msgSend -n $remEnv $process "KILL" "" >/dev/null
	    if [ $? != 0 ]
	    then
		echo "FAILED"
	    else
		echo "done"
	    fi
	fi
    fi
    process="ccdsht_$camera"
    msgSend -n $remEnv $process "PING" "" >/dev/null  
    if [ $? = 0 ] 
    then
	echo "Kill CCD LCU shutter process for camera $camera ... \c"
	msgSend -n $remEnv $process "KILL" "" >/dev/null
	if [ $? != 0 ]
	then
	    echo "FAILED"
	else
	    echo "done"
	fi
    fi
    process="ccdrdt_$camera"
    msgSend -n $remEnv $process "PING" "" >/dev/null  
    if [ $? = 0 ] 
    then
	echo "Kill CCD LCU readout process for camera $camera ... \c"
	msgSend -n $remEnv $process "KILL" "" >/dev/null
	if [ $? != 0 ]
	then
	    echo "FAILED"
	else
	    echo "done"
	fi
    fi
    process="ccdit_$camera"
    msgSend -n $remEnv $process "PING" "" >/dev/null  
    if [ $? = 0 ] 
    then
	echo "Kill CCD LCU image transfer process for camera $camera ... \c"
	msgSend -n $remEnv $process "KILL" "" >/dev/null
	if [ $? != 0 ]
	then
	    echo "FAILED"
	else
	    echo "done"
	fi
    fi
    process="ccdip_$camera"
    msgSend -n $remEnv $process "PING" "" >/dev/null  
    if [ $? = 0 ] 
    then
	echo "Kill CCD LCU image processing process for camera $camera ... \c"
	msgSend -n $remEnv $process "KILL" "" >/dev/null
	if [ $? != 0 ]
	then
	    echo "FAILED"
	else
	    echo "done"
	fi
    fi
    process="ccdtel_$camera"
    msgSend -n $remEnv $process "PING" "" >/dev/null  
    if [ $? = 0 ] 
    then
	echo "Kill CCD LCU telemetry process for camera $camera ... \c"
	msgSend -n $remEnv $process "KILL" "" >/dev/null
	if [ $? != 0 ]
	then
	    echo "FAILED"
	else
	    echo "done"
	fi
    fi
    process="ccdtmp_$camera"
    msgSend -n $remEnv $process "PING" "" >/dev/null  
    if [ $? = 0 ] 
    then
	echo "Kill CCD LCU temperature process for camera $camera ... \c"
	msgSend $remEnv $process "KILL" "" >/dev/null
	if [ $? != 0 ]
	then
	    echo "FAILED"
	else
	    echo "done"
	fi
    fi
fi

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___


