#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsStart.sh,v 4.7 2005/06/16 14:47:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  18/10/94  created
# alongino  07/08/95  added INS_ROOT, load DB configuration values
# alongino  08/08/95  added loading of CDT tables
# alongino  23/08/95  added loading of CDT tables for LCU in simulation
#                     Replace ccdDcsKill with ccdDcsStop
# alongino  06/09/95  added ccdDcsLcuSim.sh
# alongino  01/02/96  incorporated ccdDcsWarmStart.sh
# alongino  29/02/96  enabling scan moved to ccdDcsWarmStart.sh
# alongino  01/07/96  final message moved to stand-alone panel
# alongino  04/07/96  load of CDTs removed: already done by cmdManager at start
# alongino  17/10/96  scan configuration removed (SPR 960543)
# alongino  09/12/96  added check of operational mode
#

#************************************************************************
#   NAME
#   ccdDcsStart.sh - startup of CCD DCS
# 
#   SYNOPSIS
#   ccdDcsStart.sh [<camera>] [<WS env.>] [<LCU env.>] [<INS root>] [kill]
# 
#   DESCRIPTION
#   This shell script performs a startup of CCD DCS.
#   No CCD stand-alone module is started.
#   To be run once at startup only!
#   It performs the following steps:
#     1 - Load database configuration values on WS
#     2 - Run warm startup (ccdDcsWarmStart.sh)
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV)
#              If value is 0, no action on WS part of CCD sw is taken
#   <LCU env.> name of LCU environment (default env. var. CCDLENV)
#              If value is 0, no action on LCU part of CCD sw is taken
#   <INS root> INS_ROOT environment variable
#   kill       kill all already running processes before starting          
# 
#   FILES
#
#   ENVIRONMENT
#   INTROOT  integration root directory
#   CCDNAME  default for camera name (e.g. ccdFors)
#   RTAPENV  default for WS local environment (e.g. wte13)
#   CCDLENV  default for LCU environment (e.g lte25) 
#   INS_ROOT default root directory for instrument data 
#   INS_USER default to SYSTEM if not defined
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#   If this script is run twice or more, the system may not behave properly
#   any more. In this case better to shutdown the whole system and restart it
#   again.
#
#   EXAMPLES
#   > ccdDcsStart.sh ccdFors wte13 lte25
#         Start the CCD DCS sw both at WS and LCU level
#         for camera "ccdFors", WS environment wte13, LCU environment lte25
#
#   > ccdDcsStart.sh ccdFors wte13 0
#         Start the CCD DCS sw at WS level only
#         for camera "ccdFors", WS environment wte13
#
#   > ccdDcsStart.sh ccdFors wte13 0 $INS_ROOT kill
#         Kill and restart the CCD DCS sw at WS level only
#         for camera "ccdFors", WS environment wte13
#
#   > ccdDcsStart.sh ccdFors 0 lte25
#         Start the CCD DCS sw at LCU level only
#         for camera "ccdFors", LCU environment lte25
#
#   SEE ALSO
#   ccdDcsStop.sh ccdDcsWarmStart.sh
#
#   BUGS
#------------------------------------------------------------------------
killCcd=""
if [ $# -ge 1 ]
then
    export CCDNAME=$1
fi
if [ $# -ge 2 ]
then
    export RTAPENV=$2
fi
if [ $# -ge 3 ]
then
    export CCDLENV=$3
fi
if [ $# -ge 4 ]
then
    export INS_ROOT=$4
fi
if [ $# -ge 5 ]
then
    killCcd=$5
fi
if [ "$INS_USER" != "" ]
then
    export INS_USER=SYSTEM
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

#
# 1 - Load database configuration values on WS
#     No write in LCU, since it should be done by the LCU itself at boot time
#
if [ "$RTAPENV" != "" ]
then
    ccdDcsDbInit.sh $CCDNAME $RTAPENV $INS_ROOT/$INS_USER/COMMON/CONFIGFILES
    if [ $? != 0 ] 
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
fi

#
# 2 - Update environment variables according to operational mode in the DB
#     
if [ "$CCDLENV" != "" ] 
then
    opModeStr=`dbRead "<alias>$CCDNAME.opMode"`
    opMode=`echo $opModeStr | awk '{print $4}'`
    if [ "$opMode" = "3" ]
    then
	export CCDLENV=""
    fi
fi

#
# 3 - Load Command Definition Tables
# Not needed: cmdManager with VLT sw JUN96 loads all CDTs it finds in
# INTROOT and VLTROOT. If a CDT is changed, then one has to run ccdDcsCdt.sh
# by hand, but this is a special development case and should not slow dwon
# every startup.
#
#if [ "$RTAPENV" != "" ]
#then
#    ccdDcsCdt.sh $CCDNAME
#    if [ $? != 0 ]
#    then
#	 export NO_ERR_DISPLAY=$noErrDisplay
#        exit 1
#    fi
#fi

#
# 4 - Add attributes to scan table
# Removed. See SPR 960543. Now it is part of the configuration procedure
#
#if [[ "$RTAPENV" != "" && "$CCDLENV" != "" ]]
#then 
#    ccdDcsScan.sh $CCDNAME $RTAPENV $CCDLENV a
#    if [ $? != 0 ]
#    then
#	 export NO_ERR_DISPLAY=$noErrDisplay
#        exit 1
#    fi
#fi

#
# 5 - Run warm startup
#
if [ "$RTAPENV" != "" ]
then
    ccdDcsWarmStart.sh $CCDNAME $RTAPENV $CCDLENV $killCcd
    if [ $? != 0 ]
    then
        export NO_ERR_DISPLAY=$noErrDisplay
        exit 1
    fi
fi

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___

