#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsStop.sh,v 4.7 2005/06/16 14:47:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  18/10/94  created
# alongino  29/02/96  added timeout to msgSend
# alongino  02/07/96  PING added to LCU processes as well
# alongino  03/07/96  check done on WS AND LCU processes before sending EXIT
# alongino  21/10/96  consider if NO_ERR_DISPLAY defined when started
# alongino  09/12/96  added check of operational mode
# P.Duhoux  04/03/98  SPR980014 + Added usage
#

#************************************************************************
#   NAME
#   ccdDcsStop.sh - shut-down CCD DCS
# 
#   SYNOPSIS
#   ccdDcsStop.sh [<camera>] [<WS env.>] [<LCU env.>] [kill]
# 
#   DESCRIPTION
#   This shell script performs a shut-down of CCD DCS.
#   If doess the following steps:
#     1 - Verify if the main process is running.
#     2 - Try to terminate CCD processes in a 'soft' way (command EXIT)
#     3 - Try to terminate CCD processes in a 'hard' way (command KILL) (opt.)
#     4 - Disable scanning of data from LCU to WS
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV).
#              If value is 0, no action on WS part of CCD sw is taken
#   <LCU env.> name of LCU environment (default env. var. CCDLENV)
#              If value is 0, no action on LCU part of CCD sw is taken
#   kill       kill all processes             
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. ccdFors)
#   RTAPENV  default for WS local environment (e.g. wte13)
#   CCDLENV  default for LCU environment (e.g lte25) 
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#   The "kill" options should be used with care. By killing processes
#   'blindly', the system could remain in a dangerous state. To be used
#   only to recover when the system gets stuck.
#
#   EXAMPLES
#   > ccdDcsStop.sh ccdFors wte13 lte25
#         Terminate in a 'soft' way the CCD sw both at WS and LCU level
#         for camera "ccdFors", WS environment wte13, LCU environment lte25
#
#   > ccdDcsStop.sh ccdFors wte13 lte25 kill
#         Terminate in a 'hard' way the CCD sw both at WS and LCU level
#         for camera "ccdFors", WS environment wte13, LCU environment lte25
#
#   > ccdDcsStop.sh ccdFors wte13 0 kill
#         Terminate in a 'hard' way the CCD sw at WS level only
#         for camera "ccdFors", WS environment wte13
#
#   > ccdDcsStop.sh ccdFors 0 lte25
#         Terminate in a 'soft' way the CCD sw at LCU level only
#         for camera "ccdFors", LCU environment lte25
#
#   SEE ALSO
#   ccdDcsStart.sh, ccdDcsKill.sh
#
#   BUGS
#------------------------------------------------------------------------
timeout=10000
killCcd=0
usage="Usage : ccdDcsStop.sh [<camera>] [<WS env.>] [<LCU env.>] [kill]"

if [ $# -ge 1 ] 
then
    camera=$1
else
    if [ "$CCDNAME" = "" ]
    then
	echo "$usage"
	echo "        Undefined CCD camera name"
	exit 1
    else
	camera=$CCDNAME
    fi
fi

if [ $# -ge 2 ]
then
    locEnv=$2
else
    if [ "$RTAPENV" = "" ]
    then
	echo "$usage"
	echo "        Undefined RTAP Environment"
	exit 1
    else
	locEnv=$RTAPENV
    fi
fi

if [ $# -ge 3 ]
then
    remEnv=$3
else
    remEnv=$CCDLENV
fi

if [ $# -ge 4 ]
then
    if [ "$4" = "kill" ]
    then
	killCcd=1
    fi
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

#
# 1 - Update environment variables according to operational mode in the DB
#     
if [ "$remEnv" != "" ]
then
    opModeStr=`dbRead "<alias>$camera.opMode"`
    opMode=`echo $opModeStr | awk '{print $4}'`

    if [ "$opMode" = "3" ]
    then
	remEnv=""
    else
	if [ "$remEnv" = "" ]
	then
	    echo "$usage"
	    echo "        Undefined LCU Environment"
	    exit 1
	fi
    fi
fi
#
# 2 - Verify if the main process is running.
# send PING to high level coordination process (ccdconCI_$ccdname)
#
if [[ "$remEnv" != "" || "$locEnv" != "" ]]
then
    wsState=1
    if [ "$locEnv" != "" ]
    then
	process="ccdconCI_$camera"
	destEnv=$locEnv
	lcuState=6
    else
	process="ccdcon_$camera"
	destEnv=$remEnv
	lcuState=1
    fi
    msgSend -n $destEnv $process "PING" "" $timeout >/dev/null
    if [ $? != 0 ] 
    then
	echo ""
	echo "Process $process environment $destEnv is NOT active"
	# It might be that LCU sw is running (e.g. after reboot). One must
	# try at least to terminate LCU sw
	if [ "$remEnv" != "" ]
	then
	    process="ccdcon_$camera"
	    destEnv=$remEnv
	    msgSend -n $destEnv $process "PING" "" $timeout >/dev/null
	    if [ $? != 0 ] 
	    then
		echo "Process $process environment $destEnv is NOT active"
		process=""
		destEnv=""
	    fi
	else
	    destEnv=""
	fi
	echo "Resetting operational state to OFF... \c"
	dbWrite "<alias>$camera.opStateLcu" $lcuState
	dbWrite "<alias>$camera.opStateWs"  $wsState
	echo "done"
	echo ""
    fi
fi

#
# 3 - Try to terminate CCD processes in a 'soft' way
# send EXIT to coordination process
#
if [ "$destEnv" != "" ]
then
    echo ""
    echo "Sending command EXIT to process $process environment $destEnv... \c"
    msgSend -n $destEnv $process "EXIT" "" $timeout >/dev/null
    if [ $? != 0 ]
    then
	echo "FAILED"
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
    echo "done"
    echo ""
fi

if [ "$locEnv" != "" ]
then
    process="ccditWs_$camera"
    msgSend -n $locEnv $process "PING" "" $timeout >/dev/null
    if [ $? = 0 ]
    then
	echo "Kill CCD WS image transfer process for camera $camera ... \c"
	msgSend -n $locEnv $process "KILL" "" >/dev/null  
	if [ $? != 0 ]
	then
	    echo "FAILED"
	    export NO_ERR_DISPLAY=$noErrDisplay
	    exit 1
	else
	    echo "done"
	fi
    fi
fi

#
# 4 - Now kill all processes
# 
if [ $killCcd = 1 ]
then
    if [[ "$locEnv" != "" || "$remEnv" != "" ]] 
    then
	ccdDcsKill.sh $camera $locEnv $remEnv
	if [ $? != 0 ]
	then
	    export NO_ERR_DISPLAY=$noErrDisplay
	    exit 1
	fi
    fi
fi

#
# 5 - Disable scanning of data from LCU to WS
#
if [[ "$locEnv" != "" && "$remEnv" != "" ]]
then
    ccdDcsScan.sh $camera $locEnv $remEnv d
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
        exit 1
    fi
fi

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

# ___oOo___
