#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdDcsWarmStart.sh,v 4.7 2005/06/16 14:47:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  01/02/96  created
# alongino  21/02/96  added check if LCU process started
# alongino  28/02/96  stack size for ccdconStartupCamera increased to 50000 bytes
# pduhoux   29/02/96  set task flag to VX_FP_TASK for ccdconStartupCamera
# alongino  29/02/96  added enabling of scan data from LCU to WS
# alongino  16/04/96  added timeout to msgSend
# alongino  10/06/96  ccdconStartupCamera syntax changed for LCC parsing
# alongino  02/07/96  VERSION replaced with PING for LCU processes as well
# pduhoux     /05/96  Added parameters for ccdconStartupCamera via msgSchedule
# alongino  18/10/96  added definition of INS_USER for backwards compatibility
# alongino  21/10/96  added definition of INS_HOST for remote INS_ROOT
#                     consider if NO_ERR_DISPLAY defined when started
# alongino  09/12/96  added check of operational mode
# alongino  10/01/97  bug with $argv[5] corrected
# pduhoux   03/11/97  increased startup timeout to 20s
# pduhoux   12/01/99  SPR980693: increased PING timeout to 10s
#

#************************************************************************
#   NAME
#   ccdDcsWarmStart.sh - warm startup of CCD DCS
# 
#   SYNOPSIS
#   ccdDcsWarmStart.sh [<camera>] [<WS env.>] [<LCU env.>] [kill]
# 
#   DESCRIPTION
#   This shell script performs a warm startup of CCD DCS.
#   No CCD stand-alone module is started.
#   To be run once at startup only!
#   It performs the following steps:
#
#     1 - Terminate all running processes. If requested, kill them
#     2 - Update environment variables according to operational mode in the DB
#     3 - Set LCU sw in appropriate simulation mode (at CCS Message System level)
#     4 - Schedule LCU processes
#     5 - Schedule WS processes
#     6 - Enable scanning of data from LCU to WS
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV)
#              If value is 0, no action on WS part of CCD sw is taken
#   <LCU env.> name of LCU environment (default env. var. CCDLENV)
#              If value is 0, no action on LCU part of CCD sw is taken
#   kill       kill all already running processes before starting          
# 
#   FILES
#
#   ENVIRONMENT
#   INTROOT  integration root directory
#   CCDNAME  default for camera name (e.g. ccdFors)
#   RTAPENV  default for WS local environment (e.g. wte13)
#   CCDLENV  default for LCU environment (e.g lte25)
#   INS_USER  default to SYSTEM if not defined
#   INS_HOST Host where the INS_ROOT directory is mounted. Default HOST
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#   If this script is run twice or more, the system may not behave properly
#   any more. In this case better to shutdown the whole system and restart it
#   again.
#
#   EXAMPLES
#   > ccdDcsWarmStart.sh ccdFors wte13 lte25
#         Start the CCD DCS sw both at WS and LCU level
#         for camera "ccdFors", WS environment wte13, LCU environment lte25
#
#   > ccdDcsWarmStart.sh ccdFors wte13 0
#         Start the CCD DCS sw at WS level only
#         for camera "ccdFors", WS environment wte13
#
#   > ccdDcsWarmStart.sh ccdFors wte13 0 kill
#         Kill and restart the CCD DCS sw at WS level only
#         for camera "ccdFors", WS environment wte13
#
#   > ccdDcsWarmStart.sh ccdFors 0 lte25
#         Start the CCD DCS sw at LCU level only
#         for camera "ccdFors", LCU environment lte25
#
#   SEE ALSO
#   ccdDcsStart.sh ccdDcsStop.sh
#
#   BUGS
#------------------------------------------------------------------------
killCcd=""

if [ $# -ge 1 ] 
then
    export CCDNAME=$1
fi

if [ $# -ge 2 ]
then
    export RTAPENV=$2
fi

if [ $# -ge 3 ]
then
    export CCDLENV=$3
fi

if [ $# -ge 4 ]
then
    killCcd=$4
fi

# If INS_USER is NOT defined, ccdconCI would crash on slx call
if [ "$INS_USER" = "" ]
then
    export INS_USER=SYSTEM
fi

# If INS_HOST is NOT defined, set it to local HOST
if [ "$INS_HOST" = "" ]
then
    export INS_HOST=$HOST
fi
noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

#
# 1 - Terminate all running processes. If requested, kill them
# 
if [[ "$RTAPENV" != "" || "$CCDLENV" != "" ]]
then
    # Since this is a clean-up procedure, possible failure on not running processes
    # must be ignored. The env. var. NO_ERR_DISPLAY must be set to avoid that
    # errDisplay blocks execution with dialog box
    ccdDcsStop.sh $CCDNAME $RTAPENV $CCDLENV $killCcd
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
fi

#
# 2 - Update environment variables according to operational mode in the DB
#     
if [ "$CCDLENV" != "" ]
then
    opModeStr=`dbRead "<alias>$CCDNAME.opMode"`
    opMode=`echo $opModeStr | awk '{print $4}'`
    if [ "$opMode" = "3" ]
    then
	export CCDLENV=""
    fi
fi
if [ "$CCDLENV" = "" ]
then
    export CCSSTATE=SIMULATION
fi

#
# 3 - Set LCU sw in appropriate simulation mode (at CCS Message System level)
#
ccdDcsLcuSim.sh $CCDNAME $RTAPENV $CCDLENV

#
# 5 - Schedule WS processes
#
if [ "$RTAPENV" != "" ]
then
    #
    # schedule program ccdconCI (high-level coordination) on WS
    #
    nohup ccdconCI $CCDNAME > /dev/null 2>&1 &
    if [ $? != 0 ]
    then
	echo "FAILED"
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
    
    #
    # schedule program ccditWs (image transfer) on WS
    #
    nohup ccditWs $CCDNAME > /dev/null 2>&1 &
    if [ $? != 0 ]
    then
	echo "FAILED"
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
    echo ""
    echo "Scheduling CCD WS processes for camera $CCDNAME ... \c"
    #
    # Check that processes registered to CCS (5 secs timeout)
    #
    notRegistered=1
    timeout=10
    time=1
    process="ccdconCI_$CCDNAME"
    # Timeout added to msgSend for consistency with LCU case. Better to avoid
    # script hanging due to any possible messy environment situation.
    # However, no case on WS registered yet (on the contrary of LCU !)
    msgSendTimeout=10000
    while [ $notRegistered = 1 ]
    do
	msgSend -n $RTAPENV $process "PING" "" $msgSendTimeout >/dev/null
	if [ $? = 0 ]
	then
	    notRegistered=0
	else
	    sleep 1
	    time=`expr $time + 1`
	fi
	if [ $time -gt $timeout ]
	then
	    ccdDcsStop.sh $CCDNAME $RTAPENV $CCDLENV $killCcd
	    export NO_ERR_DISPLAY=$noErrDisplay
	    echo "TIMEOUT"
	    exit 1
	fi
	if [[ $notRegistered = 0 && "$process" = "ccdconCI_$CCDNAME" ]]
	then
	    process="ccditWs_$CCDNAME"
	    notRegistered=1      
	fi
    done
    echo "done"
    echo ""
fi

#
# 4 - Schedule LCU processes
#
if [ "$CCDLENV" != "" ]
then
    #
    # If LCU environment specified, schedule program ccdconStartupCamera on LCU
    #

    # AL960610 LCC uses parser. Parameters of type string must be quoted.
    #          ccdconStartupCamera wants "$CCDNAME $INS_HOST $INS_ROOT" as one
    #          single string separated by blanks
    run="ccdconStartupCamera \"$CCDNAME $INS_HOST $INS_ROOT\""
    echo "Scheduling CCD LCU processes for camera $CCDNAME ... \c"
    msgSchedule $CCDLENV "$run" 100 8 50000 ccdStartup >/dev/null
    if [ $? != 0 ] 
    then
	echo "FAILED"
	ccdDcsStop.sh $CCDNAME $RTAPENV $CCDLENV $killCcd
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
    notRegistered=1
    # Normally it takes a few seconds (~ 3). 5 secs would be at the limit
    timeout=10
    time=1
    process="ccdcon_$CCDNAME"
    # Sometime the LCU process terminates abnormally and cannot reply. It is
    # therefore necessary to add a timeout to msgSend, otherwise this script,
    # and the panel calling it, if any, would hang forever
    sleep 5
    msgSendTimeout=10000
    while [ $notRegistered = 1 ]
    do
	msgSend -n $CCDLENV $process "PING" "" $msgSendTimeout >/dev/null
	if [ $? = 0 ]
	then
	    notRegistered=0
	else
	    sleep 1
	    time=`expr $time + 1`
	fi
	if [ $time -gt $timeout ]
	then
	    echo "TIMEOUT"
	    ccdDcsScan.sh $CCDNAME $RTAPENV $CCDLENV e
	    ccdDcsStop.sh $CCDNAME $RTAPENV $CCDLENV $killCcd
	    export NO_ERR_DISPLAY=$noErrDisplay
	    exit 1
	fi
    done
    echo "done"
    echo ""
fi

#
# 6 - Enable scanning of data from LCU to WS
#
if [[ "$RTAPENV" != "" && "$CCDLENV" != "" ]]
then 
    ccdDcsScan.sh $CCDNAME $RTAPENV $CCDLENV e
    if [ $? != 0 ]
    then
	ccdDcsStop.sh $CCDNAME $RTAPENV $CCDLENV $killCcd
	export NO_ERR_DISPLAY=$noErrDisplay
	exit 1
    fi
fi

export NO_ERR_DISPLAY=$noErrDisplay
exit 0
#
# ___oOo___
#
