#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdEnvFromTemplate.sh,v 1.79 2005/05/09 22:34:45 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  14/04/97  created
#

#************************************************************************
#   NAME
#   ccdEnvFromTemplate.sh - Build CCD stand-alone environments from template
# 
#   SYNOPSIS
#   ccdEnvFromTemplate.sh [<camera>] [<WS env.>] [<LCU env.>]
# 
#   DESCRIPTION
#   This shell script builds a WS and LCU environment for one single
#   CCD stand-alone system.
# 
#   FILES
#   $VLTDATA/ENVIRONMENTS/<WS env.> contains the file of the new WS env.
#   $VLTDATA/ENVIRONMENTS/<LCU env.> contains the file of the new LCU env.
#
#   ENVIRONMENT
#   INTROOT  integration root directory
#   HOST     name of WS host node (e.g. te13)
#   CCDNAME  default for camera name (e.g. ccdFors)
#   RTAPENV  default for WS local environment (e.g. wte13)
#   CCDLENV  default for LCU environment (e.g lte25)
#   VLTDATA  directory containing the ENVIRONMENTS definitions
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   > ccdEnvFromTemplate.sh ccdTEC wccd lte53
#   Build environments wccd (WS) and lte53 (LCU) containing camera ccdTEC
#   > ccdEnvFromTemplate.sh ccdTEC wccd
#   Build environment wccd (WS) containing camera ccdTEC. No LCU environment
#
#------------------------------------------------------------------------
if [ $# -ge 1 ]
then
    camera=$1
else
    camera=$CCDNAME
fi

if [ $# -ge 2 ]
then
    locEnv=$2
else
    locEnv=$RTAPENV
fi

if [ $# -ge 3 ]
then
    remEnv=$3
else
    remEnv=$CCDLENV
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

#
# Look for directory where template files are located (INTROOT or VLTROOT)
#
templDir=$VLTROOT/templates/forCCD
if [ "$INTROOT" != "" ]
then
    if [ -d $INTROOT/templates/forCCD ]
    then
	templDir=$INTROOT/templates/forCCD
    fi
fi

#
# Build WS environment
#
# stop environment
msgSend -n $locEnv msgServer "PING" "" 5000 >/dev/null
if [ $? = 0 ]
then
    echo "Environment $locEnv already running. Stopping it ... \c"
    vccEnvStop -v -e $locEnv -h $HOST
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	echo "FAILED"
	exit 1
    fi
    echo "done"
fi

# save old environment
cd $VLTDATA/ENVIRONMENTS
if [ -d $locEnv ]
then
    if [ -d $locEnv.tmp ]
    then
	rm -rf $locEnv.tmp
    fi
    mv -f $locEnv $locEnv.tmp
fi

# create minimum template
echo "Creating environment $locEnv ... \c"
vccEnvCreate -v -e $locEnv -h $HOST
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    echo "FAILED"
    exit 1
fi
echo "done"
    
# now add CCD definitions
cd $locEnv

# change names within DB files
cd dbl
sed -e "s|myccd|$camera|g" $templDir/wmyccd/ccddbl/DATABASE.db >! DATABASE.db
sed -e "s|lmyccd|$remEnv|g" $templDir/wmyccd/ccddbl/USER.db >! USER.db
make clean db

echo "Initializing environment $locEnv ... \c"
vccEnvInit -v -e $locEnv -h $HOST
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    echo "FAILED"
    exit 1
fi
echo "done"

echo "Starting environment $locEnv ... \c"
vccEnvStart -v -e $locEnv -h $HOST
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    echo "FAILED"
    exit 1
fi
echo "done"

#
# Build LCU environment
#
if [ "$CCDLENV" != "" ]
then
    cd $VLTDATA/ENVIRONMENTS
    if [ -d $remEnv ]
    then
	if [ -d $remEnv.tmp]
	then
	    rm -rf $remEnv.tmp
	fi
	mv -f $remEnv $remEnv.tmp
    fi

    echo "Creating environment $remEnv ... \c"
    vccEnvCreate -v -e $remEnv -h $HOST -s "$templDir/lmyccd" -w $locEnv -m "template"
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	echo "FAILED"
	exit 1
    fi
    echo "done"

    rm -rf $remEnv/ccddbl
    cd $remEnv/dbl
    sed -e "s|myccd|$camera|g" $templDir/lmyccd/ccddbl/DATABASE.db >! DATABASE.db
    sed -e "s|wmyccd|$locEnv|g" $templDir/lmyccd/ccddbl/USER.db >! USER.db
    make clean db
fi

# reboot the LCU. NOte that it works ONLY if the boot EPROM parameters do not
# need to be changed
echo "Rebooting LCU $remEnv ... \c"
lccBoot $remEnv 120000
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    echo "FAILED"
    exit 1
fi
echo "done"

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

#___oOo___
