#! /bin/ksh
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdInstall.sh,v 1.79 2005/05/09 22:34:44 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  25/09/95  created
# alongino  20/10/95  file $CCDNAME.dbos created
#                     added creation of INS_ROOT sub-directories needed
#                     by the CCD software
# alongino  16/02/96  files not binary moved from bin to config
#                     ccdStandAloneTemplate.dbos removed
#                     ccdDcsInstall.sh moved here
# alongino  22/02/96  simplified. Common files not needed by LCU sw
#                     remain in VLTROOT(INTROOT)/config. Not copied any more
# alongino  26/02/96  $CCDNAME.dbcfg first saved, then overwritten
# alongino  20/03/96  bug with absolute path for $template corrected
# alongino  20/05/96  added installation of setupHdTemplate (needed by oslx)
# alongino  15/10/96  DB storage moved in ccdDbFromTemplate.sh
#                     Dictionary copied in INS tree (needed by new slx/oslx)
# alongino  20/03/97  removed installation of setupHdTemplate (oslx MAY97)
# pduhoux   11/11/97  added dbWrite @$RTAPENV<alias>$CCDNAME.installed
#                     "`date "+%Y-%m-%dT%H:%M:%S.000"`"
# pduhoux   19/11/97  added option -u to UNIX command 'date' for UTC
#

#************************************************************************
#   NAME
#   ccdInstall.sh - Install CCD files in INS_ROOT
# 
#   SYNOPSIS
#   ccdInstall.sh <DbFile> [<ins_root_dir>]
# 
#   DESCRIPTION
#   This shell script installs all files needed to run a CCD system in the
#   root directory for the instrument it belongs to. Additionally, it
#   stores in the on-line database the values contained in configuration
#   files.
#
#   <DbFile> Name of the .dbcfg file in $VLTROOT/config containing DB 
#            init values.
#   <ins_root_dir> root directory for the instrument the CCD belongs to
#                  Default: $INS_ROOT (env. variable)
# 
#   FILES
#   $VLTROOT/config/ccd*.dbcfg 
#   Files containing on-line database initialisation values for CCD systems
#   known.
#   For technical CCDs there is one file for each system available. They
#   normally do not need to be changed by the user. The name is 
#   ccdTec<nn>.dbcfg whereby <nn> is the number of camera head. 
#   Example: head number 21 ---> ccdTec21.dbcfg
#   For scientific CCDs there is one file for each system to be delivered.
#   They normally have to be checked and possibly modified by experts of
#   the specific CCD system. The name is ccdSci<id>.dbcfg whereby <id> is
#   a symbolic name of the system.
#   Example: Fors CCD ---> ccdSciFors.dbcfg
#   
#
#   ENVIRONMENT
#   INTROOT  integration root directory
#   CCDNAME  CCD camera name
#   RTAPENV  CCD Rtap environment name
#   INS_ROOT default instrument root directory
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   > ccdInstall.sh ccdTec21.dbcfg $INS_ROOT
#   Install all what needed for technical CCD head 21
#   > ccdInstall.sh ccdSciFors.dbcfg
#   Install all what needed for Fors scientific CCD
#
#   SEE ALSO
#   INS common sw specification
#
#------------------------------------------------------------------------
if [ $# -lt 1 ]
then
    echo "Usage: ccdInstall.sh <DbFile> [<ins_root_dir>]"
    exit 1
fi
file=$1
template=$VLTROOT/config/$file
if [ "$INTROOT" != "" ]
then
    if [ -f $INTROOT/config/$file ]
    then
	template=$INTROOT/config/$file
    fi
fi

if [ $# -ge 2 ]
then
    export INS_ROOT=$2
fi

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

curdir=$cwd
#
# 1 - Create INS_ROOT sub-directories needed by the CCD software
#
cd $INS_ROOT
dir=SYSTEM
if [ ! -d $dir ]
then
    mkdir $dir
fi
dir=SYSTEM/Dictionary
if [ ! -d $dir ]
then
    mkdir $dir
fi
dir=SYSTEM/DETDATA
if [ ! -d $dir ]
then
    mkdir $dir
fi
dir=SYSTEM/COMMON
if [ ! -d $dir ]
then
    mkdir $dir
fi
dir=SYSTEM/COMMON/CONFIGFILES
if [ ! -d $dir ]
then
    mkdir $dir
fi
dir=SYSTEM/COMMON/SETUPFILES
if [ ! -d $dir ]
then
    mkdir $dir
fi
dir=SYSTEM/COMMON/SETUPFILES/DET
if [ ! -d $dir ]
then
    mkdir $dir
fi
dir=SYSTEM/COMMON/SETUPFILES/REF
if [ ! -d $dir ]
then
    mkdir $dir
fi

#
# 2 - Copy CCDDCS data dictionary
# 
cd $INS_ROOT/SYSTEM/Dictionary
file=ESO-VLT-DIC.CCDDCS
fullPath=$VLTROOT/config/$file
if [ "$INTROOT" != "" ]
then
    if [ -f $INTROOT/config/$file ]
    then
	fullPath=$INTROOT/config/$file
    fi
fi
rm -f $file
cp -f $fullPath ./$file

#
# 3 - Copy setup file templates
# 
cd $INS_ROOT/SYSTEM/COMMON/SETUPFILES/DET

# template for complete setup file
file=ccdSetupComplete.det
fullPath=$VLTROOT/config/$file
if [ "$INTROOT" != "" ]
then
    if [ -f $INTROOT/config/$file ]
    then
	fullPath=$INTROOT/config/$file
    fi
fi
rm -f $file
cp $fullPath ./$file

# CCD commands support setup files
setupFileList=`ls $VLTROOT/config/ccdCmd*.det`
if [ "$INTROOT" != "" ]
then
    if [ -f $INTROOT/config/ccdCmdBias.det ]
    then
	setupFileList=`ls $INTROOT/config/ccdCmd*.det`
    fi
fi
rm -f ccdCmd*.det
cp -f $setupFileList .

#
# 4 - Copy database configuration files 
#
dir=$INS_ROOT/SYSTEM/COMMON/CONFIGFILES
cd $dir

# copy file containing backup of database fixed values
file=ccdDcsFixed.dbcfg
fullPath=$VLTROOT/config/$file
if [ "$INTROOT" != "" ]
then
    if [ -f $INTROOT/config/$file ]
    then
	fullPath=$INTROOT/config/$file
    fi
fi
rm -f $file
cp -f $fullPath ./$file

# template files 
configFileList=`ls $VLTROOT/config/ccd*Template.dbcfg`
if [ "$INTROOT" != "" ]
then
    if [ -f $INTROOT/config/ccdTecTemplate.dbcfg ]
    then
	configFileList=`ls $INTROOT/config/ccd*Template.dbcfg`
    fi
fi
rm -f ccd*Template.dbcfg
cp -f $configFileList .

#
# 5 - copy contents of database backup files into database
#
dbRestore -f $dir/ccdDcsFixed.dbcfg -co "<alias>$CCDNAME"
if [ $? != 0 ]
then
    echo "ccdInstall.sh : Failure invoking 'dbRestore -f $dir/ccdDcsFixed.dbcfg -co \"<alias>$CCDNAME\"'"
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi
ccdDbFromTemplate.sh $template $INS_ROOT
if [ $? != 0 ]
then
    echo "ccdInstall.sh : Failure invoking 'ccdDbFromTemplate.sh $template $INS_ROOT'"
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

dbPath="@$RTAPENV<alias>$CCDNAME.installed"
dbWrite "$dbPath" "`date -u "+%Y-%m-%dT%H:%M:%S.000"`"
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

cd $curdir 

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

#___oOo___



