#! /bin/ksh
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccdosScan.sh,v 1.79 2005/05/09 22:34:44 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  05/06/96  created
# alongino  05/07/96  added readout.opState and readout.status
# alongino  18/10/96  added RtapForceSnap
# alongino  21/10/96  consider if NO_ERR_DISPLAY defined when started
# pduhoux   13/03/97  upgrade to new process structure for 2 windows
# pduhoux   08/04/97  added links to telemetry/temperature
# pduhoux   10/04/97  use CCS scanLinks utility (MAY97)
#                     added option e & d, removed option w
# pduhoux   15/04/97  modified functionality:
#                     a - scanDevOn + scanLinks -E
#                     e - scanDevOn
#                     d - nothing
#                     c - scanLinks -d
# pduhoux   2000-07-11 Replaced RtapForceSnap by dbForceSnap
#

#************************************************************************
#   NAME 
#   ccdosScan.sh - configure the DB values to be scanned from LCU for CCD OS
# 
#   SYNOPSIS
#   ccdosScan.sh [<camera>] [<WS env.>] [<LCU env.>] [<option>]
# 
#   DESCRIPTION
#   This shell script configures the Scan System to retrieve DB values
#   from the CCD LCU to WS (stand-alone part only).
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV)
#   <LCU env.> name of LCU environment (default env. var. CCDLENV)
#   <option>   a    add entries to scan list
#              c    clean entries from scan list
#              d    disable scanning from the LCU
#              e    enable scanning from the LCU
#              default: a 
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. ccdFors)
#   RTAPENV  default for WS local environment (e.g. wte13)
#   CCDLENV  default for LCU environment (e.g lte25) 
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#   1 - It is assumed here that CCD branch in the LCU database is attached
#       directly to the root point (not a sub-point of something else).
#   2 - If this script is run twice or more, the system may not behave properly
#       any more. In this case better to shutdown the whole system and restart 
#       it again.
#
#   EXAMPLES
#   ccdosScan.sh ccdFors wte13 lte25
#        Add entries for LCU lte25 and WS wte13 camera ccdFors
#   ccdosScan.sh ccdFors wte13 lte25 c
#        Clean entries for LCU lte25 and WS wte13 camera ccdFors
#   ccdosScan.sh ccdFors wte13 lte25 e
#        Enable entries for LCU lte25 and WS wte13 camera ccdFors
#
#   SEE ALSO
#   ccdDcsScan.sh
#
#   BUGS
#------------------------------------------------------------------------
add=0
delete=0
disable=0
enable=0
option=0
message=""
module="ccdos"

if [ $# -ge 1 ]
then
    camera=$1
else
    camera=$CCDNAME
fi

if [ $# -ge 2 ]
then
    locEnv=$2
else
    locEnv=$RTAPENV
fi

if [ $# -ge 3 ]
then
    remEnv=$3
else
    remEnv=$CCDLENV
fi

if [ $# -ge 4 ]
then
    option=$4
    if [ $option = "a" ]
    then
	add=1
    fi
    if [ $option = "c" ]
    then
	delete=1
    fi
    if [ $option = "d" ]
    then
	disable=1
    fi
    if [ $option = "e" ]
    then
	enable=1
    fi
else
    add=1
fi

export RTAPENV=$locEnv

noErrDisplay=$NO_ERR_DISPLAY
export NO_ERR_DISPLAY=""

option="-l $locEnv -e $remEnv -s <alias>$camera -r <alias>$camera"

file=ccdosScan.links
scanFile=$VLTROOT/config/$file
if [ "$INTROOT" != "" ]
then
    if [ -f $INTROOT/config/$file ]
    then
	scanFile=$INTROOT/config/$file
    fi
fi

#
# Configure scan links from specified scan device
#
if [ $add = 1 ]
then
    message="Enabling scan device $remEnv of CCD $camera DB data to $locEnv"
    echo ""
    echo "$message ... \c"
    scanDevOn $remEnv >/dev/null
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	echo "FAILURE"
	exit 1
    else
	echo "done"
	logUserData $module 0 "$message"
    fi

    message="Checking & Enabling scan links from $remEnv to $locEnv for CCD $camera stand-alone part"
    echo ""
    echo "$message ... \c"
    scanLinks -c -E $option -f $scanFile >/dev/null
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	echo "FAILURE"
	exit 1
    else
	echo "done"
	logUserData $module 0 "$message"
    fi
fi

#
# Clean scan links from specified scan device
#
if [ $delete = 1 ]
then
    message="Cleaning scan links from $remEnv to $locEnv for CCD $camera stand-alone part"
    echo ""
    echo "$message ... \c"
    scanLinks -c -d $option -f $scanFile -o >/dev/null
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	echo "FAILURE"
	exit 1
    else
	echo "done"
	logUserData $module 0 "$message"
    fi
fi

#
# Disable scanning from specified scan device
#
if [ $disable = 1 ]
then
    message="Disabling scanning of CCD $camera DB data from $remEnv to $locEnv stand-alone part"
#    echo ""
#    echo "$message ... \c"
#    scanLinksDisable -l $locEnv -r <alias>$camera $scanFile >/dev/null
#    if [ $? != 0 ]
#    then
#	export NO_ERR_DISPLAY=$noErrDisplay
#	echo "FAILURE"
#	exit 1
#    else
#	echo "done"
#	logUserData $module 0 "$message"
#    fi
fi  

#
# Enable scanning from specified scan device
#
if [ $enable = 1 ]
then
    message="Enabling scan device $remEnv of CCD $camera DB data to $locEnv"
    echo ""
    echo "$message ... \c"
    scanDevOn $remEnv >/dev/null
    if [ $? != 0 ]
    then
	export NO_ERR_DISPLAY=$noErrDisplay
	echo "FAILURE"
	exit 1
    else
	echo "done"
	logUserData $module 0 "$message"
    fi

#    message="Enabling scanning of CCD $camera DB data from $remEnv to $locEnv stand-alone part"
#    echo ""
#    echo "$message ... \c"
#    scanLinksEnable -l $locEnv -r <alias>$camera $scanFile >/dev/null
#    if [ $? != 0 ]
#    then
#	export NO_ERR_DISPLAY=$noErrDisplay
#	exit 1
#    else
#	echo "done"
#	logUserData $module 0 "$message"
#    fi
fi

dbForceSnap -e $locEnv    
if [ $? != 0 ]
then
    export NO_ERR_DISPLAY=$noErrDisplay
    exit 1
fi

export NO_ERR_DISPLAY=$noErrDisplay
exit 0

#
# __oOo___
#
