#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsDbInit.sh,v 2.130 2005/09/08 12:50:18 vltsccm Exp $"
#
# who          when      what
# --------     --------  ----------------------------------------------
# A.Balestra   04/06/97  created (ported from CCD software)
# A.Balestra   25/10/04  Now imageDir is taken from environment (FCD_IMGDIR) or,
#                        if the db is set to "", to the usual default 
#                        ($INS_ROOT/$INS_USER/DETADATA). See SPR no. 
#                        VLTSW20040320
# A.Balestra   27/10/04  Temporary removed check on db point for imageDirectory
#                        null as backward incompatible
# A.Balestra   28/10/04  Now imageDir is taken from environment variable 
#                        FCD_IMGDIR unless FCD_IMGDIR is null or not defined. 
#                        In these cases $INS_ROOT/$INS_USER/DETADATA is used
#

#************************************************************************
#   NAME
#   fcdDcsDbInit.sh - Initialise database values for the FIERA DCS part
# 
#   SYNOPSIS
#   fcdDcsDbInit.sh [<camera>] [<env>] [<backup_dir>]
# 
#   DESCRIPTION
#   This shell script initialises some values in the database needed at 
#   startup by the FIERA software.
#
#   <camera>    camera name, also root point in DB (default env. var. CCDNAME)
#   <env>       environment where to write (default env. var. RTAPENV)
#   <bckup_dir> directory where backup files are looked for 
#               (default $INS_ROOT/$INS_USER/COMMON/CONFIGFILES)
# 
#   FILES
#   fcdDcsFixed.dbcfg  contains fixed values for all cameras
#   <camera>.dbcfg     contains values for <camera>
#
#   ENVIRONMENT
#   CCDNAME   default for camera name (e.g. fcdFors)
#   RTAPENV   default environment (e.g.wte16)
#   INS_ROOT  default for instrument root directory
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   fcdDcsDbInit.sh fcdFors wte16 $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#
#   SEE ALSO
#
#   BUGS
#------------------------------------------------------------------------
#
# WS part
#
#
# set environment variables (if not yet done)
#
if !($?INS_USER) then
  setenv INS_USER SYSTEM
endif

if ($#argv >= 1) then
  setenv CCDNAME $argv[1]
endif
if ($#argv >= 2) then
  set environ = $argv[2]
else
  set environ = $RTAPENV
endif
if ($#argv >= 3) then
  set bckdir = $argv[3]
else if ($?INS_ROOT) then
  set bckdir = $INS_ROOT/$INS_USER/COMMON/CONFIGFILES
else
  echo $INS_ROOT
  exit 1
endif

# Note: \c is not standard and on Sun it does not work
echo ""
echo "Load DB configuration in environment $environ for camera $CCDNAME"
#
# Restore CCD DCS fixed configuration values from the backup file
#
set bckfile = $bckdir/fcdDcsFixed.dbcfg
echo -n "Loading $bckfile .... "
dbRestore -f $bckfile -co "@$environ<alias>$CCDNAME"
if ($status != 0) then
  echo "FAILED"
  exit 1
endif
echo ""

#
# Restore FIERA DCS system-dependant  configuration values from the backup file
#
set bckfile = $bckdir/$CCDNAME.dbcfg
echo -n "Loading $bckfile .... "
dbRestore -f $bckfile -co "@$environ<alias>$CCDNAME"
if ($status != 0) then
  echo "FAILED"
  exit 1
endif
echo ""

#
# Ensure that the directories in which the SETUP and DET files will
# be are set up with this INS_ROOT
#
dbWrite "<alias>${CCDNAME}:observations.setupDirectory" "$INS_ROOT/$INS_USER/COMMON/SETUPFILES/DET"

#
#
# if FCD_IMGDIR is set and not null, set imagedir to its value and also
# check if it is a valid directory
# if FCD_IMGDIR is not set or it is set to null then set imageDirectory to 
# default
if (($?FCD_IMGDIR)) then
  if ($FCD_IMGDIR != "") then
    if (! -d $FCD_IMGDIR) then
      echo "$FCD_IMGDIR is not a valid directory for images"
      echo "FAILED"
      exit 1
    endif
    dbWrite "<alias>${CCDNAME}:images.imageDirectory" "$FCD_IMGDIR"
  else
    # Set image dorectory to the default
    dbWrite "<alias>${CCDNAME}:images.imageDirectory" "$INS_ROOT/$INS_USER/DETDATA"
  endif
else 
  # Set image dorectory to the default
  dbWrite "<alias>${CCDNAME}:images.imageDirectory" "$INS_ROOT/$INS_USER/DETDATA"
endif

echo "done"
echo ""
