#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsScan.sh,v 2.130 2005/09/08 12:50:18 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  04/06/97  created (ported from CCD software)
# ccumani   14/05/98  header for manpage modified
# abalestr  04/08/00  force snap using rtap or ccs_lite depending on RTAPROOT
# abalestr  05/12/00  force snap using dbForceSnap that works both on CCS 
#                     and CCS_LITE
#

#************************************************************************
#   NAME 
#   fcdDcsScan.sh - configure the DB values to be scanned from SLCU for
#   FIERA DCS
# 
#   SYNOPSIS
#   fcdDcsScan.sh [<camera>] [<WS env.>] [<SLCU env.>] [<option>]
# 
#   DESCRIPTION
#   This shell script configures the Scan System to retrieve DB values
#   for the FIERA SLCU to WS.
#
#   <camera>   camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>  name of workstation environment (default env. var. RTAPENV)
#   <SLCU env.> name of SLCU environment (default env. var. CCDLENV)
#   <option>   a (default) - add entries to scan list
#              c           - clean entries from scan list
#              d           - disable scanning from the SLCU
#              e           - enable scanning from the SLCU
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. myccd)
#   RTAPENV  default for WS local environment (e.g. myws)
#   CCDLENV  default for SLCU environment (e.g. myfiera) 
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#   1 - It is assumed here that FIERA branch in the SLCU database is attached
#       directly to the root point (not a sub-point of something else).
#   2 - If this script is run twice or more, the system may not behave properly
#       any more. In this case better to shutdown the whole system and restart 
#       it again.
#
#   EXAMPLES
#   fcdDcsScan.sh myccd myws myfiera
#        Add entries for camera "myccd", SLCU environment "myfiera",
#        WS environment "myws"
#   fcdDcsScan.sh myccd myws myfiera c
#        Clean entries for camera "myccd", SLCU environment "myfiera",
#        WS environment "myws"
#   fcdDcsScan.sh 0 myws myfiera d
#        Disable scanning from SLCU environment "myfiera" to
#        WS environment "myws"
#   fcdDcsScan.sh 0 myws myfiera e
#        Enable scanning from SLCU environment "myfiera" to
#        WS environment "myws"
#
#   SEE ALSO
#
#   BUGS
#------------------------------------------------------------------------
set add = 0
set delete = 0
set disable = 0
set enable = 0
set option = 0
set message = ""
set module = fcdws

if ($#argv >= 1) then
  set camera = $argv[1]
else
  set camera = $CCDNAME
endif

if ($#argv >= 2) then
  set locEnv = $argv[2]
else
  set locEnv = $RTAPENV
endif

if ($#argv >= 3) then
  set remEnv = $argv[3]
else
  set remEnv = $CCDLENV
endif

if ($#argv >= 4) then
  set option = $argv[4]
  if ($option == "a") then
    set add = 1
  endif
  if ($option == "c") then
    set delete = 1
  endif
  if ($option == "d") then
    set disable = 1
  endif
  if ($option == "e") then
    set enable = 1
  endif
else
  set add = 1
endif

#
# If the local and remote environments are the same then do nothing
# RHD
if ($locEnv == $remEnv) then
    exit 0
endif

setenv RTAPENV $locEnv
set noErrDisplay = $?NO_ERR_DISPLAY
setenv NO_ERR_DISPLAY ""  

set option = "-l $locEnv -e $remEnv -s <alias>$camera -r <alias>$camera"

set file = fcdDcsScan.links
if ($?INTROOT) then
  set scanFile = $INTROOT/config/$file
  if (!(-f $scanFile)) then
     set scanFile = $VLTROOT/config/$file
  endif
else
  set scanFile = $VLTROOT/config/$file
endif

#
# Configure scan links from specified scan device
#
if ($add == 1) then
  set message = "Enabling scan device $remEnv of CCD $camera DB data to $locEnv"
  echo ""
  echo -n "$message..."
  scanDevOn $remEnv >/dev/null
  if ($status == 0) then
    echo "done"
    logUserData $module 0 "$message"
  else
    echo "FAILURE"
    exit 1
  endif

  set message = "Checking & Enabling scan links from $remEnv to $locEnv for CCD $camera"
  echo ""
  echo -n "$message..."
  set command = "scanLinks -c -E $option -f $scanFile"
  $command >/dev/null
  if ($status == 0) then
    echo "done"
    logUserData $module 0 "$message"
  else
    echo "FAILURE"
  endif
endif

#
# Clean scan links from specified scan device
#
if ($delete == 1) then
  set message = "Cleaning scan links from $remEnv to $locEnv for CCD $camera"
  echo ""
  echo -n "$message..."
  set command = "scanLinks -c -d $option -f $scanFile -o"
  $command >/dev/null
  if ($status == 0) then
    echo "done"
    logUserData $module 0 "$message"
  else
    echo "FAILURE"
  endif
endif

#
# Disable scanning from specified scan device
#
if ($disable == 1) then
#  set message = "Disabling scanning of CCD $camera DB data from $remEnv to $locEnv"
#  echo ""
#  echo -n "$message..."
#  set command = "scanLinksDisable -l $locEnv -r <alias>$camera $scanFile"
#  $command >/dev/null
#  if ($status == 0) then
#    echo "done"
#    logUserData $module 0 "$message"
#  else
#    echo "FAILURE"
#  endif
endif

#
# Enable scanning from specified scan device
#
if ($enable == 1) then
  set message = "Enabling scan device $remEnv of CCD $camera DB data to $locEnv"
  echo ""
  echo -n "$message..."
  scanDevOn $remEnv >/dev/null
  if ($status == 0) then
    echo "done"
    logUserData $module 0 "$message"
  else
    echo "FAILURE"
    exit 1
  endif

#  set message = "Enabling scanning of CCD $camera DB data from $remEnv to $locEnv stand-alone part"
#  set command = "scanLinksEnable -l $locEnv -r <alias>$camera $scanFile"
#  $command >/dev/null
#  if ($status == 0) then
#    echo "done"
#    logUserData $module 0 "$message"
#  else
#    echo "FAILURE"
#  endif
endif

if ($status == 0) then
  dbForceSnap -e $locEnv
endif

if ($noErrDisplay == 0) then
  unsetenv NO_ERR_DISPLAY
endif
