#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsStart.sh,v 2.130 2005/09/08 12:50:19 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  04/06/97  created (ported from CCD software)
# rdonalds  29/01/98  Added some logging logUserData
# ccumani   14/05/98  header for manpage modified
# rdonalds  03/09/98  No longer tries to keep the opMode from before the
#                     database is loaded. Used to copy to tmpMode and then
#                     write the value back after dbRestore. Caused problems
#                     with completely empty database
# rdonalds  08/09/98  Change introduced above caused an error in WS simulation
#                     the echo $opMode caused a failure.
# ccumani   06/02/01  Removed the setting of SLCU_INS_ROOT.
# ccumani   14/03/01  manpage corrected (reference to INTROOT deleted)
# abalestr  14/06/05  added parameter for external dsp reset
#

#************************************************************************
#   NAME
#   fcdDcsStart.sh - startup of FIERA DCS
# 
#   SYNOPSIS
#   fcdDcsStart.sh [<camera>] [<WS env.>] [<SLCU env.>] [<INS root>] [kill]
# 
#   DESCRIPTION
#   This shell script performs a startup of FIERA DCS.
#   No FIERA stand-alone module is started.
#   To be run once at startup only!
#   It performs the following steps:
#     1 - Load database configuration values on WS
#     2 - Run warm startup (fcdDcsWarmStart.sh)
#
#   <camera>    camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>   name of workstation environment (default env. var. RTAPENV)
#               If value is 0, no action on WS part of FIERA sw is taken
#   <SLCU env.> name of SLCU environment (default env. var. CCDLENV)
#               If value is 0, no action on SLCU part of FIERA sw is taken
#   <INS root>  INS_ROOT environment variable
#   kill        kill all already running processes before starting          
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME        default for camera name (e.g. myccd)
#   RTAPENV        default for WS local environment (e.g. myws)
#   CCDLENV        default for SLCU environment (e.g myfiera) 
#   INS_ROOT       default root directory for instrument data 
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   > fcdDcsStart.sh myccd myws myfiera
#         Start the FIERA DCS sw both at WS and SLCU level
#         for camera "myccd", WS environment "myws", SLCU environment "myfiera"
#
#   > fcdDcsStart.sh myccd myws 0
#         Start the FIERA DCS sw at WS level only
#         for camera "myccd", WS environment "myws"
#
#   > fcdDcsStart.sh myccd myws 0 $INS_ROOT kill
#         Kill and restart the FIERA DCS sw at WS level only
#         for camera "myccd", WS environment "myws"
#
#   > fcdDcsStart.sh myccd 0 myfiera
#         Start the FIERA DCS sw at SLCU level only
#         for camera "myccd", SLCU environment "myfiera"
#
#   SEE ALSO
#   fcdDcsStop.sh fcdDcsWarmStart.sh
#
#   BUGS
#------------------------------------------------------------------------

set killCcd = ""
set dspExternalReset = ""
if ($#argv >= 1) then
  setenv CCDNAME $argv[1]
endif
if ($#argv >= 2) then
  setenv CCDWENV $argv[2]
else
  setenv CCDWENV $RTAPENV
endif
if ($#argv >= 3) then
  setenv CCDLENV $argv[3]
endif
if ($#argv >= 4) then
  setenv INS_ROOT $argv[4]
endif
if ($#argv >= 5) then
  set killCcd = $argv[5]
endif
if ($#argv >= 6) then
  set dspExternalReset = $argv[6]
endif

#
# Set the INS_HOST to a sensible value (if not set)
if (!($?INS_HOST)) then
  setenv INS_HOST $HOST
endif

# tell people what is happening
logUserData fcd 0 "fcdDcsStart starting for $CCDNAME in $CCDLENV"


#
# 1 - Load database configuration values on WS
#     No write in SLCU, since it should be done by the SLCU itself at boot time
#
if ($CCDWENV != 0) then
  fcdDcsDbInit.sh $CCDNAME $CCDWENV $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
  if ($status != 0) exit $status
endif

#
# 2 - Update environment variables according to operational mode in the DB
#     
set opMode = 3
if ($CCDLENV != 0) then
  set opMode = `dbRead "<alias>$CCDNAME.opMode"`
  if ($#opMode >= 4) then
    if ($opMode[4] == 3) then
      setenv CCDLENV 0
    endif
  endif
else
# Anyway make sure the db reflects what we want, i.e. force simulation
# This is useful only for tat tests
  dbWrite "<alias>$CCDNAME.opMode" $opMode # which is 3
endif

#
# 
#
echo "fcdDcsStart ... running warm start with mode $opMode"

#
# 3 - Run warm startup
#
if ($CCDWENV != 0) then
  fcdDcsWarmStart.sh $CCDNAME $CCDWENV $CCDLENV $killCcd $dspExternalReset
  if ($status != 0) exit $status
endif
