#! /bin/csh -f
# option -f to avoid running .cshrc and override PATH set with setmod
# if one would use the tsch, and run this script in background the output 
# would be suspended !!!!
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdDcsStop.sh,v 2.130 2005/09/08 12:50:19 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  04/06/97  created (ported from CCD software)
# ccumani   21/06/97  header corrected
# ccumani   14/05/98  header for manpage modified
# abalestr  15/10/99  bug fixed: kill parameter was considered as 5th argv
# abalestr  11/11/99  bug fixed: EXIT now correctly sent also to sim process
#                     kill procedure now more effective... kill sent before 
#                     exiting to avoid hanging, i.e. KILL and EXIT mutually 
#                     exclusive
# abortolu  13/01/04  quich change in WS simulation management (TO BE IMPROVED)
#

#************************************************************************
#   NAME
#   fcdDcsStop.sh - shut-down FIERA DCS
# 
#   SYNOPSIS
#   fcdDcsStop.sh [<camera>] [<WS env.>] [<SLCU env.>] [kill]
# 
#   DESCRIPTION
#   This shell script performs a shut-down of FIERA DCS.
#   It does the following steps:
#     1 - Verify if the main process is running.
#     2 - Try to terminate FIERA processes in a 'soft' way (command EXIT)
#     3 - Try to terminate FIERA processes in a 'hard' way (command KILL) (opt.)
#     4 - Disable scanning of data from SLCU to WS
#
#   <camera>    camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>   name of workstation environment (default env. var. RTAPENV).
#               If value is 0, no action on WS part of FIERA sw is taken
#   <SLCU env.> name of SLCU environment (default env. var. CCDLENV)
#               If value is 0, no action on SLCU part of FIERA sw is taken
#   kill        kill all processes             
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. myccd)
#   RTAPENV  default for WS local environment (e.g. myws)
#   CCDLENV  default for SLCU environment (e.g myfiera) 
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#   The "kill" options should be used with care. By killing processes
#   'blindly', the system could remain in a dangerous state. To be used
#   only to recover when the system gets stuck.
#
#   EXAMPLES
#   > fcdDcsStop.sh myccd myws myfiera
#         Terminate in a 'soft' way the FIERA sw both at WS and SLCU level
#         for camera "myccd", WS environment "myws", SLCU environment "myfiera"
#
#   > fcdDcsStop.sh myccd myws myfiera kill
#         Terminate in a 'hard' way the FIERA sw both at WS and SLCU level
#         for camera "myccd", WS environment "myws", SLCU environment "myfiera"
#
#   > fcdDcsStop.sh myccd myws 0 kill
#         Terminate in a 'hard' way the FIERA sw at WS level only
#         for camera "myccd", WS environment "myws"
#
#   > fcdDcsStop.sh myccd 0 myfiera
#         Terminate in a 'soft' way the FIERA sw at SLCU level only
#         for camera "myccd", SLCU environment "myfiera"
#
#   SEE ALSO
#   fcdDcsStart.sh, fcdDcsKill.sh
#
#   BUGS
#------------------------------------------------------------------------
set timeout = 10000
set exitTimeout = 100000
set killCcd = 0


if ($#argv >= 1) then
  set camera = $argv[1]
else
  set camera = $CCDNAME
endif
if ($#argv >= 2) then
  set locEnv = $argv[2]
else
  set locEnv = $RTAPENV
endif
#
# Set the CCDLENV to a sensible value (if not set)
# do this to avoid the script crashing in simulation mode
if (!($?CCDLENV)) then
  setenv CCDLENV 0
endif

#
# 1 - Update environment variables according to operational mode in the DB
#     
if ($CCDLENV != 0) then
  set opMode = `dbRead "<alias>$CCDNAME.opMode"`
  if ($#opMode >= 4) then
    if ($opMode[4] == 3) then
      setenv CCDLENV $RTAPENV
    endif
  endif
else
  setenv CCDLENV $RTAPENV
endif

if ($#argv >= 3) then
  set remEnv = $argv[3]
else
  set remEnv = $CCDLENV
endif
set remHost = `fcdEnvToHost $remEnv`

if ($#argv >= 4) then
  if ($argv[4] == "kill" || $argv[4] == "Kill") then
    set killCcd = 1
  endif
endif
set noErrDisplay = $?NO_ERR_DISPLAY

#
# 2 - If kill requested, just kill 'em all...
# 
if ($killCcd == 1) then
  if (($locEnv != 0) || ($remEnv != 0)) then
    fcdDcsKill.sh $camera $locEnv $remEnv $remHost
    if ($status != 0) exit $status
  endif
endif


#
# 3 - Soft way... Even if preocesses already killed, go through just to do 
#     some "house cleaning"
# Verify if the main process is running.
# Send PING to high level coordination process (fcdconCI_$ccdname)
#
if (($remEnv != 0) || ($locEnv != 0)) then
  set wsState = 1
  if ($locEnv != 0) then
    set process = fcdconCI_$camera
    set destEnv = $locEnv
    set slcuState = 6
  else
    set process = fcdcon_$camera
    set destEnv = $remEnv
    set slcuState = 1
  endif
  setenv NO_ERR_DISPLAY ""
  msgSend -n $destEnv $process "PING" "" $timeout >/dev/null
  if ($status != 0) then
    echo ""
    echo "Process $process environment $destEnv is NOT active"
# It might be that SLCU sw is running (e.g. after reboot). One must
# try at least to terminate SLCU sw
    if ($remEnv != 0) then
      set process = fcdcon_$camera
      set destEnv = $remEnv
      msgSend -n $destEnv $process "PING" "" $timeout >/dev/null
      if ($status != 0) then
        echo "Process $process environment $destEnv is NOT active"
        set process = 0
        set destEnv = 0
      endif
    else
      set destEnv = 0
    endif
    echo -n "Resetting operational state to OFF..."
    dbWrite "<alias>$camera.opStateSlcu" $slcuState
    dbWrite "<alias>$camera.opStateWs" $wsState
    echo "done"
    echo ""
  endif

#
# 4 - Try to terminate FIERA processes in a 'soft' way
# send EXIT to coordination process
#
  if ($destEnv != 0) then
    echo ""
    echo "Sending command EXIT to process $process environment $destEnv..."
    msgSend -n $destEnv $process "EXIT" "" $exitTimeout >/dev/null
    if ($status != 0) then
      echo "FAILED"
      exit 1
    endif
    echo "done"
    echo ""
  endif

  if ($remEnv == 0) then
    set process = fcdsimWs_$camera
    set destEnv = $locEnv
    setenv NO_ERR_DISPLAY ""
    msgSend -n $destEnv $process "PING" "" $timeout >/dev/null
    if ($status == 0) then
       echo "Sending command EXIT to process $process environment $destEnv..."
       msgSend -n $destEnv $process "EXIT" "" $exitTimeout >/dev/null
    endif
  endif
endif
if ($noErrDisplay == 0) then
  unsetenv NO_ERR_DISPLAY
endif

#
# 5 - Disable scanning of data from SLCU to WS
#
if (($locEnv != 0) && ($remEnv != 0) && ($locEnv != $remEnv)) then 

  fcdDcsScan.sh $camera $locEnv $remEnv $remHost d

  if ($status != 0) exit $status
endif










