#!/bin/sh
if [ "`uname`" = "Linux" ]; then enable -n echo; fi
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdInstall.sh,v 2.27 2005/05/17 17:13:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  05/06/97  created (ported from CCD software)
# rdonalds  14/01/98  Now copies the sequence files and voltage
#                     tables into INS_ROOT/SYSTEM/COMMON/CONFIGFILES
# rdonalds  23/02/98  Checks existance of destination config directory and
#                     moves out of the way before installing new one
# rdonalds  25/02/98  Fixed bug introduced in previous change. It only
#                     worked if the directory already existed
# ccumani   14/05/98  header for manpage modified
# ccumani   01/09/98  installation of commands support setup files
#                     corrected
# abalestr  04/03/99  added check of definition of INS_ROOT and CCDNAME
# abalestr  04/03/99  changed behaviour with respect to INS_USER:
#                     now the script checks if INS_USER is defined, if yes
#                     the defined value is used otherwise INS_USER is set to 
#                     the default value of SYSTEM
# ccumani   01/03/01  checking existence of dbcfg file added
# ccumani   14/03/01  manpage corrected (reference to INTROOT deleted)
# ccumani   19/01/03  usage of MODROOT introduced (SPR VLTSW20020696)
# abalestr  26/11/03  checking dbcfg file added
# ccumani   26/11/03  definition of Config Path and File corrected
#                     (it was not working if the <DbFile> parameter
#                     contained the .dbcfg extension)
# abalestr  30/08/04  added 3rd parameter to script in order to work also if
#                     env not active. Use it carefully!!!
#
#************************************************************************
#   NAME
#   fcdInstall.sh - Install FIERA files in INS_ROOT
# 
#   SYNOPSIS
#   fcdInstall.sh <DbFile> [<ins_root_dir>]
# 
#   DESCRIPTION
#   This shell script installs all files needed to run a FIERA system in the
#   root directory for the instrument it belongs to. Additionally, it
#   stores in the on-line database the values contained in configuration
#   files.
#
#   <DbFile>       Name of the .dbcfg file in $VLTROOT/config containing DB 
#                  init values.
#   <ins_root_dir> root directory for the instrument the FIERA belongs to
#                  Default: $INS_ROOT (env. variable)
# 
#   FILES
#   $VLTROOT/config/fcd*.dbcfg 
#   Files containing on-line database initialisation values for FIERA systems
#   known.
#   For scientific CCDs there is one file for each system to be delivered.
#   They normally have to be checked and possibly modified by experts of
#   the specific CCD system. The name is fcdSci<id>.dbcfg whereby <id> is
#   a symbolic name of the system.
#   Example: "myccd" CCD ---> fcdSciMyccd.dbcfg
#   
#
#   ENVIRONMENT
#   CCDNAME  CCD camera name
#   RTAPENV  FIERA Rtap environment name
#   INS_ROOT default instrument root directory
#   INS_USER default to SYSTEM
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#
#   EXAMPLES
#   > fcdInstall.sh fcdSciMyccd.dbcfg
#   Install all what needed for "myccd" scientific CCD
#
#   SEE ALSO
#   INS common sw specification
#
#------------------------------------------------------------------------
if [ $# -lt 1 ]
then
	echo "Usage: fcdInstall.sh <DbFile> [<ins_root_dir>]"
	exit 1
fi

if [ $# = 2 ]
then
	INS_ROOT=$2
	export INS_ROOT
fi

NODB="0"
export NODB

if [ $# = 3 ]
then
	NODB="1"
	export NODB
fi

# Check that INS_ROOT is defined
INS_ROOT=${INS_ROOT:?"not defined nor passed as argument"}
export INS_ROOT

# Check that INS_ROOT exists and is a directory
if [ ! -d $INS_ROOT ]
then
  echo $INS_ROOT does not exist or it is not a directory
  exit 1
fi

# Check that CCDNAME is defined
CCDNAME=${CCDNAME:?"not defined"}
export CCDNAME

# If INS_USER not defined default to SYSTEM
INS_USER=${INS_USER:="SYSTEM"}
export INS_USER

# First we get rid of any possible trailing directory
baseName=`basename $1`

# See if we have also the extension
#case $baseName in
#	*.dbcfg)
#	fileName=$baseName
#	;;
#	*)
#	fileName=$baseName.dbcfg
#	;;
#esac

# WRONG!!! if baseName has the .dbcfg extension, then
# at the end of the next step BOTH configFilePath AND configDirPath
# will have the .dbcfg extension ... 
# Better do in the following way:

# 1. rip-out the extension, if existing, to define the directory
baseName=`basename $baseName .dbcfg`

# 2. add the extension, to define the file
fileName=$baseName.dbcfg

# Now work out the names of the .dbcfg file and identify the
# directory where the sequences come from
# we search MODROOT, INTROOT and, finally, VLTROOT

if [ -f "$MODROOT/config/$fileName" ]
then
	configFilePath="$MODROOT/config/$fileName"
	configDirPath="$MODROOT/config/$baseName"
elif [ -f "$INTROOT/config/$fileName" ]
then
	configFilePath="$INTROOT/config/$fileName"
	configDirPath="$INTROOT/config/$baseName"
elif [ -f "$VLTROOT/config/$fileName" ]
then
	configFilePath="$VLTROOT/config/$fileName"
	configDirPath="$VLTROOT/config/$baseName"
else
	echo "Cannot find dbcfg file $fileName"
	exit 1
fi

noSeq=0

if [ ! -d $configDirPath ]
then
	echo "There are no sequence files for this configuration"
	noSeq=1
fi

echo "Installing from db file $fileName and config dir $configDirPath"

# save current directory
curdir=`pwd`

#
# 1 - Create INS_ROOT sub-directories needed by the FIERA software
#
cd $INS_ROOT

dirs="$INS_USER $INS_USER/Dictionary $INS_USER/DETDATA $INS_USER/COMMON $INS_USER/COMMON/CONFIGFILES $INS_USER/COMMON/SETUPFILES $INS_USER/COMMON/SETUPFILES/DET $INS_USER/COMMON/SETUPFILES/REF"

for d in $dirs
do
	mkdir -p $d
done

#
# 2 - Copy CCDDCS data dictionary and FCDDCS
# 
if [ -f "$MODROOT/config/ESO-VLT-DIC.CCDDCS" ]
then
	cp $MODROOT/config/ESO-VLT-DIC.CCDDCS $INS_ROOT/$INS_USER/Dictionary
elif [ -f "$INTROOT/config/ESO-VLT-DIC.CCDDCS" ]
then
	cp $INTROOT/config/ESO-VLT-DIC.CCDDCS $INS_ROOT/$INS_USER/Dictionary
elif [ -f "$VLTROOT/config/ESO-VLT-DIC.CCDDCS" ]
then
	cp $VLTROOT/config/ESO-VLT-DIC.CCDDCS $INS_ROOT/$INS_USER/Dictionary
else
	echo "Cannot find CCD Dictionary"
	exit 1
fi

if [ -f "$MODROOT/config/ESO-VLT-DIC.FCDDCS" ]
then
	cp $MODROOT/config/ESO-VLT-DIC.FCDDCS $INS_ROOT/$INS_USER/Dictionary
elif [ -f "$INTROOT/config/ESO-VLT-DIC.FCDDCS" ]
then
	cp $INTROOT/config/ESO-VLT-DIC.FCDDCS $INS_ROOT/$INS_USER/Dictionary
elif [ -f "$VLTROOT/config/ESO-VLT-DIC.FCDDCS" ]
then
	cp $VLTROOT/config/ESO-VLT-DIC.FCDDCS $INS_ROOT/$INS_USER/Dictionary
else
	echo "Cannot find FCD Dictionary"
	exit 1
fi

#
# 3 - Copy setup file templates
# 
copyFile="fcdSetupComplete.det"
targetDirectory=$INS_ROOT/$INS_USER/COMMON/SETUPFILES/DET
if [ -f "$MODROOT/config/$copyFile" ]
then
	originDirectory=$MODROOT/config
elif [ -f "$INTROOT/config/$copyFile" ]
then
	originDirectory=$INTROOT/config
elif [ -f "$VLTROOT/config/$copyFile" ]
then
	originDirectory=$VLTROOT/config
else
	echo "Cannot find $copyFile"
	exit 1
fi
echo "Installing Setup File $copyFile from $originDirectory"
cp $originDirectory/$copyFile $targetDirectory

#
# 4 - Copy database configuration files 
#
copyFile="fcdDcsFixed.dbcfg"
targetDirectory=$INS_ROOT/$INS_USER/COMMON/CONFIGFILES
if [ -f "$MODROOT/config/$copyFile" ]
then
	originDirectory=$MODROOT/config
elif [ -f "$INTROOT/config/$copyFile" ]
then
	originDirectory=$INTROOT/config
elif [ -f "$VLTROOT/config/$copyFile" ]
then
	originDirectory=$VLTROOT/config
else
	echo "Cannot find $copyFile"
	exit 1
fi
echo "Installing dbcfg $copyFile from $originDirectory"
cp $originDirectory/$copyFile $targetDirectory

# template files 
copyFile="fcdSciTemplate.dbcfg"
targetDirectory=$INS_ROOT/$INS_USER/COMMON/CONFIGFILES
if [ -f "$MODROOT/config/$copyFile" ]
then
	originDirectory=$MODROOT/config
elif [ -f "$INTROOT/config/$copyFile" ]
then
	originDirectory=$INTROOT/config
elif [ -f "$VLTROOT/config/$copyFile" ]
then
	originDirectory=$VLTROOT/config
else
	echo "Cannot find $copyFile"
	exit 1
fi
echo "Installing dbcfg $copyFile templates from $originDirectory"
cp $originDirectory/$copyFile $targetDirectory

#
# 5 - copy contents of database backup files into database
#
if [ $NODB = "0" ]
then
  dbRestore -f $INS_ROOT/$INS_USER/COMMON/CONFIGFILES/fcdDcsFixed.dbcfg -co "<alias>$CCDNAME"
  if [ $? -ne 0 ]
  then
	  echo "dbRestore FAILED"
	  exit 1
  fi
fi
# Work on the dbcfg file
fcdDbFromTemplate.sh $configFilePath $INS_ROOT $NODB
if [ $? -ne 0 ]
then
	echo "fcdDbFromTemplate FAILED"
	exit 1
fi

# Copy the sequence files and voltage tables (if needed)
# If the destination already exists, then move it out of the way
if [ $noSeq = 0 ]
then
	targetDirectory=$INS_ROOT/$INS_USER/COMMON/CONFIGFILES/$CCDNAME
	if [ -d $targetDirectory ]
	then
		echo "Destination already exists move to $targetDirectory.$$"
		mv $targetDirectory $targetDirectory.$$
	fi
	echo "Copying sequence files to $targetDirectory"
	cp -r $configDirPath $targetDirectory
else
	echo "Not copying sequences"
fi

cd $curdir 
