#!/bin/sh
if [ "`uname`" = "Linux" ]; then enable -n echo; fi
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fcdosScan.sh,v 2.27 2005/05/17 17:13:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# abalestr  05/06/97  created (ported from CCD software)
# ccumani   14/05/98  header for manpage modified
# abalestr  04/08/00  force snap using rtap or ccs_lite depending on RTAPROOT
# abalestr  13/06/03  ported to Bourne shell
# abalestr  12/01/04  added workaround for oslx setting INTROOT to NOT-USED if
#                     undefined. See spr no. VLTSW20030555
# abalestr  16/02/04  wrong logic when searching for fcdosScan.links. 
#                     Unfortunately, porting from csh the case where INTROOT is
#                     defined but the file is not there, so that one should 
#                     revert to VLTROOT file, was lost.
#

#************************************************************************
#   NAME 
#   fcdosScan.sh - configure the DB values to be scanned from SLCU for CCD OS
# 
#   SYNOPSIS
#   fcdosScan.sh [<camera>] [<WS env.>] [<SLCU env.>] [<option>]
# 
#   DESCRIPTION
#   This shell script configures the Scan System to retrieve DB values
#   from the FIERA SLCU to WS (stand-alone part only).
#
#   <camera>    camera name, also root point in DB (default env. var. CCDNAME)
#   <WS env.>   name of workstation environment (default env. var. RTAPENV)
#   <SLCU env.> name of SLCU environment (default env. var. CCDLENV)
#   <option>    a (default) - add entries to scan list
#               c           - clean entries from scan list
#               d           - disable scanning from the SLCU
#               e           - enable scanning from the SLCU
# 
#   FILES
#
#   ENVIRONMENT
#   CCDNAME  default for camera name (e.g. myccd)
#   RTAPENV  default for WS local environment (e.g. myws)
#   CCDLENV  default for SLCU environment (e.g myfiera) 
#
#   RETURN VALUES
#   0 if SUCCESS
#   1 if FAILURE
#
#   CAUTIONS
#   1 - It is assumed here that CCD branch in the SLCU database is attached
#       directly to the root point (not a sub-point of something else).
#   2 - If this script is run twice or more, the system may not behave properly
#       any more. In this case better to shutdown the whole system and restart 
#       it again.
#
#   EXAMPLES
#   fcdosScan.sh myccd myws myfiera
#        Add entries for camera "myccd", SLCU environment "myfiera",
#        WS environment "myws"
#   fcdosScan.sh myccd myws myfiera c
#        Clean entries for camera "myccd", SLCU environment "myfiera",
#        WS environment "myws"
#   fcdosScan.sh myccd myws myfiera e
#        Enable entries for camera "myccd", SLCU environment "myfiera",
#        WS environment "myws"
#
#   SEE ALSO
#   fcdDcsScan.sh
#
#   BUGS
#------------------------------------------------------------------------
add=0
delete=0
disable=0
enable=0
option=0
message=""
module="fcdos"

if [ $# -ge 1 ] 
then
  CCDNAME=$1
fi
if [ $# -ge 2 ] 
then
  RTAPENV=$2
fi
if [ $# -ge 3 ] 
then
  CCDLENV=$3
fi

if [ $# -ge 4 ]
then
	if [ "$4" = "a" ]
	then
		add=1
	elif [ "$4" = "c" ]
	then
		delete=1
	elif [ "$4" = "d" ]
	then
		disable=1
	elif [ "$4" = "e" ]
	then
		enable=1
	fi
else
	add=1
fi

noErrDisplay=${NO_ERR_DISPLAY:="0"}
NO_ERR_DISPLAY=""  

option="-l $RTAPENV -e $CCDLENV -s <alias>$CCDNAME -r <alias>$CCDNAME"

# Workaround to fix problem with oslx that sets INTROOT to "NOT-USED" if the
# environment variable is not set. See Spr no. VLTSW20030555
if [ ${INTROOT} = "NOT-USED" ]
then
	INTROOT=""
fi

if [ -r ${INTROOT}/config/fcdosScan.links ]
then
	scanFile=${INTROOT}/config/fcdosScan.links
elif [ -r ${VLTROOT}/config/fcdosScan.links ]
then
	scanFile=${VLTROOT}/config/fcdosScan.links
else
	echo "Cannot find fcdosScan in both INTROOT and VLTROOT"
	echo "FAILURE"
	exit 1
fi

#
# Configure scan links from specified scan device
#
if [ "$add" = "1" ]
then
	message="Enabling scan device $CCDLENV of CCD $CCDNAME DB data to $RTAPENV"
	echo ""
	echo "$message..."
	scanDevOn $CCDLENV > /dev/null
	if [ $? = 0 ]
	then
		echo "done"
		logUserData $module 0 "$message"
	else
		echo "FAILURE"
		exit 1
	fi

	message="Checking & Enabling scan links from $CCDLENV to $RTAPENV for CCD $CCDNAME stand-alone part"
	echo ""
	echo "$message..."
	command="scanLinks -c -E $option -f $scanFile"
	$command > /dev/null
	if [ $? = 0 ]
	then
		echo "done"
		logUserData $module 0 "$message"
	else
		echo "FAILURE"
	fi
fi

#
# Clean scan links from specified scan device
#
if [ "$delete" = "1" ]
then
	message="Cleaning scan links from $CCDLENV to $RTAPENV for CCD $CCDNAME stand-alone part"
	echo ""
	echo "$message..."
	command="scanLinks -c -d $option -f $scanFile -o"
	$command > /dev/null
	if [ $? = 0 ]
       	then
		echo "done"
		logUserData $module 0 "$message"
	else
		echo "FAILURE"
	fi
fi

#
# Disable scanning from specified scan device
#
if [ "$disable" = "1" ]
then
	message="Disabling scanning of CCD $CCDNAME DB data from $CCDLENV to $RTAPENV stand-alone part"
	echo ""
	echo "$message..."
	destRoot="<alias>$CCDNAME"
	command="scanLinksDisable -l $RTAPENV -r $destRoot $scanFile"
	$command > /dev/null
	if [ "$?" = "0" ]
       	then
		echo "done"
		logUserData $module 0 "$message"
	else
		echo "FAILURE"
	fi
fi

#
# Enable scanning from specified scan device
#
if [ "$enable" = "1" ]
then
	message="Enabling scan device $CCDLENV of CCD $CCDNAME DB data to $RTAPENV"
	echo ""
	echo "$message..."
	scanDevOn -l $RTAPENV $CCDLENV #> /dev/null
	if [ $? = 0 ]
       	then
		echo "done"
		logUserData $module 0 "$message"
	else
		echo "FAILURE"
	fi
fi

if [ $? = 0 ]
then
	dbForceSnap -e $RTAPENV
fi

#
# __oOo___
#
