#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: inducer.tcl,v 1.65 2003/12/17 19:26:25 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 15.03.1995 created
#

#************************************************************************
#   NAME
#	inducer.tcl - Common part for all Inducer Test Access Panels
# 
#   SYNOPSIS
#	source [findVLTFile bin/inducer.tcl]
#
#   DESCRIPTION
#	When this file is sourced, it performs the following actions:
#
#	1. the common part for all Inducer Test Access Panels is created
#	2. a frame for additional device-specific widgets is appended
#	3. widget-names are stored into well-defined variables
#	4. some widgets are re-configured and bindings are added
#	5. common data are initialized
#	6. the configuration files are searched and executed
#
#	After completion of these steps, device-specific actions
#	can be done by the calling program.
#
#   FILES
#	inducer.pan - Panel definition file, searched in $xxxROOT/bin
#	inducer.config - configuration file, searched in $xxxROOT/config
#	~/.inducer - secondary configuration file
#
#   ENVIRONMENT
#	HOST - hostname to display in title (optional)
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

###############################################################################
#
#	G L O B A L   V A R I A B L E S
#	"Declaration" of global variables
#
###############################################################################
# Rules:
# All global variable-names start with the module-name (inducer)
# Usually everything is stored in arrays.
#
# Array indices starting with a single lowercase letter have a special meaning:
#	c	Constant
#	e	Derived from environment variable
#	f	Boolean flag with value 0/1 or on/off etc.
#	n	Names defined in configuration file
#	v	Variable
#	w	Widget name
#
# Abbreviations (as part of names):
#	Env	Environment
#	Dev	Device
#
###############################################################################

set inducer(version) [lindex {$Revision: 1.65 $} 1]

#
# Global Variables and Flags:
#
global inducer(vEnv)		;# Environment name
global inducer(vDev)		;# Device name
global inducer(vDevFilter)	;# glob-style filter, e.g. "/acro*"
global inducer(vMode)		;# open-mode: TEST,EXCLUSIVE,SHARED,READONLY
global inducer(vMessage)	;# contents of message-output
global inducer(vDeviceId)	;# contents of device-ID-output
global inducer(vTickRate)	;# contents of ticks/sec-output
global inducer(vCommands)	;# 
global inducer(fOnline)		;# flag for on-line buttons
global inducer(vServerVersion)	;# set as returned by VERSION command


###############################################################################
#
#	P R O C E D U R E S
#
###############################################################################
# Rules:
#	- All proc-names start with the module-name (e.g. inducer)
#	- Parameters start with upper-case letter (e.g. Environment)
#	- Local variables start with lower-case letter (e.g. item)
#	- In man-pages optional parameters are enclosed in "?...?"
#
# Abbreviations (as part of names):
#	Evh	Event Handler
#
###############################################################################



###############################################################################
#
#	C O M M O N   M A I N
#
###############################################################################
# Rules:
#	- Put here initialization sequences that shall be common to 
#	  ALL Inducer applications that source this file.
#
#	- initialization sequences that shall be evaluated during run-time
#	  can be put into the configuration-file stated below
#
###############################################################################

puts {inducer: $Revision: 1.65 $}

#------------------------------------------------------------------------------
#
# Process common Panel and do proper interfacing of names:
#

source [findVLTFile bin/inducer.pan]

#
# Some window management:
#
wm minsize .  800  134	;# values depend on inducer.pan
wm maxsize . 1200 1024	;# set to max. screen size
wm geometry . {}	;# resize window to natural size

# make standard main window, install a wm quit handler to clean-up,
# and source config files:
vccStdMain {inducerEvhQuit}

#
# Add the Warning-Logo button on the top right:
#
button $cv.wLogo -image [image create pixmap -file [vccFindPixmapFile]] \
	-relief flat -command inducerEvhLogo
pack $cv.wLogo -side right -anchor ne

#
# Make a frame where all device-specific extensions will go in:
# NOTE: the window-path is dependent on the panel-editor (parent of $cv)!
#
regsub {\.[^.]+$} $cv "" inducer(wExt)
set inducer(wExt) [frame $inducer(wExt).ext -relief groove -bd 2]
pack $cv -side top -expand no
pack $inducer(wExt) -side top -fill both -expand yes 

#
# Widget-Names:
#
#   inducer(wExt)	see above
set inducer(wSelEnv)	$cv.wEnv		;# `Environment' Selector
set inducer(wSelDev)	$cv.wDev		;# `Device' Selector
set inducer(wObutMod)	$cv.www17		;# `Mode' Optionbutton
set inducer(wOutpMsg)	$cv.www9		;# `Message' Output
set inducer(wOutpDid)	$cv.www11		;# `Device-ID' Output
set inducer(wOutpTck)	$cv.www16		;# `Ticks/sec' Output
set inducer(wButtRes)	$cv.www12		;# `Reset Device' Button
set inducer(wRbutOnl)	$cv.www13		;# `On-Line' Radiobutton
set inducer(wRbutOff)	$cv.www14		;# `On-Line' Radiobutton

set inducer(wBeginMacro) $bcv.wBeginMacro	;# `Begin Macro' Actionbutton
set inducer(wEndMacro)   $bcv.wEndMacro		;# `End Macro' Actionbutton
set inducer(wExecMacro)  $bcv.wExecMacro	;# `Exec Macro' Actionbutton
set inducer(wSaveMacro)  $bcv.wSaveMacro	;# `Save Macro' Actionbutton
set inducer(wLoadMacro)  $bcv.wLoadMacro	;# `Load Macro' Actionbutton

#
# Re-configure Widgets:
#
$inducer(wObutMod) configure -variable inducer(vMode) \
	-command inducerEvhMode				;# `Mode' Optionbutton
$inducer(wOutpMsg) configure -variable inducer(vMessage)
$inducer(wOutpDid) configure -variable inducer(vDeviceId)
$inducer(wOutpTck) configure -variable inducer(vTickRate)
$inducer(wButtRes) configure -command inducerEvhResetDevice
$inducer(wRbutOnl) configure -variable inducer(fOnline) -onVal 1 \
	-command inducerEvhOnline
$inducer(wRbutOnl).uifRadioButton configure -selectcolor Green
$inducer(wRbutOff) configure -variable inducer(fOnline) -onVal 0 \
	-command inducerEvhOffline
$inducer(wRbutOff).uifRadioButton configure -selectcolor Red

$inducer(wEndMacro)  configure -state disabled
$inducer(wExecMacro) configure -state disabled
$inducer(wSaveMacro) configure -state disabled

#
# Menu-Bar Entries:
#
# create a dummy VccSelectButton for the `Driver' menu in the menubar, 
# where only the MakeMenu method is used lateron:

# get widget name of Driver menu in menubar:
if {[set inducer(wMenuDrv) [panGetMbPath Driver]] == ""} {
    error "internal error: cannot get widget name of Driver menu. EXIT"
}
set inducer(wMenuDrv) [$inducer(wMenuDrv) cget -menu]

set inducer(wSelDrv) [\
	VccSelectButton $cv.wSelDrvDummy \
	-command inducerEvhDriver \
	-menuType command \
	]


#------------------------------------------------------------------------------
#
# Initialization of Data:
#

#
# INIT Variables:
#
set inducer(vEnv)	""
set inducer(vDev)	""
set inducer(vDevFilter)	"*"
set inducer(vMode)	READONLY
set inducer(vMessage)	""
set inducer(vDeviceId)	?
set inducer(vTickRate)	?
set inducer(vCommands)	{}
set inducer(fOnline)	0
set inducer(vServerVersion) UNKNOWN


#------------------------------------------------------------------------------

set inducer(wDev) [InducerDevice $inducer(wExt).nil] ;# dummy Device object

# ___oOo___
