#ifndef FCDTELEMETRY_H
#define FCDTELEMETRY_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: fcdTELEMETRY.h,v 2.140 2005/10/17 15:17:04 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* rdonalds  04/03/98  created
*/

/************************************************************************
*   NAME            
*        fcdTELEMETRY
*   
* 
*   SYNOPSIS
*        Class for implementing Temperature and Telemetry sensors.
*   
*   PARENT CLASS
*
* 
*   DESCRIPTION
*        The class is primamrily manipulated by the fcdtel process. 
*        Other processes send messages to this process. The fcdtel
*        process performs periodic updates.
*
*   PUBLIC METHODS
*
*
*   PUBLIC DATA MEMBERS
*
*
*   PROTECTED METHODS
*
*
*   PROTECTED DATA MEMBERS
*
*
*   PRIVATE METHODS
*
*
*   PRIVATE DATA MEMBERS
*
*
*   FILES
*
*   ENVIRONMENT
*
*   COMMANDS
*
*   RETURN VALUES
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS   
* 
*------------------------------------------------------------------------
*/

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

// Need to forward declare detector
class fcdDETECTOR;

// Include files for standard C++ library components
#include <string>
#include <iostream>
#include <fstream>

// Normal fcd includes
#include "fcdINTERNAL.h"
#include "fcdDBDIRECT.h"      // for direct DB access
#include "fcdC40.h"           // to access the electronics
#include "fcdPULPO_CONFIG.h"  // to access PULPO


/*
 *# Maximum number of sensors in array, this MUST be >= to fcdMAXTELSRC and
 *# fcdMAXTEMPSRC
 */
#define fcdMAX_SENSOR fcdMAXTELSRC

/*
 *# fcdSENSOR_TYPE
 *# 
 *# a sensor is either something connected to the electronics
 *# or to a PULPO unit
 */
typedef enum {
    fcdSENSOR_PULPO = 1,
    fcdSENSOR_C40
} fcdSENSOR_TYPE;

/*
 *# fcdT_SENSOR_DEF
 *#
 *# Fully describes where the sensor value comes from
 */
typedef struct
{
    vltLOGICAL     enabled;     /* on ? */
    vltBYTES16     description; /* description of sensor */
    vltBYTES16     id;          /* id of sensor */
    vltINT32       sensorType;  /* fcdSENSOR_PULPO, fcdSENSOR_C40 */
    vltINT32       boardId;     /* either board id or PULPO unit number */
    vltINT32       sensorId;    /* either peripheral id or PULPO sensor */
    vltDOUBLE      convFact;    /* current = source * conversion */
    vltDOUBLE      low;         /* limits */
    vltDOUBLE      high;       
} fcdT_SENSOR_DEF;

/*
 *# fcdT_SENSOR
 *#
 *# Entry for each sensor in shared memory.
 */
typedef struct
{
    fcdT_SENSOR_DEF    def;        /* sensor definition */
    vltINT32           dbIndex;    /* index into original sensor array */
    vltDOUBLE          current;    /* current value */
    vltLOGICAL         valueIsValid; /* is the current value valid */
                                     /* was it read OK             */
    vltDOUBLE          start;      /* start value */
    vltDOUBLE          end;        /* end value */
    vltDOUBLE          mean;       /* average during exposure */
    vltDOUBLE          drift;      /* drift during exposure */
    vltDOUBLE          max;        /* max during exposure */
    vltDOUBLE          min;        /* min during exposure */
    vltDOUBLE          valSum;     /* running total */
    vltDOUBLE          valSumSq;   /* square of value */
} fcdT_SENSOR;
/*
 * T_TELEMETRY
 *
 * The sensor information which goes into shared memory as part of the
 * T_EXPOSURE structure for a particular detector.
 *
 * The array of sensors is built from the database table. But each entry
 * in the database has an "enabled" switch. So we have an in-memory
 * array of only the valid sensors, and each sensor keeps an index of
 * its origin. We use a vector for current values which we write into the
 * database.
 */
typedef struct
{
    /*
     * The root point of the sensors which we are setting up
     * <alias>CAMERA.telemetry or .temperature
     */
    dbSYMADDRESS dbRootPt;

    /*
     *# Read from DB available flag
     */
    vltLOGICAL   available;

    /*
     *# Periodicity and last update.
     */
    vltDOUBLE    period;

    /*
     *# Logging periodicity
     */
    vltINT32    logPeriod;

    /*
     *# Periodic active ?
     */
    vltLOGICAL   active;

    /*
     *# opState reflected in OL_DB
     */
    vltINT32            opState;

    /*
     *# tmOutOfLimit reflected in OL_DB
     */
    vltINT8            tmOutOfLimit;
    
    /*
     *# Exposure active
     */
    vltLOGICAL   expActive;

    /*
     *# Array of current values, this is written into the DB
     */
    vltDOUBLE    currentValues[fcdMAX_SENSOR];

    /*
     *# Number of available sensors and an entry for each
     */
    vltINT32     numSensors; /* number of valid sensors */
    vltINT32     numSamples; /* number of samples which have been taken */
                             /* during exposure                         */
    fcdT_SENSOR  sensor[fcdMAX_SENSOR];     /* entry for each sensor */
} fcdT_TELEMETRY;
/*
 * fcdTELEMETRY
 *
 * Class for manipulation of an array of sensors
 */
class fcdTELEMETRY
{
  public:
    fcdTELEMETRY();
    ~fcdTELEMETRY();

    /***********************************************************************
      INITIALISATION / UNINITIALISATION
      ***********************************************************************/
    ccsCOMPL_STAT Create(fcdDETECTOR *det,
			 fcdT_TELEMETRY    *sensors);
    ccsCOMPL_STAT Attach (fcdDETECTOR *det,
			  fcdT_TELEMETRY  *sensors);
    ccsCOMPL_STAT Detach();

    /*
     *# Return available switch
     */
    vltLOGICAL Available() { return (shTelemetry->available);}
    
    /**********************************************************************
      MANIPULATORS
      **********************************************************************/
    /*
     *# Update the current value vector and each sensor entry
     */
    ccsCOMPL_STAT GetCurrentValues(fcdC40 *c40, fcdPULPO_CONFIG *gPulpo);

    /*
     *# Return the number of sensors available and a pointer to
     *# the individual sensor
     */
    vltINT32 NumSensors() {return (shTelemetry->numSensors);}
    fcdT_SENSOR *Sensor(vltINT32 i)
                {return (&shTelemetry->sensor[i]);}  

    /*
     *# Start, Stop called at beginning and end of an exposure
     */
    ccsCOMPL_STAT ExpStart ();
    ccsCOMPL_STAT ExpStop ();

    vltLOGICAL    ExpActive () {return (shTelemetry->expActive);}
    void          ExpActive (vltLOGICAL a) {shTelemetry->expActive = a;}
    /*
     *# Number of GetValues() since Start()
     */
    vltINT32 NumSamples() {return (shTelemetry->numSamples);}
    void     NumSamples(vltINT32 v) {shTelemetry->numSamples = v;}

    /*
     *# Get/Set the sampling period
     */
    vltDOUBLE     Period () {return (shTelemetry->period);}
    void          Period (vltDOUBLE period) {shTelemetry->period = period;}

    /*
     *# Get/Set the telemetry logging period
     */
    vltINT32     LogPeriod () {return (shTelemetry->logPeriod);}
    void          LogPeriod (vltINT32 logPeriod)
                                         {shTelemetry->logPeriod = logPeriod;}

    vltLOGICAL    Active () {return (shTelemetry->active);}
    void          Active (vltLOGICAL a);

    /*
     *# Set the op state in the OL-DB
     */
    ccsCOMPL_STAT OpState (vltINT32 state);
    vltINT32      OpState () {return (shTelemetry->opState);};

    /*
     *# Set the tmOutOfLimit in the OL-DB
     */
    ccsCOMPL_STAT TmOutOfLimit (vltINT8 flag);
    vltINT8       TmOutOfLimit () {return (shTelemetry->tmOutOfLimit);};

  private:
    // DB direct address for writing current values
    fcdDBDIRECT         dbDirectCurrent;

    // Access for opState in OL-DB
    fcdDBDIRECT         opStateDirect;

    // Access for tmOutOfLimit in OL-DB
    fcdDBDIRECT         tmOutOfLimitDirect;

    fcdT_TELEMETRY      *shTelemetry;
    fcdDETECTOR         *det;

    // timer for log updates
    time_t lastTime;

    /*
     *# Get individual sensor values
     */
    ccsCOMPL_STAT GetSensorValue (vltINT32, fcdC40 *, fcdPULPO_CONFIG *);
    ccsCOMPL_STAT GetPulpoValue (fcdT_SENSOR *, fcdC40 *, fcdPULPO_CONFIG *);
    ccsCOMPL_STAT GetC40Value (fcdT_SENSOR *, fcdC40 *, fcdPULPO_CONFIG *);

    /*
     *# Monitor the current values against their limits
     *# logging FITS UNFORSEEN messages and raising an alarm in 
     *# the database
     */
    ccsCOMPL_STAT MonitorValues();

    /*
     *# Initialisation of dbDirect
     */
    ccsCOMPL_STAT  Init();

    /*
     *# Read the configuration 
     */
    ccsCOMPL_STAT  ReadConfigFromDB();

    /*
     *# Write current values to the DB
     */
    ccsCOMPL_STAT  WriteValuesToDB();
};
#endif
