/*************************************************************************
* E.S.O. - VLT project
#
# "@(#) $Id: log.h,v 4.26 2005/10/24 07:59:42 vltsccm Exp $" 
*
* <log.h> - CCS LOGGING SYSTEM Interface File
*
* who        when      what
* ---------  --------  ----------------------------------------------
* B.GILLI    29/04/93  Preliminary version
*    "       27/05/93  logLOC_ID & control of automatic logging
*
* M.COMIN    20/07/93  Typedef  logLOC_ID moved to ccs.h 
*
* M.COMIN    05/01/94  Routine to enable/disable automatic logs moved to
*                      private routines.
*
*                      Routine logData returnes no error structure.
*
* M.COMIN    25/01/94  Modifications according to the design 1.0
*                      - Add format of the log message from LCU
*
* M.COMIN    02/11/94  Add routine logFormatTime
*                      Add declaration for non-POSIX functions
* G.CHIOZZI  07/11/94  Deleted declarations for gettimeofday()
*
* B.GUSTAFSSON 28/11/94 Including specific CCS/LCC include files
* B.GUSTAFSSON 01/12/94 Added C++ support
*
* M.COMIN      31/01/95 Added logNORM_USER defining the lowest value
*                       for user's log Id.
* M.COMIN      05/10/95 Add logToSyslog 
*
* mcomin       28/01/97 Add data structures to support FITS logging
* B.Gilli      23/11/99 Preliminary cleaning of new gcc warnings.
*
*************************************************************************/

#ifndef LOG_H
#define LOG_H

#ifdef __cplusplus
extern "C" {
#endif

/* 
 * Header files
 */

#include <stdarg.h>
#include "ccs.h"
#include "msg.h"


/*
 ****************************************************** 
 *  Logging System types and constant definitions     *
 ******************************************************
 */

#define logMAX_LEN     256  /* max. length of a log                      */
#define logTEXT_LEN    176  /* max. length of the user part of the log   */
#define logSYS_LEN      80  /* max. length of the system part of the log */

/*
 *   Define logId for Automatic logs of specific modules
 */

#define logAUTO_RANGE  60   /* logs with  0 < Id < 50 are automatic */

#define logAUTO_MSG    20    /* Message System Automatic logs */
#define logAUTO_DB     21    /* Database Automatic logs       */
#define logAUTO_SCAN   22    /* Scan System Automatic logs    */

#define logAUTO_USR1   58    /* User defined */
#define logAUTO_USR2   59    /* User defined */

#define logNORM_USER   100   

/*
 *   Define LogId for specific CCS/LCC modules 
 */

#define logLOG      (1 + logAUTO_RANGE)
#define logERR      (2 + logAUTO_RANGE)
#define logSCAN     (3 + logAUTO_RANGE)

/*
 * Message constants
 */

#define logSCOPE_ALL_STRING	 "All"
#define logSCOPE_SPECIFIC_STRING "Specific"

#define logEVT_MESSAGE_STRING	 "Message"
#define logEVT_READDB_STRING	 "Read DB"
#define logEVT_WRITEDB_STRING	 "Write DB"
#define logEVT_ANALOG_STRING	 "Analog"
#define logEVT_DIGITAL_STRING	 "Digital"


/*
 * Message formats
 */

typedef struct {                    /* Reply for LOGSTRT command        */
    ccsTIMEVAL      timeStamp;      /* Time stamp (UTC) of the log      */
    ccsPROCNAME     processName;    /* Name of process requesting log	*/
    ccsMODULEID     moduleId;       /* Name of module requesting log	*/
    vltUINT8        logId;          /* Log format definition identifier */
    ccsPROCNUM      procNum;        /* Procnum of process requesting log*/
    vltUINT8        fill[2];        /* Filling characters to have same  */
                                    /* structure length on WS and LCU   */
} logLOGSTRT_REPLY_HEADER;

/************************************************************************
 *                 Definitions for FITS logging
 ************************************************************************/

/* Define names for comment signature */

#define logOBSERVER            "OB"
#define logREMOTE_CONTROL      "RC"
#define logNIGHT_ASSISTANT     "NA"
#define logSTAFF               "SA"
#define logNULL                NULL

#define  logMASK_LEN         7
#define  logCATEGORY_LEN     3
#define  logKEY_LEN         24
#define  logDICT_LEN        12
#define  logACTION_LEN      24
#define  logFITS_TEXT_LEN  220 
#define  logOPERATOR_LEN     2
#define  logKEY_FIELD_SIZE  72

typedef  char logOPERATOR[logOPERATOR_LEN + 1];
typedef  char logMASK[logMASK_LEN + 1];
typedef  char logCATEGORY[logCATEGORY_LEN + 1];
typedef  char logFITS_KEY[logKEY_LEN + 1];
typedef  char logDICTIONARY[logDICT_LEN + 1];
typedef  char logACTION[logACTION_LEN + 1];

/* Define Types of Operational Logs  */

typedef enum {
    FITS_UNKNOWN = 1,  /* Fits log not recognized  */
    FITS_UNFORSEEN, 
    FITS_RECOVERY, 
    FITS_COMMENT,
    FITS_COMMENT_NN,
    FITS_ACTION,
    FITS_PAR_RECORD,
    FITS_PAR_ARRAY        
} logFITS_TYPE;       /* List log FITS type        */


/* Structure of the Fits log reply  */

typedef struct {
    logFITS_TYPE  fitsType;
    logMASK       fitsMask;
    logOPERATOR   fitsWho;
    logDICTIONARY fitsDict; 
    logFITS_KEY   fitsCat;
    logFITS_KEY   fitsSys;
    logFITS_KEY   fitsParam;
    logACTION     fitsAction;
    vltINT32      fitsDataSize;
    vltINT16      fitsFirst;
    vltINT16      fitsLast;
    vltUINT32     fitsLen;
   /* 
    * This member is used to propagate the flag to indicate if the
    * numerical data need byte swapping. Byte swapping is needed
    * when logs come from the LCU to a Linux environment.
    */
    vltLOGICAL    swapFlag; 
} logFITS_BODY;


/*
 *******************************************************************
 * LOGGING SYSTEM  : Application  Programmer  Routines             *
 *******************************************************************
 */

vltINT32 logCntPercentage (char *logText);

void logData1 (const ccsMODULEID  moduleId,vltUINT8 logId, const char *logText, ... )
__attribute__((format(printf,3,4)));

ccsCOMPL_STAT logData (         /* Stores a normal log in a log file       */
    const ccsMODULEID moduleId, /* Module reporting the log.               */
          vltUINT8    logId,    /* The Log format definition identifier.   */
    const char        *logText  /* ASCII string containing the data to be  */
                                /* logged, already formatted according to  */
                                /* logId. maximum 255 chars.               */
    );
 
    
/* This routine is shared with the error system          */
/* It should not be used to program control applications */ 
 
ccsCOMPL_STAT logToSyslog ( 
    char         logType,  /* Type of log */
    const char  *mylog     /* Complete log text */
    ); 


/* Sets the default value for parameter 'mask' in Fits logs */

void logFitsSetMask (
   const ccsMODULEID moduleId,
   const logMASK mask 
);

void logFitsGetMask (logMASK mask ); /* Internal routine */

/* Logs a string as Operational Log in Fits Format */

ccsCOMPL_STAT logFitsComment (
   const ccsMODULEID moduleId,  /* Module reporting the log.     */
   const char *who,             /* Who generated the log         */
   const char *text             /* String describing the comment */
);

/* Logs a string describing an 'Unforseen' event or a 'Recovery' action */
/* as Operational Log in Fits Format                                    */

ccsCOMPL_STAT logFitsEvent (
   const ccsMODULEID  module, /* Module reporting the log.                   */
   logFITS_TYPE fitsType,     /* Type of log : FITS_UNFORSEEN, FITS_RECOVERY */
   const char   *text         /* String describing the condition             */
);

/* Generates a log describing an action as Operational Log in Fits Format */

ccsCOMPL_STAT logFitsAction ( 
   const ccsMODULEID   module,     /* Module reporting the log.         */
   const logDICTIONARY fitsDict,   /* Dictionary where to check the log */
   const logCATEGORY   fitsCat,    /* Category                          */
   const logFITS_KEY   fitsSys,    /* System : identifies a component   */
   const logACTION     fitsAction  /* Verb describing the action        */
);


/* Generates a log recording information on the status of the components   */
/*  of a given system : typical examples are wind speed, guide star RA/DEC */
/*  or exposure number.                                                    */

ccsCOMPL_STAT logFitsParRecord ( 
   const ccsMODULEID    module,    /* Module reporting the log.          */
   const logDICTIONARY  fitsDict,  /* Dictionary where to check the log  */
   const logCATEGORY    fitsCat,   /* Category                           */
   const logFITS_KEY    fitsSys,   /* System : identifies a component    */
   const logFITS_KEY    fitsParam, /* System Parameter             */
   const void          *value      /* Current value for Parameter  */
);


/* The following routines support the different VLT data types    */

ccsCOMPL_STAT logStringParRecord ( 
   const ccsMODULEID    module,    /* Module reporting the log.          */
   const logDICTIONARY  fitsDict,  /* Dictionary where to check the log  */
   const logCATEGORY    fitsCat,   /* Category                           */
   const logFITS_KEY    fitsSys,   /* System : identifies a component    */
   const logFITS_KEY    fitsParam, /* System Parameter             */
         char           *value      /* Current value for Parameter */
);


ccsCOMPL_STAT logIntParRecord ( 
   const ccsMODULEID    module,    /* Module reporting the log.          */
   const logDICTIONARY  fitsDict,  /* Dictionary where to check the log  */
   const logCATEGORY    fitsCat,   /* Category                           */
   const logFITS_KEY    fitsSys,   /* System : identifies a component    */
   const logFITS_KEY    fitsParam, /* System Parameter            */
         int            value      /* Current value for Parameter */
);


ccsCOMPL_STAT logRealParRecord ( 
   const ccsMODULEID    module,    /* Module reporting the log.          */
   const logDICTIONARY  fitsDict,  /* Dictionary where to check the log  */
   const logCATEGORY    fitsCat,   /* Category                           */
   const logFITS_KEY    fitsSys,   /* System : identifies a component    */
   const logFITS_KEY    fitsParam, /* System Parameter            */
         double         value      /* Current value for Parameter */
);


/* Logs array of numbers describing the status of a the components   */
/* of a given system.                                                */

ccsCOMPL_STAT logFitsParArray ( 
   const ccsMODULEID    module,    /* Module reporting the log.          */
   const logDICTIONARY  fitsDict,  /* Dictionary where to check the log  */
   const logCATEGORY    fitsCat,   /* Category                           */
   const logFITS_KEY    fitsSys,   /* System : identifies a component    */
   const logFITS_KEY    fitsParam, /* System Parameter                   */
         vltINT16       first,     /* Index of the first element         */
         vltINT16       last,      /* Index of the last element          */
         vltINT16       elemSize,  /* Array element size                 */
         char          *array      /* Pointer to the first element       */
);

ccsCOMPL_STAT logAddUserFits (
   char      *msgBuf, 
   msgLENGTH  buffLen,
   ccsERROR  *error 
);


/***********************************************************************/
/*         Internal Routines but shared between CCS/LLC                */      
/***********************************************************************/

/* Sends the description of a Fits log to the logManager */

ccsCOMPL_STAT logSendFits ( 
   ccsENVNAME envName,        /* Module reporting the log.   */
   char      *buffer,         /* Pointer to message buffer   */
   vltINT32  bufLen           /* Buffer length               */
);

/* Fills the first part of the message buffer with the 'reply' header */
/* followed by the fits body : the latter gives a description of the  */
/* type of fits log                                                   */


void logBuildFitsHeader (
   char               *buffer,     /* Pointer to message buffer   */
   const ccsMODULEID   module,     /* Module reporting the log.   */
   logFITS_BODY       *fitsBody,   /* Structure describing the fits log */
   ccsENVNAME         envName,     /* Environment Name            */
   vltINT32           *bufferLen   /* returned buffer length      */
);

/* Return the name of the current default dictionary */

void logFitsGetDict( logDICTIONARY fitsDict );

/* Sets the name of the current default dictionary  */

void logFitsSetDict ( 
   const ccsMODULEID mod,
   const logDICTIONARY fitsDict 
);

/* Returns specific information to add to the log */

ccsCOMPL_STAT logGetInfo ( 
   ccsTIMEVAL   *timeVal,   /* Time stamp       */
   ccsPROCNAME  procName,   /* Process Name     */
   ccsPROCNUM   *procNum,   /* Process Number   */
   ccsENVNAME   envName     /* Environment Name */
);



#ifdef MAKE_VXWORKS
#include "logLCC.h"
#else
#include "logCCS.h"
#endif

#ifdef __cplusplus
}
#endif

#endif /*!LOG_H*/
