#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: acroxDevice.tcl,v 1.34 2005/09/26 15:59:37 vltsccm Exp $" 
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 16.03.1995 created
# imunoz     14.04.1996 itcl 2.0 port
#

#************************************************************************
#   NAME
#
#	AcroxDevice - Class description for a "acro" digital I/O device
# 
#   SYNOPSIS
#
#	AcroxDevice <object-name> ?<options>?
#
#   DESCRIPTION
#
#	AcroxDevice defines the class for a "acro" digital I/O device.
#	It makes the functionality of the device and the corresponding 
#	"acro" driver available through a graphical user interface.
#
#	An object of this class is created by:
#
#		AcroxDevice <object-name> -<Option> <Value> ...
#
#	This will create all sub-objects for user interaction and
#	an object to handle the connection to a server-task in the 
#	LCU environment i.e. effectively to the physical device.
#
#   OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<object-name> configure -<Option> <Value> ...
#
#	The following device-specific options are publicly available:
#
#	-numberOfSignals	specifies the total number of signals
#				of the device. The maximum value and default
#				is 64.
#
#	-numberOfRows		specifies the number of rows in which the
#				signals shall be organized on the layout.
#				The default is 2.
#
#	-firstSignal		specifies the number of the first signal.
#				The default is 0.
#
#	Several options are inherited from InducerDevice and are therefore also
#	available within the scope of this class. See InducerDevice(n) for a 
#	detailed description of those common options.
#
#   METHODS
#	Methods represent procedures on the object and are invoked with:
#
#		<object-name> <Method> <Arguments> ...
#
#	There are no special device-specific methods available.
#
#	Several method are inherited from InducerDevice and overloaded here
#	with device-specific actions. See InducerDevice(n) for a detailed
#	description of those common methods.
#
#   PROCS
#
#   EXAMPLES
#
#   SEE ALSO
#	acroxInducer(1),
#	InducerClass(n), InducerDevice(n), InducerLine(n)
#
#   BUGS
#
#------------------------------------------------------------------------


#------------------------------------------------------------------------
#*# class: AcroxDevice

class AcroxDevice {
    
    inherit InducerDevice
    
    #--------------------------------------------------------------------
    # Device-specific members of AcroxDevice class
    #--------------------------------------------------------------------
    
    #*# constructor
    constructor { args } { }
    
    # widget's options
    itk_option define -numberOfRows     numberOfRows    NumberOfRows     2 
    itk_option define -numberOfSignals	numberOfSignals NumberOfSignals  64
    itk_option define -firstSignal      firstSignal	FirstSignal      0
    #--
    protected variable signalW		{}  ;# list of object-names of Signals
    protected variable configW          {}

    #--------------------------------------------------------------------
    # Overloaded standard members of InducerDevice class
    #--------------------------------------------------------------------

    public method GetVersion { } { }
    public method InitDevice { } { }
    public method ExitDevice { } { }
    public method ResetDevice { } { }
    public method GetDeviceId { } { }
    public method Disconnect { } { }
    public method Resume { } { } 
    public method GetSignalConfig { } { }
    public method SaveDevice { } { }
    public method LoadDevice { data } { }
    public method LoadFromLCC { } { }

    #-------------------------------------------------------------
    # Local Methods
    #-------------------------------------------------------------
   
    private method getDigitalConfig { name } { }
    private method setPreconfSignal { reply  } { }
    private method indexOf { number src } { }
}

#*# constructor
body AcroxDevice::constructor { args } {
    
    # integrate composite options
    eval itk_initialize $args
    set numberOfSignals $itk_option(-numberOfSignals)
    set numberOfRows    $itk_option(-numberOfRows)
    set firstSignal     $itk_option(-firstSignal)
    
    # build the Signals of which the Device consists:
    set signalW {} ;# list of signals (object-names)
    set configW {}
    set numberOfSignalsPerRow [expr {$numberOfSignals / $numberOfRows}]
    set iMax [expr {$numberOfRows - 1}]
    set jMax [expr {$numberOfSignalsPerRow - 1}]

    # loop for each row:
    for {set i 0} {$i <= $iMax} {incr i} {
	itk_component add F$i {
	    set F [frame $itk_interior.$i]
	}
	pack $F -side top -anchor n -pady 4 -fill both -expand 1
	
	# loop for each signal in a row:
	for {set j 0} {$j <= $jMax} {incr j} {
	    set f [expr {$j == 0} ? 1 : 0]
	    set k [expr {$numberOfSignalsPerRow*$i+$j+$firstSignal}]
	    # create signal object:
	    itk_component add S$i.$j {
		set S [AcroxSignal $F.$k \
			-signalNumber $k -showLabels $f \
			-borderwidth 1 -relief sunken]
	    }
	    pack $S -side right -anchor e -fill both -expand 1
	    lappend signalW $S ;# keep track of the object-names
	}
	
	# all Signals must use the same Line object:
	AcroxSignal::setLine $lineW
    }
    
}
#--------------------------------------------------------------------
# Overloaded standard members of InducerDevice class
#--------------------------------------------------------------------

body AcroxDevice::GetVersion { } {
    debugPrint
    global acrox argv0
    return "[file tail $argv0] - Version $acrox(version)"
}

body AcroxDevice::InitDevice { } {
    debugPrint

    set err [InducerDevice::InitDevice] ;# common part to open channel
    if {$err != ""} {return $err}

    AcroxSignal::startMonitor
    # set onLine state
    AcroxSignal::setOnLine 1
    return ""
}

body AcroxDevice::ExitDevice { } {
    debugPrint
    AcroxSignal::stopMonitor
    return [InducerDevice::ExitDevice]	;# common part to close channel
}

body AcroxDevice::ResetDevice { } {
    debugPrint
    return [$lineW Ioctl RESET 0]
}

body AcroxDevice::GetDeviceId { } {
    debugPrint
    set id [$lineW Ioctl READ_BOARD_ID 0]
    # The driver returns a pointer to a string. No chance to read it!
    return <unknown>
}

body AcroxDevice::Disconnect { } {
    
    InducerDevice::Disconnect

    AcroxSignal::stopMonitor
    AcroxSignal::setOnLine 0
    
}

body AcroxDevice::Resume { } {

    InducerDevice::Resume

    AcroxSignal::setOnLine 1
    AcroxSignal::updateAll
    AcroxSignal::startMonitor
}

body AcroxDevice::GetSignalConfig { } {
    debugPrint
    # Father's behavior - Get cnfList
    InducerDevice::GetSignalConfig
    foreach i $cnfList {
	set name [givemeName $i]
	set type [givemeType $i]
	if { $type == "Digital" } {
	    set reply [getDigitalConfig $name]
	    set reply [split $reply ,]
	    # Check device's name
	    if { [lindex $reply 0] != $itk_option(-vDev) } {
		continue
	    }
	    lappend reply $name
	    lappend configW $reply 
	    #
	    # Don't try to set LCC info until it's requested by the user
	    # setPreconfSignal $reply
	}
    }
    debugPrint $configW
    return $configW
}

body AcroxDevice::SaveDevice { } {
    set data "#-- $itk_option(-vEnv)\($itk_option(-vDev)\) digital signals configuration\n"
    foreach w $signalW {
	set line [$w report]
	
	set number "NUMBER : [lindex $line 0]\n"
	set name   "  NAME : [lindex $line 1]\n"
	set io     " INOUT : [lindex $line 2]\n"
	set state  " STATE : [lindex $line 3]\n"
	set active "ACTIVE : [lindex $line 4]\n"
	set gw     " GROUP : [lindex $line 5]\n"
	
	set data "$data\n $number $name $io $state $active $gw"
    }
    return $data
}

body AcroxDevice::LoadDevice { data } { 
    
    set cnf [split $data \n]
    
    set number -1
    set name   ""
    set io     -1
    set state  -1 
    set hl      ""
    set gw     -1
    
    foreach el $cnf {
	# skip blank lines
	if {[regexp {^[ \t]*$} $el ]} {
	    continue
	}   
	
	#skip comments
	if {[string match \#* $el]} {
	    continue
	}
	regexp {[A-Z]+ :} $el head
	regsub {[A-Z]+ : *} $el "" tok
	regsub {^[ \t]+} $tok "" token
	
	switch $head {
	    "NUMBER :" {
		set number $token
	    }
	    "NAME :" {
		set name $token
	    }
	    "INOUT :" {
		set io $token
	    }
	    "STATE :" {
		set state $token
	    }
	    "ACTIVE :" {
		set hl $token
	    }
	    "GROUP :" {
		set gw $token
		# tiny error handling
		if { $number < 0 } {
		    set name   ""
		    set io     -1
		    set state  -1 
		    set hl      ""
		    set gw     -1
		    continue
		}
		set msg "dummy $number $gw $io $hl dummy [list $name]" 
		setPreconfSignal $msg		    
		set i [indexOf $number $signalW]
		#		    if {$i >= 0} {
		#			set signal [lindex $signalW $i]
		#			$signal setState $state
		#                    }
	    }
	}
    }
}

body AcroxDevice::LoadFromLCC { } {
    # first reset all signals to idle state:
    foreach i $signalW {
	#                ?? num gw io hl ?? name
	$i setData [list "" ""  1  0  0  "" ""  ]
    }
    # then set those signals that are actually LCC-configured:
    foreach i $configW {
	debugPrint $i
	setPreconfSignal $i
	##update
    }
}

#-------------------------------------------------------------
# Local Methods
#-------------------------------------------------------------

body AcroxDevice::getDigitalConfig { name } {
    return [sendServer $itk_option(-vEnv) lccServer DIOGCNF $name ]
}

body AcroxDevice::setPreconfSignal { reply  } {
    debugPrint
    set number [lindex $reply 1]
    set i [indexOf $number $signalW]
    # check if the signal is present
    if { $i >= 0 } {
	set signal [lindex $signalW $i]
	$signal setData $reply		;# set config incl. master-bit
	
	# group's stuff
	set gw [lindex $reply 2]
	if { $gw > 1 } {
	    set count 1
	    while { $count < $gw } {
		set signal [lindex $signalW [expr {$i+$count}]]
		if { $signal == "" } {
		    break
		}
		$signal setData $reply	;# set slave equal to master
		$signal setMaster 0		;# and define as slave
		incr count
	    }
	}
    }
}

body AcroxDevice::indexOf { number src } {
    set found 0
    set pos 0
    foreach i $src {
	set snum [$i info variable signalNumber -value]
	if {$snum == $number} {
	    set found 1
	    break
	}
	incr pos
    }
    if { $found == 0 } {
	return -1
    }
    return $pos
}
#___oOo___
