#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: acroxSignal.tcl,v 1.34 2005/09/26 15:59:37 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# bgustafs 2002-04-19 changed default pulse lenght to 1 tick
# S.Sandrock 16.03.1995 created
# S.Sandrock 11.11.1995 added bitmaps for buttons
# imunoz     14.04.1996 itcl2.0 port
#

#************************************************************************
#   NAME
#
#	AcroxSignal - Class description for signal of a digital I/O device
# 
#   SYNOPSIS
#
#	AcroxSignal <widget-name> ?<options>?
# 
#   DESCRIPTION
#
#	AcroxSignal defines the class for a signal of a digital I/O device.
#	It makes the functionality of the signal and the corresponding 
#	"acro" driver available through a graphical user interface.
#
#	An object of this class is created by:
#
#		AcroxSignal <object-name> -<Option> <Value> ...
#
#	which is typically done by the constructor of the AcroxDevice class.
#
#   OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<object-name> configure -<Option> <Value> ...
#
#	The following options are publicly available:
#
#	-signalNumber	specifies the number of the signal within the device.
#			It can be in the range 0 to 63, the default is 0.
#
#	-linearNumbering selects linear (0-63) or byte-wise (0.0-7.7) labels
#			Default: 1 (linear)
#
#	-showLabels	specifies whether the buttons of the signal shall be
#			displayed with or without labels. Usually only the
#			rightmost signal in a row should be displayed with
#			labels, otherwise there would be too little space.
#			"yes" shows the labels, which is the default.
#
#	-useBitmaps	whether to use symbols or pure text as labels
#			Default: 1
#
#	-pedantic	if 1, some control elements are enabled or disabled
#			dynamically depending on some other values.
#			Since this is done inside a monitor-loop, it can be
#			VERY time-consuming. That is why it is usually better
#			not to do this dynamic widget-update.
#			The default is 0 (off).
#
#	The following colors can be set usually via the option database:
#
#	-dirSelectColor selector of Input/Output checkbutton	Yellow
#	-actSelectColor	selector of active H/L checkbutton	Blue
#	-setSelectColor	selector of SET radiobutton		Red
#	-clrSelectColor	selector of CLEAR radiobutton		Green
#	-mstSelectColor	selector of Master/Slave checkbutton	Black
#	-configColor	indicator for configured signal		Orange
#
#   METHODS
#	Methods are invoked with:
#
#		<object-name> <Method> <Arguments> ...
#
#	Most of the methods in this class are only used locally.
#
#	The following methods are currently intended to be used from outside:
#
#	startMonitor	- to start the monitoring of the device-status to
#			  update the corresponding signal-objects.
#			  The link to the LCU environment via a InducerLine
#			  object must have been established before.
#
#	stopMonitor	- to stop the monitoring.
#
#   PROCS
#	Procs are invoked with:
#
#		<object-name> <Proc> <Arguments> ...
#
#	The following procs are publicly available:
#
#	setLine <object-name>	- make the object-name of the InducerLine
#		object known to the signal, that is used to communicate with
#		the LCU environment. All signals of a device must use the
#		same line-object.
#
#   CAUTIONS
#	The class uses common variables.
#	Only up to 64 signal (blocks 0 and 1) are supported.
#	All signals must belong to the same device.
#	In this implementation it is not possible to have signals
#	that belong to different devices at the same time.
#
#   EXAMPLES
#
#   SEE ALSO
#	acroxInducer(1),
#	InducerClass(n), InducerDevice(n), AcroxDevice(n)
#
#   BUGS
#
#------------------------------------------------------------------------

# NOTE: Tk widgets in this class have to be connected to global 
# variables (vars in the class scope do not work with Tk). These vars 
# are stored in an array named as the object's name, i.e. "${this}".
# The value of such a variable can be queried with "[set ${this}(..)]".
# All methods using these vars must include a `global' statement:

# Define bitmaps:
blt::bitmap define acrox_nil {{1 16}  {0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0}}
blt::bitmap define acrox_16  {{16 16} {
    0xff, 0xff, 0x01, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01, 0x80,
    0x01, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01, 0x80,
    0x01, 0x80, 0x01, 0x80, 0x01, 0x80, 0xff, 0xff}}
blt::bitmap define acrox_io  {{16 16} {
    0xff, 0xff, 0x41, 0x83, 0x21, 0x87, 0x11, 0x8f, 0x09, 0x9f, 0x05, 0xbf,
    0x03, 0xff, 0x01, 0xff, 0x01, 0xff, 0x03, 0xff, 0x05, 0xbf, 0x09, 0x9f,
    0x11, 0x8f, 0x21, 0x87, 0x41, 0x83, 0xff, 0xff}}
blt::bitmap define acrox_as {{16 16} {
    0xff, 0xff, 0xc1, 0x83, 0xe1, 0x87, 0xf1, 0x8f, 0xf9, 0x9f, 0xfd, 0xbf,
    0xff, 0xff, 0xff, 0xff, 0x01, 0x80, 0x03, 0xc0, 0x05, 0xa0, 0x09, 0x90,
    0x11, 0x88, 0x21, 0x84, 0x41, 0x82, 0xff, 0xff}}
blt::bitmap define acrox_s1 {{16 16} {
    0xff, 0xff, 0x41, 0x82, 0x21, 0x84, 0x11, 0x89, 0x89, 0x91, 0xc5, 0xa1,
    0xc3, 0xc1, 0x81, 0x81, 0x81, 0x81, 0x83, 0xc1, 0x85, 0xa1, 0xc9, 0x93,
    0xd1, 0x8b, 0x21, 0x84, 0x41, 0x82, 0xff, 0xff}}
blt::bitmap define acrox_s0 {{16 16} {
    0xff, 0xff, 0x41, 0x82, 0x21, 0x84, 0x91, 0x89, 0xc9, 0x93, 0x65, 0xa6,
    0x63, 0xc6, 0x61, 0x86, 0x61, 0x86, 0x63, 0xc6, 0x65, 0xa6, 0xc9, 0x93,
    0x91, 0x89, 0x21, 0x84, 0x41, 0x82, 0xff, 0xff}}
blt::bitmap define acrox_p  {{16 16} {
    0xff, 0xff, 0x01, 0x80, 0x01, 0x80, 0xf9, 0x9f, 0x19, 0x98, 0x19, 0x98,
    0x19, 0x98, 0x19, 0x98, 0x19, 0x98, 0x19, 0x98, 0x19, 0x98, 0x19, 0x98,
    0x1f, 0xf8, 0x01, 0x80, 0x01, 0x80, 0xff, 0xff}}
blt::bitmap define acrox_r  {{16 16} {
    0xff, 0xff, 0x01, 0x80, 0x01, 0x88, 0x3f, 0x88, 0x31, 0x88, 0x31, 0x88,
    0x31, 0x88, 0x31, 0x88, 0x31, 0xbe, 0x31, 0x9c, 0x31, 0x88, 0x31, 0x80,
    0xf1, 0xff, 0x01, 0x80, 0x01, 0x80, 0xff, 0xff}}
blt::bitmap define acrox_o  {{16 16} {
    0xff, 0xff, 0x03, 0x80, 0x05, 0x80, 0xf9, 0xff, 0x09, 0x80, 0xc9, 0x9f,
    0xc9, 0x9a, 0x49, 0x95, 0xc9, 0x9a, 0x49, 0x95, 0xc9, 0x9a, 0x49, 0x95,
    0xc9, 0x9a, 0xc9, 0x9f, 0x09, 0x80, 0xff, 0xff}}
blt::bitmap define acrox_m  {{16 16} {
    0xff, 0xff, 0xff, 0x83, 0xbf, 0x85, 0x9f, 0x89, 0x8f, 0x91, 0x87, 0xa1,
    0x83, 0xc1, 0xff, 0xff, 0xff, 0xff, 0x83, 0xc1, 0x87, 0xa1, 0x8f, 0x91,
    0x9f, 0x89, 0xbf, 0x85, 0xff, 0x83, 0xff, 0xff}}

# class AcroxSignal

option add *AcroxSignal.dirSelectColor Yellow   widgetDefault
option add *AcroxSignal.actSelectColor Blue     widgetDefault
option add *AcroxSignal.setSelectColor Red      widgetDefault
option add *AcroxSignal.clrSelectColor Green    widgetDefault
option add *AcroxSignal.mstSelectColor Black    widgetDefault
option add *AcroxSignal.configColor    Orange   widgetDefault


usual AcroxSignal { keep -relief}

class AcroxSignal {
    
    inherit InducerClass
    
    #*# constructor
    constructor { args } { }

    #*# Class options
    itk_option define -signalNumber    signalNumber    SignalNumber    0
    itk_option define -linearNumbering linearNumbering LinearNumbering 1
    itk_option define -showLabels      showLabels      ShowLabels      1
    itk_option define -useBitmaps      useBitmaps      UseBitmaps      1
    itk_option define -configColor     configColor     ConfigColor     Orange

    #--------------------------------------------------------------------
    #*# method: shortHelp
    method shortHelp { } { }
    #--------------------------------------------------------------------
    protected variable numberOfSignals 64 ;# constant value
    protected variable signalsPerBlock 32 ;# constant value
    
    protected variable blockNumber 0 ;# calc. from signalNumber/signalsPerBlock
    protected variable bitMask	;# calculated from signalNumber and blockNumber

    public variable pedantic 0	;# if 1, re-configure some widget-enable-states
				;# which are dependent on others.
				;# Is VERY time-consuming!
                                ;# See method showSignalConfig
    
    protected variable signalNumber ;# linear number 0..63

    protected variable normalColor  ;# ie. color for unconfigured signals

    common linearNumbering  1	;# global flag: linear or byte-wise labels
#@@    proc SetNumbering {} {}	;# sets label numbering for all class instances

    common useBitmaps  1	;# global flag: text or bitmaps as labels
    proc SetLabels {} {}	;# sets labels for all class instances

    #--------------------------------------------------------------------
    # METHODS FOR SINGLE SIGNAL MODIFICATION
    #--------------------------------------------------------------------
    
    method setSignal { } { }
    method clearSignal { } { }
    method getState { } { }
    method setState { state } { }
    method writeSignal { {State ""} } { }
    method configSignal { {InputOutput ""} {HighLow ""} } {}

    #--------------------------------------------------------------------
    # METHODS FOR SINGLE SIGNAL WIDGET UPDATE FROM BITVECTORS
    #--------------------------------------------------------------------

    method showSignalState { } { }
    method showSignalConfig { } { }

    #--------------------------------------------------------------------
    # METHODS FOR BITVECTORS UPDATE
    #--------------------------------------------------------------------

    method readSignalState { } { }

    # NOTE: there is no `readSignalConfig', because a change of config may
    # always result in a change of state, e.g. due to change of activeState 
    # the state may change!

    method readSignal { } { }
    #--------------------------------------------------------------------
    # METHODS FOR PULSE SINGLE SIGNALS
    #--------------------------------------------------------------------

    method pulseSignal { {Ticks ""} } { }
    method resetPulse { } { }
    method readPulseState { } { }
    #--------------------------------------------------------------------
    # METHODS FOR SIGNAL GROUPS
    #--------------------------------------------------------------------

    method writeGroup { {Value ""} } { }
    method readGroup { } {}
    method setGroupWidth { Width } { }
    method configGroup { InputOutput HighLow } { }
    
    #--------------------------------------------------------------------
    # METHODS FOR MASTER HANDLING
    #--------------------------------------------------------------------

    method setMaster { {YesNo ""} } { }
    #--------------------------------------------------------------------
    # OPTION PANEL
    #--------------------------------------------------------------------

    method optionPanel { } { }

    #-------------------------------------------------------------------------
    #  setData from LCC, Report to Load/Save
    #-------------------------------------------------------------------------
    method setData { data } { }
    method report { } { }
    #-------------------------------------------------------------------------
    #  common variables and procs
    #-------------------------------------------------------------------------

    common objectNames {}
    common onLine 0

    proc setOnLine { onOff } {  set onLine $onOff }
    proc getOnLine { } { }
    proc showAll { } { }
    proc updateAll { } { }

    # bit-wise representation of all signals: `xxx(blockNumber)'

    common masterBits
    set masterBits(0)	0xffffffff	;# signal # 31..0
    set masterBits(1)	0xffffffff	;# signal # 63..32

    common stateBits
    set stateBits(0)	0x00000000	;# signal # 31..0
    set stateBits(1)	0x00000000	;# signal # 63..32

    common dirBits
    set dirBits(0)	0x00000000	;# signal # 31..0
    set dirBits(1)	0x00000000	;# signal # 63..32

    common polBits
    set polBits(0)	0x00000000	;# signal # 31..0
    set polBits(1)	0x00000000	;# signal # 63..32

    proc readConfigBits { } { }
    proc writeConfigBits { } { }
    proc readStateBits { } { }
    proc writeStateBits { } { }
    proc readBits { } { }
    proc writeBits { } { }

    # update bits from an external reply, used internally for monitoring:
    proc monitorCB { Reply } { }
    proc startMonitor { } { }
    proc stopMonitor { } { }

    #---------------------------------------------
    #  Local procs 
    #---------------------------------------------

    common lineW ""	;# object-name of the LCU connection, set by Device-obj

    proc setLine { Line } { }
    # send IOCTL command, ignore driver errors:
    proc sendIoctl { Cmdlist Arglist {Sizlist ""} } { }
    # send IOCTL command, trap any driver errors with a dialog-box:
    proc sendIoctlTrap { Cmdlist Arglist {Sizlist ""}} { }
}

body AcroxSignal::constructor { args } {
    global ::${itk_interior}	;# global array of "pseudo-local" variables

    itk_option add hull.relief
    itk_option add hull.borderwidth

    # initialize "pseudo-local" variables:
    set ${itk_interior}(signalDir)	0	;# 0: input	1: output
    set ${itk_interior}(activeState)	0	;# 0: act-low	1: act-high
    set ${itk_interior}(signalState)	0	;# 0: CLEAR	1: SET
    set ${itk_interior}(masterSignal)	1	;# 0: Slave	1: Master
    
    set ${itk_interior}(pulseTicks)	1	;# number of ticks for pulse
    set ${itk_interior}(pulseState)	0	;# 0: Reset	1: Pulsed
    
    set ${itk_interior}(groupValue)	0	;# value of signal-group
    set ${itk_interior}(groupFormat)	%d	;# format-specifier: %d or 0x%x
    set ${itk_interior}(groupWidth)	1	;# width of signal-group
    set ${itk_interior}(groupLabel)	""	;# text-label "(max..min)"
    set ${itk_interior}(signalName)     ""      ;# LCC or personal signal name

    itk_component add nr {
	menubutton $itk_interior.nr -relief flat -width 2
    }
    set normalColor [$itk_component(nr) cget -background] ;# query current bg

    itk_component add nrmenu {
	menu $itk_component(nr).menu -tearoff 0
    } {
	ignore -tearoff
    }

    $itk_component(nr) configure -menu $itk_component(nrmenu)

    $itk_component(nrmenu) add command -label Config... \
	    -command [code $this optionPanel]
    $itk_component(nrmenu) add separator
    $itk_component(nrmenu) add radiobutton -label "All linear" \
	    -variable [scope linearNumbering] -value 1 \
	    -command [code SetLabels]
    $itk_component(nrmenu) add radiobutton -label "All byte-wise" \
	    -variable [scope linearNumbering] -value 0 \
	    -command [code SetLabels]
    $itk_component(nrmenu) add separator
    $itk_component(nrmenu) add radiobutton -label "All symbols" \
	    -variable [scope useBitmaps] -value 1 \
	    -command [code SetLabels]
    $itk_component(nrmenu) add radiobutton -label "All text" \
	    -variable [scope useBitmaps] -value 0 \
	    -command [code SetLabels]

    itk_component add io {
	checkbutton $itk_interior.io -bd 1 -anchor e -bitmap acrox_nil \
		-variable ${itk_interior}(signalDir) \
		-command [code $this configSignal]
    } {
	usual
	rename -selectcolor -dirSelectColor dirSelectColor DirSelectColor
    }
    itk_component add as {
	checkbutton $itk_interior.as -bd 1 -anchor e -bitmap acrox_nil \
		-variable ${itk_interior}(activeState) \
		-command [code $this configSignal]
    } {
	usual
	rename -selectcolor -actSelectColor actSelectColor ActSelectColor
    }
    itk_component add s1 {
	radiobutton $itk_interior.s1 -bd 1 -anchor e -bitmap acrox_nil \
		-variable ${itk_interior}(signalState) -value 1 \
		-command [code $this setSignal]
    }  {
	usual
	rename -selectcolor -setSelectColor setSelectColor SetSelectColor
    }
    itk_component add s0 {
	radiobutton $itk_interior.s0 -bd 1 -anchor e -bitmap acrox_nil \
		-variable ${itk_interior}(signalState) -value 0 \
		-command [code $this clearSignal]
    }  {
	usual
	rename -selectcolor -clrSelectColor clrSelectColor ClrSelectColor
    }
    itk_component add p {
	button $itk_interior.p -bd 1 -anchor center -bitmap acrox_p \
		-command [code $this pulseSignal]
    } {
	usual
    }
    itk_component add r {
	button $itk_interior.r -bd 1 -anchor center -bitmap acrox_r \
		-command [code $this resetPulse]
    } {
	usual
    }
    itk_component add o {
	button $itk_interior.o -bd 1 -anchor center -bitmap acrox_o \
	    -command [code $this optionPanel]
    } {
	usual
    }
    itk_component add m {
	checkbutton $itk_interior.m -bd 1 -anchor e -bitmap acrox_nil \
		-variable ${itk_interior}(masterSignal) \
		-command [code $this setMaster]
    } {
	usual
	rename -selectcolor -mstSelectColor mstSelectColor MstSelectColor
    }

    eval itk_initialize $args
    # set short help text for each widget
    shortHelp
    
    # pack the widgets all in a vertical row:
    pack  \
	    $itk_component(nr) $itk_component(io) $itk_component(as)\
	    $itk_component(s1) $itk_component(s0) \
	    $itk_component(p) $itk_component(r) $itk_component(o) \
	    $itk_component(m) \
	    -side top -fill both -expand 1
    #-------------------------------------------
    # add object name to the list:
    # NOTE that the signalNumbers must be created in order!
    lappend objectNames $this
    
}

#--------------------------------------------------------------------
#*# method: shortHelp
    
body AcroxSignal::shortHelp { } { 
    # the `wHelparea' is inherited and set in `inducerClass'
    
    configure -helptext  "Digital Signal $itk_option(-signalNumber) \
    	    (Port [expr {$signalNumber/8}].[expr {$signalNumber%8}])"
    
    setShortHelp  $itk_component(io)\
	    "Yellow means this signal is an OUTPUT, otherwise INPUT.\
	    Click on the button to toggle the signal direction."
    
    setShortHelp $itk_component(as)\
	    "Blue means this signal is Active HIGH, otherwise Active LOW.\
	    Click on the button to toggle the signal activity."
    
    setShortHelp $itk_component(s1)\
	    "Red means this signal is SET (active).\
	    Click on the button to set it to the active state\
	    (only for outputs)."
    
    setShortHelp $itk_component(s0)\
	    "Green means this signal is CLEARed (inactive).\
	    Click on the button to clear it to inactive state\
	    (only for outputs)."
   
    setShortHelp  $itk_component(p)\
	    "Single Pulse this signal.\
	    Click 'Option...' to specify the pulse-width."
   
    setShortHelp $itk_component(r)\
	    "Reset Single Pulse before it has expired"
   
    setShortHelp $itk_component(o)\
	    "Open options panel for this signal\
	    with signal-name, pulsing, signal-groups.\
	    This is disabled for slave-signals."
   
    setShortHelp $itk_component(m)\
	    "Black means this signal is a group-MASTER\
	    (first signal in a group), otherwise SLAVE."
}

configbody AcroxSignal::signalNumber  {
    set signalNumber $itk_option(-signalNumber)

    if {[winfo exists $itk_component(nr)]} {
	$itk_component(nr) configure -text $signalNumber
    }
    set blockNumber [expr {$signalNumber / $signalsPerBlock}]
    set bitMask [expr 1 << \
		     {[expr {$signalNumber - $signalsPerBlock * $blockNumber}]}]
    debugPrint "signalNumber=$signalNumber blockNumber=$blockNumber\
	    bitMask=[format 0x%08x $bitMask]"
}

configbody AcroxSignal::showLabels  {

    if {$itk_option(-linearNumbering)} {
	set label $itk_option(-signalNumber)
    } else {
	set label [expr {$itk_option(-signalNumber) / 8}]
	append label .[expr {$itk_option(-signalNumber) % 8}]
    }
    $itk_component(nr) configure -text $label

    if {$itk_option(-useBitmaps)} {
      if {$itk_option(-showLabels)} {
	  $itk_component(io) configure -anchor center -bitmap acrox_io
	  $itk_component(as) configure -anchor center -bitmap acrox_as
	  $itk_component(s1) configure -anchor center -bitmap acrox_s1
	  $itk_component(s0) configure -anchor center -bitmap acrox_s0
	  $itk_component(p)  configure -anchor center -bitmap acrox_p
	  $itk_component(r)  configure -anchor center -bitmap acrox_r
	  $itk_component(o)  configure -anchor center -bitmap acrox_o
	  $itk_component(m)  configure -anchor center -bitmap acrox_m
      } else {
	  $itk_component(io) configure -anchor center -bitmap acrox_nil
	  $itk_component(as) configure -anchor center -bitmap acrox_nil
	  $itk_component(s1) configure -anchor center -bitmap acrox_nil
	  $itk_component(s0) configure -anchor center -bitmap acrox_nil
	  $itk_component(p)  configure -anchor center -bitmap acrox_p
	  $itk_component(r)  configure -anchor center -bitmap acrox_r
	  $itk_component(o)  configure -anchor center -bitmap acrox_o
	  $itk_component(m)  configure -anchor center -bitmap acrox_nil
      }
    } else {
      if {$itk_option(-showLabels)} {
	  $itk_component(io) configure -anchor w -bitmap {} -text Output/Input
	  $itk_component(as) configure -anchor w -bitmap {} -text ActiveHi/Lo
	  $itk_component(s1) configure -anchor w -bitmap {} -text SET
	  $itk_component(s0) configure -anchor w -bitmap {} -text CLEAR
	  $itk_component(p)  configure -bitmap {} -text Pulse
	  $itk_component(r)  configure -bitmap {} -text Reset
	  $itk_component(o)  configure -bitmap {} -text Option...
	  $itk_component(m)  configure -anchor w  -bitmap {} -text Master/Slave
      } else {
	  $itk_component(io) configure -anchor w -bitmap {} -text {}
	  $itk_component(as) configure -anchor w -bitmap {} -text {}
	  $itk_component(s1) configure -anchor w -bitmap {} -text {}
	  $itk_component(s0) configure -anchor w -bitmap {} -text {}
	  $itk_component(p)  configure -bitmap {} -text P
	  $itk_component(r)  configure -bitmap {} -text R
	  $itk_component(o)  configure -bitmap {} -text ...
	  $itk_component(m)  configure -anchor w -bitmap {} -text {}
      }
    }
}

body AcroxSignal::SetLabels { } {
    # reconfigure widget components:
    foreach w $objectNames {
	$w configure -useBitmaps $useBitmaps -linearNumbering $linearNumbering\
		-showLabels [$w cget -showLabels]
    }
    # update wm options:
    wm minsize . 0 0
    update
    wm geometry . {} ;# to natural size
    global acrox
    scan [wm geometry .] {%dx%d} acrox(w) acrox(h)
    if {$acrox(w) < 784} {set acrox(w) 784}
    wm geometry . $acrox(w)x$acrox(h)
    wm minsize . $acrox(w) $acrox(h)
    wm resizable . 1 1
}

#--------------------------------------------------------------------
# METHODS FOR SINGLE SIGNAL MODIFICATION
#--------------------------------------------------------------------

body AcroxSignal::setSignal { } {
    global ${itk_interior}
    
    set ${itk_interior}(signalState) 1
    debugPrint "signalState [set ${itk_interior}(signalState)]"
    sendIoctlTrap WRITE_SET_BIT $signalNumber
    readSignalState   ;# read-back & update signal-specific widgets
}

body AcroxSignal::clearSignal { } {
    global ${itk_interior}
    
    set ${itk_interior}(signalState) 0
    debugPrint "signalState [set ${itk_interior}(signalState)]"
    sendIoctlTrap WRITE_CLEAR_BIT $signalNumber
    readSignalState   ;# read-back & update signal-specific widgets
}
body AcroxSignal::getState { } {
    global ${itk_interior}
    return [set ${itk_interior}(signalState)]
} 

body AcroxSignal::setState { state } {
    global ${itk_interior}
    
    set ${itk_interior}(signalstate) $state
}

body AcroxSignal::writeSignal { {State ""} } {
    global ${itk_interior}
    # Write if it's output
    if {! [set ${itk_interior}(signalDir)] } {
	return [set ${itk_interior}(signalState)]
    }
    
    if {$State != ""} {set ${itk_interior}(signalState) $State}
    debugPrint "signalstate [set ${itk_interior}(signalState)]"
    
    if {[set ${itk_interior}(signalState)]} {
	sendIoctlTrap WRITE_SET_BIT $signalNumber
    } else {
	sendIoctlTrap WRITE_CLEAR_BIT $signalNumber
    }
    readSignalState   ;# read-back & update signal-specific widgets
    return  [set ${itk_interior}(signalState)]
}

body AcroxSignal::configSignal { {InputOutput ""} {HighLow ""} } {
    global ${itk_interior}
    
    # translate the given arguments into 0/1:
    if {$InputOutput != ""} {
	set ${itk_interior}(signalDir) [expr {$InputOutput=="OUTPUT"}]
    }
    if {$HighLow != ""} {
	set ${itk_interior}(activeState) [expr {$HighLow=="HIGH"}]
    }
    
    sendIoctlTrap WRITE_CONFIG_BITS \
	    "$signalNumber;1;[set ${itk_interior}(signalDir)];[set ${itk_interior}(activeState)]"
    
    debugPrint "signalDir=[set ${itk_interior}(signalDir)]\
	    activeState=[set ${itk_interior}(activeState)]"
    
    readSignal  ;# read-back & update signal-specific widgets
}

#--------------------------------------------------------------------
# METHODS FOR SINGLE SIGNAL WIDGET UPDATE FROM BITVECTORS
#--------------------------------------------------------------------

body AcroxSignal::showSignalState { } {
    global ${itk_interior}
    set state [expr {($stateBits($blockNumber) & $bitMask)} != 0]
    set ${itk_interior}(signalState) $state
}

body AcroxSignal::showSignalConfig { } {
    global ${itk_interior}
    set dir [expr {($dirBits($blockNumber) & $bitMask)} != 0]
    set pol [expr {($polBits($blockNumber) & $bitMask)} != 0]
    
    # SET/CLEAR buttons must be disabled for inputs, but not for outputs:
    # NOTE:	if the button is not `disabled', then it may be `active' if  
    #	the pointer is located on it. Therefore the button should be
    #	configured with either `normal' or `active'. But this seems
    #	to be very difficult to do. Here always `normal' is taken:
    if {$pedantic} {
	# NOTE: this widget re-configuration is VERY time-consuming, and
	# here even done in a monitor-loop. Avoid it if not necessary!
	set disState disabled
	if {$dir} {set disState normal}
	$itk_component(s0) configure -state $disState
	$itk_component(s1) configure -state $disState
	$itk_component(p)  configure -state $disState
	$itk_component(r)  configure -state $disState
    }
    set ${itk_interior}(signalDir)   $dir
    set ${itk_interior}(activeState) $pol
}

#--------------------------------------------------------------------
# METHODS FOR BITVECTORS UPDATE
#--------------------------------------------------------------------
body AcroxSignal::readSignalState { } {
    if {! $onLine}  return 
    readStateBits ;# read back the state of all signals into common vars
    showSignalState
}

# NOTE: there is no `readSignalConfig', because a change of config may
# always result in a change of state, e.g. due to change of activeState 
# the state may change!

body AcroxSignal::readSignal { } {
    if {! $onLine } return
    readBits ;# read back the state+config of all signals into common vars
    showSignalConfig
    showSignalState
}

#--------------------------------------------------------------------
# METHODS FOR PULSE SINGLE SIGNALS
#--------------------------------------------------------------------
body AcroxSignal::pulseSignal { {Ticks ""} } {
    global ${itk_interior}
    if {$Ticks != ""} {set ${itk_interior}(pulseTicks) $Ticks}
    debugPrint "width = [set ${itk_interior}(pulseTicks)] ticks"
    sendIoctlTrap WRITE_PULSE_BIT "$signalNumber;[set ${itk_interior}(pulseTicks)]"
    readSignalState   ;# read-back & update signal-specific widgets
    
    # read the state again after the pulse has expired:
    # (NOTE: it is assumed that 1 tick is equal to 10 ms)
    set time [expr {10*[set ${itk_interior}(pulseTicks)]}]
    after $time		$this readSignalState
    after $time		$this readPulseState
    after [expr {$time+500}]	$this readSignalState
    after [expr {$time+500}]	$this readPulseState
    after [expr {$time+1000}]	$this readSignalState
    after [expr {$time+1000}]	$this readPulseState
    after [expr {$time+2000}]	$this readSignalState
    after [expr {$time+2000}]	$this readPulseState
}

body AcroxSignal::resetPulse { } {
    debugPrint
    sendIoctlTrap WRITE_PULSE_BIT_RESET $signalNumber
    readSignalState   ;# read-back & update signal-specific widgets
    readPulseState
}


body AcroxSignal::readPulseState { } {
    global ${itk_interior}
    set r [sendIoctl CHECK_PULSE_BIT $signalNumber]
    set s 0
    if {[lindex $r 1] == 0} {set s 1}
    set ${itk_interior}(pulseState) $s
    debugPrint "state = $s"
    return $s
}

#--------------------------------------------------------------------
# METHODS FOR SIGNAL GROUPS
#--------------------------------------------------------------------

body AcroxSignal::writeGroup { {Value ""} } {
    global ${itk_interior}
    if {$Value != ""} {set ${itk_interior}(groupValue) $Value}
    
    # send command to set the value of multiple bits:
    set num $signalNumber
    set wid [set ${itk_interior}(groupWidth)]
    set fmt [set ${itk_interior}(groupFormat)]
    if {[scan [set ${itk_interior}(groupValue)] %i val] == 0} return
    debugPrint "from=$num width=$wid value=[format $fmt $val]"
    sendIoctlTrap WRITE_BITS "$num;$wid;$val"
    
    ;# for all affected signals: read-back & update signal-specific widgets
    readGroup
    foreach w [lrange $objectNames $signalNumber [expr {$num+$wid}]] {
	$w showSignalState
    }
}

body AcroxSignal::readGroup { } {
    global ${itk_interior}
    
    # send command to read the value of multiple bits:
    set num $signalNumber
    set wid [set ${itk_interior}(groupWidth)]
    set r [sendIoctl READ_BITS "$num;$wid;0"]
    if {[scan [lindex $r 0] "%d;%d;%d" x x val] < 3} {return ""}
    
    set fmt [set ${itk_interior}(groupFormat)]
    set ${itk_interior}(groupValue) [format $fmt $val]
    debugPrint "value=[set ${itk_interior}(groupValue)]"
    return $val
}

body AcroxSignal::setGroupWidth { Width } {
    global ${itk_interior}
    set wid $Width
    # limit the value if beyond max. signal:
    if {$wid + $signalNumber > $numberOfSignals} {
	set wid [expr {$numberOfSignals-$signalNumber}]
    }
    set ${itk_interior}(groupWidth) $wid
    set ${itk_interior}(groupLabel) \
	    "(Signal [expr {$signalNumber+$wid-1}]...$signalNumber)"
    debugPrint "group-interval: [set ${itk_interior}(groupLabel)]"
    readGroup
    return $wid
}

body AcroxSignal::configGroup { InputOutput HighLow } {
    global ${itk_interior}
    # arguments given in 0/1:
    sendIoctlTrap WRITE_CONFIG_BITS \
	    "$signalNumber;[set ${itk_interior}(groupWidth)];$InputOutput;$HighLow"
    debugPrint
    readGroup  ;# read-back & update
}


#--------------------------------------------------------------------
# METHODS FOR MASTER HANDLING
#--------------------------------------------------------------------

body AcroxSignal::setMaster { {YesNo ""} } {
    global ${itk_interior}
    if {$YesNo != ""} {set ${itk_interior}(masterSignal) $YesNo}
    
    set ena [set ${itk_interior}(masterSignal)]
    
    # for slaves, disable Option button and destroy Option-Panel:
    $itk_component(o) configure -state [expr {($ena?"normal":"disabled")}]
    catch {destroy $itk_component(option) }
    debugPrint "now [set ${itk_interior}(masterSignal)]"
}

#--------------------------------------------------------------------
# OPTION PANEL
#--------------------------------------------------------------------

body AcroxSignal::optionPanel { } {
    global ${itk_interior}

    set w ${itk_interior}(option)	;# window-name of the option-panel

    # If option-panel exists already, then only put it on top and return:
    if {[winfo exists $w]} { raise $w; return }

    # Build new toplevel window for the options of this signal:
    toplevel $w
    wm title $w "[wm title .]: Option Signal #$signalNumber"
    wm iconbitmap $w [wm iconbitmap .]
    # set iconname to assigned signal-name resp. fixed number:
    if {[set ${itk_interior}(signalName)] == ""} {
	wm iconname $w "Signal #$signalNumber"
    } else {
	wm iconname $w [set ${itk_interior}(signalName)]
    }
   
    # Build short help area
    UifShortHelp  $w.sh 
    pack $w.sh -side bottom -anchor w -fill x
   
    # "SIGNAL NAME" FRAME:
    frame		$w.0	-relief groove -bd 2
    label		$w.0.1	-width 11 -anchor w -text "Signal Name:"
    UifEntry	$w.0.2	-variable ${itk_interior}(signalName) -width 46
    pack	$w.0	-side top  -anchor w -fill x -expand yes
    pack	$w.0.1 	-side left -anchor w
    pack	$w.0.2 	-side left -anchor w -fill x -expand yes
   
    # "PULSE" FRAME:
    frame		$w.1	-relief groove -bd 2
    label		$w.1.1	-width 11 -anchor w -text "Pulse Ticks:"
    UifEntry	$w.1.2	-variable ${itk_interior}(pulseTicks) -width 5
    frame		$w.1.3	-relief flat
    radiobutton	$w.1.3.1 -width 6 -text Pulse -selectcolor Red \
	    -variable ${itk_interior}(pulseState) -value 1 \
	    -command [code $this pulseSignal]
    radiobutton	$w.1.3.2 -width 6 -text Reset -selectcolor Green \
	    -variable ${itk_interior}(pulseState) -value 0 \
	    -command [code $this resetPulse]
    UifScale	$w.1.4	-label "Pulse Width in Ticks:" -orient h \
	    -minVal 0 -maxVal 999 -length 250 \
	    -variable ${itk_interior}(pulseTicks)
    $w.1.4 setValue [set ${itk_interior}(pulseTicks)] ;# init the scale with actual val
    # bind Return-key in entry to update the scale:
    bind $w.1.2 <Return> "$w.1.4 set \[set ${itk_interior}(pulseTicks)\]"
    pack	$w.1		-side top  -anchor w -fill x -expand yes
    pack	$w.1.1 $w.1.2	-side left -anchor w
    pack	$w.1.3		-side left -anchor w -padx 2
    pack	$w.1.3.1	-side top
    pack	$w.1.3.2	-side bottom
    pack	$w.1.4		-side left -anchor e -padx 4 -pady 2
   
    # "GROUP" FRAME:
    # limit the group-width depending on the signal-number:
    set maxWidth [expr {$numberOfSignals-$signalNumber}]
    if {$maxWidth > 32} {set maxWidth 32} ;# driver-related maximum
   
    frame		$w.2	-relief groove -bd 2
    frame		$w.2.1	
    frame		$w.2.1.1
    label		$w.2.1.1.1 -width 11 -anchor w -text "Group Value:"
    UifEntry	$w.2.1.1.2 -variable ${itk_interior}(groupValue) -width 20
    frame		$w.2.1.2
    label		$w.2.1.2.1 -width 11 -anchor e -text "Format: "
    radiobutton	$w.2.1.2.2 -width 6 -text Hex \
	    -variable ${itk_interior}(groupFormat) -value 0x%x \
	    -command [code $this readGroup]
    radiobutton	$w.2.1.2.3 -width 6 -text Dec \
	    -variable ${itk_interior}(groupFormat) -value %d \
	    -command [code $this readGroup]
    frame		$w.2.1.3
    label		$w.2.1.3.1 -width 11 -anchor w -text "Group Config:"
    button	$w.2.1.3.2 -width 4 -text "in.L" \
	    -command [code $this configGroup 0 0]
    button	$w.2.1.3.3 -width 4 -text "in.H" \
	    -command [code $this configGroup 0 1]
    button	$w.2.1.3.4 -width 4 -text "out.L" \
	    -command [code $this configGroup 1 0]
    button	$w.2.1.3.5 -width 4 -text "out.H" \
	    -command [code $this configGroup 1 1]
    frame		$w.2.2
    button	$w.2.2.1 -width 8 -text Read \
	    -command [code $this readGroup]
    button	$w.2.2.2 -width 8 -text Write \
	    -command [code $this writeGroup]
    button	$w.2.2.3 -width 8 -text Clear \
	    -command [code $this writeGroup 0]
    frame		$w.2.3
    UifScale	$w.2.3.1 -label "Group Width:" -orient h \
	    -minVal $maxWidth -maxVal 1 -length 124 

    label		$w.2.3.2 -anchor s -textvariable ${itk_interior}(groupLabel)
    $w.2.3.1 setValue [set ${itk_interior}(groupWidth)]
    $w.2.3.1 configure -command [code $this setGroupWidth]
   
    pack $w.2		-side top  -anchor w -fill x -expand yes
    pack $w.2.1		-side left -anchor w -fill y 
    pack $w.2.1.1 $w.2.1.2 $w.2.1.3	-side top -fill both -expand yes
    pack $w.2.1.1.1 $w.2.1.1.2 -side left -anchor w
    pack $w.2.1.2.1 $w.2.1.2.2 $w.2.1.2.3 -side left -anchor w
    pack $w.2.1.3.1		-side left -anchor w
    pack $w.2.1.3.2 $w.2.1.3.3 $w.2.1.3.4 $w.2.1.3.5 \
	    -side left -anchor s -fill x -expand yes
    pack $w.2.2		-side left -anchor w -padx 2 -expand yes
    pack $w.2.2.1 $w.2.2.2 $w.2.2.3 -side top -fill x -expand yes
    pack $w.2.3		-side left -anchor e -padx 2 -expand yes
    pack $w.2.3.1 $w.2.3.2	-side top  -anchor s
   
    # set short help text for the above widgets
    # "PULSE" FRAME:
    UifShortHelp::add $w.sh $w.0.2  "Name of this signal"
    UifShortHelp::add $w.sh $w.1.2  "How many ticks to pulse"
    UifShortHelp::add $w.sh $w.1.3.1 "Single-Pulse the signal"
    UifShortHelp::add $w.sh $w.1.3.2 "Reset the pulse"
    UifShortHelp::add $w.sh $w.1.4   "Define pulse-width"
   
    # "GROUP" FRAME:
    UifShortHelp::add $w.sh $w.2.1.1.2 \
	    "Set the value of the group"
    UifShortHelp::add $w.sh $w.2.1.2.2  \
	    "The group value will be displayed in Hexadecimal"
    UifShortHelp::add $w.sh $w.2.1.2.3  \
	    "The group value will be displayed in Decimal"
    UifShortHelp::add $w.sh $w.2.1.3.2  \
	    "Configure the group to be Input/Active-Low"
    UifShortHelp::add $w.sh $w.2.1.3.3  \
	    "Configure the group to be Input/Active-High"
    UifShortHelp::add  $w.sh $w.2.1.3.4 \
	    "Configure the group to be Output/Active-Low"
    UifShortHelp::add $w.sh $w.2.1.3.5  \
	    "Configure the group to be Output/Active-High"
   
    UifShortHelp::add  $w.sh $w.2.2.1 "Read back the group value"
    UifShortHelp::add  $w.sh $w.2.2.2  "Write the group value"
    UifShortHelp::add  $w.sh $w.2.2.3 "Clear the group value to 0"
    UifShortHelp::add  $w.sh $w.2.3.1  "Set group width"
} 

#-------------------------------------------------------------------------
#  setData from LCC, Report to Load/Save
#-------------------------------------------------------------------------
body AcroxSignal::setData { data } {
    global ${itk_interior}
    
    debugPrint "$data"
    set width [lindex $data 2]
    set io [string toupper [lindex $data 3]]
    set hl [string toupper [lindex $data 4]]
    
    if {$width > 0} {
	setGroupWidth $width
	set ${itk_interior}(masterSignal) 1
    }
    set ${itk_interior}(signalName) [lindex $data 6]
    set ${itk_interior}(signalDir) [expr { $io=="INPUT" ? 0:1}]
    set ${itk_interior}(activeState) [ expr {$hl=="LOW" ? 0:1} ]
    
    if {[set ${itk_interior}(signalName)] == ""} {
	# indicate signal as not configured:
	$itk_component(nr) configure -background $normalColor
	setShortHelp $itk_component(nr) ""
    } else {
	# indicate signal as configured:
	$itk_component(nr) configure -background $itk_option(-configColor)
	setShortHelp $itk_component(nr) [set ${itk_interior}(signalName)]
    }
    #update
}

body AcroxSignal::report { } {
    global ${itk_interior}
    
    set name   [set ${itk_interior}(signalName)]
    set io     [expr { [set ${itk_interior}(signalDir)]==0 ? "INPUT":"OUTPUT"}]
    set state  [set ${itk_interior}(signalState)]
    set active [expr { [set ${itk_interior}(activeState)]==0 ? "LOW":"HIGH"}]
    set gw     [set ${itk_interior}(groupWidth)]
    
    if { ! [set ${itk_interior}(masterSignal)] } {
	set gw 0
    }
    return "$signalNumber [list $name] $io $state $active $gw"
}

#-------------------------------------------------------------------------
#  common procs
#-------------------------------------------------------------------------

body AcroxSignal::getOnLine { } {
    return $onLine
}

body AcroxSignal::showAll { } {
    foreach w $objectNames {
	$w showSignalState
	$w showSignalConfig
	#@# there should also be an update of the group-value!
    }
}

body AcroxSignal::updateAll { } {
    foreach w $objectNames {
	set state [$w getState]
	$w configSignal
	$w writeSignal $state
    }
}

# bit-wise representation of all signals: `xxx(blockNumber)'

body AcroxSignal::readConfigBits { } {
    if {! $onLine} return
    
    set r [sendIoctl READ_CONFIG_BITS {0;32;0;0 32;32;0;0}]
    scan [lindex $r 0] "%d;%d;%d;%d" x x dirBits(0) polBits(0)
    scan [lindex $r 1] "%d;%d;%d;%d" x x dirBits(1) polBits(1)
    return [format "0x%08x 0x%08x 0x%08x 0x%08x"\
	    $dirBits(0) $dirBits(1) $polBits(0) $polBits(1)]
}

body AcroxSignal::writeConfigBits { } {
    sendIoctl WRITE_CONFIG_BITS \
	    [list "0;32;$dirBits(0);$polBits(0)" "32;32;$dirBits(1);$polBits(1)"]
}

body AcroxSignal::readStateBits { } {
    if {! $onLine } return
    
    set r [sendIoctl READ_BITS {0;32;0 32;32;0}]
    scan [lindex $r 0] "%d;%d;%d" x x stateBits(0)
    scan [lindex $r 1] "%d;%d;%d" x x stateBits(1)
    return [format "0x%08x 0x%08x" $stateBits(0) $stateBits(1)]
}

body AcroxSignal::writeStateBits { } {
    sendIoctl WRITE_BITS [list "0;32;$stateBits(0)" "32;32;$stateBits(1)"]
}

body AcroxSignal::readBits { } {
    if { !$onLine } return
    
    set r [sendIoctl {READ_BITS READ_CONFIG_BITS} {0;32;0;0 32;32;0;0}]
    scan [lindex $r 0] "%d;%d;%d" x x stateBits(0)
    scan [lindex $r 1] "%d;%d;%d" x x stateBits(1)
    scan [lindex $r 2] "%d;%d;%d;%d" x x dirBits(0) polBits(0)
    scan [lindex $r 3] "%d;%d;%d;%d" x x dirBits(1) polBits(1)
    return [format "0x%08x 0x%08x 0x%08x 0x%08x 0x%08x 0x%08x"\
	    $stateBits(0) $stateBits(1)\
	    $dirBits(0) $dirBits(1) $polBits(0) $polBits(1)]
}

body AcroxSignal::writeBits { } {
    writeStateBits
    writeConfigBits
}

# update bits from an external reply, used internally for monitoring:
body AcroxSignal::monitorCB { Reply } {
    scan [lindex $Reply 0] "%d;%d;%d" x x stateBits(0)
    scan [lindex $Reply 1] "%d;%d;%d" x x stateBits(1)
    scan [lindex $Reply 2] "%d;%d;%d;%d" x x dirBits(0) polBits(0)
    scan [lindex $Reply 3] "%d;%d;%d;%d" x x dirBits(1) polBits(1)
    showAll
}

body AcroxSignal::startMonitor { } {
    $lineW MonitorStart \
	    {READ_BITS READ_CONFIG_BITS} {0;32;0;0 32;32;0;0} {} \
	    {AcroxSignal::monitorCB}
}

body AcroxSignal::stopMonitor { } {
    $lineW MonitorStop
}


#---------------------------------------------
#  Local procs 
#---------------------------------------------

body AcroxSignal::setLine { Line } {
    set lineW $Line	;# called by the Device object to define the LCU link
}

# send IOCTL command, ignore driver errors:
body AcroxSignal::sendIoctl { Cmdlist Arglist {Sizlist ""} } {
    if { ! $onLine }    return  
    return [$lineW Ioctl $Cmdlist $Arglist $Sizlist]
}

# send IOCTL command, trap any driver errors with a dialog-box:
body AcroxSignal::sendIoctlTrap { Cmdlist Arglist {Sizlist ""} } {
    if { ! $onLine } return 
    return [$lineW IoctlTrap $Cmdlist $Arglist $Sizlist]
}

#-----------------------------------------
# Class Options
#-----------------------------------------
#___oOo___
