#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: aioxDevice.tcl,v 1.34 2005/09/26 12:28:57 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# bgustafs 2005-09-22 changed aioSignal structure
# imunoz  24/04/95  created
# imunoz  20/04/96  itcl2.0 port
#

#************************************************************************
#   NAME
#       AioxDevice - Class description for an "aio" Analog I/O device
#
#   SYNOPSIS
#       AioxDevice <object-name> ?<options>?
#
#   DESCRIPTION
#
#	AioxDevice defines the class for a "aio" analog I/O device.
#	It makes the functionality of the device and the corresponding 
#	"aio" driver available through a graphical user interface.
#
#	An object of this class is created by:
#
#		AioxDevice <object-name> -<Option> <Value> ...
#
#	This will create all sub-objects for user interaction and
#	an object to handle the connection to a server-task in the 
#	LCU environment i.e. effectively to the physical device.
#
#   OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<object-name> configure -<Option> <Value> ...
#
#	The following device-specific options are publicly available:
#
#	-nInputSignals	        specifies the total number of input signals
#				of the device. The maximum value and default
#				is 16. This number must to be divisible by 4.
#
#	-nFirstSignal		specifies the number of the first input signal.
#				The default is 0.
#
#	Several options are inherited from InducerDevice and are therefore also
#	available within the scope of this class. See InducerDevice(n) for a 
#	detailed description of those common options.
#
#   METHODS
#	Methods represent procedures on the object and are invoked with:
#
#		<object-name> <Method> <Arguments> ...
#
#	There are no special device-specific methods available.
#
#	Several method are inherited from InducerDevice and overloaded here
#	with device-specific actions. See InducerDevice(n) for a detailed
#	description of those common methods.
#
#   PROCS
#
#   EXAMPLES
#
#   SEE ALSO
#	aioxInducer(1),
#	InducerClass(n), InducerDevice(n), InducerLine(n)
#
#   BUGS
#
#------------------------------------------------------------------------
#

#*# class: AioxDevice
class AioxDevice {

    inherit InducerDevice


    #--------------------------------------------------------------------
    # Device-specific members of AioxDevice class
    #--------------------------------------------------------------------

    #*# constructor
    constructor { args } { }
    itk_option define  -nInputSignals nInputSignals NInputSignals 16   
    itk_option define  -nFirstSignal nFirstSignal   NFirstSignal   0
    
    #------------------------------------------------
    # Create <howMany> input widgets on <rows> rows. 
    #------------------------------------------------
    protected method createInputs { howMany rows } { }

    #-------------------------------------------------
    # Create <howMany> output widgets on <rows> rows.
    #-------------------------------------------------
    protected method createOutputs {howMany rows } { }
	
    #--------------------------------
    # Create Device's control Panel
    #--------------------------------
    protected method createButtons { } { }
    
    #---------------------------------------------
    # Set Display Format of input signals
    # If it is needed stop the current monitor
    # process and start another with a diferent
    # read command.
    #----------------------------------------------
    public method setDisplayFormat { mode } { }
    public method updateView { } { }

    protected variable  inputW    {}     ;# list of object-names of input signals
    protected variable  outputW   {}     ;# list of object-names of outputsignals
    protected variable  inputCnf  {}     ;# list of cnf. of inputSignals
    protected variable  outputCnf {}
    protected variable  displayList {"Float" "Int" "Hex" "Bin"}
    protected variable  monitorCmd  "READ_VALUE"
    protected variable  monitorValueArg 0.0
    protected variable  scanArgs "%d;%d;%d;%f"
    #--------------------------------------------------------------------
    # Overloaded standard members of InducerDevice class
    #--------------------------------------------------------------------

    method GetVersion { } { }
    method InitDevice { } { }
    method ExitDevice { } { }
    method ResetDevice { } { }
    method GetDeviceId { } { }
    method Disconnect { } { }
    method Resume { } { }

    #---------------------------
    # Get LCC Configuration
    #---------------------------
    method GetSignalConfig { } { }
    method SaveDevice { } { }
    method LoadDevice {data } { }
    method LoadFromLCC { } { }
	    
    #--------------------------------------------------------------------
    # Local methods
    #--------------------------------------------------------------------
    
    private method indexOf { signalNumber signalList } { }
    private method setPreconfSignals { } { }
    private method getAnalogConfig { name } { }
    private method mustBeCnf { signalList number } { }

    #-----------------------------
    # Monitor Process 
    #-----------------------------
    private method StartMonitorVLev {} { }
    private method MonitorVLevCB { Reply } { }
}

body AioxDevice::constructor { args } {
    global ::${itk_interior}
    #-------------------------
    # Create device's signals
    #-------------------------
    eval itk_initialize $args
 
    if { $itk_option(-nInputSignals) > 0 } {
	set numberOfRows [expr {($itk_option(-nInputSignals)-1)/4 + 1}]
    } else {
	set numberOfRows 0
    }
    createInputs $itk_option(-nInputSignals) $numberOfRows;	

    createOutputs 2 1;	#create outputSignals
#    # set connection to inducerLine object  
    AioxSignal::setLine $lineW
    
#    # initialization needed for
#    # binary display of values
     AioxSignal::initbin
    
     createButtons
#    #-------------------------------
#    # Set Display Mode of signals
#    #-------------------------------
     set ${itk_interior}(displayFormat)      "Float"
     setDisplayFormat [set ${itk_interior}(displayFormat)]
}

#------------------------------------------------
# Create <howMany> input widgets on <rows> rows. 
#------------------------------------------------
body AioxDevice::createInputs { howMany rows } {
    
    if { $rows != 0 } {
	set numberOfSignalsPerRow [expr {$howMany / $rows}]
	if {[expr {$numberOfSignalsPerRow * $rows}] < $howMany} {
	    incr numberOfSignalsPerRow
	}
    } else {
	set numberOfSignalsPerRow 0
    } 
    set jMax [expr {$rows} ]
    set iMax [expr {$numberOfSignalsPerRow} ]
    
    #loop on each row
    for {set j 0} {$j < $jMax} { incr j } {
	itk_component add Fi$j {
	    # -relief groove
	    set F [frame $itk_interior.i$j  -bd 3 -relief groove]
	}
	pack $F -side top -anchor n 

	for {set i 0} {$i < $iMax} {incr i} {
	    set n [expr {$i+$j*$iMax}]
	    if {$n == $howMany} break
	    itk_component add S.i$n {
		#-bd 2
		set S [Aiox_InputSignal $F.i$n  -relief raised -borderwidth 2\
			-sNum [expr $n+$itk_option(-nFirstSignal)] ]
	    }
	    
	    pack $S -side right -anchor e 
	    lappend inputW $S;         #keep track of the object names
	}
    }
}

#-------------------------------------------------
# Create <howMany> output widgets on <rows> rows.
#-------------------------------------------------
body AioxDevice::createOutputs {howMany rows } {
    if { $rows != 0 } {
	set numberOfSignalsPerRow [expr {$howMany / $rows}]
    } else {
	set numberOfSignalsPerRow 0
    }
    set jMax [expr {$rows} ]
    set oMax [expr {$numberOfSignalsPerRow} ]
    
    for {set j 0} {$j < $jMax} { incr j } {
	itk_component add Fo$j {
	    set F [frame $itk_interior.o$j -bd 3 -relief groove ]
	}
	pack $F -side top -anchor n 
	
	for {set o 0} {$o < $oMax } {incr o} {
	    set n [expr {$o+$j*$oMax}]
	    itk_component add S.o$n {
		set S [Aiox_OutputSignal $F.o$n -relief raised\
			-borderwidth 2 -sNum $n]
	    }
	    pack $S -side right -anchor e 
	    lappend outputW $S;         #keep track of the object names
	}
    }
}

#--------------------------------
# Create Device's control Panel
#--------------------------------
body AioxDevice::createButtons { } {
    global ::{itk_interior}
    itk_component add F0 {
	set F [frame $itk_interior.f0 -relief groove -bd 2]
    }
    # label
    itk_component add F0.1 {
	label $itk_component(F0).1 -text "Display Format:" -width 14
    }
    # Display formats button
    itk_component add F0.2 {
	UifOptionButton $itk_component(F0).2 \
		-itemList $displayList \
		-variable ${itk_interior}(displayFormat) \
		-command  [code $this setDisplayFormat]
    }
    # set short help
    setShortHelp $itk_component(F0.2) "Select  Display Format of input Values.\
	    Types are Float, Int, Hex and Bin."
    
    # 'Update View' Button
    itk_component add F0.3 {
	UifButton $itk_component(F0).3 -label "Update View" \
		-command [code $this updateView] 
    }
    # short help
    setShortHelp $itk_component(F0).3  "Update display of all signals."

#   pack $itk_component(F0)  -side top -anchor w -padx 1 
    pack $itk_component(F0)  -side top -padx 1 
    pack $itk_component(F0).1 $itk_component(F0).2 -side left 
    pack $itk_component(F0).3 -side right -anchor w -padx 20
}

#---------------------------------------------
# Set Display Format of input signals
# If it is needed stop the current monitor
# process and start another with a diferent
# read command.
#----------------------------------------------
body AioxDevice::setDisplayFormat { mode } {
    global ${this}

    debugPrint

    if {$mode == "Float"} {
	set cmd  READ_VALUE
	set argValue "0.0"
	set sArgs "%d;%d;%d;%f"
    } else {
	set cmd  READ_BIN_VALUE 
	set argValue "0"
	set sArgs "%d;%d;%d;%d"
    }
    AioxSignal::setDisplayMode $mode
    if { $cmd == $monitorCmd } {
	debugPrint "Mode not changed"
	return 1
    } 
    set monitorCmd $cmd
    set monitorValueArg $argValue
    set scanArgs $sArgs

    if {[AioxSignal::getOnLine]} {
	
	$lineW MonitorStop
	#---------------------------
	# wait 500 ms
	#---------------------------
	after 500 
	StartMonitorVLev
    }
}

body AioxDevice::updateView {} {
    debugPrint

    # read curent status of input lines
    foreach i $inputW {
	$i updateSignal
    }
    
    # read curent configuration of output lines
    foreach o $outputW {
	$o readSignalConfig
    }
}

#--------------------------------------------------------------------
# Overloaded standard members of InducerDevice class
#--------------------------------------------------------------------

body AioxDevice::GetVersion { } {
    debugPrint
    global aiox argv0
    return "[file tail $argv0] - Version $aiox(version)"
}

body AioxDevice::InitDevice { } {
    debugPrint

    # base behavior 
    set err [InducerDevice::InitDevice] ;# common part to open channel
    if {$err != ""} {return $err}
    
    AioxSignal::setOnLine 1

    # read current status of I/O lines
    updateView

    # start monitoring of input signals
    StartMonitorVLev

    return ""
}

body AioxDevice::ExitDevice { } {
    debugPrint

    $lineW MonitorStop
    return [InducerDevice::ExitDevice]	;# common part to close channel
}

body AioxDevice::ResetDevice { } {
    debugPrint
    return [$lineW Ioctl RESET 0]
}

body AioxDevice::GetDeviceId { } {
    debugPrint
    set id [$lineW Ioctl GET_BOARD_ID {0}]
    
    return [lindex $id 0]
}

body AioxDevice::Disconnect { } {
    global inducer
    
    set inducer(fOnline) 0
    AioxSignal::setOnLine 0
}

body AioxDevice::Resume { } {
    global inducer

    set inducer(fOnline) 1
    
    AioxSignal::setOnLine 1
    # update state of the driver acording user's changes
    foreach i $inputW {
	$i resumeConnection
    }
    foreach i $outputW {
	$i resumeConnection
    }
}

#---------------------------
# Get LCC Configuration
#---------------------------
body AioxDevice::GetSignalConfig { } {
    debugPrint
    # Father's behavior - Get cnfList
    InducerDevice::GetSignalConfig

    foreach i $cnfList {
	set name [givemeName $i]
	set type [givemeType $i]
	set io   [givemeIO   $i]

	if {$type == "Analog"} {
	    set reply [getAnalogConfig $name] 
	    set reply [split $reply ,]

	    # Check Device's name
	    if { [lindex $reply 0] != $itk_option(-vDev)} {
		continue
	    }
	    lappend reply $name
	    switch $io { 
		"Input"  {
		    lappend inputCnf $reply
		}
		"Output" { 
		    lappend outputCnf $reply
		}
	    } 
	} 
    }
    return [join $inputCnf $outputCnf]
}

body AioxDevice::SaveDevice { } {
    set data "\n#--- Input Lines Configuration\n"
    
    # save input Lines
    foreach i $inputW {
	set lineInfo [$i report]
	
	set line   " INPUT : [lindex $lineInfo 0]\n"
	set name   "  NAME : [lindex $lineInfo 1]\n"
	set config "CONFIG : [lindex $lineInfo 2]\n"
	set gain   "  GAIN : [lindex $lineInfo 3]\n"
	
	# seudo-append
	set data "$data\n $line $name $config $gain"
    }
    set data "$data\n#---Output Lines Configuration\n"
    
    ## -- save output-lines
    foreach i $outputW {
	set lineInfo [$i report]
	
	set line   "OUTPUT : [lindex $lineInfo 0]\n"
	set name   "  NAME : [lindex $lineInfo 1]\n"
	set config "CONFIG : [lindex $lineInfo 2]\n"
	set value  " VALUE : [lindex $lineInfo 3]\n"
	# seudo-append
	set data "$data\n $line $name $config $value"
    }
    return $data
}

body AioxDevice::LoadDevice {data } {
    
    set cnf [split $data \n]
    set seq 0

    set name ""
    set number 0
    set gain ""
    set config ""
    set value 0
    set io 0

    foreach el $cnf {
	# skip blank lines
	if {[regexp {^[ \t]*$} $el]} {
	    continue
	}
	# skip comments
	if {[string match #* $el ]} {
	    continue
	}
	regexp {[A-Z]+ :} $el head 
	regsub {[A-Z]+ : *} $el "" tok
	regsub {^[ \t]+} $tok "" token

	switch $head {
	    "INPUT :" {
		set io 0
		set number $token
	    }
	    "OUTPUT :" {
		set io 1
		set number $token
	    }
	    "NAME :" {
		set name [list $token]
	    }
	    "CONFIG :" {
		set config [list $token]
	    }
	    "GAIN :" {
		set gain [list $token]
		set i [indexOf $number $inputW]
		if { $i >= 0 } {
		    set signal [lindex $inputW $i] 
		    $signal loadData "$number $name $config $gain"
		}
	    }
	    "VALUE :" {
		set value $token
		set i [indexOf $number $outputW]
		if { $i >= 0 } {
		    set signal [lindex $outputW $i] 
		    $signal loadData "$number $name $config $value"
		}
	    }
	}
    }
}

body AioxDevice::LoadFromLCC { } {
    setPreconfSignals
}

#--------------------------------------------------------------------
# Local methods
#--------------------------------------------------------------------

body AioxDevice::indexOf { signalNumber signalList } {
    set pos 0
    set found 0
    
    foreach i $signalList {
	set num [$i info variable sNum -value]
	if { $num == $signalNumber} {
	    set found 1
	    break
	}
	incr pos
    }
    if {$found} {
	return $pos
    } 
    return -1
}

body AioxDevice::setPreconfSignals { } {
    foreach i $inputW {
	
	set sNum [$i info variable sNum -value]
	set cnfPos [mustBeCnf $inputCnf $sNum]
	debugPrint "in: $sNum cnf: $cnfPos"
	if { $cnfPos >= 0 } {
	    set l  [lindex $inputCnf $cnfPos]
	    $i configLcc $l 
	    # show this signal as LCC Cconfigured by 
	    # changing its background color
	    $i.nr configure -bg orange
	} else {
	    $i.nr configure -bg grey80
	}
    }
    foreach i $outputW {
	set sNum [$i info variable  sNum -value]
	set cnfPos [mustBeCnf $outputCnf $sNum]
	if { $cnfPos >= 0 } {
	    set l  [lindex $outputCnf $cnfPos]
	    $i configLcc $l
	    # show this signal as LCC Cconfigured by 
	    # changing its background color
	    $i.nr configure -bg orange
	} else {
	    $i.nr configure -bg grey80
	    
	}	    
    }
}

body AioxDevice::getAnalogConfig { name } {
    return [sendServer $itk_option(-vEnv) lccServer AIOGCNF $name]
}

body AioxDevice::mustBeCnf { signalList number } { 
    set reply 0
    set found 0
    
    foreach i $signalList {
	if {[lindex $i 1] == $number } {
	    set found 1
	    break
	}
	incr reply
    }
    if { ! $found } { set reply -1}
    return $reply
}

#-----------------------------
# Monitor Process 
#-----------------------------
body AioxDevice::StartMonitorVLev {} {
    set a $monitorValueArg
    $lineW MonitorStart \
	    [list $monitorCmd] \
	[list "0;0;0;$a 1;0;0;$a 2;0;0;$a 3;0;0;$a 4;0;0;$a 5;0;0;$a 6;0;0;$a 7;0;0;$a 8;0;0;$a \
	    9;0;0;$a 10;0;0;$a 11;0;0;$a 12;0;0;$a 13;0;0;$a 14;0;0;$a 15;0;0;$a"] {} \
	    [code $this MonitorVLevCB]
}

body AioxDevice::MonitorVLevCB { Reply } {
    if { [AioxSignal::getOnLine] == 0 } { 
	return
    }
    set iMax [llength $inputW]
    for {set i 0} {$i < $iMax } { incr i} {
	scan [lindex $Reply $i] $scanArgs n n n n
	set line [lindex $inputW $i]
	$line configure -vLevel $n 
    }
}

#
# ___oOo___