#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bltplotGroup.tcl,v 1.4 2005/07/07 10:46:52 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/06/01  created
#

#************************************************************************
#   NAME
#   BltplotGroup - plot a group of "scan" lines
# 
#   SYNOPSIS
#   BltplotGroup <widget-name> ?widget options?
# 
#   DESCRIPTION
#   BltplotGroup handles a group of scans and associated controls.
#   Each scan is handled by class BltplotPlotScan.
#
#   SEE ALSO
#   graph(n), iwidgets(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

itk::usual BltplotGroup {}

class BltplotGroup {
    inherit BltplotBase

    constructor {args} {
        eval itk_initialize $args

	# create a paned window for the scan group
	#
	set wscans_ $w_.pw
	[panedwindowCmd] $wscans_ \
	    -sashcursor  sb_v_double_arrow \
	    -thickness   2 \
	    -orient      horizontal
	pack $wscans_ -expand 1 -fill both

	# create scans
	#
	set idx 0
	loop sc 0 [llength $plotScanList] {
	    set scanList [lindex $plotScanList $sc]
	    set Id [lindex $scanList 0]
	    trace_addvar hideScan$Id [code $this hide_scan $idx]

	    incr idx [$w_ make_scan $wscans_ $sc $idx]
	}

	# horizontal scrollbar for scan group
	#
	set scrollbar_ [scrollbar $w_.hscroll -orient horizontal -command [code $this xscrolled]]
	if { $with_scale } {
	    pack $scrollbar_ -fill x -side bottom
	    shelp $scrollbar_ "Group $id: Slider to scroll the scan lines in group '$id'"
	}

	pack $w_ -expand 1 -fill both

	trace_addvar imgEvt [code $this update_wdg]
	trace_addvar groupAcut$id [code $this call_objs auto_cut]
	trace_addvar groupCuts$id [code $this set_cuts]
    }

    # create a scan frame
    
    private method make_scan { w sc idx } {
	set scanList [lindex $plotScanList $sc]
	if { [llength $scanList] == 1 } {
	    set scanList [join $scanList]
	}

	# check scanline spec
	#
	set cc [catch {label $w.cc -foreground [lindex $scanList 2]}]
	catch {destroy $w.cc}
	if { $cc || [llength $scanList] < 3 || \
		 [catch {expr {[lindex $scanList 1] + 1}}] } {
	    puts stderr "invalid spec of scanline: $scanList"
	    return 0
	}
	set idlab [lrange $scanList 3 10]

	# w is the paned window (see package iwidgets)
	$w add $idx \
	    -margin 2 \
	    -minimum 5

	set scanObj "[$w childsite $idx].scan$idx"
	BltplotScan $scanObj \
	    -image         $image \
	    -motionCb      [code $this motion_event] \
	    -id            [lindex $scanList 0] \
	    -idlab         [join $idlab] \
	    -yline         [lindex $scanList 1] \
	    -color         [lindex $scanList 2] \
	    -swidth        $swidth \
	    -show_cross    $show_cross \
	    -show_pixmark  $show_pixmark \
	    -with_xticks   $with_xticks \
	    -with_yticks   $with_yticks \
	    -plotbg        $plotbg

	lappend scanObjs_ $scanObj
	$scanObj set_zoomCb [code $this zoom_changed $scanObj]
	return 1
    }

    # hide/show a scan

    public method hide_scan { idx array name args } {
	global $array
	set bool [set ${array}($name)]
	set scanObj [lindex $scanObjs_ $idx]
	if { $bool != [$scanObj cget -visible] } { return }

	if { $bool } {
	    $wscans_ hide $idx
	    $scanObj config -visible 0
	} else {
	    $wscans_ show $idx
	    $scanObj config -visible 1
	}
    }

    # update group widget after an image event

    private method update_wdg { args } {
	set width [expr { $swidth == 0 ? [$image_ width] : $swidth }]
	if { ! [$image_ isclear] && $imgwidth_ != $width && "[get_obj]" != "" } {
	    set imgwidth_ $width
	    call_objs config -swidth $swidth
	    call_objs set_scale 0.5 [expr {$width + 0.5}]
	    xscrolled init
	}
    }

    # get an active scanline object

    private method get_obj { } {
	foreach scan $scanObjs_ {
	    if { [$scan cget -activated] && [$scan cget -visible] } {
		return $scan
	    }
	}
	return {}
    }

    # motion event: propagate to objects
    #
    # The time for updating all objects can be significant on slow
    # X-terminals. When the mouse pointer is quickly moved over to another scan
    # then the leave and enter events are skipped.

    public method motion_event { args } {
	catch {
	    if {"[lindex $args 0]" == "LEAVE"} {
		after 300 [code eval $this call_objs motion_event LEAVE]
		return
	    }
	    eval call_objs motion_event $args
	    update idletasks
	}
    }
    
    # method set_scale is called after values have been entered.

    public method set_scale { left right } {
	call_objs set_scale $left $right
	xscrolled init
    }
    
    # reset scale

    public method reset_scale { } {
	call_objs reset_scale
	xscrolled init
    }
    
    # zoom changed, callback

    public method zoom_changed { scanObj evt } {
	lassign [$scanObj info] xmin xmax grmin grmax

	if { "$evt" == "ZoomIn" } {
	    set delta [expr {($grmax - $grmin) / 4.0}]
	    if { $delta < 5.0 } { return }
	    set left  [expr {$grmin + $delta}]
	    set right [expr {$grmax - $delta}]
	} else {
	    set delta [expr {($grmax - $grmin) / 2.0}]
	    if { $delta < 5.0 } { return }
	    set left  [expr {max($xmin, $grmin - $delta)}]
	    set right [expr {min($xmax, $grmax + $delta)}]
	}
	set_scale $left $right
    }

    # apply cuts to all scans

    public method set_cuts { array name opt } {
	global $array
	set sobj [set ${array}($name)]
	lassign [$sobj get_cuts] low high
	call_objs set_cuts $low $high
    }
    
    # call methods of objects belonging to the scan group

    public method call_objs { args } {
	foreach scanObj $scanObjs_ {
	    eval $scanObj $args
	}
    }

    # set x-axis range for a group

    public method set_xrange {with_range xstart xdelta} {
	call_objs config -xstart $xstart -xdelta $xdelta -with_range $with_range
	call_objs reset_scale
	update idletasks
	xscrolled init
    }
    
    # set left margin for y-axis labels

    public method set_leftmargin { numpix } {
	call_objs config -leftmargin $numpix
    }

    # callback after scrollbar actions

    private method xscrolled { args } {
	if { "[set obj [get_obj]]" == "" } { return }
	lassign [$obj info] xmin xmax grmin grmax
	if { $xmax == 0 } { return }

	trace_setvar scalex$id $grmin
	trace_setvar scaley$id $grmax

	lassign $args cmd val unit
	switch -exact -- $cmd {
	    moveto {
		set frWidth [expr {($grmax - $grmin) / $xmax}]
		set lfr $val
		if { $lfr < 0.0 } { set lfr 0.0 }
		set rfr [expr {$lfr + $frWidth}]
		if { $rfr > 1.0 } {
		    set rfr 1.0
		    set lfr [expr {1.0 - $frWidth}]
		}
		set left  [expr {$lfr * $xmax}]
		set right [expr {$rfr * $xmax}]

		call_objs set_scale $left $right
		$scrollbar_ set $lfr $rfr
	    }
	    scroll {
		if {"$unit" == "units"} {
		    set grmin [expr {$grmin + $val}]
		    if {$grmin < $xmin} {return}
		    set grmax [expr {$grmax + $val}]
		    if {$grmax > $xmax} {return}
		    set_scale $grmin $grmax
		} else {
		    set page [expr {$val * ($grmax - $grmin)}]
		    $w_ xscrolled scroll $page units
		}
	    }	    
	    init {
		$scrollbar_ set [expr {$grmin / $xmax}] [expr {$grmax / $xmax}]
	    }	    
	}
    }

    # return scan objects handled by this class

    public method get_scanObjs { } {
	return $scanObjs_
    }

    # -- private vars -- 

    private variable wscans_ {}       ; # paned window for scans
    private variable scanObjs_ {}     ; # scan line objects
    private variable scrollbar_ {}    ; # scrollbar widget
    private variable imgwidth_ 0      ; # width of image
    
    # -- public variables --

    # scan line list
    public variable plotScanList {}

    # widget is activated, bool
    public variable activated {0} {
	call_objs config -activated [cget -activated]
	update_wdg
    }
}

#
# ___bltplot___
