#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bltplotScan.tcl,v 1.4 2005/07/07 10:46:51 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/06/04  created
#

#************************************************************************
#   NAME
#   BltplotScan - plot a horizontal scanline
#
#   SYNOPSIS
#   BltplotScan <widget-name> ?widget options?
# 
#   DESCRIPTION
#   This itcl widget class plots a defined single horizontal line of an Rtd image
#   frame. The actual plotting is done via method 'update_plot' which calls
#   the rtdimage subcommand 'bltgraph xline'.
#   The BltplotScan widget is build with (and based on) features provided by
#   the BLT package.
#   The public methods need to be defined as 'public' since they are used by
#   class BltplotGroup and the toplevel widget BltplotCtrl. They are
#   not public to users (and hence not installed in the public library).
#
#   BltplotScan provides for:
#   - creating the plot widget (via the constructor)
#   - update the plot after an image event
#   - markers for displaying values and pixels at cursor position
#   - return info of the plot (range, min/max cut levels)
#   - setting cut levels
#   - setting the scale
#   - processing motion events (display of x,y and value at cursor position)
#   - display of a cross, marker at cursor position
#   - freezing a x-position and updating the y-value during image events
#   - notification events to parent (cursor movements which are propagated
#     to all members of the group)
#   - postscript print of a scanline
#
#   Interaction with the main consumer widgets BltplotCtrl and BltplotGroup
#   is implemented with trace variables whenever possible.
#
#   SEE ALSO
#   rtdimage(n)
#   graph(n), also barchart(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

itk::usual BltplotScan {}

class BltplotScan {
    inherit BltplotBase

    constructor {args} {
	eval itk_initialize $args

	make_scan $w_
	pack $w_ -expand 1 -fill both
	make_markers
	markers_color 1

	set tag motion$id
	$graph_ element bind $tag <Any-Motion> [code $this notify_motion MOTION %x $id]
	$graph_ element bind $tag <Any-Leave>  [code $this notify_motion LEAVE]
	$graph_ element bind $tag <Double-ButtonPress-1> "[code eval $this mtoggle %x $id]"

	# add trace variables
	trace_addvar imgEvt  [code $this update_plot]
	trace_addvar acuts   [code $this auto_cut]
	trace_addvar cross   [code $this show_cross]
	trace_addvar pixmark [code $this show_pixmark]
	trace_addvar grid    [code $this show_grid]
    }

    # make a scan consisting of a graph and x,y vector
    
    private method make_scan { w } {
	set graph_    $w.graph
	set xVector_  x$id
	set yVector_  y$id
	set elem_     e$id

	# create a graph object (see graph(n))
	#
	blt::graph $graph_ \
	    -lm 1 -tm 1 -bm 1 -rm 1 \
	    -title          {} \
	    -halo           2i \
	    -borderwidth    0 \
	    -plotbackground $plotbg

	foreach axis "x y" {
	    $graph_ axis configure $axis -showticks 1 -subdivisions 0 \
		-ticklength $ticklength -tickfont $tickfont
	}
	
	if { $with_xticks } {
	    $graph_ configure -bm 0  ; # bottom margin
	}

	if { $with_yticks } {
	    $graph_ configure -lm $leftmargin
	}

	# create a vector of floating point values (see vector(n))
	#
	blt::vector create $xVector_ $yVector_ -variable ""
	
	set pen_ pen$id
	$graph_ pen create    $pen_ -symbol {}
	$graph_ pen configure $pen_ -color $color

	$graph_ element create $elem_ \
	    -label           {} \
	    -linewidth       1 \
	    -outlinewidth    0 \
	    -xdata           $xVector_ \
	    -ydata           $yVector_ \
	    -activepen       $pen_ \
	    -symbol          {} \
	    -bindtags        "motion$id scale$id"

	shelp $graph_ \
	    "[labelId] / Y=$yline: <2> Zoom in, <3> Zoom out, <Double-1> toggle crosshair on/off"

	Blt_ClosestPoint $graph_
	Blt_PrintKey     $graph_

	$graph_ element activate $elem_
	pack $graph_ -fill both -expand 1
    }

    # add markers for displaying values and pixels

    private method make_markers {} {
	# marker for values
	set marker_ \
	    [$graph_ marker create text \
		 -justify      left \
		 -anchor       w \
		 -under        yes \
		 -coords       { -Inf Inf } \
		 -anchor       w \
		 -padx         0 \
		 -pady         0]

	# marker for pixels
	set marker2_ [$graph_ marker create line -linewidth 15 -under yes]
    }

    # return info of displayed image and plot

    public method info { } {
	set xmin 1; set xmax $nx_
	lassign [get_xaxis_limits] grmin grmax

	foreach num "xmin xmax grmin grmax" {
	    set $num [expr { double([set $num]) }]
	}
	return "$xmin $xmax $grmin $grmax"
    }

    # return x axis limits (integer values)

    private method get_xaxis_limits { } {
	lassign [$graph_ axis limits x] grmin grmax
	set nx    [uxval $nx_]
	set grmin [uxval $grmin]
	set grmax [uxval $grmax]
	if { $grmax > $nx_ } {
	    set grmin 1; set grmax $nx_
	}
	return "[expr {int($grmin)}] [expr {int($grmax)}]"
    }
    
    # update the plots (e.g. after an image event)
    
    private method update_plot { args } {
	if { [$image_ isclear] || ! $activated || ! $visible } { return }

	if { $with_range } {
	    $image_ bltgraph xline $graph_ $elem_ $xVector_ $yVector_ $yline 1 $nx_ \
		[expr {$xstart + $xdelta / 2.0}] $xdelta
	} else {
	    $image_ bltgraph xline $graph_ $elem_ $xVector_ $yVector_ $yline 1 $nx_
	}

	if { "$lastMove_" != ""} {
	    lassign $lastMove_ what x
	    catch {eval motion_event MOTION $x -1}
	}
    }

    # set cut levels

    public method set_cuts { low high } {
	if { [catch {$graph_ axis configure y -min $low -max $high}] } { return }
	trace_setvar lcut$id $low
	trace_setvar hcut$id $high
    }

    # return cut levels

    public method get_cuts { } {
	return "[$graph_ axis cget y -min] [$graph_ axis cget y -max]"
    }
    
    # set cuts to min / max of the visible plot.

    public method auto_cut { args } {
	lassign [get_xaxis_limits] xmin xmax
	set xmin [expr {max($xmin - 0.0, 1)}]  ; # subtract 0.0 to workaround bug in tcl
	if { [catch {expr {round($xmax - $xmin - 1.0)}} w] } { return }
	if { $w < 2 } { return }

	lassign [$image_ info minmax $xmin $yline $w 1] low high
	if { [catch {$graph_ axis configure y -min $low -max $high}] } { return }

	trace_setvar lcut$id $low
	trace_setvar hcut$id $high
    }
    
    # reset scale

    public method reset_scale { } {
	$graph_ axis configure x -min {} -max {}
    }
    
    # set scale (left, right unit as pixel number)

    public method set_scale { left right } {
	if { $left < 1 } { set left 1 }
	if { $right > [$image_ width] } { set right [$image_ width] }
	if { $left == $right } {
	    set right [expr {$left + 1}]
	}
	$graph_ axis configure x -min [xuval $left] -max [xuval $right]
    }
    
    # define callback command when zoom changes
    
    public method set_zoomCb { cmd } {
	global EDITING
	if { ! $EDITING } {
	    $graph_ element bind scale$id <2> "eval [code $cmd ZoomIn]"
	    $graph_ element bind scale$id <3> "eval [code $cmd ZoomOut]"
	}
    }

    # toggle marker movements

    private method mtoggle { xw tag } {
	if { $stopMarkers_ } {
	    set stopMarkers_ 0
	    markers_color 1
	    return
	}
	lassign [$graph_ invtransform $xw 0] xi y
	if {$xi > 1.0e+10} { return }
	markers_color 0
	set stopMarkers_ [expr {int($xi) + 0}]
    }

    # return x in user units

    private method xuval { x } {
	if { $with_range } {
	    set x [expr {$xstart + ($x - 1) * $xdelta}]
	}
	return $x
    }

    # return x in pixel units

    private method uxval { x } {
	if { $with_range } {
	    set x [expr {($x - $xstart) / $xdelta + 1}]
	}
	return $x
    }

    # return label id

    private method labelId { } {
	if { "$idlab" != "" } {
	    return $idlab
	}
	return $id
    }

    # cursor motion event: will be managed by parent

    public method notify_motion { args } {
	if { "$motionCb" != ""} {
	    eval $motionCb $args
	}
    }

    # process motion event

    public method motion_event { what {xw ""} {tag ""} } {
	switch -exact -- $what {
	    MOTION {
		if { $stopMarkers_ } {
		    set xi $stopMarkers_
		} else {
		    lassign [$graph_ invtransform $xw 0] xi y
		    if {$xi > 1.0e+10} { return }  ; # avoids core dump
		    set lastMove_ "MOTION $xw $tag"
		}
		lassign [$image_ get [expr {[uxval $xi] + 0.5}] $yline image] xx yy yval
		if {[catch {lassign [$graph_ transform $xi $yval] xw yw}]} { return }

		#
		# move crosshair and markers
		#
		if { $show_cross } {
		    $graph_ crosshairs configure -position @$xw,$yw
		    $graph_ crosshairs on
		}
		set d [expr {$with_range ? $xdelta : 1}]
		if { $show_pixmark } {
		    $graph_ marker configure $marker2_ -coords \
			"[expr {$xi - $d}] $yval [expr {$xi + $d}] $yval"
		}
		$graph_ marker configure $marker_ \
		    -text "[labelId]: [format [expr {$with_range ? "%-.3f" : "%-.0f"}] $xi], [format %-10g $yval]"
	    }
	    LEAVE {
		if { $stopMarkers_ } { return }
		set lastMove_ {}
		$graph_ marker configure $marker_ -text {}
		$graph_ marker configure $marker2_ -coords "0 0 0 0"
		$graph_ crosshairs off
	    }
	}	
    }

    # get setting of global array variable

    private method get_gvar { array name } {
	global $array
	return [set ${array}($name)]
    }

    # show the grid

    private method show_grid { array name opt } {
	$graph_ grid configure -minor 0 \
	    -hide [expr {[get_gvar $array $name] ? "no" : "yes"}]
    }

    # crosshair on/off

    private method show_cross { array name opt } {
	set bool [get_gvar $array $name]
	config -show_cross $bool
	$graph_ crosshairs [expr {$bool ? "on" : "off"}]
    }

    # pixel marker on/off

    private method show_pixmark { array name opt } {
	set bool [get_gvar $array $name]
	config -show_pixmark $bool
	$graph_ marker configure $marker2_ -linewidth [expr {$bool ? 15 : 1}]
    }

    # change markers color

    private method markers_color { bool } {
	set fg white; set bg $plotbg
	if { ! $bool } {
	    set fg red; set bg white
	}
	$graph_ crosshair configure -color $fg
	$graph_ marker configure $marker_ -outline $fg -background $bg
	$graph_ marker configure $marker2_ -outline $fg
    }

    # make a hard copy of the graph display

    public method print { } {
	utilReUseWidget util::GraphPrint .print -graph $graph_
	after 0 [code wm title .print "Scan [cget -id]"]
    }


    # -- private variables --

    private variable elem_  {}         ; # element name
    private variable stopMarkers_ 0    ; # markers stopped on/off
    private variable graph_ {}         ; # name of graph widget
    private variable lastMove_ {}      ; # parameters from last motion event
    private variable marker2_ {}       ; # line marker at pos. x,y
    private variable marker_ {}        ; # marker used for x,y display
    private variable nx_ 0             ; # number of pixels (0 = size of line)
    private variable pen_ {}           ; # pen id
    private variable xVector_ {}       ; # x vector for graph
    private variable yVector_ {}       ; # y vector for graph

    # -- public variables --

    # tickfont, ticklength, etc.
    public variable tickfont {-Adobe-helvetica-*-r-normal-*-10*}
    public variable ticklength {3}
    public variable leftmargin {0} {
	if { "$graph_" != "" && [cget -with_yticks] } {
	    $graph_ configure -lm [cget -leftmargin]
	}
    }

    # pen color
    public variable color {black} {
	if { "$graph_" != "" } {
	    $graph_ pen configure $pen_ -color [cget -color]
	}
    }
    # motion callback
    public variable motionCb {}

    # scanline number
    public variable yline {0}

    # widget is activated, bool
    public variable activated {0} {
	if { "$graph_" != "" } {
	    update_plot
	}
    }
    # widget is visible, bool
    public variable visible {1} {
	if { "$graph_" != "" } {
	    update_plot
	}
    }
    # scaline width (0 = width of image)
    public variable swidth {0} {
	if { ! [$image_ isclear] && "$graph_" != "" } {
	    set nx_ [cget -swidth]
	    set iw [$image_ width]
	    if { $nx_ == 0 || $nx_ > $iw } { set nx_ $iw }
	    set_scale 1 $nx_
	}
    }
    # x-axis range (start, delta)
    public variable xstart {0}
    public variable xdelta {0.1}
    public variable with_range {0} {
	if { "$graph_" != "" } {
	    update_plot
	}
    }
    # label used for short help and scan window
    public variable idlab {}
}

#
# ___bltplot___



