#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bltplotRtdPlot.tcl,v 1.4 2005/07/07 10:46:52 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/05/04  created
#

#************************************************************************
#   NAME
#   BltplotRtdPlot - Plot x-lines (scans) from an Rtd image using BLT
# 
#   SYNOPSIS
#   BltplotRtdPlot <widget-name> ?widget options?
# 
#   DESCRIPTION
#   BltplotRtdPlot is a public itk widget class to plot scanlines of
#   an Rtd image using the image area. This class can be loaded as a
#   mega widget from the panel editor and configured (see OPTIONS).
#
#   Scanlines are specified in scanline groups which permits to set
#   cut levels and the scale for an entire group with one command.
#   Normally, scanlines which have the same "nature" are grouped, e.g.
#   normalized vectors which values are in the range (0, 1) are in one
#   group and scanlines which show raw data are placed into another
#   group.
#   Scanline groups and the individual scanlines can be configured
#   via a control panel and public methods (see PUBLIC METHODS).
#
#   OPTIONS
#   plotGroupList
#     - defines the scanline groups and is a tcl list with the elements
#
#                 <group id> <length of scanlines> ?<xstart> <xdelta>?
#
#       o <group id> is a group identifier which name appears in the scan
#         control panel and in short help texts.
#       o <length of scanlines> if set to 0 the entire scanline is displayed,
#         else it defines the number of pixels to be displayed starting with
#         the first pixel.
#       o OPTIONALLY the following parameters can be specified:
#         xstart: is the x-axis value for the first pixel
#         xdelta: is the delta value of two pixels
#         The values can be changed with the method set_xrange
#     - Example:
#         "Group0 0" "Group1 250" "Group2 0 100. 0.1"
#
#   plotScanList0
#     - defines the scanline configuration for group 0 (similar plotScanList1 defines
#       the scanline configuration for group 1, etc.). Each scanline
#       specification consists of the following elements (delimited by quotes):
#
#                 <scanline id> <scanline number> <scanline color> ?<scanline label>?
#
#       o <scanline id> is a scanline identifier which name appears in the scan
#         control panel and in short help texts.
#       o <scanline number> is the line of the array (starting with 1).
#       o <scanline color> is the scanline color.
#       o OPTIONALLY the following parameter can be specified:
#         scanline label: is the text which appears in the short help or scanline window.
#                         The default label is the scanline id.
#     - Example:
#         "SL1 1 yellow" "SL2 2 aquamarine {Power spectrum}"
#
#   plotHideGroup
#     - groups to hide after startup (specify Group Id list). The public method
#       to show/hide a group is show_group.
# 
#   plotHideScan
#     - scans to hide after startup (specify Scan Id list). The public method
#       to show/hide a scan is show_scan.
#
#   with_scale
#     - show/hide the scale widgets (in the control panel) and scrollbars
#       (for the plot groups).
#
#   with_ctrl
#     - show/hide the control panel. If option -with_ctrl is 0 then no
#       control panel will be available. If it is 1 then the control panel
#       will be build and its visibility can be changed with method show_ctrl.
#
#   show_cross
#     - default: show a crosshair when moving the mouse cursor over the plot area.
#
#   show_pixmark
#     - default: show a pixel marker when moving the mouse cursor over the plot area.
#
#   plotbg
#     - plot background color
#
#   width
#     - width of the plot and image window (in number of screen bins)
#
#   height
#     - height of the plot and image window (in number of screen bins)
#
#   with_xticks
#     - with x axis tick labels. Draw tick labels along the x axis.
#
#   with_yticks
#     - with y axis tick labels. Draw tick labels along the y axis.
#
#   scrollbars
#     - show the scrollbars of RtdImage
#
#   leftmargin
#     - left margin for y-labels (i.e. the space in number of pixels
#       to allocate for y-labels). Default is 0 (=dynamic allocation).
#       Typical values are 40-60 pixels.
#       See also method set_leftmargin.
#
#   PUBLIC METHODS
#     public method show_ctrl { {flag 1} }
#       - show/hide the control widget
#
#     public method show_group { groupId {flag 1} }
#       - show/hide a group
#
#     public method show_scan { scanId {flag 1} }
#       - show/hide a scanline
#
#     public method show_plot { {flag 1} }
#       - show/hide plot widget (toggle display of image and plot)
#
#     public method getRtdImage { }
#       - return name of the RtdImage widget (see RtdImage(n))
#
#     public method attach { {flag 1} {camera ""} }
#       - camera attach/detach. Default camera name is RTD_CAMERA.
#
#     public method set_cuts { id low high }
#       - set cut levels for a group or scan (identified by the identifier id)
#
#     public method assign_scan { scanId groupId { groupPos 0 } }
#       - assign scanline to group. This methods puts the scanline identified by 'scanId'
#         into the group identified by 'groupId'. The position of the scanline within
#         the group is specified by 'groupPos' (starting from 0 if it is the first
#         one within the group).
#
#     public method set_xrange { groupId {with_range 0} {xstart 1} {xdelta 1}}
#       -  set x-axis range for a group
#          with_range: if true, the user units xstart and xdelta are used, else
#                      the pixel coords
#          xstart:     the x-value at the first pixel
#          xdelta:     the delta value of two pixels
#
#     public method set_leftmargin { numpix }
#       -  set left margin for y-axis labels (see option -leftmargin)
#
#     public method print { scanId }
#       - print scanline identified by 'scanId'
#
#   GRAPH CONTROL
#     Each individual graph or a group of graphs can be controlled with
#     the methods and configuration options which are available for BLT graph.
#     For this BltplotRtdPlot provides for the method get_pathName. Direct
#     control of graphs can be used e.g. for configuring the axis.
#     See also the original man page of BLT graph which gets installed
#     as bltgraph(n) after 'make install'.
#
#     public method get_pathName { id }
#       - if 'id' identifies a scanline then the pathname of the graph
#         is returned.
#       - if 'id' identifies a scanline group then a tcl list of all
#         graphs within the group is returned.
#
#   BINDINGS
#     The following mouse pointer bindings are used:
#       - mouse button <2> click in a scanline area: zoom in
#       - mouse button <3> click in a scanline area: zoom out
#       - double mouse button <1> click in a scanline area: toggle hold ("freeze")
#         or release x-position.
#       - mouse movement in a scanline area: show x,y values at mouse x-position
#         as markers
#       - mouse movement in the image area: show x,y values at mouse x,y position
#         in the short help area
#
#   PARENT CLASS
#   itk::Widget - not BltplotBase since the itk options
#     (public variables) can be set via the panel editor.
#
#   CAVEATS
#     - the tcl command 'update' must be avoided while the BltplotRtdPlot widget
#       is programatically configured. New configurations are executed after
#       the tcl shell returned to its idle point.
#
#   SEE ALSO
#   graph(n), RtdImage(n), rtdimage(n)
#   The original man page of BLT graph is installed as bltgraph(n)
#
#   BUGS
#     - options which are changed via the panel editor need to
#       be applied first ("Apply") before pressing "OK". If you
#       just press "OK" the right mouse click does not popup the
#       Configuration Panel anylonger. In this case you can save
#       the panel and run the panel editor again.
#     - it was reported that entering numbers into the cut level
#       entries do not work after pressing 'A'. After deleting the 
#       whole number it should work.
#
#------------------------------------------------------------------------
#

# help the user to find the tcl library
if { [catch {seq_findFile lib/libbltplotPrivate.tcl } path] } { 
    puts stderr "Tcl library libbltplotPrivate.tcl not found " 
    exit -1
} else {
    global auto_path
    lappend auto_path $path
}
itk::usual BltplotRtdPlot {}

class BltplotRtdPlot {
    inherit itk::Widget

    constructor {args} {
	global EDITING
	set w_ [namespace tail $this]

	bltplot_defaults  ; # set widget defaults

	eval itk_initialize $args
	set itkinitd_ 1

	# create Rtd widget
	#
	set wrtd_ $w_.rtd
	BltplotRtd $wrtd_ \
	    -width       $width \
	    -height      $height \
	    -scrollbars  $scrollbars

	set rtdImage_ [$wrtd_ cget -image]

	# create scans and associated control window
	#
	make_scans
	update  ; # required due to 'after' scripts by method make_scans
	show_plot $EDITING

	# add trace variables for events
	wtrace 0
    }

    # create scans (called by constructor and when options are configured)

    private method make_scans { } {
	if { ! $itkinitd_ || $new_options_ } { return }
	set new_options_ 1

	catch {[$rtdImage_ get_image] camera stop}

	# wait until we have got all new config options
	#
	update idletasks ; # needed to catch other options
	after 0 [code $this create_scans]
    }

    # create scans according to current configuration
    
    private method create_scans { } {
	# if an option was changed then re-make the plot window
	#
	if { ! $new_options_ } { return }

	wtrace
	wdestroy wplt_
	wdestroy wctrl_

	set new_options_ 0
	make_plot 
	if { $with_ctrl } {
	    make_ctrl
	}

	show_plot $plot_shown_

	catch {[$rtdImage_ get_image] camera continue}
    }

    # show/hide control widget

    public method show_ctrl { {flag 1} } {
	if { $with_ctrl } {
	    wm [expr {$flag ? {deiconify} : {withdraw}}] $wctrl_
	}
    }

    # set cut levels for a group or scan 

    public method set_cuts { id low high } {
	check_update
	foreach gr [$wplt_ get_groupObjs] {
	    if { "[$gr cget -id]" == "$id" } {
		$gr call_objs set_cuts $low $high
		return
	    }
	    foreach sc [$gr get_scanObjs] {
		if { "[$sc cget -id]" == "$id" } {
		    $sc set_cuts $low $high
		    return
		}
	    }
	}
	puts stderr "set_cuts: could not find scan/group '$id'"
    }

    # set x-axis range for a group

    public method set_xrange { groupId {with_range 0} {xstart 1} {xdelta 1}} {
	check_update
	foreach gr [$wplt_ get_groupObjs] {
	    if { "[$gr cget -id]" == "$groupId" } {
		$gr set_xrange $with_range $xstart $xdelta
		return
	    }
	}
	puts stderr "set_xrange: could not find group '$groupId'"
    }

    # show/hide group

    public method show_group { groupId {flag 1} } {
	check_update
	set idx 0
	foreach gr [$wplt_ get_groupObjs] {
	    if { "[$gr cget -id]" == "$groupId" } {
		BltplotBase::trace_setvar hideGroup$idx [expr { ! $flag }]
		return
	    }
	    incr idx
	}
    }

    # show/hide scanline

    public method show_scan { scanId {flag 1} } {
	check_update
	BltplotBase::trace_setvar hideScan$scanId [expr { ! $flag }]
    }

    # check if "update" is needed after possible pending config work

    private method check_update { } {
	if { $new_options_ } {
	    update
	}
    }

    # destroy widget

    private method wdestroy { name } {
	set w [set $name]
	set $name {}
	if { "$w" != "" } { 
	    update ; # wait for completion of possible "after" scripts
	    destroy $w
	}
    }

    # trace variables

    private method wtrace { { uset 1 } } {
	global bltplot
	if { $uset } {
	    catch {unset bltplot} ; # removes trace variables
	}
	BltplotBase::trace_addvar resetPlot [code $this resetPlotCb]
    }

    # make plot window

    private method make_plot { } {
	set gl [parse_config_list $plotGroupList 2]
	
	# max. number of groups is defined by the variable maxgroups_
	set groupList [lrange $gl 0 [expr {$maxgroups_ -1}]]

	# parse scanline options
	#
	set sll {}
	loop i 0 $maxgroups_ {
	    set sl [parse_config_list [set plotScanList$i] 3]
	    lappend sll $sl
	}

	BltplotPlot $w_.plt \
	    -width         $width \
	    -height        $height \
	    -image         $rtdImage_ \
	    -plotGroupList $groupList \
	    -plotScanList  $sll \
	    -show_cross    $show_cross \
	    -show_pixmark  $show_pixmark \
	    -plotbg        $plotbg \
	    -with_scale    $with_scale \
	    -with_xticks   $with_xticks \
	    -with_yticks   $with_yticks

	set wplt_ $w_.plt

	# hide groups and scans
	foreach grId $plotHideGroup {
	    show_group $grId 0
	}
	foreach scId $plotHideScan {
	    show_scan $scId 0
	}
	set_leftmargin $leftmargin
    }

    # make control panel

    private method make_ctrl { } {
	BltplotCtrl $w_.ctrl \
	    -withdraw      1 \
	    -center        0 \
	    -plotWdg       $wplt_ \
	    -with_scale    $with_scale \
	    -show_cross    $show_cross \
	    -show_pixmark  $show_pixmark

	set wctrl_ $w_.ctrl
    }

    # show/hide plot widget

    public method show_plot { {flag 1} } {
	check_update
	set plot_shown_ $flag
	$wrtd_ show_rtd [expr { ! $flag }]
	$wplt_ show_plot $flag
	show_ctrl $flag
    }

    # return name of the RtdImage widget

    public method getRtdImage { } {
	return $rtdImage_
    }

    # print scanline

    public method print { scanId } {
	check_update
	foreach gr [$wplt_ get_groupObjs] {
	    foreach sc [$gr get_scanObjs] {
		if { "[$sc cget -id]" == "$scanId" } {
		    $sc print
		    return
		}
	    }
	}
	puts stderr "print: could not find scan '$scanId'"
    }
    
    # set left margin for y-axis labels

    public method set_leftmargin { numpix } {
	check_update
	foreach gr [$wplt_ get_groupObjs] {
	    $gr set_leftmargin $numpix
	}
    }

    # check if string str is delimited by quotes or braces

    private method check_delimit { str } {
	set c [string index [string trim $str] 0]
	if { "$c" == "\"" || "$c" == "\{" } { return 1 }
	return 0
    }

    # parse and return a config list (plotScanList or plotGroupList)

    private method parse_config_list { plotlist nitems } {
	# case 1: the list may not be delimited by quotes or braces
	set lst $plotlist
	if { ! [check_delimit $lst] && [llength $lst] >= $nitems } {
	    set lst [list $lst]
	    return $lst
	}

	# case 2: list elements may be delimited by braces and quotes.
	# Remove empty items as well.
	set lst {}
	foreach el $plotlist {
	    if { ! [lempty $el] } {
		lappend lst [join $el]
	    }
	}
	return $lst
    }

    # assign scanline to group

    public method assign_scan { scanId groupId { groupPos 0 } } {
	set idx 0
	foreach grl $plotGroupList {
	    if { "[lindex $grl 0]" == "$groupId" } {
		set gridx $idx
		break
	    }
	    incr idx
	}
	if { ! [info exists gridx] } {
	    puts stderr "could not find group id '$groupId'"
	    return 
	}

	loop grl 0 $maxgroups_ {
	    set idx 0
	    set psl [parse_config_list [cget -plotScanList$grl] 3]
	    foreach scl $psl {
		if { "[lindex $scl 0]" == "$scanId" } {
		    # 1. remove scan from old group
		    set plist [lreplace $psl $idx $idx ""]
		    config -plotScanList$grl [parse_config_list $plist 3]

		    # 2. place scan in new group
		    set psl [parse_config_list [cget -plotScanList$gridx] 3]
		    set plist [linsert $psl $groupPos $scl]
		    config -plotScanList$gridx [parse_config_list $plist 3]
		    return
		}
		incr idx
	    }
	}
	puts stderr "could not find scan id '$scanId'"
    }

    # camera attach/detach

    public method attach { {flag 1} {camera ""} } {
	global env
	if { "$camera" == "" } {
	    if { ! [info exists env(RTD_CAMERA)] } {
		puts stderr "RTD_CAMERA was not set. Skipping attach command."
		return
	    } 
	    set camera $env(RTD_CAMERA)
	}
	eval $rtdImage_ [expr {$flag ? "attach_camera $camera" : "detach_camera"}]
    }

    # re-make plot widget (via trace variable set by class BltplotCtrl)

    protected method resetPlotCb { args } {
	wdestroy wplt_
	# make control window busy while creating the plot window
	$wctrl_ busy [code $this make_plot]
	show_plot $plot_shown_
    }

    # return the pathname of a scan or a tcl list of pathnames of a group
    # (identified by 'id')

    public method get_pathName { id } {
	check_update
	foreach gr [$wplt_ get_groupObjs] {
	    if { "[$gr cget -id]" == "$id" } {
		return [$wplt_ get_groupObjs]
	    }
	    foreach sc [$gr get_scanObjs] {
		if { "[$sc cget -id]" == "$id" } {
		    return $sc
		}
	    }
	}
	puts stderr "get_pathName: could not find scan/group '$id'"
	return ""
    }

    # -- private variables --

    private variable itkinitd_ 0         ; # itk_initialize was done, bool
    private variable maxgroups_ 10       ; # max number of groups
    private variable new_options_ 0      ; # new config options, bool
    private variable plot_shown_ 0       ; # currently plot is shown, bool
    private variable rtdImage_           ; # RtdImage widget name
    private variable w_                  ; # shorter name for $itk_component(hull)
    private variable wctrl_ {}           ; # ctrl window name
    private variable wplt_ {}            ; # plot window name
    private variable wrtd_               ; # Rtd window name


    # -- public variables -- configurable via panel editor --

    # show scrollbars of RtdImage
    public variable scrollbars {0} {
	# catch to workaround bug in Rtd
	catch {$wrtd_ config -scrollbars [cget -scrollbars]}
    }
    # width of Rtd Image
    public variable width {400} {
	if { [info exists wrtd_ ] } {
	    $wrtd_ config -width [cget -width]
	    $wplt_ config -width [cget -width]
	}
    }
    # hight of Rtd Image
    public variable height {400} {
	if { [info exists wrtd_ ] } {
	    $wrtd_ config -height [cget -height]
	    $wplt_ config -height [cget -height]
	}
    }
    # default: show cross
    public variable show_cross {0} { make_scans }

    # default: show pixel marker
    public variable show_pixmark {1} { make_scans }

    # plot background color
    public variable plotbg {midnightblue} { make_scans }

    # left margin for y-labels
    public variable leftmargin {0} { make_scans }

    # with scale widgets and scrollbars
    public variable with_scale {1} { make_scans }

    # with x axis tick labels
    public variable with_xticks {0} { make_scans }

    # with y axis tick labels
    public variable with_yticks {0} { make_scans }

    # default: build and show control panel
    public variable with_ctrl {1} {
	if { "$wctrl_" != "" } {
	    show_ctrl [cget -with_ctrl]
	}
    }
    # just a hint for the panel editor user
    public variable HELP {See man page of BltplotRtdPlot(n)}

    # groups to hide after startup (specify Group Id list)
    public variable plotHideGroup {}

    # scans to hide after startup (specify Scan Id list)
    public variable plotHideScan {}

    # scanline group configuration
    public variable plotGroupList {"Group0 0" "Group1 0"} {
	make_scans
    }
    # scanline configuration
    public variable plotScanList0 {"SL1 1 yellow" "SL2 2 aquamarine"} { make_scans }
    public variable plotScanList1 {"SL3 3 sienna1"}                   { make_scans }
    public variable plotScanList2 {} { make_scans }
    public variable plotScanList3 {} { make_scans }
    public variable plotScanList4 {} { make_scans }
    public variable plotScanList5 {} { make_scans }
    public variable plotScanList6 {} { make_scans }
    public variable plotScanList7 {} { make_scans }
    public variable plotScanList8 {} { make_scans }
    public variable plotScanList9 {} { make_scans }
}

#
# ___bltplot___










