#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: canioxAnalogOSignal.tcl,v 1.3 2005/09/26 15:16:56 vltsccm Exp $"
#
# Makefile of ........
#
# who       when      what
# --------  --------  ----------------------------------------------
# vltuser5  06/02/02  created
#


#************************************************************************
#   NAME
#   
#       CanioxAnalogOSignal - Class description for an output signal of an analog
#                     I/O device.
#
#   SYNOPSIS
#
#       CanioxAnalogOSignal <widget-name> ?<options>?
# 
#   DESCRIPTION
#
#       CanioxAnalogOSignal inherits from the base class canioxAnalogSignal general
#       attributes common to I/O signals and defines particular
#       attributes of output signals of an analog I/O device. 
#       This class is intended to be used only as a base class. Derived
#       classes must take care of the interaction with the user and call the 
#       methods provided here to comunicate with the device's input line.  
#      
#       An object of this class is created by:
#
#       CanioxAnalogOSignal <object-name> -<Option> <Value> ...
#
#   which is typically done by the constructor of the derived class.
#       
#
#   OPTIONS
#   Options represent public variables of the object and can be set with:
#
#       <object-name> configure -<Option> <Value> ...
#
#   The are no special options for this class.
#
#
#   METHODS
#   Methods are invoked with:
#
#       <object-name> <Method> <Arguments> ...
#
#   All the methods in this class are currently intended to be used from 
#       derived classes.
#
#       writeValue       -  write value in volts or in the internal
#               representation to the output signal. 
#
#   loadData     -  set internal state from data list (saved
#               with a Save Panel).
#
#   configLcc    - set internal state from data list with LCC
#                          configuration for analog I/O signals.
#
#       This class inherits methods from the base clas AioxSignal.
#
#   CAUTIONS
#   The class uses common variables.
#       The class asumes the existence of the global array ${this}, defined in
#       the base class and extends it.
#   All signals must belong to the same device.
#   In this implementation it is not possible to have signals
#   that belong to different devices at the same time.
#
#   EXAMPLES
#
#   SEE ALSO
#   aioxInducer(1),
#   InducerClass(n), InducerDevice(n), AioxDevice(n)
#       AioxSignal(n)
#
#   BUGS
#
#------------------------------------------------------------------------
#
class CanioxAnalogOSignal {
    inherit CanioxAnalogSignal

    constructor {args} { }
    
    method readSignalConfig { } { }
    method setSignalConfig { configLiteral } { }
    method setSignalConfig2 { } { }
    method writeValue {} { }
    method getValue {} { }
    
    method updateSignal { } {}
    method resumeConnection { } { }
    method report { } { }
    method loadData {data} { }

    #---------------------------------
    # Config from LCC
    #---------------------------------
    method configLcc { cnfString } { }

}

body CanioxAnalogOSignal::constructor {args} {
    global ::${itk_interior}
    
    set ${itk_interior}(signalGain) 1
    set ${itk_interior}(writeCmd) "WRITE_VALUE"
    eval itk_initialize $args
}

body CanioxAnalogOSignal::readSignalConfig { } {
    global ::${itk_interior}

    debugPrint
    CanioxAnalogSignal::readSignalConfig GET_OUTPUT_CONFIG
    set ${itk_interior}(signalConfigLiteral) \
        [$this configToLiteral [set ${itk_interior}(signalConfig)]]
}

body CanioxAnalogOSignal::setSignalConfig { configLiteral } {
    global ::${itk_interior}

    set ${itk_interior}(signalConfig) [$this literalToConfig $configLiteral]
    CanioxAnalogSignal::setSignalConfig SET_OUTPUT_CONFIG
}

body CanioxAnalogOSignal::setSignalConfig2 { } {
    global ::${itk_interior}
    
    CanioxAnalogSignal::setSignalConfig SET_OUTPUT_CONFIG
    readSignalConfig
}

body CanioxAnalogOSignal::writeValue {} {
    if {$onLine == 0} { return }
    debugPrint
    global ::${itk_interior}
    
    set cmd [set ${itk_interior}(writeCmd)]
    if {$cmd == "WRITE_VALUE"} {
    #ensure double format to vLevel
	set vlev [expr {[set ${itk_interior}(vLevel)] + 0.0}]
    } else {
    set vlev [set ${itk_interior}(vLevel)]
    set vlev [expr {int($vlev)}]
    }
    set args "$itk_option(-signalNumber);0;0;$vlev"
    set r [sendIoctlTrap $cmd $args]
}

body CanioxAnalogOSignal::getValue {} {
    if {$onLine == 0} { return }
    debugPrint
    global ::${itk_interior}
    
    set cmd [set ${itk_interior}(writeCmd)]
    set vLevel 0
    if {$cmd == "WRITE_VALUE"} {
        set r [sendIoctlTrap "GET_VALUE" "$itk_option(-signalNumber);0;0;0.0"]
        scan [lindex $r 0] "%d;%d;%d;%lf" x x x vLevel
        set vLevel [expr {round(10000.0 * $vLevel)/10000.0}]
        set ${itk_interior}(vLevel) [expr {$vLevel + 0.0}]
    } else {
        set r [sendIoctlTrap "GET_BIN_VALUE" "$itk_option(-signalNumber);0;0;0"]
        scan [lindex $r 0] "%d;%d;%d;%d" x x x vLevel
        set ${itk_interior}(vLevel) [expr {int($vLevel)}]
    }
}

body CanioxAnalogOSignal::updateSignal { } {
    readSignalConfig
    getValue
}

body CanioxAnalogOSignal::resumeConnection { } {
    setSignalConfig2
    writeValue
}

body CanioxAnalogOSignal::report { } {
    global ::${itk_interior}
    
    set number  "$itk_option(-signalNumber)"
    set name    "[list $itk_option(-signalName)]"
    set config  "[list [set ${itk_interior}(signalConfigLiteral)]]"
    set value   "[set ${itk_interior}(vLevel)]"
    return "$number $name $config $value"
}

body CanioxAnalogOSignal::loadData {data} {
    global ::${itk_interior}
    
    # Configure default bg color
    $itk_component(nr) configure -bg grey80

    $this configure -signalNumber [lindex $data 0]
    $this configure -signalName [lindex $data 1]
    set ${itk_interior}(signalConfigLiteral) [lindex $data 2]
    set ${itk_interior}(signalConfigl) \
        [$this literalToConfig [set ${itk_interior}(signalConfigLiteral)]] 
    set ${itk_interior}(vLevel) [lindex $data 3]

}

#---------------------------------
# Config from LCC
#---------------------------------
body CanioxAnalogOSignal::configLcc { cnfString } {
    global ::${itk_interior}
    #-----------------------
    # Assign config values
    #-----------------------
    set name  [lindex $cnfString [expr {[llength $cnfString] -1}] ]
    $this configure -signalName $name
    set factor [lindex $cnfString 3]
    set lowerRange [expr {round([lindex $cnfString 4]/$factor)}]
    set higherRange [expr {round( [lindex $cnfString 5]/$factor)}]

    #---------------------
    # Range checking
    #---------------------
    if {$lowerRange == -10 && $higherRange==10}  {
    set ${itk_interior}(signalConfig) 1
    } else {
    ::inducerDialog "Invalid Range in Lcc Configuration"
    set ${itk_interior}(signalConfig) 0
    }
    
}
