#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: canioxAnalogSignal.tcl,v 1.3 2005/09/26 15:16:56 vltsccm Exp $"
#
# Makefile of ........
#
# who       when      what
# --------  --------  ----------------------------------------------
# vltuser5  06/02/02  created
#

#************************************************************************
#   NAME
#   
#       CanioxAnalogSignal - Class description for a signal of an analog
#                        I/O device.
#
#   SYNOPSIS
#
#       CanioxAnalogSignal <widget-name> ?<options>?
# 
#   DESCRIPTION
#
#       CanioxAnalogSignal defines a base class for a signal in analog I/O 
#       device. This class is intended to be used only as a base class. 
#       Derived classes must take care of the differences between
#       input and output signals.
#
#   OPTIONS
#   Options represent public variables of the object and can be set with:
#
#       <object-name> configure -<Option> <Value> ...
#
#   The following options are publicly available:
#
#   -sNum           specifies the number of the signal within the device.
#                       The valid range depends of the type of signal.
#   METHODS
#   Methods are invoked with:
#
#       <object-name> <Method> <Arguments> ...
#
#   All the methods in this class are currently intended to be used from 
#       derived classes.
#
#       setInputConfig   -  specifies the input signal range. 
#                          
#       readSignalConfig -  read configuration value of the signal.
#
#   PROCS
#   Procs are invoked with:
#
#       <object-name> <Proc> <Arguments> ...
#
#   The following procs are publicly available:
#
#   setLine <object-name>   - make the object-name of the InducerLine
#       object known to the signal, that is used to communicate with
#       the LCU environment. All signals of a device must use the
#       same line-object.
#
#   CAUTIONS
#   The class uses common variables.
##  All signals must belong to the same device.
#   In this implementation it is not possible to have signals
#   that belong to different devices at the same time.
#
#   EXAMPLES
#
#   SEE ALSO
#   aioxInducer(1),
#   InducerClass(n), InducerDevice(n), AioxDevice(n)
#
#   BUGS
#
#------------------------------------------------------------------------
#
class CanioxAnalogSignal {
    
    inherit InducerClass 
    
    constructor { args } { }
    #---------------------------
    # widget's options
    #---------------------------
    itk_option define -signalNumber signalNumber SignalNumber 0 {set signalNumber $itk_option(-signalNumber)}
    itk_option define -vLevel vLevel VLevel 0.0 
    itk_option define -signalConfig signalConfig SignalConfig  0 
    itk_option define -signalName signalName SignalName  "<Signal Name>" 

    #------------------------------
    # Signal handling methods
    #------------------------------
    method setSignalConfig {opCode} { }
    method readSignalConfig {opCode} { }
    method resumeConnection { } { }
    
    #------------------
    # public symbols
    #------------------
  
    #------------------------------
    # Protected symbols
    #------------------------------
    protected variable configList { "+-10" }
    protected variable signalNumber
    #---------------------------
    # Literal to Config
    # Conversion Routines
    #---------------------------
    method literalToConfig { literal } {  }
    
    method configToLiteral {configL} {  }
    #------------------------------
    # Local Procs
    #------------------------------
    common lineW "" ;# object-name of the LCU connection, set by Device-object 
    common onLine 0; # to manage Disconnect/Resume 
    common displayMode "Float"  
    
    proc setDisplayMode {mode} {    }
    proc getDisplayMode { } {   }
    proc setLine {Line} {  }
    proc setOnLine { onOff } {  }
    proc getOnLine { } { }
    
    #send IOCTL command, ignore driver errors
    proc sendIoctl {Cmdlist Arglist {Sizlist ""}} { }
    
    #send IOCTL command, trap any driver error with dialog-box
    proc sendIoctlTrap {Cmdlist Arglist {Sizlist ""}} { }

    #
    # idea borrowed from ikonx
    # modified for hex -> bin
    common bin ;# common array for hex->binary display conversion
    common bin4 { 0000 0001 0010 0011 0100 0101 0110 0111\
        1000 1001 1010 1011 1100 1101 1110 1111 }
    
    proc initbin { } { }
}

body CanioxAnalogSignal::constructor { args } {
    
    # NOTE: Tk widgets in this class have to be connected to global 
    # variables (vars in the class scope do not work with Tk). These vars 
    # are stored in an array named as the object's name, i.e. "${this}".
    # The value of such a variable can be queried with "[set ${this}(..)]".
    # All methods using these vars must include a `global' statement:
    
    global ::${itk_interior}
 
    eval itk_initialize $args

    set ${itk_interior}(signalConfig)           $itk_option(-signalConfig)
    set ${itk_interior}(vLevel)                 $itk_option(-vLevel)
    set ${itk_interior}(signalName)             $itk_option(-signalName)
    set ${itk_interior}(signalConfigLiteral)    "N/A"
}

#------------------------------
# Signal handling methods
#------------------------------
body CanioxAnalogSignal::setSignalConfig {opCode} {
    debugPrint
    global ::${itk_interior}
    if {$onLine == 0} {return}
    set args "$itk_option(-signalNumber);[set ${itk_interior}(signalConfig)];0;0.0"
    sendIoctlTrap [list $opCode] "$args"
}

body CanioxAnalogSignal::readSignalConfig {opCode} {
    global ::${itk_interior}
    debugPrint
    if {$onLine == 0} {return}
    set args "$itk_option(-signalNumber);[set ${itk_interior}(signalConfig)];0;0.0"
    set r [sendIoctlTrap $opCode $args]
    scan [lindex $r 0] "%d;%d;%d;%lf" n ${itk_interior}(signalConfig) n n
    debugPrint "Config : [set ${itk_interior}(signalConfig)]"
}

body CanioxAnalogSignal::resumeConnection { } {
    ::inducerDialog "Resume Connection"
}

#------------------
# public symbols
#------------------
configbody CanioxAnalogSignal::vLevel {
    global ::${itk_interior}
    
    switch $displayMode {
    "Int"   { }
    "Hex"  -
    "Bin"  {
        # Dec->Hex
        set itk_option(-vLevel) [format "%4.4X" $itk_option(-vLevel)] 
        if {[string length $itk_option(-vLevel)] > 4 } {
        # The aio board always returns 2 bytes values,
        # but if the values is negative it's represented 
        # with 4 bytes then I strip the first two bytes
        # when the value is negative
        
        # strip first 4 characters (should be FFFF)
        set itk_option(-vLevel) [string range $itk_option(-vLevel) 4 end]
        }
        # Hex -> Bin
        if {$displayMode == "Bin"} { 
        for {set i 0} {$i<4} {incr i} {
            append b "$bin([string index $itk_option(-vLevel) $i]) "
        }
        set itk_option(-vLevel) $b
        }
    }
    }
    set ${itk_interior}(vLevel) $itk_option(-vLevel)
}

configbody CanioxAnalogSignal::signalConfig {
    global ::${itk_interior}
    
    set ${itk_interior}(signalconfig) $itk_option(-signalConfig)
}

configbody CanioxAnalogSignal::signalName {
    global ::${itk_interior}
    set ${itk_interior}(signalName) $itk_option(-signalName)
}

#---------------------------
# Literal to Config
# Conversion Routines
#---------------------------
body CanioxAnalogSignal::literalToConfig { literal } {
    set config [expr {[lsearch $configList $literal]+1}]
    return $config
}

body CanioxAnalogSignal::configToLiteral {configL} {
    set pos [expr {$configL -1}]
    return [lindex $configList $pos]
}

#------------------------------
# Local Procs
#------------------------------
body CanioxAnalogSignal::setDisplayMode {mode} {
    set displayMode $mode
}
body CanioxAnalogSignal::getDisplayMode { } {
    return $displayMode
}
body CanioxAnalogSignal::setLine {Line} {
    set lineW $Line ;#called by the Device object
}
body CanioxAnalogSignal::setOnLine { onOff } {
    set onLine $onOff
}
body CanioxAnalogSignal::getOnLine { } {
    return $onLine
}

#send IOCTL command, ignore driver errors
body CanioxAnalogSignal::sendIoctl {Cmdlist Arglist {Sizlist ""}} {
    return [$lineW Ioctl $Cmdlist $Arglist $Sizlist]
}

#send IOCTL command, trap any driver error with dialog-box
body CanioxAnalogSignal::sendIoctlTrap {Cmdlist Arglist {Sizlist ""}} {
    return [$lineW IoctlTrap $Cmdlist $Arglist $Sizlist]
}

body CanioxAnalogSignal::initbin { } {
    set hexList { 0 1 2 3 4 5 6 7 8 9 A B C D E F}
    foreach el $hexList {
    set bin($el) [lindex $bin4 "0x$el"]
    }
}