#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: canioxDevice.tcl,v 1.3 2005/09/26 15:16:56 vltsccm Exp $"
#
# Makefile of ........
#
# who       when      what
# --------  --------  ----------------------------------------------
# vltuser5  06/02/02  created
#

#************************************************************************
#   NAME
#
#   CanioxDevice - Class description for a "canio" I/O device
# 
#   SYNOPSIS
#
#   CanioxDevice <object-name> ?<options>?
#
#   DESCRIPTION
#
#   CanioxDevice defines the class for a "canio" I/O device.
#   It makes the functionality of the device and the corresponding 
#   "canio" driver available through a graphical user interface.
#
#   An object of this class is created by:
#
#       CanioxDevice <object-name> -<Option> <Value> ...
#
#   This will create all sub-objects for user interaction and
#   an object to handle the connection to a server-task in the 
#   LCU environment i.e. effectively to the physical device.
#
#   OPTIONS
#   Options represent public variables of the object and can be set with:
#
#       <object-name> configure -<Option> <Value> ...
#
#   The following device-specific options are publicly available:
#
#   -numberOfSignals    specifies the total number of signals per signal
#               type displayed in the panel. If the value is 0, all signals 
#               available on the device are displayed. 
#
#   -firstSignal        specifies the number of the first signal.
#               The default is 0.
#
#   Several options are inherited from InducerDevice and are therefore also
#   available within the scope of this class. See InducerDevice(n) for a 
#   detailed description of those common options.
#
#   METHODS
#   Methods represent procedures on the object and are invoked with:
#
#       <object-name> <Method> <Arguments> ...
#
#   There are no special device-specific methods available.
#
#   Several method are inherited from InducerDevice and overloaded here
#   with device-specific actions. See InducerDevice(n) for a detailed
#   description of those common methods.
#
#   PROCS
#
#   EXAMPLES
#
#   SEE ALSO
#   canioxInducer(1),
#   InducerClass(n), InducerDevice(n), InducerLine(n)
#
#   BUGS
#
#------------------------------------------------------------------------


#------------------------------------------------------------------------
#*# class: CanioxDevice

class CanioxDevice {
    
    inherit InducerDevice
    
    #--------------------------------------------------------------------
    # Device-specific members of CanioxDevice class
    #--------------------------------------------------------------------
    
    #*# constructor
    constructor { args } { }
    
    # widget's options
    itk_option define -firstSignal      firstSignal FirstSignal             0
    itk_option define -numberOfSignals  numberOfSignals NumberOfSignals     0
    #--
    protected variable analogOutSignalsW    {}
    protected variable analogInSignalsW     {}
    protected variable digitalOutSignalsW   {}
    protected variable digitalInSignalsW    {}
    protected variable analogSignalsW       {}
    protected variable digitalSignalsW      {}
    protected variable configW              {}
    protected variable firstSignalW         {}
    protected variable lastSignalW          {}
    protected variable numberOfSignalsW     {}
    protected variable signalTypeW          {}
    protected variable scrollPaneW	    {}
    protected variable initialized          0 
    protected variable deviceSwitch         0
    protected variable activePage           0  
    protected variable monitoring           0
    protected variable digitalSignalPage    0
    protected variable analogSignalPage     0

    #--------------------------------------------------------------------
    # Overloaded standard members of InducerDevice class
    #--------------------------------------------------------------------

    public method GetVersion        { } { }
    public method InitDevice        { } { }
    public method ExitDevice        { } { }
    public method ResetDevice       { } { }
    public method GetDeviceId       { } { }
    public method Disconnect        { } { }
    public method Resume            { } { } 
    public method GetSignalConfig   { } { }
    public method SaveDevice        { } { }
    public method LoadDevice        { data } { }
    public method LoadFromLCC       { } { }

    #-------------------------------------------------------------
    # Local Methods
    #-------------------------------------------------------------
   
    private method getDigitalConfig { name } { }
    private method getAnalogConfig  { name } { }
    private method getNumSignals    { } { }
    private method updateView       { } { }
    public  method switchActivePage { page } { }
    public  method destroyView      { } { }
    private method initSignals      { } { }
    
    #-----------------------------
    # Monitor Process 
    #-----------------------------
    public  method startMonitor     { } { }
    public  method stopMonitor      { } { }
    private method analogMonitorCB  { Reply } { }
    private method digitalMonitorCB { Reply } { }
}

#*# constructor
body CanioxDevice::constructor { args } {
    
    # integrate composite options
    eval itk_initialize $args
    
    # initialize member variables
    set initialized         0
    set monitoring          0
    set deviceSwitch        0
    set activePage          0
    set analogInSignalsW    {}
    set analogOutSignalsW   {}
    set digitalInSignalsW   {}
    set digitalOutSignalsW  {}
    set analogSignalsW      {}
    set digitalSignalsW     {}
    set configW             {}
    set numberOfSignalsW    {}
    set signalTypeW         {}       
    set firstSignalW        {}
    set lastSignalW         {}
}
#--------------------------------------------------------------------
# Overloaded standard members of InducerDevice class
#--------------------------------------------------------------------

body CanioxDevice::GetVersion { } {
    debugPrint
    global caniox argv0
    return "[file tail $argv0] - Version $caniox(version)"
}

body CanioxDevice::InitDevice { } {
    debugPrint

    set err [InducerDevice::InitDevice] ;# common part to open channel
    if {$err != ""} {
        return $err
    }

    # if already initialized, no complete GUI build up is needed
    if { $initialized > 0 } {
    
        stopMonitor

	# if device has changed, rebuild the signal panels
	if { $deviceSwitch > 0 } {
	    initSignals
	    set deviceSwitch 0
	}
                
        # all Signals must use the same Line object:
        CanioxDigitalSignal::setLine $lineW
        CanioxAnalogSignal::setLine $lineW

        CanioxDigitalSignal::setOnLine 1
        CanioxAnalogSignal::setOnLine 1

        # read current status of I/O lines
        updateView

        startMonitor

        return ""
    }

    # main frame
    itk_component add Mf0 {
	set Mf [frame $itk_interior.mf ] 
    } {
	usual
    }
    pack $Mf -side top -anchor n -fill both -expand yes -padx 1 -pady 1

    # tab panel
    itk_component add Tnb0 {
	set Tnb [iwidgets::tabnotebook $Mf.tnb]
    } {
	usual
    }
    pack $Tnb -anchor n -fill both -expand yes -padx 1 -pady 1
    
    # create 2 tabs
    $Tnb add -label "Digital Signals" \
	    -command [code $this switchActivePage 0]
    $Tnb add -label "Analog Signals"  \
	    -command [code $this switchActivePage 1]

    # page 0 is digital, page 1 is analog
    set digitalSignalPage [ $Tnb childsite 0 ]
    set analogSignalPage [ $Tnb childsite 1 ]

    # startup tab is digital page
    $Tnb select $activePage         

    # build up the signal panels
    initSignals
 
    CanioxAnalogSignal::setDisplayMode "Float"
    
    # all Signals must use the same Line object:
    CanioxDigitalSignal::setLine $lineW
    CanioxAnalogSignal::setLine $lineW
    
    # go online
    CanioxDigitalSignal::setOnLine 1
    CanioxAnalogSignal::setOnLine 1
 
    # read current status of I/O lines
    updateView

    # the monitor performs regular updates of the signal states
    # depending which page is currently active
    startMonitor
   
    # now the GUI is initialized
    set initialized 1
    
    return ""
}

body CanioxDevice::initSignals { } {
    debugPrint 
    # get options
    set firstSignal     $itk_option(-firstSignal)
    set numberOfSignals $itk_option(-numberOfSignals)

    # retrieves the number of signals supported by the device
    getNumSignals

    # digital input and output signals are numbered in the
    # following scheme: 
    # signal number: 0 ... <number of input signals - 1> 
    #   <number of input signals> ... <number of input signals
    #   + number of output signals - 1>
    # this numbering was chosen to be compatible with the
    # acroInducer panel 
    set n 0
    set offset 0
    foreach numSignal $numberOfSignalsW {
        set type [lindex $signalTypeW $n]
        switch -regexp $type {
            {[dD]igital[iI]n*} {
                # offset for digital out signal numbers
                set offset $numSignal
            }
            default { }
        }
        incr n  
    }
    
    # get the number of rows
    set n 0
    set row 1
    set signalIndexW {}
    set lastSignal 0    
    foreach numSignal $numberOfSignalsW {
        # any signals available
        if {$numSignal > 0 } {
            if { $firstSignal < $numSignal } {
                if { $numberOfSignals > 0 } {      
                    set lastSignal [ expr {$firstSignal + $numberOfSignals - 1}]
                    if { $lastSignal > $numSignal } {
                        set lastSignal [ expr {$numSignal - 1}]
                    }   
                } else {
                    set lastSignal [ expr {$firstSignal + $numSignal - 1}]                    
                }   
                    
                set type [lindex $signalTypeW $n]
                
                # choose the appropriate page for each signal type
                switch -regexp $type {
                    {[dD]igital*} {
                        set page $digitalSignalPage
                    }
                    {[Aa]nalog*} {
                        set page $analogSignalPage
                    }
                    { default } {
                        set page ""
                    }
                }

                # add a scroll pane
                itk_component add F$row {
                    set Sf [iwidgets::scrolledframe $page.$row \
                    -hscrollmode static -vscrollmode none ]
                } {
                    usual
                }
                pack $Sf -side top -anchor n -fill both -expand 1 -pady 2
				lappend scrollPaneW $Sf

                set F [ $Sf childsite ]
        
		# create a signal panel for each signal and put it
		# on the approriate page
                for {set j $lastSignal} {$j >= $firstSignal} \
                    { set j [expr {$j-1}]} {
                    # create the appropriate signal object
                    # for each type
                    switch -regexp $type {
                        
                        # digital input signals
                        {[dD]igital[iI]n*} {
                            itk_component add S$row.$j {
                                set S [CanioxDigitalSignal $F.$j \
                                    -signalNumber $j -inputOutput 0 \
                                    -borderwidth 1 -relief sunken]
                            }
                            pack $S -side right -anchor e -fill y -expand 1
                            # put the new object in the signal list
                            lappend digitalInSignalsW $S        
                            lappend digitalSignalsW $S      
                        }
                        
                        # digital output signals
                        {[dD]igital[oO]ut*} {
                            itk_component add S$row.$j {
                                set S [CanioxDigitalSignal $F.$j \
                                    -signalNumber [expr $j + $offset] \
                                    -inputOutput 1 \
                                    -borderwidth 1 -relief sunken]
                            }
                            pack $S -side right -anchor e -fill y -expand 1
                            # put the new object in the signal list
                            lappend digitalOutSignalsW $S       
                            lappend digitalSignalsW $S      
                        }
                        
                        # analog input signals
                        {[Aa]nalog[iI]n*} {
                            itk_component add S$row.$j {
                                set S [CanioxAnalogInputSignal $F.$j \
                                    -signalNumber $j \
                                    -borderwidth 1 -relief sunken]
                            }
                            pack $S -side right -anchor e -fill y -expand 1
                            # put the new object in the signal list
                            lappend analogInSignalsW $S     
                            lappend analogSignalsW $S       
                        }
                        
                        # analog output signals
                        {[Aa]nalog[oO]ut*} {
                            itk_component add S$row.$j {
                                set S [CanioxAnalogOutputSignal $F.$j \
                                    -signalNumber $j \
                                    -borderwidth 1 -relief sunken]
                            }
                            pack $S -side right -anchor e -fill y -expand 1
                            # put the new object in the signal list
                            lappend analogOutSignalsW $S        
                            lappend analogSignalsW $S       
                        }
                        default { }
                    }
                }
            incr row
            }   
        } 
        incr n  
    }
}

body CanioxDevice::ExitDevice { } {
    # device change
    set deviceSwitch 1
    debugPrint
    stopMonitor
    # remove scroll panes and signal panels
    destroyView
    return [InducerDevice::ExitDevice]  ;# common part to close channel
}

body CanioxDevice::ResetDevice { } {
    debugPrint
    set status [$lineW Ioctl RESET 0]
    updateView
    return status
}

body CanioxDevice::GetDeviceId { } {
    debugPrint
    set id [$lineW Ioctl READ_BOARD_ID 0]
    return [lindex $id 0]
}

body CanioxDevice::Disconnect { } {
    InducerDevice::Disconnect

    stopMonitor
    
    # go offline
    CanioxDigitalSignal::setOnLine 0
    CanioxAnalogSignal::setOnLine 0
    
}

body CanioxDevice::Resume { } {
    InducerDevice::Resume

    # go online again
    CanioxDigitalSignal::setOnLine 1
    CanioxAnalogSignal::setOnLine 1

    updateView  

    startMonitor
}

body CanioxDevice::GetSignalConfig { } {
    debugPrint
    # Father's behavior - Get cnfList
    InducerDevice::GetSignalConfig
    foreach i $cnfList {
        set name [givemeName $i]
        set type [givemeType $i]
        if { $type == "Digital" } {
            set reply [getDigitalConfig $name]
        } else {
            set reply [getAnalogConfig $name]
        }
        set reply [split $reply ,]
        # Check device's name
        if { [lindex $reply 0] != $itk_option(-vDev) } {
            continue
        }
        lappend reply $name
        lappend configW $reply 
    }
    debugPrint configW
    return $configW
}

body CanioxDevice::SaveDevice { } {
}

body CanioxDevice::LoadDevice { data } { 
}

body CanioxDevice::LoadFromLCC { } {
}

#-------------------------------------------------------------
# Local Methods
#-------------------------------------------------------------

body CanioxDevice::getNumSignals { } {
    debugPrint
    # get the number of digital input signals
    set response [$lineW Ioctl NUM_DIGITAL_INPUTS 0]
    lappend numberOfSignalsW [lindex $response 0]
    lappend signalTypeW "DigitalInputs"

    # get the number of digital output signals
    set response [$lineW Ioctl NUM_DIGITAL_OUTPUTS 0]
    lappend numberOfSignalsW [lindex $response 0]
    lappend signalTypeW "DigitalOutputs"
    
    # get the number of analog input signals
    set response [$lineW Ioctl NUM_ANALOG_INPUTS 0]
    lappend numberOfSignalsW [lindex $response 0]
    lappend signalTypeW "AnalogInputs"
    
    # get the number of analog output signals
    set response [$lineW Ioctl NUM_ANALOG_OUTPUTS 0]
    lappend numberOfSignalsW [lindex $response 0]
    lappend signalTypeW "AnalogOutputs"
}

body CanioxDevice::getDigitalConfig { name } {
    return [sendServer $itk_option(-vEnv) lccServer DIOGCNF $name ]
}


body CanioxDevice::getAnalogConfig { name } {
    return [sendServer $itk_option(-vEnv) lccServer AIOGCNF $name]
}

body CanioxDevice::updateView {} {
    debugPrint
    if { $activePage > 0 } {
        foreach s $analogSignalsW {
            $s updateSignal
        }
    } else {
        foreach s $digitalSignalsW {
            $s showSignalState
            $s showSignalConfig
        }
    }
}

body CanioxDevice::destroyView {} {
    debugPrint
    # remove analog signal panels
    foreach s $analogSignalsW {
        destroy $s 
    }

    # clear analog object lists
    set analogSignalsW {}
    set analogInSignalsW {}
    set analogOutSignalsW {}

    # remove digital signal panels
    foreach s $digitalSignalsW {
		destroy $s 
    }

    # clear digital object list
    set digitalSignalsW {}
    set digitalInSignalsW {}
    set digitalOutSignalsW {}
    set numberOfSignalsW {}
    set signalTypeW {}

    #remove scroll panels
    foreach s $scrollPaneW {
	destroy $s	
    }
    set scrollPaneW {}
}

body CanioxDevice::switchActivePage { page } {
    if { $initialized > 0 } {       
        set activePage $page
        stopMonitor

        updateView
        
        startMonitor
    }       
}

#-----------------------------
# Monitor Process 
#-----------------------------
body CanioxDevice::startMonitor { } {   
    if { $activePage > 0 } {
        set iMax [llength $analogInSignalsW]
        # Only up to 20 values can be specified to the MONITOR command
	if {$iMax > 20} {
	    set iMax 20
	}
	if { $iMax > 0 } {
	    set cmds "READ_VALUE"
	    set args ""
	    for {set i 0} {$i < $iMax } { incr i} {
		if { $i > 0 } {
		    set args "$args $i;0;0;0.0"     
		} else {
		    set args "$i;0;0;0.0"       
		}
	    }

	    $lineW MonitorStart \
		    [list $cmds] \
		    [list $args] {} \
		    [code $this analogMonitorCB]
	    set monitoring 1
	}
    } else {
        set iMax [llength $digitalSignalsW]
	if { $iMax > 0 } {
	    $lineW MonitorStart \
		    {READ_BITS READ_CONFIG_BITS} {0;32;0;0 32;32;0;0} {} \
		    [code $this digitalMonitorCB]
	    set monitoring 1
	}
    }
}

body CanioxDevice::stopMonitor {} {
    if { $monitoring > 0 } {
        $lineW MonitorStop
	set monitoring 0
    }       
}

body CanioxDevice::analogMonitorCB { Reply } {
    if { [CanioxAnalogSignal::getOnLine] > 0 } { 
        set iMax [llength $analogInSignalsW]
        for {set i 0} {$i < $iMax } { incr i} {
            scan [lindex $Reply $i] "%d;%d;%d;%f" x x x vLevel
            set vLevel [ expr {round(10000.0 * $vLevel)/10000.0}]
            set line [lindex $analogInSignalsW [expr {$iMax - $i - 1}]]
            $line configure -vLevel $vLevel 
        }
    }
}

body CanioxDevice::digitalMonitorCB { Reply } {
    if { [CanioxDigitalSignal::getOnLine] > 0 } {
        scan [lindex $Reply 0] "%d;%d;%d" x x \
        CanioxDigitalSignal::stateBits(0)
        scan [lindex $Reply 1] "%d;%d;%d" x x \
        CanioxDigitalSignal::stateBits(1)
        scan [lindex $Reply 2] "%d;%d;%d;%d" x x \
        CanioxDigitalSignal::dirBits(0) \
        CanioxDigitalSignal::polBits(0)
        scan [lindex $Reply 3] "%d;%d;%d;%d" x x \
        CanioxDigitalSignal::dirBits(1) \
        CanioxDigitalSignal::polBits(1)
        CanioxDigitalSignal::showAll
    } 
}

#___oOo___
