#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiCmdLib.tcl,v 3.11 2005/09/14 14:11:14 vltsccm Exp $" 
#
# ccseiCmdLib.tcl - Library of interface-procs to the CCS/cmd module
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 13.01.1995 Created
#	     22.10.1995 Added support for synonyms
#	     26.06.1995 Adapted to changes in CCS/cmd 1.12
# mcomin     01/05/1997 Add additional checks in ccseiCmdQuery 
#

#************************************************************************
#   NAME
#	ccseiCmdLib,
#	ccseiCmdQuery, 
#	ccseiCmdQueryLoadedCDTs,
#	ccseiCmdQueryCommands,
#	ccseiCmdQueryParameters,
#	ccseiCmdQueryMatchingCommands,
#	ccseiCmdQueryHelpString,
#	ccseiCmdQueryHelpText,
#	ccseiCmdQueryProcessesInSimulation,
#	ccseiCmdSetSimulationMode,
#	ccseiCmdSetNormalMode,
#	ccseiCmdQuerySynonyms,
#	ccseiCmdQueryTranslateSynonym,
#	ccseiCmdQueryParamType,
#	ccseiCmdQueryReplyParams,
#	ccseiCmdListCDTs,
#	ccseiCmdLoadCDT,
#	ccseiCmdCheckCDT,
#	ccseiCmdCheckEnvironment
#	- Tcl procedure interface to the CCS/cmd module
#
#   SYNOPSIS
#	ccseiCmdQuery				Cmd Par
#	ccseiCmdQueryLoadedCDTs
#	ccseiCmdQueryCommands			Process
#	ccseiCmdQueryParameters			Process Command
#	ccseiCmdQueryMatchingCommands		Process Command
#	ccseiCmdQueryHelpString			Process Command
#	ccseiCmdQueryHelpText			Process Command
#	ccseiCmdQueryProcessesInSimulation
#	ccseiCmdSetSimulationMode		Process
#	ccseiCmdSetNormalMode			Process
#	ccseiCmdQuerySynonyms			Process
#	ccseiCmdQueryTranslateSynonym		Process Synonym
#	ccseiCmdQueryParamType			Process Command Parameter
#	ccseiCmdQueryReplyParams		Process Command
#
#	ccseiCmdListCDTs
#	ccseiCmdLoadCDT				CdtName
#	ccseiCmdCheckCDT			CdtName
#
#	ccseiCmdCheckEnvironment
#
#   DESCRIPTION
#	A command is sent to the cmdManager running in the local environment
#	to query the requested value(s).
#	If the required function is not implemented in the cmdManager, then
#	other tools of the cmd module are used (cmdSetup, cmdUser).
#
#   PARAMETERS
#	Process - name of the process resp. CDT
#	Command - name of the command
#	Parameter - name of formal parameter
#	Synonym - name of command-synonym
#	CdtName - name of the CDT file
#
#   RETURNS
#	For Query commands:
#		Reply of the command, properly adjusted as a Tcl list
#		-1 on error
#
#	    The more complex return values are:
#
#	    for ccseiCmdQueryParamType: 
#		a list consisting of 9 elements:
#		0. PAR_NAME
#		1. PAR_UNIT
#		2. PAR_TYPE
#		3. PAR_OPTIONAL
#		4. PAR_REPETITION_FACTOR
#		5. PAR_MAX_REPETITION
#		6. PAR_RANGE_spec [ENUM,MIN/MAX]
#		7. PAR_RANGE_val: either a list of enumeration values (ENUM),
#			or a list consisting of min and max value (MIN/MAX).
#		8. PAR_DEF_VAL
#
#	    for ccseiCmdQueryReplyParams: 
#		a variable length list of lists, each sub-list consist of:
#		0. PAR_NAME
#		1. PAR_UNIT
#		2. PAR_TYPE
#		3. PAR_REPETITION_FACTOR
#		4. PAR_DEF_VAL
#
#	For CDT commands:
#		The standard output of the invoked utility program
#
#   ENVIRONMENT
#
#   VARIABLES
#	seq_ccsEnvName	Name of the local environment on the host workstation
#
#	The following variables are intended for possible name re-definitions:
#	ccsei_n(cmdManager) - name of CCS/cmd daemon process
#	ccsei_n(cmdSetup) - name of CCS/cmd setup executable
#	ccsei_n(cmdUser) - name of CCS/cmd user-command executable
#
#	ccseiCmd_v(trapErrorReply) - [0,1] if set , give a pop-up window
#		when the cmdManager returns an error-reply.
#
#   EXTERNAL PROCEDURES
#	ccseiDialog, ccseiTBD
#
#   CAUTIONS
#	If process-name or commands of the cmdManager change, then only these
#	procs have to be adjusted; no other procs should contain dependencies.
#
#   EXAMPLES
#
#   SEE ALSO
#	ccsei(1),
#	cmdManager(1), cmdSetup(1), cmdUser(1),
#	CCS User Manual VLT-MAN-ESO-17210-0619
#
#------------------------------------------------------------------------


###############################################################################
#
#	G L O B A L   V A R I A B L E S
#	"Declaration" of global variables
#
###############################################################################
# Rules: see ccsei.tcl
###############################################################################

# Set names if not already defined:

if {! [info exists ccsei_n(cmdManager)]} {set ccsei_n(cmdManager) cmdManager}
if {! [info exists ccsei_n(cmdSetup)]}   {set ccsei_n(cmdSetup) cmdSetup}
if {! [info exists ccsei_n(cmdUser)]}    {set ccsei_n(cmdUser) cmdUser}


# Do we want a pop-up window when the cmdManager returns an error-reply?
# 0=no, 1=yes.
if {! [info exists ccseiCmd_v(trapErrorReply)]} {
    set ccseiCmd_v(trapErrorReply) 0
}

# Global variable that indicated whether the cmdManager is dead or alive:
set ccseiCmd_v(dead) 0


###############################################################################
#
#	P R O C E D U R E S
#
###############################################################################
# Rules: see ccsei.tcl
###############################################################################

#
# cmdManager
#

proc ccseiCmdQuery { Cmd Par } {
    global seq_ccsEnvName ccsei_n ccseiCmd_v

    if {$ccseiCmd_v(dead)} { return {} }

    if {$Cmd == "CMDPAR"} {
       # Check if 'Par' has the format : "process,command"

       if {[regexp {([a-zA-Z]+),([a-zA-Z]+)} $Par] == 0 } {
          set    msg "Command NOT RECOGNIZED by cmdManager\n\n"
          append msg "Use Error Checking : NO_CHECK"
          tk_dialog .dialog "ccseiCmdQuery - Error" $msg info 0 OK
          return {} 
       }
    }

    set error 0
    set lr 0
    # send a command to the cmdManager in the local environment:
    set s_env $seq_ccsEnvName
    set s_pro $ccsei_n(cmdManager)
    set cmdId [seq_msgSendCommand $s_env $s_pro $Cmd $Par]
    set reply [seq_msgRecvReply $cmdId error lr]

    if {$error == 0} { return $reply } 

    if {$ccseiCmd_v(trapErrorReply)} {
	ccseiDialog "\"$Cmd $Par\" yields error #$error from $s_pro.\
		Reply:\n: $reply"
	return -1
    } else {
	return {}
    }
}

proc ccseiCmdQueryLoadedCDTs { } {
    set reply [ccseiCmdQuery CMDCLST ""]
    set cdtlist [lrange [split $reply ","]  1 end]
    return $cdtlist
}

proc ccseiCmdQueryCommands { Process } {

    if {[set reply [ccseiCmdQuery CMDLIST $Process ]] == -1 } {
       return -1
    }

    set comlist [split $reply ","]
    set comlist [lrange $comlist 1 end]
    return $comlist
}

proc ccseiCmdQueryParameters { Process Command } {

    if {[set reply [ccseiCmdQuery CMDPAR $Process,$Command ]] == -1 } {
       return -1
    }

    set parlist [split $reply ","]
##    if {[llength $parlist] <= 1} {return -1}
    set parlist [lrange $parlist 1 end]
    return $parlist
}

proc ccseiCmdQueryMatchingCommands { Process Command } {
    ccseiTBD
    #...
    # not implemented yet in the cmdManager! Will produce an error.
    set reply [ccseiCmdQuery CMDMTCH $Process,$Command ]
    if {$reply == -1} {return -1}
    set comlist [split $reply ","]
    set comlist [lrange $comlist 1 end]
    return $comlist
}

# 25.01.1995: replaced by ccseiCmdQueryReplyParams, see below
#proc ccseiCmdQueryReply { Process Command } {
#    set reply [ccseiCmdQuery CMDRPLY $Process,$Command ]
#    if {$reply == -1} {return -1}
#    return [lrange [split $reply ","] 1 end]
#}

proc ccseiCmdQueryHelpString { Process Command } {
    ccseiTBD
    #...
}

proc ccseiCmdQueryProcessesInSimulation { } {
    set reply [ccseiCmdQuery CMDSLST ""]
    if {$reply == -1} {return -1}
    set simlist [split $reply ","]
    set simlist [lrange $simlist 1 end]
    return $simlist
}

proc ccseiCmdSetSimulationMode { Process } {
    return [ccseiCmdQuery SIMSET $Process]
}

proc ccseiCmdSetNormalMode { Process } {
    return [ccseiCmdQuery SIMUSET $Process]
}

proc ccseiCmdQuerySynonyms { Process } {
    global seq_ccsCmdCheck ;# NOP if cmd-checking is off
    if {$seq_ccsCmdCheck == "NO_CHECK"} {return -1}

    set reply [ccseiCmdQuery SYNLST $Process]
    if {$reply == -1} {return -1}
    return [lrange [split $reply ","] 1 end]
}

proc ccseiCmdQueryTranslateSynonym { Process Synonym } {
    global seq_ccsCmdCheck ;# NOP if cmd-checking is off
    if {$seq_ccsCmdCheck == "NO_CHECK"} {return $Synonym}

    return [ccseiCmdQuery SYNTOC "$Process,$Synonym"]
}



# added 25.01.1995

proc ccseiCmdQueryParamType { Process Command Parameter } {
    set separator1 ","
    set separator2 ":"
    set reply [ccseiCmdQuery PARTYPE $Process,$Command,$Parameter ]
    set list [lrange [split $reply $separator1] 0 end]
    set len [llength $list]
    for {set i 0} {$i < $len} {incr i} {
	set list [lreplace $list $i $i [split [lindex $list $i] $separator2]]
    }
    return $list
}

proc ccseiCmdQueryReplyParams { Process Command } {
    set separator1 ","
    set separator2 ";"
    set reply [ccseiCmdQuery CMDRPLY $Process,$Command ]
    set list [lrange [split $reply $separator1] 1 end]
    set len [llength $list]
    for {set i 0} {$i < $len} {incr i} {
	set list [lreplace $list $i $i [split [lindex $list $i] $separator2]]
    }
    return $list
}



#
# cmdUser
#

proc ccseiCmdQueryHelpText { Process Command } {
    global ccsei_n ccseiCmd_v
    if {$ccseiCmd_v(dead)} { return {} }
    # this will probably replaced by a command to the cmdManager.
    # for now `cmdUser' is called:
    set exe bin/$ccsei_n(cmdUser)
    set exe [findVLTFile $exe] ;# locate the executable in the VLT areas
    # invoke utility, catch any error:
    if {[catch {exec $exe -p $Process -c $Command} helptext]} {
	ccseiDialog "The help-text for $Command is not available!\
		Are you sure that the correct CDT for $Process is loaded?\
		Run-time error message is:\n$helptext"
	set helptext ""
    }
#    set helptext [exec $exe -p $Process -c $Command] ;# call the utility
    return $helptext
}


#
# cmdSetup
#

proc ccseiCmdListCDTs { } {
    global ccsei_n ccseiCmd_v
    if {$ccseiCmd_v(dead)} { return {} }
    set exe bin/$ccsei_n(cmdSetup)
    set exe [findVLTFile $exe];# locate the executable in the VLT areas
    return [exec $exe] ;# call the utility with no option
}

proc ccseiCmdLoadCDT { CdtName } {
    global ccsei_n ccseiCmd_v
    if {$ccseiCmd_v(dead)} { return {} }
    # this will probably replaced by a command to the cmdManager.
    # for now `cmdSetup' is called:
    if {$CdtName == ""} {return "ccseiCmdLoadCDT: CDT filename not specified"}
    set exe bin/$ccsei_n(cmdSetup)
    set exe [findVLTFile $exe];# locate the executable in the VLT areas
    return [exec $exe -parse $CdtName] ;# call the utility with option
}

proc ccseiCmdCheckCDT { CdtName } {
    global ccsei_n ccseiCmd_v
    if {$ccseiCmd_v(dead)} { return {} }
    # this will probably replaced by a command to the cmdManager.
    # for now `cmdSetup' is called:
    if {$CdtName == ""} {return "ccseiCmdCheckCDT: CDT filename not specified"}
    set exe bin/$ccsei_n(cmdSetup)
    set exe [findVLTFile $exe];# locate the executable in the VLT areas
    return [exec $exe -syntax $CdtName] ;# call the utility with option
}



#
# etc.
#

# check if all parts of CCS/cmd are available.
# return "" if OK, else a message describing the problem
proc ccseiCmdCheckEnvironment { } {
    global ccsei_n ccseiCmd_v seq_ccsCmdCheck

    set ccseiCmd_v(dead) 0 ;# by default the cmdManger is alive

    set errStr "" ;# found problem descriptions are appended here

    # check if the `cmdManager' process is running in the local environment:
    set name $ccsei_n(cmdManager)
    if {[catch {ccseiCmdQuery VERSION ""}]} {
	set ccseiCmd_v(dead) 1
	set seq_ccsCmdCheck NO_CHECK
	append errStr \
		"\nThe `$name' is apparently not alive in the local\
		environment. This program can only work in degraded mode then,\
		e.g. it will not be possible to query any CDT information.\
		Command error checking will be switched off (NO_CHECK).\
		Ask your system manager, or see ${name}(1) how to start it.\n"
    }

    # check if there is an executable `cmdSetup' in the VTL-path:
    set name $ccsei_n(cmdSetup)
    catch {findVLTFile bin/$name} path
    if {$path == ""} {
	 append errStr \
		 "\nThe executable `$name' is apparently not present in the\
		 VLT areas. Without it ccsei will loose some functionality.\n"
    }

    # check if there is an executable `cmdUser' in the VTL-path:
    set name $ccsei_n(cmdUser)
    catch {findVLTFile bin/$name} path
    if {$path == ""} {
	 append errStr \
		 "\nThe executable `$name' is apparently not present in the\
		 VLT areas. Without it ccsei will loose some functionality.\n"
    }

    # check if there are some CDTs loaded:
    if {[catch ccseiCmdQueryLoadedCDTs cdtlist]} {
	set ccseiCmd_v(dead) 1
	set seq_ccsCmdCheck NO_CHECK
	append errStr \
		"\nCould not query the list of loaded CDTs.\
		Probably the $ccsei_n(cmdManager) is not working properly.\
		Try to restart the local environment.\n"
    } elseif {[llength $cdtlist] == 0} {
	append errStr \
		"\nThere are no CDTs loaded in the environment.\
		See $ccsei_n(cmdSetup)(1) how to load CDTs\
		or use the `CDT File Setup' in the `Windows' menu.\n"
    }

    # Prefix string if a problem was found:
    if {$errStr != ""} {
	set errStr "Found following problems:\n${errStr}"
    }

    return $errStr
}

#_oOo_
