#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiCommandBox.tcl,v 3.11 2005/09/14 14:11:17 vltsccm Exp $" 
#
# ccseiCommandBox.tcl - Class for a command-line oriented text-widget
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 03.01.1995 created from M.Comin's uifListBox.tcl
#			Note: Made only some patches. Not tested.
#			X-scrollbar not possible with text-widget, removed.
#			width and height are used instead of geometry.
#	     06.01.1995 revised
#	     18.01.1995 added history-buffer
# S.Sandrock 15.10.1995 revised, dependent on vcc
# S.Sandrock 11.04.1996 Tk4/incrTk porting
#

#************************************************************************
#   NAME
#
#	CcseiCommandBox - Class for a command-line oriented text-widget
# 
#   SYNOPSIS
#
#	CcseiCommandBox <widget-name> ?<options>?
# 
#   DESCRIPTION
#
#	CcseiCommandBox defines the class for the command input box in the
#	CCS Engineering Interface (ccsei). It is a command-line oriented
#	text-box with editing capabilities and history-buffer.
#	Command-lines consist of "fields" marked with tags, as described in
#	text(n). There may be any number of fields in a command line, but 
#	each tag must occur not more than once.
#
#	The user creates a widget of this class (called instance) by :
#
#		CcseiCommandBox <widget-name> <config-parameters>
#	where :
#
#	<widget-name>         is any valid TK widget name
#	<config-parameters>   sequence of widget resources
#
#	Standard widget resources can be set using :
#
#              <widget-name> configure -<resource-name> <value>
#
#	The current value of the widget resources is returned by :
#
#		itcl_public <name-of-instance> <resource-name>
#
#	The user must provide a procedure that is called when the command-line
#	is terminated with the Enter-key (see Option `hookEnter').
#
#	The intended tags should be configured with the Methods `configField'
#	resp. `bindField'.
#	During the command-line editing, fields can be altered as described
#	in the Methods `askField', `getField', `putField' and `putPrompt'.
#
#   INHERITANCE
#	FrameWidget, UifWidget, VccDebug
#
#   PUBLIC OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<widget-name> configure -<Option> <Value> ...
#
#	The following options are publicly available:
#
#	keyEnter	defines the key to be used for entering, the usual and
#			default value is <Return>.
#
#	hookEnter	this hook must be configured with a procedure-name.
#			It is called when the `keyEnter' key was pressed.
#			One argument is passed to this procedure, namely
#			the command-line in the special list format (Event):
#				{<tag1> <field1>} {<tag2> <field2>} ...
#			Each field contained in the command-line is passed as
#			one element of a list, where <tag> is the tag-name
#			and <field> is its contents.
#			The hook-procedure may either return an empty string
#			for no further operation, or a modified Event, which 
#			is then recorded in the history instead of the 
#			previously passed Event.
#
#	complOnEnter	enables automatic completion before the line is 
#			processed upon `keyEnter'. Default is 0 (off).
#
#	keyCompletion	defines the key to be used for completion, the usual 
#			and default value is <Tab>.
#
#	hookCompletion	this hook can be configured with a procedure-name.
#			It is called when the `keyCompletion' key was pressed.
#			One argument is passed to this procedure, namely
#			the field to be completed in the special format
#			described at `hookEnter', but unlike there it is 
#			always only a single field in the list.
#			The hook-procedure may either return an empty string
#			for no further operation, or a modified Event, which 
#			is then displayed instead of the previous one.
#
#
#	historySize	size of the history-buffer, default is 20
#
#	hookHistory	this hook can be configured with a procedure-name.
#			It is called whenever the `history' method is called.
#			One argument is passed to this procedure, namely
#			the current event on the history stack.
#			The hook-procedure may either return an empty string
#			for no further operation, or a modified Event, which 
#			is then recorded instead of the previous one.
#
#
#	width		x-size of the display
#	height		y-size of the display
#	font		name of the font in the display
#
#   METHODS
#	Methods represent procedures on the object and are invoked with:
#
#		<widget-name> <Method> <Arguments> ...
#
#	The following methods are publicly available:
#
#	configField <tag> <options> - as "tag configure ..." in text(n)
#	bindField <tag> <options>   - as "tag bind ..." in text(n)
#	beginField <tag>  - begin new field at insert-cursor, evt. add space
#	endField    - end the field at insert-cursor, evt. add space
#	getField <tag> - get the specified field from the current line
#	putField <tag> <chars> - insert/replace the field into current line
#	appendField <tag> <chars> - append the field into current line
#	putPrompt <chars> - insert/replace a Prompt into current line
#	askField - return the tag of the field at the insert-cursor
#
#	putChars <tags> <chars> - insert characters with given tag at end
#	setBind <options> - as "bind" for text(n)
#	setFocus - set input-focus to the text-widget
#	setFocusDefault - set default input-focus to the text-widget
#
#	history <Operation>
#	  where <Operation> is one of:
#	      record	record the current command-line in the history-buffer
#	      recall	recall and display command-line from the history-buffer
#	      up	recall previous command-line
#	      down	recall next command-line
#	      last	get last command-line, format described above
#	      nextid	get next number, e.g. to display in the prompt
#	  Note that the history-buffer is managed completely internally,
#	  there is no need for the user to invoke the `history' method.
#
#   TAGS
#	Reserved tag-name: "Prompt" for tagging the begin of a command-line.
#	Reserved tag-names should only be used for their intended purpose.
#
#   BINDINGS
#	The following keys have special functions:
#
#	<Return>		- terminate command-line, record event in the
#				  history-buffer, and call `hookEnter'
#	<Key-Left>, <Key-Right>	- move cursor, but not beyond prompt or end
#	<Key-Home>, <Key-End>	- move cursor to begin/end of command-line
#	<BackSpace>, <Delete>	- delete previous character
#	<Key-Up>, <Key-Down>	- traverse history-buffer
#	Mouse-Buttons are partly disabled
#
#   EXAMPLES
#
#   SEE ALSO
#	text(n), ccseiMsg(1)
#
#   BUGS
#	Cut and paste in the usual fashion is not supported yet.
#	Mouse interactions are not foreseen yet and may cause malfunctions.
#
#------------------------------------------------------------------------

class CcseiCommandBox {

    inherit UifText VccDebug

    constructor { args } {}
    destructor {}

    #--------------------------------------------------------------------
    #  Public Widget Options
    #--------------------------------------------------------------------
    itk_option define -keyCompletion  keyCompletion  KeyCompletion  <Tab>
    itk_option define -hookCompletion hookCompletion HookCompletion {}
    itk_option define -keyEnter       keyEnter       KeyEnter       <Return>
    itk_option define -hookEnter      hookEnter      HookEnter      {}
    itk_option define -complOnEnter   complOnEnter   ComplOnEnter   0
    itk_option define -historySize    historySize    HistorySize    20
    itk_option define -hookHistory    hookHistory    HookHistory    {}

    itk_option define -width          width          Width          96
    itk_option define -height         height         Height         20
    itk_option define -font font Font -Adobe-helvetica-bold-r-normal--12*

    #--------------------------------------------------------------------
    #  Public Methods
    #--------------------------------------------------------------------
    public method configField {tag args}
    public method bindField {tag args}
    public method beginField {tag}
    public method endField { }
    public method getField {tag}
    public method putField {tag field}
    public method appendField {tag field}
    public method putPrompt {prompt}
    public method askField { }
    public method putChars { Tags Chars }
    public method setBind {key action}
    public method setFocus { }
    public method history { Operation }

    public method doEnter {} ;# not really public, but secret!

    # methods that map 1-to-1 into the Tk text widget:
    public method delete  args	{eval $wText delete $args}
    public method yview   args	{eval $wText yview  $args}
    public method mark    args	{eval $wText mark   $args}
    public method tag     args	{eval $wText tag    $args}

    #--------------------------------------------------------------------
    #  Local Methods
    #--------------------------------------------------------------------
    protected method doCompletion {}

    #--------------------------------------------------------------------
    #  Local Variables 
    #--------------------------------------------------------------------
    protected variable wText		;# name of the embodied text widget
    protected variable historyCount   0 ;# number of the current (latest) event
    protected variable historyPointer 0 ;# points to recalled event number

}


#--------------------------------------------------------------------
#  constructor
#--------------------------------------------------------------------

body CcseiCommandBox::constructor { args } {
    debugEnter

    set wText $itk_component(uifText) ;# name of embodied text widget

    # integrate options:
    eval itk_initialize $args

    # reorder binding tags for text widget to get desired behaviour:
    bindtags $wText [list Text $wText [winfo toplevel $wText] all]

    # undefine bindings for own electric keys:
    foreach key {-keyCompletion -keyEnter} {
	foreach tag [bindtags $wText] {
	    bind $tag $itk_option($key) {}	;# remove any existing binding
	}
    }

    # history traversal:
    bind $wText <Key-Up>   "[code $this history up] ; break"
    bind $wText <Key-Down> "[code $this history down] ; break"

    # disable cursor movement, always fixed at the end:
    bind $wText <1> {%W mark set insert end ; break}
    bind $wText <2> {%W mark set insert end ; break}
    bind $wText <3> {%W mark set insert end ; break}

    # editing and cursor movement keys:
    bind $wText <Delete>    {}
    bind $wText <Key-Home>  {%W mark set insert "Prompt.last + 1 char"}
    bind $wText <Key-End>   {%W mark set insert end}
    #bind $wText <Key-Right> {%W mark set insert "insert + 1 char"}
    bind $wText <Key-Right> {
	#puts "%W: [%W index insert]: [%W tag names insert]" ;# DEBUG
    }
    bind $wText <Key-Left>  {
	if {[%W compare [%W index insert] < [%W index Prompt.last]]} {
	    %W mark set insert Prompt.last
	}
	#puts "%W: [%W index insert]: [%W tag names insert]" ;# DEBUG
    }
    bind $wText <BackSpace> {
	if {[%W compare [%W index insert] <= [%W index Prompt.last]]} {
	    # this assumes that there is always a space as last char in prompt:
	    if {[%W get "Prompt.last - 1c"] != " "} {
		%W insert Prompt.last " " [concat Prompt [%W tag names insert]]
	    }
	}
    }

    # define default tag:
    $wText tag configure Prompt
    $wText tag add Prompt 1.0 end

    debugReturn
}


#--------------------------------------------------------------------
# Debugging Options
#--------------------------------------------------------------------
# (inherited from VccDebug)


#--------------------------------------------------------------------
# Completion
#--------------------------------------------------------------------

# this defines the key to be used for completion:
configbody CcseiCommandBox::keyCompletion {
    debugPrint "keyCompletion = $itk_option(-keyCompletion)"
    bind $wText $itk_option(-keyCompletion) "[code $this doCompletion] ; break"
}
 
# this hook must be configured from outside with a procedure-name:
configbody CcseiCommandBox::hookCompletion {
    debugPrint "hookCompletion = $itk_option(-hookCompletion)"
}

# this is called upon Completion-key:
body CcseiCommandBox::doCompletion { } {
    debugEnter

    # delete previously inserted character (Tk4):
    $wText delete "insert - 1 char"

    # display the `coffee-break' cursor:
    $wText configure -cursor watch

    if {! [lempty $itk_option(-hookCompletion)]} {
	# ask which field shall be completed and its current contents:
	set what  [$this askField]
	set field [$this getField $what]
	lappend event [list $what $field]

	# call the hook and pass the field to be completed to it
	debugPrint "entering $itk_option(-hookCompletion) $event"
	set completedEvent [$itk_option(-hookCompletion) $event]
	debugPrint "returned $itk_option(-hookCompletion) ==> $completedEvent"

	# insert the returned completed field(s) (may be more that 1):
	foreach item $completedEvent {
	    $this putField [lindex $item 0] [lindex $item 1]
	}
    }

    # back to the normal cursor, make sure that cursor is visible:
    $wText configure -cursor xterm
    $wText yview -pickplace end

    debugReturn
}


#--------------------------------------------------------------------
# Command entering
#--------------------------------------------------------------------

# this defines the key to be used for entering:
configbody CcseiCommandBox::keyEnter {
    debugPrint "keyEnter = $itk_option(-keyEnter)"
    bind $wText $itk_option(-keyEnter) "[code $this doEnter] ; break"
}

# this hook must be configured from outside with a procedure-name:
configbody CcseiCommandBox::hookEnter {
    debugPrint "hookEnter = $itk_option(-hookEnter)"
}

# this enables automatic completion before the line is processed:
configbody CcseiCommandBox::complOnEnter {
    debugPrint "complOnEnter = $itk_option(-complOnEnter)"
}

# this is called upon Enter-key:
body CcseiCommandBox::doEnter { } {
    debugEnter

    # delete previously inserted character (Tk4):
    $wText delete "insert - 1 char"

    # complete line before processing it:

    if {$itk_option(-complOnEnter)} {doCompletion}

    $this history record		;# record this in the history-buffer
    $wText mark set insert end		;# move cursor to the very end
    $wText insert insert \n		;# do line break
    $wText configure -cursor watch	;# display wait-cursor

    if {! [lempty $itk_option(-hookEnter)]} {
	# call the hook and pass the last command-line to it
	debugPrint "entering $itk_option(-hookEnter)"
	set refinedEvent [$itk_option(-hookEnter) [$this history last]]
	debugPrint "returned $itk_option(-hookEnter): {$refinedEvent}"
	# if the recorded Event needs a modification, re-record it:
	if {$refinedEvent != ""} {
	    ::history change $refinedEvent
	    #...
	}
    }

    # back to the normal cursor, make sure that cursor is visible:
    $wText configure -cursor xterm
    $wText yview -pickplace end

    debugReturn
}


#--------------------------------------------------------------------
# History Buffer Management
#--------------------------------------------------------------------
# Note: `::history' refers to the global Tcl command

# size of the history-buffer, can be modified from outside:
configbody CcseiCommandBox::historySize	{
    ::history keep $itk_option(-historySize)
    $this debugPrint "historySize = $itk_option(-historySize)"
}

# this hook must be configured from outside with a procedure-name:
configbody CcseiCommandBox::hookHistory {
    debugPrint "hookHistory = $itk_option(-hookHistory)"
}

# internal variables:
##protected historyCount	0	;# number of the current (latest) event
##protected historyPointer	0	;# points to recalled event number

# perform action on history-buffer:
body CcseiCommandBox::history { Operation } {
    debugEnter

    switch $Operation {

	record {
	    set startPos Prompt.last   ;# start searching after last prompt
	    set allTags [$wText tag names]	;# get all tags in the text
	    # determine all tags contained in the command-line:
	    set event {}
	    foreach tag $allTags {
		set err [catch {$wText compare $tag.last > $startPos} x]
		if {! $err && $x} {
		    # get the found tags+fields and save them in a list:
		    set field [$this getField $tag]
		    lappend event [list $tag $field]
		    lappend foundTags $tag
		}
	    }
	    # record the event, i.e. fields and tags:
	    ::history add $event
	    set historyPointer [::history nextid]
	    set historyCount [expr {$historyPointer-1}]
	    debugPrint "recorded new event: {$event}"
	    ##debugPrint "recorded events:\n[::history info]"
	    set return $historyPointer
	}

	recall {
	    set event ""
	    if {$historyCount == 0} {
		set return ""
		break
	    }
	    # don't allow the history-pointer to go out of limits:
	    set min [expr {$historyCount-$itk_option(-historySize)+1}]
	    if {$min < 1} {set min 1}
	    set max $historyCount
	    if {$historyPointer < $min} {
		set historyPointer $min
	    }
	    # to have an empty line as last "pseudo-event", remove this:
	    if {$historyPointer > $max} {
		set historyPointer $max
	    }
	    # delete the last line from the prompt on:
	    $wText delete "Prompt.last - 1 char" "end + 1 char"
	    #@Tk3: $wText delete Prompt.last end
	    $wText mark set insert end
	    if {$historyPointer > $max} {
		set historyPointer [expr {$max+1}]
	    } else {
		# query from the history-buffer and insert into line:
		set event [::history event $historyPointer]
		foreach item $event {
		    set tag   [lindex $item 0]
		    set field [lindex $item 1]
		    $this putField $tag $field
		}
		# make sure that cursor is visible:
		$wText yview -pickplace end
		debugPrint "recalled event #$historyPointer: {$event}"
	    }

	    if {! [lempty $itk_option(-hookHistory)]} {
		# call the hook and pass the last command-line to it
		debugPrint "entering $itk_option(-hookHistory)"
		set refinedEvent [$itk_option(-hookHistory) $event]
		debugPrint "returned $itk_option(-hookHistory): $refinedEvent"

		# if the recorded Event needs a modification, re-record it:
		if {$refinedEvent != ""} {::history change $refinedEvent}
	    }


	    set return ""
	}

	up {
	    incr historyPointer -1
	    $this history recall
	    set return ""
	}

	down {
	    incr historyPointer 1
	    $this history recall
	    set return ""
	}

	last {
	    set n $historyCount
	    set event [::history event $n]
	    set return $event
	}

	nextid {
	    set return [::history nextid]
	}

    } ;# switch

   debugReturn $return
} ;# history


#--------------------------------------------------------------------
# Command-line Field Management
#--------------------------------------------------------------------

#SSa: added xxxField - methods on tagged chars in the current line
#--------------------------------------------------------------------
#*# method: configField - as "tag configure ..."
#*# method: bindField   - as "tag bind ..."
#*# method: beginField  - begin new field at insert-cursor, evt. add space
#*# method: endField    - end the field at insert-cursor, evt. add space
#*# method: getField - get the specified field from the current line
#*# method: putField - insert/replace the specified field into current line
#*# method: appendField - append the specified field into current line
#*# method: putPrompt - insert/replace a Prompt into current line
#*# method: askField - return the tag of the field at the insert-cursor

body CcseiCommandBox::configField {tag args} {
    debugEnter
    if {$debugFlag} {append args " -underline 1"}
    debugReturn [eval $wText tag configure $tag $args]
}

body CcseiCommandBox::bindField {tag args} {
    debugEnter
    debugReturn [eval $wText tag bind $tag $args]
}

body CcseiCommandBox::beginField {tag} {
    debugEnter

    set pos "insert - 1 char"

    # insert a space before if not already there:
    if {[$wText get $pos] != " "} {
	$wText insert insert " " $tag
    }

    # delete remaining chars and add a sticky tag:
    $wText delete insert end
    $wText tag add $tag $pos "end + 1 char"

    debugReturn ""
}

body CcseiCommandBox::endField { } {
    debugEnter

    set pos "insert - 1 char"
    # insert a space before if not already there:
    if {[$wText get $pos] != " "} {
	$wText insert insert " "
    }
    # remove all existing tags and add the new tag for the inserted text:
    foreach t [$wText tag names $pos] {
	#@Tk3: $wText tag remove $t $pos
	$wText tag remove $t $pos "end + 1 char"
    }

    debugReturn ""
}

body CcseiCommandBox::getField {tag} {
    debugEnter

    set range [$wText tag nextrange $tag {insert linestart} {insert lineend}]
    debugPrint "range = {$range}"

    set field ""
    if {![lempty $range]} {
	set index1 [lindex $range 0]
	set index2 [lindex $range 1]
	set field [$wText get $index1 $index2]
	set field [string trim $field] ;# remove leading+trailing white space
    }
    debugReturn $field
}

body CcseiCommandBox::putField {tag field} {
    debugEnter

    # position to insert, by default the insertion cursor:
    set pos [$wText index insert]

    # first check if the field is already contained in the line:
    set range [$wText tag nextrange $tag {insert linestart} {insert lineend}]

    # if so, delete it, and set the position to the beginning:
    if {$range != ""} {
	set index1 [lindex $range 0]
	set index2 [lindex $range 1]
	$wText delete $index1 $index2
	set pos $index1
	debugPrint "deleted existing field: range = {$range}"
    }

    # check if there is a space before, add one if not, preserving the tags:
    set tag1 $tag
    if {[$wText get "$pos - 1 char"] != " "} {
	set field " $field"
	set tag1 [$wText tag names "$pos - 1 char"]
    }

    set index1 $pos
    set index2 "$pos + [expr {[string length $field]+1}] chars"

    # insert the new text:
    $wText insert $pos $field

    # remove all existing tags and add the new tag for the inserted text:
    foreach t [$wText tag names $pos] {$wText tag remove $t $index1 $index2}
    $wText tag add $tag $index1 $index2
    debugPrint "inserted field: range = $index1 $index2"

    # restore the saved separator tags:
    foreach t $tag1 {$wText tag add $t $pos}

    debugReturn ""
}

body CcseiCommandBox::appendField {tag field} {
    debugEnter
    debugReturn [putField $tag "[getField $tag]$field"]
}

body CcseiCommandBox::putPrompt {prompt} {
    debugEnter

    # first check if there is already a prompt in the line:
    set range [$wText tag nextrange Prompt {insert linestart} {insert lineend}]

    # if so, delete it, and determine the tag after it:
    set tag ""
    if {$range != ""} {
	set tag [$wText tag names Prompt.last]
	debugPrint "next tag: $tag"

	eval $wText delete $range
	debugPrint "deleted existing field: range = $range"
    }

    # insert prompt text:
    $wText insert {insert linestart} $prompt Prompt

    # put a space after prompt if not already there:
    lvarcat tag Prompt
    if {[$wText get Prompt.last] != " "} {
	$wText insert Prompt.last " " $tag
	debugPrint "restore separator with tag: $tag"
    } else {
	# restore the saved separator tags:
	foreach t $tag {$wText tag add $t Prompt.last}
	debugPrint "restore next tag: $tag"
    }

    debugReturn ""
}

body CcseiCommandBox::askField { } {
    debugEnter
    # ignore `Prompt' here:
    lassign [intersect3 [$wText tag names "insert - 1 char"] Prompt] tag
    debugReturn $tag
}



#--------------------------------------------------------------------
# General Methods
#--------------------------------------------------------------------

# insert characters with given tag at end:
body CcseiCommandBox::putChars { Tags Chars } {
    debugEnter
    debugReturn [$wText insert end $Chars $Tags]
}

# set binding to text widget:
body CcseiCommandBox::setBind {key action} {
    debugEnter

    # undefine bindings in hierarchy:
    foreach tag [bindtags $wText] {
	bind $tag $key {}	;# remove any existing binding
    }

    debugReturn [bind $wText $key $action]
}


# set input-focus to the text-widget:
body CcseiCommandBox::setFocus { } {
    debugEnter
    debugReturn [focus $wText]
}


#--------------------------------------------------------------------
# usual widget option handling
#--------------------------------------------------------------------

usual CcseiCommandBox {
    keep -background -cursor -foreground -font
}

#___oOo___
