#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiListLib.tcl,v 3.11 2005/09/14 14:11:15 vltsccm Exp $" 
#
# ccseiListLib.tcl - Library for internal list-management of ccsei
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 13.01.1995 Created
#	     22.01.1995 Added support for Synonyms
#	     25.01.1995 Added support for Parameters
#	     08.03.1995 Defined but not-existing xxxROOT etc. is now ignored
# S.Sandrock 15.10.1995 major revision for vcc module
#

#************************************************************************
#   NAME
#	ccseiListLib,
#	ccseiGetListOfEnvironments, 
#	ccseiGetListOfProcesses,
#	ccseiGetListOfCommands, 
#	ccseiGetListOfParameters,
#	ccseiGetHelpText,
#	ccseiGetListOfLoadedCDTs, 
#	ccseiGetListOfProcessesInSimulation,
#	ccseiTranslateSynonym,
#	ccseiOptionOnAllLists
#	- Tcl interface to VLT environment data and list management of ccsei
# 
#   SYNOPSIS
#	ccseiGetListOfEnvironments	Session
#	ccseiGetListOfProcesses		Environment
#	ccseiGetListOfCommands		Process
#	ccseiGetListOfParameters	Process Command
#	ccseiGetHelpText		Process Command
#	ccseiGetListOfLoadedCDTs
#	ccseiGetListOfProcessesInSimulation
#	ccseiTranslateSynonym		Process Synonym
#	ccseiOptionOnAllLists		Option
#
#   DESCRIPTION
#	Tcl procedures to retrieve the requested items as a list:
#
#	ccseiGetListOfEnvironments - query environment list from vcc
#
#	ccseiGetListOfProcesses - query process list for Environment from vcc
#
#	ccseiGetListOfCommands - queries the available commands for a
#		specified Process from the cmdManager.
#		The corresponding CDT for the Process must have been loaded.
#		Also queries the available Synonyms for the Process.
#		The global variable ccsei_v(synMode) defines whether the
#		returned list consists of Commands, Synonyms, or both.
#		See GLOBAL VARIABLES.
#
#	ccseiGetListOfParameters - queries the formal Parameter names for a
#		specified Process and Command from the cmdManager.
#		The corresponding CDT for the Process must have been loaded.
#
#	ccseiGetHelpText - queries the help-text for a
#		specified Process and Command.
#		The corresponding CDT for the Process must have been loaded.
#
#	ccseiGetListOfLoadedCDTs - queries the loaded CDTs from the cmdManager.
#
#	ccseiGetListOfProcessesInSimulation - queries the list of Processes
#		which are currenly in Simulation-Mode from the cmdManager.
#
#	ccseiTranslateSynonym - translate a Synonynm into corresponding Command
#		Returns its argument untranslated if Synonyms are not enabled,
#		if the argument is already a valid Command, or if it is not a
#		valid Synonym.
#
#	ccseiOptionOnAllLists - performs a specified option on all lists:
#		clear:	clears all the internal lists, so that they are
#			actually queried next time they are requested.
#		info:	return an information string about the lists.
#
#   PARAMETERS
#	Depending on specific procedure:
#
#	Session - any identifier string
#	Environment - name of the environment, e.g. "lte21", "wte16"
#	Process - name of the process, e.g. "lccServer", "cmdManager"
#	Command - name of the command, e.g. "LCCGVER", may also be a Synonym
#	Option - name of an option (see specific proc description)
#	Synonym - name of a synonym of a command, may also be a Command
#
#   RETURNS
#	List of requested items.
#
#   ENVIRONMENT
#	VLTROOT - path of VLT root (mandatory)
#	INTROOT - path of integration root (optional)
#	MODROOT - path of module development root (optional)
#
#	The respective path must be existent and readable for the user, 
#	otherwise it is ignored. Changes of these varaibles after start-up
#	are ignored.
#
#   FILES
#	VccInfo(n) may indirectly access configuration files.
#
#   GLOBAL VARIABLES
#	Public:
#		ccsei_v(synMode) - mode for usage of Commands and/or Synonyms:
#			Com	- use only Commands
#			Syn	- use only Synonyms
#			ComSyn	- use both, first Commands, then Synonyms
#			SynCom	- use both, first Synonyms, then Commands
#
#		ccsei_v(synFlag) - enables translation of Synonyms:
#			0 - don't translate, use literally
#			1 - translate into valid Command, if possible
#
#		env(...) - environment variables
#
#	Private: 
#		ccseiListX(...)
#		  where X is the abbreviation constant for Env/Pro/Com/Par,
#		  plus some more.
#
#   CAUTIONS
#	All lists/information are "cached" once they have been retrieved.
#	Use `ccseiOptionOnAllLists' to flush the cache.
#
#   EXAMPLES
#
#   SEE ALSO
#	ccsei(1), ccseiCmdLib(n), 
#	VccInfo(n), VccCompletion(n),
#	CCS User Manual VLT-MAN-ESO-17210-0619
#
#------------------------------------------------------------------------


###############################################################################
#
#	G L O B A L   V A R I A B L E S
#
#	Declaration and initialization of global variables
#
###############################################################################
# Rules: see ccsei.tcl
#
# Abbreviations (as part of names):
#	Ses	Session
#	Env	Environment
#	Pro	Process
#	Com	Command
#	Par	Parameter
#	Val	Value (of enumeration parameter)
#	Syn	Synonym (for Command)
#	Tra	Translation (of Synonym into Command)
#
###############################################################################

# Directory-names under VLTROOT resp. INTROOT resp. MODROOT:
# Set names if not already defined:

if {! [info exists ccsei_n(CDT)]} {
    set ccsei_n(CDT) CDT			;# dir-name with CDTs
}
if {! [info exists ccsei_n(CIT)]} {
    set ccsei_n(CIT) CIT			;# dir-name with CITs
}


# Global lists to be accessible from all procedures:
# The lists are built when needed the first time. Once built they are re-used
# each time they are accessed.
global ccseiListEnv()	;# list of environments(Ses)
			;# (whatever `session' should stand for).
global ccseiListPro()	;# list of processes(Env),
			;# e.g. ...Pro(lte21)
global ccseiListCom()	;# list of commands(Pro),
			;# e.g. ...Com(lccServer)
			;# Note: a process has always the same commands
			;#       even on different environments.
global ccseiListPar()	;# list of parameters(Pro,Com),
			;# e.g. ...Par(lccServer,LCCGVER)
			;# Note: 2-dim arrays are not supported by Tcl.
			;#       The ',' is part of the index-string!
global ccseiListHelp()	;# list of help-texts(Pro,Com)
global ccseiListCDT()	;# list of loaded CDTs in the cmdManager (no index)
global ccseiListSimul()	;# list of processes in simulation

# synonym-support:
global ccseiListSyn()	;# list of synonyms(Pro),
			;# e.g. ...Syn(lccServer)
			;# Note: a process has always the same synonyms
			;#       even on different environments.
global ccseiListTra()	;# Translation(Pro,Syn) of a synonym, i.e. a Command,
			;# not actually a list, but a single word,
			;# e.g. ...Tra(lccServer,lccGetVersion) is LCCGVER
			;# Note: 2-dim arrays are not supported by Tcl.
			;#       The ',' is part of the index-string!

# parameter-support:
global ccseiListParType() ;# type-information(Pro,Com,Par),
			;# e.g. ...ParType(lccServer,LCCCMD,par1)
global ccseiListVal()	;# parameter enum values information(Pro,Com),
			;# e.g. ...Val(lccServer,LCCCMD)
global ccseiListValTree() ;# parameter name and enum values (Pro,Com),
			;# e.g. ...ValTree(lccServer,LCCCMD)

# this list shall contain all array-names of lists stated here:
set ccseiListOfLists {\
	ccseiListEnv ccseiListPro ccseiListCom ccseiListPar \
	ccseiListHelp ccseiListCDT ccseiListSimul \
	ccseiListSyn ccseiListTra \
	ccseiListParType ccseiListVal ccseiListValTree }


# define the mode for command/synonym usage, if not already done:
if {! [info exists ccsei_v(synMode)]} {
    set ccsei_v(synMode)	Com	;# use only commands, no synonyms
}
if {! [info exists ccsei_v(synFlag)]} {
    set ccsei_v(synFlag)	1	;# translate Synonym into Commands
}


# define completion objects:
# --------------------------
vccSetDef ccsei_c(complModeEnv) {enabled ignorecase leading trailing}
vccSetDef ccsei_c(complModePro) {enabled ignorecase leading trailing}
vccSetDef ccsei_c(complModeCom) {enabled ignorecase leading trailing}
vccSetDef ccsei_c(complModePar) {enabled ignorecase trailing}
set ccsei_w(complEnv) \
	[VccCompletion wComplEnv -completionMode $ccsei_c(complModeEnv)]
set ccsei_w(complPro) \
	[VccCompletion wComplPro -completionMode $ccsei_c(complModePro)]
set ccsei_w(complCom) \
	[VccCompletion wComplCom -completionMode $ccsei_c(complModeCom)]
set ccsei_w(complPar) \
	[VccCompletion wComplPar -completionMode $ccsei_c(complModePar)]


###############################################################################
#
#	P R O C E D U R E S
#
###############################################################################
# Rules: see ccsei.tcl
###############################################################################


proc ccseiGetListOfEnvironments { Session } {
    global ccseiListEnv ccsei_e ccsei_n ccsei_w

    # Retrieve the data only if not already done:
    if { [info exists ccseiListEnv($Session)] } {
	set envlist $ccseiListEnv($Session)
    } else {
	set envlist [vccInfo GetEnvs]
	set ccseiListEnv($Session) $envlist
    }

    # add the pseudo-Environment "SIM" for Simulation, plus {} for separator:
    lappend envlist {} SIM

    # register the list in the completion package:
    $ccsei_w(complEnv) Completion new $envlist
    return $envlist
}

proc ccseiGetListOfProcesses { Environment } {
    global ccseiListPro ccsei_e ccsei_n ccsei_w

    # Retrieve the data only if not already done:
    # special handling of pseudo-Environment SIM:
    if {$Environment == "SIM"} {
	set prolist [ccseiGetListOfProcessesInSimulation]
    } elseif { [info exists ccseiListPro($Environment)] } {
	set prolist $ccseiListPro($Environment)
    } else {
	set prolist [vccInfo GetProcsByEnv $Environment]
	set ccseiListPro($Environment) $prolist
    }

    # register the list in the completion package:
    $ccsei_w(complPro) Completion new $prolist
    return $prolist
}

proc ccseiGetListOfCommands { Process } {
    global ccseiListCom ccseiListSyn ccsei_v ccsei_w

    # Check for Commands, Retrieve the data only if not already done:
    if { [info exists ccseiListCom($Process)] } {
	set comlist $ccseiListCom($Process)
    } else {
	# check if a CDT is loaded, do it if not:
	if {[lsearch -exact [ccseiCmdQueryLoadedCDTs] $Process] == -1} {
	    ccseiCmdLoadCDT $Process	;# load selected CDT if not there
	    ccseiPuts "CDT of $Process implicitly loaded"
	}
	# send a command to the cmdManager to query the list of commands:
	set comlist {}
	set reply [ccseiCmdQueryCommands $Process]
	if {$reply != -1 && $reply != {}} {
	    # save the list if no error occured:
	    set comlist $reply
	    set ccseiListCom($Process) $comlist
	}
    }

    # Check for Synonyms, Retrieve the data only if not already done:
    if { [info exists ccseiListSyn($Process)] } {
	set synlist $ccseiListSyn($Process)
    } else {
	# send a command to the cmdManager to query the list of synonyms:
	set synlist {}
	set reply [ccseiCmdQuerySynonyms $Process]
	if {$reply != -1 && $reply != {}} {
	    # save the list if no error occured:
	    set synlist $reply
	    set ccseiListSyn($Process) $synlist
	}
	# check if there are at least as many synonyms as commands:
	if {$ccsei_v(synMode) == "Syn" && \
		[llength $comlist] > [llength $synlist]} {
	    ccseiDialog \
		    "Note: There are more Commands than Synonyms\
		    defined for this Process! Hence you will not be able\
		    to access all valid Commands only through Synonyms."
	}
    }

    # assemble the returned list from commands and/or synonyms:
    switch $ccsei_v(synMode) {
	Com	{set retlist $comlist}
	Syn	{set retlist $synlist}
	ComSyn	{set retlist [concat $comlist $synlist]}
	SynCom	{set retlist [concat $synlist $comlist]}
	default	{set retlist $comlist}
    }

    # register the list for completion and return it:
    $ccsei_w(complCom) Completion new $retlist
    return $retlist
}

proc ccseiGetListOfParameters { Process Command } {
    global ccseiListPar ccsei_w

    # a synonym may ge given, so first translate the argument into a command:
    set Command [ccseiTranslateSynonym $Process $Command]

    # Retrieve the data only if not already done:
    if { [info exists ccseiListPar($Process,$Command)] } {
	set parlist $ccseiListPar($Process,$Command)
    } else {
	# send a command to the cmdManager to query the list of parameters:
	set parlist {}
	set reply [ccseiCmdQueryParameters $Process $Command]
	if {$reply != -1} {
	    # save the list if no error occured:
	    set parlist $reply
	    set ccseiListPar($Process,$Command) $parlist
	}
    }

    # register the list in the completion package:
    # (modify for named parameter format with prefix `-')
    set complList {}
    foreach i $parlist {lappend complList -$i}
    $ccsei_w(complPar) Completion new $complList
    return $parlist
}

proc ccseiGetHelpText { Process Command } {
    global ccseiListHelp

    # a synonym may ge given, so first translate the argument into a command:
    set Command [ccseiTranslateSynonym $Process $Command]

    # Retrieve the data only if not already done:
    if {[info exists ccseiListHelp($Process,$Command)]} {
	set helptext $ccseiListHelp($Process,$Command)
    } else {
	set helptext [ccseiCmdQueryHelpText $Process $Command]
	if {$helptext != ""} {
	    # save it for later re-use:
	    set ccseiListHelp($Process,$Command) $helptext
	}
    }
    return $helptext
}


proc ccseiGetListOfLoadedCDTs { } {
    global ccseiListCDT
    # the number of loaded CDTs may change, so it must be queried each time
    set cdtlist [ccseiCmdQueryLoadedCDTs]
    set ccseiListCDT() $cdtlist
    return $cdtlist
}

proc ccseiGetListOfProcessesInSimulation { } {
    global ccseiListSimul
    # may change, so it must be queried each time
    set simlist [ccseiCmdQueryProcessesInSimulation]
    set ccseiListSimul() $simlist
    # .. do not register in the completion package ...
    return $simlist
}


proc ccseiTranslateSynonym { Process Synonym } {
    global ccseiListTra ccseiListCom ccseiListSyn ccsei_v

    # if we don't translate Synonyms at all, return immediately; that's faster:
    if {! $ccsei_v(synFlag)} {return $Synonym}
 
    # Retrieve the data only if not already done:
    if {[info exists ccseiListTra($Process,$Synonym)]} {
	set command $ccseiListTra($Process,$Synonym)
    } elseif {[info exists ccseiListCom($Process)] && \
	    [lsearch -exact $ccseiListCom($Process) $Synonym] >= 0} {
	# the Synonym is actually a valid Command, so save and return it:
	set command $Synonym
	set ccseiListTra($Process,$Synonym) $Synonym
    } elseif {[info exists ccseiListSyn($Process)] && \
	    [lsearch -exact $ccseiListSyn($Process) $Synonym] >= 0} {
	# the Synonym is valid, so query a translation for it:
	set command [ccseiCmdQueryTranslateSynonym $Process $Synonym]
	if {$command != ""} {
	    # save it for later re-use:
	    set ccseiListTra($Process,$Synonym) $command
	}
    } else {
	# the given Synonym is apparently not valid, so we can't translate it:
	set command $Synonym
    }

    return $command
}



proc ccseiOptionOnAllLists { Option } {
    global ccseiListOfLists

    set value ""
    # loop on whole array-variables:
    foreach arrayvar $ccseiListOfLists {
	global $arrayvar
	if {! [info exists $arrayvar]} continue
	set elems [array names $arrayvar]
	switch $Option {
	    clear {
		foreach name $elems {
		    unset ${arrayvar}($name)
		    ccseiPuts "ccseiClearAllLists: removed ${arrayvar}($name)"
		}
	    }
	    info {
		set s "$arrayvar: [array size $arrayvar] entries"
		set value "$value\n$s"
	    }
	}
    }
    return $value
}



#--------------------------------------
# Special Parameter support functions
#--------------------------------------

#************************************************************************
#   NAME
#	ccseiGetTreeOfParamValues
# 
#   SYNOPSIS
#	ccseiGetTreeOfParamValues Process Command
#
#   DESCRIPTION
#
#   PARAMETERS
#
#   RETURNS
#	The values for parameters of enumeration-type,
#	as a list-representation of a cascaded menu, suitable as argument
#	for `MakeMenu' method of `VccSelector'.
#
#   EXAMPLE
#	If the list of parameters is {pointName mode recursiveFlag fileName}
#	and the parameters `mode' and `recursiveFlag' have enumeration-type
#	while the others have not, then the resulting list could be:
#
#	{pointName {mode File Buffer} {recursiveFlag Branch Point} fileName}
#
#   ENVIRONMENT
#
#   GLOBAL VARIABLES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#	VccSelector(n)
#
#------------------------------------------------------------------------

proc ccseiGetTreeOfParamValues { Process Command } {
    global ccseiListParType ccseiListVal ccseiListValTree ccsei_w

    # a synonym may ge given, so first translate the argument into a command:
    set Command [ccseiTranslateSynonym $Process $Command]

    # Retrieve the data only if not already done:
    if { [info exists ccseiListValTree($Process,$Command)] } {
	set vallist $ccseiListVal($Process,$Command)
	set valtree $ccseiListValTree($Process,$Command)
    } else {
	set vallist {}
	set valtree {}

	set parlist [ccseiGetListOfParameters $Process $Command]

	foreach par $parlist {
	    # Retrieve the data only if not already done:
	    if { [info exists ccseiListParType($Process,$Command,$par)] } {
		set typlist $ccseiListParType($Process,$Command,$par)
	    } else {
		# send a command to the cmdManager to query the list:
		set typlist {}
		set reply [ccseiCmdQueryParamType $Process $Command $par]
		if {$reply != -1} {
		    # save the list if no error occured:
		    set typlist $reply
		    set ccseiListParType($Process,$Command,$par) $typlist
		}
	    }

	    # extract values from the type-list, if existing:
	    set values {}
	    if {[llength $typlist] >= 8} {
		if {[lindex $typlist 6] == "ENUM"} {
		    set values [lindex $typlist 7]
		}
	    }

	    set vallist [concat $vallist $values]
	    if {[llength $values] > 0} {
		# assemble the tree:
		# the {} indicates  a sub-menu, trailing space as separator:
		set valtree1 [list {} "-$par "]
		lappend valtree [concat $valtree1 $values]
	    } else {
		lappend valtree -$par
	    }
	}
	set ccseiListVal($Process,$Command)	$vallist
	set ccseiListValTree($Process,$Command)	$valtree
    }

    # register the value list in the completion package:
    $ccsei_w(complPar) Completion add $vallist

    return $valtree
}


#_oOo_
