#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ccseiMsg.tcl,v 3.11 2005/09/14 14:11:16 vltsccm Exp $" 
#
# ccseiMsg.tcl   -   Message-system part of the CCS Engineering User Interface
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 03.01.1995 Created
# S.Sandrock 28.06.1995 Added asynchronous display of replies
# S.Sandrock 15.10.1995 major revision for vcc module
# S.Sandrock 04.11.1995 added RmonMonitorcommand-script support
# S.Sandrock 12.04.1996 Tk4 porting
# mcomin     04/10/96   Add possibility to get error stack from message
#                       error replies
#            21/10/98   Re-added external window where to see the reply.
#

#************************************************************************
#   NAME
#	ccseiMsg - Message-Command-Window of the CCS Engineering User Interface
#   SYNOPSIS
#	ccseiMsg &
#   DESCRIPTION
#     This is the engineering GUI for the CCS message-system.
#
#     Its allows to send commands to processes running on either workstation
#     or LCU environments, and to view the resulting replies.
#     It also supports the setup of Command-Definition-Tables (CDTs) in the
#     local environment, and the Simulation-Mode facility of CCS.
#
#     The main window consists of following parts :
#
#     Default Selection
#
#       All what is specified here (not empty) need not be specified on
#       the command-line in the Command-Box (see below).
#       Usually the Environment and Process will be selected here, while
#       Commands and Parameters are specified on the command-line.
#
#       All entries will accept arbitrary values, the nearby located menus
#       will show the valid values for the corresponding entry.
#       All menu are dynamically created and depend on the selection
#       in their predecessor.
#
#       Environment: the menu contains all valid Environments on LCUs and
#       workstations, plus the pseudo-entry "SIM" which puts all processes
#       registered for simulation into the Process-menu.
#
#       Process: the menu contains all valid Processes in the selected
#                Environment that accept commands. Processes marked with "SIM"
#                are registered for simulation. If the menu entry is displayed
#	         in orange color, then simulation is active for this process.
#	         See section SIMULATION MODE for more.
#
#       Command: the menu contains all valid Commands of the selected process.
#	         The corresponding CDT must have been loaded in order to
#                retrieve this information, otherwise the menu is empty.
#
#       Parameter: the menu contains all valid Parameters of the selected
#                Command in "named parameter format", including possible
#                parameter values of enumeration type.
#
#     Command-Box
#
#       Here you enter the command-lines to send command-messages.
#	When your input is expected, the following prompt appears:
#
#       7 ...(Com)> 
#       ^  ^   ^
#       |  |   |
#       |  |   +-this field is expected next to be input (Env/Pro/Com/Par)
#       |  +-----this repeats the current default selection
#       +--------the history-number, repeated for replies in the Reply-Box
#
#       Each command line is separated into input-FIELDS, i.e. Environment 
#       (Env), Process (Pro), Command (Com), Parameter (Par).
#       All fields except for Par must be present, unless they are already
#       specified in the Default Selection. At any time you can override
#       the default selection on the command-line. Use Ctrl-Tab for this.
#
#       You should use `Tab' to begin the next input-field, since the field
#       resolving will be done immediately then. However, it is always tired
#       to resolve everything correctly before the line is processed.
#
#       Most input-fields have a completion-function with the Tab-key.
#       The completion is performed on both ends of the word, therefore
#       it is usually not possible to perform partial completion of words.
#       If there is no valid completion, you can press Tab twice to force
#       to proceed with next field. For incomplete Commands all matching 
#       completions are shown in the help-window "On Commands...", if it is
#       present on the screen. For completed Commands the help-text is
#       automatically displayed there.
#
#       All command-lines can be recalled with the cursor-keys up/down.
#
#       The following keys have special functions in the Command-Box:
#       Enter:     terminates the command-line to process it.
#       Tab:       performs completion and begins the next input-field.
#       Ctrl-Tab:  lets you begin with the first input-field.
#       Up/Down:   goes through the history-buffer.
#
#       Keys can also be re-configured (but only by modifying the code).
#
#
#     Replies-Box
#
#       Each time a command is sent, the resulting reply or replies are
#       displayed here with the following format :
#
#       REPLY 2/1: reply timed out
#       REPLY 2/L: @(#) LCC  1.3.1  NOVEMBER 1994
#       ERROR 3/1:32: Definition...
#         ^   ^ ^  ^      ^
#         |   | |  |      |
#         |   | |  |      +---- body of (error-)reply
#         |   | |  +----------- error-number (only for error-replies)
#         |   | +-------------- the reply-sequence-number, indicates
#         |   |			the n-th reply for this command,
#	  |   |			or `L' for "Last", `T' for "Timeout"
#         |   +---------------- the command-sequence-number,this number
#         |			corresponds to the command-box prompt
#         +-------------------- indicates normal or error reply
#
#       You can double-click on a specific item to get a more readable 
#       display for the entry.
#
#    Error Stack
#
#       Displays the whole error stack when the command returns an error 
#       reply. The whole error stack of an old error reply can also be
#       displayed by Cicking on an error reply in the 'Replies-Box' 
#
#
#    List Of Pending Commands
#
#       Lists all the command not yet completed, that is, for which we have
#       yet received the 'Last' reply.
#
#   MENUS
#
#   File:

#     Execute Command Script:  lets you select a file that is executed.
#     Open LOg File :
#     Close Log File :
#     Quit:                    quits the application and all its sub-windows.
#
#   Options:
#
#      Refresh Panel: sets main and sub panels to a consistent state.
#                     Use this whenever in doubt about the cmdManager status.
#
#      Clear Cache:   clears all internal lists that are used to cache
#                     informations previously queried from the outside world.
#                     It does not clear the command-line history buffer.
#
#      Clear History: clear the lines in the Command-Box and the Reply-Box
#                     (but not the command-line history buffer).
#
#      Simulation Mode: this option has a major impact on the behaviour
#                      of this tool. See section SIMULATION MODE.
#
#      Translate Synonyms: specifies whether entered Synonyms shall be
#		      translated into valid Commands before they are passed 
#                     to the message-system.
#
#      Synonym Mode:  specified whether and how Synonyms are displayed
#		      for help, and used for the completion mechanism. 
#                     However, you can enter Commands or Synonyms at any time.
#
#                     SCommands only:  use Commands, hide Synonyms
#                     SSynonyms only:  use Synonyms, hide Commands
#                     Commands first:  use both, display Commands first
#                     Synonyms first:  use both, display Synonyms first
#
#      Error Checking: requests preventive checking from message-system.
#		       Possible choices are (see msgSendCommand(3)):
#
#           NO_CHECK:   don't check anything, just send and see what happens
#           CHECK_CMD:  check validity of command-messages
#           CHECK_ALL:  check everything
#
#      Timeout on Recv: specifies the maximum time to wait for a reply.
#
#      Continuous Update: With this option, during receiving of multiple 
#		replies, the Reply-Box will be updated on each incoming reply,
#		otherwise only when there is idle time available. Note that
#		the tool will be noticeably slower when this option is active!
#
#      Enable Completion:   enables the completion mechanism.
#
#      Enable Debug:        enables debugging outputs to standard-output.
#
#
#   Windows:
#
#      Simulation Setup:  supports the setup of the simulation-mode.
#      CDT File Setup:    supports the setup of CDT files.
#
#   Help:
#
#      Info:           displays the version of the program.
#
#      On Commands:     pops-up the help-window for Commands on the screen.
#
#      On Status :     displays the status of the environment.
#      Extended Help:  invokes the VLT man-page browser and the program's 
#                      own man-page should appear.
#
#   SIMULATION MODE
#     Simulation Mode provides the possibilty to catch commands directed
#     to a specific Process and respond with a simulated reply rather than
#     actually sending the command to the real Process.
#
#     The corresponding entry in the Options menu switches between NORMAL and
#     SIMULATION mode, which is signaled by the environment variable CCSSTATE.
#
#     The sub-window "Simulation Setup" accessible in the Windows menu
#     provides support to register Processes for simulation and to define 
#     values of reply parameters (if they have database references) for 
#     specific Commands, from which a default reply is assembled.
#
#     All Processes which are registered for simulation are indicated by
#     "SIM" in the process-menus. Furthermore their color is changed to
#     SCREAMING ORANGE when Simulation Mode is selected to indicate that
#     all communication with this Process will now be simulated.
#     Note that Processes registered for simulation are NOT simulated when
#     Simulation Mode is not active.
#
#     For more see the description in the CCS/cmd module.
#
#   CDT FILES
#     All information about a Process is taken from its Command Definition
#     Table (CDT). In order to access help-texts etc. for a Command in the
#     process its CDT must have been loaded in the local environment
#     by the CCS/cmd module. Completion is also dependent on it.
#
#     The sub-window "CDT File Setup" accessible in the Windows menu
#     provides support to load or clear CDTs interactively.
#
#     For more see the description in the CCS/cmd module.
#
#   FILES
#     The following files are searched in the directories given by
#     VLTROOT and (if existing) INTROOT and (not for CDTs) MODROOT:
#
#     bin/cmdManager - Daemon necessary for ccsei
#     CDT/<process>.cdt - One CDT per process, required by cmdManager
#     Configuration files are indirectly accessed by VccInfo(n).
#
#   ENVIRONMENT
#     RTAPENV  - Name of local Rtap environment (optional, not for CCS-lite)
#     CCSSTATE - Default operation mode [NORMAL,SIMULATION]
#   VARIABLES
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   BUGS
#     Command-Box:
#     - multiple occurences of a field in a line are not inhibited or checked
#   SEE ALSO
#     ccsei(1), ccseiDb(1), CcseiCommandBox(n), ccseiCmdLib(n)
#------------------------------------------------------------------------
#

###############################################################################
#
#	G L O B A L   V A R I A B L E S
#	"Declaration" of global variables
#
###############################################################################
# Rules: see ccsei.tcl
#
###############################################################################

# Tags used exclusively by ccseiMsg in the command-box:
# (Note: The tag-strings are used literally throughout the file. This list
#	 only defines the order)
set ccseiMsg_c(tags) { Env Pro Com Par } ;# internal tags for msg items

# beside those, the following tags are used:
#	Prompt - as already defined by the class `CcseiCommandBox'
#	Error - for error-messages


# The size of the history-buffer of command-lines:
vccSetDef ccseiMsg_c(historySize)	100

# Enable for automatic completion on Enter:
vccSetDef ccseiMsg_c(complOnEnter) 1

# Switch for reply-handling:
# 0: Asynchronous with event-handling in background
# 1: Synchronous with fixed loop until last-reply
vccSetDef ccseiMsg_c(asyncMode)	1

# Switch for verbose prompting:
vccSetDef ccseiMsg_c(fullPrompt) 1

# Set of valid sizes and fonts for the Commands-Box (name x-size y-size font):
set ccseiMsg_c(boxComFontSet) { \
	{ Default   98 17 -Adobe-helvetica-bold-r-normal--12* } \
	{ Small-1   98 20 -*-lucidatypewriter-bold-*-*-*-12-*-*-*-*-*-*-* } \
	{ Medium-1  76 18 -*-lucidatypewriter-bold-*-*-*-14-*-*-*-*-*-*-* } \
	{ Large-1   67 15 -*-lucidatypewriter-bold-*-*-*-17-*-*-*-*-*-*-* } \
	{ Small-2  112 22  6x13 } \
	{ Medium-2  75 19  9x15 } \
	{ Large-2   67 14 10x20 } \
    }
vccSetDef ccseiMsg_c(boxComFontNr) 0
set ccseiMsg_c(boxComFont) \
	[lindex $ccseiMsg_c(boxComFontSet) $ccseiMsg_c(boxComFontNr)]


# Global variables to be accessible from procedures:

set ccseiMsg_v(defSes)	""
set ccseiMsg_v(defEnv)	""	;# contents of the `Environment' entry
set ccseiMsg_v(defPro)	""	;# contents of the `Process' entry
set ccseiMsg_v(defCom)	""	;# contents of the `Command' entry
set ccseiMsg_v(defPar)	""	;# contents of the `Parameter' entry

set ccseiMsg_v(curEnv)	""	;# current Environment (after completion)
set ccseiMsg_v(curPro)	""	;# current Process (after completion)
set ccseiMsg_v(curCom)	""	;# current Command (after completion)
set ccseiMsg_v(curPar)	""	;# current Parameter (after completion)

set ccseiMsg_v(lastEnv) ""	;# last Environment word used for completion
set ccseiMsg_v(lastPro) ""	;# last Process word used for completion
set ccseiMsg_v(lastCom) ""	;# last Command word used for completion
set ccseiMsg_v(lastPar) ""	;# last Parameter word used for completion

set ccseiMsg_v(numCmdLine)  1	;# current number of the command-line
set ccseiMsg_v(curTag)	""	;# current field-tag during input
set ccseiMsg_v(nowTags) {}	;# list of entered field-tags
set ccseiMsg_v(cancelScript) 0	;# flag can be set during command script exec
set ccseiMsg_v(errorFlag) 0	;# most recent command/reply error condition:
				;#   0: OK
				;#  >0: number of error-reply
				;#  -1: hard SendCommand error
				;#  -2: hard RecvReply error

#
# flags used for checkbuttons in Options menu:
#
# Simulation Mode:
if {[info exists env(CCSSTATE)] && $env(CCSSTATE) == "SIMULATION"} {
    set ccseiMsg_v(simMode) 1
} else {
    set ccseiMsg_v(simMode) 0
    set env(CCSSTATE) NORMAL
}

# Update on Last Reply:
set ccseiMsg_v(contUpdate)	0

# enable completion:
set ccseiMsg_v(enaCompletion)	1


#
# Widget-names of sub-windows:
#

set ccseiMsg_w(winHelp)	.help	;# widget-name of toplevel help-window

set ccseiMsg_w(winRmon) .rmon	;# optional ReplyMonitor
set ccseiMsg_w(winCdt) .cdtdlg	;# widget-name of toplevel CDT-setup-window
set ccseiMsg_w(winSim) .simdlg	;# widget-name of toplevel Sim-setup-window



###############################################################################
#
#	P R O C E D U R E S
#
###############################################################################
# Rules: see ccsei.tcl
###############################################################################


#************************************************************************
#   NAME
#	ccseiMsgEvhDefaultSelect... - Event-handlers for default selections
# 
#   DESCRIPTION
#	These callback-procedures are invoked when an item is selected in the
#	corresponding menu resp. when <Return> is pressed in the entry.
#	(ccseiMsgEvhDefaultSelectSes is mainly for "symmetry" reasons, no menu 
#	 is associated with it.)
#
#   PARAMETERS
#	The text of the selected menu-item is passed to the procedure
#
#   RETURNS
#	None
#
#   GLOBALS
#	ccseiMsg_v(defX)	default selection
#				where X corresponds to Env/Pro/Com/Par
#   CAUTIONS
#
#   EXAMPLES
#
#------------------------------------------------------------------------
proc ccseiMsgEvhDefaultSelectSes { Session } {
    global ccseiMsg_v ccseiMsg_w
    set ccseiMsg_v(defSes) $Session ;# save selected value in a global variable
    set envlist [ccseiGetListOfEnvironments $ccseiMsg_v(defSes)]
    $ccseiMsg_w(selEnv) MakeMenu $envlist	;# attach next menu
    ccseiMsgCboxBeginNextField
    ccseiMsgResetCompletion ;# reset completion buffers for default values
    return $Session ;# necessary for VccSelector to return proper value
}

proc ccseiMsgEvhDefaultSelectEnv { Environment } {
    global ccseiMsg_v ccseiMsg_w
    set ccseiMsg_v(defEnv) $Environment ;# save selected value in global var
    set prolist [ccseiGetListOfProcesses $Environment]
    set menuW [$ccseiMsg_w(selPro) MakeMenu $prolist]	;# attach next menu
    ccseiMsgModifyProcessMenu $menuW		;# mark simulated processes
    ccseiMsgMarkFieldPresent Env
    ccseiMsgCboxBeginNextField
    ccseiMsgResetCompletion ;# reset completion buffers for default values
    return $Environment ;# necessary for VccSelector to return proper value
}

proc ccseiMsgEvhDefaultSelectPro { Process } {
    global ccseiMsg_v ccseiMsg_w
    set ccseiMsg_v(defPro) $Process ;# save selected value in global var
    set comlist [ccseiGetListOfCommands $Process]
    $ccseiMsg_w(selCom) MakeMenu $comlist	;# attach next menu
    ccseiMsgMarkFieldPresent Pro
    ccseiMsgCboxBeginNextField
    ccseiMsgResetCompletion ;# reset completion buffers for default values
    ccseiMsgHelpShow $Process "" ;# show help for all commands of this process
    return $Process ;# necessary for VccSelector to return proper value
}

proc ccseiMsgEvhDefaultSelectCom { Command } {
    global ccseiMsg_v ccseiMsg_w
    set ccseiMsg_v(defCom) $Command ;# save selected value in global var
    set parlist [ccseiGetTreeOfParamValues $ccseiMsg_v(defPro) $Command]
    $ccseiMsg_w(selPar) MakeMenu $parlist	;# attach next menu
    ccseiMsgMarkFieldPresent Com
    ccseiMsgCboxBeginNextField
    ccseiMsgResetCompletion ;# reset completion buffers for default values
    ccseiMsgHelpShow $ccseiMsg_v(defPro) $Command ;# show help for this command
    return $Command ;# necessary for VccSelector to return proper value
}

proc ccseiMsgEvhDefaultSelectPar { Parameter } {
    global ccseiMsg_v ccseiMsg_w

    if { [ $ccseiMsg_w(selPar) cget -userMenuEnable ] == 0 } {
	set ccseiMsg_v(defPar) $Parameter ;# save selected value in global var
    } else {
	# append selected value to global var
	set ccseiMsg_v(defPar) [ concat $ccseiMsg_v(curPar) " " $Parameter ]
    }

    set ccseiMsg_v(defPar) $Parameter ;# save selected value in global var
    ccseiMsgMarkFieldPresent Par
    ccseiMsgCboxBeginNextField
    ccseiMsgResetCompletion ;# reset completion buffers for default values
    return $Parameter ;# necessary for VccSelector to return proper value
}



#************************************************************************
#   NAME
#	ccseiMsgRefresh,
#	ccseiMsgMarkFieldPresent,
#	ccseiMsgModifyProcessMenu,
#	ccseiMsgRefineFields,
#	ccseiMsgResetCompletion
#	- general procedures
#
#   DESCRIPTION
#	ccseiMsgRefresh - performs a re-selection of all default values
#		and keeps all sub-windows up to date.
#
#	ccseiMsgMarkFieldPresent - marks a field as being entered.
#
#	ccseiMsgModifyProcessMenu - marks all processes in the given menu
#		which are registered for simulation.
#		Note that cascaded menus or continuation menus made by
#		the `MakeMenu' method are not supported.
#
#	ccseiMsgRefineFields - tries to rearrange Fields from a given Event
#		so that improper Field-breaks are corrected. This is only
#		necessary when the user didn't terminate all Fields correctly,
#		but e.g. wrote everything in the Field `Env' but intended to
#		enter Fields `Env', `Pro' and `Com'. This procedure will try
#		to separate them properly and returns the refined Event.
#		In case of multiple occurences the last item is taken, except
#		for field Par all items are concatenated.
#		Emtpy fields are preserved.
#
#	ccseiMsgResetCompletion - resets the completion buffers.
#		All fields contained in the optional `Event' argument 
#		override the current default values.
#
#   PARAMETERS
#	Tag - field identifier [Env,Pro,Com,Par]
#	Menu - widget-name of the menu to be modified
#	Event - list of Fields, in the form: {Env abc} {Pro xyz} ...
#		not all field-tags need to be present.
#
#   RETURNS
#
#   GLOBALS
#
#   CAUTIONS
#
#   EXAMPLES
#
#------------------------------------------------------------------------

proc ccseiMsgRefresh { } {
    global ccseiMsg_c ccseiMsg_v

    # check cmd module again:
    ccseiCmdCheckEnvironment

    # re-invoke all default selections:
    foreach tag $ccseiMsg_c(tags) {
	if {$ccseiMsg_v(def$tag) != ""} {
	    ccseiMsgEvhDefaultSelect$tag $ccseiMsg_v(def$tag)
	}
    }
    # refresh the other Sub-Windows:
    ccseiMsgHelpRefresh
    ccseiMsgRmonRefresh
    ccseiMsgSimRefresh
    ccseiMsgCdtRefresh
}

proc ccseiMsgMarkFieldPresent { Tag } {
    global ccseiMsg_v
    if {[lsearch -exact $ccseiMsg_v(nowTags) $Tag] == -1} {
	lappend ccseiMsg_v(nowTags) $Tag
    }
}

proc ccseiMsgModifyProcessMenu { Menu } {
    global env
    # get all processes which are registered for simulation:
    set simlist [ccseiGetListOfProcessesInSimulation]

    # go through all menu-entry and mark simulated processes with "SIM":
    set imax [$Menu index last]
    if {$imax == "none"} return
    for {set i 0} {$i <= $imax} {incr i} {
	# Note: index 4 of the returned list for -label should contain the
	# actual label. This is true for Tk 3.6, but may change (?)
	if {[catch {set process [lindex [$Menu entryconfigure $i -label] 4]}]} {
	    continue ;# ignore errors, e.g. with separator entries
	}
	if {[lsearch -exact $simlist $process] != -1} {
	    $Menu entryconfigure $i -accelerator SIM
	    if {$env(CCSSTATE) == "SIMULATION"} {
		# also change to a SCREAMING color when in Simulation-Mode:
		$Menu entryconfigure $i -background Orange \
					-activebackground Red
	    }
	} else {
	    $Menu entryconfigure $i -accelerator {} \
		    -background {} -activebackground {}
	}
    }
}

proc ccseiMsgRefineFields { Event } {
    global ccseiMsg_c ccseiMsg_v ccseiMsg_w

    # initialize:
    foreach tag $ccseiMsg_c(tags) { 
	set f($tag) 0
	set s($tag) ""
    }

    # extract the fields from the current history-event:
    foreach item $Event {
	set tag   [lindex $item 0]
	set field [lindex $item 1]
	set f($tag) 1
	if {$tag == "Par"} {
	    append s($tag) $field
	} else {
	    set s($tag) $field
	}
    }

    if {[llength $s(Env)] > 1 && $s(Pro) == ""} {
	set s(Pro) [lrange $s(Env) 1 end]
	set s(Env) [lindex $s(Env) 0]
    }
    if {[llength $s(Pro)] > 1 && $s(Com) == ""} {
	set s(Com) [lrange $s(Pro) 1 end]
	set s(Pro) [lindex $s(Pro) 0]
    }
    if {[llength $s(Com)] > 1 && $s(Par) == ""} {
	set s(Par) [lrange $s(Com) 1 end]
	set s(Com) [lindex $s(Com) 0]
    }

    set refinedEvent {}
    foreach tag $ccseiMsg_c(tags) {
	if {$f($tag) || $s($tag) != ""} {
	    set ccseiMsg_v(cur$tag) $s($tag) ;# save as current value
	    lappend refinedEvent [list $tag $s($tag)]
	}
    }

    ccseiPuts "ccseiMsgRefineFields: $Event ==> $refinedEvent"

    return $refinedEvent
}


proc ccseiMsgResetCompletion { {Event ""} } {
    global ccsei_w ccseiMsg_c ccseiMsg_v

    # set current tags to default tags:
    foreach tag $ccseiMsg_c(tags) {
	set ccseiMsg_v(cur$tag) $ccseiMsg_v(def$tag)
    }

    # override by given tags:
    foreach ev $Event {
	set tag			[lindex $ev 0]
	set ccseiMsg_v(cur$tag)	[lindex $ev 1]
    }

    # reset completion buffers for all tags:
    foreach tag $ccseiMsg_c(tags) {
	if {$ccseiMsg_v(cur$tag) != ""} {
	    switch $tag {
		Env {catch {ccseiGetListOfProcesses  $ccseiMsg_v(curEnv)}}
		Pro {catch {ccseiGetListOfCommands   $ccseiMsg_v(curPro)}}
		Com {catch {ccseiGetListOfParameters \
			$ccseiMsg_v(curPro) $ccseiMsg_v(curCom)}}
		Par {}
	    }
	}
    }

    # forget the last words used for completion:
    foreach tag $ccseiMsg_c(tags) {set ccseiMsg_v(last$tag) ""}

    return ""
}


#************************************************************************
#   NAME
#	ccseiMsgCbox... - general routines for the command-box
#	ccseiMsgRbox... - general routines for the reply-box
#
#   DESCRIPTION
#	ccseiMsgCboxBeginNextField - begin the next field from the list of
#		of not yet entered fields.
#
#	ccseiMsgCboxBeginNewCommandLine - initialize the list of not yet
#		entered fields and issue a new prompt.
#
#	ccseiMsgCboxUpdateCommandPrompt - update the current prompt to reflect
#		the currently entered field
#
#	ccseiMsgCboxDisplayError - output an error-line
#
#	ccseiMsgRboxDisplayReply - display a reply in the reply-box
#
#	ccseiMsgRboxShowReply - show reply in a separate pop-up window
#
#	ccseiMsgRboxShowError - show error-reply/stack in a separate window
#
#   PARAMETERS
#
#   RETURNS
#
#   GLOBALS
#
#   CAUTIONS
#
#   EXAMPLES
#
#------------------------------------------------------------------------

proc ccseiMsgCboxBeginNextField { {nextFieldTag ""} args } {
    global ccseiMsg_c ccseiMsg_v ccseiMsg_w
    if {$nextFieldTag == ""} {
	set nextFieldTag Par ;#...???
	foreach fieldTag $ccseiMsg_c(tags) {
	    if {[lsearch -exact $ccseiMsg_v(nowTags) $fieldTag] == -1 && \
		    [$ccseiMsg_w(boxCom) getField $fieldTag] == ""} {
		set nextFieldTag $fieldTag
		break
	    }
	}
    }
    set ccseiMsg_v(curTag) $nextFieldTag
    ccseiMsgCboxUpdateCommandPrompt	;# the tag is displayed in the prompt
    # now start the new field in the line:
    if {$nextFieldTag != "" && [lsearch $args "-nooutput"] == -1} {
	$ccseiMsg_w(boxCom) beginField $nextFieldTag
    }
    return $nextFieldTag
}

proc ccseiMsgCboxBeginNewCommandLine { } {
    global ccseiMsg_c ccseiMsg_v ccseiMsg_w
    # get the line-number from the history-buffer:
    set ccseiMsg_v(numCmdLine) [$ccseiMsg_w(boxCom) history nextid]
    # save the field-tags that are already in default-entries:
    set ccseiMsg_v(nowTags) {}
    foreach tag $ccseiMsg_c(tags) {
	if {$ccseiMsg_v(def$tag) != ""} {lappend ccseiMsg_v(nowTags) $tag}
    }

    # reset completion buffers for default values:
    ccseiMsgResetCompletion

    # start the first field:
    ccseiMsgCboxBeginNextField
}


proc ccseiMsgCboxUpdateCommandPrompt { } {
    global ccseiMsg_c ccseiMsg_v ccseiMsg_w

    set prompt $ccseiMsg_v(numCmdLine)
    if {$ccseiMsg_c(fullPrompt)} {
	foreach tag $ccseiMsg_c(tags) {
	    if {$ccseiMsg_v(def$tag) != ""} {
		append prompt " $ccseiMsg_v(def$tag)"
	    }
	}
    }
    append prompt " ($ccseiMsg_v(curTag))>"
    $ccseiMsg_w(boxCom) putPrompt $prompt
}

proc ccseiMsgCboxDisplayError { Errstr } {
    global ccseiMsg_w
    $ccseiMsg_w(boxCom) putChars Error " ERROR: ${Errstr}\n"
}

#
# This procedure has been introduced to re-use the error system utilities
# used to display the error stack on the screen.
# The current stack is extracted and re-formatted . 
# When the user requests to see the stack, it is stored into a temporary
# file and displayed using the errShowStack utility.
#

proc ccseiGetErrorStack { idx } {
    global ccseiMsg_stack

    set ccseiMsg_stack($idx) {}
    set stackSize [seq_errGetStackSize]

    for {set i 1 } {$i <= $stackSize} {incr i} {
        set tmp     [::seq_errGetFromStack $i]
        set seqNumb [lindex $tmp 2]
        set stackId [lindex [lindex $tmp 1] 1]
        set    buf [lindex $tmp 8]
        append buf " [lindex $tmp 0 ] [lindex $tmp 4]"
        append buf " proc [lindex $tmp 3] [lindex $tmp 5]"
        append buf " $stackId $seqNumb [lindex $tmp 6] { [lindex $tmp 7] }"

        lappend ccseiMsg_stack($idx) $buf
#        puts "DBG> ccseiMsg_stack($idx) $buf"
    }

    return $ccseiMsg_stack($idx)
}    



proc ccseiMsgRboxDisplayReply { Reply ErrorFlag Mark } {
    global ccseiMsg_v ccseiMsg_w ccseiMsg_stack gvar

    set prefix [expr {$ErrorFlag ? "ERROR" : "REPLY"}]
    append prefix " $Mark:"

    set stack {}
    if {$ErrorFlag != 0} {
       set stack [ccseiGetErrorStack [$ccseiMsg_w(boxRpl) getSize]]
       append prefix "$ErrorFlag:" 
    }

    set tmp [split $Reply "\n"]

    if {[lempty $tmp]} {
          $ccseiMsg_w(boxRpl) addItem end "$prefix  "
    } else {

       if {$gvar(splitMultiReply)} {
          foreach rp $tmp {
             $ccseiMsg_w(boxRpl) addItem end "$prefix [string trim $rp " "]"
          }
       } else {
          $ccseiMsg_w(boxRpl) addItem end "$prefix $Reply"
       }
    }

    $ccseiMsg_w(boxRpl) setCursor end

    # display the reply also on the Reply-Monitor, if it's active:
    ccseiMsgRmonShowReply $prefix $Reply $stack

    # debugging output:
    ccseiPuts "ccseiMsgRboxDisplayReply: $prefix $Reply"

    # check for the update-policy:
    # `update idletasks' takes longer for dislay, but without you probably
    # won't see anything before the last reply came in.
    if {$ccseiMsg_v(contUpdate)} { update idletasks }
}


proc ccseiMsgRboxShowReply { reply } {
    global ccseiMsg_v ccseiMsg_w

    # reply has the form: "REPLY 1/2: Message-body..."
    # separate prefix and reply-body:
    set prefix "<no reply-ID>"
    set id ""
    set i [string first : $reply]
    if {$i >= 0 && $i < 20} {
	set prefix [string range $reply 0 $i]
	scan $prefix "%s%s:" x id
	set reply  [string range $reply [incr i 2] end]
    }

    # make pop-up window with single text-widget and display reply:
    set W .reply$id
    catch {destroy $W}
    toplevel $W
    wm title $W "[wm iconname .]: Show Reply $id"
    wm iconname $W "[wm iconname .]: $id"
    wm iconbitmap $W [wm iconbitmap .]
    wm minsize $W 1 1
    text $W.text -background Grey60 \
	    -font -*-lucidatypewriter-medium-*-*-*-15-*-*-*-*-*-*-*
    pack $W.text -side top -expand yes -fill both
    $W.text insert 1.0 $reply
    # mark the text to see immediately its size:
    $W.text tag configure Reply -background Grey90 -foreground Black
    $W.text tag add Reply 1.0 end
}

proc ccseiMsgRboxShowError { idx } {
   global ccseiMsg_stack

#   puts "DBG> ccseiMsgRboxShowError show stack for line $idx"

   if {[info exists ccseiMsg_stack($idx)] == 0 } { return }
   
   #
   # Get stackId and open temporary file
   #
   set stackId [lindex [lindex $ccseiMsg_stack($idx) 0] 7]
   set fh  [open /tmp/stack_$stackId "w+" ]; 
   set i   1
   foreach item $ccseiMsg_stack($idx) {
      puts $fh "set myStack($i) {$item}" ;incr i
   }

   close $fh
   exec errShowStack $stackId &
}

# bound to single-click in reply-box to show either reply or error:
proc ccseiMsgEvhRboxSelect1 { } {
    global ccseiMsg_w ccseiMsg_stack

    if {[catch {$ccseiMsg_w(boxRpl) getSelection} reply]} { return }
    if {[catch {$ccseiMsg_w(boxRpl) getIndex} index]} { return }

    set tmp [string trim $reply "\{ \}"]
    set prefix       [ctoken tmp " \n"]	        ;# up to first new-line
    append prefix  " [ctoken tmp " \n"]"
    set reply        [string trim $tmp " "]

    set stack {}
    catch {set stack $ccseiMsg_stack($index)}

    # display the reply also on the Reply-Monitor, if it's active:

    ccseiMsgRmonShowReply $prefix $reply $stack

}

# bound to double-click in reply-box to show either reply or error:
proc ccseiMsgEvhRboxSelect2 { } {
    global ccseiMsg_w

    if {[catch {$ccseiMsg_w(boxRpl) getSelection} reply]} { return }

    if {[string first "REPLY" $reply] == 0} {
	ccseiMsgRboxShowReply $reply
    } else {
	ccseiMsgRboxShowError [$ccseiMsg_w(boxRpl) getIndex]
    }
}



#************************************************************************
#   NAME
#	ccseiMsgCboxHookEnter,
#	ccseiMsgCboxHookCompletion,
#	ccseiMsgCboxHookReverseCompletion,
#	ccseiMsgCboxHookHistory,
#	ccseiMsgEvhRecvReply
#	- event-handlers and hook-routines that react on specific keys
#	  pressed in the command-box
#
#   DESCRIPTION
#	ccseiMsgHookEnter - processes the command-line given as argument,
#		displays the reply or an error, and then begins a new line.
#
#	ccseiMsgCboxHookCompletion - performs context-dependent completion.
#		This procedure has finally become fairly complicated.
#		Its strategy can be roughly summarized as follows:
#		First it tries to resolve the argument to valid fields and
#		then tries to complete each field individually.
#		Found completions are inserted, the others remain as they were.
#		Additionally some field-specific actions are performed, like
#		showing matching commands or help-texts in the help-window.
#
#	ccseiMsgCboxHookReverseCompletion - lets you discard any default
#		fields to override them in the command-line.
#
#	ccseiMsgCboxHookHistory - resets the completion buffers after an 
#		history operation.
#
#	ccseiMsgEvhRecvReply is installed during `ccseiMsgHookEnter' as an
#		event-handler to display asynchronously all incoming replies
#		for the sent command.
#
#   PARAMETERS
#	Event - argument for ccseiMsgHookEnter and ccseiMsgCboxHookCompletion:
#		the contents of the command-line (i.e. fields) to be processed;
#		not literally, but in a special format, namely a list of:
#		{ <tag1> <field1> } { <tag2> <field2> } ...
#		For each <tag> its corresponding contents <field> is passed.
#		See CcseiCommandBox(n) for more.
#
#   RETURNS
#	All hook procedures must return a `modified' Event, which will then be
#	processed appropriately, or an empty string if no further action is 
#	necessary. See CcseiCommandBox(n) for more.
#
#   GLOBALS
#	ccseiMsg_v(comNr,<cmdId>) - to save the command-sequence-nr for `cmdId'
#	ccseiMsg_v(repNr,<cmdId>) - to save the reply-sequence-nr for `cmdId'
#	and a lot more ...
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#	CcseiCommandBox(n)
#
#------------------------------------------------------------------------

# event-handler installed by `ccseiMsgCboxHookEnter':
proc ccseiMsgEvhRecvReply { CmdId Reply ErrorFlag LastFlag TimeoutFlag } {
    global ccseiMsg_v

    set ccseiMsg_v(errorFlag) $ErrorFlag
    set mark $ccseiMsg_v(comNr,$CmdId)	;# get command sequence number of CmdId
    append mark /
    if {$LastFlag} {
	unset ccseiMsg_v(comNr,$CmdId)	;# don't need command number anymore
	unset ccseiMsg_v(repNr,$CmdId)	;# don't need reply number anymore
	append mark L			;# mark the last reply with a `L'
    } elseif {$TimeoutFlag} {
	append mark T			;# mark timeouts with a `T'
	set ccseiMsg_v(errorFlag) -2	;# hard reply error
    } else {
	append mark [incr ccseiMsg_v(repNr,$CmdId)] ;# else mark with number
    }
    ccseiMsgRboxDisplayReply $Reply $ErrorFlag $mark ;# display the reply
}


proc ccseiMsgCboxHookEnter { Event } {

    # `Event' has the form: {tag field}..., e.g. {Pro lccServer} {Com lccgver}

    global ccseiMsg_c ccseiMsg_v ccseiMsg_w

    # try to correct fields in case of improper input:
    set refinedEvent [ccseiMsgRefineFields $Event]

    # extract the fields from the current history-event:
    foreach tag $ccseiMsg_c(tags) { set s($tag) "" }
    foreach item $refinedEvent {
	set tag   [lindex $item 0]
	set field [lindex $item 1]
	set s($tag) $field
    }

    # for omitted fields take the default values from the entries:
    foreach tag $ccseiMsg_c(tags) { 
	if {$s($tag) == ""} {set s($tag) $ccseiMsg_v(def$tag) }
    }

    # check if all mandatory fields are there, display error if not:
    set missingFields ""
    if {$s(Env) == ""} {append missingFields "Environment "}
    if {$s(Pro) == ""} {append missingFields "Process "}
    if {$s(Com) == ""} {append missingFields "Command "}
    if {$missingFields != ""} {
	ccseiMsgCboxDisplayError "field(s) not specified: $missingFields"
    } else {
	# send the command, get and display the replies until `last reply':

	set cmd [list ccseiMsgSendCommand $s(Env) $s(Pro) $s(Com) $s(Par)]

	if {[catch $cmd cmdId]} {
	    # indicate error in the command-box:
	    ccseiMsgCboxDisplayError \
		    "ccseiMsgSendCommand: $cmdId\n [seq_errPrint]"
	    set ccseiMsg_v(errorFlag) -1 ;# hard command error
	    # and a dummy reply in the reply-box, sequence-number set to zero:
	    ccseiMsgRboxDisplayReply \
		    $cmdId $ccseiMsg_v(errorFlag) $ccseiMsg_v(numCmdLine)/0
	} else {
	  if {$ccseiMsg_c(asyncMode)} {
	    set ccseiMsg_v(comNr,$cmdId) $ccseiMsg_v(numCmdLine)
	    set ccseiMsg_v(repNr,$cmdId) 0 ;# incremented on each reply
	    set script "ccseiMsgEvhRecvReply $cmdId %R %E %L %T"
	    ccseiMsgEvtAttach $cmdId $script
	  } else {
	    set replyNr 0	;# is incremented until `last reply'
	    set lastReplyFlag 0	;# is set upon reception of `last reply'
	    set cmd [list ccseiMsgRecvReply $cmdId errorFlag lastReplyFlag]

	    while { ! $lastReplyFlag} {
		if {[catch $cmd reply]} {
		    # indicate error in the command-box:
		    ccseiMsgCboxDisplayError \
			    "ccseiMsgRecvReply: $reply\n [seq_errPrint]"
		    set ccseiMsg_v(errorFlag) -2 ;# hard reply error
		    break;
		} else {
		    ccseiMsgRboxDisplayReply $reply $errorFlag \
			    $ccseiMsg_v(numCmdLine)/[incr replyNr]
		    set ccseiMsg_v(errorFlag) $errorFlag ;# evt. soft error > 0
		}
	    }
	  }
	}
    }

    # issue a new prompt:
    ccseiMsgCboxBeginNewCommandLine

    # if the Event needs to re-recorded in the history, return it:
    if {$Event != $refinedEvent} {
	return $refinedEvent
    } else {
	return ""
    }
}


proc ccseiMsgCboxHookCompletion { Event } {
    global ccseiMsg_c ccseiMsg_v ccseiMsg_w ccsei_w

    # this forms the final Event and is extended with each found completion
    set completedEvent ""

    # bring the Event to a defined state, resolve multiple words in a field:
    set refinedEvent [ccseiMsgRefineFields $Event]
    # if there was no resolving, there's no need to keep it in the final Event:
    if {$Event != $refinedEvent} {set completedEvent $refinedEvent}

    # now perform an individual completion for each field:
    set complDone 0	;# number of successful completions
    set complFailed 0	;# number of failed completions
    foreach item $refinedEvent {
	# determine which field shall be completed and its current contents:
	set what  [lindex $item 0]	;# Tag
	set field [lindex $item 1]	;# contents

	# determine position of last word in field:
	set pos [llength $field]
	if {$pos > 0} {
	    incr pos -1
	    set word [lindex $field $pos]
	} else {
	    set word $field
	}

	# ask for a unique completion for the field:
	set completion [$ccsei_w(compl$what) Completion get $word]
	if {$pos > 0} {
	    set completedField [lreplace $field $pos $pos $completion]
	} else {
	    set completedField $completion
	}

	# if there is actually a unique completion...
	if {$completion != ""} {
	    incr complDone
	    # ... add it as new field to the final Event:
	    lappend completedEvent [list $what $completedField]
	    # ... and let other procedures know that it's there:
	    ccseiMsgMarkFieldPresent $what
	    # perform field-specific actions, if any:
	    switch $what {
		Env {
		    ccseiGetListOfProcesses $completion ;# init process-compl.
		}
		Pro {
		    ccseiGetListOfCommands $completion  ;# init command-compl.
		    ccseiMsgHelpShow $completion ""     ;# show all commands
		}
		Com {
		    ccseiGetTreeOfParamValues $ccseiMsg_v(curPro) $completion
		    ccseiMsgHelpShow "" $completion     ;# show help on command
		}
		Par {
		    ##should be added for better Parameter-completion:
		    ##ccseiGetListOfParamValues \
		    ##	 $ccseiMsg_v(defPro) $ccseiMsg_v(defCom) $completion
		    ##ccseiMsgHelpShow "" "" $completion ;# view parameter
		}
	    }
	} else {
	    # ... if there is no unique completion ...
	    # ... but it was already tried to complete the same word ...
	    if {$field == $ccseiMsg_v(last$what)} {
		# ... then don't complete it, but just proceed with next field:
		#ccseiMsgCboxBeginNextField
		incr complDone
		# ... and let other procedures know that it's there:
		ccseiMsgMarkFieldPresent $what
	    } elseif {$what == "Com"} {
		# for commands show all matching commands:
		ccseiMsgHelpShow "" $field
		incr complFailed
	    } else {
		incr complFailed
	    }
	    ccseiPuts "ccseiMsgCboxHookCompletion: $what: no unique completion"
	}

	# save the word so that we can recall it the next time:
	set ccseiMsg_v(last$what) $field

	# save it also as current value:
	set ccseiMsg_v(cur$what) $field
    }

    # in case we found all completions begin the next field:
    if {$complDone > 0 && $complFailed == 0} {
	# determine the next missing field, display it in the prompt:
	set next [ccseiMsgCboxBeginNextField "" -nooutput]
	if {$next != ""} {lappend completedEvent [list $next ""]}
    }

    # refine once again to resolve multiple occurences of a field:
    set completedEvent [ccseiMsgRefineFields $completedEvent]

    return $completedEvent
}


# this resets the list of present fields, to begin from scratch:
proc ccseiMsgCboxHookReverseCompletion { } {
    global ccseiMsg_v
    set ccseiMsg_v(nowTags) {}
    ccseiMsgCboxBeginNextField

    return ""
}


# this resets the completion buffers after an history operation:
proc ccseiMsgCboxHookHistory { Event } {
    return [ccseiMsgResetCompletion $Event]
}


# this is now obsolete, was connected to the Space-key to begin the next field.
# but then it's not possible to type multiple spaces!
proc ccseiMsgCboxHookFieldTerminator { } {
    global ccseiMsg_v ccseiMsg_w
    ccseiMsgCboxBeginNextField
##    ccseiMsgCboxUpdateCommandPrompt
}



#************************************************************************
#   NAME
#	ccseiMsg... - Event-handlers for the menu-bar
# 
#   DESCRIPTION
#	These callback-procedures are invoked when an item is selected in the
#	corresponding menu-bar.
#
#   PARAMETERS
#	none
#
#   RETURNS
#	None
#
#   GLOBALS
#
#   CAUTIONS
#
#   EXAMPLES
#
#------------------------------------------------------------------------

#
# Event-Handlers for `File' menu
#

#
# Execute command-script, syntax as for lccei [VLT-MAN-SBI-17210-0001,2.4,309]
# with the following exceptions:
# - keyword PARAMETER_SET is ignored
# - keyword ENVIRONMENT is synonymous to NODE
# - keyword TIMEOUT is synonymous to MAX_DELAY
#
proc ccseiMsgFileExecuteCommandScript { {File ""} } {
    global ccsei_v ccseiMsg_c ccseiMsg_v ccseiMsg_w

    # select filename to open:
    set filename $File
    if {$filename == ""} {
	set filename [panSelectFile . *.cs]
    }
    if {$filename == ""} {
	return
    }

    # process file with busy cursor and main window disabled:
    ccseiPuts "ccseiMsgFileExecuteCommandScript: opening \"$filename\""
    vccWinBusy [list ccseiMsgFileExecuteCommandScript_1 $filename]
}

proc ccseiMsgFileExecuteCommandScript_1 { File } {
    global ccsei_v ccseiMsg_c ccseiMsg_v ccseiMsg_w

    set filename $File

    # save current user-settings in `prev' to restore them later:
    set prev(timeout)		$ccsei_v(timeout)
    set prev(contUpdate)	$ccseiMsg_v(contUpdate)
    set prev(complOnEnter)	$ccseiMsg_c(complOnEnter)
    set prev(asyncMode)		$ccseiMsg_c(asyncMode)
    set prev(fullPrompt)	$ccseiMsg_c(fullPrompt)

    # overwrite user-settings, constant during command-script execution:
    set ccseiMsg_v(contUpdate)	 1	;# display must be continously updated
    set ccseiMsg_c(complOnEnter) 0	;# no implicit completion
    set ccseiMsg_c(asyncMode)	 0	;# synchronous mode is needed
    set ccseiMsg_c(fullPrompt)	 1	;# verbose prompt is desirable

    # set working values during command-script execution:
    set work(Env) $ccseiMsg_v(defEnv)	;# default environment
    set work(Pro) $ccseiMsg_v(defPro)	;# default process
    set work(timeout) $ccsei_v(timeout)	;# default timeout
    set work(stopOnError) 1		;# by default stop on error reply
    set work(error) 0			;# no error so far

    # pop-up window to show script progress and provide `Cancel' button:
    set W .script
    catch {destroy $W}
    vccStdToplevel [toplevel $W]
    pack [UifListBox $W.list -scrollx 0 -scrolly 1 -geometry 60x10]
    pack [UifActionButton $W.cancel -label "Abort/Dismiss" -width 20 \
	    -command [list set ccseiMsg_v(cancelScript) 1] ]
    $W.list addItem end "== SCRIPT: $filename =="

    # process the file contents:
    set f [open $filename r]
    set ccseiMsg_v(cancelScript) 0 ;# flag can be set by `Cancel' button
    while {! $ccseiMsg_v(cancelScript) && [gets $f line] >= 0} {
	# show current line in pop-up window:
	$W.list addItem end $line
	$W.list setCursor end
	update ;# force display and allow processing of `Cancel' button

	# check for comments and empty lines:
	if {[string trim $line] == "" || [string index $line 0] == "#"} {
	    continue
	}

	# parse and process line:
	set keyword [string trim [ctoken line :]]
	set value   [string trim [crange $line 1 end]]
	ccseiPuts "ccseiMsgFileExecuteCommandScript: {$keyword} {$value}"
	switch -- $keyword {
	    ENVIRONMENT		-
	    NODE		{set work(Env) $value}
	    PROCESS		{set work(Pro) $value}
	    TIMEOUT		-
	    MAX_DELAY		{set work(timeout) $value}
	    PARAMETER_SET	{# only lccei, ignored here}
	    STOP_ON_ERROR	{set work(stopOnError) 1}
	    CONT_ON_ERROR	{set work(stopOnError) 0}
	    WAIT		{after [expr {$value * 1000}]}
	    COMMAND		{
		# extract command and parameters:
		set work(Com) [ctoken value " \t"]	;# 1st word is command
		set work(Par) $value			;# rest are parameters
		# process command with current working variables:
		ccseiPuts "ccseiMsgFileExecuteCommandScript:\n\
			$work(Env) $work(Pro) $work(Com) $work(Par)"
		set ccsei_v(timeout) $work(timeout)
		ccseiMsgCboxHookReverseCompletion ;# begin line from scratch
		foreach tag {Env Pro Com Par} {
		    # fill line (space after field is a workaround):
		    $ccseiMsg_w(boxCom) putField $tag "$work($tag) "
		    #$ccseiMsg_w(boxCom) putField $tag $work($tag) ;# fill line
		}
		# process command line, check for error-conditions:
		if {[catch {$ccseiMsg_w(boxCom) doEnter} out]} {
		    set work(error) -1
		    #puts $out ;#DEBUG
		    break			;# hard error ==> abort
		}
		if {$work(stopOnError) && $ccseiMsg_v(errorFlag) != 0} {
		    set work(error) $ccseiMsg_v(errorFlag)
		    break
		}
	    }
	    default		{set work(error) KEYWORD; break}
	}
    }
    close $f

    # restore previous user-settings:
    set ccsei_v(timeout)	 $prev(timeout)
    set ccseiMsg_v(contUpdate)	 $prev(contUpdate)
    set ccseiMsg_c(complOnEnter) $prev(complOnEnter)
    set ccseiMsg_c(asyncMode)	 $prev(asyncMode)
    set ccseiMsg_c(fullPrompt)	 $prev(fullPrompt)

    # inform about error condition:
    switch -- $work(error) {
	 0	{set str "Script completed normally"}
	-1	{set str "Script aborted due to hard msgSendCommand error"}
	-2	{set str "Script aborted due to hard msgRecvReply error"}
	KEYWORD	{set str "Script aborted: invalid keyword ($keyword)"}
	default	{set str "Script aborted due to error reply"}
    }
    $W.list addItem end "== $str =="
    $W.list setCursor end

    # wait for Cancel button, or implicit timeout when no error:
    set delay 50 ;# return automatically after 5 seconds
    while {! $ccseiMsg_v(cancelScript)} {
	update ;# allow processing of `Cancel' button
	after 100
	if {$work(error) == 0 && [incr delay -1] <= 0} break
    }
    destroy $W ;# remove pop-up window

    return
}

proc ccseiMsgFileOpenLogFile { } {
    ccseiTBD
#...
}

proc ccseiMsgFileCloseLogFile { } {
    ccseiTBD
#...
}


proc ccseiMsgEvhOptionParamAppend { } {
    global ccsei_v ccseiMsg_w
    $ccseiMsg_w(selPar) configure -userMenuEnable $ccsei_v(parAppen)
}


#
# Event-Handlers for `Options' menu
#
proc ccseiMsgEvhOptionRefreshPanel { } {
    ccseiMsgRefresh
}

proc ccseiMsgEvhOptionClearCache { } {
    ccseiOptionOnAllLists clear
}

proc ccseiMsgEvhOptionClearHistory { } {
    global ccseiMsg_w ccseiMsg_stack

    catch {seq_msgFlush -nowait all}
    $ccseiMsg_w(boxCom) delete 1.0 end
    ccseiMsgCboxBeginNewCommandLine
    $ccseiMsg_w(boxRpl) deleteItem 0 end

    if {[info exists ccseiMsg_stack]} { unset ccseiMsg_stack }
}

proc ccseiMsgEvhOptionSimulationMode { } {
    global env ccseiMsg_v ccseiMsg_w

    if {$ccseiMsg_v(simMode)} {
	set env(CCSSTATE) "SIMULATION"
    } else {
	set env(CCSSTATE) "NORMAL"
    }
    ccseiPuts "ccseiMsgEvhOptionSimulationMode: Entered $env(CCSSTATE) mode"
    ccseiMsgRefresh
}

proc ccseiMsgEvhOptionSynonymMode { } {
    global ccsei_v ccseiMsg_v
    ccseiPuts "ccseiMsgEvhOptionUseSynonyms: synMode = $ccsei_v(synMode)"
    ccseiMsgRefresh
}

proc ccseiMsgEvhOptionUpdateOnLastReply { } {
    global ccseiMsg_v
    ccseiPuts "ccseiMsgEvhOptionUpdateOnLastReply: $ccseiMsg_v(contUpdate)"
}

proc ccseiMsgEvhOptionEnableCompletion { } {
    global ccseiMsg_v ccsei_c ccsei_w
    if {$ccseiMsg_v(enaCompletion)} {
	$ccsei_w(complEnv) configure -completionMode $ccsei_c(complModeEnv)
	$ccsei_w(complPro) configure -completionMode $ccsei_c(complModePro)
	$ccsei_w(complCom) configure -completionMode $ccsei_c(complModeCom)
	$ccsei_w(complPar) configure -completionMode $ccsei_c(complModePar)
    } else {
	$ccsei_w(complEnv) configure -completionMode {}
	$ccsei_w(complPro) configure -completionMode {}
	$ccsei_w(complCom) configure -completionMode {}
	$ccsei_w(complPar) configure -completionMode {}
    }
    ccseiPuts "ccseiMsgEvhOptionEnableCompletion: $ccseiMsg_v(enaCompletion)"
}

proc ccseiMsgEvhOptionEnableDebug { } {
    global ccsei_v ccseiMsg_v ccseiMsg_w ccsei_w
    # propagate debug flag:
    ccseiDebugControl
    # info about internal cache lists:
    ccseiPuts [ccseiOptionOnAllLists info]
    # info about internal completion:
    $ccsei_w(complEnv) Completion info {}
    $ccsei_w(complPro) Completion info {}
    $ccsei_w(complCom) Completion info {}
    $ccsei_w(complPar) Completion info {}
}


#
# Event-Handlers for `Windows' menu
#
proc ccseiMsgEvhWindowReplyMonitor { } {
    global ccseiMsg_w ccseiMsgRmon_v ccseiMsgRmon_w cv
   
    set ccseiMsgRmon_w(boxStk) [$cv.boxStk component uifText]
    set ccseiMsgRmon_w(boxQue) $cv.boxQue

    # Store original names
    set ccseiMsgRmon_w(oldBoxQue) $ccseiMsgRmon_w(boxQue)
    set ccseiMsgRmon_w(oldBoxStk) $ccseiMsgRmon_w(boxStk)

    ccseiMsgRmonSetNames
    # do an initial refresh of the window:
    ccseiMsgRmonMonitor
}

proc ccseiMsgEvhWindowSimulationSetup { } {
    global ccseiMsg_w ccseiCmd_v
    if {$ccseiCmd_v(dead)} {
	ccseiDialog "This window is not available, since the cmdManager was\
		not alive at startup. Try to restart the local environment."
    } else {
	ccseiMsgSimCreateWindow $ccseiMsg_w(winSim)
    }
}

proc ccseiMsgEvhWindowCdtFileSetup { } {
    global ccseiMsg_w ccseiCmd_v
    if {$ccseiCmd_v(dead)} {
	ccseiDialog "This window is not available, since the cmdManager was\
		not alive at startup. Try to restart the local environment."
    } else {
	ccseiMsgCdtCreateWindow $ccseiMsg_w(winCdt)
    }
}



#
# Event-Handlers for `Help' menu
#
proc ccseiMsgEvhHelpInfo { } {
    global ccseiMsg_c
    catch {ccseiCmdQuery VERSION ""} cmdVersion
    catch {infox appversion} seqVersion
    ccseiDialog "ccseiMsg Version $ccseiMsg_c(version)\
	    \nMessage-Command-Window of the\nCCS Engineering User Interface\
	    \nWorking with:\
	    \nSequencer version: $seqVersion\
	    \ncmdManager version: $cmdVersion"
}

proc ccseiMsgEvhHelpOnCommands { } {
    global ccseiMsg_w ccseiCmd_v
    if {$ccseiCmd_v(dead)} {
	ccseiDialog "This window is not available, since the cmdManager was\
		not alive at startup. Try to restart the local environment."
    } else {
	ccseiMsgHelpCreateWindow $ccseiMsg_w(winHelp)
    }
}

proc ccseiMsgEvhHelpOnStatus { } {
    set W .status
    catch {destroy $W}
    vccStdToplevel [toplevel $W]
    pack [CcseiCommandBox $W.box -width 80 -height 35]
    pack [UifActionButton $W.dismiss -label Dismiss -width 10\
	    -command [list destroy $W] ]
    ccseiMsgWelcomePage $W.box
    $W.box putChars Text2 [ccseiCmdCheckEnvironment]
    $W.box putChars Text1 \
	    "\nInternal Cache Information:\n[ccseiOptionOnAllLists info]"
}

proc ccseiMsgEvhHelpExtHelp { } {
    global argv0
    exec vltMan ccseiMsg &
}



#
# Welcome-Page
#

proc ccseiMsgWelcomePage { {CommandBoxW ""} } {
    global seq_ccsEnvName seq_ccsCmdCheck env
    global ccsei_n ccsei_v ccseiMsg_c ccseiMsg_v ccseiMsg_w
    global ccseiCmd_v

    if {$CommandBoxW == ""} {
	set W $ccseiMsg_w(boxCom)
    } else {
	set W $CommandBoxW
    }

    catch {
	$W configField Header1 -wrap word \
		-font -*-times-bold-i-*-*-25-*-*-*-*-*-*-*
	$W configField Text1 -wrap word \
		-font -*-times-medium-r-*-*-14-*-*-*-*-*-*-*
	$W configField Text2 -foreground Red \
		-font -*-times-bold-i-*-*-14-*-*-*-*-*-*-*

	$W configField News -wrap word \
		-foreground Blue \
		-relief raised -borderwidth 2 
	$W bindField News <1> ccseiMsgShowNews
    }

##  $W putChars News "   What's new   "
    $W putChars Text1 "                "

    $W putChars Text1 "                    "
    $W putChars Header1 "CCS Engineering User Interface "
    $W putChars Text1 " Version $ccseiMsg_c(version)\n\n"

    set x "The local environment is "
    set v "UNKNOWN"
    if {[info exists seq_ccsEnvName]} {set v $seq_ccsEnvName}
    # recognition of CCS-lite must be re-done. This is no good:
    #if {[info exists env(RTAPENV)]} {
    #	append v " (CCS with Rtap)"
    #} else {
    #	append v " (CCS-lite, no Rtap)"
    #}
    $W putChars Text1 $x
    $W putChars Text2 $v

    if {[catch {ccseiGetListOfEnvironments WELCOME} v]} {
	set x ". "
	set v "Can't find any other defined environments!"
    } else {
	set v [lrange $v 0 [expr {[llength $v]-3} ]] ;# remove "{} SIM" at end
	set x ". In total there are [llength $v] environments defined in your\
		network:\n"
    }
    $W putChars Text1 $x
    $W putChars Text2 \t$v\n

    if {$ccseiCmd_v(dead)} {
	set x ""
	set v "The $ccsei_n(cmdManager) is apparently NOT alive. Can't do\
		much without it!"
    } else {
	set v [ccseiGetListOfLoadedCDTs]
	set x "The $ccsei_n(cmdManager) is apparently alive. It reports that \
		[llength $v] CDTs are loaded:\n"
    }
    $W putChars Text1 $x
    $W putChars Text2 \t$v\n

    set x "Current option settings are (remember that you can change them\
	    all with the `Options' menu!):"
    $W putChars Text1 $x\n

    set x "- Simulation-Mode is "
    set v [expr {$ccseiMsg_v(simMode) ? "ACTIVE" : "OFF"}]
    $W putChars Text1 \t$x
    $W putChars Text2 $v\n

    set x "- Translation of Synonyms into Commands is "
    set v [expr {$ccsei_v(synFlag) ? "enabled" : "disabled"}]
    $W putChars Text1 \t$x
    $W putChars Text2 $v\n

    set x "- The Synonym-Mode is: "
    switch $ccsei_v(synMode) {
	Com     {set v "You are using only Commands, no Synonyms"}
	Syn     {set v "You are using only Synonyms, no Commands"}
	ComSyn  {set v "You are using both Commands and Synonyms"}
	SynCom  {set v "You are using both Synonyms and Commands"}
	default {set v "UNKNOWN"}
    }
    $W putChars Text1 \t$x
    $W putChars Text2 $v\n

    set x "- Error-Checking is set to "
    if {[set v $seq_ccsCmdCheck] == ""} {set v UNKNOWN}
    $W putChars Text1 \t$x
    $W putChars Text2 $v\n

    set x "- The Timeout for message-receive is set to "
    set v "$ccsei_v(timeout) ms"
    $W putChars Text1 \t$x
    $W putChars Text2 $v\n

    set x "See the short help below for the most important electric keys."
    append x \n\n
    $W putChars Text1 $x
}


proc ccseiMsgShowNews { } {
    global ccseiMsg_w

    # read the file containing the News:
    set f [open [findVLTFile include/ccseiNews.txt]]
    if {$f == ""} return
    set news [read $f]
    close $f

    # display it in the command-box:
    $ccseiMsg_w(boxCom) yview end		;# make cursor visible
    $ccseiMsg_w(boxCom) putChars News \n\n$news\n
    $ccseiMsg_w(boxCom) mark set insert end	;# move cursor to end
    ccseiMsgCboxBeginNewCommandLine
}

proc ccseiMsgWdgConfigure {} {
   global ccseiMsg_w ccseiMsg_c ccsei_c seq_ccsEnvName env

   if {![info exist seq_ccsEnvName]} {
      set seq_ccsEnvName ""
      if {[info exists env(RTAPENV)]} { set seq_ccsEnvName $env(RTAPENV) }
   }

# configure command-box:
# ----------------------

# set font and sizes:
if {[llength $ccseiMsg_c(boxComFont)] > 3} {
    $ccseiMsg_w(boxCom) configure -width  [lindex $ccseiMsg_c(boxComFont) 1]
    $ccseiMsg_w(boxCom) configure -height [lindex $ccseiMsg_c(boxComFont) 2]
    $ccseiMsg_w(boxCom) configure -font   [lindex $ccseiMsg_c(boxComFont) 3]
}

# configuration of tags in order 
# [text(n) implies highest priority last, but that is not important here]:
foreach tag {Prompt Error Env Pro Com Par} {

    if {[info exists ccseiMsg_c($tag.foreground)]} {
	$ccseiMsg_w(boxCom) configField $tag \
		-foreground $ccseiMsg_c($tag.foreground)
    }
    if {[info exists ccseiMsg_c($tag.background)]} {
	$ccseiMsg_w(boxCom) configField $tag \
		-background $ccseiMsg_c($tag.background)
    }
}

# configuration of the history-buffer:
$ccseiMsg_w(boxCom) configure -historySize $ccseiMsg_c(historySize)
$ccseiMsg_w(boxCom) configure -hookHistory ccseiMsgCboxHookHistory

# configuration of the electric Enter-key and hook:
$ccseiMsg_w(boxCom) configure -keyEnter $ccsei_c(keyEnter)
$ccseiMsg_w(boxCom) configure -hookEnter ccseiMsgCboxHookEnter
$ccseiMsg_w(boxCom) configure -complOnEnter $ccseiMsg_c(complOnEnter)

# configuration of the electric Completion-key and hook:
$ccseiMsg_w(boxCom) configure -keyCompletion $ccsei_c(keyCompl)
$ccseiMsg_w(boxCom) configure -hookCompletion ccseiMsgCboxHookCompletion

# bind other electric keys:
$ccseiMsg_w(boxCom) setBind $ccsei_c(keyRevCompl) \
	{ccseiMsgCboxHookReverseCompletion ; break}


# configure reply-box:
# --------------------
# (double)-button-1 should bring up the corresponding error-stack.
$ccseiMsg_w(boxRpl) setBind <1>		{ccseiMsgEvhRboxSelect1}
$ccseiMsg_w(boxRpl) setBind <Double-1>	{ccseiMsgEvhRboxSelect2}



# setup environment:
# ------------------

# check the CCS/cmd environment:
set msg [ccseiCmdCheckEnvironment]
if {$msg != ""} { ccseiDialog $msg }


# display welcome-page:
# ---------------------
if {[catch {ccseiMsgWelcomePage}]} {
    $ccseiMsg_w(boxCom) putChars "" "\nCannot display welcome-page\n\n"
}


# setup command input box:
# ------------------------

# display the first prompt:
#ccseiMsgCboxUpdateCommandPrompt
ccseiMsgCboxBeginNewCommandLine

# set the input-focus to the command-box:
##REMOVED in Tk4: $ccseiMsg_w(boxCom) setFocusDefault
$ccseiMsg_w(boxCom) setFocus


# prepare defaults:
# -----------------

# retrieve valid environments and make the menu for it:
ccseiMsgEvhDefaultSelectSes A ;# the Session indentifier is any arbitrary value

# preset environment entry with the local env (ignore any errors):
set ccseiMsg_v(defEnv) $seq_ccsEnvName

# query valid processes for this env (ignore any errors):
ccseiMsgEvhDefaultSelectEnv $seq_ccsEnvName

# window stuff:
# -------------

# open reply-monitor window:

ccseiMsgEvhWindowReplyMonitor

}
#___oOo___
