#*******************************************************************************
# E.S.O. - VLT project
#
# @(#) $Id: ctooCCONFIG.tcl,v 1.52 2005/09/06 23:52:07 vltsccm Exp $
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  16/06/02  SPR VLTSW20020376: LoadLine: better regexp.
# rschmutz  16/09/00  support multiple CONFIG.ARCHIVEi.* configs.
# rschmutz  04/09/00  default backup dir: $VLTDATA/config.
# rschmutz  02/09/00  process CONFIG.ARCHIVE.* keywords.
# rschmutz  05/08/00  created.
#

#************************************************************************
#   NAME
#	ctooCCONFIG - ctoo configuration file interface
#
#   SYNOPSIS
#	ctooCCONFIG ccfg
#
#   DESCRIPTION
#	Itcl interface to the ctoo configuration stored either in:
#	the INS_ROOT, INTROOT or VLTROOT.
#
#   PUBLIC METHODS
#	ctooCCONFIG ccfg
#           Constructor.  All configuration files '*CONFIG.cfg'
#	    found in INS_ROOT, INTROOT and VLTROOT are loaded
#	    (in that order).
#
#	GetSetIndex <name>
#	    Return the index i of CONFIG.SETi.NAME that has
#	    value <name>.
#
#	Get <keyword>
#	    Return the value of a configuration keyword.
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#       $INTROOT/config/*CONFIG.cfg
#       $VLTROOT/config/*CONFIG.cfg
#           ctoo configuration files.
#
#   ENVIRONMENT
#	INS_ROOT	Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTION
#       INS_ROOT resp. INTROOT must be defined to be able to read
#       a configuration stored in the corresponding location.
#
#   EXAMPLES
#	ctooCCONFIG ccfg
#	set set1name [cfg Get CONFIG.SET1.NAME]
#	
#   SEE ALSO
#	ctoo(5)
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: ctooCCONFIG
#------------------------------------------------------------------------

class ctooCCONFIG {
    inherit ctooOBJ_BASE

    #****************************************************************
    #*# constructor
    #
    # 1. search all *CONFIG.cfg files
    # 2. load dictionary
    # 3. load config. files
    #----------------------------------------------------------------

    constructor {} {
        ctooOBJ_BASE::constructor -objType CTOO_CONFIG
	global env

	#*# 1. search all *CONFIG.cfg files
	set seqbuf $this.cfg
	set cfgfiles ""
	foreach {envvar relpath} { INS_ROOT "/SYSTEM/COMMON/CONFIGFILES/" \
				INTROOT "/config/" \
				VLTROOT "/config/" } {
	    try_eval { set envvar $env($envvar) } {
		continue
	    }
	    set cfgfilepat [cconcat $envvar $relpath "*CONFIG.cfg"]
	    set cfgfiles [concat $cfgfiles [lsort -decreasing [glob -nocomplain $cfgfilepat]]]
	}
	if { "$cfgfiles" == "" } {
	    error "No *CONFIG.cfg files found."
	}

	#*# 2. load dictionary
	ctooDICT $this.dict -dict CTOO_CFG

	#*# 3. load config. files
	#
	# - load first log, backup and bakdir
	# - load config. sets according to name,
	#   reindex as appropriate.
	set log ""
	set backup ""
	set bakdir "$env(VLTDATA)/config"
	set names "##DUMMY##"
	set i 1
	set ari 1
	foreach cfgfile $cfgfiles {

	    #*# 3.1 load all keywords into cfgkeys
	    catch { unset cfgkeys }
	    for_file txt $cfgfile {
	        set line [LoadLine $txt]
	        if { "$line" != "" } {
	            lassign $line key val
		    if { "[cindex $val 0]" == "\"" && "[cindex $val end]" == "\"" } {
		        set val [csubstr $val 1 len-2]
		    }
		    if { [csubstr $key 0 7] == "CONFIG." } {
		    	$this.dict Check $key $val
		        keylset cfgkeys $key $val
		    }
	        }
	    }

	    #*# 3.2 update log, backup, bakdir
	    set log    [SetKey CONFIG.CON.LOG    $log]
	    set backup [SetKey CONFIG.CON.BACKUP $backup]
	    set bakdir [SetKey CONFIG.CON.BAKDIR $bakdir]

	    #*# 3.3 retrieve CONFIG.ARCHIVE.* keywords
	    set arckeys ""
	    catch { set arckeys [keylget cfgkeys CONFIG.ARCHIVE] }
	    if { $arckeys != "" } {
	        foreach key [keylkeys arckeys] {
		    set val [keylget arckeys $key]
		    set keytbl(CONFIG.ARCHIVE$ari.$key) $val
		}
		incr ari
	    }
	    set arckeys ""

	    #*# 3.4 retrieve list of config. sets: {SET<i> <name>}
	    set cfgkeys [keylget cfgkeys CONFIG]
	    catch { keyldel cfgkeys CON }
	    catch { keyldel cfgkeys ARCHIVE }
	    set namekeys ""
	    foreach skey [keylkeys cfgkeys] {
	    	lappend namekeys [list $skey [keylget cfgkeys $skey.NAME]]
	    }

	    #*# 3.5 foreach config. set:
	    foreach namekeyl $namekeys {
	    	lassign $namekeyl skey setname

		#*# 3.6 if set not already loaded: load it.
		if {![lcontain $names $setname]} {
		    lappend names $setname
		    foreach setkeyl [keylget cfgkeys $skey] {
		    	lassign $setkeyl key val
			set keytbl(CONFIG.SET$i.$key) $val
		    }
		    incr i
		}
	    }
	}
    }

    #****************************************************************
    #*# method: LoadLine
    #----------------------------------------------------------------

    protected method LoadLine { txt } {

	    #*# 1. ignore empty lines and comments
	    set txt2 $txt
	    set txt [string trim $txt]
	    if { $txt == "" || [cindex $txt 0] == "#" } {
	    	return ""
	    }

	    #*# 2. get parameter name
	    set param [ctoken txt " ;\t\n\r"]

	    #*# 3. get parameter value
	    set val ""
	    set txt [string trim $txt]
	    if { [cindex $txt 0] != ";" } {
	    	# ((escaped double quote )? + string without quotes)*
	        if {![regexp "\"(((\\\\\")?\[^\"\]*)*)\"" $txt dummy val]} {
		    set val [ctoken txt " ;\t\n\r"]
		    if {[cindex $val 0] == "#" } {
	    	        error "Syntax error loading file: '$txt2'."
		    }
		}
	    }

	    return [list $param $val]
    }

    #****************************************************************
    #*# method: SetKey
    #----------------------------------------------------------------

    private method SetKey { param val } {

    	#*# 1. if value is empty: retrieve it.
	if { $val == "" } {
	    catch { set val [keylget cfgkeys $param] }

	    #*# 2. if value found: store in seqbuf.
	    if { $val != "" } {
	    	set keytbl($param) $val
	    }
	}

	return $val
    }

    #****************************************************************
    #*# method: GetSetIndex
    #----------------------------------------------------------------

    method GetSetIndex { setname } {
	return [lsearch $names $setname]
    }


    #****************************************************************
    #*# method: Get
    #----------------------------------------------------------------

    method Get { param } {
        try_eval {
	    return $keytbl($param)
	} {
	    error "Keyword $param not found."
	}
    }


    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    private variable cfgkeys
    private variable keytbl
    private variable names   ""
}


#___oOo___
