#*******************************************************************************
# E.S.O. - VLT project
#
# @(#) $Id: ctooCONFIG.tcl,v 1.52 2005/09/06 23:52:05 vltsccm Exp $
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2004-09-06  save deleted values in backup file fixed.
# rschmutz  2004-09-05  SPR VLTSW20040290: method Delete added.
# rschmutz  16/06/02  SPR VLTSW20020376: LoadLine: better regexp.
# rschmutz  27/05/02  destructor added.
# rschmutz  12/05/02  method Category added.
# rschmutz  27/09/00  File search order fixed.
# rschmutz  16/09/00  UifArchive updated (name added).
# rschmutz  14/09/00  UifLoad, UifArchive procs. added.
# rschmutz  31/08/00  support for config. sets added.
# rschmutz  05/08/00  updated to support multiple config. files,
#                      backup file and logging.
# rschmutz  20/03/00  method CfgAppend added.
# rschmutz  22/08/99  use ctooObjectExists instead of 'info objects'.
# rschmutz  30/10/98  created.
#

#************************************************************************
#   NAME
#	ctooCONFIG_ - configuration file interface
#
#   SYNOPSIS
#	ctooCONFIG cfg name [-check none]
#
#   DESCRIPTION
#	Itcl interface to the configuration stored either in:
#	the INS_ROOT, INTROOT or VLTROOT.
#
#   PUBLIC METHODS
#	ctooCONFIG cfg name [-check 0]
#           Constructor.  The configuration files that correspond
#	    to the config. set <name> are searched in the $INS_ROOT,
#	    $INTROOT and $VLTROOT directories (in that order)
#	    and then loaded.
#	    If the config. set is not described in the *CONFIG.cfg
#	    files, then files <name>.dic and <name>.cfg are searched.
#	    If option '-check 0' is used, then dictionaries are not
#	    used.
#
#	CfgAppend name
#	    Appends the contents of the configuration file
#	    <name>.cfg to the current loaded configuration.
#	
#	Name
#	    Returns the <name> of the configuration.
#
#	Category
#	    Returns the configuration category.
#
#	Get <keyword>
#	    Return the value of a configuration keyword.
#
#	Set <keyword> <value>
#	    Set a keyword value.
#
#	Delete <keyword>
#	    Delete a configuration keyword.
#
#	The following methods are provided mainly to be
#	used from a UIF that allows to edit a configuration
#	file:
#
#	GetAll ?prefix?
#	    Store all configuration keywords in the global
#	    array gvar.  The keyword names are prefixed with
#	    <prefix> or "[set name]_".
#	    This method allows entry widgets to set the 
#	    keyword values (attaching to the gvar array).
#
#	SetAll ?prefix?
#	    Set all keywords according to the values in the
#	    global array <var>.  Only array elements whose
#	    name starts with the <prefix> are considered.
#	    The default prefix is: "[set name]_".
#	    SetAll returns 1 if any keyword value was
#	    modified, 0 otherwise.
#
#	Save ?-ask?
#	    Save the configuration.
#	    If the -ask option is used, then the user is
#	    asked if he wants to save the configuration.
#
#   PUBLIC PROCEDURES
#	The procedures below are provided to be used from
#	a user interface.  The procedures manage one 
#	ctooCONFIG instance placed in the global variable
#	'cfg'.  The FITS values can be retrieved/set from
#	the gvar array with index 'cfg__<FITS.PARAMETER.NAME>'.
#
#	UifCreate name
#	    Creates a ctooCONFIG object cfg and calls GetAll.
#	    This procedure can be called in a panel when the
#	    name of the configuration is known.
#
#	UifLoad
#	    Calls GetAll after asking for confirmation, if
#	    the data was modified.
#
#	UifSave
#	    Calls SetAll and Save. 
#	    This procedure can be called from the panel
#	    Save button.
#
#	UifArchive name
#	    Calls the utility ctooConfigArchive.
#
#	UifQuit
#	    Calls SetAll and 'Save -ask' if the data was
#	    modified.  This procedure can be called from
#	    the panel <panel>Exit procedure.
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#       $INTROOT/config/*CONFIG.cfg
#       $VLTROOT/config/*CONFIG.cfg
#	    Configuration set description.
#
#	$INS_ROOT/SYSTEM/COMMON/CONFIGFILES/<app>cfg*.cfg
#       $INTROOT/config/<app>cfg*.cfg
#       $VLTROOT/config/<app>cfg*.cfg
#	    Configuration files.
#
#   ENVIRONMENT
#	INS_ROOT	Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTION
#       INS_ROOT resp. INTROOT must be defined to be able to read
#       a configuration stored in the corresponding location.
#
#   EXAMPLES
#	ctooCONFIG cfg XXXX
#	set wsenv [cfg Get INS.CON.WSENV]
#	
#   SEE ALSO
#	ctooCONFIG(8) ctooConfigGet(1) ctoo(5)
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: ctooCONFIG
#------------------------------------------------------------------------

class ctooCONFIG {
    inherit ctooOBJ_CONTAINER

    #****************************************************************
    #*# constructor
    #
    # 1. load ctoo configuration
    # 2. search config. name in ccfg
    # 3. load config. set data
    #    3.1 load config. file names from ccfg
    #    3.2 load other data
    #    else
    #    3.3 search <name>.cfg
    #    3.4 load dictionary names
    #    3.5 set other data
    # 4. load config. files
    #----------------------------------------------------------------

    constructor { newName args } {
        ctooOBJ_BASE::constructor -name $newName -objType CONFIG
	eval configure $args
	global env

	#*# 1. load ctoo configuration
	if { $ccfg == "" } {
	    set ccfg [ctooCCONFIG $this.ccfg]
	}

	#*# 2. search config. name in ccfg
	set name $newName
	set i [$ccfg GetSetIndex $name]

	#*# 3. load config. set data
	set files ""
	set perms ""
	if { $i != -1 } {
	    #*# 3.1 load config. file names from ccfg
	    set files1 ""
	    set j 1
	    while {1} {
	    	try_eval {
		    set filepat [$ccfg Get CONFIG.SET$i.FILE$j]
		} {
		    break
		}
	    	set files1 [SearchFiles $filepat]
	        if { $files1 == "" } {
	            error "Configuration file(s) $filepat not found."
	        }
		lvarcat files $files1
		set perm 644
		catch { set perm [$ccfg Get CONFIG.SET$i.PERM$j] }
		lvarcat perms [replicate "$perm " [llength $files1]]
		incr j
	    }

	    #*# 3.2 load other data
	    set category ""
	    catch { set category [$ccfg Get CONFIG.SET$i.CATEGORY] }
	    set dict [$ccfg Get CONFIG.SET$i.DICT]
	    if { $dict == "" } {
	    	error "No dictionaries specified in CONFIG.SET$i.DICT."
	    }
	    set log "F"
	    set backup "F"
	    set bakdir "$env(VLTDATA)/config"
	    catch { set bakdir "$env(INS_ROOT)/SYSTEM/COMMON/CONFIGFILES" }
	    catch { set log    [$ccfg Get CONFIG.CON.LOG] }
	    catch { set backup [$ccfg Get CONFIG.CON.BACKUP] }
	    catch { set bakdir [$ccfg Get CONFIG.CON.BAKDIR] }
	    catch { set log    [$ccfg Get CONFIG.SET$i.LOG] }
	    catch { set backup [$ccfg Get CONFIG.SET$i.BACKUP] }
	    catch { set bakdir [$ccfg Get CONFIG.SET$i.BAKDIR] }
	    if { $log    == "T" } { set log    1 } else { set log    0 }
	    if { $backup == "T" } { set backup 1 } else { set backup 0 }

	} else {

	    #*# 3.3 search <name>.cfg
	    set files [SearchFiles [cconcat $name ".cfg"]]
	    if { $files == "" } {
	        error "Configuration $name not found."
	    }
	    if { [llength $files] != 1 } {
	        error "Too many configuration files: $files1."
	    }
	    set perms "644"

	    #*# 3.4 load dictionary names
	    set dict ""
	    set dictfile [file join [file dirname $files] "$name.dic"]
	    if { ![file isfile $dictfile] } {
	    	error "Configuration file $name.dic not found."
	    }
            for_file line $dictfile {
                set line [string trim $line]
                if { $line == "" } { continue }
                if { [cindex $line 0] == "#" } { continue }
                lappend dict $line
            }

	    #*# 3.5 set other data
	    set log 0
	    set backup 0
	    set bakdir ""
	}

	#*# 4. load dictionaries
	if { $check } {
	    ctooDICT $this.dict -dict "PAF $dict"
	}

	#*# 5. load config. files
	set i 0
	foreach file $files {
	    set paftbl($i) ""
	    for_file txt $file {
	        set line [LoadLine $txt]
	        if { "$line" != "" } {
	            lassign $line key val
		    if { $val != "##DELETE##" } {
		        if { "[cindex $val 0]" == "\"" && "[cindex $val end]" == "\"" } {
		            set val [csubstr $val 1 len-2]
		        }
			if { $check } {
			    $this.dict Check $key $val
			}
			if { [csubstr $key 0 4] == "PAF." } {
			    lappend paftbl($i) [list [translit "." "`" $key] $val]
			} else {
		            set keytbl($key) [list $val $i 0 0]
			}
		    } else {
		    	catch { unset keytbl($key) }
		    }
	        }
	    }
	    incr i
	}
    }


    #****************************************************************
    # destructor
    #----------------------------------------------------------------

    destructor {
    	catch { delete object $this.dict }
    }


    #****************************************************************
    #*# method: CfgAppend
    #
    # 1. search for <appendName>.cfg
    # 2. load config. file
    #----------------------------------------------------------------

    public method CfgAppend { appendName } {

	#*# 1. search for <appendName>.cfg
	set files1 [SearchFiles [cconcat $appendName ".cfg"]]
	if { $files1 == "" } {
	    error "Configuration file $appendName.cfg not found."
	}
	if { [llength $files1] != 1 } {
	    error "Too many configuration files: $files1."
	}

	#*# 2. load config. file
	set i [llength $files]
	foreach file $files1 {
	    lappend files $file
	    lappend perms "644"
	    set paftbl($i) ""
	    for_file txt $file {
	        set line [LoadLine $txt]
	        if { "$line" != "" } {
	            lassign $line key val
		    if { $val != "##DELETE##" } {
		        if { "[cindex $val 0]" == "\"" && "[cindex $val end]" == "\"" } {
		            set val [csubstr $val 1 len-2]
		        }
			if { $check } {
			    $this.dict Check $key $val
			}
			if { [csubstr $key 0 4] == "PAF." } {
			    lappend paftbl($i) [list [translit "." "`" $key] $val]
			} else {
		            set keytbl($key) [list $val $i 0 0]
			}
		    } else {
		    	catch { unset keytbl($key) }
		    }
	        }
	    }
	    incr i
	}
    }

    #****************************************************************
    #*# method: LoadLine
    #----------------------------------------------------------------

    protected method LoadLine { txt } {

	    #*# 1. ignore empty lines and comments
	    set txt2 $txt
	    set txt [string trim $txt]
	    if { $txt == "" || [cindex $txt 0] == "#" } {
	    	return ""
	    }

	    #*# 2. get parameter name
	    set param [ctoken txt " ;\t\n\r"]

	    #*# 3. get parameter value
	    set val ""
	    set txt [string trim $txt]
	    if { [cindex $txt 0] != ";" } {
	    	# ((escaped double quote )? + string without quotes)*
	        if {![regexp "\"(((\\\\\")?\[^\"\]*)*)\"" $txt dummy val]} {
		    set val [ctoken txt " ;\t\n\r"]
		    if {[cindex $val 0] == "#" } {
	    	        error "Syntax error loading file: '$txt2'."
		    }
		}
	    }

	    return [list $param $val]
    }

    #****************************************************************
    #*# method: Name
    #----------------------------------------------------------------

    method Name {} {
	return $name
    }

    #****************************************************************
    #*# method: Category
    #----------------------------------------------------------------

    method Category {} {
	return $category
    }

    #****************************************************************
    #*# method: Get
    #----------------------------------------------------------------

    method Get { param } {
        try_eval {
	    set val [lindex $keytbl($param) 0]
	} {
	    error "Keyword $param not found."
	}
	if { $val == "##DELETE##" } {
	    error "Keyword $param not found."
	}

	return $val
    }

    #****************************************************************
    #*# method: Set
    #----------------------------------------------------------------

    method Set { param val } {
	if { "[cindex $val 0]" == "\"" && "[cindex $val end]" == "\"" } {
	    set val [csubstr $val 1 len-2]
	}
	try_eval {
	    set keyl $keytbl($param)
	} {
	    set keyl [list "" 0 1 0]
	}
	lassign $keyl prevVal cfg changed delete
	if { !$changed } {
	    if { $val != $prevVal } {
	        set changed 1
	    }
	}
	if { $check } {
	    $this.dict Check $param $val
	}
	set keytbl($param) [list $val $cfg $changed 0]
	
	return $changed
    }

    #****************************************************************
    #*# method: Delete
    #----------------------------------------------------------------

    method Delete { param } {
	try_eval {
	    set keyl $keytbl($param)
	} {
	    return 0
	}
	set val "##DELETE##"
	lassign $keyl prevVal cfg changed delete
	if { !$changed } {
	    if { $val != $prevVal } {
	        set changed 1
	    }
	}
	set keytbl($param) [list $val $cfg $changed 0]
	
	return $changed
    }

    #****************************************************************
    #*# method: GetAll
    #----------------------------------------------------------------

    method GetAll { {prefix ""} } {
	global ::gvar

	#*# 1. retrive each keyword value and store it in gvar.
	if { "$prefix" == "" } {
	    set prefix "[set name]_"
	}
	for_array_keys key keytbl {
	    set gname [cconcat $prefix $key]
	    set val [lindex $keytbl($key) 0]
	    set gvar($gname) $val
	}
    }

    #****************************************************************
    #*# method: SetAll
    #
    # 1. init: no changes, get default prefix
    # 2. mark all keywords for deletion
    # 3. foreach gvar variable:
    #    4. if prefix matches:
    #       5. retrieve value, get keyword name
    #       6. store new value
    # 7. mark to delete keywords not set
    #----------------------------------------------------------------

    method SetAll { {prefix {}} } {
	global ::gvar

	#*# 1. init: no changes, get default prefix
	set changed 0
	if { "$prefix" == "" } {
	    set prefix "[set name]_"
	}
	set len [clength $prefix]

	#*# 2. mark all keywords for deletion
	#      (Set resets the delete flag)
	for_array_keys key keytbl {
	    set keytbl($key) [lreplace $keytbl($key) 3 3 1]
	}

	#*# 3. foreach gvar variable:
	for_array_keys gname gvar {

	    #*# 4. if prefix matches:
	    set pre [csubstr $gname 0 $len]
	    if { "$pre" == "$prefix" } {

		#*# 5. retrieve value, get keyword name
		set val $gvar($gname)
		if { "[cindex $val 0]" == "\"" && "[cindex $val end]" == "\"" } {
		    set val [csubstr $val 1 len-2]
		}
		set param [csubstr $gname $len end]

		#*# 6. store new value
		if {[Set $param $val]} {
		    set changed 1
		}
	    }
	}

	#*# 7. mark to delete keywords not set
	for_array_keys key keytbl {
	    lassign $keytbl($key) val cfg dummy delete
	    if { $delete } {
	    	set keytbl($key) [list "##DELETE##" $cfg 1 0]
	    }
	}

	return $changed
    }

    #****************************************************************
    #*# method: Save
    #
    # 1. if -ask: ask confirmation from the user
    # 2. check dictionary is loaded
    # 3. create keyword lists for each config. file
    # 4. save each config. file
    # 5. update backup file
    # 6. reset all changed flags
    #----------------------------------------------------------------

    method Save { {option {}} } {

	#*# 1. if -ask: ask confirmation from the user
	if { "$option" == "-ask" } {
	    if {[tk_dialog .ctooConfigSave {Save Configuration} \
	      	    {Save the modified configuration or discard the changes ?} \
		    warning 0 Save {Discard changes}]} {
		return -1
	    }
	} elseif { "$option" != "" } {
	    error "Wrong option $ask (only -ask is allowed)."
	}

	#*# 2. check dictionary is loaded
	if { !$check } {
	    error "No dictionaries available.  Save config. files not allowed."
	}
	set date [clock format [clock seconds] -format %Y-%m-%dT%H:%M:%S]

	#*# 3. create keyword lists for each config. file
	loop i 0 [llength $files] {
	    set change($i) 0
	}
	for_array_keys key keytbl {
	    lassign $keytbl($key) val i changed
	    lappend keys($i) [list $key $val]
	    if { $changed } {
		set change($i) 1
		if { $log } {
		    #exec logAddFits -s P NO_CHECK $key $val
		    exec tooLogComment "[translit "." " " $key] = $val /"
		}
	    }
	}

	#*# 4. save each config. file
	loop i 0 [llength $files] {
	    set file [lindex $files $i]
	    set perm [lindex $perms $i]
	    if { $change($i) } {
	    	set txt \
{#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id}
		append txt {$"
#
#************************************************************************

}
		keylset paftbl($i) PAF`LCHG`NAME $applname
		keylset paftbl($i) PAF`LCHG`DAYTIM $date
		foreach keyl $paftbl($i) {
		    lassign $keyl key val
		    append txt "[$this.dict Format [translit "`" "." $key] $val]\n"
		}
		append txt "\n"
		set tmpkeys [lsort -dictionary -index 0 $keys($i)]
		foreach keyl [lsort -dictionary -index 0 $keys($i)] {
		    lassign $keyl key val
		    if { $val != "##DELETE##" } {
		    	append txt "[$this.dict Format $key $val]\n"
		    }
		}
		append txt "\n# ___oOo___"
		write_file $file $txt
		catch { chmod 0$perm $file }
	    }
	}
	unset keys

	#*# 5. update backup file
	if { $backup } {
	    set bakfile [file join $bakdir "$name.cfglog"]
	    set bakheader \
{#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id}
	    append bakheader {$"
#
#************************************************************************

}
	    set baknew 0
	    set baklist ""
	    set bakupdate 0
	    if { ![file exists $bakfile] } {
	        set baknew 1
		set bakpaf $paftbl(0)
		keylset bakpaf PAF`CRTE`NAME $applname
		keylset bakpaf PAF`CRTE`DAYTIM $date
		foreach keyl $bakpaf {
		    lassign $keyl key val
		    append bakheader "[$this.dict Format [translit "`" "." $key] $val]\n"
		}
	    }
	    for_array_keys key keytbl {
	        lassign $keytbl($key) val cfg changed
	        if { $changed || $baknew } {
		    # we may have deleted keywords: allow this
		    lappend baklist "[$this.dict Format $key $val 1]"
		    set bakupdate 1
		}
	    }
	    if { $bakupdate } {
		set bakfd [open $bakfile "a+"]
		if { $baknew } {
		    puts $bakfd $bakheader
		} else {
		    puts $bakfd "[format %-21s PAF.LCHG.NAME]   \"$applname\";"
		    puts $bakfd "[format %-21s PAF.LCHG.DAYTIM]   \"$date\";"
		}
		puts $bakfd [join [lsort -dictionary $baklist] "\n"]
	    	puts $bakfd "# End $date.\n"
		close $bakfd
	    }
	    unset baklist
	}

	#*# 6. reset all changed flags
	for_array_keys key keytbl {
	    set keytbl($key) [lreplace $keytbl($key) 2 2 0]
	}

	return 0
    }

    #****************************************************************
    #*# method: UifCreatePages
    #----------------------------------------------------------------

    proc UifCreatePages { path {name {}} args } {
    }

    #****************************************************************
    #*# method: UifCreate
    #----------------------------------------------------------------

    proc UifCreate { name } {
	global ::cfg

	catch { delete object ::cfg }
	ctooCONFIG ::cfg $name
	cfg GetAll cfg__
    }

    #****************************************************************
    #*# proc: UifLoad
    #----------------------------------------------------------------

    proc UifLoad {} {
	global ::cfg
	
	if {[ctooObjectExists ::cfg]} {
	    if {[tk_messageBox -type okcancel -title {Load Configuration} \
	      	    -message {Any modifications will be lost.  Re-load current configuration ?} \
		    -icon question -default ok] == "cancel" } {
		return -1
	    }
	    cfg GetAll cfg__
	} else {
	    tk_dialog .ctooTmpLoad "Load" "No data.  Nothing loaded." info 0 OK
	}

	return 0
    }

    #****************************************************************
    #*# proc: UifSave
    #----------------------------------------------------------------

    proc UifSave {} {
	global ::cfg
	
	if {[ctooObjectExists ::cfg]} {
	    cfg SetAll cfg__
	    cfg Save
	} else {
	    tk_dialog .ctooTmpSave "Save" "No data.  Nothing saved." info 0 OK
	}
    }

    #****************************************************************
    #*# proc: UifArchive
    #----------------------------------------------------------------

    proc UifArchive { name } {
    	system "ctooConfigArchive -config \"$name\" &"
	sleep 1
    }

    #****************************************************************
    #*# method: UifQuit
    #----------------------------------------------------------------

    proc UifQuit {} {
	global ::cfg

	if {[ctooObjectExists ::cfg]} {
	    if {[cfg SetAll cfg__]} {
		cfg Save -ask
	    }
	}
    }

    #****************************************************************
    #*# method: SearchFiles
    #
    # 1. if file exists: return it
    # 2. build list of locations to search
    # 3. search in each location
    # 4. remove duplicate entries
    #----------------------------------------------------------------

    protected method SearchFiles { filename } {
        global env

	#*# 1. if file exists: return it
        if {[file isfile $filename]} {
            return $filename
        }

	#*# 2. build list of locations to search
	set dirs {$INS_ROOT/SYSTEM/COMMON/CONFIGFILES $INTROOT/config $VLTROOT/config}
        catch { regsub -all {\$INS_ROOT} $dirs $env(INS_ROOT) dirs }
        catch { regsub -all {\$INTROOT}  $dirs $env(INTROOT)  dirs }
        catch { regsub -all {\$VLTROOT}  $dirs $env(VLTROOT)  dirs }
        catch { regsub -all {\$VLTDATA}  $dirs $env(VLTDATA)  dirs }

	#*# 3. search in each location
	set paths ""
        foreach dir $dirs {
            set path  [file join $dir $filename]
	    lvarcat paths [lsort [glob -nocomplain $path]]
        }

	#*# 4. remove duplicate entries
	#
	#   If file xxxxCONFIG.cfg is already present in $INS_ROOT,
	#   then the same file in $INTROOT should not be loaded.
	#
	set paths2 ""
	set tails  ""
	foreach path $paths {
	    set pathtail [file tail $path]
	    if {![lcontain $tails $pathtail]} {
		lappend paths2 $path
		lappend tails  $pathtail
	    }
	}

	return $paths2
    }


    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable category ""
    public variable check    1
    public variable dict     ""
    public variable log      0
    public variable backup   0
    public variable bakdir   ""
    public variable applname "Config. Tool"

    private variable files   ""
    private variable perms   ""
    private variable keytbl
    private variable paftbl

    private common   ccfg    ""
    private common   cfgfile ""
}


#___oOo___
