#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ctooDBCFG.tcl,v 1.52 2005/09/06 23:52:06 vltsccm Exp $"
#
# ctooDBCFG.tcl
#
# who       when      what
# --------  --------  ---------------------------------------------
# rschmutz  10/12/02  VLTSW20020537: accept exponents in doubles.
# rschmutz  11/05/02  PutsTableAttr added.
# rschmutz  09/12/01  added str3/str4 to AddFile.
# rschmutz  23/05/01  Open: delete file before open (to avoid perm. failures).
# rschmutz  30/04/01  method Append added.
# rschmutz  08/08/99  created
#

#************************************************************************
#   NAME
#	ctooDBCFG - class to create a *.dbcfg file.
# 
#   SYNOPSIS
#	ctooDBCFG dbcfg ?filename?
# 
#   PARENT CLASS
#	-
#
#   DESCRIPTION
#	ctooDBCFG provides methods to create a *.dbcfg file.
#
#   PUBLIC METHODS
#	Open filename
#	    Open a new *.dbcfg file.
#	Append filename
#	    Open an existing *.dbcfg file for append.
#	PutsCwp cwp
#	    Set the current working point, writing it to the
#	    *.dbcfg file. 
#	PutsPoint point
#	    Set the current point relative to the cwp.
#	PutsScalar name value
#	    Set the scalar attribute name to value in the 
#	    current point.
#	PutsVector name value_list
#	    Set a vector attribute the specified values.
#	PutsTable name value_list ?rec?
#	    Set a table attribute to the specified values.
#	    The value_list must be a list of lists, where
#	    each main entry contains the values of each
#	    record.  If rec is specified, then the data is
#	    stored starting with the given record number.
#	PutsTableAttr name rec field value
#	    Set a table attribute name(rec,field) to the
#	    specified value.
#	AddFile name ?str1 str2? ?str3 str4?
#	    Add the contents of the <name>.dbcfg file to
#	    the current *.dbcfg file.
#	    If str1 and str2 are supplied, then str1 is
#	    replaced with str2 (with regsub).
#	    If str3 and str4 are supplied, then str3 is
#	    replaced with str4 (with regsub).
#	Close
#	    Close *.dbcfg file.
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/<name>.dbcfg
#       $INTROOT/config/<name>.dbcfg
#       $VLTROOT/config/<name>.dbcfg
#           Locations where AddFile searches for the *.dbcfg file.
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#	It responsibility of the user to call the methods in
#	the right order. I.e.: first the cwp and the relative
#	point (if necessary) have to be supplied, before 
#	attribute values can be set.
#
#   EXAMPLES
#	ctooDBCFG dbcfg
#	dbcfg Open test.dbcfg
#	dbcfg PutsCwp "@lxxics1:"
#	dbcfg PutsPoint "<alias>FILT"
#	dbcfg PutsScalar device FILT
#	dbcfg PutsScalar prefix "INS.FILT1"
#	dbcfg Close
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#/*

#************************************************************************
#*# class: ctooDBCFG
#------------------------------------------------------------------------

class ctooDBCFG {

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { {filename ""} } {
	if { "$filename" != "" } {
	    Open $filename
	}
    }


    #****************************************************************
    #*# destructor
    #----------------------------------------------------------------

    destructor {
	Close
    }


    #****************************************************************
    #*# method: Open
    #----------------------------------------------------------------

    method Open { filename } {
	Close
	file delete $filename
	set fd [open $filename w]
    }


    #****************************************************************
    #*# method: Append
    #----------------------------------------------------------------

    method Append { filename } {
	Close
	set fd [open $filename a]
    }


    #****************************************************************
    #*# method: Close
    #----------------------------------------------------------------

    method Close {} {
	if { $fd != "" } {
	    close $fd
	    set fd ""
	}
    }


    #****************************************************************
    #*# method: PutsCwp
    #----------------------------------------------------------------

    method PutsCwp { cwp } {
	puts $fd "<CWP>: $cwp"
    }


    #****************************************************************
    #*# method: PutsPoint
    #----------------------------------------------------------------

    method PutsPoint { point } {
	puts $fd "  <POINT>: <relative>$point"
    }


    #****************************************************************
    #*# method: PutsScalar
    #----------------------------------------------------------------

    method PutsScalar { name value } {
	puts $fd "    <ATTRIBUTE>: $name  <TYPE>: Scalar"
	puts $fd "      [_SetValue $value]"
    }


    #****************************************************************
    #*# method: PutsVector
    #----------------------------------------------------------------

    method PutsVector { name values } {
	set recs1 [expr {[llength $values] - 1}]
	puts $fd "    <ATTRIBUTE>: $name  <TYPE>: Vector  <REC>: 0 - $recs1"
	foreach v $values {
	    puts $fd "      [_SetValue $v]"
	}
    }


    #****************************************************************
    #*# method: PutsTable
    #----------------------------------------------------------------

    method PutsTable { name values {rec 0} } {
	set recs1 [expr {$rec + [llength $values] - 1}]
	set fields [llength [lindex $values 0]]
	set fields1 [expr {$fields - 1}]
	puts $fd "    <ATTRIBUTE>: $name  <TYPE>: Table  <REC>: $rec - $recs1  <FIELDS>: 0 - $fields1"
	foreach rec $values {
	    set i 0
	    puts -nonewline $fd "      "
	    foreach field $rec {
		puts -nonewline $fd "[_SetValue $field] "
		incr i
	    }
	    puts $fd ""
	    if { $i != $fields } {
		error "Record does not contain $fields fields (only $i)."
	    }
	}
    }


    #****************************************************************
    #*# method: PutsTableAttr
    #----------------------------------------------------------------

    method PutsTableAttr { name rec field value } {
	puts $fd "    <ATTRIBUTE>: [set name]($rec,$field)  <TYPE>: Table  <REC>: $rec - $rec  <FIELDS>: $field - $field"
	puts $fd "      [_SetValue $value]"
    }


    #****************************************************************
    #*# method: AddFile
    #----------------------------------------------------------------

    method AddFile { name {str1 {}} {str2 {}} {str3 {}} {str4 {}} } {
        global env

        #*# 1. search for <name>.dbcfg
        set path ""
        foreach {envvar tmppath} { INS_ROOT "/SYSTEM/COMMON/CONFIGFILES/" \
                                INTROOT "/config/" \
                                VLTROOT "/config/" } {
            if {[catch { set envvar $env($envvar) }]} {
                continue
            }
            set dbcfgFile [cconcat $envvar $tmppath $name .dbcfg]
            if {[file exists $dbcfgFile]} {
                set path "[cconcat $envvar $tmppath]"
                break
            }
        }
        if { "$path" == "" } {
            error "File $name.dbfg not found."
        }

        #*# 2. load file
	set text [read_file $dbcfgFile]
	if { "$str1" != "" } {
	    regsub -all $str1 $text $str2 text
	}
	if { "$str3" != "" } {
	    regsub -all $str3 $text $str4 text
	}
	puts $fd $text
    }


    #****************************************************************
    #*# method: _SetValue
    #----------------------------------------------------------------

    protected method _SetValue { value } {
	set v $value
	if { "$v" == "" } {
	    return "\"\""
	} elseif { "[ctoken v "01234567890.+-eE"]" == "" && "$v" == "" } {
	    return $value
	} else {
	    return "\"$value\""
	}
    }


    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    protected variable fd ""
}


#************************************************************************
# ___oOo___
