#*******************************************************************************
# E.S.O. - VLT project
#
# @(#) $Id: ctooDICT.tcl,v 1.52 2005/09/06 23:52:07 vltsccm Exp $
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-09-06  VLTSW20050287: env. name added to cache filename.
# rschmutz  2004-09-06  Format: accept deleted keyword.
# alongino  27/08/03  VLTSW20030361: bug in GetEntry
# rschmutz  10/12/02  VLTSW20020537: accept exponents in doubles.
# rschmutz  27/05/02  use cache to speed-up dictionary load.
# rschmutz  11/05/02  GetEntry: return also class.
# rschmutz  07/12/01  method GetEntry added.
# rschmutz  26/04/01  bug fix: allow subsystems that end with a digit.
# rschmutz  07/08/00  created.
#

#************************************************************************
#   NAME
#	ctooDICT - data interface dictionary
#
#   SYNOPSIS
#	ctooDICT dict -dict <dictionaries>
#
#   DESCRIPTION
#	Interface to a set of ESO-VLT-DIC.<name> dictionaries,
#	to allow checking the validity of keywords.
#
#   PUBLIC METHODS
#	ctooDICT dict [-dict <dictionaries>]
#           Constructor.  Loads the specified dictionaries.
#
#	Load <dictionaries>
#	    Load specified dictionaries.
#
#	GetEntry <key>
#	    Returns the list: {<class> <type> <format> <unit> <comment>}
#
#	Check <key> <value>
#	    Checks the validity of a keyword value.
#	    If ok, returns the value; otherwise raises an error.
#
#	Format <key> <value> [<allow_deleted_flag>]
#	    Return a formatted PAF line with the given keyword.
#
#   FILES
#       $INS_ROOT/SYSTEM/Dictionary/ESO-VLT-DIC.<name>
#       $INTROOT/config/ESO-VLT-DIC.<name>
#       $VLTROOT/config/ESO-VLT-DIC.<name>
#	    Dictionaries.
#
#   ENVIRONMENT
#	INS_ROOT	Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTION
#       INS_ROOT resp. INTROOT must be defined to be able to read
#       a configuration stored in the corresponding location.
#
#   EXAMPLES
#	ctooDICT dict -dict CTOO_CFG
#	dict Check CONFIG.CON.LOG T
#	
#   SEE ALSO
#	ctoo(5)
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: ctooDICT
#------------------------------------------------------------------------

class ctooDICT {
    inherit ctooOBJ_BASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
        ctooOBJ_BASE::constructor -objType DICTIONARY
	eval configure $args

	#*# 1. load dictionaries
	if { $dict != "" } {
	    Load $dict
	}
    }

    #****************************************************************
    #*# method: Load
    #----------------------------------------------------------------

    protected method Load { newDict } {
        global env

	#*# 1. init. cache info.
	set envname "ctoo"
	catch { set envname $env(RTAPENV) }
	set cachefile [translit " " "-" $newDict]
	set cachefile [file join $env(VLTDATA) tmp [cconcat $envname "." $cachefile ".dic.cache"]]
	set cachetime 0
	if { [file exists $cachefile] } {
	    set cachetime [file mtime $cachefile]
	}
	set useCache 1

	#*# 2. search dictionaries
    	set dict $newDict
	set dicfileList ""
	foreach dic $dict {
	    set found 0
	    foreach {envvar relpath} { INS_ROOT "/SYSTEM/Dictionary/" \
				       INTROOT "/config/" \
				       VLTROOT "/config/" } {
	        try_eval { set envvar $env($envvar) } {
		    continue
	        }
	        set dicfile [cconcat $envvar $relpath "ESO-VLT-DIC.$dic"]
		if {[file isfile $dicfile]} {
		    set found 1
		    break
		}
	    }
	    if { !$found } {
	        error "No dictionary $dic found."
	    }
	    lappend dicfileList $dicfile

	    #*# 3. compare timestamp with cache
	    if { [file mtime $dicfile] > $cachetime } {
	    	set useCache 0
	    }
	}

	#*# 4. if cache is newer: load it
	if { $useCache } {
	    array set keytbl [read_file $cachefile]
	    return
	}

	#*# 5. load dictionaries
	foreach dicfile $dicfileList {
	    set param   ""
	    set kclass  ""
	    set type    ""
	    set format  ""
	    set unit    ""
	    set comment ""
	    for_file txt $dicfile {
	        set line [LoadLine $txt]
	        if { "$line" != "" } {
	            lassign $line key val
		    switch $key {
		    	"Parameter Name" { set param   [translit " " "." $val] }
			"Class"          { set kclass  [translit "|" " " $val] }
			"Type"           { set type    $val }
			"Value Format"   { set format  $val }
			"Unit"		 { set unit    $val }
			"Comment Format" -
			"Comment Field"  {
			    set comment $val
			    if { $param != "" } {
				regsub -all i $param "" param
			    	set keytbl($param) [list $kclass $type $format $unit $comment]
				set param   ""
				set kclass  ""
				set type    ""
				set format  ""
				set unit    ""
				set comment ""
			    }
			}
		    }
		}
	    }
	}

	#*# 6. save cache
	file delete $cachefile
	write_file $cachefile [array get keytbl]
	chmod 0666 $cachefile
    }

    #****************************************************************
    #*# method: LoadLine
    #----------------------------------------------------------------

    protected method LoadLine { txt } {

	    #*# 1. ignore empty lines and comments
	    set txt2 $txt
	    set txt [string trim $txt]
	    if { $txt == "" || [cindex $txt 0] == "#" } {
	    	return ""
	    }

	    #*# 2. get parameter name
	    if {![regexp {^([^:]+):[ \t]*(.+)$} $txt dummy key val]} {
	    	return ""
	    }

	    return [list [string trim $key] [string trim $val]]
    }

    #****************************************************************
    #*# method: GetEntry
    #
    #   returns list: class type format unit comment
    #----------------------------------------------------------------

    method GetEntry { key } {

	#*# 1. get dictionary entry
	set key1 $key
        # first check the key as it is (e.g. INS.MIRR2.DATA2)
	if {[catch {
	    set keyl $keytbl($key1)
	}]} {
            # try by removing intermediate indexes (INS.MIRR2.DATA2 -> INS.MIRR.DATA2)
	    regsub -all {[0-9]+\.} $key1 "." key1
            try_eval {
	        set keyl $keytbl($key1)
	    } {
                # try by removing trailing index (INS.MIRR2.DATA2 -> INS.MIRR.DATA)
   	        regsub -all {[0-9]+$} $key1 "" key1
	        if {[catch {
	            set keyl $keytbl($key1)
	        }]} {
	            error "Keyword $key not found in dictionary."
	        }
	    }
	}

	return $keyl
    }

    #****************************************************************
    #*# method: Check
    #----------------------------------------------------------------

    method Check { key val } {

	#*# 1. get dictionary entry
	set keyl [GetEntry $key]
        set type [lindex $keyl 1]

	#*# 2. check value
	switch $type {
	    "logical" {
	    	if { $val != "T" && $val != "F" } {
		    error "Keyword $key: value '$val' is not a logical."
		}
	    }
	    "integer" {
	    	set val1 $val
	    	if { [ctoken val1 "+-0123456789"] != "" } {
		    error "Keyword $key: value '$val' is not an integer."
		}
	    }
	    "double" {
	    	set val1 $val
	    	if { [ctoken val1 "+-0123456789.eE"] != "" } {
		    error "Keyword $key: value '$val' is not a double."
		}
	    }
	}
	return $val
    }

    #****************************************************************
    #*# method: Format
    #----------------------------------------------------------------

    method Format { key val {allowDeleted 0} } {

	#*# 1. get dictionary entry
	set keyl [GetEntry $key]
	lassign $keyl kclass type format unit comment
	if { $allowDeleted != 0 && $val == "##DELETE##" } {
	    set type "deleted"
	}

	#*# 2. check and format value
	set txt $val
	switch $type {
	    "logical" {
	    	if { $val != "T" && $val != "F" } {
		    error "Keyword $key: value '$txt' is not a logical."
		}
	    }
	    "integer" {
	    	set val1 $val
	    	if { [ctoken val1 "+-0123456789"] != "" } {
		    error "Keyword $key: value '$txt' is not an integer."
		}
	    	set txt [format $format $txt]
	    }
	    "double" {
	    	set val1 $val
	    	if { [ctoken val1 "+-0123456789.eE"] != "" } {
		    error "Keyword $key: value '$txt' is not a double."
		}
	    	set txt [format $format $txt]
	    }
	    default {
	    	#set txt [format \"$format\" $txt]
	    	set txt \"$txt\"
	    	# \""
	    }
	}

	#*# 3. return formatted line
	if { $unit != "" } {
	    append comment " ($unit)"
	}
	set txt "[format %-23s $key] $txt;"
	set sep [replicate "\t" [expr {5 - [clength $txt] / 8}]]
	if { $sep == "" } { set sep " " }
	return "$txt$sep# $comment"
    }


    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable dict ""
    private variable keytbl
}


#___oOo___
