#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ctooOBJ.tcl,v 1.52 2005/09/06 23:52:06 vltsccm Exp $"
#
# ctooOBJ.tcl
#
# who       when      what
# --------  --------  ---------------------------------------------
# rschmutz  14/05/00  ObjDebugShow: show TCL var. and type
#		      (heritage does not work well, not sure why).
# rschmutz  05/02/00  created.
#

#************************************************************************
#   NAME
#	ctooOBJ_REPOSITORY - object repository
#	ctooOBJ_CONTAINER  - object container
#	ctooOBJ_BASE       - object base
#
#   SYNOPSIS
#	ctooOBJ_REPOSITORY cfg -config name
# 
#   PARENT CLASS
#	ctooOBJ_REPOSITORY: ctooOBJ_CONTAINER.
#	ctooOBJ_CONTAINER:  ctooOBJ_BASE.
#	ctooOBJ_BASE:       -
#
#   DESCRIPTION
#	The classes ctooOBJ_REPOSITORY, ctooOBJ_CONTAINER and
#	ctooOBJ_BASE allow to store a configuration in a
#	'repository', a hierarchical tree of Tcl objects.
#	This is useful when the configuration describes a
#	hierarchical system, e.g. the instrumentation software
#	consists of an Observation Software (OS), that acceses
#	different subsystems (TCS, ICS, DCS).  Each subsystem
#	consists of one or more processes and panels.
#
#	ctooOBJ_REPOSITORY is the frontend to the hierarchical
#	container of Tcl objects.  The contained objects must be 
#	derived either fom ctooOBJ_BASE or from ctooOBJ_CONTAINER.
#	A ctooOBJ_BASE object does not contain any other objects,
#	while a ctooOBJ_CONTAINER object can contain other objects,
#	that in turn are derived from either ctooOBJ_BASE or 
#	ctooOBJ_CONTAINER.  This allows to store several objects
#	in the hierarchical tree, all accessible from the the top
#	ctooOBJ_REPOSITORY.
#
#	Objects are added and removed to/from the ctooOBH_REPOSITORY
#	or to/from a ctooOBJ_CONTAINER with the AddObj and DeleteObj
#	methods.  Each object is assigned a unique numeric identifier
#	when it is added to the repository.
#
#	Objects in the repository can be retrieved by name (LookupName).
#	The list of objects contained in a ctooOBJ_CONTAINER object
#	can be retrieved with the Contents method.
#
#   PUBLIC METHODS
#	ctooOBJ_BASE::Name ?name?
#	    Sets/gets the object name.
#	ctooOBJ_BASE::Type ?type?
#	    Sets/gets the object type.
#	ctooOBJ_BASE::Id
#	    Gets the unique identifier (ro).
#	ctooOBJ_BASE::Parent
#	    Gets the parent object (ro).
#	ctooOBJ_BASE::Repository
#	    Gets the repository that contains the obj (ro).
#
#	ctooOBJ_CONTAINER::AddObj obj1 ?... objN?
#	    Adds the objects obj1 to objN to the container.
#
#	ctooOBJ_CONTAINER::DeleteObj obj1 ?... objN?
#	    Removes the objects obj1 to objN from the container.
#
#	ctooOBJ_CONTAINER::MoveObj obj pos
#	    Moves an existing object from it's current position
#	    within the object list to position <pos>.
#
#	ctooOBJ_CONTAINER::ExecObj name cmd args
#	    Executes the method cmd of the object with name <name> 
#	    stored in the container.  Returns the result of the
#	    method, or an error, if the object is not found or if
#	    multiple objects with the same name were found.
#
#	ctooOBJ_CONTAINER::Contents ?type? ?depth?
#	    Returns the objects contained in the container.
#	    If the type is specified, then only objects of the
#	    given type are returned (Type or the Itcl type).
#	    If the depth is specified, then the container is
#	    searched recursively until the specified depth
#	    (default depth = 0, i.e. no recursive search).
#
#	ctooOBJ_CONTAINER::ContentsR ?type? ?depth?
#	    Returns the objects contained in the container
#	    in reverse order of how they are registered.
#	    If the type is specified, then only objects of the
#	    given type are returned (Type or the Itcl type).
#	    If the depth is specified, then the container is
#	    searched recursively until the specified depth
#	    (default depth = 0, i.e. no recursive search).
#
#	ctooOBJ_CONTAINER::ContentsByName ?type? ?depth?
#	    Similar to Contents above, but returns the -name
#	    of each object found in the container.
#	    If the type is specified, then only objects of the
#	    given type are returned (Type or the Itcl type).
#	    If the depth is specified, then the container is
#	    searched recursively until the specified depth
#	    (default depth = 0, i.e. no recursive search).
#
#	ctooOBJ_CONTAINER::LookupName name ?type? ?depth?
#	    Returns the object with the given name.
#	    If the type is specified, then only objects of the
#	    given type are returned (Type or Itcl type).
#	    If the depth is specified, then the container is
#	    searched recursively until the specified depth
#	    (default depth = 0, i.e. no recursive search).
#	    If the container contains multiple objects with
#	    the same name, then all objects are returned.
#	    (equivalent to: LookupValue "-name" <name> ?type? ?depth?)
#				    
#	ctooOBJ_CONTAINER::LookupValue option value ?type? ?depth?
#	    Returns all objects whose option ("-<varName>")
#	    has the specified value.
#	    If the type is specified, then only objects of the
#	    given type are returned (Type or Itcl type).
#	    If the depth is specified, then the container is
#	    searched recursively until the specified depth
#	    (default depth = 0, i.e. no recursive search).
#				    
#	ctooOBJ_CONTAINER::LookupRValue option value ?type? ?depth?
#	    Returns all objects whose option ("-<varName>")
#	    has the specified value in the reverse order of
#	    how they are registered in the repository.
#	    If the type is specified, then only objects of the
#	    given type are returned (Type or Itcl type).
#	    If the depth is specified, then the container is
#	    searched recursively until the specified depth
#	    (default depth = 0, i.e. no recursive search).
#	
#	ctooOBJ_REPOSITORY::Config
#	    Returns the -config value.
#
#	ctooOBJ_REPOSITORY::GetNewId
#	    Internal (used only by ctooOBJ_* classes).
#	    Returns the next unique identifier.
#
#	ctooOBJ_REPOSITORY::Debug options
#	    Method provided for debugging purposes.
#	    The following options are accepted:
#	     "-objList"
#		Lists all objects in the repository tree.
#	     "-objShow"
#		Lists all objects in the repository tree
#		with all their public data (this can be
#		quite a long output).
#	     "-objShow <type> <name>"
#		Lists the object <name> of type <type>
#		with it's public data.
#	    After printing the debug information, the
#	    Tcl application exits.
#
#   RETURN VALUES
#	See individual methods above.
#
#   CAUTIONS
#	The unique identifier, parent object and repository
#	values of each object derived from ctooOBJ_BASE are
#	set by the AddObj method.  These variables should be
#	considered read-only by other software.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#/*

#************************************************************************
#*# class: ctooOBJ_BASE
#------------------------------------------------------------------------

class ctooOBJ_BASE {

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
    }

    #****************************************************************
    #*# method: ObjDebugList
    #----------------------------------------------------------------

    public method ObjDebugList { {all 0} {indent 0} } {
	ObjDebugPuts $indent "[Type] [Name]"
    }

    #****************************************************************
    #*# method: ObjDebugShow
    #----------------------------------------------------------------

    public method ObjDebugShow { {all 0} {indent 0} } {
	ObjDebugPuts $indent "[Type] [Name] \{"
	incr indent
	ObjDebugPuts $indent "[format %-15s {TCL variable}] = '$this'"
	ObjDebugPuts $indent "[format %-15s {TCL type}] = '[$this info class]'"
	#ObjDebugPuts $indent "[format %-15s {TCL heritage}] = '[$this info class] [$this info heritage]'"
	foreach var [lsort [configure]] {
	    set varName  [lindex $var 0]
	    set varValue [lindex $var 2]
	    if { "$varValue" != "" } {
	        ObjDebugPuts $indent "[format %-15s $varName] = '$varValue'"
	    }
	}
	incr indent -1
	ObjDebugPuts $indent "\}"
    }

    #****************************************************************
    #*# method: ObjDebugPuts
    #----------------------------------------------------------------

    public method ObjDebugPuts { indent text {nonewline {}} } {
	loop i 0 $indent {
	    puts -nonewline "  "
	}
	if { $nonewline != {} } {
	    puts -nonewline $text
	} else {
	    puts $text
	}
    }

    #****************************************************************
    #*# method: Set
    #----------------------------------------------------------------

    protected method _Set { var {value NOVALUE} } {
	if { "$value" != "NOVALUE" } {
	    configure -$var $value
	}
	return [cget -$var]
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable name "noName" {
    	configure -desc $name
    }
    public method Name { {value NOVALUE} } \
	{ return [_Set name $value] }

    public variable desc "noName"
    public method Desc { {value NOVALUE} } \
	{ return [_Set desc $value] }

    public variable objId 0
    public method Id { {value NOVALUE} } \
	{ return [_Set objId $value] }

    public variable objType "unknown"
    public method Type { {value NOVALUE} } \
	{ return [_Set objType $value] }

    public variable objParent "noParent"
    public method Parent { {value NOVALUE} } \
	{ return [_Set objParent $value] }

    public variable objRepository "noRepository"
    public method Repository { {value NOVALUE} } \
	{ return [_Set objRepository $value] }
}


#************************************************************************
#*# class: ctooOBJ_CONTAINER
#
#  constructor -repository <repository>
#
#  Init
#  AddObj <obj1> [<obj2> ...]
#  DeleteObj <obj1> [<obj2> ...]
#
#  Contents [<type>] [<depth>]
#  LookupName <name> [<type>] [<depth>]
#  ObjDebug
#
#------------------------------------------------------------------------

class ctooOBJ_CONTAINER {
    inherit ctooOBJ_BASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
    }

    #****************************************************************
    #*# method: Init
    #----------------------------------------------------------------

    public method Init {} {
	set ctooObjects {}
    }

    #****************************************************************
    #*# method: AddObj
    #----------------------------------------------------------------

    public method AddObj { args } {

	foreach obj $args {

	    #*# register the object, if not registered
	    set i [lsearch $ctooObjects $obj]
	    if { $i == -1 } {
		if { [$obj Id] == 0 } {
		    $obj Id [$objRepository GetNewId]
		}
		if { [string compare [$obj Parent] "noParent"] == 0 } {
		    $obj Parent $this
		}
		$obj Repository $objRepository
	        lappend ctooObjects $obj
	    }
	}

	return $ctooObjects
    }

    #****************************************************************
    #*# method: DeleteObj
    #----------------------------------------------------------------

    public method DeleteObj { args } {

	foreach obj $args {

	    #*# search the object and delete it
	    set i [lsearch $ctooObjects $obj]
	    if { $i != -1 } {
	        lvarpop ctooObjects $i
	    }
	}
    }

    #****************************************************************
    #*# method: MoveObj
    #----------------------------------------------------------------

    public method MoveObj { obj pos } {

    	set i [lsearch $ctooObjects $obj]
	if { $i == -1 } {
	    error "Object $obj not found."
	}
	lvarpop ctooObjects $i
	lvarpush ctooObjects $obj $pos
    }

    #****************************************************************
    #*# method: ExecObj
    #----------------------------------------------------------------

    public method ExecObj { objName args } {

    	#*# search object
	set objs ""
	foreach obj $ctooObjects {
	    if { [string compare $objName [$obj Name]] == 0 } {
		lappend objs $obj
	    }
	}
	if { [llength $objs] == 0 } {
	    error "Object $objName not found."
	}
	if { [llength $objs] != 1 } {
	    error "Multiple objects with name $objName found."
	}

	#*# execute method
	eval $objs $args
    }

    #****************************************************************
    #*# method: Contents
    #
    # 1. process each object:
    #    2. if type matches: append object
    #    3. get deeper objects
    #       if depth > 0
    #         foreach local object
    #           recursive call: $obj Contents type (depth - 1)
    #           add returned objects to list
    #
    #----------------------------------------------------------------

    public method Contents { {type {}} {depth 0} } {

        #*# 0. get just list ?
	if { "$type" == "" && $depth == 0 } {
	    return $ctooObjects
	}

	#*# 1. process each object:
	set ret ""
	foreach obj $ctooObjects {

	    #*# 2. if type matches: append object
	    if { $type == {} } {
	    	lappend ret $obj
	    } else {
		set isType 0
		if { [string compare $type [$obj Type]] == 0 } {
		    set isType 1
		} else {
		    catch { set isType [$obj isa $type] }
		}
		if { $isType } {
		    lappend ret $obj
		}
	    }

	    #*# 3. get deeper objects
	    if { $depth > 0 } {
	    	if { [$obj isa ctooOBJ_CONTAINER] } {
		    set ret [concat $ret [$obj Contents $type [expr {$depth - 1}]]]
		}
	    }
	}

	return $ret
    }

    #****************************************************************
    #*# method: ContentsR
    #
    # 1. reverse object list
    # 2. process each object:
    #    3. get deeper objects
    #       if depth > 0
    #         foreach local object
    #           recursive call: $obj Contents type (depth - 1)
    #           add returned objects to list
    #    4. if type matches: append object
    #
    #----------------------------------------------------------------

    public method ContentsR { {type {}} {depth 0} } {

    	#*# 1. reverse object list
	set objList ""
	foreach obj $ctooObjects {
	    lvarpush objList $obj
	}

	#*# 2. process each object:
	set ret ""
	foreach obj $objList {

	    #*# 3. get deeper objects
	    if { $depth > 0 } {
	    	if { [$obj isa ctooOBJ_CONTAINER] } {
		    set ret [concat $ret [$obj ContentsR $type [expr {$depth - 1}]]]
		}
	    }

	    #*# 4. if type matches: append object
	    if { $type == {} } {
	    	lappend ret $obj
	    } else {
		set isType 0
		if { [string compare $type [$obj Type]] == 0 } {
		    set isType 1
		} else {
		    catch { set isType [$obj isa $type] }
		}
		if { $isType } {
		    lappend ret $obj
		}
	    }
	}

	return $ret
    }

    #****************************************************************
    #*# method: ContentsByName
    #----------------------------------------------------------------

    public method ContentsByName { {type {}} {depth 0} } {

    	set names ""
    	set objList [Contents $type $depth]
	foreach obj $objList {
	    lappend names [$obj Name]
	}

	return $names
    }

    #****************************************************************
    #*# method: _LookupValue
    #----------------------------------------------------------------

    private method _LookupValue { cmd option value {type {}} {depth 0} } {

	#*# search the container
	set ret {}
	foreach obj [$cmd $type $depth] {

	    #*# if value and type matches, put the object in the list
	    if { [string compare $value [$obj cget $option]] == 0 } {
		if { $type == {} } {
		    lappend ret $obj
		} else {
		    set isType 0
		    if { [string compare $type [$obj Type]] == 0 } {
		        set isType 1
		    } else {
		    	catch { set isType [$obj isa $type] }
		    }
		    if { $isType } {
		    	lappend ret $obj
		    }
		}
	    }
	}

	return $ret
    }

    #****************************************************************
    #*# method: LookupValue
    #----------------------------------------------------------------

    public method LookupValue { option value {type {}} {depth 0} } {
    	return [_LookupValue Contents $option $value $type $depth]
    }

    #****************************************************************
    #*# method: LookupRValue
    #----------------------------------------------------------------

    public method LookupRValue { option value {type {}} {depth 0} } {
    	return [_LookupValue ContentsR $option $value $type $depth]
    }

    #****************************************************************
    #*# method: LookupName
    #----------------------------------------------------------------

    public method LookupName { objName {type {}} {depth 0} } {
    	return [_LookupValue Contents "-name" $objName $type $depth]
    }

    #****************************************************************
    #*# method: ObjDebug
    #----------------------------------------------------------------

    public method ObjDebug { {indent 0} {text ""} } {
    }

    #****************************************************************
    #*# method: ObjDebugList
    #----------------------------------------------------------------

    public method ObjDebugList { {all 0} {indent 0} } {
	ObjDebugPuts $indent "[Type] [Name]"
	if { $all } {
	    incr indent
	    foreach obj $ctooObjects {
	    	$obj ObjDebugList $all $indent
	    }
	}
    }

    #****************************************************************
    #*# method: ObjDebugShow
    #----------------------------------------------------------------

    public method ObjDebugShow { {all 0} {indent 0} } {
	ObjDebugPuts $indent "[Type] [Name] \{"
	incr indent
	ObjDebugPuts $indent "[format %-15s {TCL variable}] = '$this'"
	ObjDebugPuts $indent "[format %-15s {TCL type}] = '[$this info class]'"
	#ObjDebugPuts $indent "[format %-15s {TCL heritage}] = '[$this info class] [$this info heritage]'"
	foreach var [lsort [configure]] {
	    set varName  [lindex $var 0]
	    set varValue [lindex $var 2]
	    if { "$varValue" != "" } {
	        ObjDebugPuts $indent "[format %-15s $varName] = '$varValue'"
	    }
	}
	if { $all } {
	    foreach obj $ctooObjects {
	    	$obj ObjDebugShow $all $indent
	    }
	}
	incr indent -1
	ObjDebugPuts $indent "\}"
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    protected variable ctooObjects {}
}


#************************************************************************
#*# class: ctooOBJ_REPOSITORY
#
#  constructor -config <file>
#
#  GetNewId
#
#------------------------------------------------------------------------

class ctooOBJ_REPOSITORY {
    inherit ctooOBJ_CONTAINER

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
	Name Repository
	eval configure $args
	Id [GetNewId]
	Type Repository
	Repository $this
    }

    #****************************************************************
    #*# method: GetNewId
    #----------------------------------------------------------------

    public method GetNewId {} {
	incr ctooNextId
	return $ctooNextId
    }

    #****************************************************************
    #*# method: Debug
    #----------------------------------------------------------------

    public method Debug { options } {

    	#*# 1. if -objList: list objects
	if { [lsearch $options "-objList"] != -1 } {
	    ObjDebugList 1
	    exit
	}
    	#*# 2. if -objShow [type name]: show object(s)
	set i [lsearch $options "-objShow"]
	if { $i != -1 } {
	    incr i
	    set stype [lindex $options $i]
	    incr i
	    set sname [lindex $options $i]
	    if { "$stype" != "" && "$sname" != "" } {
	    	set obj [LookupName $sname $stype 20]
		if { "$obj" == "" } {
		    puts "ERROR: Object $stype $sname not found."
		} else {
		    [lindex $obj 0] ObjDebugShow 0
		}
	    } else {
	    	ObjDebugShow 1
	    }
	    exit
	}

    }

    #****************************************************************
    #*# method: _Set
    #----------------------------------------------------------------

    protected method _Set { var {value NOVALUE} } {
	if { "$value" != "NOVALUE" } {
	    set $var $value
	}
	return [set $var]
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable config ""
    public method Config { {value NOVALUE} } \
	{ return [_Set config $value] }

    common ctooNextId 0
}


#************************************************************************
# ___oOo___
