#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: dblcbATTRIBUTE.tcl,v 1.8 2001/06/06 08:48:07 vltsccm Exp $" 
#
#
# ATTRIBUTE
# ----------------------------------------------------------------------
# Implements an RTAP point attribute object.  An RTAP attribute contains
# is a bundle of type, security, and CE information, as well as a value.
#
#   PUBLIC ATTRIBUTES:
#	
#	-name ............ name of the attribute
#	-value ........... value of the attribute
#	-deType .......... data element type of the attribute
#	-readCateg ....... read categories mask for the attribute
#	-writeCateg ...... write categories mask for the attribute
#	-type ............ attribute type (SCALAR, VECTOR, or TABLE)
#	-usage ........... usage flags
#	-definition ...... RTAP Calculation Engine definition
#	-storage ......... either STATIC or AUTOMATIC
#	-numRecords ...... number of records (VECTOR or TABLE only)
#	-fieldNames ...... list of field names for TABLE only
#	-scanAddr ........ address to be scanned
#	-scanDevice ...... device for a scan link
#	-pollType ........ either POLL or SRBX
#	-pollTime ........ poll time for a POLL scan link
#	-deadbandType .... deadband type for a SRBX scan link
#	-deadband ........ deadband for a SRBX scan link
#	-enableFlag ...... scan link initial enable/disable state
#	-overloadable .... whether or not this attribute can be overloaded
#	-deletable ....... whether or not this attribute can be deleted
#       -sourceClass ..... class where this attribute was defined first
#
#
#   METHODS:
#
#	configure ........ used to change public attributes
#	unloadString ..... return the string that should be written to
#			   a point config file
#	queryConfigData .. return a string containing "-pubVar value"
#			   for each of the public values.
#	queryHashIdData .. return a string containing "-pubVar value"
#			   for each of the public values except the
#                          sourceClass.
#	queryDeType ...... return the deType number for this attribute
#
#
#   PROCS:
#
#	UsageString ...... returns the unload string for a set of
#			   usage flags
#	ValueString ...... returns the unload string for the
#			   attribute value
#	QuoteForDeType ... type of quote needed for a value of a de type
#
#
#   COMMON ATTRIBUTES:
#
#	validDeTypes .... list of valid data element type specifiers
#	validTypes ...... list of valid attribute type specifiers
#	formatString1 ... format string used to speed production of the
#			  unloadString - the format string is used in
#			  place of multiple 'append' calls
#	formatString2 ... format string used to speed production of the
#			  unloadString - the format string is used in
#			  place of multiple 'append' calls
#	validPollTimes .. list of valid poll times - used to form the
#			  poll type mask in scan links
#	pollTypes ....... poll types for the validPollTimes
#	deTypeNum ....... array linking deTypes names and numbers
#
#
#   PROTECTED ATTRIBUTES:
#
#	oldNumRecords ... Used so that the numRecords public variable
#			  can be left unchanged by configuring it with
#			  an empty value
#
# ----------------------------------------------------------------------
#
#   AUTHOR:  Thomas J. Accardo
#            Computerized Processes Unlimited, Inc.
#            Phone: (504) 889-2784
#            E-mail: tja@cpu.com
#
# ======================================================================
# Modified:
# who             when        what
# --------------  ----------  ----------------------------------------------
# Erik Allaert    08/03/96    adapted to [incr Tcl] 2.0
# Erik Allaert    2000-02-02  adapted to Tcl/Tk 8.0, [incr Tcl] 3.0
# ======================================================================

class ATTRIBUTE {
    # ==================================================================
    #  PROTECTED DATA
    #	oldNumRecords ... Used so that the numRecords public variable
    #			  can be left unchanged by configuring it with
    #			  an empty value
    # ==================================================================
    protected variable oldNumRecords   0


    # ==================================================================
    #  COMMON DATA
    # ==================================================================
    common validPollTimes {0 1 2 5 10 30 60 120 300 600 1800 3600}
    common pollTypes
    set pollTypes(0,type)	0;	set pollTypes(0,period)		0
    set pollTypes(1,type)	1;	set pollTypes(1,period)		1
    set pollTypes(2,type)	2;	set pollTypes(2,period)		2
    set pollTypes(5,type)	4;	set pollTypes(4,period)		5
    set pollTypes(10,type)	8;	set pollTypes(8,period)		10
    set pollTypes(30,type)	16;	set pollTypes(16,period)	30
    set pollTypes(60,type)	32;	set pollTypes(32,period)	60
    set pollTypes(120,type)	64;	set pollTypes(64,period)	120
    set pollTypes(300,type)	128;	set pollTypes(128,period)	300
    set pollTypes(600,type)	256;	set pollTypes(256,period)	600
    set pollTypes(1800,type)	512;	set pollTypes(512,period)	1800
    set pollTypes(3600,type) 	1024;	set pollTypes(1024,period)	3600

    common validTypes {SCALAR RTSCALAR VECTOR RTVECTOR TABLE RTTABLE}
    common validDeTypeList {
	INT		UINT		DOUBLE		BOOLEAN
	LOGICAL		RTLOGICAL	INT8		RTINT8
	UINT8		RTUINT8		INT16		RTINT16
	UINT16		RTUINT16	INT32		RTINT32
	UINT32		RTUINT32	FLOAT		RTFLOAT
	DOUBLE		RTDOUBLE	BYTES4		RTBYTES4
	BYTES8		RTBYTES8	BYTES12		RTBYTES12
	BYTES16		RTBYTES16	BYTES20		RTBYTES20
	BYTES32		RTBYTES32	BYTES48		RTBYTES48
	BYTES64		RTBYTES64	BYTES80		RTBYTES80
	BYTES128	RTBYTES128	BYTES256	RTBYTES256
	DB_XREF		RTDB_XREF	POLAR		RTPOLAR
	RECTANGULAR	RTRECTANGULAR	DATE		RTDATE
	TIME_OF_DAY	RTTIME_OF_DAY	ABS_TIME	RTABS_TIME
    }
    common validDeTypes

    # Build the validDeTypes lookup array:
    #    - prepend rt to types without it (e.g. BYTES32 -> rtBYTES32)
    #    - use the form rt<TYPE> for everything else
    ::foreach t {
	LOGICAL		FLOAT
	INT8		UINT8
	INT16		UINT16
	INT32		UINT32
	BYTES4		BYTES8
	BYTES12		BYTES16
	BYTES20		BYTES32
	BYTES48		BYTES64
	BYTES80		BYTES128
	BYTES256	DOUBLE
	DB_XREF		POLAR
	RECTANGULAR	DATE
	TIME_OF_DAY	ABS_TIME
    } {
	set validDeTypes($t) rt$t
	set validDeTypes(RT$t) rt$t
    }

    # Other names for some of the above.  These types are considered
    # ambiguous.  If used, a warning will be issued, but processing will
    # continue.
    set validDeTypes()		rtINT32
    set validDeTypes(INT)		rtINT32
    set validDeTypes(UINT)		rtUINT32
    set validDeTypes(BYTES)		rtBYTES32
    set validDeTypes(BOOLEAN)	rtLOGICAL
    common ambiguousTypes		{{} INT UINT BYTES BOOLEAN}


    common deTypeNum
    set deTypeNum(rtLOGICAL)	1
    set deTypeNum(rtINT8)		2
    set deTypeNum(rtUINT8)		3
    set deTypeNum(rtINT16)		4
    set deTypeNum(rtUINT16)		5
    set deTypeNum(rtINT32)		6
    set deTypeNum(rtUINT32)		7
    set deTypeNum(rtFLOAT)		8
    set deTypeNum(rtDOUBLE)		9
    set deTypeNum(rtPOLAR)		10
    set deTypeNum(rtRECTANGULAR)	11
    set deTypeNum(rtBYTES4)		16
    set deTypeNum(rtBYTES8)		17
    set deTypeNum(rtBYTES12)	18
    set deTypeNum(rtBYTES16)	19
    set deTypeNum(rtBYTES20)	20
    set deTypeNum(rtBYTES32)	21
    set deTypeNum(rtBYTES48)	22
    set deTypeNum(rtBYTES64)	23
    set deTypeNum(rtBYTES80)	24
    set deTypeNum(rtBYTES128)	25
    set deTypeNum(rtBYTES256)	26
    set deTypeNum(rtDB_XREF)	28
    set deTypeNum(rtDATE)		29
    set deTypeNum(rtTIME_OF_DAY)	30
    set deTypeNum(rtABS_TIME)	31
    set deTypeNum(rtUNDEFINED)	0


    # ------------------------------------------------------------------
    #  CONSTRUCTOR
    # ------------------------------------------------------------------
    constructor {args} {
	eval configure $args
    }


    # ------------------------------------------------------------------
    #  DESTRUCTOR
    # ------------------------------------------------------------------
    destructor {}


    # ------------------------------------------------------------------
    #  METHOD:  configure - used to change public variables
    # ------------------------------------------------------------------
    #method configure {config} {}


    # ------------------------------------------------------------------
    #  METHOD:  unloadString - return the string that should be written
    #			   to a point config file
    # ------------------------------------------------------------------
    method unloadStr {{alias {}}} {
	global ::dblScanAddresses ::dblScanDeviceEntries

	# Set the name of the attribute
	set str "\nBEGIN ATTRIBUTE\n    Name = $name\n"

	# For scalars and vectors, add the De Type
	if {("$type" == "SCALAR") || ("$type" == "VECTOR")} {
	    append str "    De Type = $validDeTypes([string toupper $deType])\n"
	}

	# Now put the read/write groups and type
	append str "    Write Groups = $writeCateg\n"
	append str "    Read Groups = $readCateg\n"
	append str "    Type = rt$type\n"

	# If there are usage flags defined, add them to the
	# unload string.
	if {$usage != ""} {
	    append str [UsageString $usage]
	}

	# If there is a value defined, add it to the
	# unload string.
	if {("$type" == "SCALAR") && ($value != "")} {
	    set quote [QuoteForDeType $deType]
	    append str "    Value = $quote$value$quote\n"
	}

	# If there is a definition defined, add it to the
	# unload string.
	if {([string length $definition] > 0) && ($pollType == "")} {
	    append str "    Definition = $definition\n"
	}

	# For vectors and tables, add the record count
	if {("$type" == "VECTOR") || ("$type" == "TABLE")} {
	    append str "    Record Count = $numRecords\n"
	}

	# For tables, output the field specifiers
	if {"$type" == "TABLE"} {
	    loop i 0 [llength $deType] {
		set fieldName [lindex $fieldNames $i]
		set fieldType [lindex $deType $i]
		append str "\n"
		append str "    BEGIN FIELD\n"
		append str "        Name = $fieldName\n"
		append str "        De Type = "
		set fieldType [string toupper $fieldType]
		append str "$validDeTypes($fieldType)\n"
		append str "    END\n"
	    }
	}

	# If there is a VECTOR or TABLE value defined, add it to the
	# unload string.
	if {(("$type" == "VECTOR") || ("$type" == "TABLE")) && 
	$value != ""} {
	    append str [ValueString $type $value $deType]
	}

	# Add the END
	append str "END\n"

	# If a poll type is defined, add a scan definition
	# and remember this attribute so that the scan defn
	# table can be built later.
	if {($pollType == "POLL") || ($pollType == "SRBX")} {
	    set cwp $alias
	    set destAttr <alias>$cwp.[$this info variable name -value]
	    set device $scanDevice

	    set dblScanAddresses($device,$destAttr,source)		$scanAddr
	    set dblScanAddresses($device,$destAttr,pollType)	$pollType
	    set dblScanAddresses($device,$destAttr,pollTime)	$pollTime
	    set dblScanAddresses($device,$destAttr,deadbandType)	$deadbandType
	    set dblScanAddresses($device,$destAttr,deadband)	$deadband
	    set dblScanAddresses($device,$destAttr,enableFlag)	$enableFlag

	    lappend dblScanDeviceEntries($device) $destAttr
	}

	return $str
    }


    # ------------------------------------------------------------------
    #  METHOD:  queryConfigData - return a string containing
    #			"-pubVar value" for each of the public
    #			variables.  This method could be written in
    #			a more 'proper' manner without hardcoding
    #			the names of the public variables, but its
    #			purpose is to speed the duplication of
    #			attributes, so this method is favored instead.
    # ------------------------------------------------------------------
    method queryConfigData {} {
	set str ""
	lappend str	-name $name \
		-value $value \
		-deType $deType \
		-readCateg $readCateg \
		-writeCateg $writeCateg \
		-type $type \
		-usage $usage \
		-definition $definition \
		-storage $storage \
		-numRecords $numRecords \
		-fieldNames $fieldNames \
		-pollType $pollType \
		-scanAddr $scanAddr \
		-pollTime $pollTime \
		-deadbandType $deadbandType \
		-deadband $deadband \
		-sourceClass $sourceClass
	return $str
    }


    # ------------------------------------------------------------------
    #  METHOD:  queryHashIdData - return a string containing "-pubVar
    #                             value" for each of the public values
    #                             except the sourceClass.
    # ------------------------------------------------------------------
    method queryHashIdData {} {
	set str ""
	lappend str	-name $name \
		-value $value \
		-deType $deType \
		-readCateg $readCateg \
		-writeCateg $writeCateg \
		-type $type \
		-usage $usage \
		-definition $definition \
		-storage $storage \
		-numRecords $numRecords \
		-fieldNames $fieldNames \
		-pollType $pollType \
		-scanAddr $scanAddr \
		-pollTime $pollTime \
		-deadbandType $deadbandType \
		-deadband $deadband
	return $str
    }


    # ------------------------------------------------------------------
    #  METHOD: queryDeType - return the deType number for this attribute
    # ------------------------------------------------------------------
    method queryDeType {} {
	return $deTypeNum($validDeTypes([string toupper $deType]))
    }


    # ------------------------------------------------------------------
    #  PUBLIC: name - name of the attribute
    # ------------------------------------------------------------------
    public variable name {} {
	# length must be < 20
	if {[string length $name] > 19} {
	    error "attribute name too long: should be < 20 chars"
	}

	# check for illegal characters
	regsub -all -- {[a-zA-Z0-9_]+} $name {} illegalChars
	###set illegalChars [translit {a-zA-Z0-9_} {} $name]
	if {[string length $illegalChars]} {
	    set msg "Invalid attribute name '$name'\n"
	    append msg "Contains illegal char(s) '$illegalChars'"
	    Message Warning $msg
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: value - value of the attribute
    # ------------------------------------------------------------------
    public variable value {} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: deType - data element type of the attribute
    # ------------------------------------------------------------------
    public variable deType {} {
	# deType will be a single element for SCALAR and VECTOR
	# attributes, and a list for TABLE attributes.
	if {[llength $deType] <= 1} then {
	    if {![info exists validDeTypes([string toupper $deType])]} {
		error "Unknown data element type '$deType'"
	    } elseif {[lsearch $ambiguousTypes [string toupper $deType]] >= 0} {
		set actual $validDeTypes([string toupper $deType])
		set msg "Ambiguous type '$deType' substituted with '$actual'"
		Message Warning $msg
	    }
	    set deType $validDeTypes([string toupper $deType])
	} else {
	    # More than one deType specified, verify each one in
	    # the list.
	    set fieldList {}
	    foreach field $deType {
		set field [string toupper $field]
		if {![info exists validDeTypes($field)]} {
		    error "Unknown data element type '$field'"
		} elseif {[lsearch $ambiguousTypes [string toupper $deType]] >= 0} {
		    set actual $validDeTypes([string toupper $deType])
		    set msg "Ambiguous type '$deType' substituted with '$actual'"
		    Message Warning $msg
		}
		lappend fieldList $validDeTypes([string toupper $field])
	    }
	    set deType $fieldList
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: readCateg - read categories mask for the attribute
    # ------------------------------------------------------------------
    public variable readCateg {1111} {
	# category mask is 4 bits
	if {[string length $readCateg] != 4} {
	    error "read category mask must be 4 characters in length"
	}

	# a mask is made of 0's and 1's
	regsub -all -- {[01]+} $readCateg {} illegalChars
	###set illegalChars [translit {01} {} $readCateg]
	if {[string length $illegalChars]} {
	    set msg "illegal char(s) in read categories mask"
	    append msg " '$illegalChars'"
	    append msg ": should contain only 0's and 1's"
	    error $msg
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: writeCateg - write categories mask for the attribute
    # ------------------------------------------------------------------
    public variable writeCateg {0000} {
	# category mask is 4 bits
	if {[string length $writeCateg] != 4} {
	    error "write category mask must be 4 characters in length"
	}

	# a mask is made of 0's and 1's
	regsub -all -- {[01]+} $writeCateg {} illegalChars
	###set illegalChars [translit {01} {} $writeCateg]
	if {[string length $illegalChars]} {
	    set msg "illegal char(s) in write categories mask"
	    append msg " '$illegalChars'"
	    append msg ": should contain only 0's and 1's"
	    error $msg
	}
    }

    # ------------------------------------------------------------------
    #  PUBLIC: type - attribute type (SCALAR, VECTOR, TABLE)
    # ------------------------------------------------------------------
    public variable type {SCALAR} {
	if {[lsearch $validTypes [string toupper $type]] < 0} {
	    error "Unknown attribute element type '$type'"
	}
	switch [string toupper $type] {
	    RTSCALAR	{set type SCALAR}
	    RTVECTOR	{set type VECTOR}
	    RTTABLE		{set type TABLE}
	    default		{; # invalid types must be trapped above}
	}
    }

    # ------------------------------------------------------------------
    #  PUBLIC: numRecords - number of records (TABLE or VECTOR only)
    # ------------------------------------------------------------------
    public variable numRecords {0} {
	if {$numRecords == ""} {
	    set numRecords $oldNumRecords
	    return
	}
	if {![ctype digit $numRecords]} {
	    error "number of records must be an integer"
	}
	if {$numRecords < 0 || $numRecords > 65535} {
	    error "number of Records must be between 0 and 65535"
	}
	set oldNumRecords $numRecords
    }


    # ------------------------------------------------------------------
    #  PUBLIC: fieldNames - list of field names for TABLE only
    # ------------------------------------------------------------------
    public variable fieldNames {} {
	foreach fieldName $fieldNames {
	    # length must be < 20
	    if {[string length $fieldName] > 19} {
		set msg "table field '$fieldName' name too "
		append msg "long: should be < 20 chars"
		error $msg
	    }

	    # check for illegal characters
	    regsub -all -- {[a-zA-Z0-9_]+} $fieldName {} illegalChars
	    ###set illegalChars [translit {a-zA-Z0-9_} {} $fieldName]
	    if {[string length $illegalChars]} {
		set msg "Invalid table field name '$fieldName'\n"
		append msg "Contains illegal char(s) '$illegalChars'"
		Message Warning $msg
	    }
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: usage - usage flags
    # ------------------------------------------------------------------
    public variable usage {} {
	# usage numbers should be 128-255
	foreach num $usage {
	    if {![ctype digit $num]} {
		error "Usage indicator '$num' must be an integer"
	    }
	    if {($num < 128) || ($num > 255)} {
		set msg "Usage indicator '$num' is out of range"
		append msg ": should be in range 128-255"
		error $msg
	    }
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: definition - RTAP Calculation Engine definition
    # ------------------------------------------------------------------
    public variable definition {} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: pollType - either POLL or SRBX
    # ------------------------------------------------------------------
    public variable pollType {} {
	if {($pollType != "") && ($pollType != "POLL") && ($pollType != "SRBX")} {
	    error "Invalid poll type '$pollType'"
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: pollTime - poll time for a POLL scan link
    # ------------------------------------------------------------------
    public variable pollTime {0.0} {
	# pollTime must be a float or integer - try to use it for
	# addition to verify this
	if [catch {expr $pollTime + 1}] {
	    error "poll time '$pollTime' must be in seconds"
	}

	# pollTime must be one of the defined poll periods
	set intPollTime [format %.0f $pollTime]
	if {[lsearch $validPollTimes $intPollTime] < 0} {
	    error "invalid poll time '$intPollTime'"
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: scanAddr - address to be scanned
    # ------------------------------------------------------------------
    public variable scanAddr {} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: scanDevice - device for a scan link
    # ------------------------------------------------------------------
    public variable scanDevice {} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: deadbandType - deadband type for a SRBX scan link
    # ------------------------------------------------------------------
    public variable deadbandType {} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: deadband - deadband for a SRBX scan link
    # ------------------------------------------------------------------
    public variable deadband {} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: enableFlag - scan link initial enable/disable state
    # ------------------------------------------------------------------
    public variable enableFlag {ENABLE} {
	if {[lempty $enableFlag]} {
	    set enableFlag ENABLED
	} else {
	    if {($enableFlag != "ENABLE") && ($enableFlag != "DISABLE")} {
		error "Invalid scan link enable/disable flag '$enableFlag'"
	    }
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: storage - either STATIC or AUTOMATIC
    # ------------------------------------------------------------------
    public variable storage {AUTOMATIC} {
	if {([string toupper $storage] != "STATIC") &&
	([string toupper $storage] != "AUTOMATIC")} {
	    error "Invalid storage type '$storage'"
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: overloadable - whether or not this attribute can be
    #			  overloaded.  Generally, attributes can be
    #			  overloaded, except when they are created
    #			  from within a point definition
    # ------------------------------------------------------------------
    public variable overloadable {1} {
	# ensure that this is a valid boolean expression
	set overloadable [expr {$overloadable}]
    }


    # ------------------------------------------------------------------
    #  PUBLIC: deletable -	whether or not this attribute can be
    #			deleted.  ATTRIBUTEs can be referenced in
    #			more than one place, but sometimes they
    #			exist only as a member of a POINT, and can
    #			safely be deleted with the POINT is deleted.
    #			By default, an instance is considered _not_
    #			deletable, and must be marked otherwise by
    #			the creator of an instance.
    # ------------------------------------------------------------------
    public variable deletable {0} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: sourceClass - class where this attribute was defined
    #                        first
    # ------------------------------------------------------------------
    public variable sourceClass {} {}


    # ------------------------------------------------------------------
    # PROC: UsageString - returns the unload string for a set of
    #		      usage flags
    # ------------------------------------------------------------------
    proc UsageString {usage} {
	set usageList {}
	foreach u $usage {
	    append usageList $u,
	}
	set usageList [string trimright $usageList ,]
	return "    Usage = $usageList\n"
    }

    # ------------------------------------------------------------------
    # PROC: ValueString - returns the unload string for the
    #		      attribute value
    # ------------------------------------------------------------------
    proc ValueString {type value deType} {
	# If the deType is any one of the BYTE types (i.e. string)
	# then use double quotes - otherwise, don't.
	set quote [QuoteForDeType $deType]
	
	switch $type {
	    VECTOR	{
		set str "    BEGIN VECTOR\n"
		foreach v $value {
		    append str "        $quote$v$quote\n"
		}
		append str "    END\n"
	    }
	    TABLE	{
		append str "    BEGIN TABLE\n"
		foreach record $value {
		    append str "        $record\n"
		}
		append str "    END\n"
	    }
	    default {
		set msg "Internal error:  "
		append msg "unknown attribute type '$type'"
	    }
	}
	return $str
    }


    # ------------------------------------------------------------------
    # PROC: QuoteForDeType - type of quote needed for a value of a de type
    # ------------------------------------------------------------------
    proc QuoteForDeType {t} {
	if [string match *BYTE* [string toupper $t]] {
	    return \"
	}
	if [string match *DATE [string toupper $t]] {
	    return \"
	}
	if [string match *TIME_OF_DAY [string toupper $t]] {
	    return ""
	}
	return ""
    }
}
