#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: dblcbCLASS_DESCRIPTOR.tcl,v 1.8 2001/06/06 08:48:08 vltsccm Exp $" 
#
#
# CLASS_DESCRIPTOR
# ----------------------------------------------------------------------
# The class encapsulates information about an ESO class-type.  This
# includes default information about instances of the class, the
# position of the class in the class hierarchy, and the structure of
# the class (including attributes and children points).  Methods are
# provided to insert CLASS_DESCRIPTORS into the ESO class-type
# hierarchy, configure each of the above-mentioned aspects of an
# instance, and create instances in an RTAP database of a class-type.
#
#   PUBLIC ATTRIBUTES:
#
#	-categories .... categories which the point belongs to
#	-residence ..... point residence
#	-ceIndicator ... Calculation Engine state
#	-ceOrder ....... attribute order for Calculation Engine
#	-usage ......... usage flags
#	-storage ....... either STATIC or AUTOMATIC
#	-checksum ...... checksum value for the class descriptor based
#			 on its attributes
#	-srcFile ....... the name of the file containing the source
#                        for this class
#	-srcLine ....... the line in the file containing the source
#                        for this class
#
#
#   METHODS:
#
#	reserveClassId ...... reserve a class identifier for a future instance
#	configure ........... used to change public attributes
#	addAttr ............. add an ATTRIBUTE instance
#	addClassTypeAttr .... add a class type attribute to this class-type
#	createInstance ...... produce a tree of POINTS which is an
#			      instance of this class
#	queryAttributes ..... returns the list of ATTRIBUTES defined for
#			      this point class (both inherited and actual)
#	queryClassTypeAttrs . returns the list of CLASS_DESCRIPTORs and
#			      names which represent structured attributes
#	queryParentClass .... returns the parent class of the instance
#	querySubClasses ..... returns the list of subclasses
#	queryTemplatePoint .. returns the template point instance name
#	queryClassNum ....... returns the class number of this class
#	assignClassNum ...... force a particular class number for this class
#	insertSubClass ...... add a CLASS_DESCRIPTOR to the list of
#			      subclasses of this class
#	deleteSubClass ...... delete a CLASS_DESCRIPTOR from the list of
#			      subclasses of this class
#	unload .............. produce the class_config branch point config
#			      files
#	addMethod ........... add a method descriptor to this class
#	hashId .............. calculates a hash value for a class descriptor
#
#
#   COMMON ATTRIBUTES:
#
#	reservedClassIds .... class ids which are reserved in advance for
#			      for CLASS_DESCRIPTOR instances which may be
#			      eventually created
#
#
#   PROTECTED ATTRIBUTES:
#
#	classSource ........ an instance of INT_GROUP used for allocating
#			     class numbers
#	classNum ........... class number which will be used in the
#			     "class config" point
#	childPointList ..... list of {class name} pairs which are
#			     defined by the class description as
#			     children points of this class
#	childPoints ........ POINT instances corresponding to childPointList
#	attrList ........... list of ATTRIBUTE instances which are
#			     memebers of this class
#	inheritedAttrList .. list of ATTRIBUTES inherited from the
#			     parent class
#	templatePoint ...... a POINT instance which holds all of the
#			     default point data
#	parentClass ........ a CLASS_DESCRIPTOR instance which is the
#			     parent class for this instance
#	subClassList ....... a CLASS_DESCRIPTOR instance list which are
#			     subclasses of the class described by this
#			     instance
#	methodList ......... a list of method descriptors
#
# ----------------------------------------------------------------------
#
#   AUTHOR:  Thomas J. Accardo
#            Computerized Processes Unlimited, Inc.
#            Phone: (504) 889-2784
#            E-mail: tja@cpu.com
#
# ======================================================================
# Modified:
# who             when        what
# --------------  ----------  ----------------------------------------------
# Erik Allaert    08/03/96    adapted to [incr Tcl] 2.0
# Erik Allaert    2000-02-02  adapted to Tcl/Tk 8.0, [incr Tcl] 3.0
# ======================================================================

# This will fool the auto_mkindex procedure into sourcing this file
# so that this class can be loaded.
proc CLASS_DESCRIPTOR {} {;}; rename CLASS_DESCRIPTOR {}


# ======================================================================
# REQUIRED CLASSES
#	INT_GROUP
# ======================================================================
foreach class {INT_GROUP} {
    if {[lsearch [itcl::find classes *] $class] < 0} {
	auto_load $class
    }
}

class CLASS_DESCRIPTOR {
    common reservedClassIds

    # ==================================================================
    #  PROTECTED DATA
    #	classNum ........... class number which will be used in the
    #			     "class config" point
    #	childPointList ..... list of CLASS_DESCRIPTOR instance names
    #			     which are defined by the class description
    #			     as children points of this class
    #	childPoints ........ POINT instances corresponding to childPointList
    #	attrList ........... list of ATTRIBUTE instances which are
    #			     memebers of this class
    #	inheritedAttrList .. list of ATTRIBUTES inherited from the
    #			     parent class
    #	templatePoint ...... a POINT instance which holds all of the
    #			     default point data
    #	parentClass ........ a CLASS_DESCRIPTOR instance which is the
    #			     parent class for this instance
    #	subClassList ....... a CLASS_DESCRIPTOR instance list which are
    #			     subclasses of the class described by this
    #			     instance
    #	classSource ........ an instances of INT_GROUP used for
    #			     allocating class numbers
    #	methodList ......... a list of method descriptors
    # ==================================================================
    protected variable classNum            {0}
    protected variable className           {}
    protected variable childPointList      {}
    protected variable childPoints         {}
    protected variable attrList            {}
    protected variable inheritedAttrList   {}
    protected variable templatePoint       {}
    protected variable parentClass         {}
    protected variable subClassList        {}
    protected variable classSource         {}
    protected variable methodList          {}	



    # ------------------------------------------------------------------
    #  CONSTRUCTOR
    # ------------------------------------------------------------------
    constructor {class} {
	global ::existingClassIds

	# The parent class is a required argument, but the argument
	# name cannot conflict with the public variable
	# 'parentClass', so we use 'class' as a go-between variable
	set parentClass $class

	# Since version 2.0 of [incr Tcl], $this has leading colons,
	# as the entire absolute namespace is prepended; use the variable
	# className where this is not desired
	set className [namespace tail $this]

	# Create the template POINT object for this RTAP class
	set templatePoint [namespace inscope :: POINT #auto]
	$templatePoint configure \
		-categories $categories \
		-residence $residence \
		-ceIndicator $ceIndicator \
		-ceOrder $ceOrder \
		-usage $usage \
		-className $className

	# If this class isn't a child of any other class, i.e. the
	# BASE_CLASS, then create a new INT_GROUP for allocating
	# RTAP class numbers.  Otherwise, set up the parent/child
	# class relationship and attribute inheritance.
	if {$parentClass == ""} {
	    set classSource [namespace inscope :: INT_GROUP #auto 65535]
	    loop i 0 100 {$classSource allocate}
	} else {
	    if {[$this info class] != [$parentClass info class]} {
		error "$class is not a valid class descriptor object."
	    }

	    set classSource \
		    [$parentClass info variable classSource -value]

	    set inheritedAttrList [$parentClass queryAttributes]

	    foreach attr $inheritedAttrList {
		$templatePoint replaceAttr [lindex $attr 0]
	    }

	    foreach attrInfo [$parentClass info variable childPointList -value] {
		if {[lindex $attrInfo 2] != "STATIC"} {
		    eval \$this addClassTypeAttr $attrInfo
		}
	    }

	    set methodList [$parentClass info variable methodList -value]

	    $parentClass insertSubClass $className
	}

	# Allocate an RTAP class number for this class if one hasn't already
	# been defined.
	if {[info exists reservedClassIds($className)]} {
	    set classNum $reservedClassIds($className)
	} else {
	    set classNum [$classSource allocate]
	}
	$templatePoint configure -class $classNum
    }


    # ------------------------------------------------------------------
    #  DESTRUCTOR
    # ------------------------------------------------------------------
    destructor {
	# Destroy all of the attributes
	foreach attr $attrList {delete object [lindex $attr 0]}

	# Delete all of the subclasses of this class
	foreach class $subClassList {
	    delete object $class
	}

	# Remove self from parent class's list of subclasses
	if {$parentClass != ""} {
	    $parentClass deleteSubClass $className
	}

	# Delete the template POINT
	delete object $templatePoint
    }



    # ------------------------------------------------------------------
    #  METHOD:  assignClassNum - force a particular class number for
    #			     this class
    # ------------------------------------------------------------------
    method assignClassNum {num} {
	if {$classNum != 0} {
	    $classSource release $classNum
	}
	$classSource allocate $num
	set classNum $num
    }

    # ------------------------------------------------------------------
    #  METHOD:  configure - used to change public variables
    # ------------------------------------------------------------------
    #method configure {config} {}


    # ------------------------------------------------------------------
    #  METHOD:  reserveClassId - reserve a class identifier for a future
    #			     instance
    # ------------------------------------------------------------------
    method reserveClassId {classId className} {
	catch {$classSource allocate $classId}
	set reservedClassIds($className) $classId
    }


    # ------------------------------------------------------------------
    #  METHOD:  addAttr - add an ATTRIBUTE instance
    # ------------------------------------------------------------------
    method addAttr {attribute name} {
	# Don't allow the addition of an attribute which has already
	# been added (this doesn't include inherited attributes)
	if {[lsearch $attrList [list * $name]] >= 0} {
	    error "Attribute '$name' already exists in point"
	}

	# If this class already has the named attribute, then
	# overload it with this new one and remove the other one
	# from the inherited attribute list.
	set index [lsearch $inheritedAttrList [list * $name]]
	set position end
	if {$index >= 0} {
	    set position $index
	    set tmpAttr [lindex [lindex $inheritedAttrList $index] 0]

	    # If no deType was specified for the new attribute,
	    # then inherit the one from the attribute its overloading
	    if {"[$attribute info variable deType -value]" == ""} {
		$attribute configure -deType [$tmpAttr info variable deType -value]
	    }

	    # The attribute should keep the same source class as
	    # the existing attribute
	    $attribute configure -sourceClass [$tmpAttr info variable sourceClass -value]

	    # If this is a table attribute and no field names were
	    # specified, then inherit the ones from the attribute
	    # being overloaded.
	    if {"[$attribute info variable fieldNames -value]" == ""} {
		$attribute configure -fieldNames [$tmpAttr info variable fieldNames -value]
	    }


	    if {[catch {OverloadAttrOk $tmpAttr $attribute} err]} {
		error "Cannot overload attribute $name:\n\t$err"
	    } else {
		$templatePoint replaceAttr $attribute
	    }
	} else {
	    $attribute configure -sourceClass $className
	    $templatePoint addAttr $attribute
	}

	if {"$position" == "end"} {
	    lappend attrList [list $attribute $name]
	} else {
	    set attrList [linsert $attrList $position [list $attribute $name]]
	}
    }

    # ------------------------------------------------------------------
    #  METHOD:  addClassTypeAttr - add a child point to this class-type
    # ------------------------------------------------------------------
    method addClassTypeAttr {class name storage} {
	# See if the class type attribute already exists
	set i [lsearch $childPointList "* $name *"]

	# If yes, then make sure this new class type attribute is
	# the same class or a subclass of the old one.
	if {$i >= 0} {
	    set oldClass [lindex [lindex $childPointList $i] 0]
	    set heritage [ClassHeritage $class]

	    if {([lsearch $heritage $oldClass] < 0) &&
	    ("$oldClass" != "$class")} {
		set msg "Failed to overload attribute:\n"
		append msg "'$class' is not a subclass of"
		append msg " the existing point's class '$oldClass'"
		error $msg
	    }
	    lvarpop childPointList [lsearch $childPointList "* $name *"]
	    $templatePoint deleteChildByName $name
	}

	# Add the class type attribute
	lappend childPointList [list $class $name $storage]
	set tmpPt [$class createInstance $className $name]

	if {[llength [split $name :]] > 1} {
	    set splitPath [split $name :]
	    set path [lrange $splitPath 0 [expr [llength $splitPath]-2]]
	    set name [lindex $splitPath [expr [llength $splitPath]-1]]
	    set path [join $path :]
	    set point [PointRelativeToPoint $templatePoint $path]
	} else { 
	    set point $templatePoint
	}
	set tmpPt [$class createInstance $className $name]

	$point insertChild $tmpPt
	return $tmpPt
    }


    # ------------------------------------------------------------------
    #  METHOD:  queryAttributes - returns the list of ATTRIBUTES defined
    #			      for this point class (both inherited
    #			      and actual)
    # ------------------------------------------------------------------
    method queryAttributes {} {
	set aList {}

	set origList {}
	foreach a [$templatePoint info variable attrList -value] {
	    lappend origList [list $a [$a info variable name -value]]
	}
	foreach a [concat $origList $attrList $inheritedAttrList] {
	    if {([lsearch $aList [list * [lindex $a 1]]] < 0) &&
	    ([[lindex $a 0] info variable storage -value] != "STATIC")} {
		lappend aList $a
	    }
	}
	return $aList
    }


    # ------------------------------------------------------------------
    #  METHOD:  queryClassTypeAttrs - returns the list of CLASS_DESCRIPTORs
    #			          and names which represent structured
    #			          attributes
    # ------------------------------------------------------------------
    method queryClassTypeAttrs {} {
	return $childPointList
    }


    # ------------------------------------------------------------------
    #  METHOD:  queryParentClass - returns the parent class of the
    #			       instance
    # ------------------------------------------------------------------
    method queryParentClass {} {
	return $parentClass
    }


    # ------------------------------------------------------------------
    #  METHOD:  queryTemplatePoint - returns the template point instance
    #			         name
    # ------------------------------------------------------------------
    method queryTemplatePoint {} {
	return $templatePoint
    }


    # ------------------------------------------------------------------
    #  METHOD:  querySubClasses - returns the list of subclasses
    # ------------------------------------------------------------------
    method querySubClasses {} {
	return $subClassList
    }


    # ------------------------------------------------------------------
    #  METHOD:  queryClassNum - returns the class number of this class
    # ------------------------------------------------------------------
    method queryClassNum {} {
	return $classNum
    }


    # ------------------------------------------------------------------
    #  METHOD:  createInstance - produce a tree of POINTS which are
    #			     instances of this point
    # ------------------------------------------------------------------
    method createInstance {parentPoint name} {
	global ::WARNING_ENABLE

	# Temporarily turn off warnings so we don't get repeat
	# warnings about attribute names with spaces, etc.
	set tmp_WARNING_ENABLE $WARNING_ENABLE
	set WARNING_ENABLE 0

	# Make a POINT with my point properties
	set myInstance [$templatePoint replicate]
	$myInstance configure -name $name -class $classNum

	# Add a copy of my attributes to the point
	foreach attr [$this queryAttributes] {
	    set tmpAttr [namespace inscope :: ATTRIBUTE #auto]
	    set configData [[lindex $attr 0] queryConfigData]
	    eval \$tmpAttr configure $configData
	    $myInstance addAttr $tmpAttr
	}

	# Mark whether or not the point will need a 'class' attribute
	if {[needClassAttribute? $className]} {
	    $myInstance configure -needClassAttr 1
	}

	# Return the WARNING_ENABLE to its previous state
	set WARNING_ENABLE $tmp_WARNING_ENABLE

	# This is the root of the point branch just created
	return $myInstance
    }


    # ------------------------------------------------------------------
    #  METHOD:  insertSubClass - add a CLASS_DESCRIPTOR to the list of
    #			     subclasses of this class
    # ------------------------------------------------------------------
    method insertSubClass {class} {
	if {[$this info class] != [$class info class]} {
	    error "$class is not a valid class descriptor object."
	}
	lappend subClassList $class
    }


    # ------------------------------------------------------------------
    #  METHOD:  deleteSubClass - delete a CLASS_DESCRIPTOR from the list
    #			     of subclasses of this class
    # ------------------------------------------------------------------
    method deleteSubClass {class} {
	lvarpop subClassList [lsearch $subClassList $class]
    }


    # ------------------------------------------------------------------
    #  METHOD:  unload - produce the class_config branch point config
    #		     files
    # ------------------------------------------------------------------
    method unload {db {name -}} {
	if {$name == "-"} {set name $className}
	set alias $className

	# Make a minimal point to unload
	set unloadPt [namespace inscope :: POINT #auto \
		-alias $alias \
		-name $name \
		-residence $residence \
		-categories $categories \
		-ceIndicator $ceIndicator \
		-ceOrder $ceOrder \
		-class $classNum \
		-className $className]
	$unloadPt addComment "$className CHECKSUM VALUE: $checksum"

	# Unload each of the static attributes
	foreach attrSpec $attrList {
	    set attr [lindex $attrSpec 0]
	    if {[$attr info variable storage -value] == "STATIC"} {
		set staticAttr [eval namespace inscope :: ATTRIBUTE #auto \
			[$attr queryConfigData]]
		$staticAttr configure -storage AUTOMATIC
		$unloadPt addAttr $staticAttr
	    }
	}

	if {"class_$className" == "class_BASE_CLASS"} {
	    set attr [namespace inscope :: ATTRIBUTE #auto -name class \
		    -deType rtBYTES32 \
		    -value "" \
		    -deletable 1]
	    $unloadPt addAttr $attr
	} elseif {"class_$className" != "class_NULL_CLASS"} {
	    set attr [namespace inscope :: ATTRIBUTE #auto -name class \
		    -deType rtBYTES32 \
		    -value <alias>$parentClass \
		    -deletable 1]
	    $unloadPt addAttr $attr
	}

	# unload the method table
	if {"class_$className" != "class_NULL_CLASS"} {
	    $unloadPt addAttr [DefineMethodAttribute $methodList]
	}

	$db addBranch $unloadPt

	set oldCwp [$db info variable cwp -value]
	$db configure -cwp $className
	foreach structAttr $childPointList {
	    if {[lindex $structAttr 2] == "STATIC"} {
		set class [lindex $structAttr 0]
		set pt [$class createInstance $className [lindex $structAttr 1]]
		$db addBranch $pt
	    }
	}

	# Unload the subclasses
	foreach class $subClassList {
	    $class unload $db
	}
	$db configure -cwp $oldCwp
    }


    # ------------------------------------------------------------------
    #  METHOD:  addMethod - add a method descriptor to this class
    # ------------------------------------------------------------------
    method addMethod {name routine type {taskName {}} {priority {}} {flags {}} {stack {}}} {
	lvarpop methodList [lsearch $methodList "$name *"]
	lappend methodList [list $name $routine $type $taskName $priority $flags $stack $className]
    }


    # ------------------------------------------------------------------
    #  METHOD:  hashId - calculates a hash value for a class descriptor
    # ------------------------------------------------------------------
    method hashId {} {
	set attrInfoList {}
	foreach attrPair [$this queryAttributes] {
	    lappend attrInfoList [[lindex $attrPair 0] queryHashIdData]
	}
	set attrInfoList [lsort $attrInfoList]

	set structAttrInfoList {}
	foreach structAttrPair [$this queryClassTypeAttrs] {
	    lappend structAttrInfoList $structAttrPair
	}
	set structAttrInfoList [lsort $structAttrInfoList]

	set hashInfo [concat $attrInfoList $structAttrInfoList]
	return $hashInfo
    }


    # ------------------------------------------------------------------
    #  PUBLIC: categories - categories which the point belongs to
    # ------------------------------------------------------------------
    public variable categories {11111111111111111111111111111111} {
	$templatePoint configure -categories $categories
    }


    # ------------------------------------------------------------------
    #  PUBLIC: residence - point residence
    # ------------------------------------------------------------------
    public variable residence {RAM} {
	$templatePoint configure -residence $residence
    }


    # ------------------------------------------------------------------
    #  PUBLIC: ceIndicator - Calculation Engine state
    # ------------------------------------------------------------------
    public variable ceIndicator {enabled} {
	$templatePoint configure -ceIndicator $ceIndicator
    }


    # ------------------------------------------------------------------
    #  PUBLIC: ceOrder - attribute order for Calculation Engine
    # ------------------------------------------------------------------
    public variable ceOrder {natural} {
	$templatePoint configure -ceOrder $ceOrder
    }


    # ------------------------------------------------------------------
    #  PUBLIC: usage - usage flags
    # ------------------------------------------------------------------
    public variable usage {} {
	$templatePoint configure -usage $usage
    }

    # ------------------------------------------------------------------
    #  PUBLIC: storage - either STATIC or AUTOMATIC
    # ------------------------------------------------------------------
    public variable storage {AUTOMATIC} {
	if {([string toupper $storage] != "STATIC") &&
	([string toupper $storage] != "AUTOMATIC")} {
	    error "Invalid storage type \"$storage\""
	}
    }



    # ------------------------------------------------------------------
    #  PUBLIC: checksum - checksum value for the class descriptor based
    #		      on its attributes
    # ------------------------------------------------------------------
    public variable checksum {0} {
	
    }


    # ------------------------------------------------------------------
    #  PUBLIC: srcFile - the name of the file containing the source
    #                    for this class
    # ------------------------------------------------------------------
    public variable srcFile {} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: srcLine - the line in the file containing the source
    #                    for this class
    # ------------------------------------------------------------------
    public variable srcLine {} {
    }



    # ------------------------------------------------------------------
    #  PROC:  needClassAttribute? - decides whether or not to add the
    #				'class' attribute to a point of this
    #				class
    # ------------------------------------------------------------------
    proc needClassAttribute? {aClass} {
	set heritage [ClassHeritage $aClass]

	# If the class is NULL_CLASS or a child of it, then the answer
	# is no
	if {([string match NULL_CLASS $aClass]) ||
	([string match NULL_CLASS [lindex $heritage [expr [llength $heritage]-1]]])} {
	    return 0
	}

	# If the class is <base> or a child of it, then the answer
	# is no
	if {([string match <base> $aClass]) ||
	([string match <base> [lindex $heritage [expr [llength $heritage]-1]]])} {
	    return 0
	}

	# Otherwise, the answer is yes
	return 1
    }


    # ------------------------------------------------------------------
    #  PROC:  AddMethodAttribute - build a 'methods' table
    # ------------------------------------------------------------------
    proc DefineMethodAttribute {methodList} {
	set attr [namespace inscope :: ATTRIBUTE #auto -name methods \
		-type TABLE \
		-fieldNames {
	    MethodName	RoutineName
	    Type		TaskName
	    Priority	Flags
	    StackSize	Defclass
	} \
		-deType {
	    rtBYTES32	rtBYTES32
	    rtBYTES32	rtBYTES32
	    rtINT8		rtUINT8
	    rtUINT8		rtBYTES20
	} \
		-deletable 1]

	set value {}
	foreach method $methodList {
	    set line \"[lvarpop method]\"
	    append line ", \"[lvarpop method]\""
	    append line ", \"[lvarpop method]\""
	    append line ", \"[lvarpop method]\""

	    set Priority	[lvarpop method]
	    set Flags	[lvarpop method]
	    set StackSize	[lvarpop method]
	    if {"$Priority" == ""}	{set Priority 0}
	    if {"$Flags" == ""}	{set Flags 0}
	    if {"$StackSize" == ""}	{set StackSize 0}

	    append line ", $Priority"
	    append line ", $Flags"
	    append line ", $StackSize"

	    append line ", \"[lvarpop method]\""

	    lappend value $line
	}
	$attr configure -value $value -numRecords [llength $value]
	return $attr
    }



    # ------------------------------------------------------------------
    #  PROC:  OverloadAttrOk - Verify that attr1 can legally be
    #			   overloaded with attr2
    # ------------------------------------------------------------------
    proc OverloadAttrOk {attr1 attr2} {
	set name1 [$attr1 info variable name -value]
	set name2 [$attr2 info variable name -value]

	# Check the type - SCALAR, VECTOR, or TABLE
	set type1 [$attr1 info variable type -value]
	set type2 [$attr2 info variable type -value]
	if {$type1 != $type2} {
	    error "Attribute '$name2' not of type $type1"
	}

	switch $type1 {
	    SCALAR	{
		# Check the deType
		set deType1 [$attr1 info variable deType -value]
		set deType2 [$attr2 info variable deType -value]
		if {$deType1 != $deType2} {
		    error "deType mismatch between '$deType1' and '$deType2'"
		}
	    }
	    VECTOR	{
		# Check the deType
		set deType1 [$attr1 info variable deType -value]
		set deType2 [$attr2 info variable deType -value]
		if {$deType1 != $deType2} {
		    error "deType mismatch between '$deType1' and '$deType2'"
		}
	    }
	    TABLE	{
		# Fold the field names and attributes into a single
		# list and then compare.
		set deType1 [$attr1 info variable deType -value]
		set fieldNames1 [$attr1 info variable fieldNames -value]
		set i 0
		set typeList1 {}
		foreach field $deType1 {
		    lappend typeList1 [list $field [lindex $fieldNames1 $i]]
		    incr i
		}

		set deType2 [$attr2 info variable deType -value]
		set fieldNames2 [$attr2 info variable fieldNames -value]
		set i 0
		set typeList2 {}
		foreach field $deType2 {
		    lappend typeList2 [list $field [lindex $fieldNames2 $i]]
		    incr i
		}

		if {[lsort $typeList1] != [lsort $typeList2]} {
		    set field1 {}
		    set field2 {}
		    while {($field1 == $field2) && ![lempty $typeList1]} {
			set field1 [lvarpop typeList1]
			set field2 [lvarpop typeList2]
		    }
		    error "deType mismatch in table field '[lindex $field1 1]'"
		}
	    }
	    default {
		error "Internal error: unknown attribute type '$type'"
	    }
	}
    }
}
