#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: dblcbDATABASE.tcl,v 1.8 2001/06/06 08:48:08 vltsccm Exp $" 
#
#
# DATABASE
# ----------------------------------------------------------------------
# This class encapsulates information about the structure of an RTAP
# database.  Its methods use this structure information to access
# information about points and attributes.  The purpose of this method
# is not only to encapsulate the concept of a database, but to abstract
# the database access so that the actual information can be taken from
# either an on-line database or from an unloaded database on the
# filesystem.
#
#   PUBLIC ATTRIBUTES:
#
#	-cwp ............. current working position
#	-name ............ the name of the database
#	-pointsUnloaded .. count of the number of points which have been
#			   unloaded
#	-aliasCounter .... a counter used to generate unique aliases
#
#
#   METHODS:
#
#	getAlias ....... convert a symbolic address into an alias
#	getAbsPath ..... convert an alias into an absolute pathname
#	getDir ......... given an alias, return the point config
#			 file pathname
#	getFile ........ given an alias, return the point config
#			 file pathname to the file
#	loadPath ....... retrieve structure information from each RTAP
#		         point config file contained in a pathname
#	configure ...... used to change public attributes
#	loadBranch ..... load a branch in the database from a directory
#			 structure
#	unloadBranch ... unload a branch of the database to a 
#			 structure
#	addBranch ...... add a point (which could be the root of a
#			 branch) to the database
#	deleteBranch ... delete a branch in the database
#	pathLoaded? .... check if a point is already loaded into
#			 the database
#	aliasLoaded? ... check if a point is already loaded into
#			 the database
#	getAttrInfo .... retrieve attribute information from a point
#			 config file
#
#
#   COMMON ATTRIBUTES:
#
#	There are no common attributes.
#
#
#   PROTECTED ATTRIBUTES:
#
#	prevAliasCount .. the previous value of public var aliasCounter
#	dbStructure ..... an array containing all of the structural
#			  information about the database
#	rootDir ......... director containing the root point
#
# ----------------------------------------------------------------------
#
#   AUTHOR:  Thomas J. Accardo
#            Phone: (504) 889-2784
#            Computerized Processes Unlimited, Inc.
#            4200 S. I-10 Service Road Suite #205
#            Metairie, Louisiana  USA
#            E-mail: tja@cpu.com
#
# ======================================================================
# Modified:
# who             when      what
# --------------  --------  ----------------------------------------------
# Erik Allaert    08/03/96  adapted to [incr Tcl] 2.0
# ======================================================================

# This will fool the auto_mkindex procedure into sourcing this file
# so that this class can be loaded.
proc DATABASE {} {;}; rename DATABASE {}

class DATABASE {
	# ==================================================================
	#  PROTECTED DATA
	#	dbStructure ... an array containing all of the structural
	#			information about the database
	#	rootDir ....... director containing the root point
	# ==================================================================
	protected variable dbStructure
	protected variable rootDir .
	protected variable prevAliasCount 0


	# ------------------------------------------------------------------
	#  CONSTRUCTOR
	# ------------------------------------------------------------------
	constructor {root args} {
	    eval configure $args
		set rootDir $root
		set dbStructure(root,dir) $rootDir
		set dbStructure(root,file) ""
		set dbStructure(root,path) :
		set dbStructure(root,children) {}
		set dbStructure(root,childrenNames) {}
		set dbStructure(:,pathAlias) root
	}


	# ------------------------------------------------------------------
	#  DESTRUCTOR
	# ------------------------------------------------------------------
	destructor {}


	# ------------------------------------------------------------------
	#  METHOD:  configure - used to change public attributes
	# ------------------------------------------------------------------
	#method configure {config} {}


	# ------------------------------------------------------------------
	#  METHOD:  loadBranch - load a branch in the database from a
	#			 directory structure
	# ------------------------------------------------------------------
	method loadBranch {absPath file dir} {
		set pwd [pwd]
		if {[file isfile $file]} {
			if {![file readable $file]} {
				error "Point config file '$file' not readable."
			}
			set alias {}
			for_file line $file {
				if {[string match "Alias = *" $line ]} {
					set alias [lindex $line 2]
					break
				}
			}
		} elseif {[file isdirectory $file] && ("$absPath" == ":")} {
			set alias root
			cd $file
		} else {
			error "Branch '$file' not found."
		}

		set fileDir [file dirname $file]
		set ptName  [file tail $file]
		if {"$fileDir" != ""} {cd $fileDir}

		set children {}
		foreach f [exec ls] {
			if {[file isdirectory $f]} {lappend children $f}
		}

		set dbStructure($alias,childrenNames) $children
		set dbStructure($alias,dir)  ${dir}
		set dbStructure($alias,file)  ${dir}${file}

		if {"$absPath" == ":"} {
			set dbStructure($alias,path) ""
		} else {
			set dbStructure($alias,path) ${absPath}
		}
		set path $dbStructure($alias,path)
		set dbStructure($path,pathAlias) $alias

		foreach child $children {
			cd $child
			$this loadBranch ${path}:$child $child ${dir}$child/
			cd ..
		}
		cd $pwd
	}


	# ------------------------------------------------------------------
	#  METHOD:  getDir - given an alias, return the point config
	#		     file path
	# ------------------------------------------------------------------
	method getDir {alias} {
		return $dbStructure($alias,dir)
	}


	# ------------------------------------------------------------------
	#  METHOD:  getFile - given an alias, return the point config
	#		      file name
	# ------------------------------------------------------------------
	method getFile {alias} {
		return $dbStructure($alias,file)
	}


	# ------------------------------------------------------------------
	#  METHOD:  unloadBranch - unload a branch of the database to a 
	#			   directory structure
	# ------------------------------------------------------------------
	method unloadBranch {rootPoint file} {
		# to be implemented
	}


	# ------------------------------------------------------------------
	#  METHOD:  addBranch - add a point (which could be the root of a
	#			branch) to the database
	# ------------------------------------------------------------------
	method addBranch {point {overload {}}} {
		# Remember the cwp because we may need to temporarily change it.
		set currCwp $cwp

		# Assign a unique alias to the point if it doesn't already
		# have one.
		set alias [$point info variable alias -value]
		if {$alias == ""} {$point configure -alias P[incr aliasCounter]}
		set alias [$point info variable alias -value]

		# Check to see if the point name is a relative or absolute
		# path name.  If so, then filter the 'path' part out of the
		# name, change the cwp, and add the point.
		set name [$point info variable name -value]

		if [string match *:* $name] {
			set path [split $name :]
			set ptName [lvarpop path [expr [llength $path]-1]]
			$point configure -name $ptName
			if {[string index $name 0] == ":"} {
				set fullPath [join $path :]
			} elseif {"$cwp" == "root"} {
				set fullPath :[join $path :]
			} else {
				set fullPath $dbStructure($currCwp,path):[join $path :]
			}
			if {$fullPath == ""} {set fullPath :}

			# Since we have the full path, set the cwp to root
			# before loading the points in the path (otherwise it
			# fails if BranchRoot is set and a relative path is
			# used to specify a point)

			set cwp root
			if [catch {$this loadPath $fullPath} err] {
				set cwp $currCwp
				set msg "Point '$ptName' not loaded:\n"
				append msg "Parent path '$fullPath' does not exist."
				Message ERROR $msg
				return
			}

			set cwp $dbStructure($fullPath,pathAlias)
			set name $ptName
		}
		

		# Ensure that there is no point already with this name
		# as a child of the cwp
		if {[lempty $overload] && 
		    [lsearch $dbStructure($cwp,childrenNames) $name] >= 0} {
			set msg "Unable to add point '$name'\n"
			set cwpPath $dbStructure($cwp,path)
			append msg "branch already exists as a child of '$cwpPath'"
			Message ERROR $msg
			return
		}

		# Unload the point to a file and add it as a child of the cwp
		# DISABLED!! DISABLED!! DISABLED!!  In the class browser we
		# we don't need to actually unload the point
		#$point unload $dbStructure($cwp,dir)
		lappend dbStructure($cwp,children) $alias
		lappend dbStructure($cwp,childrenNames) $name

		# Add this point's structure info to the database
		set name [$point info variable name -value]
		set dbStructure($alias,children) {}
		set dbStructure($alias,childrenNames) {}
		set dbStructure($alias,dir)  $dbStructure($cwp,dir)/$name
		set dbStructure($alias,file)  $dbStructure($cwp,dir)/${name}/$name
		if {"$dbStructure($cwp,path)" == ":"} {
			set dbStructure($alias,path) :$name
		} else {
			set dbStructure($alias,path) $dbStructure($cwp,path):$name
		}
		set path $dbStructure($alias,path)
		set dbStructure($path,pathAlias) $alias

		# Keep track of the cwp because we need to change it
		# for this next loop
		set cwp $alias

		# Recursively add this point's children to the database
		foreach child [$point info variable children -value] {
			$this addBranch $child
			$point deleteChild $child
		}

		# Destroy the point and reset the cwp
		delete object $point
		set cwp $currCwp

		# Increment the count of points which have been unloaded
		incr pointsUnloaded
	}


	# ------------------------------------------------------------------
	#  METHOD:  loadPath - retrieve structure information from each RTAP
	#		       point config file contained in a pathname
	# ------------------------------------------------------------------
	method loadPath {path} {
		# Get the directory of the cwp
		set dir $dbStructure($cwp,dir)

		# Move down the path, loading the structure information
		# from each point's config file along the way
		set partialPath $dbStructure($cwp,path)
		foreach name [split $path :] {
			if {$name == ""} continue

			if {[$this pathLoaded? $partialPath$name]} {
				append dir /$name
				append partialPath $name:
				continue
			}

			set alias [AliasFromPConfigFile $dir/$name/$name]
			set dbStructure($alias,dir) $dir/$name
			set dbStructure($alias,file) $dir/$name/$name
			set dbStructure($alias,path) $partialPath$name
			set dbStructure($alias,childrenNames) {}

			set dbStructure($partialPath$name,pathAlias) $alias
			set children {}
			foreach f [glob $dir/$name/*] {
				lappend children [file tail $f]
			}

			lvarpop children [lsearch $children $name]
			foreach child $children {
#				lappend dbStructure($alias,children) \
				   [AliasFromPConfigFile $dir/$name/$child/$child]
#				lappend dbStructure($alias,childrenNames) \
				   [PointNameFromPConfigFile $dir/$name/$child/$child]
			}

			append dir /$name
			append partialPath $name:
		}
	}

	# ------------------------------------------------------------------
	#  METHOD:  getAlias - convert a symbolic address into an alias
	# ------------------------------------------------------------------
	method getAlias {symAddr} {
		if {[string index $symAddr 0] == ":"} {
			#
			# absolute path
			#
			if [info exists dbStructure($symAddr,pathAlias)] {
				# alias already in dbStructure
			} else {
				# alias must be extracted from files
				$this loadPath $symAddr
			}
			return $dbStructure($symAddr,pathAlias)
		} else {
			#
			# relative path
			#
			set tmpCwp [string trimright $dbStructure($cwp,path) :]
			set tmpAbs $tmpCwp:$symAddr

			if [info exists dbStructure($tmpAbs,pathAlias)] {
				# alias already in dbStructure
			} else {
				# alias must be extracted from files
				$this loadPath $tmpAbs
			}
			return $dbStructure($tmpAbs,pathAlias)

			set abs $dbStructure($cwp,path)$symAddr
			return $dbStructure($abs,pathAlias)
		}
	}


	# ------------------------------------------------------------------
	#  METHOD:  getAbsPath - convert an alias into an absolute pathname
	# ------------------------------------------------------------------
	method getAbsPath {alias} {
		if [info exists dbStructure($alias,path)] {
			return $dbStructure($alias,path)
		} else {
			error "Alias '$alias' does not exist"
		}
	}


	# ------------------------------------------------------------------
	#  METHOD:  deleteBranch - delete a branch in the database
	# ------------------------------------------------------------------
	method deleteBranch {symbolicAddress} {
		# to be implemented
	}

	# ------------------------------------------------------------------
	#  METHOD:  pathLoaded? - check if a point is already loaded into
	#		      the database
	# ------------------------------------------------------------------
	method pathLoaded? {path} {
		return [info exists dbStructure($path,pathAlias)]
	}


	# ------------------------------------------------------------------
	#  METHOD:  aliasLoaded? - check if a point is already loaded into
	#		      the database
	# ------------------------------------------------------------------
	method aliasLoaded? {alias} {
		return [info exists dbStructure($alias,path)]
	}


	# ------------------------------------------------------------------
	#  METHOD:  getAttrInfo - retrieve attribute information from a point
	#			  config file
	# ------------------------------------------------------------------
	method getAttrInfo {alias attrName} {
		if {![info exists dbStructure($alias,file)]} {
			error "Point '$alias' not loaded in database."
		}
		set fd [open $dbStructure($alias,file) r]

		set found 0
		while {!$found && ![eof $fd]} {
			set line [gets $fd]
			if {"$line" != "BEGIN ATTRIBUTE"} {
				continue
			}

			catch {unset attrInfo}
			set attrInfo(Record_Count) 0
			while {("$line" != "END") && ![eof $fd]} {
				set line [split [gets $fd] =]
				set type [join [lindex $line 0] _]
				set data [string trim [lindex $line 1]]

				if {("$type" == "Name") &&
				    ("$data" != "$attrName")} {
					break
				}
				set attrInfo($type) $data
			}
			if {[info exists attrInfo(Name)] &&
			    [info exists attrInfo(Type)] &&
			    [info exists attrInfo(De_Type)] &&
			    [info exists attrInfo(Record_Count)]} {
				close $fd
				return [list \
						$attrInfo(Type) \
						$attrInfo(De_Type) \
						$attrInfo(Record_Count)]
			}
		}
		close $fd
		error "Attribute '$attrName' not found"
	}

	# ------------------------------------------------------------------
	#  PUBLIC: cwp - current working position
	# ------------------------------------------------------------------
	public variable cwp {root} {
		set path [PathAddrToFile $dbStructure($cwp,path)]
		if {$path != ""} {
			set dbStructure($cwp,dir) [string trimright $rootDir/$path]
		}
	}


	# ------------------------------------------------------------------
	#  PUBLIC: name - the name of the database
	# ------------------------------------------------------------------
	public variable name {} {
	}


	# ------------------------------------------------------------------
	#  PUBLIC: pointsUnloaded - count of the number of points
	#			    which have been unloaded
	# ------------------------------------------------------------------
	public variable pointsUnloaded {0} {
	}

	# ------------------------------------------------------------------
	#  PUBLIC aliasCounter - a counter used to generate unique aliases
	# ------------------------------------------------------------------
	public variable aliasCounter {0} {
		if {$aliasCounter < $prevAliasCount} {
			error "Alias counter may only be increased"
		}
	}
}
