#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: dblcbINT_GROUP.tcl,v 1.8 2001/06/06 08:48:09 vltsccm Exp $" 
#
#
# INT_GROUP
# ----------------------------------------------------------------------
#  Define a group of integers and allocate/release members from the set.
#
#   PUBLIC ATTRIBUTES:
#
#	There are no public attributes.
#
#
#   METHODS:
#
#	configure .... used to change public attributes
#	allocate ..... allocate an integer from the group
#	release ...... put an integer back into the group so that it
#		       can be re-allocated later
#	resize ....... make the group size larger
#
#
#   COMMON ATTRIBUTES:
#
#	There are no common attributes.
#
#
#   PROTECTED ATTRIBUTES:
#
#	size .......... size of the group
#	inUse ......... array which marks which numbers are currently
#			in use
#	reuseStack .... stack of numbers which have been released
#	next .......... if the reuseStack is empty, this points to the
#			next number to be allocated
#
# ----------------------------------------------------------------------
#
#   AUTHOR:  Thomas J. Accardo
#            Computerized Processes Unlimited, Inc.
#            Metairie, Louisiana  USA
#            E-mail: tja@cpu.com
#
# ======================================================================
# Modified:
# who             when      what
# --------------  --------  ----------------------------------------------
# Erik Allaert    08/03/96  adapted to [incr Tcl] 2.0
# ======================================================================

# This will fool the auto_mkindex procedure into sourcing this file
# so that this class can be loaded.
proc INT_GROUP {} {;}; rename INT_GROUP {}

class INT_GROUP {
	# ------------------------------------------------------------------
	#   PROTECTED ATTRIBUTES:
	#
	#	size .......... size of the group
	#	inUse ......... array which marks which numbers are
	#			currently in use
	#	reuseStack .... stack of numbers which have been released
	#	next .......... if the reuseStack is empty, this points to
	#			the next number to be allocated
	# ------------------------------------------------------------------
	protected variable size 0
	protected variable inUse
	protected variable reuseStack {}
	protected variable next 0


	# ------------------------------------------------------------------
	#  CONSTRUCTOR
	# ------------------------------------------------------------------
	constructor {groupSize args} {
	    eval configure $args
	    set size $groupSize
	}

	# ------------------------------------------------------------------
	#  METHOD:  configure - used to change public variables
	# ------------------------------------------------------------------
	#method configure {config} {}


	# ------------------------------------------------------------------
	#  METHOD:  allocate - allocate an integer from the group
	# ------------------------------------------------------------------
	method allocate {{i -1}} {
		#
		# Check first to see if a specific number was requested from
		# the group
		#
		if {$i >= 0} {
			# Check to see if the requested number is already
			# in the reuse stack.
			set pos [lsearch $reuseStack $i]
			if {$pos >= 0} {
				lvarpop reuseStack $pos
				set inUse($i) {}
				return $i
			}

			# Make sure that the requested number is within
			# the group size limit.
			if {$i >= $size} {
				error "Requested member too large."
			}

			# Allocate it if possible
			if {[info exists inUse($i)]} {
				error "Element '$i' already allocated"
			}
			set inUse($i) {}

			# Just in case the next available element was the
			# one requested...
			while {[info exists inUse($next)]} {incr next}

			return $i
		}

		# ...otherwise, check the reuse stack first...
		if {![lempty $reuseStack]} {
			set i [lindex $reuseStack 0]
			set reuseStack [lrange $reuseStack 1 end]
			set inUse($i) {}
			return $i
		}

		# Make sure we have room to allocate another number
		if {$next >= $size} {
			error "No more available members to allocate."
		}

		# Allocate $next and mark it as used
		set i $next
		set inUse($i) {}

		# Find the next available slot
		while {[info exists inUse($next)]} {incr next}

		return $i
	}

	# ------------------------------------------------------------------
	#  METHOD:  release - put an integer back into the group so that it
	#	       can be re-allocated later
	# ------------------------------------------------------------------
	method release {i} {
		# You can't release it if it's not in use!
		if ![info exists inUse($i)] {
			error "Member \"$i\" was not in use."
		}

		# Mark it as no longer being used
		unset inUse($i)

		# Add this number to the stack of numbers which can
		# be reused
		set reuseStack [linsert $reuseStack 0 $i]

		return {}
	}

	# ------------------------------------------------------------------
	#  METHOD:  resize - make the group size larger
	# ------------------------------------------------------------------
	method resize {newSize} {
		if {$newSize < $size} {
			error "invalid size: set size cannot be decreased"
		}

		# increase the possible size
		set size $newSize
	}
}
