#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: dblcbNODE.tcl,v 1.8 2001/06/06 08:48:10 vltsccm Exp $" 
#
# NODE
# ----------------------------------------------------------------------
# This class encapsulates information about a node in a TREE object.
# A node contains information about its position, label, and the
# CLASS_DESCRIPTOR object which it represents.
#
#   PUBLIC ATTRIBUTES:
#
#	-gridX ................ Grid x position of this node within its
#                               TREE widget
#	-gridY ................ Grid y position of this node within its
#                               TREE widget
#       -foreground ........... Color of the text label
#       -background ........... Normal background color of the bounding
#                               rectangle
#       -selectbackground ..... Background color of the bounding
#                               rectangle when selected
#       -outline .............. Line color for the bounding rectangle
#       -selectoutline ........ Line color for the bounding rectangle
#       -font ................. Font for text label
#       -userData ............. Arbitrary textual data that can be
#                               associated with the node, e.g. the name
#                               of a point or class associated with the node
#
#
#   METHODS:
#
#	configure ............. Used to change public attributes
#       select ................ Change background color to 'selectbackground'
#                               and outline color to 'selectoutline'
#       deselect .............. Change background color to 'background
#                               and outline color to 'outline'
#       redraw ................ Redraw self on canvas at (gridX, gridY)
#       withdraw .............. Move the node off of the visible screen
#                               to ((gridX = -1), (gridY = -1))
#       movex ................. Adjust the node's gridX either left or right
#       movey ................. Adjust the node's gridY either up or down
#       bind .................. Bind an action to an event on a node
#
#
#   PROCS:
#
#	There are no procs.
#
#
#   COMMON ATTRIBUTES:
#
#	There are no common attributes.
#
#
#   PROTECTED ATTRIBUTES:
#
#       W ..................... Name of canvas widget where node is drawn
#       labelTAG .............. Canvas tag for the label
#       bboxTAG ............... Canvas tag for the bounding rectangle
#       connectorTAG .......... Canvas tag for the node connector
#       xScale ................ Magic number for spacing horizontally.
#       yScale ................ Magic number for spacing vertically
#                               xScale and yScale are really properties
#                               of the TREE, but they are set here once
#                               for convenience and to reduce the number
#                               of accesses of them via instance commands
#                               since they will likely never change!
# ----------------------------------------------------------------------
#
#   AUTHOR:  Thomas J. Accardo
#            Phone: (504) 889-2784
#            Computerized Processes Unlimited, Inc.
#            4200 S. I-10 Service Road Suite #205
#            Metairie, Louisiana  USA
#            E-mail: tja@cpu.com
#
# ======================================================================
# Modified:
# who             when        what
# --------------  ----------  ----------------------------------------------
# Erik Allaert    08/03/96    adapted to [incr Tcl] 2.0
# Erik Allaert    2000-02-02  adapted to Tcl/Tk 8.0, [incr Tcl] 3.0
# ======================================================================

# ======================================================================
# Trick the auto_mkindex procedure into generating an entry in the
# tclIndex file for this class
# ======================================================================
proc NODE {} {;}; rename NODE {}


class NODE {
    # ==================================================================
    #  COMMON DATA
    #
    #	There are no common attributes.
    # ==================================================================


    # ==================================================================
    #  PROTECTED DATA
    #       labelTAG .............. Canvas tag for the label
    #       bboxTAG ............... Canvas tag for the bounding rectangle
    #       connectorTAG .......... Canvas tag for the node connector
    #       xScale ................ Magic number for spacing horizontally
    #       yScale ................ Magic number for spacing vertically
    #       W ..................... Name of canvas widget where node is drawn
    # ==================================================================
    protected variable labelTAG             {}
    protected variable bboxTAG              {}
    protected variable connectorTAG         {}
    protected variable xScale               165
    protected variable yScale               20
    protected variable W                    {}
    protected variable nodeName		    {}


    # ------------------------------------------------------------------
    #  CONSTRUCTOR - Create the node.
    # ------------------------------------------------------------------
    constructor {tree label args} {
	eval configure $args
	# !!! Wacky dependency alert !!!
	# The canvas widget name is obtained through a protected variable
	# of the TREE class and a pathname from there.  The path name
	# depends on how the canvas is created in the constructor for the
	# TREE class.  By putting this piece of ugliness here, I hope to
	# prevent any other needs for direct knowledge of the TREE widget
	# construction elsewhere in the NODE class.
	set W [$tree queryWidget].f2.summary

	set nodeName [namespace tail $this]
	$W delete TAG_$nodeName

	set labelTAG TAG_LABEL_$nodeName
	$W create text [expr $gridX * $xScale + $xScale / 2] [expr ($gridY + 1) * $yScale] \
		-font $font \
		-anchor w \
		-text " $label " \
		-fill $foreground \
		-tags [list TAG_$nodeName $labelTAG]

	set connectorTAG TAG_CONNECTOR_$nodeName
	$W create line \
		[expr $gridX * $xScale + $xScale / 2 - 10] \
		[expr ($gridY + 1) * $yScale] \
		[expr $gridX * $xScale + $xScale / 2] \
		[expr ($gridY + 1) * $yScale] \
		-fill $foreground \
		-tags [list TAG_$nodeName $connectorTAG]

	set bboxTAG TAG_BBOX_$nodeName
	eval $W create rectangle [$W bbox $labelTAG] \
		-outline $outline \
		-fill $background \
		-tags {[list TAG_$nodeName $bboxTAG]}
	$W raise $labelTAG $bboxTAG
    }


    # ------------------------------------------------------------------
    #  DESTRUCTOR - Destroy the node by deleting its canvas objects
    # ------------------------------------------------------------------
    destructor {
	$W delete \
		TAG_$nodeName \
		TAG_LABEL_$nodeName \
		TAG_CONNECTOR_$nodeName \
		TAG_BBOX_$nodeName
    }


    # ------------------------------------------------------------------
    #  METHOD:  configure - used to change public attributes
    # ------------------------------------------------------------------
    #method configure {config} {}


    # ------------------------------------------------------------------
    #  METHOD:  select - Change background color to selectbackground
    #                    and outline color to selectoutline
    # ------------------------------------------------------------------
    method select {} {
	$W itemconfigure TAG_BBOX_$nodeName \
		-outline $selectoutline \
		-fill $selectbackground
    }


    # ------------------------------------------------------------------
    #  METHOD:  deselect - Change background color to normalbackground
    #                      and outline color to normaloutline
    # ------------------------------------------------------------------
    method deselect {} {
	$W itemconfigure TAG_BBOX_$nodeName \
		-outline $outline \
		-fill $background
    }

    # ------------------------------------------------------------------
    #  METHOD:  redraw - Redraw self on canvas at (gridX, gridY)
    # ------------------------------------------------------------------
    method redraw {} {
	$W coords TAG_LABEL_$nodeName \
		[expr $gridX * $xScale + $xScale / 2] \
		[expr ($gridY + 1) * $yScale]
	$W coords TAG_CONNECTOR_$nodeName \
		[expr $gridX * $xScale + $xScale / 2 - 10] \
		[expr ($gridY + 1) * $yScale] \
		[expr $gridX * $xScale + $xScale / 2] \
		[expr ($gridY + 1) * $yScale]
	eval $W coords TAG_BBOX_$nodeName [$W bbox TAG_LABEL_$nodeName]
    }


    # ------------------------------------------------------------------
    #  METHOD:  withdraw - Move the node off of the visible screen
    #                      to ((gridX = -1), (gridY = -1))
    # ------------------------------------------------------------------
    method withdraw {} {
	set gridX -1
	set gridY -1
	$this redraw
    }


    # ------------------------------------------------------------------
    #  METHOD:  movex - Adjust the node's gridY either left or right
    # ------------------------------------------------------------------
    method movex {dx} {
	incr gridX $dx
    }


    # ------------------------------------------------------------------
    #  METHOD:  movey - Adjust the node's gridY either up or down
    # ------------------------------------------------------------------
    method movey {dy} {
	incr gridY $dy
    }


    # ------------------------------------------------------------------
    #  METHOD:  bind - Bind an action to an event on a node
    # ------------------------------------------------------------------
    method bind {event handler} {
	set w [$this info variable W -value]
	$w bind TAG_$nodeName $event $handler
    }


    # ------------------------------------------------------------------
    #  PUBLIC: gridX - Grid x position of the node withing its TREE
    #                  widget
    # ------------------------------------------------------------------
    public variable gridX {-1} {
	if {[catch {format %d $gridX}]} {
	    error "Grid coordinate must be an integer"
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: gridY - Grid y position of the node withing its TREE
    #                  widget
    # ------------------------------------------------------------------
    public variable gridY {-1} {
	if {[catch {format %d $gridY}]} {
	    error "Grid coordinate must be an integer"
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: foreground - Color of the text label
    # ------------------------------------------------------------------
    public variable foreground {black} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: background - Normal background color of the bounding
    #                       rectangle
    # ------------------------------------------------------------------
    public variable background {bisque} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: selectbackground - Background color of the bounding
    #                             rectangle when selected
    # ------------------------------------------------------------------
    public variable selectbackground {bisque3} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: outline - Line color for the bounding rectangle
    # ------------------------------------------------------------------
    public variable outline {bisque} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: selectoutline - Line color for the bounding rectangle
    #                          when selected
    # ------------------------------------------------------------------
    public variable selectoutline {black} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: font - Font for text label
    # ------------------------------------------------------------------
    public variable font {helvb12} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: userData - Arbitrary textual data that can be associated
    #                     with the node, e.g. the name of a point or
    #                     class associated with the node
    # ------------------------------------------------------------------
    public variable userData {} {
    }
}
