#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: dblcbPOINT.tcl,v 1.8 2001/06/06 08:48:10 vltsccm Exp $" 
#
#
# POINT
# ----------------------------------------------------------------------
# Implements an RTAP point object.  An RTAP point object contains
# various header information (name, alias, point class, etc.) as
# well as the names of a collection of attribute objects.  It also
# contains the names of its parent and children.
#
#   PUBLIC ATTRIBUTES:
#
#	-alias ......... alias of the point
#	-name .......... name of the point
#	-class ......... point class
#	-className ..... point class name
#	-categories .... categories which the point belongs to
#	-residence ..... point residence
#	-ceIndicator ... Calculation Engine state
#	-ceOrder ....... attribute order for Calculation Engine
#	-usage ......... usage flags
#	-parent ........ parent POINT
#	-children ...... list of children POINTs
#	-storage ....... either STATIC or AUTOMATIC
#	-needClassAttr . whether or not this point should have a 'class'
#			 attribute when it is unloaded
#
#
#   METHODS:
#
#	configure .......... used to change public attributes
#	unload ............. unload point into an RTAP point config file
#	insertChild ........ add a child point
#	deleteChild ........ delete a child point
#	deleteChildByName .. delete a child point given a point name
#	addAttr ............ add an ATTRIBUTE to the point
#	deleteAttr ......... delete an ATTRIBUTE from the point
#	replaceAttr ........ replace an existing ATTRIBUTE with another
#	replicate .......... create a replicate of self
#	addComment ......... insert a comment line into the point config file
#	queryAttributes .... returns the ATTRIBUTEs for this point
#
#
#   COMMON ATTRIBUTES:
#	formatString .. format string used to speed production of the
#			unload file string - the format string is used
#			in place of multiple 'append' calls
#
#
#   PROTECTED ATTRIBUTES:
#	attrList ....... list of ATTRIBUTES - one for each attribute of
#			 the point
#	commentList .... a list of comment lines which will be put in the
#			 point config file
#
#
# ----------------------------------------------------------------------
#
#   AUTHOR:  Thomas J. Accardo
#            Computerized Processes Unlimited, Inc.
#            Phone: (504) 889-2784
#            E-mail: tja@cpu.com
#
# ======================================================================
# Modified:
# who             when        what
# --------------  ----------  ----------------------------------------------
# Erik Allaert    08/03/96    adapted to [incr Tcl] 2.0
# Erik Allaert    2000-02-02  adapted to Tcl/Tk 8.0, [incr Tcl] 3.0
# ======================================================================

class POINT {

    common formatString [namespace eval :: eval {
	global INTROOT VLTROOT
	set commentFile include/dblPtComment.h
	if {[file exists $commentFile]} {
	} elseif {[file exists [set f [file join .. $commentFile]]]} {
	    set commentFile [file join .. $commentFile]
	} elseif {[file exists [set f [file join $INTROOT $commentFile]]]} {
	    set commentFile [file join $INTROOT $commentFile]
	} elseif {[file exists [set f [file join $VLTROOT $commentFile]]]} {
	    set commentFile [file join $VLTROOT $commentFile]
	} 
	if {[catch {read_file $commentFile} comment]} {
	    set comment {}
	}
	unset commentFile
	append comment "\n#" \
		"\n# Date Created: [clock format [clock seconds] -format %c]" \
		"\n# VLTROOT     : $env(VLTROOT)" \
		"\n#" \
		"\n%s" \
		"\nName = %s" \
		"\nAlias = %s" \
		"\nResidence = %s" \
		"\nCategories = %s" \
		"\nCE Indicator = %s" \
		"\nCE Order = %s" \
		"\nClass = %s"
    } ]
	

    # ==================================================================
    #  PROTECTED DATA
    #	attrList ...... list of ATTRIBUTES - one for each attribute
    #			of the point
    #	commentList ... a list of comment lines which will be put in
    #			the point config file
    # ==================================================================
    protected variable attrList {}
    protected variable commentList {}


    # ------------------------------------------------------------------
    #  CONSTRUCTOR - create new point
    # ------------------------------------------------------------------
    constructor {args} {
	eval configure $args
    }


    # ------------------------------------------------------------------
    #  DESTRUCTOR - destroy the point, its attributes and all of
    #		its children
    # ------------------------------------------------------------------
    destructor {
	# delete all of the attributes
	foreach attr $attrList {
	    # If for some reason this fails, we still want to delete
	    # the POINT
	    catch {
		if [$attr info variable deletable -value] {delete object $attr}
	    }
	}

	# delete all of the children
	foreach pt $children {
	    delete object $pt
	}
    }


    # ------------------------------------------------------------------
    #  METHOD:  config - change public attributes
    # ------------------------------------------------------------------
    #method configure {config} {}


    # ------------------------------------------------------------------
    #  METHOD:  unload - unload point into an RTAP point config file
    # ------------------------------------------------------------------
    method unload {dir} {
	# Prepare any comments
	set commentStr {}
	foreach line $commentList {
	    append commentStr "# $line\n"
	}

	# These things appear in every point.  Everything
	# else is context dependent.
	set str [format $formatString $commentStr \
		$name $alias $residence $categories \
		$ceIndicator $ceOrder $class]

	# Unload usage flags if present
	if {"$usage" != ""} {
	    append str [UsageString $usage]
	}

	# Add the 'class' attribute
	if $needClassAttr {
	    $this addAttr [namespace inscope :: ATTRIBUTE #auto \
		    -name class \
		    -deType rtBYTES32 \
		    -value <alias>$className]
	}

	# Unload each of the attributes
	foreach attr $attrList {
	    if {[$attr info variable storage -value] != "STATIC"} {
		append str [$attr unloadStr $alias]
	    }
	}

	# Create the point directory if necessary
	if {![file isdirectory "$dir/$name"]} {
	    if {[catch {exec mkdir $dir/$name} ret]} {
		set msg "Unable to create point directory:\n$ret"
		Message ERROR $msg
		return
	    }
	}

	# Create the RTAP point config file
	set fd [open "$dir/$name/$name" {WRONLY CREAT TRUNC}]
	puts $fd $str
	close $fd
	puts stdout $dir/$name/$name
    }


    # ------------------------------------------------------------------
    #  METHOD:  insertChild - add a child POINT
    # ------------------------------------------------------------------
    method insertChild {point {overloadSrc point}} {
	if {[$this info class] != [$point info class]} {
	    error "$point is not a valid child point object."
	}

	set pointNames {}
	foreach p $children {lappend pointNames [$p info variable name -value]}
	set index [lsearch $pointNames [$point info variable name -value]]

	# This child already exists, so we must attempt to overload it with
	# the new definition.
	if {$index >= 0} {
	    set oldClass [[lindex $children $index] info variable className -value]
	    set newClassName [$point info variable className -value]
	    set heritage [ClassHeritage $newClassName]

	    if {([lsearch $heritage $oldClass] < 0) &&
	    ("$oldClass" != "$newClassName")} {
		set msg "Failed to overload attribute:\n"
		append msg "'$newClassName' is not a subclass of"
		append msg " the existing point's class '$oldClass'"
		error $msg
	    }

	    # Add this point to the list of children
	    set oldPoint [lvarpop children $index]

	    # If we are really overloading with a subclass, then the "old"
	    # point should be the subclass template point, not the existing
	    # point.
	    if {$oldClass != $newClassName} {
		set oldPoint [$newClassName info variable templatePoint -value]
	    }


	    # Copy all of data from the attributes of the existing
	    # point to the new point's attributes.
	    foreach attr [$oldPoint info variable attrList -value] {
		set tmpAttrListByName([$attr info variable name -value]) $attr
	    }
	    foreach attr [$point info variable attrList -value] {
		set tmpName [$attr info variable name -value]
		if {[info exists tmpAttrListByName($tmpName)]} {
		    set configData [$tmpAttrListByName($tmpName) queryConfigData]
		    eval \$attr configure $configData
		}
	    }
	    catch {unset tmpAttrListByName}
	    foreach attr [$point info variable attrList -value] {
		set tmpAttrListByName([$attr info variable name -value]) $attr
	    }
	    foreach attr [$oldPoint info variable attrList -value] {
		set tmpName [$attr info variable name -value]
		if {![info exists tmpAttrListByName($tmpName)]} {
		    set tmpAttr [namespace inscope :: ATTRIBUTE #auto]
		    set configData [$attr queryConfigData]
		    eval \$tmpAttr configure $configData
		    $point addAttr $tmpAttr
		}
	    }
	}

	# Add the class type attribute
	lappend children $point
    }


    # ------------------------------------------------------------------
    #  METHOD:  deleteChild - delete a child POINT
    # ------------------------------------------------------------------
    method deleteChild {point} {
	lvarpop children [lsearch $children $point]
    }


    # ------------------------------------------------------------------
    #  METHOD:  deleteChildByName - delete a child POINT given its name
    # ------------------------------------------------------------------
    method deleteChildByName {name} {
	set i 0
	foreach child $children {
	    if {"[$child info variable name -value]" == "$name"} {
		lvarpop children $i
		return
	    }
	    incr i
	}
	error "Point '$name' not a child"
    }


    # ------------------------------------------------------------------
    #  METHOD:  addAttr - add an ATTRIBUTE to the point
    # ------------------------------------------------------------------
    method addAttr {attribute {position end}} {
	if {[namespace tail [$attribute info class]] != "ATTRIBUTE"} {
	    error "$attribute is not a valid ATTRIBUTE object."
	}

	# If this class already has the named attribute, then
	# overload it with this new one and remove the other one
	# from the inherited attribute list.

	# Get the names of existing attributes
	set attrNames {}
	foreach a $attrList {lappend attrNames [$a info variable name -value]}
	set index [lsearch $attrNames [$attribute info variable name -value]]

	# If we found the new attribute name in the list of already
	# existing names, then overload it.
	if {$index >= 0} {
	    set position $index
	    set tmpAttr [lindex $attrList $index]

	    # If no deType was specified for the new attribute,
	    # then inherit the one from the attribute its overloading
	    if {"[$attribute info variable deType -value]" == ""} {
		$attribute configure -deType [$tmpAttr info variable deType -value]
	    }

	    # If this is a table attribute and no field names were
	    # specified, then inherit the ones from the attribute
	    # being overloaded.
	    if {"[$attribute info variable fieldNames -value]" == ""} {
		$attribute configure -fieldNames [$tmpAttr info variable fieldNames -value]
	    }

	    # The attribute should keep the same source class as
	    # the existing attribute
	    $attribute configure -sourceClass [$tmpAttr info variable sourceClass -value]

	    if {[catch {OverloadAttrOk $tmpAttr $attribute} err]} {
		set msg "Cannot overload attribute '[$attribute info variable name -value]':\n"
		append msg "	$err"
		error $msg
	    } else {
		set oldAttr [lvarpop attrList \
			[lsearch $attrNames [$attribute info variable name -value]]]
		# Copy everything we know about the old attribute
		# into the new attribute
		set itemList [list \
			value \
			numRecords \
			readCateg writeCateg \
			usage definition \
			scanAddr \
			pollType pollTime \
			deadbandType deadband]
		foreach item $itemList {
		    $attribute configure -$item [$oldAttr info variable $item -value]
		}
	    }
	}
	if {($position == "end") || ($position < 0)} {
	    lappend attrList $attribute
	} else {
	    set attrList [linsert $attrList $position $attribute]
	}
    }



    # ------------------------------------------------------------------
    #  METHOD:  deleteAttr - delete an ATTRIBUTE from the point
    # ------------------------------------------------------------------
    method deleteAttr {attributeName} {
	lvarpop attrList [lsearch $attrList [list * $attributeName]]
    }


    # ------------------------------------------------------------------
    #  METHOD:  replaceAttr - replace an existing ATTRIBUTE with another
    # ------------------------------------------------------------------
    method replaceAttr {attribute} {
	set attributeName [$attribute info variable name -value]
	set position [lsearch $attrList [list * $attributeName]]
	lvarpop attrList $position
	$this addAttr $attribute $position
    }


    # ------------------------------------------------------------------
    #  METHOD:  replicate - create a replicate of self
    # ------------------------------------------------------------------
    method replicate {{needAttrs 0}} {
	set dup [namespace inscope :: POINT #auto]
	$dup configure \
		-alias $alias \
		-name $name \
		-class $class \
		-className $className \
		-categories $categories \
		-residence $residence \
		-ceIndicator $ceIndicator \
		-ceOrder $ceOrder \
		-usage $usage \
		-parent $parent

	foreach child $children {
	    if {[$child info variable storage -value] == "AUTOMATIC"} {
		$dup insertChild [$child replicate 1]
	    }
	}

	if {$needAttrs} {
	    foreach attr $attrList {
		set tmpAttr [namespace inscope :: ATTRIBUTE #auto]
		set configData [[lindex $attr 0] queryConfigData]
		eval \$tmpAttr configure $configData
		$tmpAttr configure -deletable 1
		$dup addAttr $attr
	    }
	}
	return $dup
    }


    # ------------------------------------------------------------------
    #  METHOD: addComment - insert a comment line into the point config file
    # ------------------------------------------------------------------
    method addComment {str} {
	lappend commentList $str
    }


    # ------------------------------------------------------------------
    #  METHOD: queryAttributes - returns the ATTRIBUTEs for this point
    # ------------------------------------------------------------------
    method queryAttributes {} {
	return $attrList
    }



    # ------------------------------------------------------------------
    #  PUBLIC: alias - RTAP point alias restricted to ESO specs.
    # ------------------------------------------------------------------
    public variable alias {} {
	global ::CURRENT_FILE ::CURRENT_LINE

	# length must be < 20
	if {[string length $alias] > 19} {
	    set msg "point alias '$alias' too long: "
	    append msg "should be < 20 chars"
	    Message Warning $msg
	}

	# check for illegal characters
	regsub -all -- {[a-zA-Z0-9_]+} $alias {} illegalChars
	###set illegalChars [translit {a-zA-Z0-9_} {} $alias]
	if {[string length $illegalChars]} {
	    set msg "Invalid point alias '$alias'\n"
	    append msg "Contains illegal char(s) '$illegalChars'"
	    Message Warning $msg
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: name - RTAP point name restricted to ESO specs
    # ------------------------------------------------------------------
    public variable name {} {
	global ::CURRENT_FILE ::CURRENT_LINE

	set pathNameList [split $name :]
	set ptName [lindex $pathNameList [expr [llength $pathNameList]-1]]

	# length must be < 20
	if {[string length $ptName] > 19} {
	    set msg "point name '$ptName' too long: "
	    append msg "should be < 20 chars"
	    Message Warning $msg
	}

	# check for illegal characters
	regsub -all -- {[a-zA-Z0-9_]+} $ptName {} illegalChars
	###set illegalChars [translit {a-zA-Z0-9_} {} $ptName]
	if {[string length $illegalChars]} {
	    set msg "Invalid point name '$ptName'\n"
	    append msg "Contains illegal char(s) '$illegalChars'"
	    Message Warning $msg
	}
    }

    # ------------------------------------------------------------------
    #  PUBLIC: class - RTAP point class identifier.
    # ------------------------------------------------------------------
    public variable class {0} {
	# the class must be an integer
	if {![ctype digit $class]} {
	    error "class must be an integer"
	}

	# the class number must be between 0 and 65535 inclusive
	if {($class < 0) || ($class > 65535)} {
	    error "illegal class: must be from 0 to 65535"
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: className - point class name
    # ------------------------------------------------------------------
    public variable className {} {
    }

    # ------------------------------------------------------------------
    #  PUBLIC: categories - RTAP point categories mask
    # ------------------------------------------------------------------
    public variable categories {11111111111111111111111111111111} {
	# category mask is 32 bits
	if {[string length $categories] != 32} {
	    error "category mask must be 32 characters in length"
	}

	# a mask is made of 0's and 1's
	regsub -all -- {[01]+} $categories {} illegalChars
	###set illegalChars [translit {01} {} $categories]
	if {[string length $illegalChars]} {
	    set msg "illegal char(s) in Categories mask '$illegalChars'"
	    append msg ": should contain only 0's and 1's"
	    error $msg
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: residence - RTAP point residence
    # ------------------------------------------------------------------
    public variable residence {RAM} {
	# the residence must be one of these words
	set tmpResidence [string toupper $residence]
	if {($tmpResidence != "RAM") && ($tmpResidence != "DISK")} {
	    error "illegal Residence: should be RAM or Disk"
	}
	set residence [string toupper $residence]
    }


    # ------------------------------------------------------------------
    #  PUBLIC: ceIndicator - Calculation Engine indicator
    # ------------------------------------------------------------------
    public variable ceIndicator {ENABLED} {
	# the ceIndicator must be one of these words
	set tmpCeInd [string toupper $ceIndicator]
	if {($tmpCeInd != "ENABLED") &&
	($tmpCeInd != "DISABLED") &&
	($tmpCeInd != "OPTIMIZED")} {
	    set msg "illegal CE Indicator '$ceIndicator'"
	    append msg ": should be Enabled, Disabled, or Optimized"
	    error $msg
	}
	set ceIndicator [string toupper $ceIndicator]
    }

    # ------------------------------------------------------------------
    #  PUBLIC: ceOrder - Calculation Engine attribute calculation order
    # ------------------------------------------------------------------
    public variable ceOrder {NATURAL} {
	# the ceOrder must be one of these words
	set tmpCeOrder [string toupper $ceOrder]
	if {($tmpCeOrder != "NATURAL") &&
	($tmpCeOrder != "USER")} {
	    set msg "illegal CE Order '$ceOrder'"
	    append msg ": should be Natural or User"
	    error $msg
	}
	set ceOrder [string toupper $ceOrder]
    }

    # ------------------------------------------------------------------
    #  PUBLIC: usage - usage flags
    # ------------------------------------------------------------------
    public variable usage {} {
	# usage numbers should be 128-255
	foreach num $usage {
	    if {![ctype digit $num]} {
		error "Usage indicator '$num' must be an integer"
	    }
	    if {($num < 128) || ($num > 255)} {
		set msg "Usage indicator '$num' is out of range"
		append msg ": should be in range 128-255"
		error $msg
	    }
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: parent - alias of parent point
    # ------------------------------------------------------------------
    public variable parent {point} {}


    # ------------------------------------------------------------------
    #  PUBLIC: children - list of alias of children points
    # ------------------------------------------------------------------
    public variable children {} {}


    # ------------------------------------------------------------------
    #  PUBLIC: storage - either STATIC or AUTOMATIC
    # ------------------------------------------------------------------
    public variable storage {AUTOMATIC} {
	if {([string toupper $storage] != "STATIC") &&
	([string toupper $storage] != "AUTOMATIC")} {
	    error "Invalid storage type \"$storage\""
	}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: needClassAttr - whether or not this point should have a
    #			   'class' attribute when it is unloaded
    # ------------------------------------------------------------------
    public variable needClassAttr {0} {
    }


    # ------------------------------------------------------------------
    # PROC: UsageString - returns the unload string for a set of
    #		      usage flags
    # ------------------------------------------------------------------
    proc UsageString {usage} {
	set usageList {}
	foreach u $usage {
	    append usageList $u,
	}
	set usageList [string trimright $usageList ,]
	return "Usage = $usageList\n"
    }

    # ------------------------------------------------------------------
    #  PROC: OverloadAttrOk - check to see if attr1 can be overloaded
    #			  with attr2
    # ------------------------------------------------------------------
    proc OverloadAttrOk {attr1 attr2} {
	set name1 [$attr1 info variable name -value]
	set name2 [$attr2 info variable name -value]

	# First check to see if attr1 is allowed to be overloaded
	if {![$attr1 info variable overloadable -value]} {
	    error "POINT's attribute '$name1' cannot be overloaded. Define it in a CLASS."
	}

	# Check the type - SCALAR, VECTOR, or TABLE
	set type1 [$attr1 info variable type -value]
	set type2 [$attr2 info variable type -value]
	if {$type1 != $type2} {
	    error "Attribute '$name2' not of type $type1"
	}

	switch $type1 {
	    SCALAR	{
		# Check the deType
		set deType1 [$attr1 info variable deType -value]
		set deType2 [$attr2 info variable deType -value]
		if {$deType1 != $deType2} {
		    error "deType mismatch between '$deType1' and '$deType2'"
		}
	    }
	    VECTOR	{
		# Check the deType
		set deType1 [$attr1 info variable deType -value]
		set deType2 [$attr2 info variable deType -value]
		if {$deType1 != $deType2} {
		    error "deType mismatch between '$deType1' and '$deType2'"
		}
	    }
	    TABLE	{
		# Fold the field names and attributes into a single
		# list and then compare.
		set deType1 [$attr1 info variable deType -value]
		set fieldNames1 [$attr1 info variable deType -value]
		set i 0
		foreach field $deType1 {
		    lappend typeList1 [list $field [lindex $fieldNames1 $i]]
		    incr i
		}

		set deType2 [$attr2 info variable deType -value]
		set fieldNames2 [$attr2 info variable deType -value]
		set i 0
		foreach field $deType2 {
		    lappend typeList2 [list $field [lindex $fieldNames2 $i]]
		    incr i
		}

		if {[lsort $typeList1] != [lsort $typeList2]} {
		    error "deType mismatch between '$typeList1' and '$typeList2'"
		}
	    }
	}
    }
}
