#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: dblcbTREE.tcl,v 1.8 2001/06/06 08:48:12 vltsccm Exp $" 
#
#
# TREE
# ----------------------------------------------------------------------
# The TREE class is an implementation of a tree widget in Tk.  This
# particular implementation is intended specifically for use with the
# VLT Database Loader Class Browser tool, and may have specific
# features for this purpose, i.e. it is not necessarily intended for
# general use.
#
#   PUBLIC ATTRIBUTES:
#
#	-background ........... Background color for the tree as well as
#                               the default color for all NODES in the
#                               tree
#	-foreground ........... Color for the lines connecting nodes on
#                               the tree 
#       -font ................. Font for text labels in nodes
#       -actionCommand ........ Command to execute when a node is
#                               double-clicked with mouse button 1
#       -selectCommand ........ Command to execute when a node is
#                               single-clicked with mouse button 1
#       -getChildrenCommand ... Command to execute when a node is
#                               expanded.  It should return a list of
#                               NODES which are children of the selection
#
#   METHODS:
#
#	configure ............. Used to change public attributes
#	insertChild ........... Insert a child node into the node tree
#       deleteBranch .......... Delete a branch of the node tree
#       refresh ............... Redraw all or part of the node tree
#       reposition ............ Set the grid position of a branch in
#                               the TREE
#       move .................. Move the a branch in the TREE by some
#                               relative amount of grid units
#       queryWidget ........... Returns the name of the container frame
#                               widget
#       select ................ Change the selected NODE and execute
#                               the selectCommand
#       selectParent .......... Change the selected NODE to the parent
#                               of a given NODE and execute the
#                               selectCommand
#       action ................ Perform the actionCommand on a node
#       toggleChildDisplay .... Expand or contract the display of the
#                               children of a node
#       toggleParentDisplay ... Expand or contract the display of the
#                               parent of a node
#       clearSelection ........ Deselect any selected NODE
#
#
#   PROCS:
#
#	There are no procs.
#
#
#   COMMON ATTRIBUTES:
#
#	There are no common attributes.
#
#
#   PROTECTED ATTRIBUTES:
#
#       W ..................... The parent widget (a frame) for the
#                               canvas and scrollbars
#       C ..................... For convenience, the name of the
#                               canvas widget
#       structure ............. Array which contains the structural
#                               relationships between nodes
#       xScale ................ Width of x grid for spacing horizontally
#       yScale ................ Height of y grid for spacing vertically
#       selectedNode .......... The currently selected NODE
#
# ----------------------------------------------------------------------
#
#   AUTHOR:  Thomas J. Accardo
#            Phone: (504) 889-2784
#            Computerized Processes Unlimited, Inc.
#            4200 S. I-10 Service Road Suite #205
#            Metairie, Louisiana  USA
#            E-mail: tja@cpu.com
#
# ======================================================================
# Modified:
# who             when      what
# --------------  --------  ----------------------------------------------
# Erik Allaert    08/03/96  adapted to [incr Tcl] 2.0
# ======================================================================

# ======================================================================
# Trick the auto_mkindex procedure into generating an entry in the
# tclIndex file for this class
# ======================================================================
proc TREE {} {;}; rename TREE {}



class TREE {
    # ==================================================================
    #  COMMON DATA
    # ==================================================================


    # ==================================================================
    #  PROTECTED DATA
    #       W ..................... The parent widget (canvas) for the
    #                               tree widget
    #       C ..................... For convenience, the name of the
    #       structure ............. Array which contains the structural
    #                               relationships between nodes
    #                               canvas widget
    #       xScale ................ Width of x grid for spacing horizontally
    #       yScale ................ Height of y grid for spacing vertically
    #       selectedNode .......... The currently selected NODE
    # ==================================================================
    protected variable W                    {}
    protected variable structure
    protected variable C                    {}
    protected variable xScale               165
    protected variable yScale               20
    protected variable selectedNode         {}

    # ------------------------------------------------------------------
    #  CONSTRUCTOR - Create a TREE instance.
    # ------------------------------------------------------------------
    constructor {w args} {
	global ::tk_version

	eval configure $args

	if {[lindex [split $tk_version .] 0] >= 4} {
	    set SCROLLINCR "-xscrollincrement 1 -yscrollincrement 1"
	} else {
	    set SCROLLINCR "-scrollincrement 1"
	}

	set W $w

	if {[winfo exists $W]} {
	    error "window '$W' already exists"
	}

	frame $W

	frame $W.f1
	frame $W.f1.spacer
	scrollbar $W.f1.vertical -command "$W.f2.summary yview"

	frame $W.f2
	scrollbar $W.f2.horizontal -command "$W.f2.summary xview" -orient horizontal

	set C $W.f2.summary

	canvas $W.f2.summary \
		-relief sunken \
		-borderwidth 2 \
		-scrollregion {0 0 1 1} \
		-xscrollcommand "$W.f2.horizontal set" \
		-yscrollcommand "$W.f1.vertical set"
	eval [concat [list $W.f2.summary] configure $SCROLLINCR]

	pack $W.f2 -side left -fill both -expand 1 
	pack $W.f2.summary -side top -fill both -expand 1
	pack $W.f2.horizontal -fill x

	pack $W.f1 -fill both -expand 1
	pack $W.f1.vertical -side top -fill y -expand 1
	pack $W.f1.spacer -pady [expr .5 * [winfo reqheight $W.f2.horizontal]]

	# The empty node must have no children.  This will be the
	# boundary condition which lets us add children to the root
	# node
	set structure(,children) {}

	# Create a binding so that scrolling can be done by dragging
	# mouse button 2
	$W.f2.summary configure
	bind $W.f2.summary <2> {%W scan mark %x %y}
	bind $W.f2.summary <B2-Motion> {%W scan dragto %x %y}
    }


    # ------------------------------------------------------------------
    #  DESTRUCTOR - 
    # ------------------------------------------------------------------
    destructor {
    }


    # ------------------------------------------------------------------
    #  METHOD:  configure - used to change public variables
    # ------------------------------------------------------------------
    #method configure {config} {}


    # ------------------------------------------------------------------
    #  METHOD:  insertChild - insert a child node into the node tree
    # ------------------------------------------------------------------
    method insertChild {parent child} {
	# If this 'parent' doesn't have a parent, then give it a default
	# for parent and the other required attributes.  (This will
	# only happen for the first NODE added to the TREE.  Also, make
	# this a child of the empty node so that we can easily get to
	# the root of the tree without actually knowing what the root
	# node is.)
	if {![info exists structure($parent,parent)] || ($parent == "")} {
	    set structure($parent,parent) ""
	    set structure($parent,children) ""
	    set structure($parent,verticalSpace) 1
	    set gridX -1
	    set gridY 0
	} else {
	    # Get the position of the parent
	    set gridX [$parent info variable gridX -value]
	    set gridY [$parent info variable gridY -value]
	}

	# Find out how much room (in grid spaces) the current
	# children of $parent require vertically
	set vertOffset 0
	foreach sibling $structure($parent,children) {
	    incr vertOffset $structure($sibling,verticalSpace)
	}

	# Add the child into the parent/child structure
	set structure($child,parent) $parent
	set structure($child,children) ""
	set structure($child,verticalSpace) 1

	lappend structure($parent,children) $child

	# Configure the position of the child
	$child configure \
		-gridX [expr $gridX + 1] \
		-gridY [expr $gridY + $vertOffset]

	# If the parent is not the root node...
	if {($parent != "") && ([$C find withtag TAG_CHILD_CONNECTOR_$parent] == "")} {
	    # Draw the line which connects all of the children nodes
	    # to each other and to this parent
	    $C delete TAG_CHILD_CONNECTOR_$parent
	    set height $structure($parent,verticalSpace)
	    $C create line \
		    [expr $gridX                 * $xScale + $xScale / 2     ] \
		    [expr ($gridY + 1)           * $yScale                   ] \
		    [expr ($gridX + 1)           * $xScale + $xScale / 2 - 10] \
		    [expr ($gridY + 1)           * $yScale                   ] \
		    [expr ($gridX + 1)           * $xScale + $xScale / 2 - 10] \
		    [expr ($gridY + $height)     * $yScale                   ] \
		    -fill $foreground \
		    -tags TAG_CHILD_CONNECTOR_$parent
	    $C lower TAG_CHILD_CONNECTOR_$parent
	}

	# If there is now more than one child, we have to shift
	# all of the parent's siblings which appear below it
	# vertically downward one grid space
	if {[llength $structure($parent,children)] > 1} {
	    incr structure($parent,verticalSpace)

	    set P $structure($parent,parent)
	    set currNode $parent
	    while {$P != ""} {
		set siblings $structure($P,children)
		set nodePosition [lsearch $siblings $currNode]

		set moveList [lrange $siblings [expr $nodePosition + 1] end]
		foreach sibling $moveList {
		    $this move $sibling 0 1
		}

		incr structure($P,verticalSpace)
		$this refresh $P

		set currNode $P
		set P $structure($P,parent)
	    }
	}

	# Refresh the parent branch
	$this refresh $parent

	# Bind the child to perform the selectCommand
	$child bind <1> "$this select $child"

	# Bind the child to perform the actionCommand
	$child bind <Double-1> "$this action $child"

	# Bind the child for expanding/contracting its branch
	$child bind <2> "$this toggleChildDisplay $child"

	# Bind the child for expanding/contracting its branch
	$child bind <3> "$this toggleParentDisplay $child"

	# Reset the canvas size
	$C configure -scrollregion [$C bbox all]
    }

    # ------------------------------------------------------------------
    #  METHOD: action - Perform the default action on a node
    # ------------------------------------------------------------------
    method action {node} {
	if {$node == ""} {
	    if {$selectedNode == ""} {
		tk_dialog \
			[dblcb_NewToplevel] \
			Warning \
			"You must first select a point to view a definition" \
			warning \
			0 \
			Cancel
		return
	    } else {
		set node $selectedNode
	    }
	}
	set args [$node info variable userData -value]
	if {[catch {eval $actionCommand $args} ret] == 1} {
	    error "while selecting node: $ret"
	}
	$node select
	set selectedNode $node
    }


    # ------------------------------------------------------------------
    #  METHOD: select - Change the selected NODE and execute the select
    #                   command.  If called with no arguments, it
    #                   returns the selected NODE name.
    # ------------------------------------------------------------------
    method select {{node {}}} {
	# Return the selected node if nothing specified
	if {$node == ""} {
	    return $selectedNode
	}

	# Otherwise, change the selected node
	if {$selectedNode != ""} {
	    # Trap this just in case this node has been deleted, such
	    # as when a branch containing the selected node is removed
	    catch {$selectedNode deselect}
	}

	# Evaluate the selectCommand
	set args [$node info variable userData -value]
	if {[catch {eval $selectCommand $args} ret] == 1} {
	    error "while selecting node: $ret"
	}
	$node select
	set selectedNode $node
    }


    # ------------------------------------------------------------------
    #       selectParent - Change the selected NODE to the parent of a
    #                      given NODE and execute the selectCommand
    # ------------------------------------------------------------------
    method selectParent {{node {}}} {
	# If nothing specified, get the parent of the currently
	# selected node
	if {$node == ""} {
	    set node $selectedNode
	}

	# Get the parent of the selected node
	set parent $structure($node,parent)

	# Select the parent node
	$this select $parent
    }

    # ------------------------------------------------------------------
    #  METHOD:  deleteBranch - delete a branch of the node tree
    # ------------------------------------------------------------------
    method deleteBranch {node} {
	if {$node == ""} {set node $structure(,children)}

	set deletedHeight $structure($node,verticalSpace)

	set parent $structure($node,parent)
	set children $structure($node,children)

	set nodePosition [lsearch $structure($parent,children) $node]
	lvarpop structure($parent,children) $nodePosition

	$node delete
	$C delete TAG_CHILD_CONNECTOR_$node

	set moveList [lrange $structure($parent,children) $nodePosition end]
	foreach sibling $moveList {
	    $this move $sibling 0 -$deletedHeight
	}

	set deleteList $children
	while {![lempty $deleteList]} {
	    set child [lvarpop deleteList]
	    $C delete TAG_CHILD_CONNECTOR_$child
	    if {$structure($child,children) != ""} {
		eval lappend deleteList $structure($child,children)
	    }
	    unset structure($child,children)
	    unset structure($child,parent)
	    unset structure($child,verticalSpace)
	    $child delete
	    if {$child == $selectedNode} {
		set selectedNode ""
	    }
	}
	$this refresh $parent

	if {[llength $structure($parent,children)] > 0} {
	    incr structure($parent,verticalSpace) -$deletedHeight

	    set P $structure($parent,parent)
	    set currNode $parent
	    while {$P != ""} {
		set siblings $structure($P,children)
		set nodePosition [lsearch $siblings $currNode]

		set moveList [lrange $siblings [expr $nodePosition + 1] end]
		foreach sibling $moveList {
		    $this move $sibling 0 -1
		}

		incr structure($P,verticalSpace) -1
		$this refresh $P

		set currNode $P
		set P $structure($P,parent)
	    }
	}
    }


    # ------------------------------------------------------------------
    #  METHOD:  refresh - redraw all or part of the node tree
    # ------------------------------------------------------------------
    method refresh {node} {
	foreach child $structure($node,children) {
	    $this refresh $child
	}
	if {$node == ""} return
	$node redraw
	if {$structure($node,children) == ""} {
	    $C delete TAG_CHILD_CONNECTOR_$node
	} else {
	    set gX [$node info variable gridX -value]
	    set gY [$node info variable gridY -value]
	    set numChildren [llength $structure($node,children)]
	    set lastChild [lindex $structure($node,children) [expr $numChildren-1]]
	    set lastChildY [$lastChild info variable gridY -value]
	    set height [expr $lastChildY - $gY + 1]
	    $C coords TAG_CHILD_CONNECTOR_$node \
		    [expr $gX                 * $xScale + $xScale / 2     ] \
		    [expr ($gY + 1)           * $yScale                   ] \
		    [expr ($gX + 1)           * $xScale + $xScale / 2 - 10] \
		    [expr ($gY + 1)           * $yScale                   ] \
		    [expr ($gX + 1)           * $xScale + $xScale / 2 - 10] \
		    [expr ($gY + $height)     * $yScale                   ]
	}
    }


    # ------------------------------------------------------------------
    #  METHOD:  reposition - Set the grid position of a branch in the
    #                        TREE
    # ------------------------------------------------------------------
    method reposition {node X Y} {
	set currX [$node info variable gridX -value]
	set currY [$node info variable gridY -value]
	set dx [expr $X - $currX]
	set dy [expr $Y - $currY]

	$node movex $dx
	$node movey $dy

	foreach child $structure($node,children) {
	    $this move $child $dx $dy
	}
    }



    # ------------------------------------------------------------------
    #  METHOD:  move - Move the a branch in the TREE by some relative
    #                  amount of grid units
    # ------------------------------------------------------------------
    method move {node dx dy} {
	$node movex $dx
	$node movey $dy
	foreach child $structure($node,children) {
	    $this move $child $dx $dy
	}
    }



    # ------------------------------------------------------------------
    #  METHOD:  queryWidget - Returns the name of the container frame
    #                         widget
    # ------------------------------------------------------------------
    method queryWidget {} {
	return $W
    }


    # ------------------------------------------------------------------
    #  METHOD:  toggleChildDisplay - Expand or contract the display of
    #                                the children of a node
    # ------------------------------------------------------------------
    method toggleChildDisplay {node} {
	echo toggling child state of '$node'
    }


    # ------------------------------------------------------------------
    #  METHOD:  toggleParentDisplay - Expand or contract the display of
    #                                 the children of a node
    # ------------------------------------------------------------------
    method toggleParentDisplay {node} {
	echo toggling parent state of '$node'
    }


    # ------------------------------------------------------------------
    #  METHOD:  clearSelection - Deselect any selected NODE
    # ------------------------------------------------------------------
    method clearSelection {} {
	set selectedNode {}
    }


    # ------------------------------------------------------------------
    #  PUBLIC:  getChildrenCommand - Command to execute when a node is
    #                                expanded.  It should return a list
    #                                of NODES which are children of the
    #                                selection
    # ------------------------------------------------------------------
    public variable getChildrenCommand {} {
	return {}
    }


    # ------------------------------------------------------------------
    #  PUBLIC: background - Background color for the tree as well as
    #                       the default color for all NODES in the tree
    # ------------------------------------------------------------------
    public variable background {bisque} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: foreground - Color for the lines connecting nodes on
    #                       the tree
    # ------------------------------------------------------------------
    public variable foreground {black} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: font - Font for text labels in nodes
    # ------------------------------------------------------------------
    public variable font {} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: actionCommand - Default command to execute when a node
    #                          is double-clicked with mouse button 1
    # ------------------------------------------------------------------
    public variable actionCommand {list} {
    }


    # ------------------------------------------------------------------
    #  PUBLIC: selectCommand - Default command to execute when a node
    #                          is single-clicked with mouse button 1
    # ------------------------------------------------------------------
    public variable selectCommand {list} {
    }
}
