#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: dxfFileTransf.tcl,v 2.51 2004/11/22 13:55:33 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dimarcan  23/01/01  created
#

#************************************************************************
#   NAME
#    dxfFileTransf - Tcl procedure to invoke the transfer mechanism between 
#     VLT WS environments.
# 
#   SYNOPSIS
#    dxfFileTransf <sourceFile> <destEnv> [<destFile>] [<sourcePath>] 
#     [<destPath>]
# 
#   DESCRIPTION
#    This procedure is used in a Tcl script to transfer a file between 
#    two VLT environments. The pair of client/server
#    processes used for such a purpose must be already registered
#    whitin CCS (see the corresponding man pages for "dxfFilesend" and/or
#    "dxfFileReceive").
#
#    Allowed arguments:
#
#    sourceFile (mandatory)  the name of the file to be transferred
#                            without path.
#
#    destEnv    (mandatory)  destination environment where the above
#                            specified file should be transferred to.
#
#    destFile    (optional)  the name of the saved file (if this
#                            parameter is not set, the source file name
#                            is used to store the file in the
#                            destination environment).
#
#    sourcePath  (optional)  path of the source file. If not set,
#                            the file is retrieved from
#                            $INS_ROOT/SYSTEM/DETDATA.
#
#    destPath    (optional)  path where the destination file is saved.
#                            If not set, the file is stored in
#                            $INS_ROOT/SYSTEM/DETDATA of the destination
#                            environment.
#
#   RETURN VALUES
#    0 - SUCCESS (after the completion of the data transfer)
#    1 - FAILURE (if an error occurs)
#
#   CAUTIONS
#    The processes (dxfFileSend/dxfFileReceive), which
#    actually take care of the transfer mechanism, must be already
#    active in their respective environment.
#
#   EXAMPLES
#    set sourceFile     "fileToTransfer.bin"
#    set destEnv        XXXX
#    set destFile       "receivedFile.bin"
#    set sourcePath     [pwd]
#    set destPath       ~yyyy/zzzz/
#
#    if { [dxfFileTransf $sourceFile $destEnv $destFile \
#                                  $sourcePath $destPath] == 1 } {
#       seq_errDisplay -suspend
#    }
#
#   SEE ALSO
#    dxfFileSend/dxfFileReceive
#
#------------------------------------------------------------------------
#


proc dxfFileTransf { sourceFile destEnv { destFile "" } \
                     { sourcePath "" } { destPath "" } } {


   global env errStack

   set buffer "-sourceFile $sourceFile -destEnv $destEnv"
   
   if { ![cequal $destFile ""] } {
      set buffer "$buffer -destFile $destFile"
   }

   if { ![cequal $sourcePath ""] } {
      set buffer "$buffer -sourcePath $sourcePath"
   }

   if { ![cequal $destPath ""] } {
      set buffer "$buffer -destPath $destPath"
   }

   set cmdId 0

   if {[catch {
      eval seq_msgSendCommand $env(RTAPENV) dxfFileSend TRANSF $buffer} ret]} {
      return 1
   } else {
      set cmdId $ret
   }

   if {[catch { seq_msgRecvReply $cmdId errNr lr -last} ret]} {
      return 1
   }
   if { $errNr != 0 } { return 1 }

   return 0

}


#
# ___oOo___
