# dialog.tcl --
#
# This file defines the procedure tk_dialog, which creates a dialog
# box containing a bitmap, a message, and one or more buttons.
#
# RCS: @(#) $Id: ibacDialog.tcl,v 1.33 2005/09/01 15:18:04 vltsccm Exp $
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ibacDialog.tcl,v 1.33 2005/09/01 15:18:04 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# gzins     09/04/00  created
# gzins     16/05/00  removed automatic text wrapping
#

#************************************************************************
#   NAME
#   ibacDialog - modified tk_dialog box to use user-defined text font &
#                alignment 
#
#   DESCRIPTION
#   ibacDialog {w title text bitmap default args}
#     See tk_dialog manual.
#
#   proc ibacSetDialogTextFont { textFont }
#   proc ibacGetDialogTextFont { }
#     Set/Get the font used to display message in the dialog box.
#
#   proc ibacSetDialogTextAlignment { textAlignment }
#   proc ibacGetDialogTextAlignment { }
#     Set/Get the text alignment used to display message in the dialog box.
#
#   proc ibacInfoDialog {msg {title "User Info"} }
#   proc ibacErrorDialog {msg {title "Error"} } {    
#     Displays a dialog box with the information or error  message <msg> to the
#     user, and wait for confirmation from user.
#
#   proc ibacDialogYesNo {question {title "User Query"} {default 0}}
#     Displays a dialog box with the question <question> to the user, and
#     wait for answer from user. It returns the string "YES" or "NO" according
#     to the answer. The default gives the default button for the dialog (0 for
#     'YES' and 1 for 'NO')
#   
#   proc ibacExitDialog {msg {exitProc ""} {title "Error"} }
#     Displays a dialog box with the error message <msg> to the user. When user
#     has acknowledged the error, the <exitProc> is executed (if any), then the
#     application exits.
#   
#   proc ibacWarningDialog {msg {title "User Warning"} }
#     Displays a dialog box with the warning message <msg> to the user. The user
#     can either accept (button OK) or cancel. The user choice is returned.
#

#------------------------------------------------------------------------

proc ibacDialog {w title text bitmap default args} {
    global tkPriv tcl_platform gvar

    # 0a. Set text font & alignement
    if {[info exists gvar(ibacDialogTextFont)]} {
        set textFont $gvar(ibacDialogTextFont)
    } else {
        set textFont {Times 18}
    }

    if {[info exists gvar(ibacDialogTextAlignment)]} {
        set textAlignment $gvar(ibacDialogTextAlignment)
    } else {
        set textAlignment left 
    }

    # 0b. Check text length
    foreach line [split $text \n] {
        if {[string length $line] > 80} {
            set line "[string range $line 0 76] ..."
        }
        append ltext "${line}\n"
    }

    # 1. Create the top-level window and divide it into top
    # and bottom parts.

    catch {destroy $w}
    toplevel $w -class Dialog
    wm title $w $title
    wm iconname $w Dialog
    wm protocol $w WM_DELETE_WINDOW { }

    # The following command means that the dialog won't be posted if
    # [winfo parent $w] is iconified, but it's really needed;  otherwise
    # the dialog can become obscured by other windows in the application,
    # even though its grab keeps the rest of the application from being used.

    wm transient $w [winfo toplevel [winfo parent $w]]
    if {$tcl_platform(platform) == "macintosh"} {
        unsupported1 style $w dBoxProc
    }

    frame $w.bot
    frame $w.top
    if {$tcl_platform(platform) == "unix"} {
        $w.bot configure -relief raised -bd 1
        $w.top configure -relief raised -bd 1
    }
    pack $w.bot -side bottom -fill both
    pack $w.top -side top -fill both -expand 1

    # 2. Fill the top part with bitmap and message (use the option
    # database for -wraplength so that it can be overridden by
    # the caller).

    option add *Dialog.msg.wrapLength 0 widgetDefault
    label $w.msg -justify $textAlignment -text $ltext
    if {$tcl_platform(platform) == "macintosh"} {
        $w.msg configure -font system
    } else {
        $w.msg configure -font $textFont
    }
    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
    if {$bitmap != ""} {
        if {($tcl_platform(platform) == "macintosh") && ($bitmap == "error")} {
            set bitmap "stop"
        }
        label $w.bitmap -bitmap $bitmap
        pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
    }

    # 3. Create a row of buttons at the bottom of the dialog.

    set i 0
    foreach but $args {
        button $w.button$i -text $but -command "set tkPriv(button) $i"
        if {$i == $default} {
            $w.button$i configure -default active
        } else {
            $w.button$i configure -default normal
        }
        grid $w.button$i -in $w.bot -column $i -row 0 -sticky ew -padx 10
        grid columnconfigure $w.bot $i
        # We boost the size of some Mac buttons for l&f
        if {$tcl_platform(platform) == "macintosh"} {
            set tmp [string tolower $but]
            if {($tmp == "ok") || ($tmp == "cancel")} {
                grid columnconfigure $w.bot $i -minsize [expr 59 + 20]
            }
        }
        incr i
    }

    # 4. Create a binding for <Return> on the dialog if there is a
    # default button.

    if {$default >= 0} {
        bind $w <Return> "
            $w.button$default configure -state active -relief sunken
            update idletasks
            after 100
            set tkPriv(button) $default
            "
    }

    # 5. Create a <Destroy> binding for the window that sets the
    # button variable to -1;  this is needed in case something happens
    # that destroys the window, such as its parent window being destroyed.

    bind $w <Destroy> {set tkPriv(button) -1}

    # 6. Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.

    wm withdraw $w
    update idletasks
    set x [expr {[winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
        - [winfo vrootx [winfo parent $w]]}]
    set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
        - [winfo vrooty [winfo parent $w]]}]
    wm geom $w +$x+$y
    wm deiconify $w

    # 7. Set a grab and claim the focus too.

    set oldFocus [focus]
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
        set grabStatus [grab status $oldGrab]
    }
    grab $w
    if {$default >= 0} {
        focus $w.button$default
    } else {
        focus $w
    }

    # 8. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.  Finally,
    # restore any grab that was in effect.

    tkwait variable tkPriv(button)
    catch {focus $oldFocus}
    catch {
        # It's possible that the window has already been destroyed,
        # hence this "catch".  Delete the Destroy handler so that
        # tkPriv(button) doesn't get reset by it.

        bind $w <Destroy> {}
        destroy $w
    }
    if {$oldGrab != ""} {
        if {$grabStatus == "global"} {
            grab -global $oldGrab
        } else {
            grab $oldGrab
        }
    }
    return $tkPriv(button)
}

proc ibacSetDialogTextFont { textFont } {
    global gvar

    set gvar(ibacDialogTextFont)  $textFont
}

proc ibacGetDialogTextFont { } {
    global gvar

    return $gvar(ibacDialogTextFont)
}

proc ibacSetDialogTextAlignment { textAlignment } {
    global gvar

    set gvar(ibacDialogTextAlignment)  $textAlignment
}

proc ibacGetDialogTextAlignment { } {
    global gvar

    return $gvar(ibacDialogTextAlignment)
}

#### Dialog boxes
proc ibacInfoDialog {msg {title "User Info"} } {    
    ibacDialog .ibacDialog $title $msg info 0 OK
}

proc ibacErrorDialog {msg {title "Error"} } {    
    ibacDialog .ibacDialog $title $msg error 0 OK
}

proc ibacDialogYesNo {question {title "User Query"} {default 0}} {    
    set answer \
        [ibacDialog .ibacDialog $title $question question $default Yes No]
    if { $answer == 0} {
        return YES 
    } else {
        return NO
    }
}

# Display error message before exiting.
proc ibacExitDialog {msg {exitProc ""} {title "Error"} } {

    # Open dialog window
    ibacDialog .ibacDialog $title $msg warning 0 OK

    # If exit procedure is defined
    if { $exitProc != "" } {
        # Call exit procedure
        eval $exitProc
    } 

    exit 0
}

# Display warning message.
proc ibacWarningDialog {msg {title "User Warning"} } {

    # Open dialog window
    return [ibacDialog .ibacDialog $title $msg warning 0 OK CANCEL]
}

