#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ibacTABNOTE_BOOK.tcl,v 1.33 2005/09/01 15:18:05 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# marteau   22/02/00  created
# marteau   03/05/00  added patch to activate short help for contained classes
# gzins     16/05/01  added namespace ::iwidgets when instanciate tabnotebook
#                     widget

#************************************************************************
#   NAME
# 
#   ibacTABNOTE_BOOK - itk widget for easy use of tabnotebook in Panel Editor
#
#   SYNOPSIS
#   source ibacTABNOTE_BOOK.tcl
#   ibacTABNOTE_BOOK myLogWindow
# 
#   PARENT CLASS
#   itk::Widget
#
#   DESCRIPTION
#   This mega-widget encapsulates a itk tabnotebook widget to enable an easy
#   use in the VLT Panel Editor. "Pages" of the tabnotebook are mega-widgets
#   (in the Panel sense, i.e. Uif Classes). In this sense, ibacTABNOTE_BOOK is
#   widely inspired by the class container widget already available from the
#   Panel Editor.
#
#   WIDGET OPTIONS
#   Some option names are directly copied from classContainer
#
#     -background   <color>    Specifies a background color to use for
#                              displaying a page and its associated tab.
#     -height       <pixels>   Height of the pages in pixels.
#     -nameInstance <string>   Name of the instance.
#     -titleFont    <font>     Font for the labels of the tabs.
#                              The following alias are valid:
#                               - normal, big, small
#                               - bold, bigbold, smallbold
#     -titleList    <namelist> List of labels for the tabs.
#     -wdgClassList <namelist> List of classes to be displayed.
#     -wdgConfig    <list>     For each class there is a list of configuration
#                              parameters which are invoked when the class is
#                              displayed.
#                              If the class does not need configuration
#                              parameters, the corresponding list must be empty.
#     -wdgLibrary   <list>     List of library names where the classes to be
#                              displayed are stored.
#                              At run time the search path follows the VLT
#                              searching rules: module, INTROOT, VLTROOT.
#     -width        <pixels>   Width of the pages in pixels.
#
#   PUBLIC METHODS
#
#   constructor {args}
#    Class constructor
#
#   PRIVATE METHODS
#
#   SetFont {}
#     Internally used to associate font aliases (used in -titleFont option) to
#     real font names.
#
#   PRIVATE DATA MEMBERS
#
#   wdgConfigLen_
#    number of elements in -wdgConfig option
#
#   initialized_
#    set to 1 by the constructor
#
#   SEE ALSO
#   classContainer
#
#   BUGS     
#
#------------------------------------------------------------------------
#

catch {delete class ibacTABNOTE_BOOK}

itk::usual ibacTABNOTE_BOOK {}

class ibacTABNOTE_BOOK { 
    inherit itk::Widget

	# widget options
	itk_option define -helparea     helparea     Helparea     ""
	itk_option define -helptext     helptext     Helptext     ""
	itk_option define -wdgLibrary   wdgLibrary   WdgLibrary   ""
	itk_option define -wdgConfig    wdgConfig    WdgConfig    {}
	itk_option define -nameInstance nameInstance NameInstance "myInstance"
	itk_option define -wdgClassList wdgClassList WdgClassList ""
	itk_option define -titleList    titleList    TitleList    ""
	itk_option define -titleFont    titleFont    TitleFont    default

	# public methods
	constructor {args} {}
	destructor {}

    # private methods
	private method SetFont {}

	# private members
	private variable wdgConfigLen_ 0
	private variable initialized_ 0

}

## ---------------- OPTIONS -----------------

# option -wdgLibrary
configbody ibacTABNOTE_BOOK::wdgLibrary {

   global ::auto_path

   # Widget libraries are searched in module, INTROOT, VLTROOT, and added to
   # global variable 'auto_path'
   foreach item $itk_option(-wdgLibrary) {
	   if {[lsearch $auto_path "*/${item}.tcl"] == -1 } {
		   set libPath [findVLTFile lib/${item}.tcl]
		   if {![cequal $libPath "" ]} { lappend auto_path $libPath }
	   }
   }

}

# option  -wdgConfig
configbody ibacTABNOTE_BOOK::wdgConfig {
   set wdgConfigLen_ [llength $itk_option(-wdgConfig)]
}

# option  -titleList
configbody ibacTABNOTE_BOOK::titleList {
	
	set len [llength $itk_option(-titleList)]

	# Tabs are given names contained in -titleList option through the
	# 'pageconfigure' standard method of ::iwidgets::tabnotebook
	for { set i 0 } { $i < $len } { incr i } {
		set title [lindex $itk_option(-titleList) $i]
		if { [ $itk_component(tabnotebook) index $i ] != -1 } {
			$itk_component(tabnotebook) pageconfigure $i -label $title
		}
	}
}

# option  -wdgClassList
configbody ibacTABNOTE_BOOK::wdgClassList {

	# Global variable containing name of short-help area
	global shArea

	# Delete all pages
	catch { $itk_component(tabnotebook) delete 0 end }

	set count 0

	# For each widget class in -wdgClassList option
	foreach wdg $itk_option(-wdgClassList) {

		# Set tab names (read -titleList option)
		set title [lindex $itk_option(-titleList) $count]
		$itk_component(tabnotebook) add -label $title

		# Create new page in tabnotebook
		set page [$itk_component(tabnotebook) childsite $count]

		# Read possible options for the class to be displayed
		if { $count < $wdgConfigLen_ } {
			set wdgOptions [lindex $itk_option(-wdgConfig) $count]
		} else {
			set wdgOptions ""
		}

		# Create child widget in current page
		set myInstance $page.$itk_option(-nameInstance)

		# Create instance of the class with corresponding options and catch
		# possible error.
		if { [ catch { eval $wdg $myInstance -relief flat \
				-helparea \"$itk_option(-helparea)\" $wdgOptions } ret ] } {
			tk_dialog .dialog "Dialog Box" "Error loading $wdg\n$ret" \
					info 0 OK
			return
		}

		# Display the new widget
		pack $myInstance -padx 2 -pady 2

		# Set the short-help area to common short-help
		$myInstance configure -helparea $shArea

		# If widget is a 'uif class', call specific display and init methods
		if { [string match "*_uifClass" $wdg] } {
			$myInstance display
			$myInstance refreshInput
		}

		incr count
	}

}

# option  -titleFont
configbody ibacTABNOTE_BOOK::titleFont {
	if { $initialized_ } {
		SetFont
	}
}


## ---------------- PUBLIC METHODS ----------------

# Class constructor
body ibacTABNOTE_BOOK::constructor {args} {

	# Create 'tabnotebook' component
	itk_component add tabnotebook {
		::iwidgets::tabnotebook $itk_interior.tn -raiseselect yes -tabpos n \
        -equaltabs 0
	} {
		keep -width -height -background
	}

	# Display the tabnotebook
	pack $itk_component(tabnotebook) -fill both -expand yes

	# Evaluate user options
	eval itk_initialize $args

	set initialized_ 1

}

## ---------------- PRIVATE METHODS ----------------

body ibacTABNOTE_BOOK::SetFont {} {

   if {[cequal $itk_option(-titleFont) ""]} { return }

   # Set real font name according to given alias in -titleFont option
   switch $itk_option(-titleFont) {
      big        {set newFont -Adobe-helvetica-medium-r-normal--24*}
      small      {set newFont -Adobe-helvetica-medium-r-normal--12*}
      bold       {set newFont -Adobe-helvetica-bold-r-normal--14*}
      bigbold    {set newFont -Adobe-helvetica-bold-r-normal--24*}
      smallbold  {set newFont -Adobe-helvetica-bold-r-normal--12*}
      default    {set newFont -Adobe-helvetica-medium-r-normal--14*}
   }

   # Set the new font property
   $itk_component(tabnotebook) configure -font $newFont

}

#
# ___oOo___
