#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ic0TEST.tcl,v 2.14 2005/09/22 07:54:36 vltsccm Exp $"
#
# ic0TEST.tcl
#
# who        when      what
# ---------- --------  ---------------------------------------------
# therlin    30/09/96  created from seqTestLib
# therlin    09/10/96  updated man page
# therlin    12/10/96  added support for error method
#

#************************************************************************
#   NAME
#	ic0TEST - incr tcl class for a building test programs
# 
#   SYNOPSIS
#
#       ic0TEST <test-name>
# 
#   DESCRIPTION
#  
#     The ic0TEST is an framework for writing test programs in tcl. It
#     is written as a incr tcl class in order combine both global settings
#     with methods for convenience like sending a command, reading the 
#     database or viewing a FITS header.
#

#     The user create an instance of this class by :
#
#         ic0TEST  <test-name> <config-parameters>
#
#   METHODS
#
#     Comment <string>
#        Log's a comment string to the stdout.
#
#     Time
#        Log's the timofday to the stdout
#
#     DbRead <attr>
#        Read attribute in the database. The value is logged but also returned.
#
#     DbWrite <attr> <value>
#        Returns list elements in position <pos>
#        If <pos> is a wrong number, it return an empty string ("").
#
#     Call <command> <arg>
#        Send a command with parameters <arg> to the default process 
#        destination.
#
#     Error
#        Return the error code of an error message. 0 = no error.
#      
#     CatFits <file>
#        Shows the output of a FITS header stored in a file.
#
#     SyncDb <attr> <value>
#        Syncronizes on a specific database value. The synchronization
#        is done until the expected value occurs or until at timeout
#        has elapsed. The timeout is passed as option (-syncTimeout <value>).
#
#   OPTIONS
#
#
#     -name        <string>       Name of the test program
#     -env         <environment>  Name of the destination environment
#     -proc        <process>      Name of the destination process
#     -syncTimeout <sec>          DB syncronization timeout in sec (default 50)
#     -msgTimeout  <sec>          Waiting for reply timeout in sec (default 50)
#     -stopOnError 0|1            Determine if script should stop on errors or
#                                 continue.
#
#
#   EXAMPLES
#
#     Create a test instance
#
#     ic0TEST .test  -name myTest -env "" -proc progToTest
#     #Initialize the .test instance to send commands to progToTest in
#     #local environment
#
#     .test configure -env myLCU -proc newLCUprog
#     #Change the communication to a new processe running on myLCU.
# 
#     .test Comment "starting the test"
#     #Log a simple comment string.
#
#     .test Call INIT 
#     #Sending INIT command to progToTest and logging the reply
#
#     set val [.test DbRead <alias>progToTest.myVal]
#     .test Comment "The value is $val"
#     #Read a value from the database.
#
#     set reply [.test Call MYCMD -function abc]
#     if { $reply != "OK" } {
#        .test Comment "Not the expected reply"
#     }
#     #Example of how the reply value can be retrieved.
#     
#
#   SEE ALSO
#     eccsTestDriver
#
#   BUGS
#
#------------------------------------------------------------------------
#/*


#************************************************************************
#*# class: ic0TEST
#------------------------------------------------------------------------

class ic0TEST {

    inherit itk::Widget
    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    common seq_ccsCmdCheck
    
    constructor { args } {
	global ::env

	eval itk_initialize $args

	# remove window
	wm withdraw .
	#*# 1. call seq_ccsInit
	seq_ccsInit $itk_option(-name)
	set seq_ccsCmdCheck CHECK_ALL
	#*# 2. log
	LogStart "Running $itk_option(-name)"
	Log      "Begin: $itk_option(-name)"
    }

    #****************************************************************
    #*# destructor
    #----------------------------------------------------------------

    destructor {
	#puts ""
	#puts "End  : $itk_option(-name)"
	LogStart ""
	#sleep 5
	#seq_ccsExit
    }

    #****************************************************************
    #*# method: Error
    #----------------------------------------------------------------

    public method Error { } {
	return $errVar
    }

    #****************************************************************
    #*# method: Log
    #----------------------------------------------------------------

    private method Log { text } {
	global ::env
	
	if {[catch {set logWindow $env(IC0_LOG_WINDOW)}]} {set logWindow ""}
	if { $logWindow != "" } {   	    
	    if {[string length "$text"] > 80} {
		set text [string range "$text"  0 80]
	    }
	    catch {LocalSend $logWindow SCRIPT "ic0LogAddInfo \"$text\""}
	    after 200
	} 
	puts "$text"
    }

    private method LogStart { text } {
	global ::env
	
	if { $text != "" } { puts "$text" }
	if {[catch {set logWindow $env(IC0_LOG_WINDOW)}]} {set logWindow ""}
	if { $logWindow != "" } {   
	    catch {LocalSend $logWindow SCRIPT "ic0LogSetState \"$text\""}
	} 
    }
    #****************************************************************
    #*# local method: LocalSend
    #----------------------------------------------------------------

    private method LocalSend { procName command text } {

	#*# 1. send the command
	set cmdId [seq_msgSendCommand "" $procName $command "$text"]

	#*# 2. wait for last reply
	retun [seq_msgRecvReply $cmdId error -last]
    }
    #****************************************************************
    #*# method: Time
    #----------------------------------------------------------------

    public method Time { } {
	Log [format "Time  : [exec date]"]
    }
    
    #****************************************************************
    #*# method: Comment
    #----------------------------------------------------------------

    public method Comment { text } {
	Log [format "Msg  : %s" $text]
    }
    
    #****************************************************************
    #*# method: DbRead
    #----------------------------------------------------------------

    public method DbRead { attr } {

	if {[catch {

	    #*# 1. read attribute
	    set value [seq_dbReadSymbolic $attr]

	    #*# 2. log read
	    Log "Read : $attr = '$value'"

	} result]} {

	    #*# 3. print error reply
	    Log "Read : $attr"
	    Log "Error: $result"
	    ErrPrint
	    
	    return ""
	}

	return $value
    }

    #****************************************************************
    #*# method: DbWrite
    #----------------------------------------------------------------

    public method DbWrite { attr value } {

	if {[catch {

	    #*# 1. write attribute
	    seq_dbWriteSymbolic $attr $value

	    #*# 2. log read
	    Log "Write: $attr = '$value'"

	} result]} {

	    #*# 3. print error reply
	    Log "Write: $attr = '$value'"
	    Log "Error: $result"
	    ErrPrint

	}

    }

    #****************************************************************
    #*# method: Call
    #----------------------------------------------------------------

    public method Call { command args } {

	set envName  $itk_option(-env)
	set procName $itk_option(-proc)

	#*# 1. log call
	Log ""
	Log "Cmd  : $envName/$procName $command $args"

	set errVar 0
	set reply ""
	if {[catch {

	    #*# 2. send command
	    set reply [eval {MsgCall $envName $procName $command \
			errVar} $args]

	    #*# 3. print normal reply
            if { $errVar } {
                Log "Error Reply: $reply"
                if {  $itk_option(-stopOnError) } {
                    Log "Error detected stopping test script !!"
                    exit 1
                }
		set reply ""
            } else {
                Log "Reply: $reply"
            }
	} result]} {

	    #*# 4. print error reply
	    Log "Error: $result"
	    ErrPrint
	    if {  $itk_option(-stopOnError) } {
		Log "Error detected stopping test script !!"
		exit 1
	    }
	    return ""
	}
	
	return $reply
    }

    #****************************************************************
    #*# local method: MsgCall
    #----------------------------------------------------------------

    private method MsgCall { envName procName command error args } {

	#*# 1. send the command
	set cmdId [eval {seq_msgSendCommand $envName $procName \
				$command} $args]

	#*# 2. wait for last reply
	set $error 0
	set timeout [expr {1000 * $itk_option(-msgTimeout)}]
	set last 0

	while { $last == 0 } {
	set reply [seq_msgRecvReply $cmdId $error last -$timeout]

	    if { !$last } {
                Log "Intermediate Reply: $reply" 
	    }
	}

	return $reply
    }

    #****************************************************************
    #*# local method: ErrPrint
    #----------------------------------------------------------------

    public method ErrPrint {} {

	#*# 1. get stack
	set errStack [seq_errPrint]
	if { "$errStack" == " {0 0} 0   {}" } {
	    return
	}

	#*# 2. get stack data
	set num		[lindex [lindex $errStack 1] 1]
	set severity	[lindex $errStack 2]
	switch -- $severity {
	    0	    { set severity "W" }
	    1	    { set severity "S" }
	    2	    { set severity "F" }
	    default { set severity "U" }
	}
	set location	[lindex $errStack 3]
	set module	[lindex $errStack 4]
	set text	[lindex $errStack 5]

	#*# 3. print data
	Log ""
	Log " ---------------- Error Structure ----------------"
	Log [format "Error Number : %-8d Severity : %s" $num $severity]
	Log [format "Module       : %-8s Location : %s" $module $location]
	Log [format "Error Text   : %s" $text]
	Log ""
	Log ""

    }

    #****************************************************************
    #*# public method: CatFits
    #----------------------------------------------------------------

    public method CatFits { file } {
	set lineCount 0
	# Open the file for read
	Log "Reading file: $file"

	if {![file exists $file]} {
	    Log "Invalid file specified"
	    return
	}
	
	set fd [open $file r]

	# Loop over the lines in the file.
	# Each line correspond to a test
	while { 1 } {
	    set line [read $fd 80]
	    if { [llength $line] == 0} return
	    Log "$line"
	    
	    incr lineCount
	}
    
    }
    #****************************************************************
    #*# public method: SyncDb
    #----------------------------------------------------------------

    public method SyncDb { dbRef value } {

	set total 2
	while { 1 } {
	    set dbValue [seq_dbReadSymbolic $dbRef]
	    if { $dbValue == $value } { return 0 }
	    sleep 2
	    incr total 2
	    
	    if { $total >  $itk_option(-syncTimeout) } {
		Comment "SyncDb reached timeout of $total seconds"
		return 1
	    }
	}
    
    }
    #****************************************************************
    #*# public variables
    #----------------------------------------------------------------

    itk_option define -name        name        Name        "testProg"
    itk_option define -env         env         Env         ""
    itk_option define -proc        proc        Proc        "noProcName"
    itk_option define -syncTimeout syncTimeout SyncTimeout 50
    itk_option define -msgTimeout  msgTimeout  MsgTimeout  50
    itk_option define -stopOnError stopOnError StopOnError 0

    private variable errVar   0
}


#************************************************************************
# ___oOo___
