#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanAssembly.tcl,v 1.47 2005/07/01 13:25:42 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  25/11/04  created
#

#************************************************************************
#   NAME
#    icbpanAssembly - wrapper class for icbpanAssembly_uifClass
# 
#   SYNOPSIS
#    icbpanSensor .w \
#              -background Grey90 \
#              -relief flat       \
#              -cfgObj $cfgObj    \
#              -cwp "<alias>${devName}" \
#              -devType $devType]
#
#     where cfgObj is a ctooCONFIG object;
#           devType is extracted from INS.CON.DEVICE${i}
# 
#   DESCRIPTION
#    Itcl wrapper class to handle assemblies. The corresponding
#    ibcpanAssembly_uifClass contains a set of widgets for setup and
#    status actions. A checkbutton is used to indicate if the 
#    corresponding assembly is selected.
#
#   PUBLIC METHODS
#    icbpanAssembly  xxx
#       constructor. 
#    isSelected
#       get the current status of the checkbutton (0/1).
#    setSelected 0/1
#       set the checkbutton state to 0 or 1 (unselected or selected).
#    dbUpdate
#       update output widgets associated to OLDB vector "status"
#    evtDbRestart
#       detach and attach event on OLDB vector "status"
#
#   PUBLIC VARIABLES
#    cfgObj
#       ctooCONFIG object (configuration file interface)
#    devType
#       device type extracted from INS.CON.DEVICE${i}
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTIONS
#
#   SEE ALSO
#    ctooCONFIG(8), icbpanGUIcfg(8)
#
#   BUGS     
#
#------------------------------------------------------------------------
#


class icbpanAssembly {
   inherit icbpanAssembly_uifClass

    constructor {args} {
      global ::shArea

      # VLTSW20020413: extract values of devType and cfgObj from arguments
      #                without calling twice itk_initialize (done also at the end)
      #eval itk_initialize -helparea $shArea -relief flat $args
      set remainingArgs {}
      foreach {var val} $args {
          switch -exact -- $var {
              -devType { set devType $val }
              -cfgObj  { set cfgObj  $val }
              default  { lappend remainingArgs $var $val }
	  }
      }
      # set widget name and width
      set name [icbpanGetAssemblyName $cfgObj $devType]
      set namelen [string length $name]
      set totwidth [$itk_interior.cv cget -width]
      set wdgwid [$cv.device cget -width]
      if { $namelen > $wdgwid } { 
          $cv.device configure -width $namelen
          incr totwidth [expr {($namelen - $wdgwid) * 9}]
      }
      upvar #0 [$cv.device cget -variable] devName
      set devName $name

      # retrieve OLDB status vector index associated to this assembly
      set firstIndex [icbpanGetAssemblyOldbIndex $cfgObj $devType]

      # retrieve keys and values associated to this assembly
      set assName [$cfgObj Get $devType ]
      set values [icbpanGetAssemblyValues $cfgObj $devType]
      set valnum [llength $values]
      set keys [icbpanGetAssemblyKeys $cfgObj $devType]
      set keynum [llength $keys]
      if { $keynum == 0 } { 
         set keynum 1
         lappend keys $assName
      }

      # create widgets associated to the keys
      set labellen 0
      if { $keynum > 1 } {
          loop i 0 $keynum {
	      set key [lindex $keys $i]
              set keylen [string length $key]
              if { $keylen > $labellen } { set labellen $keylen }
	  }
      }
      set totheight [$itk_interior.cv cget -height]
      if { $valnum > 0 } {
          loop i 0 $keynum {
              set key [lindex $keys $i]
              set units [icbpanDefaultUnits $key]
              set type [icbpanDefaultType $key]
              if { "$type" == "double" } { 
                  set type "float"
              } elseif { "$type" == "string" } { 
                  set type "any"
	      }
              # add setup widget
              set index [expr { $firstIndex + $i }]
              if { [llength $statusValues] == 0 } { dbUpdate }
              set last [lindex $statusValues $index]
              set valString [join $values]
              if { [string first * $valString] == -1 } { set wildcard 0
	      } else { set wildcard 1 }
              if { $wildcard } {; # entry widget
                  set lengthSetup 10
                  if { $labellen > 0 } { 
                      set label $key
                      set offset 12
                  } else { 
                      set label ""
                      set offset 6
                  }
                  itk_component add devSetupKey${i} {
                      icbpanEntry $itk_interior.devSetupKey${i} -background Grey90 -colBadVal red \
                          -font -Adobe-courier-medium-r-normal--14* \
                          -foreground Black -id undefined  \
                          -initialState normal -justify left  \
                          -labelColor Black -labelFont -Adobe-courier-medium-r-normal--14*  \
                          -label $label -labelWidth $labellen \
                          -selectbackground Black -selectforeground Grey90  \
                          -state normal -undo 0 -valType $type  \
                          -variable ${devType}_Setup${i} \
                          -minVal "" -maxVal "" -default $last \
                          -width $lengthSetup
                  } { }
                  set setupWdgWid [expr { $offset + ($labellen + $lengthSetup) * 9 }]
                  lappend entryWidgets devSetupKey${i}
                  # short help text
                  set shorthelp ""
                  if { "$units" != "" } { set shorthelp "Units: $units" }
                  if { "$valString" == "*" } { set valString "any value" }
                  set shorthelp "$shorthelp Valid values: $valString"
                  catch {UifShortHelp::add $itk_option(-helparea) \
                                           $itk_component(devSetupKey${i}) \
                                           $shorthelp }

	      } else {; # option button
                  set lengthSetup 0
                  loop j 0 $valnum {
	              set val [lindex $values $j]
                      set vallen [string length $val]
                      if { $vallen > $lengthSetup } { set lengthSetup $vallen }
	          }
                  itk_component add devSetupKey${i} {
                       UifOptionButton $itk_interior.devSetupKey${i} -canvasAnchor nw \
                          -itemList $values -itemValue $values \
                          -default $last \
                          -length $lengthSetup -selectMode label \
                          -font -Adobe-courier-medium-r-normal--14* \
                          -variable ${devType}_Setup${i} \
                          -command [code $this updateWdg]                          
                  } { }
                  set setupWdgWid [expr { 48 + $lengthSetup * 9 }]
	      }

              set xPos $totwidth
              set yPos [expr { 4 + $i * 22 }]
              place $itk_component(devSetupKey${i}) -x $xPos -y $yPos -bordermode inside

              # add status output widget
              set lengthStatus $lengthSetup
              itk_component add devStatusKey${i} {
                  UifOutput $itk_interior.devStatusKey${i} -anchor w -canvasAnchor nw \
                       -font -Adobe-courier-medium-r-normal--14* \
                       -frame 1 \
                       -variable ${devType}_Status${i} \
                       -width $lengthStatus
              } { }
              set statusWdgWid [expr { 4 + $lengthStatus * 9 }]
              incr xPos $setupWdgWid 
              place $itk_component(devStatusKey${i}) -x $xPos -y $yPos -bordermode inside         
          }

          # resize megawidget
          if { $keynum > 0 } {
              incr totheight [expr { ($keynum - 1) * 22 }]
              incr totwidth [expr { 4 + $setupWdgWid + $statusWdgWid }]
          }
      }
      $itk_interior.cv configure -width $totwidth -height $totheight

      eval itk_initialize -helparea $shArea -relief flat $remainingArgs
      display

      after idle [code $this initFunction]
   }


   destructor {
      evtDbStop
   }


   method initFunction { } {
      evtDbStart
      updateStatus statusValues "" w
      trace variable statusValues w [code $this updateStatus]
   }  


   method evtDbStart {} {

       if { $evt == "" } {
           set dbPoint $itk_option(-cwp)
           dbUpdate
           catch {seq_evtAttach "$dbPoint.status" "w" [code $this dbUpdate]} evt
       }
   }


   method evtDbStop {} {
       if { $evt != "" } {
           catch { seq_evtDetach $evt }
           set evt ""
       }
   }


   public method dbUpdate { } {

      set dbPoint $itk_option(-cwp)
      set statusValues [icbpanDbRead "$dbPoint.status"]
   }

   # update variable associated to status widget
    protected method updateStatus { name element op } {

      if { $valnum > 0 } {
          loop i 0 $keynum {
              set index [expr { $firstIndex + $i }]
              set value [lindex $statusValues $index]
              upvar #0 [$itk_component(devStatusKey${i}) cget -variable] var
              set var $value
	  }
      }
   }

   public method evtDbRestart {} {
      evtDbStop
      evtDbStart
   }


   public method updateWdg { item } {
      global ::gvar

      if {[cequal $item ""]} {return 0}

      # select check button
      setSelected 1
   }

   # return status of the checkbutton
   public method isSelected { } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      return $sel
   }


   # set status of the checkbutton
   public method setSelected { val } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      set sel $val
   }


   # return associated setup keyword
   public method getKeyw { } {
      global ::gvar

      set setup ""
      if { $valnum > 0 } {
          loop i 0 $keynum {
              upvar #0 [$itk_component(devSetupKey${i}) cget -variable] keyval
              if {![cequal $keyval ""]} {
	         set key [lindex $keys $i]
                 set setup "$setup $key $keyval"
	      }
	  }
      }
      return $setup
   }

   # set setup keyword+value
   public method setKeyw { val } {
      global ::gvar

      set keyw [lindex $val 0]
      set value [lindex $val 1]

      if {![cequal $value ""]} {
          set i [lsearch -exact $keys $keyw]
          if { $i > -1 } { 
             $itk_component(devSetupKey${i}) configure -default $value
             setSelected 1
	  }
      }
   }


   # return the list of entry widgets whose variable must be traced
   public method entryWdg { } {

       return $entryWidgets
   }
   
   public variable devType ""
   public variable cfgObj ""

   protected variable keynum 1
   protected variable keys {}
   protected variable firstIndex 0

   #  shared among objects of the same class
   protected common evt ""
   protected common statusValues {}

   private variable valnum 0
   private variable entryWidgets {}
}


#
# ___oOo___
