#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanContPos.tcl,v 1.47 2005/07/01 13:25:36 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dimarcan  17/11/00  created
#

#************************************************************************
#   NAME
#    icbpanContPos - wrapper class for icbpanContPos_uifClass
# 
#   SYNOPSIS
#    icbpanContPos .w \
#              -background Grey90 \
#              -relief flat       \
#              -cfgObj $cfgObj    \
#              -cwp "<alias>${devName}" \
#              -devType $devType]
#
#     where cfgObj is a ctooCONFIG object;
#           devType is extracted from INS.CON.DEVICE${i}
#           devName is extracted from ${devType}.DEVNAME
# 
#   DESCRIPTION
#    Itcl wrapper class to handle ICB POS device. The corresponding
#    ibcpanContPos_uifClass contains a set of output widget (linked to OLDB)
#    which in turn display the device name, keyword used to identify the
#    device, the current simulation level (N=normal,S=simulate), the current
#    device state, handset mode (H, if active) and the current position
#    (named and in encoder unit). The class contains also an option
#    button to select among various setup parameters and an entry widget
#    for setting position values. Finally a checkbutton is used to
#    indicate if the corresponding device is selected for some actions.
#
#   PUBLIC METHODS
#    icbpanContPos  xxx
#       constructor. Calls "initFunction" which initialize the option
#       button with appropriate parameters. 
#    isSelected
#       get the current status of the checkbutton (0/1).
#    setSelected 0/1
#       set the checkbutton state to 0 or 1 (unselected or selected).
#    getKeyw
#       returns the associated setup keyw. + value. This method reads
#       from the option button and entry widget the current user
#       settings and prepare a string to be sent to ICS control process.
#       This method is normally called when the SETUP cmd is invoked.
#    setKeyw val
#       extract from "val" the keyword parameter + value and fill the
#       corresponding option button + entry. This method is called usually
#       during a reading of a previously saved setup file.
#    
#   PROTECTED METHODS
#    updateWdg item
#       this method is called to modify on-the-fly the appearance of
#       the widgets. E.g. if a particular param. selection requires also some
#       input values, the corresponding entry widget is enabled,
#       otherwise it is disabled. User can override this method to 
#       implement some specific behavior.
#
#   PUBLIC VARIABLES
#    cfgObj
#       ctooCONFIG object (configuration file interface)
#    devType
#       device type extracted from INS.CON.DEVICE${i}
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTIONS
#
#   SEE ALSO
#    ctooCONFIG(8)
#
#   BUGS     
#
#------------------------------------------------------------------------
#


class icbpanContPos {
   inherit icbpanContPos_uifClass

   constructor {args} {
      global ::shArea
      eval itk_initialize -helparea $shArea -relief flat $args
      display
      after idle [code $this initFunction]
   }

   method initFunction { } {

      # determine prefix to be used
      set prefix [icbpanGetDevicePrefix $cfgObj $devType]

      # get units
      if {[catch { $cfgObj Get ${devType}.UNIT } units ]} {; # use default from dictionary
          set units [icbpanDefaultUnits ${prefix}.POS]
      }

      # initialize option lists
      lappend elements $units
      if {[catch { $cfgObj Get ${devType}.POSMIN } val ]} { set val "" }
      lappend minVal $val
      set minTxt $val
      if {[cequal $minTxt ""]} { set minTxt "N/A" }
      if {[catch { $cfgObj Get ${devType}.POSMAX } val ]} { set val "" }
      lappend maxVal $val
      set maxTxt $val
      if {[cequal $maxTxt ""]} { set maxTxt "N/A" }
      lappend helps "Set position value (Range: $minTxt : $maxTxt)"

      lappend elements $encoder
      lappend helps "Set encoder value"
      lappend elements $encrel
      lappend helps "Set encoder relative value"

      # initialize status widget
      set txt "Current position (Units: $units)"
      UifShortHelp::add [$cv.devStatus0 cget -helparea] $cv.devStatus0 $txt

      # initialize option button widget
      set last $units
      $cv.devSetup configure -itemList $elements -default $last \
                             -command [code $this updateWdg]

      # initialize encoder entry widget (value from status)
      configEnc $last
      upvar #0 [ $cv.devStatus0 cget -variable] wdgVar
      setEnc $wdgVar
      setEnc [getEnc 1]; # clean entry if illegal value

   }


   public method updateWdg { item } {
      global ::gvar

      if {[cequal $item ""]} {return 0}

      # select check button
      setSelected 1

      # configure entry widget
      configEnc $item
      if {[cequal $item $last]} {; # option unchanged
	  set value [getEnc]
      } else {; # option changed
          set value ""; # clean entry widget value
      }
      setEnc $value

      set last $item
   }

   # return status of the checkbutton
   public method isSelected { } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      return $sel
   }


   # set status of the checkbutton
   public method setSelected { val } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      set sel $val
   }


   # return associated setup keyword
   public method getKeyw { } {
      global ::gvar

      upvar #0 [$cv.devSetupEnc cget -variable] val
      upvar #0 [$cv.devSetup cget -variable] nameval

      if {[cequal $nameval ""]} { return "" }

      if {[cequal $nameval $encoder]} {
         set setup "${prefix}.ENC $val"
      } elseif {[cequal $nameval $encrel]} {
         set setup "${prefix}.ENCREL $val"
      } else {
         set setup "${prefix}.POS $val"
      }

      return $setup
   }

   # set setup keyword+value
   public method setKeyw { val } {
      global ::gvar

      set keyw [lindex $val 0]
      set value [lindex $val 1]

      set newOption ""
      if {[cequal ${prefix}.POS $keyw]} {
         set newOption $units
      } elseif { [cequal ${prefix}.ENC $keyw] } {
         set newOption $encoder
      } elseif { [cequal ${prefix}.ENCREL $keyw] } {
         set newOption $encrel
      }
      if {![cequal $newOption ""]} {
          set last $newOption
          $cv.devSetup configure -default $last
          configEnc $last
          setEnc $value
      }
   }

   # return the list of entry widgets whose variable must be traced
   public method entryWdg { } {

       return {devSetupEnc}
   }
   
   # configure encoder entry widget
   protected method configEnc { item } {

       set min ""
       set max ""
       set txt ""
       set valType float
       set index [lsearch -exact $elements $item]
       if {$index >= 0} {; # valid option
	   if {[cequal $item $encoder] || [cequal $item $encrel]} { 
               set valType integer
           } else {
	       set min [lindex $minVal $index]
	       set max [lindex $maxVal $index]
	   }
	   set txt [lindex $helps  $index]
       }
       $cv.devSetupEnc configure -id $item
       $cv.devSetupEnc configure -valType $valType
       $cv.devSetupEnc configure -minVal $min
       $cv.devSetupEnc configure -maxVal $max
       $cv.devSetupEnc configHelp $txt
   }
   
   # set value in encoder entry widget
   protected method setEnc { value } {
       $cv.devSetupEnc configure -default $value
       $cv.devSetupEnc setValue $value
   }

   # get value in encoder entry widget
   protected method getEnc { {check 0 } } {
       if {$check} { 
           set value [ $cv.devSetupEnc getSelected]
       } else { 
           upvar #0 [ $cv.devSetupEnc cget -variable] value
       }
       return $value
   }

   public variable devType ""
   public variable cfgObj ""

   private variable prefix ""
   private variable encoder "enc"
   private variable encrel  "encrel"
   private variable elements {}
   private variable helps {}
   private variable minVal {}
   private variable maxVal {}
   private variable last   ""
   private variable units  ""

}


#
# ___oOo___
