#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanDpol.tcl,v 1.47 2005/07/01 13:25:39 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dimarcan  17/11/00  created
#

#************************************************************************
#   NAME
#    icbpanDpol - wrapper class for icbpanDpol_uifClass
# 
#   SYNOPSIS
#    icbpanDpol .w \
#              -background Grey90 \
#              -relief flat       \
#              -cfgObj $cfgObj    \
#              -cwp "<alias>${devName}" \
#              -devType $devType]
#
#     where cfgObj is a ctooCONFIG object;
#           devType is extracted from INS.CON.DEVICE${i}
#           devName is extracted from ${devType}.DEVNAME
# 
#   DESCRIPTION
#    Itcl wrapper class to handle ICB DPOR device. The corresponding
#    ibcpanDpol_uifClass contains a set of output widget (linked to OLDB)
#    which in turn display the device name, keyword used to identify the
#    device, the current simulation level (N=normal,S=simulate), the current
#    device state, handset mode (H, if active) and the current position
#    (named and in encoder unit). The class contains also an option
#    button to select among various setup parameters and an entry widget
#    for setting position values. Finally a checkbutton is used to
#    indicate if the corresponding device is selected for some actions.
#
#   PUBLIC METHODS
#    icbpanDpol  xxx
#       constructor. Calls "initFunction" which initialize the option
#       button with appropriate parameters. 
#    isSelected
#       get the current status of the checkbutton (0/1).
#    setSelected 0/1
#       set the checkbutton state to 0 or 1 (unselected or selected).
#    getKeyw
#       returns the associated setup keyw. + value. This method reads
#       from the option button and entry widget the current user
#       settings and prepare a string to be sent to ICS control process.
#       This method is normally called when the SETUP cmd is invoked.
#    setKeyw val
#       extract from "val" the keyword parameter + value and fill the
#       corresponding option button + entry. This method is called usually
#       during a reading of a previously saved setup file.
#    
#   PROTECTED METHODS
#    updateWdg item
#       this method is called to modify on-the-fly the appearance of
#       the widgets. E.g. if a particular param. selection requires also some
#       input values, the corresponding entry widget is enabled,
#       otherwise it is disabled. User can override this method to 
#       implement some specific behavior.
#
#   PUBLIC VARIABLES
#    cfgObj
#       ctooCONFIG object (configuration file interface)
#    devType
#       device type extracted from INS.CON.DEVICE${i}
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTIONS
#
#   SEE ALSO
#    ctooCONFIG(8)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

class icbpanDpol {
   inherit icbpanDpol_uifClass

   constructor {args} {
      global ::shArea
      eval itk_initialize -helparea $shArea -relief flat $args
      display
      after idle [code $this initFunction]
   }

   method initFunction { } {

      # determine prefix to be used
      set prefix [icbpanGetDevicePrefix $cfgObj $devType]

      # initialize option lists
      foreach name $names {
          lappend elements $name
          lappend helps ""
      }
      lappend elements $encoder
      lappend helps "Set encoder value"
      lappend elements $encrel
      lappend helps "Set encoder relative value"

      # initialize option button widget (value from status)
      upvar #0 [ $cv.devStatus0 cget -variable] wdgVar
      set last [getOptName $wdgVar]
      $cv.devSetup configure -itemList $elements -default $last \
                             -command [code $this updateWdg]
      # initialize encoder entry widget
      configEnc $last

   }


   public method updateWdg { item } {
      global ::gvar

      if {[cequal $item ""]} {return 0}

      # select check button
      setSelected 1

      # configure entry widget
      configEnc $item
      if {[cequal $item $last]} {; # option unchanged
	  set value [getEnc]
      } else {; # option changed
          set value ""; # clean entry widget value
      }
      setEnc $value

      set last $item
   }

   # return status of the checkbutton
   public method isSelected { } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      return $sel
   }


   # set status of the checkbutton
   public method setSelected { val } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      set sel $val
   }


   # return associated setup keyword
   public method getKeyw { } {
      global ::gvar

      upvar #0 [$cv.devSetupEnc cget -variable] val
      upvar #0 [$cv.devSetup cget -variable] nameval

      if {[cequal $nameval ""]} { return "" }

      if {[cequal $nameval $encoder]} {
         set setup "${prefix}.ENC $val"
      } elseif {[cequal $nameval $encrel]} {
         set setup "${prefix}.ENCREL $val"
      } else {
         set setup "${prefix}.ST [getOptVal $nameval]"
      }

      return $setup
   }

   # set setup keyword+value
   public method setKeyw { val } {
      global ::gvar

      set keyw [lindex $val 0]
      set value [lindex $val 1]

      set newOption ""
      set newValue ""
      if {[cequal ${prefix}.ST $keyw]} {
         setSelected 1
	 set newOption [getOptName $value]
      } elseif { [cequal ${prefix}.ENC $keyw] } {
         setSelected 1
         set newValue $value
         set newOption $encoder
      } elseif { [cequal ${prefix}.ENCREL $keyw] } {
         setSelected 1
         set newValue $value
         set newOption $encrel
      } else {
         set newOption ""
      }
      if {![cequal $newOption ""]} {
          set last $newOption
          $cv.devSetup configure -default $last
          configEnc $last
          setEnc $newValue
      }
   }

   # return the list of entry widgets whose variable must be traced
   public method entryWdg { } {

       return {devSetupEnc}
   }
   
   # configure encoder entry widget
   protected method configEnc { item } {

       set index [lsearch -exact $elements $item]
       if {$index >= 0} {; # valid option
	   set txt [lindex $helps  $index]
       } else {; #unknown option
           set txt ""
       }
       $cv.devSetupEnc configure -id $item
       $cv.devSetupEnc configure -valType integer
       $cv.devSetupEnc configure -minVal ""
       $cv.devSetupEnc configure -maxVal ""
       $cv.devSetupEnc configHelp $txt
       if {[cequal $item $encoder] || [cequal $item $encrel]} {
           $cv.devSetupEnc enable
       } else {
           $cv.devSetupEnc disable
       }
   }
   
   # set value in encoder entry widget
   protected method setEnc { value } {
       $cv.devSetupEnc configure -default $value
       $cv.devSetupEnc setValue $value
   }

   # get value in encoder entry widget
   protected method getEnc { } {
       upvar #0 [ $cv.devSetupEnc cget -variable] wdgVar
       return $wdgVar
   }

   # return value from name
   private method getOptVal { name } {
       set index [lsearch -exact $elements $name]
       if { $index < 0 } { set value $name
       } else { set value [lindex $values $index] }
       return $value
   }

   # return name from value
   private method getOptName { value } {
      set name $value
      if { [lsearch -exact $elements $value] < 0 } {
          set index [lsearch -exact $values $value]
          # In WS simulation, it writes 0/1
          if {$index < 0} { set index [lsearch -exact $simvalues $value] }
          if {$index < 0} { set name ""
          } else { set name [lindex $elements $index] }
      }
      return $name
   }

   public variable devType ""
   public variable cfgObj  ""

   private variable prefix ""
   private variable encoder "enc"
   private variable encrel  "encrel"
   private variable elements {}
   private variable helps {}
   private variable values    {T F}
   private variable simvalues {1 0}
   private variable names  {ON OFF}
   private variable last   ""

}


#
# ___oOo___
