#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanEntry.tcl,v 1.47 2005/07/01 13:25:41 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------------------
# alongino  19/11/04  VLTSW20040349
# pbiereic  11/04/03  created
#
#*******************************************************************************
#   NAME
#     icbpanEntry - UifEntry derived class
#
#   SYNOPSIS
#     icbpanEntry  <widget-name>
#
#   DESCRIPTION
#     icbpanEntry is a mega-widget which can be imported and configured
#     with the panel editor.
#     It inherits the standard UifEntry class and adds specific features
#     required by icbpan:
#
#     - validate entry value:   highlight background when a wrong value is
#                               entered or undo "bad" typing
#     - handle key events:      see SPECIFIC BINDINGS
#     - return entry value
#
#   PUBLIC METHODS
#     public method getSelected { updateBackground }
#       - return entry value after checking for validity.
#         Return empty string if entry value is not valid.
#         If updateBackground=1, update background according to validity
#         default=0 (do not update background)
#     public method configHelp { txt }
#       - configure short help text
#     public method setValue { val }
#       - set entry value
#
#   WIDGET OPTIONS
#     valType     <string>      Value type (integer, float, any), default=float
#     command     <string>      Command callback: evaluated when a non-empty value
#                               was entered; optional, default={}. Adds entry value.
#     justify     <string>      Where entered characters start, default=left
#     undo        <logical>     Automatically undo "bad" typing (0=no, 1=yes), 
#                               default=no
#     colBadVal   <string>      Background color when a "bad" char was entered, 
#                               default=red
#     id          <string>      Entry id. Used to distinguish among different entry 
#                               types (e.g. user units versus encoder units)
#
#   SPECIFIC BINDINGS
#     <KeyRelease>              Checks the value
#     <KeyPress-Escape>         Clears the entry
#     <ButtonRelease-3>         Clears the entry
#     <Key-Up>                  Displays last entered value (or the value returned by
#                               method getSelected)
#     <Return>                  Returns the value
#
#   CAUTIONS
#
#   EXAMPLES
#     $cv.icbpanMOTEnc config \
#         -valType     integer \
# 	  -command     [code $this encoderEnterCb] \
# 	  -justify     right \
# 	  -undo        0 \
#         -colBadVal   red
#
#   SEE ALSO
#     UifEntry
#
#   BUGS     
#
#------------------------------------------------------------------------
#

class icbpanEntry {
    inherit UifEntry

    constructor {args} {
	global shArea EDITING
        # If editing with panel editor and return here, all settings (e.g. width, label etc)
        # are lots, i.e. when class containing this widget is reloaded the default settings
        # is taken instead of the last edited one.
	#if { $EDITING } { return }
	eval itk_initialize -helparea $shArea $args

	set uifEntry_ $itk_component(uifEntry)
	set bgcol_    [$uifEntry_ cget -background]

	# configure widget
	config -variable "$wthis.[cget -variable]"
	$uifEntry_ configure -justify $justify
        configure -units [cget -units]; # needed to show units on the right side

        set autoHelp_ -1
        # User defined short help text is loaded only after construction
        # Useless to check here: it will always result in empty user defined short-help
	#configHelp

 	# Add binding to handle key events
 	bind $uifEntry_ <Return>          [code $this enterCb]
 	bind $uifEntry_ <KeyRelease>      [code $this traceEntry]
 	bind $uifEntry_ <KeyPress-Escape> [code $this erase]
 	bind $uifEntry_ <ButtonRelease-3> [code $this eraseCb]
 	bind $uifEntry_ <Key-Up>          [code $this entryLast]
	update
    }

    #
    # return the value of the entry field
    #
    public method getSelected { {updateBackground 0} } {
	if { ! [valid] } {
            if { $updateBackground } {
	        $uifEntry_ configure -background $colBadVal
	    }
            return {} 
        }
        if { $updateBackground } {
	    $uifEntry_ configure -background $bgcol_
	}

        # store last entered value for later usage (Up key)
        set last_($id) [string trim [get]]
	return $last_($id)
    }

    #
    # configure help text
    #
    public method configHelp { {txt "Set encoder/position value"} } {
        global ::panWidgetHlp

        if { $autoHelp_ < 0 } {; # first time this method is called
   	    if { "[cget -helptext]" == "" } { 
                set autoHelp_ 1
            } else {
                set autoHelp_ 0
            }
	}
        if { $autoHelp_ == 0 } { return }

        set help $txt
        if { $help != "" } {
	    set help "$help     <Up>=last, <Escape> or <Right click>=clear"

   	    if { "[cget -command]" != "" } {
   	        set help "$help, <Return>=enter value"
    	    }
        }

        # Panel does not handle an empty string properly: add a blank as workaround
        if { $help == "" } { set help " " }
        configure -helptext "$help"
    }

    #
    # set entry value
    #
    public method setValue { val } {
        set insert [$uifEntry_ index insert]
        erase
	$uifEntry_ insert 0 $val
        # update background color
        traceEntry
        # keep cursor where it was (VLTSW20040349)
        $uifEntry_ icursor $insert
    }

    #
    # enable
    # Need to overload for proper return on cget -state
    #
    public method enable { } {
        $this configure -state normal
	UifEntry::enable
    }

    #
    # disable
    # Need to overload for proper return on cget -state
    #
    public method disable { } {
        $this configure -state disabled
	UifEntry::disable
    }

    #
    # enterCb: called when a value was entered with <Return>
    #
    private method enterCb { } {
	set value [getSelected]
	if { "$value" == "" } { return }

        # To allow external tracing (e.g. to activate a check button)
        # rewrite the value in the entry field
        setValue $value

	# call the command callback
	if { "$command" != "" } {
	    eval $command {"$value"}
	    erase
	}
    }

    #
    # eraseCb: erase entry contents and remove bad background
    #
    private method eraseCb { } {
        erase
        traceEntry
    }

    #
    # trace typed chars. Highlight background or undo if there is a typing error
    #
    private method traceEntry { } {
	if { [valid 1] } {
	    $uifEntry_ configure -background $bgcol_
	    return
 	}
	if { ! $undo } {
	    $uifEntry_ configure -background $colBadVal
	} else {
	    set insert [expr {[$uifEntry_ index insert] - 1}]
	    $uifEntry_ delete  $insert
	    $uifEntry_ icursor $insert
	}
    }

    #
    # Check if entry string is valid. Return 1 if yes else 0.
    #
    private method valid { { typing 0 } } {
	if { "$valType" == "any" } {return 1 }

	set value [string trimleft [get]]
        set min $itk_option(-minVal)
        set max $itk_option(-maxVal)
	if { $typing && ("$value" == "-" || "$value" == "") } { return 1 }

	if { "$valType" == "integer" } {
	    set status [expr {! [catch {string index dummy $value}]}]
	} elseif { "$valType" == "float" } {
	    if { "$value" == "" } { set status 0
            } else { set status [expr {! [catch {expr {$value + 1.0}}]}]}
	}
        # check range, if defined
        if {![cequal $min "" ] && $status} {
            if { $value < $min } { set status 0 }
        }
        if {![cequal $max "" ] && $status} {
            if { $value > $max } { set status 0 }
        }
	return $status
    }

    #
    # insert last entry value
    #
    private method entryLast { } {
        if { ![info exists last_($id)] } {
            set last_($id) $itk_option(-default)
        }
        if {[cequal $last_($id) "" ]} { return }
	erase
	$uifEntry_ insert 0 $last_($id)
    }

    #-------------------------------------------------------------------
    # Private variables
    #-------------------------------------------------------------------
    # last value entered
    private variable last_

    # uifEntry component
    private variable uifEntry_

    # default background color
    private variable bgcol_

    # Change automatically short-help text, depending on the context
    private variable autoHelp_

    #-------------------------------------------------------------------
    # Public variables
    #-------------------------------------------------------------------
    public variable valType   {float}
    public variable undo      {0}
    public variable command   {}
    public variable justify   {left}
    public variable colBadVal {red}
    public variable id        {undefined}
}

#___oOo___
