#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanGUIcfg.tcl,v 1.47 2005/07/01 13:25:40 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  24/11/04  support for assemblies (VLTSW20040336)
# alongino  14/08/03  implemented mutli-tiers notebook (VLTSW20030303)
# alongino  12/12/01  implemented offset for state output widget (VLTSW20010690)
# dimarcan  16/07/01  created
#

#************************************************************************
#   NAME
#    icbpanGUIcfg - itcl class used to construct ICB control panel.
# 
#   SYNOPSIS
#    icbpanGUIcfg xxinsGUI xxinsCfg
# 
#   DESCRIPTION
#    This itcl class is an utility class which contains a set
#    of methods to pack notebooks and control classes on the 
#    ICB control panel.
#
#   PUBLIC METHODS
#    icbpanGUIcfg xxinsGUI xxinsCfg
#       constructor. An object of type ctooCONFIG must be passed to
#       properly read all the requested information.
#    lcuBootMenu <menuPath>
#       add on the specified panel menu (individuated by the passed menuPath
#       string) commands to reboot LCU(s)
#       (the number of added cmds correspond to the number of defined LCUs).
#    wdgICScontrol <frameICS1>
#       pack class icbpanICSControl on frame frameICS1.
#    wdgICScmd <frameICS2>
#       pack class icbpanICSCmd on frame frameICS2.
#    funcICScontrol <frameFunc> <devicesList>
#       pack class icbpanICSDevControl on frame frameFunc. DevicesList
#       contains the number of configured devices. This method must be
#       called after notebooks are created.
#    createNotebooks <framesList>
#       pack notebooks on frames extracted from framesList.
#       The number of tabs in each notebook and the relative labels
#       are extracted from INSi.UIFi.TABS and INSi.UIFi.LABELj. 
#       After notebooks construction the method reads the configuration
#       file and places all configured devices in the specific 
#       notebook. The look and feel is controlled by a set of dedicated
#       configuration keywords.
#       At the end it returns the number of drawn devices as a list.
#       The just created list could be used then as a parameter for
#       the "funcICScontrol" method.
#    getNotebooks <cv>
#       return a list consisting of { <framesList> <framesGeometry> } where
#       <framesList> is the list of notebook frames to be passed to createNotebooks
#       <framesGeometry> is a list of items { <xCoord> <yCoord> <width> <height> }
#       <cv> is the panel variable associated to the canvas where 
#       notebook frames are.
#    setNotebookDim [<nbIdx 1>] [<width 300>] [<height 300>]
#       set new dimension for the notebook identified by a number.
#    setNotebookDimAll <framesGeometry>
#       set new dimension for all notebooks in <framesGeometry>
#       Every element of <framesGeometry> consists of <xCoord> <yCoord> <width> <height>
#    setPanelDim [<framesGeometry/width >] [<height>]
#       set new dimension for the panel. 
#       If two parameters are passed, they are interpreted as <width> <height>
#       If only one parameter is passed, it is interpreted as <framesGeometry>
#       as from getNotebooks
#    initClass <cv> <dimList>
#       initialize uifClasses and frames in the panel 
#       At the end it returns the number of drawn devices as a list.
#       <cv> is the panel variable associated to the canvas where 
#       notebook frames are.
#       <dimList> is the list of notebooks dimension
#       Every element of <dimList> consists of <width> <height>
#
#   PRIVATE METHOD
#    putUifClasses
#       For every device defined in the cfg file, pack the relative
#       class. The position (notebook+tab) is extracted from
#       $devType.UIFNB and $devType.UIFTB (devType==INSi.CON.DEVICE); the
#       class to be placed is read either from devType.UIFCLASS or
#       extracted with the method getUifClass of icbpanUIF class.
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTIONS
#
#   SEE ALSO
#    ctooCONFIG(8)
#
#   BUGS     
#
#------------------------------------------------------------------------
#


class icbpanGUIcfg {

   constructor { insCfg args } {

      set cfgObj $insCfg
      eval configure $args
      set cfgcat [icbpanCfgCat $cfgObj]
      setInternalCfg
      ctooDICT $this.dicObj -dict $icsDict
      set dicObj $this.dicObj
      icbpanDictionary $dicObj
   }

   destructor {
       delete $dicObj
       foreach evt $evtList {
          seq_evtDetach $evt
       }
   }

   public method lcuBootMenu { menuPath } {

      if {[cequal $cfgObj ""]} {return ""}
      set lcuList [icbpanLcus $cfgObj]
      foreach lcu $lcuList {
	 set lcuenv [lindex $lcu 0]
	 set lcuindex [lindex $lcu 1]
         set cmd "icbpanLcuBoot $lcuenv"
         if {[ catch { addMenuCmd $menuPath "Reboot LCU$lcuindex ($lcuenv)" 0 $cmd } res ]} {
            icbpanDialog "addMenuCmd error; wrong widget path: $menuPath"
            return ""
         }
      }
   }


   public method wdgICSState { frameICSState {ctrlClass icbpanICSState} } {
      set ret [setInternalCfg]
      if {$ret} { return "" }

      set tmp [$ctrlClass ${frameICSState}.icbpanICSState \
               -background Grey90   \
               -relief raised       \
               -cwp "<alias>${icsDbRoot}:PROCESSES:WS:icsControl" \
               -procName $procName \
               -cfgObj $cfgObj]
      pack $tmp -side left -anchor e
   }


   public method wdgICScontrol { frameICS1 {ctrlClass icbpanICSControl} } {
      set ret [setInternalCfg]
      if {$ret} { return "" }

      set tmp [$ctrlClass ${frameICS1}.icbpanICSControl \
               -background Grey90   \
               -relief raised       \
               -cwp "<alias>${icsDbRoot}:PROCESSES:WS:icsControl" \
               -procName $procName \
               -cfgObj $cfgObj]
      pack $tmp -side left -anchor e
   }


   public method wdgICScmd { frameICS2 {ctrlClass icbpanICSCmd} } {
      set ret [setInternalCfg]
      if {$ret} { return "" }

      set tmp [$ctrlClass ${frameICS2}.icbpanICSCmd \
               -background Grey90 \
               -relief flat       \
               -cwp "<alias>${icsDbRoot}:PROCESSES:WS:icsControl" \
               -procName $procName]
      pack $tmp -side left -anchor e
   }




   public method funcICScontrol { frameFunc devicesList {ctrlClass icbpanICSDevControl} } {
      set ret [setInternalCfg]
      if {$ret} { return "" }

      set tmp [$ctrlClass ${frameFunc}.icbpanICSDevControl \
                               -background Grey90       \
                               -allDevices $devicesList \
                               -cfgObj $cfgObj          \
                               -procName $procName      \
                               -relief raised]
      pack $tmp -side top -anchor w
   }


  
   public method createNotebooks { framesList framesGeometry } {
      if {[cequal $cfgObj ""]} {return ""}

      if {[ catch { $cfgObj Get $cfgcat.UIF.NBOOKS } nbooks ]} {
         set nbooks 1
      }
      set nframes [llength $framesList]
      if { $nframes != $nbooks } {
         icbpanDialog "notebooks frames ($nframes) and $cfgcat.UIF.NBOOKS ($nbooks) must be the same!"
         return ""
      }
      incr nbooks

      loop i 1 $nbooks {
	 set idx [expr {$i - 1}]
         set frameName [lindex $framesList $idx]
         set frameGeometry [lindex $framesGeometry $idx]
	 set frameWidth [lindex $frameGeometry 2]

         set tmpNotebook \
            [icbpan::tabnotebook ${frameName}.tab \
                         -tabpos n        \
                         -tiers 5         \
                         -width $frameWidth \
                         -backdrop Grey90 \
                         -equaltabs false \
                         -raiseselect 1]
         pack $tmpNotebook -fill both -side left -expand yes
         update

         # structure used to save current notebook name
         keylset notebookStruct $i $tmpNotebook
   
         set tabChild {}
         if {[ catch { $cfgObj Get $cfgcat.UIF${i}.TABS } tabsNum ]} {
            set tabsNum 1
         }
         if { $tabsNum < 1 } {
            icbpanDialog "$cfgcat.UIF${i}.TABS must be > 0 !!"
            return "" 
         }
         incr tabsNum
         loop j 1 $tabsNum {
	    set newIdx [expr {$j - 1}]
            if {[ catch { $cfgObj Get $cfgcat.UIF${i}.LABEL${j} } lbl ]} {
               set lbl ""
            }
	    lappend tabChild [$tmpNotebook add -label $lbl]
         }

         $tmpNotebook view 0
         # structure used to save current notebook childsite
         keylset notebookStruct LBL$i $tabChild
      }

      return [putUifClasses $nbooks]

   }



   private method putUifClasses { nbooks } {
      if {[cequal $cfgObj ""]} {return ""}

      set allDevices {}

      catch {eval delete object [find objects *handleUif*]}
      if {[ catch { icbpanUIF handleUif -cfgObj $cfgObj} res ]} {
         icbpanDialog $res
         return ""
      }

      set devNo [icbpanGetDeviceNum $cfgObj]
      set devTypes {}
      set devNames {}
      set devDescs {}
      set n [expr {$devNo +1}]
      loop i 1 $n {
         if {[ catch { $cfgObj Get $cfgcat.CON.DEVICE${i} } devType ]} {
            icbpanDialog $devType
            return "" 
         }
         if {[ catch { $cfgObj Get ${devType}.DEVNAME } devName ]} {
            icbpanDialog $devName
            return "" 
         }
         if {[ catch { $cfgObj Get ${devType}.DEVDESC } devDesc ]} {
            set devDesc "${devType}.DEVDESC not found"
         }
         lappend devTypes $devType
         lappend devNames $devName
         lappend devDescs $devDesc
      }
      # consider also assemblies (VLTSW20040336)
      set assNo [icbpanGetAssemblyNum $cfgObj]
      set n [expr {$assNo +1}]
      loop i 1 $n {
         set devType $cfgcat.ASSEMBLY${i}
         lappend devTypes $devType
         if {[ catch { $cfgObj Get ${devType}.NAME } devName ]} {
            set devName "${devType}.NAME not found"
	 }
         if {[ catch { $cfgObj Get ${devType}.DESC } devDesc ]} {
            set devDesc "${devType}.DESC not found"
	 }
         lappend devNames $devName
         lappend devDescs $devDesc         
      }

      set devAssNo [expr {$devNo + $assNo}]
      set n [expr {$devNo + $assNo}]
      loop i 0 $n {

	 set devType [lindex $devTypes $i]
	 set devName [lindex $devNames $i]
	 set devDesc [lindex $devDescs $i]
         set subsysBegin [expr {[string length $cfgcat] + 1}]
         set className [string tolower [csubstr $devType $subsysBegin end]]
         if { [icbpanIsAssembly $devType] } {
            set devDbRoot ${icsDbRoot}:ASSEMBLIES
         } else {
            set devDbRoot [string toupper $devName]
	 }

         if {[ catch { $cfgObj Get ${devType}.UIFCLASS } classType ]} {
            set classType [handleUif getUifClass $devType]
         }
         set addUifClass 1
         if {[cequal $classType ""]} { set addUifClass 0 }
         # For assemblies, if UIFTB is not specified, do not display any widget (back compatible)
         if { [icbpanIsAssembly $devType] } {
	     if { [catch { $cfgObj Get ${devType}.UIFTB }] } { set addUifClass 0 }
	 }
         if {$addUifClass} {

            if {[ catch { $cfgObj Get ${devType}.UIFNB } nbIdx ]} {
               set nbIdx 1
            }
            if {[ catch { $cfgObj Get ${devType}.UIFTB } tbIdx ]} {
               set tbIdx 1
            }

            if {[ catch { $cfgObj Get ${devType}.UIFSPECIAL } stateOffset ]} {
               set stateOffset F
            }

            if { ($nbIdx > [expr {$nbooks -1}]) || ($nbIdx < 1) } {
               icbpanDialog "Wrong ${devType}.UIFNB"
               return "" 
            }

            set nbook LBL${nbIdx}
            try_eval {
               set tab [keylget notebookStruct $nbook]
            } {
               icbpanDialog "Notebook frame not found."
               return "" 
            }
            incr tbIdx -1
            set frame [lindex $tab $tbIdx]
            if { [cequal $frame ""] } {
               icbpanDialog "Wrong ${devType}.UIFTB"
               return "" 
            }

            if {[ catch { $classType ${frame}.$className \
                                      -background Grey90 \
                                      -relief flat       \
                                      -cfgObj $cfgObj    \
                                      -cwp "<alias>${devDbRoot}" \
                                      -devType $devType } tmp ]} {
               icbpanDialog "Error creating widget of type $classType"
               return "" 
            }
            set devWdg ${frame}.$className
            lappend allDevices $devWdg
            update idletasks; # needed otherwise initFunction is executed later
            # state values for icb special devices are offsetted by one with 
            # respect to standard devices (historical reason).
            # The state output field must be correspondingly corrected,
            # whenever a default icbpan widget class is used for a special device
            if {[cequal $stateOffset "T"]} {
               set values [$devWdg.cv.devState cget -listValues]
               set valDim [llength $values]
               for { set j 0 } { $j < $valDim } { incr j } {
                  set value [lindex $values $j]
                  incr value
                  set values [lreplace $values $j $j $value]
	       }
               $devWdg.cv.devState configure -listValues $values
               # update value displayed
               upvar #0 [ $devWdg.cv.devState cget -variable] wdgVar
               $devWdg.cv.devState setValue $wdgVar
	    }
            # attach events for simulation and handset
            if { ![icbpanIsAssembly $devType] } {
               set script "[code $this updateSim $devWdg $devDbRoot]"
               $script
               if {![catch {seq_evtAttach "<alias>${devDbRoot}.simulation" W $script} evt]} {
                   lappend evtList $evt
	       }
               if {![catch {seq_evtAttach "<alias>${devDbRoot}.handset" W $script} evt]} {
                   lappend evtList $evt
	       }
            }

            # Add tracing criteria for entry widgets (VLTSW20030305)
            if {![ catch { $devWdg entryWdg} entryWidgets]} {
                if {[llength $entryWidgets] > 0 } {
                    set wdgList {}
                    foreach entryWdg $entryWidgets {
                        lappend entryVarDev $devWdg
                        set wdg cv.$entryWdg
                        if {[ catch { $devWdg.$wdg cget -variable } entryVar]} {
                            set wdg $entryWdg
                            set entryVar [$devWdg.$wdg cget -variable]
			}
                        lappend entryWdgList $wdg
                        lappend wdgList $wdg
                        lappend entryVarList $entryVar
                        upvar #0 $entryVar var
                        trace variable var w [code $this updateSelect]
		    }
                    # if device deselected, entry widgets must be cleaned from e.g. red background
                    set cmd [$devWdg.cv.devUse cget -command]
                    if {[cequal $cmd ""]} {
                        set cmd "[code $this updateEntry] $devWdg {$wdgList}"
                        $devWdg.cv.devUse configure -command $cmd
		    }
                }               
            }

            # add short help text for the whole device
            if {[ catch { $devWdg shortHelp} txt]} {
                set devPrefix [icbpanGetDevicePrefix $cfgObj $devType]
                set devLcuId [icbpanGetDeviceLcuId $cfgObj $devType]
                set devLcuName [icbpanGetDeviceLcuName $cfgObj $devType]
                set txt "$devDesc      FITS prefix: $devPrefix"
                if { $devLcuId != "" } {
                   set txt "$txt      LCU: $devLcuId ($devLcuName)"
                }
	    }
            catch {UifShortHelp::add [$devWdg.cv.device cget -helparea] $devWdg.cv.device $txt}

            # pack device widget
            pack $tmp -side top -anchor w
         }

      }

      return $allDevices
   }


   public method getFrameGeometry { cv name wdgName } {

      set geometry {}
      set coord [$cv coord $name]
      set x [expr {round([lindex $coord 0])}]
      set y [expr {round([lindex $coord 1])}]
      set width  [$wdgName cget -width]
      set height [$wdgName cget -height]
      return [list $x $y $width $height]
   }

   public method getNotebooks { cv } {

      set ret [setInternalCfg]
      if {$ret} { return "" }

      set framesWdg {}
      set framesGeometry {}
      if {[ catch { $cfgObj Get $cfgcat.UIF.NBOOKS } nbooks ]} {
         set nbooks 1
      }
      if { $nbooks < 1 } {
         icbpanDialog "$cfgcat.UIF.NBOOKS must be > 0 !!"
         return "" 
      }

      incr nbooks
      loop i 1 $nbooks {
          set frameName ${icsPrefix}ipanNotebook$i
          set frameNotebook($i) $cv.$frameName.uifFrame
          if {[catch { $frameNotebook($i) configure -relief flat } ret ]} {
	      icbpanDialog "frame $frameNotebook($i) not found !"
              return "" 
          }
          lappend framesGeometry [getFrameGeometry $cv $frameName $frameNotebook($i)]
          lappend framesWdg $frameNotebook($i) 
      }
      return [list $framesWdg $framesGeometry]
  }



   public method setNotebookDim { {nbIdx 1} {width 300} {height 300} } {
      try_eval {
         set tabHull [keylget notebookStruct $nbIdx]
         $tabHull configure -width $width -height $height
      } {
         icbpanDialog "Cannot resize notebook $nbIdx!"
         return "" 
      }
   }



   public method setNotebookDimAll { framesGeometry } {
      if {[ catch { $cfgObj Get $cfgcat.UIF.NBOOKS } nbooks ]} {
         set nbooks 1
      }

      set sizeDim [llength $framesGeometry]
      if { $sizeDim != $nbooks } {
          icbpanDialog "list of notebook sizes must have $cfgcat.UIF.NBOOKS items !"
          return "" 
      }
      loop i 0 $nbooks {
          set geometry [lindex $framesGeometry $i]
          set width  [lindex $geometry 2]
          set height [lindex $geometry 3]
          setNotebookDim [expr {$i + 1}] $width $height
      }
   }



   public method setPanelDim { {framesGeometry ""} {height ""} } {
      global ::bcv

      # user settings
      set width $framesGeometry
      if { (![cequal $width ""]) && (![cequal $height ""]) } {
         wm geometry . ${width}x${height}+0+0
         return ""
      }

      set maxY  0
      set maxX  0
      set nframes [llength $framesGeometry]
      loop i 0 $nframes {
         try_eval {
            set geometry [lindex $framesGeometry $i]
            # takes the notebook with the highest value of yCoord+height
            set height [lindex $geometry 3]
            set yCoord [lindex $geometry 1]
	    set heiDev [expr {$yCoord + $height}]
            if { $heiDev > $maxY } { set maxY $heiDev }

            # takes the notebook with the highest value of xCoord+width
            set width [lindex $geometry 2]
            set xCoord [lindex $geometry 0]
	    set widDev [expr {$xCoord + $width}]
            if { $widDev > $maxX } { set maxX $widDev }
         } {
            # default setting; should never happen
            wm geometry . 1000x1000+0+0
            return ""
         }
      }
      set panHeight [expr {200 + $maxY}]
      if {![ catch { $bcv.icbpanSetup cget -command } ]} {; # action buttons are present
           incr panHeight 25
      }
      set panWidth [expr {10 + $maxX}]
      # Avoid to truncate menu options
      if {$panWidth < $minPanWid} { set panWidth $minPanWid }
      wm geometry . ${panWidth}x${panHeight}+0+0
   }



   public method initClass { cv } {
       global pollRate

       set ret [setInternalCfg]
       if {$ret} { return "" }

       # ***********************************************************
       # STEP 1:
       # the following frames are used to position global ICS
       # control uifClasses
       # (single function control class is constructed at the end)
       #
       # xxipanICSControl1 xxipanICSControl2 xxipanFuncControl obsolete (VLTSW20030305)
       # Kept for backwards compatibility
       set frameICS1Name ${icsPrefix}ipanICSControl1
       set frameICS1 $cv.$frameICS1Name.uifFrame
       set frameICS2Name ${icsPrefix}ipanICSControl2
       set frameICS2 $cv.$frameICS2Name.uifFrame
       set frameFuncName ${icsPrefix}ipanFuncControl
       set frameFunc $cv.$frameFuncName.uifFrame
       # Some or all of them may be missing
       catch {
           $frameICS1 configure -background black
           wdgICScontrol $frameICS1
       }
       catch {
           $frameICS2 configure -background black
           wdgICScmd $frameICS2
       }
       catch {
           $frameFunc configure -background red  
       }
       # xxipanICSState added (VLTSW20030305)
       set frameICSStateName ${icsPrefix}ipanICSState
       set frameICSState $cv.$frameICSStateName.uifFrame
       # In old panels it may be missing
       catch {
           $frameICSState configure -background black
           wdgICSState $frameICSState
       }


       # END STEP 1
       # *************************************************************

       # **************************************************************
       # STEP 2:
       #   - set global variables for action buttons
       #
       icbpanCfgSet $cfgObj
       icbpanProcname $procName
       icbpanDbRoot $icsDbRoot
       icbpanTimeoutMove $cfgObj
       # END STEP 2
       # *************************************************************

       # **************************************************************
       # STEP 3:
       # the following frame(s) are used to position the tabnotebook(s)
       #   - read INS cfg file and load on the ICS control panel the
       #     selected uifClasses
       #   - get the list of drawn devices (variable allDevices)
       #

       set notebookFrames [getNotebooks $cv]
       if {[cequal $notebookFrames ""]} { return "" }
       set framesList [lindex $notebookFrames 0]
       set framesGeometry [lindex $notebookFrames 1]
       set allDevices [createNotebooks $framesList $framesGeometry]
       #if {[cequal $allDevices ""]} { return "" }
       icbpanDevices $allDevices

       # END STEP 3
       # *************************************************************

       # **************************************************************
       # STEP 4:
       #   - resize notebooks and panel
       #

       setNotebookDimAll $framesGeometry
       catch {funcICScontrol $frameFunc $allDevices}

       if {![ catch { getFrameGeometry $cv $frameICS1Name $frameICS1 } frameICS1Geometry ]} {
           lappend framesGeometry $frameICS1Geometry
       }
       if {![ catch { getFrameGeometry $cv $frameICS2Name $frameICS2 } frameICS2Geometry ]} {
           lappend framesGeometry $frameICS2Geometry
       }
       if {![ catch { getFrameGeometry $cv $frameFuncName $frameFunc } frameFuncGeometry ]} {
           lappend framesGeometry $frameFuncGeometry
       }
       setPanelDim $framesGeometry

       update

       # END STEP 4
       # *************************************************************
       
       # **************************************************************
       # STEP 5:
       #   - start polling of OLDB values, if needed
       #
       icbpanDbPoll $allDevices $pollRate

       # END STEP 5
       # *************************************************************

       return $allDevices
    }


   # update device selection check button on change of entry value
   protected method updateSelect {name element op} {

       if {![cequal $element ""]} { set name ${name}($element) }
       set index [lsearch -exact $entryVarList $name]
       set devWdg [lindex $entryVarDev $index]
       upvar $name value
       set entryWdg [lindex $entryWdgList $index]
       $devWdg.$entryWdg setValue $value; # update background
       set state [$devWdg.$entryWdg cget -state]
       if {[cequal $state normal]} {; # if disabled, ignore it
           if {![cequal $value ""]} { $devWdg setSelected 1
           } else { $devWdg setSelected 0 }
       }
   }

   # update device entry fields when device selected/deselected
   protected method updateEntry { devWdg entryWidgets } {

       if { ![$devWdg isSelected] } {; # if deselected, clean red background
           foreach entryWdg $entryWidgets {
               set value [$devWdg.$entryWdg getSelected]
               if {[cequal $value ""]} {
                   $devWdg.$entryWdg setValue $value
	       }
	   }
       }
   }

   # update simulation field
   private method updateSim { devWdg devDbRoot } {

       set sim  [icbpanDbRead "<alias>${devDbRoot}.simulation"]
       set hset [icbpanDbRead "<alias>${devDbRoot}.handset"]
       set value [expr {$hset + 2 * $sim}]
       catch {$devWdg.cv.devSimHset setValue $value}
   }

   private method setInternalCfg { } {
      if {[cequal $cfgObj ""]} {return 1}

      set insId [icbpanInsId $cfgObj]
      if {[cequal $insId ""]} {return 1}

      set icsPrefix [icbpanIcsPrefix $cfgObj]
      if {[cequal $icsPrefix ""]} {return 1}

      set procName "${icsPrefix}iControl"
      set icsDbRoot ${insId}:ICS
      set icsDict   ${insId}_ICS

      return 0
   }


   private variable cfgObj     ""
   private variable dicObj     ""

   private variable insId    ""
   private variable icsDbRoot  ""
   private variable icsDict    ""
   private variable procName   ""
   private variable icsPrefix  ""
   private variable cfgcat     ""

   private variable notebookStruct {}
   private variable entryVarList {}
   private variable entryWdgList {}
   private variable entryVarDev {}
   private variable minPanWid 470
   private variable evtList {}

}

#
# ___oOo___
