#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanGrat.tcl,v 1.47 2005/07/01 13:25:39 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  08/01/02  extended to more than 2 positions
# oat       17/11/00  created
#

#************************************************************************
#   NAME
#    icbpanGrat - wrapper class for icbpanGrat_uifClass
# 
#   SYNOPSIS
#    icbpanGrat .w \
#              -background Grey90 \
#              -relief flat       \
#              -cfgObj $cfgObj    \
#              -cwp "<alias>${devName}" \
#              -devType $devType]
#
#     where cfgObj is a ctooCONFIG object;
#           devType is extracted from INS.CON.DEVICE${i}
#           devName is extracted from ${devType}.DEVNAME
# 
#   DESCRIPTION
#    Itcl wrapper class to handle ICB GRAT device. The corresponding
#    ibcpanGrat_uifClass contains a set of output widget (linked to OLDB)
#    which in turn display the device name, keyword used to identify the
#    device, the current simulation level (N=normal,S=simulate), the current
#    device state, handset mode (H, if active) and the current position
#    (named and in encoder unit). The class contains also an option
#    button to select among various setup parameters and two entry widget
#    for setting position values (wlen/encoder + order). Finally a 
#    checkbutton is used to indicate if the corresponding device is 
#    selected for some actions.
#
#   PUBLIC METHODS
#    icbpanGrat  xxx
#       constructor. Calls "initFunction" which initialize the option
#       button with appropriate parameters. 
#    isSelected
#       get the current status of the checkbutton (0/1).
#    setSelected 0/1
#       set the checkbutton state to 0 or 1 (unselected or selected).
#    getKeyw
#       returns the associated setup keyw. + value. This method reads
#       from the option button and entry widget the current user
#       settings and prepare a string to be sent to ICS control process.
#       This method is normally called when the SETUP cmd is invoked.
#    setKeyw val
#       extract from "val" the keyword parameter + value and fill the
#       corresponding option button + entry. This method is called usually
#       during a reading of a previously saved setup file.
#    
#   PROTECTED METHODS
#    updateWdg item
#       this method is called to modify on-the-fly the appearance of
#       the widgets. E.g. if a particular param. selection requires also some
#       input values, the corresponding entry widget is enabled,
#       otherwise it is disabled. User can override this method to 
#       implement some specific behavior.
#
#   PUBLIC VARIABLES
#    cfgObj
#       ctooCONFIG object (configuration file interface)
#    devType
#       device type extracted from INS.CON.DEVICE${i}
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTIONS
#
#   SEE ALSO
#    ctooCONFIG(8)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

class icbpanGrat {
   inherit icbpanGrat_uifClass

   constructor {args} {
      global shArea
      eval itk_initialize -helparea $shArea -relief flat $args
      display
      after idle [code $this initFunction]
   }

   method initFunction { } {

      # determine prefix to be used
      set prefix [icbpanGetDevicePrefix $cfgObj $devType]
      # get units
      set units [icbpanDefaultUnits ${prefix}.WLEN]

      # initialize option lists
      set posNum [$cfgObj Get ${devType}.POSNUM]
      loop i 1 [incr posNum 1] {
         lappend elements "[$cfgObj Get ${devType}.NAME$i]$wltxt"
         if {[catch { $cfgObj Get ${devType}.WLENMIN$i } val ]} { set val "" }
         lappend minVal $val
         set minTxt $val
	 if {[cequal $minTxt ""]} { set minTxt "N/A" }
         if {[catch { $cfgObj Get ${devType}.WLENMAX$i } val ]} { set val "" }
         lappend maxVal $val
         set maxTxt $val
	 if {[cequal $maxTxt ""]} { set maxTxt "N/A" }
         lappend helps "Set central wavelength (Units: $units Range: $minTxt : $maxTxt)"
         if {[catch { $cfgObj Get ${devType}.ORDERMAX$i } val ]} { set val "" }
         lappend maxOrd $val
         set maxTxt $val
	 if {[cequal $maxTxt ""]} { set maxTxt "N/A" }
         lappend helpsOrd "Set order (Range: $minOrd : $maxTxt)"
      }
      lappend elements $encoder
      lappend helps "Set encoder value"
      lappend helpsOrd "Set order"
      lappend elements $encrel
      lappend helps "Set encoder relative value"
      lappend helpsOrd "Set order"

      
      # initialize wavelength status widget
      set txt "Central wavelength (Units: $units)"
      UifShortHelp::add [$cv.devStatus1 cget -helparea] $cv.devStatus1 $txt

      # initialize option button widget (value from status)
      upvar #0 [ $cv.devStatus0 cget -variable] wdgVar
      set last "${wdgVar}$wltxt"
      if { [lsearch -exact $elements $last] < 0 } { set last "" }
      $cv.devSetup configure -itemList $elements -default $last \
                             -command [code $this updateWdg]

      # initialize encoder entry widget (value from status)
      configEnc $last
      upvar #0 [ $cv.devStatus1 cget -variable] wdgVar
      setEnc $wdgVar
      setEnc [getEnc 1]; # clean entry if illegal value

      # initialize order entry widget
      configOrd $last
      setOrd $minOrd
   }


   public method updateWdg { item } {
      global ::gvar

      if {[cequal $item ""]} {return 0}

      # select check button
      setSelected 1

      # configure entry widget
      configEnc $item
      configOrd $item
      if {[cequal $item $last]} {; # option unchanged
	  set valEnc [getEnc]
	  set valOrd [getOrd]
      } else {; # option changed
          set valEnc ""; # clean entry widget value
          set valOrd ""; # clean entry widget value
      }
      setEnc $valEnc
      setOrd $valOrd

      set last $item
   }

   # return status of the checkbutton
   public method isSelected { } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      return $sel
   }


   # set status of the checkbutton
   public method setSelected { val } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      set sel $val
   }


   # return associated setup keyword
   public method getKeyw { } {
      global ::gvar

      upvar #0 [$cv.devSetupEnc cget -variable] encval
      upvar #0 [$cv.devSetup cget -variable] nameval
      upvar #0 [$cv.devSetupOrd cget -variable] ordval

      if {[cequal $nameval ""]} { return "" }

      if {[cequal $nameval $encoder]} {
         set setup "${prefix}.ENC $encval"
      } elseif {[cequal $nameval $encrel]} {
         set setup "${prefix}.ENCREL $encval"
      } else {
	 set len [string first $wltxt $nameval]
         incr len -1
	 set grat [string range $nameval 0 $len]
         set setup "${prefix}.NAME $grat ${prefix}.WLEN $encval"
         set setup "$setup ${prefix}.ORDER $ordval"
      }

      return $setup
   }


   # set setup keyword+value
   public method setKeyw { val } {  
      global ::gvar

      set keyw [lindex $val 0]
      set value [lindex $val 1]

      set newOption ""
      if {[cequal ${prefix}.NAME $keyw]} {
         setSelected 1
         set newOption "${value}$wltxt"
      } elseif { [cequal ${prefix}.WLEN $keyw] } {
         setSelected 1
         setEnc $value
      } elseif { [cequal ${prefix}.ORDER $keyw] } {
         setSelected 1
         setOrd $value
      } elseif { [cequal ${prefix}.ENC $keyw] } {
         setSelected 1
         setEnc $value
         set newOption $encoder
      } elseif { [cequal ${prefix}.ENCREL $keyw] } {
         setSelected 1
         setEnc $value
         set newOption $encrel
      }
      if {![cequal $newOption ""]} {
          set last $newOption
          $cv.devSetup configure -default $last
          configEnc $last
          configOrd $last
      }

   }

   # return the list of entry widgets whose variable must be traced
   public method entryWdg { } {

       return {devSetupEnc devSetupOrd}
   }
   
   # configure encoder entry widget
   protected method configEnc { item } {

       set min ""
       set max ""
       set txt ""
       set valType float
       set index [lsearch -exact $elements $item]
       if {$index >= 0} {; # valid option
	   if {[cequal $item $encoder] || [cequal $item $encrel]} { 
               set valType integer
           } else {
	       set min [lindex $minVal $index]
	       set max [lindex $maxVal $index]
	   }
	   set txt [lindex $helps  $index]
           $cv.devSetupEnc configure -id $item
           $cv.devSetupEnc configure -valType $valType
           $cv.devSetupEnc configure -minVal $min
           $cv.devSetupEnc configure -maxVal $max
           $cv.devSetupEnc enable
       } else {
           $cv.devSetupEnc disable
       }
       $cv.devSetupEnc configHelp $txt
   }
   
   # set value in encoder entry widget
   protected method setEnc { value } {
       $cv.devSetupEnc configure -default $value
       $cv.devSetupEnc setValue $value
   }

   # get value in encoder entry widget
   protected method getEnc { {check 0 } } {
       if {$check} { 
           set value [ $cv.devSetupEnc getSelected]
       } else { 
           upvar #0 [ $cv.devSetupEnc cget -variable] value
       }
       return $value
   }

   # configure order entry widget
   protected method configOrd { item } {

       set min ""
       set max ""
       set txt ""
       set index [lsearch -exact $elements $item]
       if {$index >= 0} {; # valid option
           if {[cequal $item $encoder] || [cequal $item $encrel]} {
               $cv.devSetupOrd disable
           } else {
	       set min $minOrd
	       set max [lindex $maxOrd $index]
	       set txt [lindex $helpsOrd  $index]
               $cv.devSetupOrd configure -id $item
               $cv.devSetupOrd configure -valType integer
               $cv.devSetupOrd configure -minVal $min
               $cv.devSetupOrd configure -maxVal $max
               $cv.devSetupOrd enable
	   }
       } else {
           $cv.devSetupOrd disable
       }
       $cv.devSetupOrd configHelp $txt
   }
   
   # set value in order entry widget
   protected method setOrd { value } {
       $cv.devSetupOrd configure -default $value
       $cv.devSetupOrd setValue $value
   }

   # get value in order entry widget
   protected method getOrd { } {
       upvar #0 [ $cv.devSetupOrd cget -variable] wdgVar
       return $wdgVar
   }

   public variable devType ""
   public variable cfgObj  ""

   private variable prefix ""
   private variable encoder "enc"
   private variable encrel  "encrel"
   private variable elements {}
   private variable helps {}
   private variable minVal {}
   private variable maxVal {}
   private variable maxOrd {}
   private variable minOrd 1
   private variable helpsOrd {}
   private variable last   ""
   private variable wltxt  ", wl"
   private variable units  ""

}


#
# ___oOo___
