#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanICSControl.tcl,v 1.47 2005/07/01 13:25:38 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  19/08/03  VLTSW20030305
# dimarcan  16/11/00  created
#

#************************************************************************
#   NAME
#    icbpanState, icbpanStartup, 
#    icbpanShutdown - Tcl GUI procedures operating on the whole ICS
#    
# 
#   SYNOPSIS
#    icbpanState <command>
#    icbpanStartup
#    icbpanShutdown
#
#   DESCRIPTION
#    Tcl procedures, used within ICS panels to handle ICS global operations.
#
#    icbpanState executes the state transition <command> for ICS
#                <command> can be OFF STANDBY or ONLINE
#    icbpanStartup executes a start-up of the ICS WS processes
#    icbpanShutdown executes a shut-down of the ICS WS processes
#
#   FILES
#
#   ENVIRONMENT
#
#   CAUTIONS
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc icbpanState { cmd } {
    global ::gvar ::env

    set procName [icbpanProcname]
    set timeout [icbpanTimeoutMove]
    icbpanCmd $env(RTAPENV) $procName $cmd "" $timeout 2
}


proc icbpanStartup { } {
    global ::gvar ::env

    set cfgObj [icbpanCfgSet]
    set icsPrefix [icbpanIcsPrefix $cfgObj]
    if {[cequal $icsPrefix ""]} { return }

    set icsId [icbpanIcsId $cfgObj]
    exec ${icsPrefix}insStart -log -proc $icsId
}


proc icbpanShutdown { } {
    global ::gvar ::env

    set cfgObj [icbpanCfgSet]
    set icsPrefix [icbpanIcsPrefix $cfgObj]
    if {[cequal $icsPrefix ""]} { return }

    set icsId [icbpanIcsId $cfgObj]
    exec ${icsPrefix}insStop -log -proc $icsId
}


proc icbpanRestartGui { } {
    global ::gvar ::env

    set cfgObj [icbpanCfgSet]
    set icsPrefix [icbpanIcsPrefix $cfgObj]
    if {[cequal $icsPrefix ""]} { return }

    set icsId [icbpanIcsId $cfgObj]
    exec ${icsPrefix}insStart -log -restart -panel $icsId
}


proc icbpanICSSubstate {name element op} {

    icbpanICSIsIdle ${name}($element)
}


proc icbpanICSIsIdle {name} {
    global ::gvar

    set substate [set $name]
    if {$substate == 1} {
        set gvar(idle) 1
    } else {
        set gvar(idle) 0
    }
}


# This class replaces icbpanICSControl (VLTSW20030305)
class icbpanICSState {
   inherit icbpanICSState_uifClass

   constructor {args} {
      global ::shArea
      eval itk_initialize -helparea $shArea -relief flat $args
      display
      after idle [code $this initFunction]
   }


   method initFunction { } {
      global ::gvar

      # trace substate variable
      set substateVar [$cv.icbpanSubstate cget -variable]
      icbpanICSIsIdle $substateVar
      trace variable $substateVar rw icbpanICSSubstate

      # start monitoring LCU status
      lcuStatus [icbpanLcus $cfgObj]
   }

   method lcuStatus { lcuList } {
       global ::gvar
       global seq_errLogging
       global icbpan

       set status OK
       # if op. mode is WS simulation, do nothing
       upvar #0 [ $cv.icbpanSimulation cget -variable] simulation
       if { $simulation == 1 } {
           set status "IGNORED"
       } else {
           set errMode $seq_errLogging
           set seq_errLogging off; # avoid to log error on OLDB read failure
           set lcuAv 0
           set lcuNum [llength $lcuList]
           for { set i 0 } { $i < $lcuNum } { incr i } {
               set lcu [lindex $lcuList $i]
	       set lcuenv [lindex $lcu 0]
	       set lcuindex [lindex $lcu 1]
	       set available [lindex $lcu 2]
               if { $available == "F" } { continue }
               incr lcuAv
               set failure "$lcuenv NOT OK"
               # first check if LCU OLDB point is accessible (lcc3 > 4.68)
               if {![info exist icbpan(lccBootStatus)]} {
                   if {[catch {seq_dbGetFamilyNames "@${lcuenv}:LCC" absolute} attrList]} {
                       seq_errResetStack
                       set status $failure
                       break
                   } elseif {[lsearch -exact $attrList ":LCC:BOOTSTATUS"] == -1} {
                       set icbpan(lccBootStatus) 0; # lcc < 4.69
		   } else {
                       set icbpan(lccBootStatus) 1; # lcc >= 4.69
                   }
	       }
               if {[info exist icbpan(lccBootStatus)]} {
                   if {$icbpan(lccBootStatus) == 0} {; # no OLDB boot status info available
                       set status ""
                       break
		   } else {
                       # check here if the ICB LCU OLDB is accessible
                       set oldbAttr "@${lcuenv}:LCC:BOOTSTATUS.status"
                       if {[catch {seq_dbReadSymbolic $oldbAttr -noBadQLog } lcuStatus]} {
                           set status $failure
                           break
                       } elseif { $lcuStatus != 2 } {
                           set status $failure
                           break
		       }
		   }
               }
	   }
           # If no LCU is available, show same label as for WS simulation (VLTSW20040054)
           if { $lcuAv == 0 } { set status "IGNORED" }
           set seq_errLogging $errMode
       }

       # update widget
       set [$cv.icbpanLcuStatus cget -variable] $status

       # reshuffle the LCUs list, such that next time other LCUs are checked first (VLTSW20040054)
       if { [string match "* NOT OK" $status] } {
           set lcuList [lreplace $lcuList $i $i]
           lappend lcuList $lcu
       }

       # call next iteration after 5 sec
       after 5000 [code $this lcuStatus $lcuList]
   }

   public variable  procName ""
   public variable  cfgObj   ""

}


# This class is obsolete and kept for backward compatibility only (VLTSW20030305)
class icbpanICSControl {
   inherit icbpanICSControl_uifClass

   constructor {args} {
      global ::shArea
      eval itk_initialize -helparea $shArea -relief flat $args
      display

      after idle [code $this initFunction]
   }


   method initFunction { } {
      global ::gvar

      # configure State button
      upvar #0 [ $cv.icbpanState cget -variable] wdgVar
      set stateIndex [expr {$wdgVar - 2}]
      set states [$cv.icbpanChgState cget -itemList]
      set value [lindex $states $stateIndex]
      $cv.icbpanChgState configure -default $value -command icbpanState

      # configure Startup button
      $cv.icbpanStartup configure -command icbpanStartup
      # configure Shutdown button
      $cv.icbpanShutdown configure -command icbpanShutdown

      # trace substate variable
      set substateVar [$cv.icbpanSubstate cget -variable]
      icbpanICSIsIdle $substateVar
      trace variable $substateVar rw icbpanICSSubstate

   }

   public variable  procName ""
   public variable  cfgObj   ""

}


#
# ___oOo___
