#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanICSDevControl.tcl,v 1.47 2005/07/01 13:25:38 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  19/08/03  VLTSW20030305
# dimarcan  16/11/00  created
#

#************************************************************************
#   NAME
#    icbpanDeviceList, icbpanSetupDevList, icbpanSensorDevList,
#    icbpanSetup, icbpanStop, icbpanDevState, 
#    icbpanSimulation - Tcl GUI procedures handling a set of ICS devices
# 
#   SYNOPSIS
#    icbpanDeviceList
#    icbpanSetupDevList
#    icbpanSensorDevList
#    icbpanSetup
#    icbpanStop
#    icbpanDevState <command>
#    icbpanSimulation <YES/NO>
 
#   DESCRIPTION
#    Tcl procedures, used within ICS panels to handle ICS devices.
#
#    icbpanDeviceList    returns all devices in the panel
#    icbpanSetupDevList  returns all devices in the panel supporting SETUP
#    icbpanSensorDevList returns all sensors devices in the panel
#    icbpanSetup    executes a SETUP command for all selected devices
#    icbpanStop     executes a STOP command for all selected devices
#    icbpanDevState executes the state transition <command> for all selected devices
#                   <command> can be OFF STANDBY or ONLINE
#    icbpanSimulation <YES/NO> executes a SIMULAT/STOPSIM command for all selected devices
#
#   FILES
#
#   ENVIRONMENT
#
#   CAUTIONS
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc icbpanDeviceList { } {

    set devices [icbpanDevices]
    set idx [llength $devices]
    if { $idx == 0 } {
        icbpanDialog "Empty list of devices!"
        return ""
    }
    set setupList {}
    set sensorList {}
    loop i 0 $idx {
        set obj [lindex $devices $i] 
        if {[catch { $obj info variable isSensor } ret ]} {
            lappend setupList $obj 
         } else {
            lappend sensorList $obj 
        }
    }
    return [list $setupList $sensorList]
}


proc icbpanSetupDevList { } {

    set deviceList [icbpanDeviceList]
    return [lindex $deviceList 0]
}


proc icbpanSensorDevList { } {

    set deviceList [icbpanDeviceList]
    return [lindex $deviceList 1]
}


proc icbpanSetup { } {
    global ::env

    set setupBuf {}
    set deviceList [icbpanSetupDevList]
    set objNo [llength $deviceList]
    set cfgObj [icbpanCfgSet]
    set procName [icbpanProcname]
    set timeout [icbpanTimeoutMove]

    loop i 0 $objNo {

        set currObj [lindex $deviceList $i]
        set currSel [$currObj isSelected]

        if {$currSel} {
            set currSetup [$currObj getKeyw]
            # check if any empty or wrong entry fields
            if {[ icbpanCheckEntry $currObj]} {
                set devName [icbpanGetDeviceName $cfgObj [$currObj cget -devType]]
                icbpanDialog "Device $devName Wrong value or empty field"
                return
	    }
            if {![cequal $currSetup ""]} {lappend setupBuf $currSetup}
        }
    }

    set setupBuf [join $setupBuf]

    if {![cequal $setupBuf ""]} {
        icbpanCmd $env(RTAPENV) $procName SETUP "-function $setupBuf" $timeout 2
    } else {
        icbpanDialog "Setup buffer is empty ! Check selected devices"
    }
}
  

proc icbpanStop {} {
    global ::gvar ::env

    set gvar(stop) 0
    set devList {}

    set deviceList [icbpanSetupDevList]
    set objNo [llength $deviceList]
    set cfgObj [icbpanCfgSet]
    set procName [icbpanProcname]
    set timeout [icbpanTimeoutMove]

    loop i 0 $objNo {

        set currObj [lindex $deviceList $i]
        set currSel [$currObj isSelected]

        if {$currSel} {
            set devName [icbpanGetDeviceName $cfgObj [$currObj cget -devType]]
            if {![cequal $devName ""]} {
                lappend sendList $devName
            }
        }
    }

    set devList [join $devList]

    if {![cequal $devList ""]} {
        icbpanCmd $env(RTAPENV) $procName STOP "-function $devList" $timeout 2
    } else {
        icbpanDialog "No device selected !"
    }
    set gvar(stop) 1
}


proc icbpanDevState { cmd } {
    global ::gvar ::env

    set sendList {}

    set deviceList [join [icbpanDeviceList]]
    set objNo [llength $deviceList]
    set cfgObj [icbpanCfgSet]
    set procName [icbpanProcname]
    set timeout [icbpanTimeoutMove]

    loop i 0 $objNo {

        set currObj [lindex $deviceList $i]
        set currSel [$currObj isSelected]

        if {$currSel} {
            set devName [icbpanGetDeviceName $cfgObj [$currObj cget -devType]]
            if {![cequal $devName ""]} {
                lappend sendList $devName
            }
        }
    }

    set sendList [join $sendList]

    if {![cequal $sendList ""]} {
        icbpanCmd $env(RTAPENV) $procName $cmd "-function $sendList" $timeout 2
    } else {
        icbpanDialog "No device selected !"
    }
}


proc icbpanSimulation { mode } {
    global ::gvar ::env

    set sendList {}

    set deviceList [join [icbpanDeviceList]]
    set objNo [llength $deviceList]
    set cfgObj [icbpanCfgSet]
    set procName [icbpanProcname]

    loop i 0 $objNo {

        set currObj [lindex $deviceList $i]
        set currSel [$currObj isSelected]

        if {$currSel} {
            set devName [icbpanGetDeviceName $cfgObj [$currObj cget -devType]]
            if {![cequal $devName ""]} {
                lappend sendList $devName
            }
        }
    }

    set sendList [join $sendList]

    if {[cequal $sendList ""]} {
        icbpanDialog "No device selected !"
        return
    }

    set text "Changing simulation mode may require new initialization. OK?"
    set val [tk_dialog .getValue "ICS Dialog" $text question 1 Yes No]
    if { $val != 0 } { return 0 }

    if {"$mode" == "NO"} { set cmd STOPSIM
    } else { set cmd SIMULAT }
    icbpanCmd $env(RTAPENV) $procName $cmd "-function $sendList" 100000 2
}


# This class is obsolete and kept for backward compatibility only (VLTSW20030305)
class icbpanICSDevControl {
   inherit icbpanICSDevCtrl_uifClass

   constructor {args} {
      global ::shArea

      eval itk_initialize -helparea $shArea -relief flat $args
      display
      after idle [code $this initFunction]
   }

   method initFunction { } {

      # configure Setup button
      $cv.icbpanMove configure -command icbpanSetup

      # configure Stop button
      $cv.icbpanStop configure -foreground red -command icbpanStop

      # configure State button
      $cv.icbpanChgDevState configure -command icbpanDevState

      # configure Op. mode button
      $cv.icbpanChgDevOpMode configure -command icbpanSimulation

   }

   public variable procName   ""
   public variable cfgObj     ""
   public variable allDevices  {}
}


#
# ___oOo___
