#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanLib.tcl,v 1.47 2005/07/01 13:25:38 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dimarcan  15/11/00  created
#

#************************************************************************
#   NAME
#    icbpanCmd, icbpanCmdQueryCommands, icbpanCmdQueryParameters,
#    icbpanExecCmd, icbpanWaitPid, icbpanDbRead, icbpanSelectAll,
#    icbpanDeselectAll, icbpanEditCfgFile, icbpanSetLCUCfg,
#    icbpanSaveAsSetup, icbpanSaveSetup, icbpanLoadSetup,
#    icbpanLcuBoot, icbpanGetDevice, icbpanGetDevicePrefix,
#    icbpanGetDeviceName, icbpanGetDeviceLcuId, icbpanGetDeviceLcuName,
#    icbpanCfgCat, icbpanInsCat, icbpanIcsPrefix, icbpanIcsId,
#    icbpanLcus - tcl procedures used within ICB control panel.
# 
#   SYNOPSIS
#    icbpanCmd       <envName><procName> <cmd> [parms] [timeout] [disp]
#    icbpanCmdQueryCommands   <Process>
#    icbpanCmdQueryParameters <Process> <Command>
#    icbpanExecCmd            <args> [bg] [wait] [xterm]
#    icbpanWaitPid            <pid>
#    icbpanDbRead             <attr>
#    icbpanSelectAll          <listAllDevices>
#    icbpanDeselectAll        <listAllDevices>
#    icbpanEditCfgFile        <file>
#    icbpanSetLCUCfg          <INS>
#    icbpanSaveAsSetup        <listAllDevices> <dic>
#    icbpanSaveSetup          <allDevices> <file> <dic>
#    icbpanLoadSetup          <allDevices> <file> <dic>
#    icbpanLcuBoot            <lcuenv>
#    icbpanGetDevice          <listAllDevices> <devName>
#    icbpanGetDevicePrefix    <cfg object> <devType>
#    icbpanGetDeviceName      <cfg object> <dev Type>
#    icbpanGetDeviceLcuId     <cfg object> <dev Type>
#    icbpanGetDeviceLcuName   <cfg object> <dev Type>
#    icbpanCfgCat             <cfg object>
#    icbpanInsCat             <cfg object>
#    icbpanIcsPrefix          <cfg object>
#    icbpanIcsId              <cfg object>
#    icbpanLcus               <cfg object>
# 
#   DESCRIPTION
#    icbpanCmd
#       sends a command to ICS front-end process and waits for last reply
#    icbpanCmdQueryCommands
#       query cmdManager for cmds accepted by Process
#    icbpanCmdQueryParameters
#       query cmdManager for parameters accepted by Command of Process
#    icbpanExecCmd
#       execute args; wait=0 return immediately; wait=1 waits completion
#       of invoked process; xterm=1 start application in a new xterm;
#       bg = "" doesn't start in background
#    icbpanWaitPid
#       wait until process terminate
#    icbpanDbRead
#       is the specific implementation of seq_dbReadSymbolic
#    icbpanSelectAll, icbpanDeselectAll
#       set/unset checkbutton for all devices in "listAllDevices"
#    icbpanEditCfgFile
#       invoke emacs on the cfg file
#    icbpanSetLCUCfg 
#       invoke icbConfigSet on instrument INS
#    icbpanSaveAsSetup, icbpanSaveSetup
#       saves parameters of the selected devices to a file
#    icbpanLoadSetup
#       loads the content of a setup file and sets panel widgets 
#       accordingly
#    icbpanLcuBoot
#       reboot LCU lcuenv
#    icbpanGetDevice
#       return the object corresponding to the device <devName>.
#    icbpanGetDevicePrefix
#       return the prefix associated to the device type <devType>.
#    icbpanGetDeviceName
#       return the name associated to the device <devType>
#    icbpanGetDeviceLcuId
#       return the LCU index associated to the device <devType>
#    icbpanGetDeviceLcuName
#       return the LCU name associated to the device <devType>
#    icbpanCfgCat
#       return the configuration category associated to <cfg object>
#    icbpanInsCat
#       return the instrument category associated to <cfg object>
#    icbpanIcsPrefix
#       return the ICS prefix associated to <cfg object>
#    icbpanIcsId
#       return the ICS id used by stoo and associated to <cfg object>
#    icbpanLcus
#       return the ICS LCUs associated to <cfg object>
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# send a command to a process; returns 1 in case of error
# disp=0 -> no display
# disp=1 -> display reply on panel reply window
# disp=2 -> display reply on panel reply window + raise seq_errDisplay

proc icbpanCmd {envName procName cmd {parms ""} {timeout 10000} {disp 0}} {

   seq_errResetStack

   if {[llength $parms] > 1} { set parms "\"$parms\""}

   catch {seq_msgFlush all -nowait}

   if {$disp} {panDisplayCommand "$cmd $parms"}
   if {[catch {
      eval seq_msgSendCommand $envName $procName $cmd $parms} cmdId] } {
      if {$disp == 2} { seq_errDisplay } 
      return 1
  }

  set errNr 0
  set err [catch { seq_msgRecvReply $cmdId errNr lr -last -$timeout } ret]
  if {$disp} {panDisplayReply $cmd $cmdId $procName $errNr 1 0 $ret}

  if {$err || $errNr} {
      if {$disp == 2} { seq_errDisplay }
      return 1
  }
  return $ret

}


# return list of commands for a process
proc icbpanCmdQueryCommands { Process } {
    global gvar env

    set reply [icbpanCmd $env(RTAPENV) cmdManager CMDLIST $Process ]
    if {$reply == 1} { return $reply }

    set comlist [split $reply ","]
    set comlist [lrange $comlist 1 end]
    return $comlist
}


# return list of parameters for a command
proc icbpanCmdQueryParameters { Process Command } {
    global gvar env

    set par "$Process,$Command"
    set reply [icbpanCmd $env(RTAPENV) cmdManager CMDPAR $par ]
    if {$reply == 1} { return $reply }

    set parlist [split $reply ","]
    set parlist [lrange $parlist 1 end]
    return $parlist
}


# execute script
# bg = "" doesn't start in background
# wait=0 return immediately
# wait=1 wait completion of invoked process
# xterm=1 start applictation in a new xterm

proc icbpanExecCmd {arguments {bg &} {wait 0} {xterm 0} } {

   panDisplayComment "Executing script: $arguments"
   update idletasks

   if {$xterm} {
      set tmpFile "/tmp/icbpan.errXTerm"
      catch { eval exec rm -f $tmpFile }
      if {[catch {eval exec $arguments > $tmpFile} pid]} {
         tk_dialog .getValue "ICS Dialog" $pid info 0 OK
         return 1
      }
      if {[file exists $tmpFile]} {
         if { [file size $tmpFile] != 0 } {
            panExecTerm cat $tmpFile
         }
      }
      catch { eval exec rm -f $tmpFile }
      panDisplayComment "Script $arguments - done."
      return 0
   }

   set pid 0

   if {[catch {eval exec $arguments >& /dev/null $bg} pid]} {
      tk_dialog .getValue "ICS Dialog" $pid info 0 OK
      return 1
   }
   if {$wait} { icbpanWaitPid $pid }
   panDisplayComment "Script $arguments - done."
   return 0

}


# wait until process exists

proc icbpanWaitPid {pid} {
    while {! [catch {wait -nohang $pid}]} {
        update
        after 500
    }
}


# read OLDB attribute value

proc icbpanDbRead {attr} {
    if {[catch {seq_dbReadSymbolic $attr -noBadQLog } msg]} {
        tk_dialog .getValue "ICS Dialog" $msg info 0 OK
        return 0
    }
    return $msg
}


proc icbpanSelectAll { listAllDevices } {

   set objNo [llength $listAllDevices]
   loop i 0 $objNo {
      set currObj [lindex $listAllDevices $i]
      $currObj setSelected 1
   }
}


proc icbpanDeselectAll { listAllDevices } {

   set devList $listAllDevices

   set objNo [llength $listAllDevices]
   loop i 0 $objNo {
      set currObj [lindex $listAllDevices $i]
      $currObj setSelected 0
   }
}


proc icbpanEditCfgFile { file } {
   global ::env ::gvar

   set cfgFile "$env(INS_ROOT)/$env(INS_USER)/COMMON/CONFIGFILES/${file}"

   set editor emacs
   if {[info exists env(EDITOR)]} {
      set editor $env(EDITOR)
   }
   icbpanExecCmd "$editor $cfgFile" & 0 0

}


proc icbpanSetLCUCfg { INS } {

   set lcuConfig "icbConfigSet $INS"
   set ret [icbpanExecCmd $lcuConfig "" 0 1]
   return $ret
}


proc icbpanGetSetupFileDir { listAllDevices } {

   set setupFileDir "INS"

   set cfgObj [icbpanCfgSet]
   set objNo [llength $listAllDevices]
   loop i 0 $objNo {
      set currObj [lindex $listAllDevices $i]
      if {[catch { $currObj info variable isSensor } ]} {
         set prefix [icbpanGetDevicePrefix $cfgObj [$currObj cget -devType]]
         if {![string match "INS.*" $prefix]} {
               set setupFileDir "REF"
         }
      }
   }
   return $setupFileDir
}


proc icbpanSaveAsSetup { listAllDevices dic } {
   global ::env

   set type [icbpanGetSetupFileDir $listAllDevices]
   set dir $env(INS_ROOT)/$env(INS_USER)/COMMON/SETUPFILES/$type


   puts "panSelectFile $dir *"
   if {[set file [panSelectFile $dir "*" ]] != ""} {
      icbpanSaveSetup $listAllDevices $file $dic
   }

}


proc icbpanSaveSetup { allDevices file dic } {

   # sensors obj are skipped
   set listAllDevices {}
   set idx [llength $allDevices]
   if { $idx == 0 } {
      icbpanDialog "Empty list of devices!"
      return ""
   }
   loop i 0 $idx {
      set obj [lindex $allDevices $i]
      if {[catch { $obj info variable isSensor } ]} {
         lappend listAllDevices $obj
      }
   }

   set setupBuf {}

   set cfgObj [icbpanCfgSet]
   set objNo [llength $listAllDevices]

   set type [icbpanGetSetupFileDir $listAllDevices]
   if { ![string compare $type INS] } {
      set setupFileType "Instrument Setup"
   } else {
      set setupFileType "Reference Setup"
   }
   loop i 0 $objNo {

      set currObj [lindex $listAllDevices $i]
      set currSel [$currObj isSelected]

      if {$currSel} {
         set currSetup [$currObj getKeyw]
         # check if any empty or wrong entry fields
         if {[ icbpanCheckEntry $currObj]} {
             set devName [icbpanGetDeviceName $cfgObj [$currObj cget -devType]]
             icbpanDialog "Device $devName Wrong value or empty field"
             return
	 }
	 if {![cequal $currSetup ""]} { lappend setupBuf $currSetup }
      }
   }
   set setupBuf [join $setupBuf]

   if {![cequal $setupBuf ""]} {
      if {[catch {seq_oslxCmd OSETUP -loadDic icbOslx $dic} ret]} {
        icbpanDialog $ret
        return 1
      }
      if {[catch {seq_oslxCmd OSETUP -storeKeyword icbOslx $setupBuf} ret]} {
        icbpanDialog $ret
        return 1
      }
      if {[catch {seq_oslxCmd OSETUP -saveAs icbOslx $file \"$setupFileType\"} ret]} {
        icbpanDialog $ret
        return 1
      }
      if {[catch {seq_oslxCmd OSETUP -clear icbOslx} ret]} {
        icbpanDialog $ret
        return 1
      }
   } else {
       icbpanDialog "No device selected !"
   }
}


proc icbpanLoadSetup { allDevices file dic } {
   global ::env

   # sensors obj are skipped
   set listAllDevices {}
   set idx [llength $allDevices]
   if { $idx == 0 } {
      icbpanDialog "Empty list of devices!"
      return ""
   }
   loop i 0 $idx {
      set obj [lindex $allDevices $i]
      if {[catch { $obj info variable isSensor } ]} {
         lappend listAllDevices $obj
      }
   }

   set setupBuf {}
   set type [icbpanGetSetupFileDir $listAllDevices]
   set setupDir $env(INS_ROOT)/$env(INS_USER)/COMMON/SETUPFILES/$type
   set type [string tolower $type]

   if { [cequal $file ""] } {
      if {[set setupFile [panSelectFile $setupDir "*.$type" ]] == ""} {
         return 0
      }
   } else {
      set setupFile "${setupDir}/${file}.$type"

      # if setup file doesn't exist exit
      if {![file exists $setupFile]} { return 0 }
   }

   if {[catch {seq_oslxCmd OSETUP -loadDic icbBuf $dic} ret]} {
      icbpanDialog $ret
      return 1
   }
   if {[catch {seq_oslxCmd OSETUP -load icbBuf $setupFile} ret]} {
      icbpanDialog $ret
      return 1
   }

   icbpanDeselectAll $allDevices

   if {[catch {seq_oslxCmd OSETUP -noSetupKeywords icbBuf} num]} {
      icbpanDialog $num
      return 1
   }
   if {[catch {seq_oslxCmd OSETUP -resetKeywordPtr icbBuf} ret]} {
      icbpanDialog $ret
      return 1
   }
   loop i 0 $num {
      if {[catch {seq_oslxCmd OSETUP -getNextKeyword icbBuf} pair]} {
         icbpanDialog $pair
         return 1
      }
      if {[catch {regsub -all "\"" $pair "\ " keyw} ret]} {
         icbpanDialog $ret
         return 1
      }
      set objNo [llength $listAllDevices]
      loop i 0 $objNo {
         set currObj [lindex $listAllDevices $i]
         $currObj setKeyw $keyw
      }
   }
   if {[catch {seq_oslxCmd OSETUP -clear icbBuf} ret]} {
      icbpanDialog $ret
      return 1
   }

}


proc icbpanLcuBoot { lcuenv } {
   global env

   set args "lccBoot $lcuenv"

   panDisplayComment "Executing script: $args"
   icbpanExecCmd $args & 1
   panDisplayComment "Script $args - done."

}


proc icbpanDbPoll { listAllDevices pollRate } {

      icbpanUpdateOutput $listAllDevices 1
      after $pollRate icbpanDbPoll [list $listAllDevices] $pollRate
}


proc icbpanUpdateScan { } {
   # refresh scan for all LCUs
   set lcuList [icbpanLcus [icbpanCfgSet]]
   foreach lcu $lcuList {
       set lcuName [lindex $lcu 0]
       icbpanExecCmd "scanDevOff $lcuName" "" 1
       icbpanExecCmd "scanDevOn $lcuName"  "" 1
   }
   # refresh output (need delay)
   after 2000
   icbpanUpdateOutput [icbpanDevices]
}


proc icbpanUpdateOutput { listAllDevices {onlyPoll 0} } {
   global panWidget i_type

   set objNo [llength $listAllDevices]
   loop i 0 $objNo {
      set obj [lindex $listAllDevices $i]
      set devWdg [$obj getClassPath]
      set devDbList [$obj getPollList]
      if { ! $onlyPoll } {
         set devDbList [concat $devDbList [$obj getEventList]]
      }
      set dbNo [llength $devDbList]
      loop j 0 $dbNo {
          set name [lindex $devDbList $j]
          set type [lindex $panWidget($name) $i_type ]
          set newValue [panPollDbRead $name $type]
          if {! [cequal $newValue panFAILURE]} {
             panUpdateLocalVar $name $type $newValue
          }
      }
      # For sensors, the vector "last" is handled by the icbpan uifClass
      if { ! $onlyPoll } {
         if { ! [catch { $obj info variable isSensor } ret ]} {
             $obj dbUpdate
         }
      }
  }
  update idletasks
  return
}


proc icbpanEvtDbRestart { listAllDevices } {

   set objNo [llength $listAllDevices]
   loop i 0 $objNo {
      set obj [lindex $listAllDevices $i]
      set devWdg [$obj getClassPath]
      set devEvtList [$obj getEventList]
      set evtNo [llength $devEvtList]
      loop j 0 $evtNo {
          set name [lindex $devEvtList $j]
          set item [string range $devWdg 2 end]; # remove trailing ::
          set last [string last "." $name]
          set item ${item}[string range $name $last end]
          panEvtDbDetach $name
          panEvtDbAttach $item $name
      }
      # For sensors, the vector "last" is handled by the icbpan uifClass
      if { ! [catch { $obj info variable isSensor } ret ]} {
          $obj evtDbRestart
      }
  }
}


proc icbpanGetDevice { listAllDevices devName } {
    set objNo [llength $listAllDevices]
    set obj ""
    loop i 0 $objNo {
       set currObj [lindex $listAllDevices $i]
       set currDevName [$currObj cget -devType]
       if {[cequal $currDevName $devName]} {
          set obj $currObj
          break
       }
   }
   return $obj
}



proc icbpanGetDeviceNum { cfgObj } {
    set cfgcat [icbpanCfgCat $cfgObj]
    if {[ catch { $cfgObj Get $cfgcat.CON.DEVNUM } devNo ]} {
        set devNo 1
    }
    return $devNo
}


proc icbpanIsAssembly { devType } {
    if { [string match *.ASSEMBLY* $devType] } { set isAssembly 1
    } else { set isAssembly 0 }
    return $isAssembly
}


proc icbpanAssemblies { cfgObj } {
    global icbpanAssemblyNo icbpanAssemblyId icbpanAssemblyName 
    global icbpanAssemblyOldbIndex icbpanAssemblyKeys icbpanAssemblyValues

    if { ![info exists icbpanAssemblyNo] } {
        set icbpanAssemblyNo 0
        set icbpanAssemblyId {}
        set icbpanAssemblyName {}
        set icbpanAssemblyOldbIndex {}
        set icbpanAssemblyKeys {}
        set icbpanAssemblyValues {}
        set cfgcat [icbpanCfgCat $cfgObj]
        loop i 1 100 {
            set devType $cfgcat.ASSEMBLY${i}
	    if {[ catch { $cfgObj Get $devType } devName ]} { break }
	    set icbpanAssemblyNo $i
            lappend icbpanAssemblyId $devType
            lappend icbpanAssemblyName $devName
            set keys {}
            loop j 1 100 {
	        if {[ catch { $cfgObj Get $devType.FITSKEY${j} } keyname ]} { break }
                lappend keys $keyname
	    }
            lappend icbpanAssemblyKeys $keys
            set values {}
            loop j 1 100 {
	        if {[ catch { $cfgObj Get $devType.KEY${j} } value ]} { break }
                if { $value != "" } { lappend values $value }
	    }
            lappend icbpanAssemblyValues $values
            if { $i == 1 } { 
                lappend icbpanAssemblyOldbIndex 1
            } else {
                set previous [expr {$i - 2}]
                set index [lindex $icbpanAssemblyOldbIndex $previous]
                set keys  [lindex $icbpanAssemblyKeys $previous]
                set keynum [llength $keys]
                if { $keynum == 0 } { set keynum 1 }
                lappend icbpanAssemblyOldbIndex [expr {$index + $keynum}]
	    }        
        }
    }
    return $icbpanAssemblyNo
}


proc icbpanGetAssemblyNum { cfgObj } {

    return [icbpanAssemblies $cfgObj]
}


proc icbpanGetAssemblyName { cfgObj devType} {
    if {[ catch { $cfgObj Get ${devType}.NAME } assName ]} { 
        set assName [$cfgObj Get $devType ]
    }
    return $assName
}


proc icbpanGetAssemblyOldbIndex { cfgObj devType } {
    global icbpanAssemblyId icbpanAssemblyOldbIndex

    icbpanAssemblies $cfgObj
    set index [lsearch -exact $icbpanAssemblyId $devType]
    return [lindex $icbpanAssemblyOldbIndex $index]
}


proc icbpanGetAssemblyKeys { cfgObj devType } {
    global icbpanAssemblyId icbpanAssemblyKeys

    icbpanAssemblies $cfgObj
    set index [lsearch -exact $icbpanAssemblyId $devType]
    return [lindex $icbpanAssemblyKeys $index]
}


proc icbpanGetAssemblyValues { cfgObj devType } {
    global icbpanAssemblyId icbpanAssemblyValues

    icbpanAssemblies $cfgObj
    set index [lsearch -exact $icbpanAssemblyId $devType]
    return [lindex $icbpanAssemblyValues $index]
}


proc icbpanGetDevicePrefix { cfgObj devType } {
    if { [icbpanIsAssembly $devType] } {
        set prefix [$cfgObj Get $devType]
    } else {
        if {[catch { $cfgObj Get ${devType}.PREFIX } prefix ]} {
            set devTypeL [split $devType .]
            set cfgcat [icbpanCfgCat $cfgObj]
            if {[catch { $cfgObj Get $cfgcat.CON.CATEGORY } category ]} {
               set category [lindex $devTypeL 0]
            }
            set devTypeL [lreplace $devTypeL 0 0]
            set devTypeS [join $devTypeL .]
            set prefix $category.$devTypeS
	}
    }
    return $prefix
}


proc icbpanGetDeviceName { cfgObj devType} {
    set devName $devType
    if {![cequal $devName ""]} {
        if { [icbpanIsAssembly $devType] } {
            set devName [$cfgObj Get $devType ]
	} else {
            set devName [$cfgObj Get ${devType}.DEVNAME]
	}
    }
    return $devName
}


proc icbpanGetDeviceLcuId { cfgObj devType} {
    if { [icbpanIsAssembly $devType] } { return "" }
    set cfgcat [icbpanCfgCat $cfgObj]
    if {[catch { $cfgObj Get ${devType}.LCUID } lcu ]} {
        set lcu 1
    }
    return $lcu
}


proc icbpanGetDeviceLcuName { cfgObj devType} {
    if { [icbpanIsAssembly $devType] } { return "" }
    set lcuId [icbpanGetDeviceLcuId $cfgObj $devType]
    set lcuList [icbpanLcus $cfgObj]
    set lcuName [lindex $lcuList [expr {$lcuId - 1}]]
    set lcuName [lindex $lcuName 0]
    return $lcuName
}


proc icbpanCfgCat { cfgObj  } {
    set cfgcat ""
    catch { $cfgObj Category } cfgcat
    if { ![string match INS* $cfgcat] } { set cfgcat "INS" }
    return $cfgcat
}

proc icbpanInsCat { cfgObj  } {
    set inscat [icbpanCfgCat $cfgObj]
    catch { $cfgObj Get $inscat.CON.CATEGORY } inscat
    return $inscat
}

proc icbpanIcsPrefix { cfgObj  } {

    set icsPrefix ""
    set cfgcat [icbpanCfgCat $cfgObj]
    if {[ catch { $cfgObj Get $cfgcat.CON.PREFIX } icsPrefix ]} {
         icbpanDialog $icsPrefix
    }
    return $icsPrefix
}

proc icbpanInsId { cfgObj  } {

    set insId ""
    set cfgcat [icbpanCfgCat $cfgObj]
    if {[ catch { $cfgObj Get $cfgcat.CON.ID } insId ]} {
         icbpanDialog $insId
    }
    return $insId
}

proc icbpanIcsId { cfgObj  } {

    set icsId "ICS"
    set cfgcat [icbpanCfgCat $cfgObj]
    if {[string compare $cfgcat "INS"]} {; # secondary ICS
        catch { $cfgObj Get $cfgcat.CON.ID } icsId
    }
    return $icsId
}

proc icbpanLcus { cfgObj } {

    set lcuList {}
    set cfgcat [icbpanCfgCat $cfgObj]
    if {[ catch { $cfgObj Get $cfgcat.CON.LCUNUM } no ]} { set no 1 }
    for { set i 1 } { $i <= $no } { incr i } {
        if {[ catch {$cfgObj Get $cfgcat.CON.LCUENV${i} } lcuenv ]} {
            icbpanDialog $lcuenv
            return ""
        }
        if {[ catch { $cfgObj Get $cfgcat.CON.LCUAV${i} } available ]} { set available T }
        lappend lcuList [list $lcuenv $i $available]
    }
    return $lcuList
}

proc icbpanDevices { {devices ""} } {
    global icbpanAllDevices

    if { (![cequal $devices ""]) || (![info exists icbpanAllDevices]) } {
        set icbpanAllDevices $devices
    }
    return $icbpanAllDevices
}

proc icbpanCfgSet { {cfgset ""} } {
    global icbpanConfigSet

    if {![cequal $cfgset ""]} {
        set icbpanConfigSet $cfgset
    }
    return $icbpanConfigSet
}

proc icbpanDbRoot { {point ""} } {
    global icbpanOldbRoot

    if {![cequal $point ""]} {
        set icbpanOldbRoot $point
    }
    return $icbpanOldbRoot
}

proc icbpanProcname { {name ""} } {
    global icbpanProc

    if {![cequal $name ""]} {
        set icbpanProc $name
    }
    return $icbpanProc
}

proc icbpanTimeoutMove { {cfgObj ""} } {
    global icbpanMoveTout

    if {![cequal $cfgObj ""]} {
        set timeout 120000
        set cfgcat [icbpanCfgCat $cfgObj]
        if {![ catch { $cfgObj Get $cfgcat.CON.MOVETOUT } moveTimeout ]} {
           if {$moveTimeout > $timeout} { 
               set timeout [expr {$moveTimeout + 20000}] 
           }
	}
        set icbpanMoveTout $timeout
    }
    return $icbpanMoveTout
}

proc icbpanDictionary { {dicObj ""} } {
    global icbpanDictObj

    if {![cequal $dicObj ""]} {
        set icbpanDictObj $dicObj
    }
    return $icbpanDictObj
}

proc icbpanDefaultUnits { key } {

    set dicObj [icbpanDictionary]
    if {![ catch { $dicObj GetEntry $key } keyl ]} {
        set val [lindex $keyl 3]
    } else { set val "" }

    return $val
}


proc icbpanDefaultType { key } {

    set dicObj [icbpanDictionary]
    if {![ catch { $dicObj GetEntry $key } keyl ]} {
        set val [lindex $keyl 1]
    } else { set val "" }

    return $val
}


# check if any empty or wrong entry fields
proc icbpanCheckEntry { currObj } {

    if {![ catch { $currObj entryWdg} entryWidgets]} {
        if {[llength $entryWidgets] > 0 } {
            foreach entryWdg $entryWidgets {
                set wdg cv.$entryWdg
                if {[ catch { $currObj.$wdg cget -state } state]} {
                    set wdg $entryWdg
                    set state [$currObj.$wdg cget -state]
		}
                if {![cequal $state disabled]} {
                    set value [$currObj.$wdg getSelected 1]
                    if {[cequal $value ""]} { return 1 }
		}
	    }
        }
    }
    return 0
}

# pop-up a dialog box for acknowledgment
proc icbpanDialog { txt } {
    tk_dialog .icbpanDialog "ICS Dialog" $txt info 0 OK
    update idletasks
    return
}

#
# ___oOo___
