#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanSensor.tcl,v 1.47 2005/07/01 13:25:40 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  11/08/04  VLTSW20030226 handle keywords INS.SENSORi.UIFROW UIFCOL UIFWID
# alongino  30/09/02  VLTSW20020413 VLTSW20020444
# dimarcan  16/11/00  created
#

#************************************************************************
#   NAME
#    icbpanSensor - wrapper class for icbpanSensor_uifClass
# 
#   SYNOPSIS
#    icbpanSensor .w \
#              -background Grey90 \
#              -relief flat       \
#              -cfgObj $cfgObj    \
#              -cwp "<alias>${devName}" \
#              -devType $devType]
#
#     where cfgObj is a ctooCONFIG object;
#           devType is extracted from INS.CON.DEVICE${i}
#           devName is extracted from ${devType}.DEVNAME
# 
#   DESCRIPTION
#    Itcl wrapper class to handle sensor devices. The corresponding
#    ibcpanSensor_uifClass contains a set of output widget (linked to OLDB)
#    which in turn display the device name, the current simulation level
#    (N=normal,S=simulate)  and the current device state. A checkbutton
#    is used to indicate if the corresponding device is selected.
#
#   PUBLIC METHODS
#    icbpanSensor  xxx
#       constructor. Read from ICS configuration file the number of
#       sensors, the corresponding names and descriptions. Dynamically
#       construct a set of output widgets linked to OLDB; every widget displays
#       the current value reported by the corresponding sensor. The
#       description is used as a short help text. 
#       Calls "initFunction" which initialize every output widget with
#       data read from OLDB. It also attach an OLDB event to display every
#       change in the sensor's reported value. This is done by means of
#       the public method "dbUpdate".
#    isSelected
#       get the current status of the checkbutton (0/1).
#    setSelected 0/1
#       set the checkbutton state to 0 or 1 (unselected or selected).
#    dbUpdate
#       update output widgets associated to OLDB vector "last"
#    evtDbRestart
#       detach and attach event on OLDB vector "last"
#
#   PUBLIC VARIABLES
#    cfgObj
#       ctooCONFIG object (configuration file interface)
#    devType
#       device type extracted from INS.CON.DEVICE${i}
#    isSensor
#       used (internally) during the construction
#       of the control panel to distinguish between a "normal" device
#       and a "sensor" device (see icbpanGUIcfg). 
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTIONS
#
#   SEE ALSO
#    ctooCONFIG(8), icbpanGUIcfg(8)
#
#   BUGS     
#
#------------------------------------------------------------------------
#


class icbpanSensor {
   inherit icbpanSensor_uifClass

    constructor {args} {
      global shArea

      set nCol 1; # default vertical
      set nRow -1
      # VLTSW20020413: extract values of devType and cfgObj from arguments
      #                without calling twice itk_initialize (done also at the end)
      #eval itk_initialize -helparea $shArea -relief flat $args
      set remainingArgs {}
      foreach {var val} $args {
          switch -exact -- $var {
              -devType { set devType $val }
              -cfgObj  { set cfgObj  $val }
              -nCol    { set nCol    $val }
              -nRow    { set nRow    $val }
              default  { lappend remainingArgs $var $val }
	  }
      }
      set labWidth 4
      if { ![ catch { $cfgObj Get ${devType}.NUM } sensNum ] } {
          incr sensNum
          loop i 1 $sensNum {
             set senName [$cfgObj Get ${devType}.NAME$i]
	     set nameLen [string length $senName]
	      if { $nameLen > $labWidth } { set labWidth $nameLen }
             lappend sensorsName $senName
             lappend sensorsDesc [$cfgObj Get ${devType}.DESC$i]
	     if { [ catch { $cfgObj Get ${devType}.FITS$i } fits ] } { 
                 set fits "unknown" 
                 set units "" 
             } else {
                 set units [icbpanDefaultUnits $fits]
	     }
	     catch { set units [$cfgObj Get ${devType}.SENUNIT$i] }
             lappend sensorsFits $fits
             lappend sensorsUnits $units
	     set sensValues {}
	     if { ![ catch { $cfgObj Get ${devType}.SIGWIDTH$i } sigwidth ] } {
                 set nval [expr {pow(2,$sigwidth)}]
                 set i2 [format "%02d" $i]
                 loop j 0 $nval {
	             if { [ catch { $cfgObj Get ${devType}.SIGVAL${i2}$j } val ] } {
                         set val ""
		     }
                     lappend sensValues $val
		 }
	     }
             lappend sensorsValues $sensValues
             set index [expr {$i - 1}]
	     set sensorsAlarm($index) 0            
          }
      } else {
          set sensNum 2
          lappend sensorsName [$cfgObj Get ${devType}.NAME]
          lappend sensorsDesc [$cfgObj Get ${devType}.DESC]
          lappend sensorFits  $devType
          lappend sensorUnits [icbpanDefaultUnits $devType]
      }
      catch { set nCol [$cfgObj Get ${devType}.UIFCOL] }
      catch { set nRow [$cfgObj Get ${devType}.UIFROW] }
      if { $nRow > 0 } { set nCol -1 }
      if { $nCol < 0 } { 
	 if { $nRow > 0 } {      
             set nCol [expr {($sensNum + $nRow - 2)/ $nRow}]
	 } else {
             set nCol [expr {round(sqrt($sensNum - 1))}] 
	 }
      }
      if { $nRow < 0 } { set nRow [expr {($sensNum + $nCol - 1)/ $nCol}] }

      set pos 0
      set hei 30
      set wid 30
      set iCol 0
      set widMax $wid
      set heiMax $hei
      set varWid 5
      incr labWidth
      catch { set varWid [$cfgObj Get ${devType}.UIFWID] }
      set wdgWid [expr {15 + 9 * ($labWidth + $varWid)}]
      loop j 1 $sensNum {
         itk_component add sens${j} {
            UifOutput $itk_interior.sens${j} -anchor w -canvasAnchor nw \
                   -color $colOKVal -font -Adobe-courier-medium-r-normal--14* \
                   -frame 1 -label [lindex $sensorsName $pos]:  \
                   -labelFont -Adobe-courier-medium-r-normal--14* \
                   -labelWidth $labWidth -variable ${devType}_sensCO${j} \
                   -width $varWid
         } { }

         set sensDesc [lindex $sensorsDesc $pos]
         set sensFits [lindex $sensorsFits $pos]
         set sensUnits [lindex $sensorsUnits $pos]
         set txt "$sensDesc     FITS keyword: $sensFits     Units: $sensUnits"
         catch {UifShortHelp::add $itk_option(-helparea) \
                                  $itk_component(sens${j}) \
                                  $txt }

         place $itk_component(sens${j}) -x $wid -y $hei -bordermode inside
         lappend sensorsClass $itk_interior.sens${j}
         incr pos
         incr iCol
	  if {$iCol < $nCol} { incr wid $wdgWid
         } else {
            set wid 30
            incr hei 22
            set iCol 0
	 }
         if {$wid > $widMax} { set widMax $wid }
         if {$hei > $heiMax} { set heiMax $hei }
      }

      incr widMax $wdgWid
      set devWdgWidth [$itk_interior.cv cget -width]
      if {$widMax < $devWdgWidth} { set widMax $devWdgWidth}
      incr heiMax 22
      $itk_interior.cv configure -width $widMax -height $heiMax

      eval itk_initialize -helparea $shArea -relief flat $remainingArgs
      display

      after idle [code $this initFunction]
   }


   destructor {
      evtDbStop
   }


   method initFunction { } {
      evtDbStart
   }  


   method evtDbStart {} {

       set dbPoint $itk_option(-cwp)
       set attrList [seq_dbGetAttrNames $dbPoint]
       loop j 1 $sensNum {
           set index [expr {$j - 1}]
           set dbAttr ${alarmPrefix}$index
	   if { [lsearch -exact $attrList $dbAttr] >= 0 } {; # alarm flag attribute found
               set sensorsAlarm($index) [icbpanDbRead "$dbPoint.$dbAttr"]
               seq_evtAttach "$dbPoint.$dbAttr" "!=" [code $this alarmTrigger $index %A %D]
           }
       }
       dbUpdate
       catch {seq_evtAttach "$dbPoint.last" "w" [code $this dbUpdate]} evt
   }


   method evtDbStop {} {
      catch { seq_evtDetach $evt }
   }


   public method dbUpdate { } {

      set dbPoint $itk_option(-cwp)
      set valList [icbpanDbRead "$dbPoint.last"]
      loop j 1 $sensNum {
	 set index [expr {$j - 1}]
         set value [lindex $valList $index]
	 set value [getValStr $index $value]
         upvar #0 [[lindex $sensorsClass $index] cget -variable] var
         set var $value
	 if {[isValid $index $value]} { set color $colOKVal 
         } else { set color $colBadVal }
         $itk_component(sens${j}) configure -color $color
      }
   }


   public method evtDbRestart {} {
      evtDbStop
      evtDbStart
   }


   # return status of the checkbutton
   public method isSelected { } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      return $sel
   }


   # set status of the checkbutton
   public method setSelected { val } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      set sel $val
   }


   # return value string for digital sensors
   protected method getValStr {index value} {
       set sensValues [lindex $sensorsValues $index]
       if { [regexp {^[0-9]+$} $value] } { 
          set strVal [lindex $sensValues $value]
       } else {; # not integer values, e.g. float
          set strVal ""
       }
       if { [string length $strVal] == 0 } { set strVal $value }
       if {[cequal $strVal $illegalVal]} {
           set strVal $illegalValStr
       }
       return $strVal
   }


   # trigger for alarm events
   protected method alarmTrigger {index dbAttr value} {\

       set sensorsAlarm($index) [lindex $value 2]
       panDisplayComment "$dbAttr changed: $sensorsAlarm($index)"
       dbUpdate
   }


   # check if the value is valid
   protected method isValid {index value} {
       if {$sensorsAlarm($index)} { set valid 0
       } else { set valid 1 }
       if {[cequal $value $illegalValStr]} {
           set valid 0
       }
       return $valid
   }


   public variable devType ""
   public variable cfgObj ""

   public variable isSensor 1

   protected variable sensorsClass {}
   protected variable sensorsName {}
   protected variable sensorsDesc {}
   protected variable sensorsFits {}
   protected variable sensorsUnits {}
   protected variable sensorsValues {}
   protected variable sensorsAlarm
   protected variable sensNum 0
   protected variable evt ""
   protected variable colBadVal red
   protected variable colOKVal Black

   private variable illegalVal 9999
   private variable illegalValStr "N/A"
   private variable alarmPrefix "alarm"
}


#
# ___oOo___
