#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanShutter.tcl,v 1.47 2005/07/01 13:25:38 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# oat  17/11/00  created
#

#************************************************************************
#   NAME
#    icbpanShutter - wrapper class for icbpanShutter_uifClass
# 
#   SYNOPSIS
#    icbpanShutter .w \
#              -background Grey90 \
#              -relief flat       \
#              -cfgObj $cfgObj    \
#              -cwp "<alias>${devName}" \
#              -devType $devType]
#
#     where cfgObj is a ctooCONFIG object;
#           devType is extracted from INS.CON.DEVICE${i}
#           devName is extracted from ${devType}.DEVNAME
# 
#   DESCRIPTION
#    Itcl wrapper class to handle ICB SHUT device. The corresponding
#    ibcpanShutter_uifClass contains a set of output widget (linked to OLDB)
#    which in turn display the device name, keyword used to identify the
#    device, the current simulation level (N=normal,S=simulate), the current
#    device state, handset mode (H, if active) and the current position
#    (named and in encoder unit). The class contains also an option
#    button to select among various setup parameters and an entry widget
#    for setting position values. Finally a checkbutton is used to
#    indicate if the corresponding device is selected for some actions.
#
#   PUBLIC METHODS
#    icbpanShutter  xxx
#       constructor. Calls "initFunction" which initialize the option
#       button with appropriate parameters. 
#    isSelected
#       get the current status of the checkbutton (0/1).
#    setSelected 0/1
#       set the checkbutton state to 0 or 1 (unselected or selected).
#    getKeyw
#       returns the associated setup keyw. + value. This method reads
#       from the option button and entry widget the current user
#       settings and prepare a string to be sent to ICS control process.
#       This method is normally called when the SETUP cmd is invoked.
#    setKeyw val
#       extract from "val" the keyword parameter + value and fill the
#       corresponding option button + entry. This method is called usually
#       during a reading of a previously saved setup file.
#    
#   PUBLIC VARIABLES
#    cfgObj
#       ctooCONFIG object (configuration file interface)
#    devType
#       device type extracted from INS.CON.DEVICE${i}
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTIONS
#
#   SEE ALSO
#    ctooCONFIG(8)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

class icbpanShutter {
   inherit icbpanShutter_uifClass

   constructor {args} {
      global ::shArea
      eval itk_initialize -helparea $shArea -relief flat $args
      display
      after idle [code $this initFunction]
   }

   method initFunction { } {

      # determine prefix to be used
      set prefix [icbpanGetDevicePrefix $cfgObj $devType]

      # initialize option lists
      foreach name $names {
          lappend elements $name
      }

      # initialize option button widget (value from status)
      upvar #0 [ $cv.devStatus0 cget -variable] wdgVar
      set last [getOptName $wdgVar]
      $cv.devSetup configure -itemList $elements -default $last \
                             -command [code $this updateWdg]
   }


   public method updateWdg { item } {
      global ::gvar

      if {[cequal $item ""]} {return 0}

      # select check button
      setSelected 1

   }

   # return status of the checkbutton
   public method isSelected { } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      return $sel
   }


   # set status of the checkbutton
   public method setSelected { val } {
      global ::gvar

      upvar #0 [$cv.devUse cget -variable] sel
      set sel $val
   }


   # return associated setup keyword
   public method getKeyw { } {
      global ::gvar

      upvar #0 [$cv.devSetup cget -variable] nameval

      if {[cequal $nameval ""]} {
          set setup ""
      } else {
          set setup "${prefix}.ST [getOptVal $nameval]"
      }

      return $setup
   }

   # set setup keyword+value
   public method setKeyw { val } {
      global ::gvar

      set keyw [lindex $val 0]
      set value [lindex $val 1]

      if {[cequal ${prefix}.ST $keyw]} {
         setSelected 1
         $cv.devSetup configure -default [getOptName $value]
      }
   }

   # return value from name
   private method getOptVal { name } {
       set index [lsearch -exact $elements $name]
       if { $index < 0 } { set value $name
       } else { set value [lindex $values $index] }
       return $value
   }

   # return name from value
   private method getOptName { value } {
      set name $value
      if { [lsearch -exact $elements $value] < 0 } {
          set index [lsearch -exact $values $value]
          # In WS simulation, it writes 0/1
          if {$index < 0} { set index [lsearch -exact $simvalues $value] }
          if {$index < 0} { set name ""
          } else { set name [lindex $elements $index] }
      }
      return $name
   }

   public variable devType ""
   public variable cfgObj  ""

   private variable prefix ""
   private variable elements {}
   private variable values    {T F}
   private variable simvalues {1 0}
   private variable names  {OPEN CLOSE}

}


#
# ___oOo___
