#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: icbpanUIF.tcl,v 1.47 2005/07/01 13:25:39 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  2005-06-02  icbpanGratNoOrd widget support added.
# alongino  24/11/04  support for assemblies (VLTSW20040336)
# alongino  10/01/02  set same default uifClass for all sensors
# dimarcan  15/11/00  created
#

#************************************************************************
#   NAME
#    icbpanUIF - itcl class used for managing icbpan devices classes 
# 
#   SYNOPSIS
#    icbpanUIF handleUif -cfgObj $cfgObj
#
#     where cfgObj is a ctooCONFIG object;
# 
#   DESCRIPTION
#    This class is used internally by the icbpan control panel to
#    associate an uifClass to the corresponding devices. It constructs 
#    an internal keyed list containing pairs device/uifClass 
#    (the key is actually the device name). 
#
#   PUBLIC METHODS
#    icbpanUIF  xxx
#       constructor. Calls two private methods. "defaultUifclasses"
#       method simply fill a keyed list (defuif) with default
#       associations; "initUifclasses" instead, by comparing
#       current instrument settings with the default keyed list,
#       extract and construct instrument specific keyed list (keylist).
#       From this list, during panel construction, the uifClass to be drawn
#       is extracted.
#    getUifClass devType
#       according to the passed "devType" the corresponding uifClass
#       is returned.
#    addUifClass devType className
#       add in the keyed list a new pair device/uifClass.
#    delUifClass devType
#       delete an entry in the keyed list having devType as a key.
#
#   PUBLIC VARIABLES
#    cfgObj
#       ctooCONFIG object (configuration file interface)
#
#   FILES
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES/*CONFIG.cfg
#
#   ENVIRONMENT
#       INS_ROOT        Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   RETURN VALUES
#
#   CAUTIONS
#    This class is used internally and should not be modified.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


class icbpanUIF {

   constructor { args } {
      eval configure $args

      set cfgcat [icbpanCfgCat $cfgObj]
      if { [llength $defuif] == 0 } {
         defaultUifclasses
         initUifclasses
      }
   }


   private method defaultUifclasses { } {

      keylset defuif  $cfgcat.OPT        icbpanNamedPos      \
                      $cfgcat.POS        icbpanContPos       \
                      $cfgcat.ADC        icbpanAdc           \
                      $cfgcat.DRO        icbpanDrot          \
                      $cfgcat.DPO        icbpanDpol          \
                      $cfgcat.FIL        icbpanNamedPos      \
                      $cfgcat.GRA        icbpanGrat          \
                      $cfgcat.GRA_NO     icbpanGratNoOrd     \
                      $cfgcat.MIR        icbpanNamedPos      \
                      $cfgcat.LAM        icbpanLamp          \
                      $cfgcat.SHU        icbpanShutter       \
                      $cfgcat.LEN2       icbpanSlit2Len      \
                      $cfgcat.WID2       icbpanSlit2Wid      \
                      $cfgcat.WHEEL      icbpanNamedPos      \
                      $cfgcat.TIL        icbpanContPos       \
                      $cfgcat.SEN        icbpanSensor        \
                      $cfgcat.DET        icbpanPmt           \
                      $cfgcat.ASS        icbpanAssembly
   }


   protected method initUifclasses { } {

      if { $cfgObj == "" } {
         error "No configuration specified."
      }

      set devices {}
      set devNo [icbpanGetDeviceNum $cfgObj]
      set n [expr {$devNo +1}]
      loop i 1 $n {
         lappend devices [$cfgObj Get $cfgcat.CON.DEVICE${i}]
      }
      set assNo [icbpanGetAssemblyNum $cfgObj]
      set n [expr {$assNo +1}]
      loop i 1 $n {
         lappend devices $cfgcat.ASSEMBLY${i}
      }

      set n [expr {$devNo + $assNo}]
      loop i 0 $n {
	 set device [lindex $devices $i]
	 set len [string length $cfgcat]
	 set id [csubstr $device 0 [expr {$len + 4}] ] 

	 if {[cequal $id "$cfgcat.SLI"]} {
            set newId [$cfgObj Get ${device}.DEVTYPE]
	    set id [cconcat [csubstr $device 0 [expr {$len + 1}] ] $newId]
         #} elseif {[cequal $id "$cfgcat.SEN"]} {
         #   set newId [$cfgObj Get ${device}.DEVTYPE]
         #   set id [cconcat [csubstr $device 0 [expr $len + 1] ] $newId]
         } elseif {[cequal $id "$cfgcat.GRA"]} {
      	    set posNum [$cfgObj Get ${device}.POSNUM]
	    set isUsedOrd 0
	    loop j 1 [incr posNum] {
               if {[catch { $cfgObj Get ${device}.ORDERMAX$j } val ]} { set val "" }
               if { $val != "" && $val != 0 } {
                  set isUsedOrd 1
                  break
               }
	    }
	    if { $isUsedOrd == 0 } {
	       set id "$cfgcat.GRA_NO"
	    }
	 }

         try_eval {
            set val [keylget defuif $id]
         } {
            set val ""
         }

	 if {![cequal $val ""]} {
            keylset keyuif $device $val
         }
      }
   }


   public method getUifClass { devType } {

      try_eval {
         set val [keylget keyuif $devType]
      } {
         set val ""
      }

      return $val

   }


   public method addUifClass { devType className } {

      try_eval {
         keylset keyuif $devType $className
      } {
         set val ""
      }
   }


   public method delUifClass { devType } {

      try_eval {
         keyldel keyuif $devType
      } {
         set val ""
      }
   }


   public variable cfgObj   ""
   private variable defuif  {}
   private variable keyuif  {}
   private variable cfgcat  ""

}


#
# ___oOo___
