#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ikonxDevice.tcl,v 1.21 2001/06/22 09:32:00 vltsccm Exp $" 
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 13.04.1995 created
# S.Sandrock 03.11.1995 upgraded for ikon 1.20: pos-scanning/scaling
# S.Sandrock 18.04.1996 Tk4/incrTk2 porting
#

#************************************************************************
#   NAME
#
#	IkonxDevice - Class description for a "ikon" device
# 
#   SYNOPSIS
#
#	IkonxDevice <object-name> ?<options>?
#
#   DESCRIPTION
#
#	IkonxDevice defines the class for a "ikon" encoder interface device.
#	It makes the functionality of the device and the corresponding 
#	"ikon" driver available through a graphical user interface.
#
#	An object of this class is created by:
#
#		IkonxDevice <object-name> -<Option> <Value> ...
#
#	This will create all sub-objects for user interaction and
#	an object to handle the connection to a server-task in the 
#	LCU environment i.e. effectively to the physical device.
#
#   OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<object-name> configure -<Option> <Value> ...
#
#	-displayFont	specifies the font to be used for the position,
#			counter and interpolation displays.
#			Default: 10x20
#
#	Several options are inherited from InducerDevice and are therefore also
#	available within the scope of this class. See InducerDevice(n) for a 
#	detailed description of those common options.
#
#   METHODS
#	Methods represent procedures on the object and are invoked with:
#
#		<object-name> <Method> <Arguments> ...
#
#	There are no special public device-specific methods available.
#
#	Several method are inherited from InducerDevice and overloaded here
#	with device-specific actions. See InducerDevice(n) for a detailed
#	description of those common methods.
#
#   PROCS
#
#   EXAMPLES
#
#   SEE ALSO
#	ikonxInducer(1),
#	InducerClass(n), InducerDevice(n), InducerLine(n)
#
#   BUGS
#
#------------------------------------------------------------------------


#------------------------------------------------------------------------
#*# class: IkonxDevice

# This class works with 48-bit values contained in floating-point format.
# In order to use the highest possible precision:
global ::tcl_precision
set tcl_precision 17


class IkonxDevice {

    inherit InducerDevice

    constructor { args } {}

    #--------------------------------------------------------------------
    # Public widget options
    #--------------------------------------------------------------------

    itk_option define -displayFont displayFont Font 10x20

    #--------------------------------------------------------------------
    # Overloaded standard members of InducerDevice class
    #--------------------------------------------------------------------

    public method GetVersion { }
    public method InitDevice { }
    public method ExitDevice { }
    public method ResetDevice { }
    public method GetDeviceId { }

    #--------------------------------------------------------------------
    # Device-specific members of IkonxDevice class
    #--------------------------------------------------------------------

    protected proc   initbin {} {}
    protected method bgLatch {}
    protected method ShowPos { Channel Pos {Ctr 0} {Ipo 0}}
    protected method SetPosFormat { Channel PosFmt }
    protected method ZeroPosition { Channel }
    protected method RefMove { Channel }
    protected method RecordCorr { Channel }
    protected method ReadParam { Channel Paramspec }
    protected method WriteParam { Channel }
    protected method StartMonitor {}
    protected method MonitorCB { Reply }

    protected variable scaleFactor 1.0		;# read in from device
    protected variable scaledPosFmt %16.0f	;# derived from scaleFactor
    protected variable cTime 500		;# const refresh period in ms

    common bin ;# array for 8-bit binary display conversion (0..255)
    common bin4 { 0000 0001 0010 0011 0100 0101 0110 0111\
	    1000 1001 1010 1011 1100 1101 1110 1111 }

    common params
    set params(X1) { \
	    "1.1-directionX1"	"2.1-axisTypeX1"	\
	    "4.1-refDistanceX1"	"5.1-sigPeriodsX1"	\
	    "6.1-corrEnableX1"	"7.1-corrRangeX1"	\
	    "8.1-corrCountX1"	"9.1-corrWidthX1"	\
	    "19.1-refOffsetX1"	"30.1-corrParamX1X2"	\
	    "70.1-extPresetX1"	"71.1-vmePresetX1"	\
	    "72.1-axisOffsetX1"	"80.1-extFunction1"	}
    set params(X2) { \
	    "1.2-directionX2"	"2.2-axisTypeX2"	\
	    "4.2-refDistanceX2"	"5.2-sigPeriodsX2"	\
	    "6.2-corrEnableX2"	"7.2-corrRangeX2"	\
	    "8.2-corrCountX2"	"9.2-corrWidthX2"	\
	    "19.2-refOffsetX2"	"30.2-corrParamX2"	\
	    "70.2-extPresetX2"	"71.2-vmePresetX2"	\
	    "72.2-axisOffsetX2"	"80.2-extFunction1"	}
    set params(X3) { \
	    "2.3-axisTypeCombi"	\
	    "3.0-validBits"		"5.3-sigPeriodsCombi"	\
	    "10.0-latchDisable"	"21.0-combiMode"	\
	    "70.3-extPresetCombi"	"71.3-vmePresetCombi"	\
	    "72.3-axisOffsetCombi"	"81.0-vmeFunction"	}

}

body IkonxDevice::constructor { args } {

    global ::shArea ;# short-help display

    # NOTE: Tk widgets in this class are connected to global variables.
    # These vars are stored in an array named as "${itk_interior}".
    # The value of a variable can be queried with "[set ${itk_interior}(..)]".
    # All methods using these vars must include a `global' statement:
    global ::${itk_interior}	;# global array of "pseudo-local" variables

    # initialize "pseudo-local" variables:
    set ${itk_interior}(pos.1)	"               ?"	;# position channel X1
    set ${itk_interior}(pos.2)	"               ?"	;# position channel X2
    set ${itk_interior}(pos.3)	"               ?"	;# position combined

    set ${itk_interior}(posReal.1)	0 ;# real numeric position channel X1
    set ${itk_interior}(posReal.2)	0 ;# real numeric position channel X2
    set ${itk_interior}(posReal.3)	0 ;# real numeric position channel cmb.

    set ${itk_interior}(posFmt.1)	DEC	;# position format X1
    set ${itk_interior}(posFmt.2)	DEC	;# position format X2
    set ${itk_interior}(posFmt.3)	DEC	;# position format cmb.

    set ${itk_interior}(ctr.1)		?	;# counter channel X1
    set ${itk_interior}(ctr.2)		?	;# counter channel X2
    set ${itk_interior}(ctr.3)		?	;# counter combined

    set ${itk_interior}(ipo.1)		?	;# interpol. channel X1
    set ${itk_interior}(ipo.2)		?	;# interpol. channel X2
    set ${itk_interior}(ipo.3)		?	;# interpol. combined

    set ${itk_interior}(status.1)	?	;# status channel X1
    set ${itk_interior}(status.2)	?	;# status channel X2
    set ${itk_interior}(status.3)	?	;# status combined

    set ${itk_interior}(par.1)	"Parameter X1"	;# parameter channel X1
    set ${itk_interior}(par.2)	"Parameter X2"	;# parameter channel X2
    set ${itk_interior}(par.3)	"Param X1+2/X3"	;# parameter combined
    set ${itk_interior}(parSpec.1)	?	;# parameter channel X1
    set ${itk_interior}(parSpec.2)	?	;# parameter channel X2
    set ${itk_interior}(parSpec.3)	?	;# parameter combined
    set ${itk_interior}(parVal.1)	?	;# parameter channel X1
    set ${itk_interior}(parVal.2)	?	;# parameter channel X2
    set ${itk_interior}(parVal.3)	?	;# parameter combined


    #REMOVED: InducerDevice::constructor	;# to build the Line object
    $lineW configure -cPeriod [expr $cTime/10]	;# configure monitor period

    set w $itk_interior

    initbin ;# init internal binary conversion

    foreach ch { 1 2 3 } {

	itk_component add F.$ch {
	    frame $w.$ch	-relief groove -bd 2
	}

	# Position display:
	itk_component add Bpos.$ch {
	    UifMenuButton $w.$ch.1 -width 11 -label "Position X$ch:" \
		    -helptext "Select here the position display format\
		    and explicit background latching"
	} {
	    usual
	    ignore -width -label
	}
	$w.$ch.1 addRadioButton decimal 0 \
		[code $this SetPosFormat $ch DEC] \
		${itk_interior}(posFmt.$ch) DEC
	$w.$ch.1 addRadioButton binary 0 \
		[code $this SetPosFormat $ch BIN] \
		${itk_interior}(posFmt.$ch) BIN

	itk_component add Opos.$ch {
	    UifOutput $w.$ch.2	-width 16 -anchor e \
		    -variable ${itk_interior}(pos.$ch) \
		    -helparea $shArea -helptext \
		    "Scaled decimal Position, or unscaled binary Position\
		    with 32-bit Counter and 16-bit Interpolation"
	} {
	    usual
	    rename -font -displayFont displayFont Font
	}

	# Counter display:
	itk_component add Lctr.$ch {
	    label $w.$ch.3	-width 2 -anchor e -text Ct:
	}
	itk_component add Octr.$ch {
	    UifOutput $w.$ch.4	-width 11 -anchor e \
		-variable ${itk_interior}(ctr.$ch) \
		-helparea $shArea -helptext \
		"Decimal 32-bit Counter value without interpolation"
	} {
	    usual
	    rename -font -displayFont displayFont Font
	}

	# Interpolation display:
	itk_component add Lipo.$ch {
	    label $w.$ch.5	-width 2 -anchor e -text Ip:
	}
	itk_component add Oipo.$ch {
	    UifOutput $w.$ch.6	-width 6 -anchor e \
		    -variable ${itk_interior}(ipo.$ch) \
		    -helparea $shArea -helptext \
		    "Decimal 16-bit Interpolation value"
	} {
	    usual
	    rename -font -displayFont displayFont Font
	}

	# Status display:
	itk_component add Lsta.$ch {
	    label $w.$ch.7	-width 5 -anchor e -text "Status:" -padx 0.2c
	}
	itk_component add Osta.$ch {
	    UifOutput $w.$ch.8	-width 8 -anchor e \
		    -variable ${itk_interior}(status.$ch) \
		    -helparea $shArea -helptext \
		    "Status bits:\
		    7=CorrInProgress 6=n/a 5=RefWait 4=FreqError\
		    3=LowAmplitude 2=CtrStarted 1=n/a 0=SignalCorr"
	} {
	    usual
	    rename -font -displayFont displayFont Font
	}

	# "Zero Pos" button:
	itk_component add Bzero.$ch {
	    UifButton $w.$ch.9	-width 8 -label "Zero Pos" \
		    -command [code $this ZeroPosition $ch] \
		    -helparea $shArea -helptext \
		    "Set position at present point to zero"
	}

	# "Ref Move" button:
	itk_component add Bref.$ch {
	    UifButton $w.$ch.10	-width 8 -label "Ref Move" \
		    -command [code $this RefMove $ch] \
		    -helparea $shArea -helptext \
		    "Initiate search for reference-point\
		    (X3 includes X1 and X2)"
	}
	#REMOVED: if {$ch == 3} {$w.$ch.10 configure -state disabled}

	# "Record" button:
	itk_component add Brec.$ch {
	    UifButton $w.$ch.11	-width 8 -label "Record" \
		    -command [code $this RecordCorr $ch] \
		    -helparea $shArea -helptext \
		    "Initiate correction value recording\
		    (not for combined channel)"
	}
	if {$ch == 3} {$w.$ch.11 configure -state disabled}

	# Parameter button:
	itk_component add Bpar.$ch {
	    UifOptionButton $w.$ch.20 -length 18 \
		    -variable ${itk_interior}(par.$ch) \
		    -itemList $params(X$ch) \
		    -default [set ${itk_interior}(par.$ch)] \
		    -command [code $this ReadParam $ch]
	} {
	    usual
	    ignore -font -variable -command
	}
	$w.$ch.20 configure \
		-helparea $shArea -helptext \
		"Parameter name and value, press <Enter> in entry to write"

	itk_component add Epar.$ch {
	    UifEntry $w.$ch.21	-width 12 \
		    -variable ${itk_interior}(parVal.$ch) \
		    -helparea $shArea -helptext \
	    	    "Parameter value - press <Enter> to write"
	}
	bind $w.$ch.21.uifEntry <Return> [code $this WriteParam $ch]

	pack $w.$ch	-side top  -anchor w -fill x -expand yes
	pack $w.$ch.1 $w.$ch.2 $w.$ch.3 $w.$ch.4 $w.$ch.5 $w.$ch.6 \
		$w.$ch.7 $w.$ch.8 $w.$ch.9 $w.$ch.10 $w.$ch.11 \
		$w.$ch.20 $w.$ch.21 \
		-side left -anchor w

	##pack $w.$ch.2 -padx 2 ;# adjustment for dec/bin display //REMOVED

    } ;# foreach

    bgLatch ;# start background process for conditional LATCH generation

    # integrate composite options:
    eval itk_initialize $args

} ;# constructor

#--------------------------------------------------------------------
# Internal data and routines:
#--------------------------------------------------------------------

body IkonxDevice::initbin { } {
    for {set i 0} {$i <= 255} {incr i} {
	set bin($i) [lindex $bin4 [expr $i>>4]][lindex $bin4 [expr $i&15]]
    }
}


body IkonxDevice::bgLatch { } {
    global ::inducer

    if {$inducer(vMode) != "READONLY"} {
	$lineW Ioctl LATCH_POSITION_X1 0
	$lineW Ioctl LATCH_POSITION_X2 0
	$lineW Ioctl LATCH_POSITION_COMBI 0
    }
    after $cTime [code $this bgLatch]
}


#--------------------------------------------------------------------
# Device specific methods:
#--------------------------------------------------------------------

# display position `Value' of `Channel' according to current format;
body IkonxDevice::ShowPos { Channel Pos {Ctr 0} {Ipo 0}} {
    global ::${itk_interior}

    set ${itk_interior}(posReal.$Channel) $Pos
    switch [set ${itk_interior}(posFmt.$Channel)] {
	DEC {
	    set ${itk_interior}(pos.$Channel) [format $scaledPosFmt $Pos]
	    set ${itk_interior}(ctr.$Channel) [format %11.0f $Ctr]
	    set ${itk_interior}(ipo.$Channel) [format  %6.0f $Ipo]
	}
	BIN {
	    set p ""
	    # binary conversion for the 6 bytes:
	    if {[catch {
		append p $bin([expr {$Ctr >> 24 & 0xff}])-
		append p $bin([expr {$Ctr >> 16 & 0xff}])-
		append p $bin([expr {$Ctr >> 8 & 0xff}])-
		append p $bin([expr {$Ctr & 0xff}]).
		append p $bin([expr {$Ipo >> 8 & 0xff}])-
		append p $bin([expr {$Ipo & 0xff}])
	    }]} {
		set p ?
	    }
	    set ${itk_interior}(pos.$Channel) $p
	    # ctr is not set, because unmapped from the screen here
	}
    }
}

body IkonxDevice::SetPosFormat { Channel PosFmt } {
    global ${itk_interior}
    debugPrint
    set w ${itk_interior}.$Channel
    # re-config & pack widgets so that effective length remains the same:
    switch $PosFmt {
	DEC {
	    $w.2 configure -width 16
	    pack $w.3 $w.4 $w.5 $w.6 $w.7 $w.8 \
		    -after $w.2 -side left -anchor w
	}
	BIN {
	    #$w.2 configure -width 54 -font 9x15bold //OLD
	    $w.2 configure -width 53
	    # remove Ctr+Ipo+Status from screen:
	    pack forget $w.3 $w.4 $w.5 $w.6 $w.7 $w.8
	}
    }
    set ${itk_interior}(posFmt.$Channel) $PosFmt
    ShowPos $Channel [set ${itk_interior}(posReal.$Channel)] ;# show in new format
}

body IkonxDevice::ZeroPosition { Channel } {
    debugPrint
    switch $Channel 1 {set Channel X1} 2 {set Channel X2} 3 {set Channel COMBI}
    $lineW IoctlTrap SET_POSITION_$Channel 0.0
}

body IkonxDevice::RefMove { Channel } {
    debugPrint
    switch $Channel 1 {set Channel _X1} 2 {set Channel _X2} 3 {set Channel ""}
    $lineW IoctlTrap REF_MOVE$Channel 0.0
}

body IkonxDevice::RecordCorr { Channel } {
    debugPrint
    $lineW IoctlTrap RECORD_CORR_X$Channel 0.0
}

body IkonxDevice::ReadParam { Channel Paramspec } {
    global ::${itk_interior}
    debugPrint
    regexp {[0-9.]*} $Paramspec spec ;# remove parameter description text
    set ${itk_interior}(parSpec.$Channel) $spec
    set ret [$lineW IoctlTrap READ_PARAMETER "$spec;0.0"]
    set ${itk_interior}(parVal.$Channel) [lindex [split [lindex $ret 0] ";"] 1]
}

body IkonxDevice::WriteParam { Channel } {
    global ::${itk_interior}
    debugPrint
    set spec [set ${itk_interior}(parSpec.$Channel)]
    set val  [format "%.1f" [set ${itk_interior}(parVal.$Channel)]]
    $lineW IoctlTrap WRITE_PARAMETER "$spec;$val"
    $lineW IoctlTrap UPDATE_PARAMS 0
    ReadParam $Channel $spec ;# read back
}


body IkonxDevice::StartMonitor { } {
    debugPrint
    $lineW MonitorStart \
	    { SCAN_POSITION_X1 SCAN_POSITION_X2 SCAN_POSITION_COMBI\
	    GET_SCALE_FACTOR } \
	    { 0.0;0;0;0 } {} \
	    [code $this MonitorCB]
}

body IkonxDevice::MonitorCB { Reply } {
    global ::${itk_interior}
    debugPrint

    if {[llength $Reply] < 5} {
	::inducerDialog "Warning: Error during Device Monitoring!"
	return
    }

    foreach ch {1 2 3} {
	set pos$ch 0; set status$ch 0; set ctr$ch 0; set ipo$ch 0
    }

    # NOTE: the "::" before "scan" is important. Otherwise the previously
    # assigned higher `tcl_precision' (17) will not be used!
    ::scan [lindex $Reply 0] "%lf;%d;%d;%d" pos1 status1 ctr1 ipo1
    ::scan [lindex $Reply 1] "%lf;%d;%d;%d" pos2 status2 ctr2 ipo2
    ::scan [lindex $Reply 2] "%lf;%d;%d;%d" pos3 status3 ctr3 ipo3
    ::scan [lindex $Reply 3] "%lf" sf

    # determine suitable display-format from scaleFactor:
    if {$sf != $scaleFactor} {
	set scaleFactor $sf ;# re-calculate only on change
	if {[catch {set logScale [expr int(ceil(-log10($sf)))]}] || \
		$logScale < 0 } {
	    set logScale 0	;# default: no fraction
	} elseif {$logScale > 15} {
	    set logScale 15	;# max. digits in fraction
	}
	set scaledPosFmt %[expr 16-${logScale}].[expr ${logScale}]f
	debugPrint "scaleFactor=$scaleFactor scaledPosFmt=$scaledPosFmt"
    }

    ShowPos 1 $pos1 $ctr1 $ipo1
    ShowPos 2 $pos2 $ctr2 $ipo2
    ShowPos 3 $pos3 $ctr3 $ipo3

    if {[catch {set ${itk_interior}(status.1) $bin([expr $status1&0xff])}]} {
	set ${itk_interior}(status.1) ????????
    }
    if {[catch {set ${itk_interior}(status.2) $bin([expr $status2&0xff])}]} {
	set ${itk_interior}(status.2) ????????
    }
    if {[catch {set ${itk_interior}(status.3) $bin([expr $status3&0xff])}]} {
	set ${itk_interior}(status.3) ????????
    }

}


#--------------------------------------------------------------------
# Overloaded standard members of InducerDevice class
#--------------------------------------------------------------------

body IkonxDevice::GetVersion { } {
    debugPrint
    global ::ikonx ::argv0
    return "[file tail $argv0] - Version $ikonx(version)"
}

body IkonxDevice::InitDevice { } {
    debugPrint
    set err [InducerDevice::InitDevice] ;# common part to open channel
    if {$err != ""} {return $err}
    $this StartMonitor
    return ""
}

body IkonxDevice::ExitDevice { } {
    debugPrint
    $lineW MonitorStop
    return [InducerDevice::ExitDevice]	;# common part to close channel
}

body IkonxDevice::ResetDevice { } {
    debugPrint
    return [$lineW Ioctl POWER_ON_SELF_TEST 0]
}

body IkonxDevice::GetDeviceId { } {
    debugPrint
    return <unknown>
}

#___oOo___
