#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: inducerDevice.tcl,v 1.65 2003/12/17 19:26:26 vltsccm Exp $" 
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 22.03.1995 created
# imunoz     09.04.1996 ported to itcl2.0
#

#************************************************************************
#   NAME
#
#	InducerDevice - Abstract class for inducer Device objects
# 
#   SYNOPSIS
#
#	inherit InducerDevice 
# 
#   DESCRIPTION
#
#	InducerDevice is used to inherit special features for the 
#	implementation of Device objects to be used in "Inducer Panels".
#	All OPTIONS and METHODS of this class are then also available
#	in the objects that inherit from this class.
#
#	Device-specific methods shall be provided in the derived classes
#	which overload the mostly empty default-methods contained herein.
#
#   OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<object-name> configure -<Option> <Value> ...
#
#	The following options are publicly available:
#
#	vEnv	- enviroment name
#	vDev	- device name
#	vMode	- open-mode to be used: TEST, EXCLUSIVE, SHARED, READONLY
#
#   METHODS
#	Methods represent procedures on the object and are invoked with:
#
#		<object-name> <Method> <Arguments> ...
#
#	The following methods are publicly available:
#
#	constructor ?<options>?
#
#		At the beginning of each device-specific class should be
#		a call of "InducerDevice::constructor". This creates an
#		InducerLine object that serves as a link to the inducerServer
#		task on the LCU. The object name is saved in the protected
#		variable `lineW'.
#
#	The following methods should be overloaded by the derived class
#	with appropriate device-specific actions. This base-class provides 
#	only some default actions. If the required action is not available
#	for the specific device, then the method need not be overloaded.
#
#	GetVersion	- must return the version-string of the program
#
#	InitDevice	- must first call `InducerDevice::InitDevice' to open 
#			  the link to the device, then start the monitor task,
#			  and possibly do specific actions.
#			  Must return an empty string on success or an
#			  error description.
#
#	ExitDevice	- must stop the monitor task, and at the end call 
#			  `InducerDevice::ExitDevice' to close the link to 
#			  the device. Must return an empty string.
#
#	ResetDevice	- must reset the connected device.
#
#	GetDeviceId	- must retrieve and return a device-ID-string
#			  to be displayed in the panel.
#
#	LoadDevice <data> - must restore the device-status from the string 
#			`data' which is in the same format as from SaveDevice.
#
#	SaveDevice	- must retrieve and return the device-status as a
#			  a string which is suitable for LoadDevice.
#
#	Disconnect	- requests that all further user interactions remain
#			  local to the panel, neither are changes forwarded
#			  to the device, nor is the panel updated from the
#			  device. However, the channel is kept opened.
#
#	Resume		- requests that the current panel settings are 
#			  forwared to the device in one go. After that the
#			  normal operation and update policy is resumed.
#
#   EXAMPLES
#
#   SEE ALSO
#	InducerClass(n), InducerLine(n)
#
#   BUGS
#
#------------------------------------------------------------------------


#------------------------------------------------------------------------
#*# class: InducerDevice

class InducerDevice {

    inherit InducerClass 

    #*# constructor
    
    # widget's options
    
    constructor { args } { }
    itk_option define -vEnv vEnv VEnv ""
    itk_option define -vDev vDev VDev ""
    itk_option define -vMode vMode VMode ""
    itk_option define -vCheckTime vCheckTime VCheckTime 5000

    destructor { }
    # object-name of associated LCU connection line
    protected variable lineW ""	

    
#    public variable vEnv ""
#    public variable vDev ""
#    public variable vMode ""
#    public variable vCheckTime 5000

    public    method GetVersion { } { }

    protected method InitDevice { } { }

    protected method ExitDevice { } { }

    protected method ResetDevice { } { }

    protected method GetDeviceId { } { }

    protected method LoadDevice { Data } { }

    protected method SaveDevice { } { }

    protected method LoadFromLCC { } { }

    protected method Disconnect { } { }

    method Resume { } { }
    
    method GetSignalConfig { } { }
    
    method Macro { Operation args } { }

    #--------------------------------------------------------------------
    # Local methods
    #--------------------------------------------------------------------
    method makeNodes { src } { }
    
    method sendServer { Env Proc Com {Par ""} } { }
    method givemeName {src} { }
    method givemeType { src } { } 
    method givemeIO { src } { }
    method checkLCUConnect { } { }
    
    protected variable cnfList {}
}

body InducerDevice::constructor {args} {
    set lineW [InducerLine $this.line]
    debugPrint "itcl_2.0:  lineW = $lineW"
    eval itk_initialize $args
}

body InducerDevice::GetVersion { } {
    ::global inducer argv0
    return "[file tail $argv0] - Version $inducer(version)"
}

body InducerDevice::InitDevice { } {
    # This will be requested by the user through a menu option
    # virtual GetSignalConfig
    
    # open channel to device:
    if {[$lineW Open $itk_option(-vEnv) $itk_option(-vDev) $itk_option(-vMode)] < 0} {
	return "could not open \"$itk_option(-vEnv)\" \"$itk_option(-vDev)\" in \"$itk_option(-vMode)\" mode"
    }
    
    # This produces some problems with the RTAP Enviroment 
    # so, see you later !.
    #after $vCheckTime [list $this checkLCUConnect]
    
    return "" ;# OK
}

body InducerDevice::ExitDevice { } {
    # close channel to device:
    $lineW Close
    return "" ;# OK
}

body InducerDevice::ResetDevice { } {
}

body InducerDevice::GetDeviceId { } {
    return <unknown>
}

body InducerDevice::LoadDevice { Data } {
    ::inducerDialog "Device-specific Load routine is not implemented!"
}

body InducerDevice::SaveDevice { } {
    ::inducerDialog "Device-specific Save routine is not implemented!"
}

body InducerDevice::LoadFromLCC { } {
    ::inducerDialog "Device-specific LoadFromLCC routine is not implemented!"
}

body InducerDevice::Disconnect { } {
    ::global inducer
    
    if { [$this info function Disconnect -name] != "InducerDevice::Disconnect" } {
	set inducer(fOnline) 0
	return
    }
    ::inducerDialog "Device-specific Disconnect option is not implemented!"
}

body InducerDevice::Resume { } {
    ::global inducer 
    
    if { [$this info function Resume -name] != "InducerDevice::Resume" } {
	set inducer(fOnline) 1
	return
    }
    ::inducerDialog "Device-specific Resume option is not implemented!"
}

body InducerDevice::GetSignalConfig { } {
    debugPrint
    set workList [sendServer $itk_option(-vEnv) lccServer IOGLST ""]
    return [makeNodes $workList]
}


body InducerDevice::Macro { Operation args } {
    return [eval $lineW Macro $Operation $args]
}

#--------------------------------------------------------------------
# Local methods
#--------------------------------------------------------------------
body InducerDevice::makeNodes { src } {
    set cnfList {} ;# clean cnfList
    
    set src [split $src ,]
    set max [lindex $src 0]
    set src [lrange $src 1 [expr {[llength $src] -1}] ] 
    for {set i 0} { $i < $max} {incr i} {
	set j [expr {$i*3}]
	lappend cnfList [lrange $src $j [expr {$j+2}]]
    } 
}

body InducerDevice::sendServer { Env Proc Com {Par ""} } {
    
    set cmdId [seq_msgSendCommand $Env $Proc $Com $Par]
    set reply [seq_msgRecvReply $cmdId errorNr lastReply]
    if {$errorNr != 0} {
	inducerDialog "Error-reply from $Env/$Proc in response to\
		\"$Com $Par\":\n$reply"
	return ""
    }
    return $reply
}

body InducerDevice::givemeName {src} {
    return [lindex $src 0]
}
body InducerDevice::givemeType { src } {
    return [lindex $src 1 ]
} 

body InducerDevice::givemeIO { src } {
    return [lindex $src 2]
}

body InducerDevice::checkLCUConnect { } {
    debugPrint
    
    if {  [catch \
	    {set cmdId [seq_msgSendCommand $itk_option(-vEnv) inducerServer VERSION ""]}]} {
	inducerEvhSelectDev ""
	::inducerDialog "Can not send command to $itk_option(-vEnv)/inducerServer. Maybe LCU is Down !!"
	return ""
    }
    if { [catch \
	    {set reply [seq_msgRecvReply $cmdId errorNr lastReply]}] } {
	inducerEvhSelectDev ""
	::inducerDialog "Can not send command to $itk_option(-vEnv)/inducerServer. Maybe LCU is Down !!"
    }
    if {$errorNr != 0} {
	inducerEvhSelectDev ""
	::inducerDialog "Error-reply from $Env/$Proc Maybe LCU is Down !!"
	return ""
    }
    
    after $vCheckTime [list $this checkLCUConnect]
}

#___oOo___
