#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: inducerGetLib.tcl,v 1.65 2003/12/17 19:26:26 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 15.03.1995 created
#

#************************************************************************
#   NAME
#	inducerGetLib,
#	inducerGetDevices,
#	inducerGetTickRate,
#	inducerGetIoctlCommands,
#	inducerGetIoctlCmdNum,
#	inducerGetTools,
#	inducerGetToolArg
#	- get information in the inducer environment
# 
#   SYNOPSIS
#	inducerGetDevices	Environment ?Filters?
#	inducerGetTickRate	Environment
#	inducerGetIoctlCommands	Device
#	inducerGetIoctlCmdNum	CmdNameOrNumList
#	inducerGetTools		Devices
#	inducerGetToolArg	Device Tool
# 
#   DESCRIPTION
#	These procs are called by the inducer program.
#
#	inducerGetDevices	- query the installed devices from a LCU
#				  environment and return them as a list,
#				  passing at least one of the optional filters
#				  in `Filters' (e.g. {/acro* /aio*}).
#				  Only devices with VLT conform names are
#				  returned, matching the regexp "/[a-z]+[0-9]+"
#
#	inducerGetTickRate	- query the system-clock-rate from the LCU
#				  in ticks per second.
#
#	inducerGetIoctlCommands	- query the command-literals for a given
#				  device from the corresponding driver
#				  header file. Returns a list-representation
#				  for a cascaded menu, suitable for MakeMenu.
#				  (same behaviour as in lcudrvTk)
#
#	inducerGetIoctlCmdNum	- translate a command-literal(-list) into it's
#				  corresponding number(-list). The literal must
#				  be given without the driver-specific prefix,
#				  e.g. "READ_BITS" (not "acroCMD_READ_BITS").
#				  No translation is done if the literal does
#				  not exist or if it is already a number.
#
#	inducerGetTools		- query the tool-functions for a given list of
#				  `Devices' from the corresponding driver
#				  header file. Returns a list of functions.
#				  (same behaviour as in lcudrvTk)
#
#	inducerGetToolArg	- determines the proper argument for a tool
#				  function `Tool' when applied on a specific
#				  `Device'. This depends on the function's
#				  argument declaration in the driver header
#				  file, which can be:
#				  - empty for a "void" argument declaration
#				  - device-name if "name" is declared
#				  - unit-number if "unit" or "axis" is declared
#				  - else empty (unknown)
#				  (same behaviour as in lcudrvTk)
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#	lcudrvTk(1)
#
#   BUGS     
#
#------------------------------------------------------------------------
#



###############################################################################
#
#	G L O B A L   V A R I A B L E S
#	"Declaration" of global variables
#
###############################################################################
# Rules: see inducer.tcl
#
###############################################################################



###############################################################################
#
#	P R O C E D U R E S
#
###############################################################################
# Rules: see inducer.tcl
#
###############################################################################

proc inducerGetDevices { Environment {Filters ""} } {
    set devs [InducerLine::Send $Environment GETDEVS]
    set devlist [split $devs ,]
    if {$Filters != ""} {
	# filter not matching device-names out:
	set devlist1 {}
	foreach dev $devlist {
	    # take only VLT conform names:
	    if {! [regexp {/[a-z]+[0-9]+} $dev]} continue
	    # check all given filters:
	    foreach filt $Filters {
		if {[string match $filt $dev]} {
		    lappend devlist1 $dev
		    break
		}
	    }
	}
	set devlist $devlist1
    }

    return $devlist
}


proc inducerGetTickRate { Environment } {
    return [InducerLine::Send $Environment GETTICK]
}


proc inducerGetIoctlCommands { Device } {
    global inducerCmdNum inducerCmdName

    # derive the driver-name from the device by removing any slashes/digits:
    set driver [string trim $Device "/0123456789"]

    # find the 'Commands' header of the driver:
    if {[catch {vccFindFile vw/include/${driver}Commands.h} filename]} {
	return ""
    }
    if {$filename == ""} {return ""}

    set prefix ${driver}CMD_
    set pre [string length $prefix]

    set cmdlist {}
    set grouplist {}

    inducerPuts "Get Commands of $driver from $filename"
    set f [open $filename]
    while {[gets $f line] >= 0} {
	if {[string first " * " $line] == 0} {
	    inducerPuts "	$line"
	    if {$grouplist != ""} {lappend cmdlist $grouplist}
	    set grouplist \"$line\"
	}
	if {[string first "#define $prefix" $line] == 0} {
	    scan $line "%s%s%s" null name num
	    inducerPuts "		$name $num"
	    set name [string range $name $pre end]
	    lappend grouplist $name
	    set inducerCmdNum($name) $num
	    set inducerCmdName($num) $name
	}
    }
    close $f
    if {$grouplist != ""} {lappend cmdlist $grouplist}
    return $cmdlist
}


proc inducerGetIoctlCmdNum { CmdNameOrNumList } {
    global inducerCmdNum inducerCmdName

    set numlist {}
    foreach cmd $CmdNameOrNumList {
	if {[info exist inducerCmdNum($cmd)]} {
	    set num $inducerCmdNum($cmd)
	} else {
	    set num $cmd
	}
	lappend numlist $num
    }

    return $numlist
}


#
# Read driver header(s) to get tool-functions.
#
proc inducerGetTools { Devices } {
    global inducerToolArg

    set tools {}
    set drvlist {}
    foreach dev $Devices {
	if {! [regexp {/[a-z]+[0-9]+} $dev]} continue ;# skip improper names
	# derive the driver-name from the device by removing slashes+digits:
	set driver [string trim $dev "/0123456789"]
	# do only once for each driver:
	if {[lsearch $drvlist $driver] != -1} continue
	lappend drvlist $driver

	# find the public header "xxx.h" of the driver:
	if {[catch {vccFindFile vw/include/${driver}.h} filename]} {
	    return ""
	}
	if {$filename == ""} {return ""}

	set prefix ${driver}
	set pre [string length $prefix]

	set name ""

	# regular expression to match tool declarations:
	#	".* (xxx[^ (]*) *\(([^,#]*)\)"
	# Special characters interpreted by Tcl and regexp must be quoted twice
	set expr ".* ($prefix\[^ \(\]*) *\\((\[^,#\]*)\\)"
	inducerPuts "Get Tools of $driver from $filename"

	set f [open $filename]
	while {[gets $f line] >= 0} {
	    if {[regexp $expr $line null name param]} {
		set inducerToolArg($name) [string trim $param]
		inducerPuts "	${name}($inducerToolArg($name))"
		lappend tools $name
	    }
	}
	close $f
    }
    return $tools
}

#
# Get the proper Argument for the tool-function,
#	use either the device-name, the unix/axis number or the 
#	 general Argument, this depends on the $inducerToolArg() variable:
#	 - device-name if "name" is part of it
#	 - unit-number if "unit" or "axis" is part of it
#	 - else empty argument
#
proc inducerGetToolArg { Device Tool } {
    global inducerToolArg

    if {[string match {void} $inducerToolArg($Tool)] } {
	set arg ""
	inducerPuts "$Tool ==> no arg (void)"
    } elseif {[string match {*[Nn]ame*} $inducerToolArg($Tool)] } {
	set arg \"$Device\"
	inducerPuts "$Tool ==> device-name $arg"
    } elseif {[string match {*[Uu]nit*} $inducerToolArg($Tool)] || \
	    [string match {*[Aa]xis*} $inducerToolArg($Tool)]} {
	set arg $Device
	set len [string length $arg]
	incr len -1
	set arg [string range $arg $len end] ;# extract number from devName
	inducerPuts "$Tool ==> unit-number $arg"
    } else {
	set arg ""
	inducerPuts "$Tool ==> no arg (unknown)"
    }

    return $arg
}


# ___oOo___
