#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: inducerLine.tcl,v 1.65 2003/12/17 19:26:26 vltsccm Exp $" 
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 23.03.1995 created
# imunoz     13.04.1996 itcl2.0 port
#

#************************************************************************
#   NAME
#
#	InducerLine - class for inducer Line object for LCU connections
# 
#   SYNOPSIS
#
#	InducerLine <object-name> ?-cPeriod <ticks>?
#
#   DESCRIPTION
#
#	Objects of the class InducerLine are used to connect from a 
#	workstation Tcl-application - e.g. a Test Access Panel - to
#	the inducerServer task running on a LCU environment.
#
#	Several methods are provided to open/close a channel to a 
#	LCU device and to send commands to it via ioctl.
#
#	A monitor facility to report changes in a device status is
#	also available, that invokes a user-script each time a change
#	is detected in the device.
#
#
#   OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<object-name> configure -<Option> <Value> ...
#
#	The following options are publicly available:
#
#	-cPeriod <ticks>
#
#		The monitor-task on the LCU started by MonitorStart
#		polls the device periodically with the given interval.
#		The lower the value, the faster the update, but the
#		higher the load of the LCU.
#		Reasonable values are 20 to 100 ticks, default is 50.
#
#
#   METHODS
#	Methods represent procedures on the object and are invoked with:
#
#		<object-name> <Method> <Arguments> ...
#
#	The following methods are publicly available:
#
#	Open { Environment Device Mode } ==> FileDes
#
#		Open a connection to `Device' on `Environment' in the
#		open-mode given by `Mode' which can be:
#			TEST, EXCLUSIVE, SHARED, READONLY
#		The file-descriptor used on the LCU is returned and stored
#		inside the object for later access (-1 if open failed).
#		Only one connection can be opened by the object at a time.
#		Any already opened connection is automatically closed before
#		opening the new one.
#
#	Close	
#		Close the previously opened connection to the LCU.
#		No action if no channel was opened.
#
#	Ioctl { Cmdlist Arglist ?Sizlist? } ==> Result
#
#		send the commands in `Cmdlist', each one with each
#		argument contained in `Arglist' through the
#		previously opened channel.
#
#		Commands may be given either as numbers or driver-
#		literals without the driver-specific prefix,
#		e.g. "READ_BITS" (not "acroCMD_READ_BITS").
#		Arguments must be given as one integer or double or as
#		semicolon-separated values if more than one is needed,
#		for a specific driver-command, e.g. "3;-4553;0;33".
#		Each number corresponds to a struct-member.
#
#		The `Result' consists of a comma-separated list of
#		all the return-values of the executed ioctl-calls.
#		The format of each list-element corresponds to the
#		format of the respective argument, unless the optional
#		`Sizlist' is given. Then the number of struct-members
#		to be returned can be explicitly specified, where
#		the index in Sizlist corresponds to the index in 
#		Cmdlist (very complicated, but fortunately rarely used).
#		
#		Errors reported by the called driver are ignored.
#		Only the number of occured errors is returned as last
#		list-element in `Result'. Detailed information about
#		any errors is not provided.
#
#	IoctlTrap { Cmdlist Arglist ?Sizlist? } ==> Result
#
#		as before, but if the called driver reports an error,
#		then a dialog-box is exposed for acknowlegement.
#
#	MonitorStart { Cmdlist Arglist ?Sizlist? ?Script? }
#
#		start a monitor task on the LCU which periodically
#		compares the ioctl result. Everytime a change is
#		is detected, `Script' is executed with the new
#		ioctl-result as argument, in the same format as
#		returned by Ioctl. The parameters except for Script
#		are the same as in Ioctl.
#		The sample period is given by the option `cPeriod'.
#
#	MonitorStop
#		stop the monitor-task (see CAUTIONS).
#
#	MonitorCollector { Script Reply Errorflag Lastflag }
#
#		Used internally for collecting monitor-replies from the LCU.
#
#
#	Macro begin		Begin to record all IOCTL commands as macro
#	Macro end		Stop recording to terminate macro
#	Macro record <line>	Record into current macro
#	Macro exec		Execute (redo) current macro
#	Macro save		Save current macro to a selectable file
#	Macro load		Load current macro from a selectable file
#	Macro keepDelay 0|1	Keep original delays during "Macro exec"
#
#   PROCS
#
#	Send { Env Com ?Par? }	==> Reply
#
#		Send the command `Com' with optional parameters `Par'
#		to the inducerServer running on the LCU environment `Env'.
#		The `Reply' of this command is returned.
#		An error-reply is trapped with a dialog-box for confirmation
#		and an empty string is returned in that case.
#		Note that multiple replies are not handled!
#
#   COMMONS
#
#	cTimeout <milliseconds>
#
#		Timeout value used for message-receive in proc `Send'.
#		Default is 5000 (5 seconds).
#
#   FILES
#	The current macro is saved in "~/.[file tail $argv0].macro".
#
#   EXAMPLES
#
#   CAUTIONS
#	The MonitorStop method stops the monitor-task on the LCU in an 
#	asynchronous fashion. It is not verified that the task has actually
#	terminated.
#
#   SEE ALSO
#	InducerClass(n), InducerDevice(n), inducerServer(1)
#
#   TODO
#	Read and Write methods.
#
#------------------------------------------------------------------------


#------------------------------------------------------------------------
#*# class: InducerLine

class InducerLine {
    
    inherit VccDebug 
    #*# constructor
    constructor {args} { eval configure $args}
    
    #--------------------------------------------------------------------
    #*# method: configure
    
    protected variable 	vEnv ""	  ;# Environment
    protected variable 	vDev ""	  ;# Device
    protected variable 	vMode ""  ;# OpenMode
    protected variable 	vFd -1	  ;# FileDescriptor (<0: not opened)
    protected variable 	fMonitor 0;# Monitor-flag (0: not active, 1:active)
    public variable  cPeriod 50	  ;# LCU monitoring period in ticks (->man-page!)
    common   cTimeout 5000 ;# Timeout of msgRecvReply in ms  (->man-page!)
    common   nServer inducerServer ;# name of LCU server task


    proc Send { Env Com {Par ""} } {
	set cmdId [seq_msgSendCommand $Env $nServer $Com $Par]
	set reply [seq_msgRecvReply $cmdId errorNr lastReply -$cTimeout -all]
	inducerPuts "InducerLine Send $Env $nServer $Com $Par ==> $reply"
	if {$errorNr != 0} {
	    ::inducerDialog "Error-reply from $Env/$nServer in response to\
		    \"$Com $Par\":\n$reply"
	    return ""
	}
	return $reply
    }

    # alternative version from IMU with catch, not used for now
    proc _SendCatch { Env Com {Par ""} } {

	# 21/06/95
	# to trap errors when the LCU goes down
	#
	set reply ""

	catch {set cmdId [seq_msgSendCommand $Env $nServer $Com $Par]}
	if {! [catch {set reply [seq_msgRecvReply $cmdId errorNr lastReply -$cTimeout]}] } {
	if {$errorNr != 0} {
	    ::inducerDialog "Error-reply from $Env/$nServer in response to\
		    \"$Com $Par\":\n$reply"
	    return ""
	}
    }
	return $reply
    }

    public method Open { Environment Device Mode } { }
    public method Close { } {}
    public method Ioctl { Cmdlist Arglist {Sizlist ""} } {}
    public method IoctlTrap { Cmdlist Arglist {Sizlist ""} } {}
    public method MonitorStart { Cmdlist Arglist {Sizlist ""} {Script ""} } {}
    public method MonitorCollector { Script Reply Errorflag Lastflag } {}
    public method MonitorStop { } {}
    
#	Exec { Function ?args? }
#
#		Execute `Function' on the LCU and return its standard-output
#		and its return-value. See CDT information of inducerServer,
#		EXEC command for more.
#
    # Exec unused at the moment!
    public method Exec { Function args } {}
    # --------------------------------------------------
    # Macros
    # --------------------------------------------------

    protected variable macroFd {}	;# descriptor of open macro file, or {}
    protected variable macroTime 0	;# time of last interaction in `getclock' fmt
    protected variable macroDelayKey <delay> ;# keyword used to record macro delays
    protected variable macroKeepDelay 0	;# keep delay between interactions on exec
    
    public method Macro { Operation args } {}
}

body InducerLine::Open { Environment Device Mode } {
    # first close any existing opened channel:
    Close
    
    set vEnv $Environment
    set vDev $Device
    set vMode $Mode
    set fd [Send $Environment OPEN $Device,$Mode]
    if {$fd == "" || $fd < 0} {set fd -1}
    set vFd $fd
    debugPrint "fd = $vFd"
    return $fd
}

body InducerLine::Close { } {
    # if there was an opened channel, close it:
    if {$vFd >= 3} {
	debugPrint "fd = $vFd"
	Send $vEnv CLOSE $vFd
	set vFd -1
    }
}

body InducerLine::Ioctl { Cmdlist Arglist {Sizlist ""} } {
    set fd $vFd
    if {$fd < 0} {return 0} ;# return fake zero-error result

    # translate all commands-literals into numbers:
    set numlist [::inducerGetIoctlCmdNum $Cmdlist]

    # send ioctl command and format returned result:
    set r [Send $vEnv IOCTL "$fd,$numlist,$Arglist,$Sizlist"]
    debugPrint "$r"

    # record in macro when enabled:
    Macro record IOCTL "$numlist,$Arglist,$Sizlist"

    return [split $r ,]
}

body InducerLine::IoctlTrap { Cmdlist Arglist {Sizlist ""} } {
    set res [Ioctl $Cmdlist $Arglist $Sizlist]
    # extract the number of occured driver errors (last list-element):
    set nerr [lindex $res [expr {[llength $res]-1}]]
    if {$nerr != 0} {
	::inducerDialog "Operation is rejected by the driver:\
		\nIn: $Cmdlist $Arglist\
		\nOut: $res"
    }
    return $res
}


body InducerLine::MonitorStart { Cmdlist Arglist {Sizlist ""} {Script ""} } {
    set fd $vFd
    if {$fd < 0} {return 0} ;# return fake zero-error result

    # translate all commands-literals into numbers:
    set numlist [::inducerGetIoctlCmdNum $Cmdlist]

    # setup the monitoring period:
    Send $vEnv SETMON $fd,$cPeriod

    # send MONITOR command:
    set cmdId [seq_msgSendCommand $vEnv $nServer\
	    MONITOR "$fd,$numlist,$Arglist,$Sizlist"]

    set fMonitor 1

    # collect all replies in background, process Script on each reply:
    seq_evtAttach $cmdId 0 \
	    "$this MonitorCollector [list $Script] %R %E %L"
    
    debugPrint "running: cmdId = $cmdId"
}

body InducerLine::MonitorCollector { Script Reply Errorflag Lastflag } {
    debugPrint
    # evaluate script, not for error-replies:
    # NOTE that Script can consist of multiple words and Reply can contain
    #	semicolons, which must be considered for the eval statement.
    #	Reply is as returned from IOCTL, it must still be converted to
    #	a list.
    if {$Errorflag == 0 && $Script != ""} {
	eval $Script [list [split $Reply ,]]
    }
    # check if the termination has been requested, warning if not:
    if {$Lastflag} {
	if {$fMonitor} {
	    ::inducerDialog "Monitor task has terminated abnormally!\
		    Select Device again to restart the task."
	}
    }
}

body InducerLine::MonitorStop { } {
    debugPrint
    if {$vFd < 0} {return}

    set fMonitor 0

    # send the stop command to the LCU:
    Send $vEnv SETMON $vFd,0

    # wait until the task has recognized it:
    set tickrate [Send $vEnv GETTICK] ;# query ticks per second
    after [expr {$cPeriod*$tickrate/5}] ;# == 2 times the monitor-period

    #!! NOTE: This is to handle the last reply from the monitor process!
    #!! Otherwise an asynchronous reply comes in after current operation.
    update
}

#	Exec { Function ?args? }
#
#		Execute `Function' on the LCU and return its standard-output
#		and its return-value. See CDT information of inducerServer,
#		EXEC command for more.
#
# Exec unused at the moment!
body InducerLine::Exec { Function args } {
    if {$vFd < 0} {return ""}

    # send EXEC command and return reply:
    set output [Send $vEnv EXEC $Function,[join $args ,]]
    debugPrint $output
    return $output
}

#----------------------
# Macros
#----------------------
body InducerLine::Macro { Operation args } {
    global argv0
 
    set macrofile ~/.[file tail $argv0].macro

    switch -- $Operation {
	begin {
	    set macroFd [open $macrofile w]
	    set macroTime [getclock]
	    puts $macroFd "# $macrofile"
	    puts $macroFd "# recorded: [fmtclock $macroTime]"
	}
	end {
	    set macroFd [close $macroFd]
	}
	record {
	    if {[lempty $macroFd]} return
	    puts $macroFd "$macroDelayKey [expr {[getclock]-$macroTime}]"
	    set macroTime [getclock]
	    puts $macroFd $args
	}
	exec {
	    if {$vFd < 0} return
	    # read current macrofile and send each line:
	    set fd [open $macrofile r]
	    set lines [read -nonewline $fd]
	    close $fd
	    set nLines 0
	    foreach line [split $lines \n] {
		if {[cindex $line 0] == "#"} continue
		set cmd [lindex $line 0]
		set arg [lindex $line 1]
		if {$cmd == $macroDelayKey} {
		    if {$macroKeepDelay} {sleep $arg}
		} else {
		    if {! [lempty $args]} {eval $args $line}
		    Send $vEnv $cmd $vFd,$arg
		    update
		    incr nLines
		}
	    }
	    return $nLines
	}
	save {
	    if {! [lempty $macroFd]} {Macro end} ;# force close before
	    # open a file-browser to select a file, abort when empty:
	    set filename [::panSelectFile . *.ipm]
	    if {$filename == ""} return
	    write_file $filename [read_file -nonewline $macrofile]
	    return $filename
	}
	load {
	    if {! [lempty $macroFd]} {Macro end} ;# force close before
	    # open a file-browser to select a file, abort when empty:
	    set filename [::panSelectFile . *.ipm]
	    if {$filename == ""} return
	    write_file $macrofile [read_file -nonewline $filename]
	    return $filename
	}
	keepDelay {
	    set macroKeepDelay $args
	}
    }
}

#___oOo___
