#*************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ist3State.tcl,v 1.49 2004/01/28 09:35:30 vltsccm Exp $" 
#
# ist3State.tcl
#
#
# who        when      what
# ---------  --------  ----------------------------------------------
# therlin    30/04/96  Taken from ISAAC isi3State and converted to incr2
# therlin    13/05/96  Removed additonal private variables
# therlin    20/08/96  Fixed bug in now method
# therlin    26/08/96  Added -orient option
# therlin    19/11/96  Added -index option
# therlin    18/03/97  Added -selectVariable and -acceptVariable
# therlin    23/05/97  Remove -variable option for backwards compatibility
# therlin    06/11/97  Support for database status feedback SPR970321
# pbiereic   30/07/99  Uses istNamespaceTail
# pbiereic   23/07/02  method select: do not expand keyword data
#
#************************************************************************
#   NAME
#
#   ist3State - three state button area widget for controlling element functions
#
#   SYNOPSIS
#
#   ist3State  <widget-name>
#
#   DESCRIPTION
#
#   ist3State is a mega widget which defines a three state button area.
#   The ist3State can be used for instrumentation panel which control an
#   instrument function such as a filter wheel. 
#
#   The ist3State widget defines a set of elements which are controled by
#   buttons. The individual buttons have three states: unselected, selected
#   and accepted.
#   In unselected state the element button looks like other default buttons.
#   In selected state the button color changes (default skyBlue) and font
#   changes. Only one element can be selected at a time. The selection is
#   done by pressing the button. A selection is equal to a setup parameter.
#   If a selection variable is passed (-selectVariable) this variable is
#   updated with the current selection.
#
#   In accpeted state the button color (default gold) changes and the font 
#   is larger then the other states in order to clearly show what the current
#   status of the element is. If a accept variable is passed (-selectVariable)
#   this variable is updated with the current accepted value.
#
#   Callbacks can be specified in order to control the selection. 
#
#
#   The title of the ist3State button area is also a button which can be used
#   to activate a function which could be a dialog showing more details of a 
#   function.
#
#  
#   The user create a widget of this class (called instance) by :
#
#     ist3State  <widget-name> <config-parameters>
#   where :
#
#     <widget-name>         is any valid TK widget name
#     <config-parameters>   sequence of widget resources
#
#   Standard widget resources can be set using :
#
#      <widget-name> configure <resource-name> <value>
#
#   The current value of the widget resources is returned by :
#
#      <name-of-instance> cget <resource-name>
#
#
#   METHODS
#
#     select <no>
#        Changes the state of an element numbert <no> to selected.
#
#     accept
#        Changes the state of the selected element to accepted.
#
#     clear
#        Clear the selection.
#
#     now     <no>
#        Indicates the temporay position of a moving functions.
#        Feature not supported yet !
#
#     selectItem
#        Returns the currently selected item text string.
#
#     selectNo
#        Returns the currently selected item number.
#
#     keyword
#        Returns the short FITS keyword of the element function.
#
#     accepted
#        Returns the currently accepted item number.
#
#     disable
#        Disable the selection of element buttons.
#
#     enable
#        Enable the selection of element buttons.
#      
#   PUBLIC VARIABLES
#
#     elements       <list of element names> 
#
#     current        <element name>           
#
#     label          <function name>          
#
#     keyword        <short FITS keyword> 
#
#     selectCB       <callback function> - function has 2 parameters: keyword value
#
#     detailCB       <callback function> - function has no parameters
#
#     selectBg       <color>  - rgb color e.g. blue
#
#     selectFont     <font>   - X11 font name 
#
#     acceptBg       <color>  - named color see rgb.txt
#
#     acceptFont     <font>   - see above.   
#
#     labelBg        <color>  - see above.    
#
#     labelFg        <color>  - see above.     
#
#     labelFont      <font>   - see above.                
#
#     nowBg          <color>  - see above.        
#
#     nowFont        <font>   - see above.        
#
#     orient         <h/v>    - h = horizontal v = vertical layout
#
#     index          <number> - start of numbering index default 0
#
#     selectVariable <var>    - variable set with selected value
#
#     acceptVariable <var>    - variable set with accepted value 
#
#     lblWidth       <width>  - label width in number of characters
#
#     statusDb       <attr>   - database point and attribute name
#
#
#   EXAMPLES
#
#     Create an instance
#
#     ist3State   .myFunction  -label myFunction -current Open \
#                         -elements {Open Closed J K} -keyword "INS.OPTI-1" \
#                         -selectCB funcSelect
# 
#     proc funcSelect { keyword value } {
#          puts "Function: $keyword has value: $value"
#     }
#
#   SEE ALSO
#
#------------------------------------------------------------------------
#
#--------------------------------------------------------------
itk::usual ist3State {
 #
 #  NB !!!!!!!!!!!
    keep -relief -borderwidth -background -foreground
}

class ist3State { 
    
    inherit UifInteractiveWidget

    constructor {args} {}
    destructor  {}

    itk_option define -elements       elements       Elements      "a b c d"
    itk_option define -current        current        Current       ""
    itk_option define -label          label          Label         "Function"
    itk_option define -keyword        keyword        Keyword       ""
    itk_option define -acceptVariable acceptVariable AcceptVariable      ""
    itk_option define -selectVariable selectVariable SelectVariable      ""
    itk_option define -selectCB       selectCB       SelectCB      ""
    itk_option define -detailCB       detailCB       DetailCB      ""

    itk_option define -selectBg       selectBg       SelectBg      "skyBlue"
    itk_option define -selectFont     selectFont     SelectFont    "-Adobe-times-bold-i-normal--*-77-*"
    itk_option define -acceptBg       acceptBg       AcceptBg      "gold"
    itk_option define -acceptFont     acceptFont     AcceptFont    "-Adobe-times-bold-i-normal--*-128-*"
    itk_option define -nowBg          nowBg          NowBg         "red"
    itk_option define -nowFont        nowFont        NowFont       "-Adobe-times-bold-i-normal--*-77-*"
    itk_option define -labelBg        labelBg        LabelBg       "#9f9bbf"
    itk_option define -labelFg        labelFg        LabelFg       "White"
    itk_option define -labelFont      labelFont      LabelFont     "-Adobe-helvetica-bold-r-normal--14*"
    itk_option define -orient         orient         Orient        "v"
    itk_option define -index          index          Index         0
    itk_option define -lblWidth       lblWidth       LblWidth      0
    itk_option define -statusDb       statusDb       StatusDb      ""

    public method  select        { no }
    public method  accept        {}
    public method  clear         {}
    public method  now           { no }
    public method  selectItem    {}
    public method  selectNo      {}
    public method  selectKeyword {}
    public method  keyword       {}
    public method  accepted      {}
    public method  disable       {}
    public method  enable        {}
    public method  detail        {}
    public method  newLabel      { label }
    public method  statusDbUpdate { attr data }

    private method  clearElements { }
    private method  updateUserVar { }
    
    private variable cnt         0
    private variable selected    -1
    private variable accepted    -1
    private variable rightNow    -1
    private variable istData
    private variable normalFont 
    private variable normalBg
    private variable currOrient   v
    private variable statusDbEvt  ""

}; # end of class

#//------------------------------------------------------
#// Constructor
#//------------------------------------------------------
body ist3State::constructor {args} {

    set wthis    [istNamespaceTail $this]
    
    itk_component add btnFrame {
	frame  $itk_interior.btnFrame -relief raised -bd 2 
    } {
	usual
    }

    #// Label acts a heading title
    itk_component add label {
       button  $itk_component(btnFrame).label  -text "test" \
	       -command "$wthis detail" -relief flat -padx 10 
   }

   #// Get defaults from a dummy widget
   button  .dummy
   set normalFont [lindex [.dummy configure -font] 4]
   set normalBg [lindex [.dummy configure -bg] 4]
   destroy .dummy

   #puts "using $normalFont $normalBg"
      
   pack $itk_component(label) -anchor c -fill x -expand 1

   pack $itk_component(btnFrame) -fill x -expand 1

   eval itk_initialize 

   if {[llength $args] != 0 } { 
       #// For compatibility remove the -variable option
       if {[set idx [lsearch $args "-variable"]] != -1} {
	   set args [lreplace $args $idx [expr {$idx +1}]]
       }
       
       if {[set idx [lsearch $args "-index"]] != -1} {
	   configure -index [lindex $args [expr {$idx +1}]]
	   set args [lreplace $args $idx [expr {$idx +1}]]
       }
       eval configure $args 
   }

}
  
  
#---------------------------------------------------------------
# METHOD : select
#---------------------------------------------------------------
body ist3State::select { no } {      
    
    set normalBg  [lindex [$this configure -background] 4]

    #// subtract index
    if { $no >= 0 } { set no [expr {$no - $itk_option(-index)}] }

    #puts "number $no selected"
    if { $no >= $cnt || $no <  0} { set no -1 }
    #// clear the now position
    now -1

    #puts "number $no selected"
    if { $selected != -1 && $selected != $accepted } {
	$itk_component(btn_$selected) configure -relief raised \
		-font $normalFont -bg $normalBg
    }
    if { $no != -1 && $no != $accepted } {
	$itk_component(btn_$no) configure -relief sunken \
		-font $itk_option(-selectFont)  -bg  $itk_option(-selectBg)
    }
    
    if { $accepted != -1 } {
	if { $no == $accepted } {
	    $itk_component(btn_$no) configure -bg $itk_option(-selectBg)
	} else {
	    $itk_component(btn_$accepted) configure -bg $itk_option(-acceptBg)
	}
    }
    #// Treat no -1 settings as a reset of the current accepted value
    if { $no == -1 && $accepted != -1} {
	$itk_component(btn_$accepted) configure -relief raised \
		-font $normalFont -bg $normalBg
	set accepted $no
    }
    set selected $no  
      
    # update user function
    if { $selected != -1 && $itk_option(-selectCB) != "" } { 
	eval "$itk_option(-selectCB)  $itk_option(-keyword) \$istData(${no}_text)" 
    }

    updateUserVar
}

#---------------------------------------------------------------
# METHOD : accept
#---------------------------------------------------------------

body ist3State::accept { } {
    #puts "accept called: $accepted $selected"
    set normalBg  [lindex [$this configure -background] 4]
    if {$selected == -1} return
    
    if {$accepted != -1} {
	$itk_component(btn_$accepted) configure -relief raised \
		-font $normalFont -bg $normalBg
    }
    set accepted $selected
    
    if {$accepted != -1} {
	$itk_component(btn_$accepted) configure -relief sunken \
		-font $itk_option(-acceptFont) -bg $itk_option(-acceptBg)
    }
    set selected -1
    
    updateUserVar
    #now -1
}; # end of accept


#---------------------------------------------------------------
# METHOD : clear
#---------------------------------------------------------------
body ist3State::clear { } {
    set normalBg  [lindex [$this configure -background] 4]
    if {$selected != -1} {
	if {$selected == $accepted} {
	    $itk_component(btn_$accepted) configure -relief sunken \
		    -font $itk_option(-acceptFont) -bg $itk_option(-acceptBg)
	} else {
	    $itk_component(btn_$selected) configure -relief raised \
		    -font $normalFont -bg $normalBg
	}
	set selected -1
    }
    
    updateUserVar
    #now -1
}; # end of clear

#---------------------------------------------------------------
# METHOD : now
#---------------------------------------------------------------

body ist3State::now { no } {
    set normalBg  [lindex [$this configure -background] 4]
    #puts "currently now is $no "
    if { $no >= $cnt } { set no -1 }

    #// subtract index
    if { $no >= 0 } { set no [expr {$no - $itk_option(-index)}] }

    #// clear previous setting
    if {$rightNow != -1} {
	$itk_component(btn_$rightNow) configure -relief raised \
		-font $normalFont -bg $normalBg
    }
    
    if {$no == -1} { return  }
    #// if selected position reached this is the same as accepted
    if { $no == $selected } { 
	accept
	set rightNow -1 
	return
    }
    if { $no != $accepted && $no != $selected && $no >= $cnt } {
	$itk_component(btn_$no) configure -relief sunken \
		-font $itk_option(-nowFont)  -bg  $itk_option(-nowBg)
	set rightNow $no  
    }    
}; # end of now

#---------------------------------------------------------------
# METHOD : selectItem
#---------------------------------------------------------------
body ist3State::selectItem { } {

    if {$selected != -1} {
	return "$istData(${selected}_text)"
    }
}; # end of selectItem
   
#---------------------------------------------------------------
# METHOD : selectNo
#---------------------------------------------------------------
body ist3State::selectNo { } { 
    return [expr {$selected + $itk_option(-index)}]
}; # end of selectNo
 

#---------------------------------------------------------------
# METHOD : selectKeyword
#---------------------------------------------------------------
body ist3State::selectKeyword { } {

    if {$selected != -1} {
	return "$itk_option(-keyword) $itk_component(${selected}_text)"
    } else {
	return ""
    }
}; # end of selectKeyword

#---------------------------------------------------------------
# METHOD : keyword
#---------------------------------------------------------------
body ist3State::keyword { } {
    return $itk_option(-keyword)
}; # end of Keyword

#---------------------------------------------------------------
# METHOD : keyword
#---------------------------------------------------------------
body ist3State::accepted { } {
    return [expr {$accepted - $itk_option(-index)}]
}; 
   

#---------------------------------------------------------------
# METHOD : disable
#---------------------------------------------------------------
body ist3State::disable { } {   

    for { set i 0 } { $i < $cnt } { incr i } {
	$itk_component(btn_${i}) configure -state disabled
    }
}; # end of disable


#---------------------------------------------------------------
# METHOD : enable
#---------------------------------------------------------------

body ist3State::enable { } {   

    for { set i 0 } { $i < $cnt } { incr i } {
	$itk_component(btn_${i}) configure -state normal
    }
}; # end of disable



#---------------------------------------------------------------
# METHOD : detail
#---------------------------------------------------------------
body ist3State::detail { } {
    
    if { $itk_option(-detailCB) != "" } { 
	eval "$itk_option(-detailCB)" 
    }
}; # end of detail

#---------------------------------------------------------------
# METHOD : newLabel
#---------------------------------------------------------------

body ist3State::newLabel { label } {
    if { $label != "" } { 
	$itk_component(label) configure -text $label 
    }
}; # end of newLabel


#---------------------------------------------------------------
# METHOD : clearElements
#---------------------------------------------------------------
body ist3State::clearElements { } {
    if { $cnt != 0 } { 
	for { set i 0 } { $i < $cnt } { incr i } {
	    destroy $itk_component(btn_$i)
	}
	set cnt 0
    }
}; # end of clearElements


#---------------------------------------------------------------
# METHOD : updateUserVar
#---------------------------------------------------------------
body ist3State::updateUserVar { } {
    
    # update user select variable
    if { $itk_option(-selectVariable) != "" } { 
	set sglobal [lindex [split $itk_option(-selectVariable) "\("] 0]
	eval global ::$sglobal
	# remember to add an offset
	if  { $selected == -1 } {
	    set $itk_option(-selectVariable) ""
	} else {
	    set $itk_option(-selectVariable) \
		[expr {$selected + $itk_option(-index)}]
	}
    }

    # update user accept variable
    if { $itk_option(-acceptVariable) != "" } { 
	set aglobal [lindex [split $itk_option(-acceptVariable) "\("] 0]
	eval global ::$aglobal
	# remember to add an offset
	if  { $accepted == -1 } {
	    set $itk_option(-acceptVariable) ""
	} else {
	    set $itk_option(-acceptVariable) \
		[expr {$accepted + $itk_option(-index)}]
	}
    }
    
}; # end of  updateUserVar


#---------------------------------------------------------------
# METHOD : 
#---------------------------------------------------------------
body  ist3State::statusDbUpdate {attr data} {

    set value [lindex $data 2]
    #puts "ist3State::statusDbUpdate $attr - $value"

    select $value
    accept
} 
#-------------------------------------------
# PUBLIC VARIABLES
#-------------------------------------------

#---------------------------------------------------------------
# OPTION : -elements
#--------------------------------------------------------------- 
configbody ist3State::elements {

    set wthis    [istNamespaceTail $this]
    set elements $itk_option(-elements)

    clearElements
    
    foreach btn $elements {
	itk_component add btn_$cnt {
	    button  $itk_component(btnFrame).btn_$cnt -text $btn \
		    -relief raised -command "$wthis select [expr $cnt + $itk_option(-index)]" 
	}
	if { [string compare $btn $itk_option(-current)] == 0 && "$itk_option(-current)" != "" } { 
	    #puts "Setting button $cnt $btn to accepted $itk_option(-acceptBg)"
	    $itk_component(btn_$cnt) configure  -relief sunken \
		    -font $itk_option(-acceptFont) -bg $itk_option(-acceptBg) 
	    set accepted $cnt
	} 
	$itk_component(btn_$cnt) configure  -highlightthickness 0
	set istData(${cnt}_text) $btn
	pack $itk_component(btn_$cnt) -fill x -padx 0 -pady 0
	incr cnt
    }
}

#---------------------------------------------------------------
# OPTION : -current
#--------------------------------------------------------------- 
configbody ist3State::current {
    set current $itk_option(-current)
    if { $cnt != 0 } { 
	for { set i 0 } { $i < $cnt } { incr i } {
	    if { "$istData(${i}_text)" == "$itk_option(-current)" } {
		#puts "Setting  button in ist3State::current"
		$itk_component(btn_${i}) configure -relief sunken \
			-font $itk_option(-acceptFont) \
			-bg   $itk_option(-acceptBg)
		set accepted $i
	    }
	}
    }
}

#---------------------------------------------------------------
# OPTION : -index
#--------------------------------------------------------------- 
configbody ist3State::index {
    #puts "setting index to $itk_option(-index)"
}

#---------------------------------------------------------------
# OPTION : -lblWidth
#--------------------------------------------------------------- 
configbody ist3State::lblWidth {
    #puts "setting lblWidth to $itk_option(-lblWidth)"
    if { $itk_option(-lblWidth) } {
	$itk_component(label) configure -width  $itk_option(-lblWidth)
    }
}
#---------------------------------------------------------------
# OPTION : -label
#--------------------------------------------------------------- 
configbody ist3State::label {
    $itk_component(label) configure -text $itk_option(-label)
}

#---------------------------------------------------------------
# OPTION : -keyword
#--------------------------------------------------------------- 
configbody ist3State::keyword {
    set keyword $itk_option(-keyword)
}

#---------------------------------------------------------------
# OPTION : -selectCB
#--------------------------------------------------------------- 
configbody ist3State::selectCB {
    set selectCB $itk_option(-selectCB)
}

#---------------------------------------------------------------
# OPTION : -detailCB
#--------------------------------------------------------------- 
configbody ist3State::detailCB {
    set detailCB $itk_option(-detailCB)
}

#---------------------------------------------------------------
# OPTION : -selectBg
#--------------------------------------------------------------- 
configbody ist3State::selectBg {
    set selectBg $itk_option(-selectBg)
}

#---------------------------------------------------------------
# OPTION : -selectFont
#--------------------------------------------------------------- 
configbody ist3State::selectFont {
    set selectFont $itk_option(-selectFont)
}

#---------------------------------------------------------------
# OPTION : -acceptBg
#--------------------------------------------------------------- 
configbody ist3State::acceptBg {
    set acceptBg $itk_option(-acceptBg)
}

#---------------------------------------------------------------
# OPTION : -acceptFont
#--------------------------------------------------------------- 
configbody ist3State::acceptFont {
    set acceptFont $itk_option(-acceptFont)
}

#---------------------------------------------------------------
# OPTION : -nowBg
#--------------------------------------------------------------- 
configbody ist3State::nowBg {
    set nowBg $itk_option(-nowBg)
}

#---------------------------------------------------------------
# OPTION : -nowFont
#--------------------------------------------------------------- 
configbody ist3State::nowFont {
    set nowFont $itk_option(-nowFont)
}

#---------------------------------------------------------------
# OPTION : -labelBg
#--------------------------------------------------------------- 
configbody ist3State::labelBg {
    $itk_component(label) configure -bg $itk_option(-labelBg)
}

#---------------------------------------------------------------
# OPTION : -labelFont
#--------------------------------------------------------------- 
configbody ist3State::labelFont {
    $itk_component(label) configure -font $itk_option(-labelFont)
}

#---------------------------------------------------------------
# OPTION : -labelFg
#--------------------------------------------------------------- 
configbody ist3State::labelFg {
    $itk_component(label) configure -fg $itk_option(-labelFg)
}

#---------------------------------------------------------------
# OPTION : -orient
#--------------------------------------------------------------- 
configbody ist3State::orient {

    set option "left"
    
    if {$itk_option(-orient) != $currOrient} {

	set widgets [lrange [pack slaves $itk_component(btnFrame)] 1 end]
	
	#puts "$itk_component(btnFrame) 3state Widgets: $widgets "

	if {$itk_option(-orient) == "h"} {
	    set option "left"
	}
	if {$itk_option(-orient) == "v"} {
	    set option "bottom"
	}
	foreach slave $widgets {
	    pack configure $slave -side $option -expand 1 
	}

	set currOrient $itk_option(-orient)

    }

}

#---------------------------------------------------------------
# OPTION : -selectVariable
#--------------------------------------------------------------- 
configbody ist3State::selectVariable {
    updateUserVar
}

#---------------------------------------------------------------
# OPTION : -acceptVariable
#--------------------------------------------------------------- 
configbody ist3State::acceptVariable {
    updateUserVar
}

#---------------------------------------------------------------
# OPTION : -statusDb
#--------------------------------------------------------------- 
configbody ist3State::statusDb {

    if {$itk_option(-statusDb) != ""} {
	if { $statusDbEvt != "" } { seq_evtDetach $statusDbEvt }
	
	if  {[catch {set statusDbEvt [seq_evtAttach $itk_option(-statusDb) NE \
					  "$wthis statusDbUpdate %A %D"]}]} {
	    puts "ist3State error ! Could not attach event to $itk_option(-statusDb)"
	} else {
	    #// update the current status
	    select [seq_dbReadSymbolic $itk_option(-statusDb)]
	    accept
	}
    }
}
# __o0o__
