#*************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ist3StateMenubutton.tcl,v 1.49 2004/01/28 09:35:32 vltsccm Exp $"
#
# ist3StateMenubutton.tcl
#
#
# who        when      what
# ---------  --------  ----------------------------------------------
# gzins      01/06/00  Created from ist3State widget - added -type option
# gzins      28/03/01  Fixed minor bug with selectCB callback when kerword
#                      or/and value is not defined or contains blanks.
#                      Added optional call to user function defined with
#                      -selectCB option.
# sskole     26/05/02  clearSelected: Clearing the menu button selection 
# sskole     21/06/02  Added option for menu button visibility
# dpopovic   14/01/04  namespace for tcl_version > 8.0 only
#                      
#
#************************************************************************
#   NAME
#   ist3StateMenubutton - three state button area widget for controlling
#                         element functions
#
#   SYNOPSIS
#   ist3StateMenubutton  <widget-name>
#
#   DESCRIPTION
#   IMPORTANT - The ist3StateMenubutton is a new version of the ist3State widget
#   providing the posibility to display the buttons either into a row/colunm
#   table (like ist3State) or into a menu widget. This feature is controlled by
#   the -type option.  The ist3StateMenubutton is fully compatible with the
#   ist3State, and should replace this one in the future development. Due to the
#   important changes needed to support the '-type' option, it has been decided
#   to implement a new widget instead of upgrading the existing one, in order to
#   avoid any risk of bugs for the existing applications.
#   In addition, the '-showLabel' and '-showCurrent' options have been added
#   and bugs in relation with the '-current' option and the selectKeyword
#   (renamed to valuedKeyword) methods have been fixed.
#
#   ist3StateMenubutton is a mega widget which defines a three state button
#   area. The ist3StateMenubutton can be used for instrumentation panel which
#   control an instrument function such as a filter wheel.
#
#   The ist3StateMenubutton widget defines a set of elements which are controled
#   with a list of buttons either placed into a table or into a menu.
#   In 'table' option (like ist3State widget), the individual buttons have three
#   states: unselected, selected and accepted. In unselected state the element
#   button looks like other default buttons. In selected state the button color
#   changes (default skyBlue) and font changes.  Only one element can be
#   selected at a time. The selection is done by pressing the button. A
#   selection is equal to a setup parameter. If a selection variable is passed
#   (-selectVariable) this variable is updated with the current selection.
#   In accepted state the button color (default gold) changes and the font
#   is larger than the other states in order to clearly show what the current
#   status of the element is. If a accept variable is passed (-acceptVariable)
#   this variable is updated with the current accepted value.
#
#   In 'menu' option, the individual buttons are selectable using a menu. The
#   selected element is displayed in menu button and the color and font change
#   (same as 'table' option). The selection is done by selecting the element in
#   the menu.
#   The accepted value is displayed in a additionnal button placed above the
#   menu. When a element is accepted, it is displayed in this button and the
#   color and font change (same as 'table' option).
#
#   Callbacks can be specified in order to control the selection.
#
#   The title of the ist3StateMenubutton button area is also a button which can
#   be used to activate a function which could be a dialog showing more details
#   of a function.
#
#
#   The user create a widget of this class (called instance) by :
#
#     ist3StateMenubutton  <widget-name> <config-parameters>
#   where :
#
#     <widget-name>         is any valid TK widget name
#     <config-parameters>   sequence of widget resources
#
#   Standard widget resources can be set using :
#
#      <widget-name> configure <resource-name> <value>
#
#   The current value of the widget resources is returned by :
#
#      <name-of-instance> cget <resource-name>
#
#
#   PUBLIC METHODS
#     select <no> <callUserFct>
#       Changes the state of an element numbert <no> to selected. 
#       If <callUserFct> is TRUE (default), the user fonction defined with
#       -selectCB option is called.
#
#     accept
#       Changes the state of the selected element to accepted.
#
#     clear
#       Clear the selection.
#
#     now     <no>
#       Indicates the temporay position of a moving functions.
#
#     selectItem
#       Returns the currently selected item text string.
#
#     selectNo
#       Returns the currently selected item number.
#
#     valuedKeyword
#       Return either an empty string when no item is selected or short FITS
#       keyword with current selected item name which could be used as parameter
#       of the SETUP command to setup the instrument.
#
#     keyword
#       Returns the short FITS keyword of the element function.
#
#     accepted
#       Returns the currently accepted item number.
#
#     disable
#       Disable the selection of element buttons.
#
#     enable
#       Enable the selection of element buttons.
#
#   WIDGET-SPECIFIC OPTIONS
#     -elements       <list of element names>
#
#     -current        <element name>
#
#     -label          <function name>
#
#     -showLabel      <0/1>    - show/hide the label
#
#     -keyword        <short FITS keyword>
#
#     -selectCB       <callback function> - function has 2 parameters: keyword
#                                           value
#
#     -detailCB       <callback function> - function has no parameters
#
#     -selectBg       <color>  - rgb color e.g. blue
#
#     -selectFont     <font>   - X11 font name
#
#     -acceptBg       <color>  - named color see rgb.txt
#
#     -acceptFont     <font>   - see above.
#
#     -showCurrent    <0/1>    - show/hide the current value (only when type
#                                menu is selected)
#     -showMenuButton <0/1>    - show/hide the menu button (only when type
#                                menu is selected)
#     -currentHelp    <help text> - short help text for the widget displaying
#                                the current value (only when type menu is
#                                selected)
#
#     -labelBg        <color>  - see above.
#
#     -labelFg        <color>  - see above.
#
#     -labelFont      <font>   - see above.
#
#     -labelHelp      <help text> - short help text for the label
#
#     -lblWidth       <width>  - label width in number of characters
#
#     -nowBg          <color>  - see above.
#
#     -nowFont        <font>   - see above.
#
#     -orient         <h/v>    - h = horizontal v = vertical layout
#
#     -type           <menu/table> - see above. By default 'table' to be
#                                    compatible with ist3State
#
#     -index          <number> - start of numbering index default 0
#
#     -selectVariable <var>    - variable set with selected value
#
#     -acceptVariable <var>    - variable set with accepted value
#
#     -buttonWidth    <width>  - button width in number of characters
#
#     -buttonHelp     <help text> - short help text for the menu button or
#                                the buttons of the table.
#
#   EXAMPLES
#
#     Create an instance
#
#     ist3StateMenubutton .myFunction  -label myFunction -current Open \
#                         -elements {Open Closed J K} -keyword "INS.OPTI-1" \
#                         -selectCB funcSelect -type menu
#
#     proc funcSelect { keyword value } {
#          puts "Function: $keyword has value: $value"
#     }
#
#   SEE ALSO
#
#------------------------------------------------------------------------
#

# Procedure to be removed when includes in ist library
proc istNamespaceTail {ns} {
    return [namespace tail $ns]
}

#--------------------------------------------------------------
itk::usual ist3StateMenubutton {
 #
 #  NB !!!!!!!!!!!
    keep -relief -borderwidth -background -foreground
}

class ist3StateMenubutton {

    inherit UifInteractiveWidget

    constructor {args} {}
    destructor  {}

    itk_option define -index          index          Index          0
    itk_option define -elements       elements       Elements       "a b c d"
    itk_option define -current        current        Current        ""
    itk_option define -label          label          Label          "Function"
    itk_option define -showLabel      showLabel      ShowLabel      1
    itk_option define -keyword        keyword        Keyword        ""
    itk_option define -acceptVariable acceptVariable AcceptVariable ""
    itk_option define -selectVariable selectVariable SelectVariable ""
    itk_option define -selectCB       selectCB       SelectCB       ""
    itk_option define -detailCB       detailCB       DetailCB       ""

    itk_option define -selectBg       selectBg       SelectBg       "skyBlue"
    itk_option define -selectFont     selectFont     SelectFont     "-Adobe-times-bold-i-normal--*-77-*"
    itk_option define -acceptBg       acceptBg       AcceptBg       "gold"
    itk_option define -acceptFont     acceptFont     AcceptFont     "-Adobe-times-bold-i-normal--*-128-*"
    itk_option define -nowBg          nowBg          NowBg          "yellow"
    itk_option define -nowFont        nowFont        NowFont        "-Adobe-times-bold-i-normal--*-128-*"
    itk_option define -showCurrent    showCurrent    ShowCurrent    1
    itk_option define -showMenuButton showMenuButton ShowMenuButton 1
    itk_option define -currentHelp    currentHelp    CurrentHelp    "Current status"
    

    itk_option define -labelBg        labelBg        LabelBg        "#9f9bbf"
    itk_option define -labelFg        labelFg        LabelFg        "White"
    itk_option define -labelFont      labelFont      LabelFont      "-Adobe-helvetica-bold-r-normal--14*"
    itk_option define -labelHelp      labelHelp      LabelHelp      "Press on title to get details"
    itk_option define -lblWidth       lblWidth       LblWidth       0

    itk_option define -orient         orient         Orient         "v"
    itk_option define -type           type           Type           "table"
    itk_option define -buttonWidth    buttonWidth    ButtonWidth    0
    itk_option define -buttonHelp     buttonHelp     ButtonHelp     "Click to select an other element"

    public method  select         {no {callUserFct 1}}
    public method  clear          {}
    public method  accept         {{no ""}}
    public method  now            {no}
    public method  selectItem     {}
    public method  selectNo       {}
    public method  keyword        {}
    public method  valuedKeyword  {}
    public method  accepted       {}
    public method  disable        {}
    public method  enable         {}
    public method  detail         {}
    public method  newLabel       {label}

    private method  clearSelected    {}
    private method  updateSelected   {newSelected}
    private method  clearCurrent     {}
    private method  updateCurrent    {newCurrent}
    private method  updateCurrentElt {{newElt ""}}
    private method  clearElements    {}
    private method  updateUserVar    {}
    private method  updateType       {}

    private variable wthis             ""
    private variable cnt               0
    private variable selected         -1
    private variable current          -1
    private variable acceptedFlag      0
    private variable normalFont
    private variable normalBg
    private variable currOrient        v

}; # end of class

#//------------------------------------------------------
#// Constructor
#//------------------------------------------------------
body ist3StateMenubutton::constructor {args} {

    set wthis    [istNamespaceTail $this]

    #// Get defaults from a dummy widget
    button  .dummy
    set normalFont [lindex [.dummy configure -font] 4]
    set normalBg [lindex [.dummy configure -bg] 4]
    destroy .dummy

    #// Create common widgets.
    itk_component add btnFrame {
        frame  $itk_interior.btnFrame -relief raised -bd 1
    } {
        usual
    }

    #// Label acts a heading title
    itk_component add label {
       button  $itk_component(btnFrame).label -relief flat -text "test" \
           -command "$wthis detail" -padx 10
    }

    #// Create widgets for menu button style.
    # Label for displaying current value
    itk_component add current {
       button  $itk_component(btnFrame).current  -text "----" \
           -command "$wthis detail" -relief sunken -padx 10
    }

    # Menubutton containing the menu.
    set menu $itk_component(btnFrame).menubutton.menu
    itk_component add menubutton {
        menubutton $itk_component(btnFrame).menubutton -menu $menu \
            -relief raised -font $normalFont -bg $normalBg -indicatoron 1
    }
    itk_component add menu {
        menu $menu
    }

    #// Pack common widgets
    pack $itk_component(label) -anchor c -fill x -expand 1

    pack $itk_component(btnFrame) -fill x -expand 1

    #// Evalute options
    eval itk_initialize

    if {[llength $args] != 0 } {
        #// For compatibility remove the -variable option
        if {[set idx [lsearch $args "-variable"]] != -1} {
            set args [lreplace $args $idx [expr {$idx +1}]]
        }

        if {[set idx [lsearch $args "-index"]] != -1} {
            configure -index [lindex $args [expr {$idx +1}]]
            set args [lreplace $args $idx [expr {$idx +1}]]
        }
        eval configure $args
    }
    $menu configure -tearoff 0
}

#---------------------------------------------------------------
# METHOD : select
#---------------------------------------------------------------
body ist3StateMenubutton::select { no {callUserFct 1} } {

    #// Subtract index
    if { $no >= 0 } { set no [expr {$no - $itk_option(-index)}] }

    if { $no >= $cnt || $no <  0} { set no -1 }

    #// Update the new selected button (if not the current one)
    updateSelected $no

    #// If no -1 then reset the current value
    if { $no == -1 && $current != -1} {
        clearCurrent
    }
    set selected $no

    #// Update user function
    if { $callUserFct == 1 } {
        set elements $itk_option(-elements)
        if { $selected != -1 && $itk_option(-selectCB) != "" } {
            eval "$itk_option(-selectCB)  \"$itk_option(-keyword)\" \
                \"[lindex $elements $no]\""
        }
    }

    updateUserVar
}

#---------------------------------------------------------------
# METHOD : clear
#---------------------------------------------------------------
body ist3StateMenubutton::clear { } {

    #// If a button is selected
    if {$selected != -1} {
        clearSelected
    }

    updateUserVar
}; # end of clear

#---------------------------------------------------------------
# METHOD : accept
#---------------------------------------------------------------
body ist3StateMenubutton::accept { {no ""} } {

    set normalBg  [lindex [$this configure -background] 4]

    #// If no button selected then return
    if {($no == "") && ($selected == -1)} return

    # Set the 'accepted' flag to TRUE
    set acceptedFlag 1
    if { $no != "" } {
        #// Subtract index
        if { $no >= 0 } { set no [expr {$no - $itk_option(-index)}] }

        if { $no >= $cnt || $no <  0} { set no -1 }
        set newCurrent $no
    } else {
        set newCurrent $selected
    }

    # Clear selected element
    clearSelected

    # Set the new current element
    updateCurrent $newCurrent

    # Update user variable
    updateUserVar

}; # end of accept

#---------------------------------------------------------------
# METHOD : now
#---------------------------------------------------------------
body ist3StateMenubutton::now { no } {
    #// Subtract index
    if { $no >= 0 } { set no [expr {$no - $itk_option(-index)}] }

    if { $no >= $cnt || $no <  0} { set no -1 }

    set acceptedFlag 0
    updateCurrent $no

}; # end of now

#---------------------------------------------------------------
# METHOD : selectItem
#---------------------------------------------------------------
body ist3StateMenubutton::selectItem { } {

    set elements $itk_option(-elements)
    if {$selected != -1} {
        return "[lindex $elements $selected]"
    } else {
        return ""
    }
}; # end of selectItem

#---------------------------------------------------------------
# METHOD : selectNo
#---------------------------------------------------------------
body ist3StateMenubutton::selectNo { } {
    return [expr {$selected + $itk_option(-index)}]
}; # end of selectNo


#---------------------------------------------------------------
# METHOD : valuedKeyword
#---------------------------------------------------------------
body ist3StateMenubutton::valuedKeyword { } {

    if {$selected != -1} {
        return "$itk_option(-keyword) [selectItem]"
    } else {
        return ""
    }
}; # end of valuedKeyword

#---------------------------------------------------------------
# METHOD : keyword
#---------------------------------------------------------------
body ist3StateMenubutton::keyword { } {
    return $itk_option(-keyword)
}; # end of Keyword

#---------------------------------------------------------------
# METHOD : keyword
#---------------------------------------------------------------
body ist3StateMenubutton::accepted { } {
    if { $acceptedFlag == 1 } {
        return [expr {$current + $itk_option(-index)}]
    } else {
        return -1
    }
};

#---------------------------------------------------------------
# METHOD : disable
#---------------------------------------------------------------
body ist3StateMenubutton::disable { } {

    ##### Table of buttons
    for { set i 0 } { $i < $cnt } { incr i } {
        $itk_component(btn_${i}) configure -state disabled
    }

    ##### Menu button
    $itk_component(menubutton) configure -state disabled
}; # end of disable


#---------------------------------------------------------------
# METHOD : enable
#---------------------------------------------------------------
body ist3StateMenubutton::enable { } {

    ##### Table of buttons
    for { set i 0 } { $i < $cnt } { incr i } {
        $itk_component(btn_${i}) configure -state normal
    }

    ##### Menu button
    $itk_component(menubutton) configure -state normal
}; # end of disable

#---------------------------------------------------------------
# METHOD : detail
#---------------------------------------------------------------
body ist3StateMenubutton::detail { } {

    if { $itk_option(-detailCB) != "" } {
        eval "$itk_option(-detailCB)"
    }
}; # end of detail

#---------------------------------------------------------------
# METHOD : newLabel
#---------------------------------------------------------------
body ist3StateMenubutton::newLabel { label } {
    if { $label != "" } {
        $itk_component(label) configure -text $label
    }
}; # end of newLabel

#---------------------------------------------------------------
# METHOD : clearSelected
#---------------------------------------------------------------
body ist3StateMenubutton::clearSelected { } {

    #// Set the color/font for the current element according to the 'accepted'
    #// flag
    set normalBg  [lindex [$this configure -background] 4]
    if { $acceptedFlag == 1 } {
        set currentBg   $itk_option(-acceptBg)
        set currentFont $itk_option(-acceptFont)
    } else {
        set currentBg   $itk_option(-nowBg)
        set currentFont $itk_option(-nowFont)
    }

    ##### Table of buttons
    #// Reset previous selected button (if necessary)
    if { $selected != -1 } {
        if {$selected != $current } {
            $itk_component(btn_$selected) configure -relief raised \
                -font $normalFont -bg $normalBg
        } else {
            $itk_component(btn_$selected) configure -relief sunken \
                -font $currentFont -bg $currentBg
        }
    }

    ##### Menu button
    $itk_component(menubutton) configure -relief raised \
        -font $normalFont -bg $normalBg

    if {$itk_option(-showCurrent) == 1} {
       $itk_component(menubutton) configure -text "" 
    }

    set selected -1
}

#---------------------------------------------------------------
# METHOD : updateSelected
#---------------------------------------------------------------
body ist3StateMenubutton::updateSelected {newSelected} {

    set normalBg  [lindex [$this configure -background] 4]
    set elements $itk_option(-elements)
    #// Reset previous selected button (if necessary)
    clearSelected

    ##### Table of buttons
    #// Update the new selected button (if not the current one)
    if { $newSelected != -1 && $newSelected != $current } {
        $itk_component(btn_$newSelected) configure -relief sunken \
            -font $itk_option(-selectFont)  -bg  $itk_option(-selectBg)
    }

    ##### Menu button
    #// Update the selected button
    if { $newSelected != -1 } {
        $itk_component(menubutton) configure -relief sunken \
            -text [lindex $elements $newSelected] \
            -font $itk_option(-selectFont) -bg $itk_option(-selectBg)
    }

    #// Update current value
    set selected $newSelected
    updateCurrent $current
}

#---------------------------------------------------------------
# METHOD : clearCurrent
#---------------------------------------------------------------
body  ist3StateMenubutton::clearCurrent {} {
    set normalBg [lindex [$this configure -background] 4]

    ##### Table of buttons
    if { $current != -1 } {
        if { $current != $selected } {
            $itk_component(btn_$current) configure -relief raised \
                -font $normalFont -bg $normalBg
        } else {
            $itk_component(btn_$current) configure -relief sunken \
                -font $itk_option(-selectFont) -bg $itk_option(-selectBg)
        }
    }

    ##### Menu button
    $itk_component(current) configure -text "----" \
        -font $normalFont -bg $normalBg

    set current -1
}

#---------------------------------------------------------------
# METHOD : updateCurrent
#---------------------------------------------------------------
body  ist3StateMenubutton::updateCurrent {newCurrent} {

    set normalBg [lindex [$this configure -background] 4]
    set elements $itk_option(-elements)

    #// Set the color/font for the current element according to the 'accepted'
    #// flag
    if { $acceptedFlag == 1 } {
        set currentBg   $itk_option(-acceptBg)
        set currentFont $itk_option(-acceptFont)
    } else {
        set currentBg   $itk_option(-nowBg)
        set currentFont $itk_option(-nowFont)
    }

    #// Reset the current value
    if {$newCurrent != $current } {
        clearCurrent
    }

    if { $newCurrent != -1 } {
        ##### Table of buttons
        #// Set the new current value
        if { $newCurrent != $selected } {
            $itk_component(btn_${newCurrent}) configure -relief sunken \
                -font $currentFont -bg $currentBg
        } else {
            $itk_component(btn_${newCurrent}) configure -relief sunken \
                -font $currentFont -bg $itk_option(-selectBg)
        }

        ##### Menu button
        $itk_component(current) configure \
            -text [lindex $elements $newCurrent] \
            -font $currentFont -bg $currentBg
    }

    set current $newCurrent
}


#---------------------------------------------------------------
# METHOD : updateCurrentElt
#---------------------------------------------------------------
body  ist3StateMenubutton::updateCurrentElt {{newElt ""}} {

    set normalBg [lindex [$this configure -background] 4]
    set elements $itk_option(-elements)
    if { $cnt != 0 } {
        #// Retrieves the index corresponding to the element
        set index [lsearch -exact $elements $newElt]

        #// If found in the list then update the current button
        if { $index != -1} {
            updateCurrent $index
        } else {
            #// If not found reset the current value
            clearCurrent
        }
    }
}

#---------------------------------------------------------------
# METHOD : clearElements
#---------------------------------------------------------------
body ist3StateMenubutton::clearElements { } {
    if { $cnt != 0 } {
        ##### Table of buttons
        for { set i 0 } { $i < $cnt } { incr i } {
            destroy $itk_component(btn_$i)
        }

        ##### Menu button
        $itk_component(menu) delete 0 end
    }
    set cnt 0
}; # end of clearElements

#---------------------------------------------------------------
# METHOD : updateUserVar
#---------------------------------------------------------------
body ist3StateMenubutton::updateUserVar { } {

    # update user select variable
    if { $itk_option(-selectVariable) != "" } {
        set sglobal [lindex [split $itk_option(-selectVariable) "\("] 0]
        eval global ::$sglobal
        # remember to add an offset
        if  { $selected == -1 } {
            set $itk_option(-selectVariable) ""
        } else {
            set $itk_option(-selectVariable) \
                [expr {$selected + $itk_option(-index)}]
        }
    }

    # update user accept variable
    if { $itk_option(-acceptVariable) != "" } {
        set aglobal [lindex [split $itk_option(-acceptVariable) "\("] 0]
        eval global ::$aglobal
        # remember to add an offset
        if  { ($acceptedFlag == 0) || ($current == -1) } {
            set $itk_option(-acceptVariable) ""
        } else {
            set $itk_option(-acceptVariable) \
                [expr {$current + $itk_option(-index)}]
        }
    }

}; # end of  updateUserVar

#---------------------------------------------------------------
# METHOD : updateType
#---------------------------------------------------------------
body  ist3StateMenubutton::updateType {} {

    if {$itk_option(-type) == "table"} {
        #// Manages table of buttons
        for { set i 0 } { $i < $cnt } { incr i } {
            pack  $itk_component(btn_$i) -fill x -padx 0 -pady 0
        }

        #// Umanages menu button
        pack forget $itk_component(current)
        pack forget $itk_component(menubutton)
    }
    if {$itk_option(-type) == "menu"} {
        #// Umanages table of buttons
        for { set i 0 } { $i < $cnt } { incr i } {
            pack forget $itk_component(btn_$i)
        }
        #// Manages menu button

        if { $itk_option(-showCurrent) == 1} {
            pack $itk_component(current) -fill x -expand 1
        }
	if { $itk_option(-showMenuButton) == 1} {
	    pack $itk_component(menubutton) -fill x -expand 1
	}
    }
}
#-------------------------------------------
# PUBLIC VARIABLES
#-------------------------------------------

#---------------------------------------------------------------
# OPTION : -elements
#---------------------------------------------------------------
configbody ist3StateMenubutton::elements {

    global gvar
    set wthis    [istNamespaceTail $this]
    set elements $itk_option(-elements)

    # Clear the menu button
    select -1
    clearElements

    ##### Table of buttons
    foreach btn $elements {
        itk_component add btn_$cnt {
            button $itk_component(btnFrame).btn_$cnt -text $btn -relief raised \
                -command "$wthis select [expr $cnt + $itk_option(-index)]" \
                -width $itk_option(-buttonWidth)
        }

        # Add help
        catch {UifShortHelp::add $itk_option(-helparea) \
               $itk_component(btn_$cnt) $itk_option(-buttonHelp)}

        $itk_component(btn_$cnt) configure  -highlightthickness 0

        incr cnt
    }

    ##### Menu button
    # Update the value displayed in menubutton.
    if { [llength $elements] != 0 } {
        $itk_component(menubutton) configure -text [lindex $elements 0]
    } else {
        $itk_component(menubutton) configure -text "----"
    }

    # Component for the menu.
    set cnt 0
    foreach btn $elements {
        $itk_component(menu) add command -label $btn -command \
            "$wthis select [expr {$cnt + $itk_option(-index)}]"
        incr cnt
    }

    # Set the current value if required
    if {"$itk_option(-current)" != ""} {
        updateCurrentElt
    }

    # Update widget
    updateType
}

#---------------------------------------------------------------
# OPTION : -current
#---------------------------------------------------------------
configbody ist3StateMenubutton::current {
    updateCurrentElt $itk_option(-current)
}

#---------------------------------------------------------------
# OPTION : -index
#---------------------------------------------------------------
configbody ist3StateMenubutton::index {
    $wthis configure -elements $itk_option(-elements)
}

#---------------------------------------------------------------
# OPTION : -lblWidth
#---------------------------------------------------------------
configbody ist3StateMenubutton::lblWidth {
    if { $itk_option(-lblWidth) } {
        $itk_component(label) configure -width  $itk_option(-lblWidth)
    }
}
#---------------------------------------------------------------
# OPTION : -buttonWidth
#---------------------------------------------------------------
configbody ist3StateMenubutton::buttonWidth {
    if { $itk_option(-buttonWidth) } {
        $itk_component(menubutton) configure -width  $itk_option(-buttonWidth)
        $itk_component(current) configure -width  $itk_option(-buttonWidth)
        for { set i 0 } { $i < $cnt } { incr i } {
            $itk_component(btn_$i) configure -width  $itk_option(-buttonWidth)
        }
    }
}

#---------------------------------------------------------------
# OPTION : -buttonHelp
#---------------------------------------------------------------
configbody ist3StateMenubutton::buttonHelp {
    
    ##### Table of buttons
    for { set i 0 } { $i < $cnt } { incr i } {
        catch {UifShortHelp::add $itk_option(-helparea) $itk_component(btn_$i) \
	           $itk_option(-buttonHelp)}
    }

    ##### Menu button
    catch {UifShortHelp::add $itk_option(-helparea) $itk_component(menubutton) \
	       $itk_option(-buttonHelp)}
}

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------
configbody ist3StateMenubutton::label {
    $itk_component(label) configure -text $itk_option(-label)
}

#---------------------------------------------------------------
# OPTION : -showLabel
#---------------------------------------------------------------
configbody ist3StateMenubutton::showLabel {
    set isPacked [catch {pack info $itk_component(label)}]
    if { $itk_option(-showLabel) == 0} {
        if { $isPacked == 0 } {
            pack forget $itk_component(label)
        }
    } else {
        if { $isPacked == 1 } {
            if { $itk_option(-type) == "menu"} {
                if { $itk_option(-showCurrent) == 1 } {
                    pack $itk_component(label) -anchor c -fill x -expand 1 \
                        -before $itk_component(current)
                } else {
                    pack $itk_component(label) -anchor c -fill x -expand 1 \
                        -before $itk_component(menubutton)
                }
            } else {
                pack $itk_component(label) -anchor c -fill x -expand 1 \
                    -before $itk_component(btn_0)
            }
        }
    }
}

#---------------------------------------------------------------
# OPTION : -showCurrent
#---------------------------------------------------------------
configbody ist3StateMenubutton::showCurrent {
    set isPacked [expr {! [catch {pack info $itk_component(current)}]}]
    if { $itk_option(-showCurrent) == 0} {
        if { $isPacked == 1 } {
            pack forget $itk_component(current)
        }
    } else {
        if { $isPacked == 0 } {
            if { $itk_option(-type) == "menu"} {
                pack $itk_component(current) -fill x -expand 1\
                    -before $itk_component(menubutton)
            }
        }
    }
}

#---------------------------------------------------------------
# OPTION : -showMenuButton
#---------------------------------------------------------------
configbody ist3StateMenubutton::showMenuButton {
    set isPacked [expr {! [catch {pack info $itk_component(menubutton)}]}]
    if { $itk_option(-showMenuButton) == 0} {
        if { $isPacked == 1 } {
            pack forget $itk_component(menubutton)
        }
    } else {
        if { $isPacked == 0 } {
            if { $itk_option(-type) == "menu"} {
                pack $itk_component(menubutton) -fill x -expand 1
            }
        }
    }
}

#---------------------------------------------------------------
# OPTION : -currentHelp
#---------------------------------------------------------------
configbody ist3StateMenubutton::currentHelp {
    catch {UifShortHelp::add  $itk_option(-helparea) $itk_component(current) \
	       $itk_option(-currentHelp)}
}

#---------------------------------------------------------------
# OPTION : -nowBg
#---------------------------------------------------------------
configbody ist3StateMenubutton::nowBg {
    set nowBg $itk_option(-nowBg)
}

#---------------------------------------------------------------
# OPTION : -nowFont
#---------------------------------------------------------------
configbody ist3StateMenubutton::nowFont {
    set nowFont $itk_option(-nowFont)
}

#---------------------------------------------------------------
# OPTION : -labelBg
#---------------------------------------------------------------
configbody ist3StateMenubutton::labelBg {
    $itk_component(label) configure -bg $itk_option(-labelBg)
}

#---------------------------------------------------------------
# OPTION : -labelFont
#---------------------------------------------------------------
configbody ist3StateMenubutton::labelFont {
    $itk_component(label) configure -font $itk_option(-labelFont)
}

#---------------------------------------------------------------
# OPTION : -labelHelp
#---------------------------------------------------------------
configbody ist3StateMenubutton::labelHelp {
    catch {UifShortHelp::add  $itk_option(-helparea) $itk_component(label) \
	       $itk_option(-labelHelp)}
}

#---------------------------------------------------------------
# OPTION : -labelFg
#---------------------------------------------------------------
configbody ist3StateMenubutton::labelFg {
    $itk_component(label) configure -fg $itk_option(-labelFg)
}

#---------------------------------------------------------------
# OPTION : -orient
#---------------------------------------------------------------
configbody ist3StateMenubutton::orient {

    set option "left"

    if {$itk_option(-orient) != $currOrient} {

        set widgets [lrange [pack slaves $itk_component(btnFrame)] 1 end]

        if {$itk_option(-orient) == "h"} {
            set option "left"
        }
        if {$itk_option(-orient) == "v"} {
            set option "bottom"
        }
        foreach slave $widgets {
            pack configure $slave -side $option -expand 1
        }
        set currOrient $itk_option(-orient)
    }
}
#---------------------------------------------------------------
# OPTION : -type
#---------------------------------------------------------------
configbody ist3StateMenubutton::type {
    updateType
}

#---------------------------------------------------------------
# OPTION : -selectVariable
#---------------------------------------------------------------
configbody ist3StateMenubutton::selectVariable {
    updateUserVar
}

#---------------------------------------------------------------
# OPTION : -acceptVariable
#---------------------------------------------------------------
configbody ist3StateMenubutton::acceptVariable {
    updateUserVar
}

# __o0o__
