#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: istDiskMon.tcl,v 1.49 2004/01/28 09:35:29 vltsccm Exp $"
#
# istDiskMon.tcl
#
#
# who        when      what
# ---------  --------  ----------------------------------------------
# T.Herlin   20/05/95  Created
# T.Herlin   11/09/95  Modified public variables (closes SPR 950430)
# T.Herlin   11/04/96  Ported to incr2
# M.COMIN    13/05/96  Add options -relief and -borderwidth to component
#                      monFrame after intergration with CCD
# T.Herlin   21/05/96  Fixed bug when setting environment INS_DISK
# T.Herlin   19/03/97  Change to use istDiskInfo and added the -small option
# T.Herlin   30/06/97  Destructor cancels afterId
# T.Herlin   21/07/97  Added option -ticks 0 to UifBarChart due to problems
#                      with viewing bar.
# pbiereic   30/07/99  Uses istNamespaceTail. Bug fixed in diskUpdateFreq
# gzins      03/08/01  SPR20000247 - changed wm properties so that the config
#                      window can't be overlayed
#                      Froze the window size and added option -cfgWinGeometry
#                      defining window geometry
#
#************************************************************************
#   NAME
#   istDiskMon - panel editor class to display the disk space usage
#
#
#   SYNOPSIS
#      istDiskMon <name>
#
#   DESCRIPTION
#      istDiskMon is a panel editor class that can be inserted when
#      developing a control panel. It displays a barchart with the
#      current disk space usage of the disk entered in the entry field
#      below. The disk space is queried by sending a query command to 
#      the istServer. 
#      The disk name can either be entered by typing into the entry 
#      field or setting the environment INS_DISK. When this environement
#      is set the entry field is locked and the disk name can not be
#      changed.
#
#      The radio buttons "Percent" and "MByte" toggle the free disk
#      space value in the respective unit.
#      When the update button is checked in the disk space is queried
#      periodically.
#   
#   OPTIONS
#     Options represent public variables of the object and can be set with:
#
#       <object-name> configure -<Option> <Value> ...
#
#      istDiskMon accepts following configuration options:
#
#       -diskName       <file> Filename of the disk monitored.
#
#       -diskUpdate     <0|1>  Toggle the update checkbutton. 1 = update the 
#                              widget with diskUpdateFreq. 0 = don't update.
#
#       -diskUpdateFreq <sec>  Update frequency in seconds (default 5).
#
#       -percent        <0|1>  Show value in percent (default).
#
#       -MByte          <0|1>  Show value in MByte.
#
#       -smallLayout    <0|1>  Change layout to a smaller size with widget 
#                              controls in a seperate dialog window.
#
#       -cfgWinGeometry <geo>  Define the geometry of the seperate configuration
#                              dialog window. Only relevant when -smallLayout
#                              is true.
#
#   ENVIRONMENT VARIABLES
#      The evironment INS_DISK is used as disk reference when it is 
#      specified.
#   
#   EXAMPLE
#    In this example the auto_path is assumed set correctly.
#    seqWish> istDiskMon .diskMonitor
#    seqWish> pack .diskMonitor
#    seqWish> .diskMonitor -diskName "/diskc" -mByte 1
#    seqWish> .diskMonitor -diskUpdate 1 -diskUpdateFreq  5    
#
#------------------------------------------------------------------------
#
#########################################################################
itk::usual istDiskMon {
    keep -relief -borderwidth -background  -foreground
}

class istDiskMon {

    inherit UifInteractiveWidget

    private common istdata

    constructor {args} {}
    destructor  {}

    itk_option define -diskName        diskName       DiskName        "/tmp"
    itk_option define -diskUpdateFreq  diskUpdateFreq DiskUpdateFreq  5
    itk_option define -diskUpdate      diskUpdate     DiskUpdate      0
    itk_option define -mByte           mByte          MByte           1
    itk_option define -percent         percent        Percent         0
    itk_option define -smallLayout     smallLayout    SmallLayout     0
    itk_option define -cfgWinGeometry  cfgWinGeometry CfgWinGeometry  +600+450

    public method updateDiskSpace  {}
    
    private method error { msg }
    private method checkInsDisk { }
    public  method configDialog { }

    private variable serverProc    "istServer"
    private variable updateFreq    3000
    private variable diskSpaceCmd  0
    private variable cv            0
    private variable afterId       0
    private variable initialized_  0
    private variable wthis         ""
    private variable entryWdg      ""
    private variable diskLblWdg    ""

}; # end of class


#//------------------------------------------------------
#// Constructor
#//------------------------------------------------------

body istDiskMon::constructor {args} {

    set wthis [istNamespaceTail $this]
    global ::${wthis}_istvalue
    
   itk_component add monFrame {
      frame  $itk_interior.monFrame -relief raised -bd 3 
   } {
      keep -relief -borderwidth
   }

   #// Build 3 frames (stacked on each other)
   itk_component add dname {
      frame  $itk_component(monFrame).dname 
   }
   
   itk_component add bar {
       frame  $itk_component(monFrame).bar
   }
   
   itk_component add cntrl {
       frame  $itk_component(monFrame).cntrl 
   }   

   #// entry area
   itk_component add lbl {
      label  $itk_component(dname).lbl  -text "Disk:" \
	      -font "-Adobe-helvetica-bold-r-normal--14*"
   }
   itk_component add entry {
      entry  $itk_component(dname).entry \
	      -textvariable [scope istdata(${wthis}_diskName)] -width 30
   }

   pack $itk_component(lbl) $itk_component(entry) -side left -fill x

   #// remember widget names in order to reconfigure when INS_DISK is 
   #// specified
   set entryWdg   $itk_component(entry)
   set diskLblWdg $itk_component(lbl)

   #//  bar chart area
   itk_component add disklbl {
      button  $itk_component(bar).lbl \
	      -textvariable [scope istdata(${wthis}_diskSpaceInfo)] \
	      -font "-Adobe-helvetica-bold-r-normal--14*" \
	      -relief flat
   }
   itk_component add diskbar {
       UifBarChart  $itk_component(bar).diskbar -barColor Blue -orient h -length 300 -variable  ${wthis}_istvalue
   }

   
   pack $itk_component(disklbl) $itk_component(diskbar) -fill x

   #// control area
   itk_component add update {
      checkbutton  $itk_component(cntrl).update -text Update -relief flat \
		             -variable [scope istdata(${wthis}_diskUpdate)]  \
			     -command "$this updateDiskSpace"
   } {
      keep -activeforeground -activebackground -highlightthickness
   }
   itk_component add percent {
       radiobutton  $itk_component(cntrl).percent -text Percent -relief flat \
		             -variable [scope istdata(${wthis}_diskInfo)] -value 0 \
			     -command "$this updateDiskSpace"
   } { 
      keep -activeforeground 
   }
   itk_component add mbyte {
       radiobutton  $itk_component(cntrl).mbyte -text MByte -relief flat \
		             -variable [scope istdata(${wthis}_diskInfo)] -value 1 \
			     -command "$this updateDiskSpace"
   } {
      keep -activeforeground 
   }

   pack $itk_component(update) -side left
   pack $itk_component(percent)  $itk_component(mbyte) -side right


   #// pack all frame areas
   pack $itk_component(bar) $itk_component(dname) $itk_component(cntrl) \
	   -fill x -expand yes -padx 4 -pady 4
   pack $itk_component(monFrame) 

   #// set bindings
   bind $itk_component(entry) <Return> "$wthis updateDiskSpace"
   
   eval itk_initialize

   set istdata(${wthis}_diskSpaceInfo) "0 MByte of 0 free"

   set initialized_ 1

   if {[llength $args] != 0 } { eval configure $args }

   updateDiskSpace
}

body istDiskMon::destructor {args} {

    catch { after cancel $afterId }
}

#---------------------------------------------------------------
# METHOD : updateDiskSpace
#---------------------------------------------------------------

body istDiskMon::updateDiskSpace {} {

    global ::env ::${wthis}_istvalue
    
    #puts "istDiskMon::updateDiskSpace $istdata(${wthis}_diskName)"

    #// check for INS_DISK environment 
    checkInsDisk
    
    if {[file exist $istdata(${wthis}_diskName)] == 0} {
	$this error "istDiskMon: Invalid filename specified - $istdata(${wthis}_diskName)"
	set istdata(${wthis}_diskUpdate) 0
	return
    }

    set error [catch {	set info [exec istDiskInfo \
	    -disk "$istdata(${wthis}_diskName)"] } msg]
    #// Could not send command return with empty string
    if {$error != 0} {
	if {$initialized_} {
	    $this error "istDiskMon: Error in calling istDiskInfo ($msg)"
	}
	return 
    }
 
    set total [format "%.1f" [expr { 1.0 * [lindex  $info 3] / 1024.0 }]]
    set free  [format "%.1f" [expr { 1.0 * [lindex  $info 6] / 1024.0 }]]
    set freePercent  [lindex  $info 8] 
    #puts "info $info ($total,$free,$freePercent)" 
    
    if { $istdata(${wthis}_diskInfo) } {
	#// set max value on  barChart
	$itk_component(diskbar) configure -maxVal $total
	catch {set ${wthis}_istvalue $free}
	if { $itk_option(-smallLayout) == 1 } {
	    set istdata(${wthis}_diskSpaceInfo) "Disk: $free MB of $total free ..."
	} else {
	    set istdata(${wthis}_diskSpaceInfo) "$free MB of $total free"
	}
    } else {	    
	$itk_component(diskbar) configure -maxVal 100
	catch {set ${wthis}_istvalue [string trim $freePercent %]}
	if { $itk_option(-smallLayout) == 1 } {
	    set istdata(${wthis}_diskSpaceInfo) "Disk: $freePercent free ..."
	} else {
	    set istdata(${wthis}_diskSpaceInfo) "$freePercent free"
	}
    }
	
    if { $afterId != 0 } {
	after cancel $afterId
	set afterId 0
    }
    if { $istdata(${wthis}_diskUpdate) } {
	set afterId [after $updateFreq  "$this updateDiskSpace"]
    }

}

#---------------------------------------------------------------
# METHOD : checkInsDisk
#---------------------------------------------------------------

body istDiskMon::checkInsDisk { } {
    global ::env 
    #// check for environment 
    if {[catch {set INS_DISK $env(INS_DISK)}] == 0} {
	set istdata(${wthis}_diskName) $INS_DISK
	catch { $entryWdg    configure -state disabled -width 27 }
	catch { $diskLblWdg  configure -text INS_DISK }
    }
}
#---------------------------------------------------------------
# METHOD : error
#---------------------------------------------------------------

body istDiskMon::error { msg } {

   set wthis [istNamespaceTail $this]
    #puts "istDiskMon::error"
    
    tk_dialog .dialog "istDiskMon Error" $msg error 0 OK
}

#---------------------------------------------------------------
# METHOD : configDialog
#---------------------------------------------------------------

body istDiskMon::configDialog { } {

    set w .istDiskConfigDialog
    set flag -1
    set gvar(isipanMessage) 0
    
    toplevel $w

    #// Set the wm properties so that the window can't be overlayed
    istWindowOnTop $w 
    
    #// Set the geometry of the dialog window and set the window's width and
    #// height not resizable.
    wm geometry $w $itk_option(-cfgWinGeometry)
    wm resizable $w 0 0
    
    #// Widgets
    frame $w.middle         -relief groove -border 1
    frame $w.bot            -relief raised -border 1
    frame $w.middle.center  -relief groove -border 1
    pack [button $w.bot.ok       -text "Dismiss" -width 10 \
	    -command "set flag 1; destroy $w"] \
	    -fill x -expand yes -padx 4 -pady 4
    #// 
    frame $w.middle.center.dname
    set diskLblWdg [label $w.middle.center.dname.lbl  -text "Disk:" \
	    -font "-Adobe-helvetica-bold-r-normal--14*"]
    set entryWdg [entry $w.middle.center.dname.entry \
	    -textvariable [scope istdata(${wthis}_diskName)] -width 30]

    #// check for INS_DISK environment
    checkInsDisk

    pack  $w.middle.center.dname.lbl $w.middle.center.dname.entry \
	    -side left -fill x

    #// 
    frame $w.middle.center.cntrl 
    checkbutton $w.middle.center.cntrl.update -text Update -relief flat \
	    -variable [scope istdata(${wthis}_diskUpdate)]  \
	    -command "$this updateDiskSpace" 
    radiobutton $w.middle.center.cntrl.percent -text Percent -relief flat \
	    -variable [scope istdata(${wthis}_diskInfo)] -value 0 \
	    -command "$this updateDiskSpace" 
    radiobutton $w.middle.center.cntrl.mbyte -text MByte -relief flat \
	    -variable [scope istdata(${wthis}_diskInfo)] -value 1 \
	    -command "$this updateDiskSpace"

    pack  $w.middle.center.cntrl.update -side left
    pack $w.middle.center.cntrl.percent  $w.middle.center.cntrl.mbyte \
	   $itk_component(mbyte) -side right
    
    pack  $w.middle.center.dname $w.middle.center.cntrl \
	    -fill x -expand yes -padx 4 -pady 4
    
#    pack append $w $w.middle   {top fillx}   $w.bot {bot fillx}
#    pack append $w.middle  $w.middle.center {right padx 10 filly fillx} 
    pack $w.middle        -side top    -fill x  
    pack $w.bot           -side bottom -fill x
    pack $w.middle.center -side right  -fill y -fill x 
    
    wm title $w "istDiskMonitor Config Dialog"
    
    tkwait visibility $w
    grab $w   
    tkwait variable flag;      # Stops until variable as been modified
}

#---------------------------------------------------------------
# OPTION : -diskName
#---------------------------------------------------------------  

configbody istDiskMon::diskName {

    set diskName $itk_option(-diskName)
    
    set istdata(${wthis}_diskName) $diskName

    checkInsDisk
}

#---------------------------------------------------------------
# OPTION : -diskUpdateFreq
#---------------------------------------------------------------  
 
configbody istDiskMon::diskUpdateFreq {
    set updateFreq [expr { 1000 * $itk_option(-diskUpdateFreq) }]

    if { $updateFreq == 0 } {
	set updateFreq 5000
    }
}


#---------------------------------------------------------------
# OPTION : -diskUpdate
#---------------------------------------------------------------  

configbody istDiskMon::diskUpdate {
    set diskUpdate $itk_option(-diskUpdate)

    set istdata(${wthis}_diskUpdate) $diskUpdate
    
}


#---------------------------------------------------------------
# OPTION : -mByte
#---------------------------------------------------------------  

configbody istDiskMon::mByte {

    if { $itk_option(-mByte) } {
	set istdata(${wthis}_diskInfo) 1
    }
}


#---------------------------------------------------------------
# OPTION : -percent
#---------------------------------------------------------------  

configbody istDiskMon::percent {
    if { $itk_option(-percent) } {
	set istdata(${wthis}_diskInfo) 0
    }
}

#---------------------------------------------------------------
# OPTION : -smallLayout
#---------------------------------------------------------------  

configbody istDiskMon::smallLayout {

    if { $itk_option(-smallLayout) == 1 } {
	#puts "reconfiguring the layout !"
	$itk_component(disklbl) configure -relief raised \
		-command "$wthis configDialog"
	pack forget $itk_component(cntrl) $itk_component(dname)
	$itk_component(disklbl) configure -relief raised
	$itk_component(diskbar) configure -length 230 -ticks 0
    }
}


#___oOo___




