#*************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: istEntry.tcl,v 1.49 2004/01/28 09:35:32 vltsccm Exp $"
#
#
# who        when      what
# ---------  --------  ----------------------------------------------
# gzins      01/06/00  created
# dpopovic   14/01/04  namespace for tcl_version > 8.0 only
#
#************************************************************************
#   NAME
#   istEntry - convenient entry widget
#
#   SYNOPSIS
#   istEntry  <widget-name>
#
#   DESCRIPTION
#   This widget displays a label, an entry and an unit label. It implements
#   convenient methods for accessing and modifying the labels and the value.
#   All the entered characters are filtered according to the entry type (i.e
#   integer, float or string input). When a new valid value is entered the
#   background color is set to 'valuedBg' color. The entry background color is
#   reset when entry is accepted or cleared.
#
#   PUBLIC METHODS
#   getValue      {}
#     Returns the currently entered value.
#
#   setValue      {value}
#     Set the new currently entered value.
#
#   accept        {}
#     Changes the state of the entered value to accepted.
#
#   clear         {}
#     Clear the currently entered value.
#
#   valuedKeyword {}
#     Return the associated keyword with the currently entered value, or an
#     empty string if no value entered.
#
#   keyword       {}
#     Return the associated keyword.
#
#   disable       {}
#     Disable the entry.
#
#   enable        {}
#     Enable the entry.
#
#   WIDGET-SPECIFIC OPTIONS
#     -label        <label>
#
#     -labelBg      <color>  - rgb color e.g. blue
#
#     -labelFg      <color>  - rgb color e.g. blue
#
#     -labelFont    <font>
#
#     -labelWidth   <width>  - label width in number of characters
#
#     -showLabel    <0/1>    - show/hide the label
#
#     -entryWidth   <width>  - entry width in number of characters
#
#     -entryFont    <font>
#
#     -entryFg      <color>  - rgb color e.g. blue
#
#     -entryType    <interger/float/string> - type of entry. User to filter out
#                                             the inserted characters.
#
#     -valuedBg     <color>  - rgb color e.g. blue
#
#     -enterCB      <callback function> - function called when <Enter> is
#                   pressed.
#
#     -cancelCB     <callback function> - function called when <Escape> is
#                   pressed.
#
#     -unit         <unit>
#
#     -unitWidth    <width>  - unit width in number of characters
#
#     -unitFont     <font>
#
#     -keyword      <short FITS keyword>
#
#   SEE ALSO
#
#------------------------------------------------------------------------
#

# Procedure to be removed when includes in ist library
proc istNamespaceTail {ns} {
    return [namespace tail $ns]
}

#--------------------------------------------------------------
itk::usual istEntry {
}

class istEntry {

    inherit UifInteractiveWidget

    constructor {args} {}
    destructor  {}

    itk_option define -label          label          Label         "Function"
    itk_option define -labelBg        labelBg        LabelBg       ""
    itk_option define -labelFg        labelFg        LabelFg       "black"
    itk_option define -labelFont      labelFont      LabelFont     "-Adobe-helvetica-medium-r-normal--14*"
    itk_option define -labelWidth     labelWidth     LabelWidth    0
    itk_option define -showLabel      showLabel      ShowLabel     1

    itk_option define -entryWidth     entryWidth     EntryWidth    1
    itk_option define -entryFont      entryFont      EntryFont     "-Adobe-times-bold-i-normal--*-77-*"
    itk_option define -entryFg        entryFg        EntryFg       "black"
    itk_option define -entryType      entryType      EntryType     "string"
    itk_option define -valuedBg       valuedBg       ValuedBg      "skyBlue"
    itk_option define -enterCB        enterCB        EnterCB       ""
    itk_option define -cancelCB       cancelCB       CancelCB       ""

    itk_option define -unit           unit           Unit          ""
    itk_option define -unitWidth      unitWidth      UnitWidth     0
    itk_option define -unitFont       unitFont       UnitFont      "-Adobe-helvetica-medium-r-normal--14*"

    itk_option define -keyword        keyword        Keyword       ""

    public method     getValue      {}
    public method     setValue      {value}
    public method     accept        {}
    public method     clear         {}
    public method     valuedKeyword {}
    public method     keyword       {}
    public method     disable       {}
    public method     enable        {}

    private method    updateEntry   {widget}
    private method    enter         {}
    private method    cancel        {}

    private variable  valued        0

}; # end of class

#//------------------------------------------------------
#// Constructor
#//------------------------------------------------------
body istEntry::constructor {args} {

    set wthis    [istNamespaceTail $this]

    #// Get defaults from a dummy widget
    set normalBg [lindex [$this configure -background] 4]

    #// Create common widgets.
    itk_component add mFrame {
        frame  $itk_interior.mFrame -relief flat
    } {
        usual
    }

    #// Label
    itk_component add label {
       label $itk_component(mFrame).label -anchor w
    }

    #// Entry widget
    itk_component add entry {
       entry  $itk_component(mFrame).entry
    }

    #// Unit
    itk_component add unit {
       label $itk_component(mFrame).unit -anchor w
    }

    #// Pack common widgets
    pack $itk_component(label) -side left -expand 1
    pack $itk_component(entry) -side left -expand 1 -fill x
    pack $itk_component(unit) -side left -expand 1

    pack $itk_component(mFrame) -fill x -expand 1

    # Add binding to handle 'key pressed' and 'Paste' events
    bind $itk_component(entry) <KeyPress> [code $this updateEntry %W]
    bind $itk_component(entry) <ButtonRelease-2> [code $this updateEntry %W]

    # Add binding to handle 'Enter' events
    bind $itk_component(entry) <Return> [code $this enter]
    bind $itk_component(entry) <KeyPress-Escape> [code $this cancel]

    # Re-order the binding tags so that 'updateEntry' is treated after the one
    # which inserts the new character(s) into the entry string.
    bindtags $itk_component(entry) [list itk-destroy-$itk_component(entry) \
        Entry . all <Button-2> $itk_component(entry)]

    eval itk_initialize
    eval configure $args

    # Configuration of the widgets
    $itk_component(label) configure -bg $normalBg
    $itk_component(unit) configure -bg $normalBg
    $itk_component(entry) configure -bg $normalBg
    $itk_component(entry) configure -font $itk_option(-entryFont)

}

#---------------------------------------------------------------
# METHOD : updateEntry
#---------------------------------------------------------------
body istEntry::updateEntry {widget} {

    # If widget disabled, ignore change
    if {[$itk_component(entry) cget -state] == "disabled" } {
        return
    }

    # Get the current entered value
    set currValue [$itk_component(entry) get]

    # If entry is numerical
    if {($itk_option(-entryType) == "integer") ||
        ($itk_option(-entryType) == "float")} {

        set insert [$itk_component(entry) index insert]

        # Filter out the non-numerical characters
        if { $itk_option(-entryType) == "integer" } {
            regsub -all {[^-\+0-9]} $currValue "" newValue
        } else {
            regsub -all {[^-\+.0-9]} $currValue "" newValue
        }

        # If any, remove extra periods (keep only the first period)
        set pIndex [string first "." $newValue]
        if { $pIndex != -1 } {
            regsub -all {\.} [string range $newValue [expr {$pIndex+ 1}] end] \
                "" subString
            set newValue "[string range $newValue 0 $pIndex]$subString"
        }

        # If any, remove extra - or + character
        regsub -all {[-\+]} [string range $newValue 1 end] "" subString
        set newValue "[string range $newValue 0 0]$subString"

        # If
        if { [string compare $newValue $currValue] != 0 } {
            $itk_component(entry) delete 0 end
            $itk_component(entry) insert 0 $newValue
            incr insert -1
            $itk_component(entry) icursor $insert
            set currValue $newValue
        }
    }

    # If entry is not empty
    if { [string length $currValue] != 0 } {
        # Set the background of the entry to 'valued' color
        $itk_component(entry) configure -bg  $itk_option(-valuedBg)
        set valued 1
    # Else
    } else {
        # Reset the background of the entry to default color
        set normalBg  [lindex [$this configure -background] 4]
        $itk_component(entry) configure -bg $normalBg
        set valued 0
    }

}; # end of updateEntry


#---------------------------------------------------------------
# METHOD : enter
#---------------------------------------------------------------
body istEntry::enter { } {
    # update user function
    if { $valued == 1 && $itk_option(-enterCB) != "" } {
        $itk_option(-enterCB) $this $itk_option(-keyword) [getValue]
    }
}; # end of enter

#---------------------------------------------------------------
# METHOD : cancel
#---------------------------------------------------------------
body istEntry::cancel { } {
    # update user function
    if { $itk_option(-cancelCB) != "" } {
        $itk_option(-cancelCB) $this
    }
}; # end of enter

#---------------------------------------------------------------
# METHOD : getValue
#---------------------------------------------------------------
body istEntry::getValue { } {
    # Return the current entered value
    return [$itk_component(entry) get]
}; # end of getValue

#---------------------------------------------------------------
# METHOD : setValue
#---------------------------------------------------------------
body istEntry::setValue {value} {
    # Return the current entered value
    $itk_component(entry) delete 0 end
    $itk_component(entry) insert insert $value
    updateEntry $wthis
}; # end of setValue


#---------------------------------------------------------------
# METHOD : accept
#---------------------------------------------------------------
body istEntry::accept { } {
    # Reset the background color
    set normalBg  [lindex [$this configure -background] 4]
    $itk_component(entry) configure -bg $normalBg
    set valued 0
}; # end of accept


#---------------------------------------------------------------
# METHOD : clear
#---------------------------------------------------------------
body istEntry::clear { } {

    # Reset the current entered value
    $itk_component(entry) delete 0 end

    # Reset the background color
    set normalBg  [lindex [$this configure -background] 4]
    $itk_component(entry) configure -bg $normalBg
    set valued 0

}; # end of clear

#---------------------------------------------------------------
# METHOD : valuedKeyword
#---------------------------------------------------------------
body istEntry::valuedKeyword { } {

    # Get the current entered value
    set currValue [$itk_component(entry) get]

    if {$valued == 1} {
        return "$itk_option(-keyword) $currValue"
    } else {
        return ""
    }
}; # end of valuedKeyword

#---------------------------------------------------------------
# METHOD : keyword
#---------------------------------------------------------------
body istEntry::keyword { } {
    return $itk_option(-keyword)
}; # end of Keyword


#---------------------------------------------------------------
# METHOD : disable
#---------------------------------------------------------------
body istEntry::disable { } {
    clear
    $itk_component(entry) configure -state disabled -relief groove
}; # end of disable


#---------------------------------------------------------------
# METHOD : enable
#---------------------------------------------------------------

body istEntry::enable { } {
    $itk_component(entry) configure -state normal -relief sunken
}; # end of disable

#-------------------------------------------
# PUBLIC VARIABLES
#-------------------------------------------

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------
configbody istEntry::label {
    set label [string trimright $itk_option(-label) " :"]
    if { $label != "" } {
        set label "$label :"
    }
    $itk_component(label) configure -text $label
}

#---------------------------------------------------------------
# OPTION : -labelBg
#---------------------------------------------------------------
configbody istEntry::labelBg {
    if { $itk_option(-labelBg) != "" } {
        set labelBg $itk_option(-labelBg)
    } else {
        button  .dummy
        set labelBg [lindex [.dummy configure -bg] 4]
        destroy .dummy
    }
    $itk_component(label) configure -bg $labelBg
}

#---------------------------------------------------------------
# OPTION : -labelFg
#---------------------------------------------------------------
configbody istEntry::labelFg {
    $itk_component(label) configure -fg $itk_option(-labelFg)
}

#---------------------------------------------------------------
# OPTION : -labelFont
#---------------------------------------------------------------
configbody istEntry::labelFont {
    $itk_component(label) configure -font $itk_option(-labelFont)
}

#---------------------------------------------------------------
# OPTION : -showLabel
#---------------------------------------------------------------
configbody istEntry::showLabel {
    set isPacked [catch {pack info $itk_component(label)}]
    if { $itk_option(-showLabel) == 0} {
        if { $isPacked == 0 } {
            pack forget $itk_component(label)
        }
    } else {
        if { $isPacked == 1 } {
            pack $itk_component(label) -side left \
                    -before $itk_component(entry)
        }
    }
}

#---------------------------------------------------------------
# OPTION : -labelWidth
#---------------------------------------------------------------
configbody istEntry::labelWidth {
    if { $itk_option(-labelWidth) } {
        $itk_component(label) configure -width $itk_option(-labelWidth)
    }
}

#---------------------------------------------------------------
# OPTION : -entryWidth
#---------------------------------------------------------------
configbody istEntry::entryWidth {
    if { $itk_option(-entryWidth) } {
        $itk_component(entry) configure -width  $itk_option(-entryWidth)
    }
}

#---------------------------------------------------------------
# OPTION : -entryFont
#---------------------------------------------------------------
configbody istEntry::entryFont {
    $itk_component(entry) configure -font $itk_option(-entryFont)
}

#---------------------------------------------------------------
# OPTION : -entryFg
#---------------------------------------------------------------
configbody istEntry::entryFg {
    $itk_component(entry) configure -fg $itk_option(-entryFg)
}

#---------------------------------------------------------------
# OPTION : -unit
#---------------------------------------------------------------
configbody istEntry::unit {
    set unit [string trim $itk_option(-unit) "( )"]
    set isPacked [catch {pack info $itk_component(unit)}]
    if { $unit != "" } {
        if { $isPacked == 1 } {
            pack $itk_component(unit) -side left \
                    -after $itk_component(entry)
        }
        set unit "($unit)"
        $itk_component(unit) configure -text $unit
    } else {
        if { $isPacked == 0 } {
            pack forget $itk_component(unit)
        }
    }
}

#---------------------------------------------------------------
# OPTION : -unitWidth
#---------------------------------------------------------------
configbody istEntry::unitWidth {
    $itk_component(unit) configure -width  $itk_option(-unitWidth)
}

#---------------------------------------------------------------
# OPTION : -unitFont
#---------------------------------------------------------------
configbody istEntry::unitFont {
    $itk_component(unit) configure -font $itk_option(-unitFont)
}

# __o0o__
