#*************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: istLinearMotor3State.tcl,v 1.49 2004/01/28 09:35:32 vltsccm Exp $"
#
# who        when      what
# ---------  --------  ----------------------------------------------
# gzins      20/06/00  Created
#
#************************************************************************
#   NAME
#   istLinearMotor3State - three state button area widget for controlling
#                          linear motor
#
#   SYNOPSIS
#   istLinearMotor3State  <widget-name>
#
#   DESCRIPTION
#   The istLinearMotor3State is dedicated widget for controlling linear  motor.
#   It inherits the basic facilities provided by istMotor3State, but enhances it
#   with specific features needed by linear motor such as:
#       - enter to new encoder position,
#       - display the current encoder position.
#
#   This widget simply includes an 'istEntry' widget allowing the user to
#   specify a new encoder position and an label widget to display the current
#   position, which is updated when the database attribute containing the
#   motor position changes.
#
#   PUBLIC METHODS
#     positionDbUpdate
#       Callback attached to the 'position' database event. It updates the
#       current position of the motor. The 'precision' option is used to format
#       the value to be displayed in widget (number of significant digits).
#
#     getEntryWdg
#       Return the reference to entry widget (istEntry object), to be used for
#       configuration.
#
#     getValue
#     setValue {value}
#       Get/Set the current entered value (e.g. new motor position).
#
#     accept  {{no ""}}
#     clear   {}
#     disable {}
#     enable  {}
#       Call the method of both parent class and 'istEntry' instance.
#
#     valuedKeyword {{encoderOrder 0}}
#       Returns the short FITS keyword with the currently selected element to
#       set-up a new named position or the short FITS keyword with the new
#       encoder value (when encoderOrder is set to 1) to move motor to the new
#       encoder position.
#
#   WIDGET-SPECIFIC OPTIONS
#     -positionDb    <position db point> - name of the database attribute where
#                                          is stored the current encoder
#                                          position of the motor.
#     -positionFg    <color>             - color used to display current encoder
#                                          position of the motor.
#
#     -positionFont  <font>              - font used to display current encoder
#                                          position of the motor.
#
#     -positionHelp  <help text>         - short help text for the widget
#                                          displaying the current position 
#     -precision     <number of digits>  - number of significant digits to
#                                          format the current encoder position
#                                          of the motor.
#     -entryHelp     <help text>         - short help text for the entry widget
#
#   EXAMPLES
#
#     Create an instance
#
#     istLinearMotor3State .myFunction  -label myFunction -current Open \
#                         -elements {Open Closed J K} -keyword "INS.OPTI1" \
#                         -selectCB funcSelect -type menu -precision 2 \
#                         -statusDb <alias>slider:mcmStatus.posIndex\
#                         -motionStatusDb <alias>slider:mcmStatus.motionStatus \
#                         -enabledDb <alias>slider.enabled \
#                         -stateDb <alias>slider.state \
#                         -positionDb <alias>slider:mcmStatus.posEnc
#
#     proc funcSelect { keyword value } {
#          puts "Function: $keyword has value: $value"
#     }
#
#------------------------------------------------------------------------
#
#--------------------------------------------------------------
itk::usual istLinearMotor3State {
}

class istLinearMotor3State {

    inherit istMotor3State

    constructor {args} {}
    destructor  {}

    itk_option define -positionDb    positionDb   PositionDb    ""
    itk_option define -positionFg    positionFg   PositionFg    "Blue"
    itk_option define -positionFont  positionFont PositionFont  "Helvetica -14 bold"
    itk_option define -positionHelp  positionHelp PositionHelp  "Current absolute position of the motor"
    itk_option define -precision     precision    Precision     1
    itk_option define -entryHelp     entryHelp    EntryHelp     "New motor position"

    public method  positionDbUpdate  {}
    public method  getEntryWdg       {}

    public method  getValue          {}
    public method  setValue          {value}
    public method  accept            {{no ""}}
    public method  clear             {}
    public method  valuedKeyword     {{encoderOrder 0}}
    public method  disable           {}
    public method  enable            {}

    private variable wthis           ""
    private variable positionDbEvt   ""
    private variable position 0


}; # end of class

#//------------------------------------------------------
#// Constructor
#//------------------------------------------------------
body istLinearMotor3State::constructor {args} {

    set wthis    [istNamespaceTail $this]

    #// Get defaults from a dummy widget
    button  .dummy
    set normalFont [lindex [.dummy configure -font] 4]
    set normalBg [lindex [.dummy configure -bg] 4]
    destroy .dummy


    #// Entry for linear motors value
    itk_component add positionFrame {
        frame $itk_component(btnFrame).positionFrame -relief groove
    }

    itk_component add entry {
        istEntry $itk_component(positionFrame).entry -showLabel 0 \
            -entryType "float"
    }

    itk_component add position {
        label $itk_component(positionFrame).position \
            -textvariable [scope position] -bd 0
    }

    pack $itk_component(positionFrame) -after $itk_component(label) -fill x
    pack $itk_component(entry) -anchor center -padx 3 -pady 3 -expand 1 -fill x
    pack $itk_component(position) -fill x -padx 3 -pady 3

    eval itk_initialize

    #// Evaluate arguments
    if {[llength $args] != 0 } {
        eval configure $args
    }

    #// This option is always lost when derivating ist3StateMenubutton class !!!
    #// It must be set again in derivated classes.
    $itk_component(btnFrame).menubutton.menu configure -tearoff 0
}

#-------------------------------------------
# PUBLIC VARIABLES
#-------------------------------------------

#---------------------------------------------------------------
# METHOD : stateDbUpdate
#---------------------------------------------------------------
body istLinearMotor3State::positionDbUpdate {} {

    # Get motion status (if 'db' defined)
    set currPosition [seq_dbReadSymbolic $itk_option(-positionDb)]

    # Set the label color according to the current position
    set precision $itk_option(-precision)
    set position [format "%2.${precision}f" $currPosition]
}

#---------------------------------------------------------------
# METHOD : getEntryWdg
#---------------------------------------------------------------
body istLinearMotor3State::getEntryWdg {} {
    # Get motion status (if 'db' defined)
    return $itk_component(entry)
}

#---------------------------------------------------------------
# METHOD : getValue
#---------------------------------------------------------------
body istLinearMotor3State::getValue {} {
    return [$itk_component(entry) getValue]
}

#---------------------------------------------------------------
# METHOD : setValue
#---------------------------------------------------------------
body istLinearMotor3State::setValue {value} {
    return [$itk_component(entry) setValue $value]
}

#---------------------------------------------------------------
# METHOD : accept
#---------------------------------------------------------------
body istLinearMotor3State::accept {{no ""}} {
    ist3StateMenubutton::accept $no
    return [$itk_component(entry) accept]
}

#---------------------------------------------------------------
# METHOD : clear
#---------------------------------------------------------------
body istLinearMotor3State::clear {} {
    ist3StateMenubutton::clear
    return [$itk_component(entry) clear]
}

#---------------------------------------------------------------
# METHOD : valuedKeyword
#---------------------------------------------------------------
body istLinearMotor3State::valuedKeyword {{encoderOrder 0}} {
    if {$encoderOrder == 0 } {
        set valKeyw [ist3StateMenubutton::valuedKeyword]
    } else {
        set valKeyw [$itk_component(entry) valuedKeyword]
    }
    return [string trim $valKeyw]
}

#---------------------------------------------------------------
# METHOD : disable
#---------------------------------------------------------------
body istLinearMotor3State::disable {} {
    ist3StateMenubutton::disable
    return [$itk_component(entry) disable]
}

#---------------------------------------------------------------
# METHOD : enable
#---------------------------------------------------------------
body istLinearMotor3State::enable {} {
    ist3StateMenubutton::enable
    return [$itk_component(entry) enable]
}

#---------------------------------------------------------------
# OPTION : -positionDb
#---------------------------------------------------------------
configbody istLinearMotor3State::positionDb {

    # If already attached, detach previous callback
    if { $positionDbEvt != "" } {
        seq_evtDetach $positionDbEvt
    }

    # If new database attribut defined
    if {$itk_option(-positionDb) != ""} {
        # Attach callback
        if  {[catch {set positionDbEvt [seq_evtAttach \
					    $itk_option(-positionDb) NE "$wthis positionDbUpdate"]}]} {
            puts "istLinearMotor3State error ! Could not attach event to \
                $itk_option(-positionDb)"
        } else {
            #// update the current status
            positionDbUpdate
        }
    }
}

#---------------------------------------------------------------
# OPTION : -positionFg
#---------------------------------------------------------------
configbody istLinearMotor3State::positionFg {
    $itk_component(position) configure -fg $itk_option(-positionFg)
}

#---------------------------------------------------------------
# OPTION : -positionFont
#---------------------------------------------------------------
configbody istLinearMotor3State::positionFont {
    $itk_component(position) configure -font $itk_option(-positionFont)
}

#---------------------------------------------------------------
# OPTION : -positionHelp
#---------------------------------------------------------------
configbody istLinearMotor3State::positionHelp {
    catch {UifShortHelp::add  $itk_option(-helparea) $itk_component(position) \
	       $itk_option(-positionHelp)}
}

#---------------------------------------------------------------
# OPTION : -entryHelp
#---------------------------------------------------------------
configbody istLinearMotor3State::entryHelp {
    catch {UifShortHelp::add  $itk_option(-helparea) $itk_component(entry) \
	       $itk_option(-entryHelp)}
}

