#*************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: istMotor3State.tcl,v 1.49 2004/01/28 09:35:33 vltsccm Exp $"
#
# who        when      what
# ---------  --------  ----------------------------------------------
# gzins      20/06/00  created
# gzins      28/09/00  added '-statusEltDb' option
#
#************************************************************************
#   NAME
#   istMotor3State - three state button area widget for controlling motor
#
#   SYNOPSIS
#   istMotor3State  <widget-name>
#
#   DESCRIPTION
#   The istMotor3State is dedicated widget for controlling motor. It inherits
#   the basic facilities provided by ist3StateMenubutton, but enhances it with
#   specific features to handle motor control states (stored in database) such
#   as:
#       - 'state' database attribute which contains the current state of the
#         controlling device. The color of the label changes to reflect the
#         current state according to the following table:
#              state = 1 (i.e. OFF)       --> RED
#              state = 2 (i.e. STANDBY)   --> GREEN
#              state = 3 (i.e. ONLINE)    --> Default color
#       - 'status' which contains the current position index of the motor or
#         'statusElt' which contains the current position name of the motor.
#         The widget is updated to show this position using either the
#         'acceptBg' color or 'nowBg' color according to the 'motion' status
#         (see below).
#       - 'motion' database attribute which informs when motor is moving. When
#         the motor is moving (motion status = 2), the background color of the
#         current position is set to the 'nowBg' color while the background
#         color is set to the 'acceptBg' color when motor is standing. In
#         addition, the foreground color of the label is set to green when the
#         motor is moving.
#       - 'enabled' database attribute. The widget state is set according to
#         this attribute, i.e. it is enabled when 'enabled' is set to 1, and
#         disabled otherwise.
#
#   The user create a widget of this class (called instance) by :
#     istMotor3State  <widget-name> <config-parameters>
#
#   where :
#     <widget-name>         is any valid TK widget name
#     <config-parameters>   sequence of widget resources
#
#   Standard widget resources can be set using :
#      <widget-name> configure <resource-name> <value>
#
#   The current value of the widget resources is returned by :
#      <name-of-instance> cget <resource-name>
#
#   PUBLIC METHODS
#     statusDbUpdate
#        Callback attached to the 'status' database event. It updates the
#        current position of the motor.
#
#     statusEltDbUpdate
#        Callback attached to the 'statusElt' database event. It updates the
#        current position of the motor.
#
#     motionStatusDbUpdate
#        Callback attached to the 'motion' database event. It updates the
#        background color of the current position with 'nowBg' color when motor
#        is moving or 'acceptBg' color when motor is standing.
#
#     enabledDbUpdate
#        Callback attached to the 'enabled' database event. It enables or
#        disables widget acoording the current 'enabled' value.
#
#     stateDbUpdate
#        Callback attached to the 'state' database event. It updates the
#        background color of the label according to the state of the controlling
#        device (RED when OFF, GREEN when STANDBY and 'default' when ONLINE).
#
#   WIDGET-SPECIFIC OPTIONS
#     -statusDb       <status db point>  - name of the database attribute where
#                                          is stored the current position index
#                                          of the motor.
#
#     -statusEltDb    <status db point>  - name of the database attribute where
#                                          is stored the current position name
#                                          of the motor.
#
#     -motionStatusDb <motion db point>  - name of the database attribute where
#                                          is stored the motion status of the
#                                          motor.
#
#     -enabledDb      <enabled db point> - name of the database attribute where
#                                          is stored the 'enable' flag of the
#                                          controlling device.
#
#     -stateDb        <state db point>   - name of the database attribute where
#                                          is stored the state of the device.
#
#     -movingBg       <color>            - foreground color of the label to
#                                          inform user when motor is moving.
#
#   EXAMPLES
#
#     Create an instance
#
#     istMotor3State .myFunction  -label myFunction -current Open \
#                         -elements {Open Closed J K} -keyword "INS.OPTI1" \
#                         -selectCB funcSelect -type menu \
#                         -statusDb <alias>slider:mcmStatus.posIndex\
#                         -motionStatusDb <alias>slider:mcmStatus.motionStatus \
#                         -enabledDb <alias>slider.enabled \
#                         -stateDb <alias>slider.state
#
#     proc funcSelect { keyword value } {
#          puts "Function: $keyword has value: $value"
#     }
#
#   CAUTIONS
#   The options '-statusDb' and '-statusEltDb' are used for the same purpose;
#   i.e. update the widget according to the current motor position, either using
#   the position index ('-statusDb' option) or the position name ('-statusEltDb'
#   option). They SHOULD NOT be used at the same time. 
#
#   SEE ALSO
#
#------------------------------------------------------------------------
#

#--------------------------------------------------------------
itk::usual istMotor3State {
}

class istMotor3State {

    inherit ist3StateMenubutton

    constructor {args} {}
    destructor  {}

    itk_option define -statusDb       statusDb       StatusDb       ""
    itk_option define -statusEltDb    statusEltDb    StatusEltDb    ""
    itk_option define -motionStatusDb motionStatusDb MotionStatusDb ""
    itk_option define -enabledDb      enabledDb      EnabledDb      ""
    itk_option define -stateDb        stateDb        StateDb        ""
    itk_option define -movingBg       movingBg       MovingBg       "green"

    public method  statusDbUpdate       {}
    public method  statusEltDbUpdate    {}
    public method  motionStatusDbUpdate {}
    public method  enabledDbUpdate      {}
    public method  stateDbUpdate        {}

    private variable wthis             ""
    private variable statusDbEvt       ""
    private variable statusEltDbEvt    ""
    private variable motionStatusDbEvt ""
    private variable enabledDbEvt      ""
    private variable stateDbEvt        ""
}; # end of class

#//------------------------------------------------------
#// Constructor
#//------------------------------------------------------
body istMotor3State::constructor {args} {

    set wthis    [istNamespaceTail $this]

    #// Get defaults from a dummy widget
    button  .dummy
    set normalFont [lindex [.dummy configure -font] 4]
    set normalBg [lindex [.dummy configure -bg] 4]
    destroy .dummy

    #// Evalute options
    eval itk_initialize

    if {[llength $args] != 0 } {
        eval configure $args
    }

    #// This option is always lost when derivating ist3StateMenubutton class !!!
    #// It must be set again in derivated classes.
    $itk_component(btnFrame).menubutton.menu configure -tearoff 0
}

#---------------------------------------------------------------
# METHOD : statusDbUpdate
#---------------------------------------------------------------
body  istMotor3State::statusDbUpdate {} {

    # If 'status db atribute' defined
    if { $itk_option(-statusDb) != ""} {
        # Get current position from db
        set status [seq_dbReadSymbolic $itk_option(-statusDb)]

        # Get motion status (if 'db' defined)
        if { $itk_option(-motionStatusDb) != ""} {
            set motionStatus [seq_dbReadSymbolic $itk_option(-motionStatusDb)]
        } else {
            set motionStatus 1
        }

        # If 'moving'
        if { $motionStatus == 2 } {
            # Update current position as 'now'
            now $status
        # Else (movement is completed)
        } else {
            # Set the current position as 'accepted'
            accept $status
        }
    }
}

#---------------------------------------------------------------
# METHOD : statusEltDbUpdate
#---------------------------------------------------------------
body  istMotor3State::statusEltDbUpdate {} {

    # If 'status db atribute' defined
    if { $itk_option(-statusEltDb) != ""} {
        # Get current position name from db
        set statusElt [seq_dbReadSymbolic $itk_option(-statusEltDb)]

        # Get the corresponding position index
        set status [lsearch -exact $itk_option(-elements) $statusElt]

        # Get motion status (if 'db' defined)
        if { $itk_option(-motionStatusDb) != ""} {
            set motionStatus [seq_dbReadSymbolic $itk_option(-motionStatusDb)]
        } else {
            set motionStatus 1
        }

        # If 'moving'
        if { $motionStatus == 2 } {
            # Update current position as 'now'
            now $status
        # Else (movement is completed)
        } else {
            # Set the current position as 'accepted'
            accept $status
        }
    }
}

#---------------------------------------------------------------
# METHOD : motionStatusDbUpdate
#---------------------------------------------------------------
body  istMotor3State::motionStatusDbUpdate {} {

    # Get motion status (if 'db' defined)
    set motionStatus [seq_dbReadSymbolic $itk_option(-motionStatusDb)]

    # If 'moving'
    if { $motionStatus == 2 } {
        #// Put color to 'movingBg' - moving
        $itk_component(label) configure -fg $itk_option(-movingBg)
    # Else (movement is completed)
    } else {
        # Put color back - standing
        $itk_component(label) configure -fg white
    }

    # Update status according to new position
    statusDbUpdate
}

#---------------------------------------------------------------
# METHOD : enabledDbUpdate
#---------------------------------------------------------------
body  istMotor3State::enabledDbUpdate {} {

    # Get motion status (if 'db' defined)
    set enabledStatus [seq_dbReadSymbolic $itk_option(-enabledDb)]

    # Enable or disable widget according to the current status
    if { $enabledStatus == 1 } {
        enable
    } else {
        disable
    }
}

#---------------------------------------------------------------
# METHOD : stateDbUpdate
#---------------------------------------------------------------
body  istMotor3State::stateDbUpdate {} {

    # Get motion status (if 'db' defined)
    set state [seq_dbReadSymbolic $itk_option(-stateDb)]

    # Set the label color according to the current state
    switch $state {
        1 { $itk_component(label) configure -bg red }
        2 { $itk_component(label) configure -bg OliveDrab }
        0 -
        3 { $itk_component(label) configure -bg $itk_option(-labelBg)}
    }
}

#---------------------------------------------------------------
# OPTION : -statusDb
#---------------------------------------------------------------
configbody istMotor3State::statusDb {

    # If already attached, detach previous callback
    if { $statusDbEvt != "" } {
        seq_evtDetach $statusDbEvt
    }

    # If new database attribut defined
    if {$itk_option(-statusDb) != ""} {
        # Attach callback
        if  {[catch {set statusDbEvt [seq_evtAttach \
					  $itk_option(-statusDb) W "$wthis statusDbUpdate"]}]} {
            puts "istMotor3State error ! Could not attach event to \
                $itk_option(-statusDb)"
        } else {
            #// update the current status
            statusDbUpdate
        }
    }
}

#---------------------------------------------------------------
# OPTION : -statusEltDb
#---------------------------------------------------------------
configbody istMotor3State::statusEltDb {

    # If already attached, detach previous callback
    if { $statusEltDbEvt != "" } {
        seq_evtDetach $statusEltDbEvt
    }

    # If new database attribut defined
    if {$itk_option(-statusEltDb) != ""} {
        # Attach callback
        if  {[catch {set statusEltDbEvt [seq_evtAttach \
					     $itk_option(-statusEltDb) W "$wthis statusEltDbUpdate"]}]} {
            puts "istMotor3State error ! Could not attach event to \
                $itk_option(-statusEltDb)"
        } else {
            #// update the current status
            statusEltDbUpdate
        }
    }
}

#---------------------------------------------------------------
# OPTION : -motionStatusDb
#---------------------------------------------------------------
configbody istMotor3State::motionStatusDb {

    # If already attached, detach previous callback
    if { $motionStatusDbEvt != "" } {
        seq_evtDetach $motionStatusDbEvt
    }

    # If new database attribut defined
    if {$itk_option(-motionStatusDb) != ""} {
        # Attach callback
        if  {[catch {set motionStatusDbEvt [seq_evtAttach \
						$itk_option(-motionStatusDb) NE "$wthis motionStatusDbUpdate"]}]} {
            puts "istMotor3State error ! Could not attach event to \
                $itk_option(-motionStatusDb)"
        } else {
            #// update the current status
            motionStatusDbUpdate
        }
    }
}

#---------------------------------------------------------------
# OPTION : -enabledDb
#---------------------------------------------------------------
configbody istMotor3State::enabledDb {

    # If already attached, detach previous callback
    if { $enabledDbEvt != "" } {
        seq_evtDetach $enabledDbEvt
    }

    # If new database attribut defined
    if {$itk_option(-enabledDb) != ""} {
        # Attach callback
        if  {[catch {set enabledDbEvt [seq_evtAttach \
					   $itk_option(-enabledDb) NE "$wthis enabledDbUpdate"]}]} {
            puts "istMotor3State error ! Could not attach event to \
                $itk_option(-enabledDb)"
        } else {
            #// update the current status
            enabledDbUpdate
        }
    }
}

#---------------------------------------------------------------
# OPTION : -stateDb
#---------------------------------------------------------------
configbody istMotor3State::stateDb {

    # If already attached, detach previous callback
    if { $stateDbEvt != "" } {
        seq_evtDetach $stateDbEvt
    }

    # If new database attribut defined
    if {$itk_option(-stateDb) != ""} {
        # Attach callback
        if  {[catch {set stateDbEvt [seq_evtAttach \
					 $itk_option(-stateDb) NE "$wthis stateDbUpdate"]}]} {
            puts "istMotor3State error ! Could not attach event to \
                $itk_option(-stateDb)"
        } else {
            #// update the current status
            stateDbUpdate
        }
    }
}

# __o0o__
