#
# E.S.O. - VLT project
#
# "@(#) $Id: istObsOption.tcl,v 1.49 2004/01/28 09:35:31 vltsccm Exp $"
#
#
# istObsOption.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# therlin    04/07/97   created
# pbiereic   30/07/99   Uses istNamespaceTail

#
#************************************************************************
#   NAME
#     istObsOption - Class implementing the option button MOTIF widget.
#   SYNOPSIS
#     istObsOption <widget-name>
#   DESCRIPTION
#
#     istObsOption defines the class for the implementation of the 
#     MOTIF option button. The label shows the last selected option.
#     The user create a widget of this class (called instance) by :
#
#         istObsOption  <widget-name> <config-parameters>
#
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     istObsOption - Option button widget
#
#   METHODS
#
#     Inherits the methods of class UifInteractiveWidget and support the 
#     following :
#
#     General Remark :
#     - Entries start with index 0
#     - Keyword "last" always identifies the last entry of the menu
#
#     addItem  <label> <command>
#        Add an entry allowing the user to invoke a TCL command.
#     
#        - <label>     String to label the entry
#        - <command>   TCL command to be executed
#
#     deleteEntry  <from> <to>
#        Delete menu items from position <from> to position <to> 
#      
#     enableEntry  <pos>
#        Enable selection of entry in position <pos>
#
#     disableEntry <pos> 
#        Disable selection of entry in position <pos>
#
#     getMenuName 
#        Get full name of the menu associated to the widget.
#
#     getSize                  
#        Returns the total number of options
#  
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifWidget class and supports :
#
#     command   <proc. name>  Name of a procedure to invoke when the user
#                             selects an option.
#                             The procedure must have an argument which will
#                             be the value of the selected option.
#     default   <value>       Default value to be shown when the widget is 
#                             created
#     font      <font name>   Text Font 
#     itemList  <list>        List of options.
#     variable  <va. name>    Name of the variable associated to the widget
#
#   SPECIFIC BINDINGS
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     istObsOption .w -itemList {item1 item3 item5} -default item5
#                        -command puts  -variable gvar(a) -length 10
#
#     Re-configure the widget
#
#     .w configure -command  myProcedure
#     .w configure -variable gvar(b) -itemList { a b c }
#
#     Add a new entry at the bottom of the list 
#
#     .w addItem myItem "myProcedure $p1 $p2"
#
#     Get configuration
#
#     set myVar      [.out cget -variable]
#     set entryList  [.out cget -itemList]
#
#   SEE ALSO
#     UifIntercativeWidget  UifWidget
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual istObsOption { }

class istObsOption {
    inherit UifInteractiveWidget

    constructor {args} {}
    
    itk_option define -variable variable Variable  ""
    itk_option define -bg       bg       Bg        ""
    itk_option define -command  command  Command   ""
    itk_option define -itemList itemList ItemList  ""
    itk_option define -font     font     Font      ""
    itk_option define -type     type     Type      "command"
    itk_option define -helpText helpText HelpText  ""
    
    public method addItem  { label cmd }
    public method setValue { label }
    public method select   { values }

    #*# method: getSize

    public method getSize { } { 
       return [$menu index last]
    }

    #*# method: deleteEntry

    public method deleteEntry { from {to {last}} } { 
       $menu delete $from $to 
    }

    #*# method: enableEntry

    public method enableEntry { pos } { 
       $menu entryconfigure $pos -state normal
    }

    #*# method: disableEntry

    public method disableEntry { pos } { 
       $menu entryconfigure $pos -state disabled
    }

    #*# method: getMenuName

    public method getMenuName {} { return $menu }
    public method execCB      {}

    private method configMenu {}

    protected variable type        "command"
    protected variable menu        ""
    protected variable frame_fg    ""
    private variable wthis         ""
    private variable obsLabel      ""

}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body istObsOption::constructor { args } {

    set wthis [istNamespaceTail $this]

    global ::${wthis}_gvar ::shArea

    #// set help area needed for NOV97
    configure -helparea $shArea

   itk_component add istObsOption {
       set me     $itk_interior.istObsOption
       menubutton $me -menu $me.menu -relief raised -text "" -padx 0 \
                  -indicatoron 1 -borderwidth 2
   } {
       keep -text -font -width -state -foreground -background \
            -activeforeground -activebackground 
   }

   pack $itk_component(istObsOption) -side left -fill both

   set menu  [menu $me.menu] 
   $menu configure -tearoff 0
   eval itk_initialize

   if {[set idx [lsearch $args "-type"]] != -1} {
       configure -type [lindex $args [expr {$idx +1}]]
       set args [lreplace $args $idx [expr {$idx +1}]]
   }

   if {[set idx [lsearch $args "-itemList"]] != -1} {
       configure -itemList [lindex $args [expr {$idx +1}]]
       set args [lreplace $args $idx [expr {$idx +1}]]
   }

   set initialized 1

   #  Explicitly handle config's that may have been ignored earlier

   if {[llength $args] != 0} { eval configure $args }
}

#---------------------------------------------------------------
# OPTION : -font
#---------------------------------------------------------------

configbody istObsOption::font {
   $itk_component(istObsOption) configure -font $itk_option(-font)    
}  

#---------------------------------------------------------------
# OPTION : -type
#---------------------------------------------------------------

configbody istObsOption::type {

    if { $itk_option(-type) == "checkbutton" } {
	set type checkbutton
    } else {
	set type command
    }
}  

#---------------------------------------------------------------
# OPTION : -helpText
#---------------------------------------------------------------

configbody istObsOption::helpText {

    if { $itk_option(-helpText) != ""} {
	catch {UifShortHelp::add $itk_option(-helparea) \
		$itk_component(istObsOption) "$itk_option(-helpText)"}
    } 
}  
#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody istObsOption::variable {
   #$itk_component(istObsOption) configure -textvariable $itk_option(-variable)
   setValue "$itk_option(-default)"
   configMenu
}

#---------------------------------------------------------------
# OPTION : -command
#---------------------------------------------------------------

configbody istObsOption::command {
   if {$initialized } { configMenu }
}

#---------------------------------------------------------------
# OPTION : -itemList
#---------------------------------------------------------------

configbody istObsOption::itemList {

    global ::${wthis}_gvar
    $menu delete 0 end
    foreach item $itk_option(-itemList) {
	if { $type == "command" } {
	    $menu add $type -label "$item"
	} else {
	    $menu add $type -label "$item" -variable ${wthis}_gvar($item)
	}
    }

    if {$initialized } { configMenu }
}


#---------------------------------------------------------------
# METHOD : setValue
#---------------------------------------------------------------

body istObsOption::setValue { label } {
    
    global ::${wthis}_gvar

    set obsLabel ""
    if { $type == "checkbutton" } {
	foreach item $itk_option(-itemList) {
	    if {[set ${wthis}_gvar($item)]} {
		if { $obsLabel != "" } {
		    set obsLabel "$obsLabel,"
		}
		set obsLabel "$obsLabel$item"
	    }
	}
    } else {
	set obsLabel $label
    }
    if {$itk_option(-variable) != "" } {
	uplevel #0 set $itk_option(-variable) \"$obsLabel\" 
    }   
}


#---------------------------------------------------------------
# METHOD : select
#---------------------------------------------------------------

body istObsOption::select { values } {
    
    global ::${wthis}_gvar

    if { $type == "checkbutton" } {
    set data [split $values ","]
    #puts "Handling data: $data"
	foreach value $data {
	    foreach item $itk_option(-itemList) { 
		if { "$item" == "$value" } {
		    set ${wthis}_gvar($item) 1
		}
	    }
	}
    } 
}


#---------------------------------------------------------------
# METHOD : execCB
#---------------------------------------------------------------

body istObsOption::execCB { } {
    
    global ::${wthis}_gvar

    if { $initialized } {
	if { $itk_option(-command) != "" } {
	    #puts "evaluating $itk_option(-command) \"$obsLabel\" "
	    eval $itk_option(-command) \"$obsLabel\"
	}
    }
}

#---------------------------------------------------------------
# METHOD : addItem
#---------------------------------------------------------------

body istObsOption::addItem { label cmd } {
   $menu add $type -label $label
   if {$cmd != ""} { append cmd "; "}
   append cmd "$this setValue \"$label\""
   eval $menu entryconfigure last -command {$cmd}
}

#---------------------------------------------------------------
# METHOD : configMenu
#---------------------------------------------------------------
body istObsOption::configMenu {} {
   if {[set maxId  [$menu index last]] == "none"} { return }

   for {set i 0} { $i <= $maxId } { incr i } {
       set lbl [lindex [$menu entryconfigure $i -label] 4]
       set cmd ""
       set tmp [string trim $lbl "\{\}"]

       set cmd "$this setValue \"$tmp\"; $this execCB"

       eval $menu entryconfigure $i -command {$cmd}
   }
}

#___oOo___



