#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: istPanScan.tcl,v 1.49 2004/01/28 09:35:32 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  20/02/97  created
# gzins     20/12/99  added istGetAttribId function - SPR VLTSW990498
#

#************************************************************************
#   NAME
#   istPanScan, panScan, PanScan, istLoadPanScan - utility to define update
#               of gvar() variables in a panel
#
#   SYNOPSIS
#   When initializing the panel (in the Tcl code):
#
#   istLoadPanScan <panel name>
#
# 
#   DESCRIPTION
#   The Pan Scan feature makes it possible to define the update of output
#   widgets in guis made with the panel editor. This is advantageous
#   compared to 'hard coding' the links between outputs and DB directly into
#   the panel themselves.
# 
#   For using the feature a file must be created with extension ".panscan"
#   and with the same name as the panel. The format of the file is as
#   follows (example):
#
#      set DATA   <alias>irdcs:dataInfo
#      set DET    <alias>irdcs:detector
#      set EXPO   <alias>irdcs:exposure
#      set GEN    <alias>irdcs:generalInfo
#      set IRACE  <alias>irdcs:irace
#      set MON    <alias>irdcs:monitor
#      
#      # Attributes in the Exposure Point
#      $EXPO.exposureNo       NE saExposureNo
#      $EXPO.DIT              NE saDIT         irdosRound3Decs
#      $EXPO.NDIT             NE saNDIT
#      $EXPO.MINDIT           NE saMINDIT      irdosRound3Decs
#      $EXPO.STARTX           NE saSTARTX
#      $EXPO.STARTY           NE saSTARTY
#      $EXPO.NX               NE saNX
#      $EXPO.NY               NE saNY
#      $EXPO.RSPEED           NE saRSPEED
#      $EXPO.RSPEEDADD        NE saRSPEEDADD
#      $EXPO.ditCount         NE saDitCount
#      $EXPO.intCount         NE saIntCount
#      $EXPO.stdevCount       NE saStdevCount
#      $EXPO.choppingMode     NE choppingMode  irdosState2OnOff
#      $EXPO.chopState        NE choppingMode  irdosChopPos
#      ...
#
#   It is possible to define constants to refer to attribute points.
#   When defining the dependencies the format as shown above is:
#
#   <attribute>   <update type>  <gvar index>  [<processing function>]
#
#   - where <attribute> is the atrribute name, <update type> is the
#   criteria for updating the attribute (see man-page of seq_evtAttach),
#   <gvar index> is the index in the global gvar array used for accessing
#   variables in the panels, and <processing function> is an optional
#   Tcl procedure that can be called on attribute read to reformat it e.g.
#   This procedure must return the processed value.
#
#   The <panel name>.panscan files must be installed in the 
#   $INTROOT/include or $VLTROOT/include directories.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# File: <panel name>.panscan
# <attr 1> <type 1> <gvar index 1> <cb fct 1>
# <attr 2> <type 2> <gvar index 2> <cb fct 2>
# <attr 3> <type 3> <gvar index 3> <cb fct 3>
#                     .
#                     .
#                     .

proc istPanScanMsg {msg} {
    puts "Pan Scan> $msg"
}


proc istLoadPanScan {panelName} {
    global gvar

    if {[string first ".panscan" $panelName] == -1} {
	set panLocal "$panelName.panscan"
    } else {
	set panLocal $panelName
    }
    set fileName [findVLTFile include/$panLocal yes]
    if {$fileName == ""} {return}
    
    set fd [open $fileName r]
    if {$fd == -1} {
	istPanScanMsg "Error opening Pan Scan Definition File: $panLocal"
	return -1
    }
    
    # Read in the file and parse the lines
    foreach line [split [read $fd] \n] {
	regsub {#(.*)} $line {} line
	    
	if {$line != ""} {
	    # Check if a variable is being defined
	    if {[string compare [lindex $line 0] set] == 0} {
		# Define the variable
		scan $line "%s %s %s" dummy gvarIndex value
		set gvar($gvarIndex) $value
	    } else {
		if {[llength $line] == 4} {
		    scan $line "%s %s %s %s" attr type gvarIndex cbFct
		} elseif {[llength $line] == 3} {
		    scan $line "%s %s %s" attr type gvarIndex
		    set cbFct ""
		} else {
		    puts "Line:\n\n\
			    $line\n\n\
			    in Pan Scan File:\n\n\
			    is illegal!"
		    return
		}
		if {[string length $attr] > 0} {
		    if {[string compare [string index $attr 0] "$"] == 0} {
			set index [string first "." $attr]
			set subSt [string range $attr 1 [expr {$index - 1}]]
			set attr "$gvar($subSt)[string range $attr $index end]"
		    }
		    if {[llength $line] == 4} {
			if {[string length [lindex $line 3]] > 0} {
			    set cbFct [lindex $line 3]
			} else {
			    set cbFct ""
			}
		    } else {
			set cbFct ""
		    }
		    # Set up the CB function
		    set ptAttrId [istGetAttribId $attr]
		    set gvar(panScanIndex${ptAttrId}) $gvarIndex
		    set gvar(panScanCbFct${ptAttrId}) $cbFct
		    set evtId [seq_evtAttach $attr $type \
			    {istPanScanUpdate %A %D}]
		    if {$evtId == -1} {
			istPanScanMsg "Error attaching event to attribute: \
				$attr"
			return -1
		    }
		    # Update the current value
		    set tmpVal [seq_dbReadSymbolic $attr]
		    if {[string length $gvar(panScanCbFct${ptAttrId})]} {
			set gvar($gvarIndex) \
				[$gvar(panScanCbFct${ptAttrId}) $tmpVal]
		    } else {
			set gvar($gvarIndex) $tmpVal
		    }
		}
	    }
	}
    }
}
    
proc istPanScanUpdate {attr value} {
    global gvar

    set ptAttrId [istGetAttribId $attr]
    if {[string length $gvar(panScanCbFct${ptAttrId})] > 1} {
	set val2 [$gvar(panScanCbFct${ptAttrId}) [lindex $value 2]]
	set gvar($gvar(panScanIndex${ptAttrId})) "$val2"
    } else {
	set gvar($gvar(panScanIndex${ptAttrId})) "[lindex $value 2]"
    }    

    return 0
}


# Get a unique Id from the attribut name. It is based on the direct address of
# the attribut.
proc istGetAttribId {attr} {

    # Get the direct address of the attribute
    # The direct address is unique in the database and it can be used to
    # identify the attribute without ambiguity.
    set dirAddr [seq_dbGetDirAddr $attr]

    # Strip the character "@"
    regsub {^@} $dirAddr "" dirAddr

    # Replace the characters '[' and ']' by "_"
    regsub -all {\[} $dirAddr "_" dirAddr
    regsub -all {\]} $dirAddr "_" dirAddr

    # Return the unique string
    return "$dirAddr"
}


# OBSOLETE FUNCTION - KEPT FOR BACKWARDS COMPATABILITY - SHOULD NOT BE USED!
proc irdosGetPtAttrib {attr} {
    return [string range $attr [string last ":" $attr] end]
}

# --- oOo ---
