#*************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: istSupportProc.tcl,v 1.49 2004/01/28 09:35:31 vltsccm Exp $" 
#
# istSupportProc.tcl
#
# who        when      what
# ---------  --------  ----------------------------------------------
# therlin    10/08/96  Created
# dpopovic   14/01/04  namespace for tcl_version > 8.0 only
#
#************************************************************************
#   NAME
#
#   istSupportProc - support procedures for instrument GUI panels
#
#   SYNOPSIS
#
#   istMenuReadFile <menu file>
#   istWindowOnTop  <window name>
#   istClassDialog  <uif class> <name> <title>
#
#   DESCRIPTION
#
#   istSupport is a series of support functions for developing instrument
#   GUI panels. There usage cover various area such as:
#
#    - dynamic menus
#    - window manager handling
#   
#   istMenuReadFile is a function that can add menu items which are described
#   in a menu file. The menu file is located in the $VLTDATA/config directory
#   and allows to add the menu elements: separator, command, checkbutton or 
#   radiobutton. 
#   The menu file has the following syntax:
#
#   menu  <menu name> separator
#   menu  <menu name> command     <text> <proc>
#   menu  <menu name> checkbutton <text> <proc> <var>
#   menu  <menu name> radiobutton <text> <proc> <var>
#
#   The <menu name> must already exist in the panel generated by panel editor.
#
#   istWindowOnTop is a procedure to allow windows to stay on top on the
#   master panel. This is especially usefull when developping dialog windows
#   where you want to avoid that the dialog 'disappears' behind the main 
#   panel.
#
#   istClassDialog build a dialog window out of a panel editor class.
#
#   GLOBAL VARIABLES
#  
#
#   ENVIRONMENT VARIABLES
#
#
#   EXAMPLES
#
#     Example of menu file:
#     ____________________________________________________________________
#     menu     Options   command     "Profile ..."   isouifProfile
#     menu     Options   command     "INS User ..."  istDummyCmd  
#     menu     Options   separator
#     menu     Options   radiobutton "No TCS"        isouifTcsMode tcsMode
#     menu     Options   radiobutton "TCS present"   isouifTcsMode tcsMode
#     menu     Status    command     "ICS ..."       {isouifStatus ICS}
#     menu     Status    command     "DCS ..."       {isouifStatus DCS}
#     menu     Status    command     "TCS ..."       {isouifStatus TCS}
#     menu     Commands  command     SETUP           isouifSetup
#     menu     Commands  separator
#     menu     Commands  command     START           isouifStart
#     menu     Commands  separator
#     menu     Commands  command     STOP            isouifStop
#     menu     Commands  separator
#     menu     Commands  command     ABORT           isouifAbort
#     ___________________________________________________________________
#
#     Example of using istClassDialog:
#
#     istClassDialog testPanelClass_uifClass test "This is a test dialog" 
#
#   SEE ALSO
#
#------------------------------------------------------------------------
#
############################################################
#  Set Environment variables
#
#

#
#  Set global variables
#

global env
global var


############################################################
#                  PROCEDURE    DEFINITION                 #
############################################################


         #    #  ######  #    #  #    #
         ##  ##  #       ##   #  #    #
         # ## #  #####   # #  #  #    #
         #    #  #       #  # #  #    #
         #    #  #       #   ##  #    #
         #    #  ######  #    #   ####

proc istMenuReadFile { file } {
    global env 
    set lineCount 0

    if {![info exist env(VLTDATA)]} {
	puts "Environment VLTDATA not specified !!!"
	return
    }
    set vltdata $env(VLTDATA)

    if {[file dir $file] == "."} {
	set file "$vltdata/config/$file"
    }
    if {![file exist $file]} {
	puts "Menu file $file does not exist !!!"
	return
    }

    #// Open the file for read
    set fd [open $file r]

    if { $fd < 0 } {
	puts "Could not open file: $file"
	return
    }

    #// Loop over the lines in the file.
    #// Each line correspond to a test
    while { [gets $fd line] >= 0} {

        #// Skips comments and empty lines
	if {[regexp {^[ ]*#.+|^#$} $line] != 0} {
	    #puts "Skipping line: $lineCount"
	    incr lineCount
	    continue
	}
	if {[llength $line] < 1} {
	    #puts "Skipping empty line: $lineCount"
	    incr lineCount
	    continue
	}	

	set type [lindex $line 0]
	
	set param [lrange $line 1 end]

	#puts "Line: $lineCount $line"

	switch $type {
	    menu { eval "istMenuAddItem $param"}
	    toolbar { eval "istToolbarAdd $param"}
	}
	
    }
    close $fd
}

proc istGetMenuPath { name } {
    if {[set mbpath [panGetMbPath $name]] == ""} {
	puts "internal error: cannot get widget name of $name menu. EXIT"
	return ""
    }
    return [$mbpath cget -menu]
}

proc istMenuAddItem { args } {

    set menuWdg [istGetMenuPath [lindex $args 0]]
    set len [llength $args]
    #puts "Menu $args lenght $len  $menuWdg"


    if { $menuWdg == "" } {
	return 0
    } 

    set type [lindex $args 1]
    switch $type {
	command {
	    if { $len != 4 } {
		puts "istMenuAddItem: wrong number of arguments for $type expected 4 got $len"
	    } else {
		$menuWdg add $type -label [lindex $args 2] -command [lindex $args 3]
	    }
	}
	checkbutton { 
	    if { $len != 5 } {
		puts "istMenuAddItem: wrong number of arguments for $type expected 5 got $len"
	    } else {
		$menuWdg add $type  -label [lindex $args 2] \
		-command [lindex $args 3] -variable [lindex $args 4]
	    }
	}
	radiobutton { 
	    if { $len != 5 } {
		puts "istMenuAddItem: wrong number of arguments for $type expected 4 got $len"
	    } else {
		$menuWdg add $type  -label [lindex $args 2] \
			-command [lindex $args 3] -variable [lindex $args 4]
	    }
	}
	separator   { 
	    $menuWdg add $type  
	} 
    }
	
}
proc OLDistMenuAddItem { menu type {text "Dummy"} {var "dummy"} {proc istDummyCmd} } {

    set menuWdg [istGetMenuPath $menu]

    #puts "Menu $menu $menuWdg $type $text $var $proc"

    if { $menuWdg == "" } {
	return 0
    } 

    switch $type {
	command {
	    $menuWdg add $type -label $text -command $proc
	}
	checkbutton { $menuWdg add $type -label $text -command $proc -variable $var}
	radiobutton { $menuWdg add $type -label $text -command $proc -variable $var}
	separator   { $menuWdg add $type  } 
    }
	    # 
}

proc istDummyCmd {{args1 ""}} {

    puts "istDummy: $args1"
}

proc istToolbarAdd { bitmap {proc istDummyCmd} } {
    global toolBar

    set fileName [findVLTFile bitmaps/$bitmap]

    if {$fileName == ""} {
	puts "istToolbarAdd: could not find file for bitmap $bitmap"
    }

    if {![info exists toolBar(bmpCnt)]} {
	set toolBar(bmpCnt) 0
    }

    set ext [file extension $fileName]
    if { $ext == ".pm" || $ext == ".xpm" } {
	button $toolBar(frame).$toolBar(bmpCnt) \
		-image [image create pixmap -file $fileName] \
		-command "$proc"
    } 

    if { $ext == ".xbm" } {
	button $toolBar(frame).$toolBar(bmpCnt) -bitmap "@$fileName" \
		-command "$proc"
    }

    pack $toolBar(frame).$toolBar(bmpCnt) -side left -anchor w

    incr toolBar(bmpCnt)
}



         ######   ####   #####   #    #    ##     #####
         #       #    #  #    #  ##  ##   #  #      #
         #####   #    #  #    #  # ## #  #    #     #
         #       #    #  #####   #    #  ######     #
         #       #    #  #   #   #    #  #    #     #
         #        ####   #    #  #    #  #    #     #


#//  istFormatHms formats a double value of type HHMMSS.ss
#//  to HH:MM:SS.ss
#//
proc istFormatHms { {value ""} } {

    #puts "Called istFormatHms with: $value"
   
    if {$value == ""} { return "--:--:--"}

    set HH 00
    set MM 00
    set SS 00
    set ss ""
    set sign ""

    set list [split $value .]

    set HHMMSS [lindex $list 0]
    if {[string range $HHMMSS 0 0] == "-"} {
	set HHMMSS [string range $HHMMSS 1 end]
	set sign "-"
    }

    if {[string length $HHMMSS] == 6} {
	set HH [string range $HHMMSS 0 1]
	set HHMMSS [string range $HHMMSS 2 5]
    }

    if {[string length $HHMMSS] == 5} {
	set HH "0[string range $HHMMSS 0 0]"
	set HHMMSS [string range $HHMMSS 1 4]
    }

    if {[string length $HHMMSS] == 4} {
	set MM [string range $HHMMSS 0 1]
	set HHMMSS [string range $HHMMSS 2 3]
    }

    if {[string length $HHMMSS] == 3} {
	set MM "0[string range $HHMMSS 0 0]"
	set HHMMSS [string range $HHMMSS 1 2]
    }

    set SS $HHMMSS

    if {[llength $list] == 2} { set ss ".[lindex $list 1]" }

    return "$sign$HH:$MM:$SS$ss"
}

 #####  ####### #     # #     #    #    #     # ######
#     # #     # ##   ## ##   ##   # #   ##    # #     #
#       #     # # # # # # # # #  #   #  # #   # #     #
#       #     # #  #  # #  #  # #     # #  #  # #     #
#       #     # #     # #     # ####### #   # # #     #
#     # #     # #     # #     # #     # #    ## #     #
 #####  ####### #     # #     # #     # #     # ######


proc istCmdDefaults {env server} {

    global gvar  seq_ccsCmdCheck

    set gvar(error) 0
    set gvar(errorTxt) ""
    set seq_ccsCmdCheck CHECK_ALL
    set gvar(istEnv) $env
    set gvar(istServer) $server
}

proc istCmdSend {cmd par} {
    global gvar  seq_ccsCmdCheck

    set lr 0
    set gvar(error) 0
    set gvar(errorTxt) ""
    set seq_ccsCmdCheck CHECK_ALL
##debug
    set error [catch {set cmdId [seq_msgSendCommand $gvar(istEnv) \
	    $gvar(istServer) $cmd $par]}]
    #puts "Error: $error"
    # Could not send command return with empty string
    if {$error != 0} {
	set gvar(error) 1
	puts "istSendCmd failed -$gvar(istEnv)- -$gvar(istServer)- -$cmd-"
	seq_logData "istSendCmd failed : $cmd $par"
	return 0
    }

    set reply [seq_msgRecvReply $cmdId error lr -last]

    set gvar(error) $error
    if {$gvar(error) != 0} {
	puts "Error $gvar(error) occured: $reply"
    }
    return $reply
}


        #     # #     #
        #  #  # ##   ##
        #  #  # # # # #
        #  #  # #  #  #
        #  #  # #     #
        #  #  # #     #
         ## ##  #     #

proc istWindowOnTop { new {master .} } {
    
    #// handle the wm properties so that the window can't be overlayed
    wm gro $new $master
    wm tran $new $master
    wm withdraw $new
    wm deico $new
}

 #####      #      ##    #        ####    ####
 #    #     #     #  #   #       #    #  #    #
 #    #     #    #    #  #       #    #  #
 #    #     #    ######  #       #    #  #  ###
 #    #     #    #    #  #       #    #  #    #
 #####      #    #    #  ######   ####    ####

proc istClassDialog {class name title {modal 0} {geo +600+450}} {
    global tk_priv shArea

    set w .$name

    #// 1. Create the top-level window 
    catch {destroy $w}
    toplevel $w -class Dialog

    
    #// 2. handle the wm properties so that the window can't be overlayed
    istWindowOnTop $w 

    #// 3. set title
    wm title $w $title
    wm iconname $w Dialog
    pack [frame $w.f -relief raised -bd 2]

    #// 4. call panel widget class and use same name for instance 
    #//    this is important for using global widget variables
    $class $w.f.$name
    pack $w.f.$name  -fill both

    #// 5. Remember to initialize panel items
    #//    and to display them
    $w.f.$name init
    $w.f.$name configure -helparea $shArea
    $w.f.$name display

    set oldFocus [focus]
    #grab $w
    focus $w

    #// 6. If modal wait for the user to respond, then restore the focus and
    #// return the index of the selected button.

    if { $modal == 1 } {
	grab $w
	tkwait variable tk_priv(button)
	destroy $w
	focus $oldFocus
	return $tk_priv(button)
    }    
    catch {wm geometry $w $geo}
    return $w    
}

# this is a wrapper for "namespace tail" that checks which Tcl version
# we are using

proc istNamespaceTail {ns} {
    return [namespace tail $ns]
}

############################################################
# END
############################################################

