#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: moteiDatabase.tcl,v 2.43 2005/04/19 12:32:59 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# imunoz    14/04/97  (moteiSaveDatabase) - saves only configuration data.
# imunoz    14/04/97  removed proc dbBackup
# imunoz    05/03/97  SPR 960782 - backups all points:CFG under <MOTOR>:SDL
# pduhoux   31/08/95  created
#

#************************************************************************
#   NAME
#       moteiLoadDatabase, moteiSaveDatabase - Backup/Restore DB to/from file
#
#   SYNOPSIS
#       moteiLoadDatabase <dbPath>
#       moteiSaveDatabase <dbPath>
#
#   DESCRIPTION
#       The procedure moteiLoadDatabase asks for a .dbcfg file selection and
#       restores the DB branch <dbPath> from this file. 
#
#       The procedure moteiSaveDatabase asks for a .dbcfg file selection and
#       backups the DB branch <dbPath> to this file. 
#
#       Internally both procedures invoke a file selector which path is kept
#       static starting at PWD.
#
#       Together with motInitDb(1), it is recommended to create backup files 
#       names <motorName=alias of DB branch>.dbcfg located under 
#       $INS_ROOT/SYSTEM/COMMON/CONFIGFILES or $[VLT|INT]ROOT/config
#       
#   FILES
#       The file containing the list of attributes to be backup is created
#       as /tmp/motSaveDb_<pid>.inp
#       It is removed at backup completion.
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#       The DB branch must contain the points SDL:<sdl> as stored in the file.
#       Ie, the configuration of a motor for a given HW configuration may not
#       be restored in a DB branch of a different HW.
#
#   EXAMPLES
#
#   SEE ALSO
#       dbRestore(1), dbBackup(1)
#       motInitDb(1), motSaveDb(1)
#
#   BUGS     
#
#------------------------------------------------------------------------
#
proc moteiSelectFileName { } {                                        
    global gvar

    set dir "."
    if {![info exists gvar(moteiDirectory)]} { set gvar(moteiDirectory) "." }

    set dir $gvar(moteiDirectory)
    set flt "*.dbcfg"

    set path [panSelectFile $dir $flt]
    set gvar(moteiConfFile) [file tail    $path]
    set dir                 [file dirname $path]
	
    if {![cequal $dir ""] && [file isdirectory $dir] == 1} {
	set gvar(moteiDirectory) $dir
    }
}

proc moteiLoadDatabase { dbPath } {                                        
    global gvar

    if {[cequal $dbPath ""]} { 
	moteiDialogBox "ERROR" "Invalid DB path"
	return 
    }

    # set CWP
    if {[catch {seq_dbSetCwp "$dbPath"}]} {
	seq_errDisplay -suspend
	return
    }
    
    # Select file name
    moteiSelectFileName 

    # Build full file name
    set file $gvar(moteiConfFile)
    set dir  $gvar(moteiDirectory)
    if {[cequal $file ""]} return
    if {[cequal $dir ""]} {
	set path [format "./%s" $file]
    } else {
	set path [format "%s/%s" $dir $file]
    }

    # Check file exists and is readable
    if {[file exists $path]} {
	if {![file readable $path]} {
	    moteiDialogBox "ERROR" "'$path': Permission denied"
	    return
	}
    } else {
	moteiDialogBox "ERROR" "'$path': No such file or directory "
	return
    }
    moteiDoRestore $path $dbPath
}

proc moteiDoRestore { path dbPath} {
    global gvar 

    set cwpList [exec grep CWP $path]
    set cwpOrig [lindex $cwpList 1]
    set cwpDest $dbPath

    set ep [moteiGetEnvDir $path]
    puts -nonewline "Restore database branch '$dbPath' from file '$ep' ... " 
    flush stdout
    set message "Restore database branch\n  \
              from file: '$ep'\n  \
              to point : '$dbPath'"
    vccWinBusy . $message {
	catch [list exec dbRestore -f $path -c "$cwpOrig=$cwpDest"]
    }
    puts "done"
}

proc moteiSaveDatabase { dbPath } {                                        
    global gvar

    if {[cequal $dbPath ""]} { 
	moteiDialogBox "ERROR" "Invalid DB path"
	return 
    }

    # set CWP
    if {[catch {seq_dbSetCwp "$dbPath"}]} {
	seq_errDisplay -suspend
	return
    }
    
    # Select file name
    moteiSelectFileName

    # Build full file name
    set file $gvar(moteiConfFile)
    set dir  $gvar(moteiDirectory)
    if {[cequal $file ""]} return
    if {[cequal $dir ""]} {
	set path [format "./%s" $file]
    } else {
	set path [format "%s/%s" $dir $file]
    }

    # Check file exists and writable
    if {[file exists $path] == 1} {
	if {[file writable $path] == 0} {
	    moteiDialogBox "ERROR" "'$path': Permission denied"
	    return
	}
    }

    # Create a dbBackInFile
    set processId [pid]
    set fname "/tmp/motSaveDb_$processId.inp"

    set fd [open $fname w+]
    if {[lempty $fd]} {
	moteiDialogBox "ERROR" "Could not create $fname"
	return
    }

    #
    # put server attributes to be backup in /tmp/motSaveDb_<processId>.inp
    #
    puts $fd "ATTRIBUTE SERVER:CONF.motor"
    puts $fd "ATTRIBUTE SERVER:CONF.boards"
    puts $fd "ATTRIBUTE SERVER:CONF.events"
    puts $fd "ATTRIBUTE SERVER:CONF.signals"
    puts $fd "ATTRIBUTE SERVER:CONF.brakeAtBoot"
    puts $fd "ATTRIBUTE SERVER:CONF.axis"
    puts $fd "ATTRIBUTE SERVER:CONF.encoder"
    puts $fd "ATTRIBUTE SERVER:CONF.specialPositions"
    puts $fd "ATTRIBUTE SERVER:CONF.offsets(0,maint:unit)"
    puts $fd "ATTRIBUTE SERVER:CONF.namedPositions(0:$,name:posUnit)"
    puts $fd "ATTRIBUTE SERVER:CONF.namedSpeeds(0:$,name:inPosTime)"
    puts $fd "ATTRIBUTE SERVER:CONF.twoStepOffset(0,offset:inPosTime)"
    puts $fd "ATTRIBUTE SERVER:CONF.indexPulseSpeed(0,speedHow:inPosTime)"
    puts $fd "ATTRIBUTE SERVER:CONF.defaultSpeed"
    
    puts $fd "ATTRIBUTE SERVER:INIT.actions(0:$,actionType)"
    puts $fd "ATTRIBUTE SERVER:INIT.actions(0:$,userFctName)"
    puts $fd "ATTRIBUTE SERVER:INIT.actions(0:$,initCode)"
    puts $fd "ATTRIBUTE SERVER:INIT.actions(0:$,motionMode)"
    puts $fd "ATTRIBUTE SERVER:INIT.actions(0:$,posHow:posType)"
    puts $fd "ATTRIBUTE SERVER:INIT.actions(0:$,speedHow:inPosTime)"
    
    puts $fd "ATTRIBUTE SERVER:UNIT.defaultUnits"
    puts $fd "ATTRIBUTE SERVER:UNIT.unitConversion(0:$,name:method)"
    puts $fd "ATTRIBUTE SERVER:UNIT.unitConversion(0:$,userFctName)"
    puts $fd "ATTRIBUTE SERVER:UNIT.unitConversion(0:$,offset:slope)"
    puts $fd "ATTRIBUTE SERVER:UNIT.unitConversion(0:$,numPoints:y50)"
  
    # put SDL<XXX>:CFG points in .dbBackupInFile
    set gap [string length <relative>]
    set sdl [seq_dbGetFamilyNames $dbPath:SDL RELATIVE]
    lvarpop sdl
    foreach el $sdl {
	puts $fd [string range $el:CFG.device $gap end]
	puts $fd [string range $el:CFG.configValues $gap end]
    }
    close $fd
    set ep [moteiGetEnvDir $path]
    puts -nonewline "Backup database branch '$dbPath' to file '$ep' ... "
    flush stdout
    set message "Backup database branch\n  \
	    from point: '$dbPath'\n  \
	    to   file : '$ep'"
    vccWinBusy . $message {
	catch [exec dbBackup \
		-c $dbPath -f $fname -o $path]
    }
    puts "done"

    file delete $fname
}

proc moteiGetEnvDir { dir } {
    global gvar env

    set gvar(vltroot) "." ; set gvar(vltdata) "."
    set gvar(introot) "." ; set gvar(insroot) "."
    if {![info exists gvar(vltroot)]} {
	if {[info exists env(VLTROOT)]} { set gvar(vltroot) $env(VLTROOT) }
    } 
    if {![info exists gvar(vltdata)]} {
	if {[info exists env(VLTDATA)]} { set gvar(vltdata) $env(VLTDATA) }
    } 
    if {![info exists gvar(introot)]} {
	if {[info exists env(INTROOT)]} { set gvar(introot) $env(INTROOT) }
    } 
    if {![info exists gvar(insroot)]} {
	if {[info exists env(INS_ROOT)]} { set gvar(insroot) $env(INS_ROOT) }
    } 
    set vltroot $gvar(vltroot) ; set vltdata $gvar(vltdata)
    set introot $gvar(introot) ; set insroot $gvar(insroot)
	
    set edl [list $vltroot $vltdata $introot $insroot]
    set edn [list VLTROOT VLTDATA INTROOT INS_ROOT]
    set i 0
    foreach ed $edl {
	if {![cequal $ed "."]} {
	    set l [string length $ed]
	    append ed "/"
	    if {[string compare [string range $dir 0 $l] $ed] == 0} {
		set s1 [lindex $edn $i] 
		set s2 [string range $dir $l end]
		set new $s1 ; append new $s2
		return "$$new"
	    }
	}
	incr i
    }
    return $dir
}

#
# ___oOo___
