#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginBASE.tcl,v 1.105 2005/10/12 19:47:32 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  05/01/03  VLTSW20020495: trap pkginEnvGet error.
# rschmutz  04/01/03  env. var. SAVED_PS1 added to envExclude.
# rsc/alo   03/09/01  for_array_keys replaced with foreach (for Tcl 8.3).
# rschmutz  25/06/01  variable logEndInfo added.
# rschmutz  24/06/01  SourceEnvVars added.
# rschmutz  14/05/01  GetDiskSpace, GetDirSize added.
# rschmutz  11/05/01  CheckOption, CheckOptions added.
# rschmutz  11/10/00  ErrorInfo method added.
# rschmutz  18/09/00  Error handling methods Error* added.
# rschmutz  13/05/00  renamed, updated, man page added.
# sskole    27/02/00  Simplified substString.
# sskole    16/12/99  Moved m_compress from pkginCInstallation to here.
# sskole    18/11/99  Added from pkginCInstallation: 
#                     logStart, overrideClass, setEnvPaths, abort, 
#                     checkAuthorization, sourceScript
# sskole    18/11/99  Moved m_sep and m_sep2 from pkginCLog to here.
# sskole    26/08/99  Depending on INSTALL.FLAG.DELETEDIR directories are
#                     now deleted in renameDir and backupDir
# sskole    25/08/99  The methods getIntermediateStatus and getStepStatus were added  
# sskole    03/07/99  created
#

#************************************************************************
#   NAME
#	pkginBASE - base for pkgin classes
# 
#   SYNOPSIS
#	pkginBASE name [-logName <name>]
#		       [-logDir <directory>]
#		       [-loglevel <level>]
# 
#   DESCRIPTION
#	pkginBASE contains useful methods for the pkgin scripts.
#	This class is used as base class for most other pkgin
#	classes.
#
#   PARENTS
#	pkginLOG
#
#   PUBLIC METHODS
#	-
#
#   PROTECTED METHODS
#	GetOptions <optionList> <options>
#	    Reads command line options.
#	    The accepted options are listed in optionList,
#	    e.g.: {-proc <name> -all}, with -all a boolean.
#	    The corresponding public class variables are set,
#	    e.g.: configure proc "<name>", configure all 0|1.
#	    Options -objList and -objShow are ignored.
#
#	BackupFile <file> <maxbaks> [<file2>]
#	    Makes a backup of a file or directory.
#	    If more than <maxbaks> backups are present, then
#	    the oldest backups are deleted, i.e. if <maxbaks>
#	    is 0, then no backups are made.
#	    Normally, a backup is given the name:
#	    '<file>.YYYY-MM-DDThh:mm:ss'
#	    If <file2> is supplied, then the backup is given
#	    the name: '<file2>' (and older backups are given
#	    the name: '<file2>.YYYY-MM-DDThh:mm:ss').
#
#	DeleteBackup <file> <maxbaks>
#	    If more than <maxbaks> files are present with the
#	    name format: '<file>.YYYY-MM-DDThh:mm:ss', then
#	    the oldest files are deleted.
#
#	Now
#	    Returns the current time with format:
#	    'YYYY-MM-DDThh:mm:ss'
#
#	Logical2Num <value>
#	    Converts 'T'/'F' to 1/0.
#
#	CheckOption <key> <val> <optList>
#	    Check that <val> is one of the options in
#	    <optList>, otherwise raise an error.
#
#	CheckOptions <key> <valList> <optList>
#	    Check that the list <valList> is a subset of
#	    the <optList> list, otherwise raise an error.
#
#	EvalStatus <cmd>
#	    Executes <cmd> and prints "OK" or "FAILURE"
#	    (using Info) according to the success of
#	    failure of the command.
#
#	ExistsEnv <name>
#	GetEnv <name>
#	SetEnv <name> <value>
#	    Specify if an environment variable exists,
#	    get/set an environment variable.
#	SourceEnvVars
#	    Simulates (to some extend) a user login and
#	    compares the env. variables with the current
#	    ones.  Sets new env. variables and returns
#	    the list of updated env. variables.
#
#	SearchFile <file_and_args> [<dirs>]
#	    Search the file specified as first argument in
#	    <file_and_args> in the directories specified in
#	    <dir>.  Returns the original <file_and_args>,
#	    contents, but including the absolute path in the
#	    first argument of <file_and_args>.
#	    <dir> may contain the strings: '$VLTROOT',
#	    '$INTROOT', '$VLTDATA' or '$INS_ROOT' to refer
#	    to the corresponding VLT directories.
#	    If <dir> is not specified, then the file is
#	    searched in: '$INTROOT/bin' and '$VLTROOT/bin'.
#	    Example:
#		SearchFile "xxinsInstallHook arg1 arg2"
#		-> "/introot/XXXX/bin/xxinsInstallHook arg1 arg2"
#
#	ErrorBegin <title> <header>
#	ErrorSet <txt>
#	ErrorInfo <txt>
#	ErrorSetHeader <header>
#	ErrorTryEval <errorResult> <errorCode> <errorInfo>
#	ErrorEnd
#	    Error handling functions.
#	    These functions allow to register errors, but to
#	    continue processing until the end of the error
#	    block is reached (see the template below).
#
#	    ErrorBegin marks the start of an error block.
#
#	    ErrorSet is used to report an error, without
#	    leaving the block (use Tcl function 'error ...'
#	    to leave the block immediately).
#	    ErrorSet calls Info pre-appending the <header>
#	    the first time ErrorSet is called within the 
#	    block.
#
#	    ErrorInfo is similar to ErrorSet, but is used
#	    to report information or a warning.
#
#	    ErrorSetHeader allows to change the <header>.
#
#	    ErrorTryEval must be located inside the try_eval
#	    function as shown in the template below.
#
#	    ErrorEnd marks the end of the error block.
#
#	    The <title> identifies the error block and is 
#	    pre-appended to the text <txt>, when <txt> is
#	    reported as an error.
#
#	    These methods are used by the pkginVerify steps.
#
#	    Info "..." 0
#	    ErrorBegin ...
#	    try_eval {
#		...
#	        if { ... } {
#		    ErrorSet ...
#		}
#		...
#	    } {
#		ErrorTryEval
#	    }
#	    ErrorEnd
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: pkginBASE
#------------------------------------------------------------------------

class pkginBASE {
    inherit pkginLOG

    #****************************************************************
    #*# method: GetOptions
    #----------------------------------------------------------------

    protected method GetOptions { optionList options } {

	#*# 1. parse optionList
	set optionNames {-objList -objShow}
	set optionHasValue {2 2}
	set len [llength $optionList]
	set i 0
	set elem [lindex $optionList $i]
	while { $i < $len } {
	    if { "[crange $elem 0 0]" != "-" } {
		Exit "Internal error: syntax error in the option list '$optionList'"
	    }
	    lappend optionNames $elem
	    set hasValue 0
	    incr i
	    set elem [lindex $optionList $i]
	    if { "[crange $elem 0 0]" == "<" } {
		set hasValue 1 
		incr i
		set elem [lindex $optionList $i]
	    }
	    lappend optionHasValue $hasValue
	}

	#*# 2. parse options
	set len [llength $options]
	set i 0
	set elem [lindex $options $i]
	while { $i < $len } {
	    set j [lsearch $optionNames $elem]
	    if { $j == -1 } {
		Exit "Wrong option $elem (options are: $optionList)"
	    }
	    if { [lindex $optionHasValue $j] != 0 } {
		set valueList ""
		incr i
		set value [lindex $options $i]
		while { "$value" != "" && "[crange $value 0 0]" != "-" } {
		    lappend valueList $value
		    incr i
		    set value [lindex $options $i]
		}
	    	if { [lindex $optionHasValue $j] != 2 } {
		    $this configure $elem $valueList
		}
		set elem $value
	    } else {
		$this configure $elem 1
		incr i
		set elem [lindex $options $i]
	    }
	}

    }

    #****************************************************************
    #*# method: BackupFile
    #----------------------------------------------------------------

    protected method BackupFile { file maxbaks {file2 {}} } {

    	#*# 1. if file does not exist: return
	if {![file exists $file]} {
	    return
	}

	#*# 2. if maxbaks <= 0: delete file
	if { $maxbaks <= 0 } { 
	    file delete -force $file
	    return
	}

    	#*# 3. delete oldest backup
	incr maxbaks -1
	if { "$file2" != "" } {
	    DeleteBackup $file2 $maxbaks 1
	} else {
	    DeleteBackup $file $maxbaks
	}

	#*# 4. backup file
	if { "$file2" != "" } {
	    if {[file exists $file2]} {
	    	file stat $file2 fileStats
		set date [clock format $fileStats(ctime) \
			 -format "%Y-%m-%dT%H:%M:%S"]
	        file rename $file2 $file2.$date
	    }
	    set bakfile $file2
	} else {
	    file stat $file fileStats
	    set date [clock format $fileStats(ctime) \
		     -format "%Y-%m-%dT%H:%M:%S"]
	    set bakfile $file.$date
	}
	file rename $file $bakfile
    }

    #****************************************************************
    #*# method: DeleteBackup
    #----------------------------------------------------------------

    protected method DeleteBackup { file maxbaks {includeFile 0} } {

    	#*# 1. if maxbaks == 0: do nothing
	if { $maxbaks <= 0 } {
	    return
	}

	#*# 2. search backups
	set dir  [file dirname $file]
	set tail [file tail $file]
	# escape '+' chars.
	regsub -all {\+} $tail {\+} regtail
        set exp [cconcat $regtail {.[1-2][0-9][0-9][0-9]-[0-1][0-9]-[0-3][0-9]T[0-9][0-9][-:][0-9][0-9][-:][0-9][0-9]$}]
        set files ""
        foreach f [readdir $dir] {
            if { ("$f" == "$tail" && $includeFile != 0) || [regexp $exp $f] == 1 } {
	    	set path [file join $dir $f]
                file stat $path fileStats
                lappend files [list $path $fileStats(ctime)]
            }
        }
        set files [lsort -integer -index 1 -decreasing $files]

	#*# 3. if too many backups: delete oldest backups
        set files [lrange $files $maxbaks end]
        foreach f $files {
            file delete -force [lindex $f 0]
        }
    }

    #****************************************************************
    #*# method: Now
    #----------------------------------------------------------------

    protected method Now {} {
	return [clock format [clock seconds] -format "%Y-%m-%dT%H:%M:%S"]
    }

    #****************************************************************
    #*# method: Logical2Num
    #----------------------------------------------------------------

    protected method Logical2Num { val } {

    	if { "$val" == "F" } {
	    return 0
	} elseif { "$val" == "T" } {
	    return 1
	}
	error "'$val' is not a boolean value."
    }

    #****************************************************************
    #*# method: CheckOption
    #----------------------------------------------------------------

    protected method CheckOption { key val optList } {
        if { ![lcontain $optList $val] } {
            error "Wrong value $key = $val.  Expected one of: $optList."
        }
    }

    #****************************************************************
    #*# method: CheckOptions
    #----------------------------------------------------------------

    protected method CheckOptions { key valList optList } {
	foreach val $valList {
            if { ![lcontain $optList $val] } {
                error "Wrong value $key = $val.  Expected one of: $optList."
            }
	}
    }

    #****************************************************************
    #*# method: EvalStatus
    #
    #   Evaluate cmd. and print OK or FAILURE, according
    #   if the cmd. evaluates ok or returns an error.
    #----------------------------------------------------------------

    protected method EvalStatus { cmd } {
    	try_eval {
	    eval $cmd
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: ExistsEnv
    #----------------------------------------------------------------

    protected method ExistsEnv { name } {
        global env
	return [info exists env($name)]
    }

    #****************************************************************
    #*# method: GetEnv
    #----------------------------------------------------------------

    protected method GetEnv { name } {
        global env
	return $env($name)
    }

    #****************************************************************
    #*# method: SetEnv
    #----------------------------------------------------------------

    protected method SetEnv { name value } {
        global env
	set env($name) $value
    }

    #****************************************************************
    #*# proc: SourceEnvVars
    #----------------------------------------------------------------

    protected method SourceEnvVars {} {
	global env

    	#*# 1. get current env. variables
	#
	#   ERASE and LOGFILE may be modified, if pkginBuild
	#   is executed under a remsh.  This can be ignored.
	Info "       Setting env. variables."
	set envExclude {"PS1" "SHLVL" "_" "PWD" "NO_ERR_DISPLAY" 
			"PECS_CONFIG_ROOTS" "CPU" "VX_CPU_FAMILY"
			"SAVED_PS1" "ERASE" "LOG_FILE"}

	# 'for_array_keys key env' fails with Tcl 8.3.
	# foreach below is equivalent and works.
	set envList1 ""
	foreach key [array names env] {
	    set key [string trim $key]
	    if { ![lcontain $envExclude $key] && $key != "" } {
        	set val [string trim $env($key)]
        	lappend envList1 [list $key $val]
    	    }
	}

    	#*# 2. get login env. variables
	#
	#      unset VLTSW_RELEASE and VLTSW_CCSTYPE
	#      to force PECS to bootstrap the VLT application.
	#
	catch { unset env(VLTSW_RELEASE) }
	catch { unset env(VLTSW_CCSTYPE) }
	set envList2 ""
	try_eval {
	    set loginEnvVars [exec pkginEnvGet]
	} {
	    # Error encountered: 
	    # Probably due to remsh -n pkginBuild execution 
	    # (see SPR VLTSW20020495).
	    # In this case stty in the simulated login fails.
	    # The env. vars. may end up anyway in variable $errorInfo.
	    set loginEnvVars $errorInfo
	}
	set loginEnvVars [split $loginEnvVars "\n"]
	foreach envLine $loginEnvVars {
	    set envLine [string trim $envLine]
	    if { $envLine == "" || [string first "=" $envLine] == -1 } {
	        continue
	    }
	    set key [string trim [ctoken envLine "="]]
	    if { ![lcontain $envExclude $key] } {
	        set val [string trim [ctoken envLine "="]]
	        lappend envList2 [list $key $val]
	    }
	}

	#*# 3. set missing env. vars.
	set envSetList [lindex [intersect3 $envList1 $envList2] 2]
	set ret ""
	if { $envSetList != "" } {
	    foreach envSet $envSetList {
	        lassign $envSet key val
	        if { $key == "INTROOT" } {
	    	    error "Please set INTROOT in e.g. .pecs/apps-all.env.  Do not set it manually."
	        }
	        set env($key) $val
		lappend ret $key
	    }
	}

	return $ret
    }

    #****************************************************************
    #*# method: SearchFile
    #----------------------------------------------------------------

    protected method SearchFile { fileName {dirs {$INTROOT/bin $VLTROOT/bin}} } {
    	global env

    	set arg $fileName
    	set name [ctoken arg " \t"]
    
    	if {[file isfile $name]} {
    	    return $fileName
    	}
	foreach dir $dirs {
	    catch { regsub -all {\$INS_ROOT} $dir $env(INS_ROOT) dir }
	    catch { regsub -all {\$INTROOT}  $dir $env(INTROOT)  dir }
	    catch { regsub -all {\$VLTROOT}  $dir $env(VLTROOT)  dir }
	    catch { regsub -all {\$VLTDATA}  $dir $env(VLTDATA)  dir }
    	    set path [file join $dir $name]
    	    if {[file isfile $path]} {
    	        return [string trim "$path $arg"]
	    }	
	}
	error "File $name not found."
    }

    #****************************************************************
    #*# method: GetDiskSpace
    #----------------------------------------------------------------

    protected method GetDiskSpace { dir } {

    	#*# 1. execute df (or bdf)
	set dspace -1
	switch -- [exec uname] {
	    "HP-UX" {
	    	catch { set dspace [lindex [exec bdf $dir] 10] }
	    }
	    "SunOS" {
	    	catch { set dspace [lindex [exec df -b $dir] 3] }
	    }
	    "Linux" {
	    	catch { set dspace [lindex [exec df -k $dir] 10] }
	    }
	    default {
	    	error "Unknown operating system."
	    }
	}
	if { $dspace == -1 } {
	    error "Error obtaining the disk space of $dir."
	}

	return $dspace
    }

    #****************************************************************
    #*# method: GetDirSize
    #----------------------------------------------------------------

    protected method GetDirSize { dir } {
    	set dspace -1
    	catch { set dspace [lindex [exec du -sk $dir] 0] }
	if { $dspace == -1 } {
	    error "Error obtaining the size of directory $dir."
	}

	return $dspace
    }

    #****************************************************************
    #*# Error handling methods
    #
    #   Info "..." 0
    #   ErrorBegin ...
    #   try_eval {
    #       ...
    #       if { ... } {
    #		ErrorSet ...
    #	    }
    #       ...
    #   } {
    #       ErrorTryEval
    #   }
    #   ErrorEnd
    #----------------------------------------------------------------

    protected method ErrorBegin { title header } {
    	set errorList ""
	set errorTitle $title
	set errorHeader $header
    }

    protected method ErrorInfo { txt } {
	Info "$errorHeader         $txt"
	set errorHeader ""
    }

    protected method ErrorSet { txt } {
	Info "$errorHeader         $txt"
	append errorList "$errorTitle$txt\n"
	set errorHeader ""
    }

    protected method ErrorSetHeader { header } {
    	if { $errorHeader != "" } {
	    set errorHeader $header
	}
    }

    protected method ErrorTryEval { errorResult errorCode errorInfo } {
	if { $errorList != "" } {
	    LogErr [csubstr $errorList 0 end]
	}
	Info "$errorHeader         $errorResult"
	LogErr "$errorTitle$errorResult"
	error "$errorList$errorTitle$errorResult\n" $errorCode $errorInfo
    }

    protected method ErrorEnd {} {
	if { $errorList != "" } {
	    LogErr [csubstr $errorList 0 end]
	    error $errorList
	}
	if { $errorHeader != "" } {
	    Info "OK."
	}
    }

    protected variable errorList ""
    protected variable errorTitle ""
    protected variable errorHeader ""

    #****************************************************************
    #*# user variables
    #----------------------------------------------------------------

    protected common logEndInfo ""

}

#
# ___oOo___
