#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginDCS.tcl,v 1.105 2005/10/12 19:47:32 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-04-08  SPR VLTSW20050066: replaced chown with rcp.
# rschmutz  2005-01-13  SPR VLTSW20040922: use $ccdname instead of $name.
# rschmutz  2004-12-19  SPR VLTSW20040292: change config. file ownership.
# rschmutz  2003-11-22  SPR VLTSW20030477: catch scan creation errors.
# rschmutz  2003-01-04  do not rsh/remsh to same user.
# rschmutz  2001-05-14  use rsh/remsh according to operating system.
# rschmutz  2001-03-04  sleeps added as workaround for SPR 20000480.
# rschmutz  2001-01-08  InstallScanLinks: parameter clean added.
# rschmutz  2001-01-07  delay added before WCS config.
# rschmutz  2000-12-27  WCS handling updated.
# rschmutz  2000-12-08  IRACE support added.
# rschmutz  2000-09-15  cfgfile -> cfgset.
# rschmutz  2000-05-13  created.
#

#************************************************************************
#   NAME
#	pkginDCS - detector
# 
#   SYNOPSIS
#	pkginDCS name -name <name>
# 
#   DESCRIPTION
#	pkginDCS is used to build and install a standard DCS.
#
#   PARENT
#	ctooOBJ_BASE pkginLOG
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure a software module from a configuration file.
#	    <cfg> should be an instance of type pkginCONFIG.
#	    The following config. FITS keywords are expected
#	    and stored in the public data described below:
#		INSTALL.DET<index>.NAME    -> name
#	    	INSTALL.DET<index>.TYPE    -> type
#	        INSTALL.DET<index>.CCDNAME -> ccdname (ACE,FIERA only)
#	        INSTALL.DET<index>.ENVNAME -> envname (optional)
#		INSTALL.DET<index>.CCDLENV -> ccdlenv (ACE,FIERA only)
#		INSTALL.DET<index>.DBFILE  -> dbfile  (ACE,FIERA only)
#		INSTALL.DET<index>.WCS     -> wcs     (optional)
#
#	    The detector data may be stored in an alternative
#	    configuration file.  In this case, <cfg> should
#	    contain only:
#		INSTALL.DET<index>.NAME    -> name
#	    	INSTALL.DET<index>.CFGSET  -> cfgset
#
#	    The alternative configuration file should contain
#	    the following keywords:
#	    	OCS.DET<index>.TYPE        -> type
#	        OCS.DET<index>.CCDNAME     -> ccdname (ACE,FIERA only)
#	        OCS.DET<index>.ENVNAME     -> envname (optional)
#		OCS.DET<index>.CCDLENV     -> ccdlenv (ACE,FIERA only)
#		OCS.DET<index>.DBFILE      -> dbfile  (ACE,FIERA only)
#		OCS.DET<index>.WCS         -> wcs     (optional)
#
#	    If the detector configuration is stored in the
#	    alternative config. file, then the alternative config.
#	    file is not accessed immediately (it may not be 
#	    present when the Config method is called).  The
#	    alternative config. file has to read calling
#	    explicitely method 'SetConfig'.
#
#	SetConfig
#	    Read the configuration from the alternative config. file.
#
#	Install
#	    Install the detector.
#
#	InstallScanlinks
#	    Install the scanlinks between the detector LCU and the WS.
#
#   PUBLIC DATA
#	name:	    detector name.
#	type:	    detector type (either ACE or FIERA).
#	ccdname:    camera name as used by DCS.
#	envname:    environment where DCS runs
#		    (default: INSTALL.RTAPENV1.NAME).
#	ccdlenv:    environment of the detector LCU.
#	dbfile:     detector *.dbcfg file.
#	wcs:	    world coordinate system (optional).
#		    The value must consist of a list of six entries:
#		    1. WCS X Coordinate of Reference Pixel.
#		    2. WCS Y Coordinate of Reference Pixel.
#		    3. WCS Number of arcseconds per pixel.
#		    4. WCS Rotation angle (clockwise positive).
#		    5. WCS Equinox of coordinates (1950 or 2000).
#		    6. WCS Epoch of coordinates
#		           (used for FK4/FK5 conversion).
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginDCS
#------------------------------------------------------------------------

class pkginDCS {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
	configure -objType DCS
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
    
    	#*# 1. name
	configure -name [$cfg Get INSTALL.DET$i.NAME]

	#*# 2. user
	set txt ""
	catch { set txt [$cfg Get INSTALL.LOGINNAME.USER] }
	configure -user $txt

    	#*# 3. config. file
	set txt ""
	catch { set txt [$cfg Get INSTALL.DET$i.CFGSET] }
	if { "$txt" != "" } {
	    # delay configuration to later:
	    # config. set may not be installed yet.
	    configure -cfgset $txt -index $i
	    return
	}
	configure -cfgset ""
	SetConfig $cfg $i
    }

    #****************************************************************
    #*# method: SetConfig
    #
    #  read either from cfg or from cfgset (if cfg is empty).
    #----------------------------------------------------------------

    public method SetConfig { {cfg ""} {i ""} } {
    
    	#*# 1. if already configured: return
	if { "$ccdname" != "" } {
	    return
	}

        #*# 2. set FITS prefix, open config. file
	if { "$cfg" != "" } {
	    set prefix "INSTALL"
  	} else {
	    set prefix "OCS"
	    set cfg cfg_$cfgset
	    set i $index
            if { "[find objects -isa ctooCONFIG $cfg]" == "" } {
	        ctooCONFIG $cfg $cfgset
	    }
	}

	#*# 3. detector type
	set txt [$cfg Get $prefix.DET$i.TYPE]
	if { [lsearch "ACE FIERA IRACE" $txt] == -1 } {
	    error "$prefix.DET$i.TYPE: unknown type '$txt'."
	}
	configure -type $txt

	#*# 4. detector WS environment
	set txt ""
	set repcfg [[Repository] Contents CONFIG]
	catch { set txt [$repcfg Get INSTALL.RTAPENV1.NAME] }
	catch { set txt [$cfg Get $prefix.DET$i.ENVNAME] }
	if { "$txt" == "" } {
	    error "$prefix.DET$i.ENVNAME missing."
	}
	configure -envname $txt
        
	#*# 5. WCS
	set txt ""
	catch { set txt [$cfg Get $prefix.DET$i.WCS] }
	configure -wcs $txt

	if { $type != "IRACE" } {
	    #*# 6. camera name
	    set txt $name
	    catch { set txt [$cfg Get $prefix.DET$i.CCDNAME] }
	    configure -ccdname $txt

	    #*# 7. detector LCU environment
	    configure -ccdlenv [$cfg Get $prefix.DET$i.CCDLENV]

	    #*# 8. detector *.dbcfg file
	    configure -dbfile [$cfg Get $prefix.DET$i.DBFILE]

	} else {
	    #*# 9. IRACE config.
	    configure -ccdname $name
	    set txt ""
	    catch { set txt [$cfg Get $prefix.DET$i.SDMAHOST] }
	    configure -ccdlenv $txt
	    configure -dbfile  ""
	}
    }

    #****************************************************************
    #*# method: Install
    #----------------------------------------------------------------

    public method Install {} {

	set tmpenv ""
	catch { set tmpenv [GetEnv RTAPENV] }
	try_eval {

            #*# 1. load config. (part 1)
	    Info "       Detector [format %-8s $name:] " 0
	    SetConfig

	    #*# 2. if ACE or FIERA:
	    if { $type != "IRACE" } {

                #*# 2. load config. (part 2)
                Info "[format %-30s "$type $ccdname in $ccdlenv ..."] " 0
	        if { "$type" == "ACE" } {
	    	    set prefix "c"
	        } elseif { "$type" == "FIERA" } {
	    	    set prefix "f"
	        } else {
	            error "Detector $name: unknown type '$txt'."
	        }
    
                #*# 3. run ccdInstall.sh
	        SetEnv RTAPENV $envname
	        SetEnv CCDNAME $ccdname
	        SetEnv CCDLENV $ccdlenv
                set cmd [cconcat $prefix cdInstall.sh]
	        System "$cmd $dbfile" {
	    	    if { [string first "Error" "%o"] != -1 } {
		        error "Error running %c: %o."
		    }
	        }
    
                #*# 4. run ccdDcsSetLogMask.sh
                set cmd [cconcat $prefix cdDcsSetLogMask.sh]
		sleep 5
	        System "$cmd $ccdname $envname $ccdlenv" {
	    	    if { [string first "FAILED" "%o"] != -1 } {
		        error "Error running %c: %o."
		    }
	        }
        
                #*# 5. run ccdDcsWcs.sh
		set wcslen [llength $wcs]
                if { $wcslen >= 6 && $wcslen <= 8 } {
		    if { $wcslen == 6 } {
		    	lappend wcs "PIXEL"
		    }
                    set cmd [cconcat $prefix cdDcsWcs.sh]
		    sleep 5
		    System "$cmd $ccdname $envname $wcs"
		    Info "OK."
                } elseif { [llength $wcs] != 0 } {
	    	    Info "OK."
	            Info "       Warning: Detector $name has incomplete WCS data:   SKIPPED."
                } else {
		    Info "OK."
	        }

		#*# 6. change dbfile ownership to user
		if { $user != "" } {
		    set ccdFile [file join [GetEnv INS_ROOT] SYSTEM COMMON \
				 CONFIGFILES [cconcat $ccdname ".dbcfg"]]
		    #
		    # Linux: Linux does not allow any user to use chown,
		    # therefore in Linux the file is renamed, then rcp-ed
		    # to the user, and the temp. renamed file deleted.
		    #
		    if { "[exec uname]" == "Linux" } {
			set tmpFile [cconcat $ccdFile ".tmp"]
			file delete -force $tmpFile
			file rename $ccdFile $tmpFile
			set host [GetEnv HOST]
			set rcpFile "$user@$host:$ccdFile"
			exec rsh -l $user $host /bin/rm -f $ccdFile
			exec rcp $tmpFile $rcpFile
			file delete -force $tmpFile
		    } else {
		        chown $user $ccdFile
		    }
		}

	    #*# 7. if IRACE: run iracqInstallData
	    } else {
	        SetEnv RTAPENV $envname
		if { $iraceInstalled == 0 } {
                    Info "[format %-30s "installing $type files ..."] " 0
		    System "iracqInstallData"
		    set iraceInstalled 1
		} else {
                    Info "[format %-30s "$type files already installed."] " 0
		}
		Info "OK."
	    }
 	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	} {
	    if { "$tmpenv" != "" } {
	        SetEnv RTAPENV $tmpenv
	    }
	}
    }

    #****************************************************************
    #*# method: InstallScanLink
    #----------------------------------------------------------------

    public method InstallScanLinks { {clean 1} } {

    	#*# 1. if IRACE: nothing to do: return.
	if { $type == "IRACE" } {
	    return
	}

	set tmpenv ""
	catch { set tmpenv [GetEnv RTAPENV] }
	try_eval {

            # 2. if specified, execute under user $user.
            if { "$user" != "" && "$user" != "[GetEnv USER]" } {
	    	if { [exec uname] == "HP-UX" } {
    	            set remcmd "remsh [GetEnv HOST] -l $user -n"
		} else {
    	            set remcmd "rsh [GetEnv HOST] -l $user -n"
		}
            } else {
    	        set remcmd ""
            }
	    if { "$type" == "ACE" } {
	        set prefix "c"
	    } elseif { "$type" == "FIERA" } {
	        set prefix "f"
	    } else {
	        error "Detector $name: unknown type '$txt'."
	    }

    	    # 3. clean table
	    SetEnv RTAPENV $envname
	    System "$remcmd scanOn" {
	    	if { [string first "Error" "%o"] != -1 } {
		    error "Error running %c: %o."
		}
	    }
	    if { $clean } {
	        System "$remcmd scanDevOff $ccdlenv" {
	    	    if { [string first "Error" "%o"] != -1 } {
		        error "Error running %c: %o."
		    }
	        }
	        System "$remcmd scanTblClean -e $ccdlenv" {
	    	    if { [string first "Error" "%o"] != -1 } {
		        error "Error running %c: %o."
		    }
	        }
	    }
	    System "$remcmd scanDevOn $ccdlenv" {
	    	if { [string first "Error" "%o"] != -1 } {
		    error "Error running %c: %o."
		}
	    }

    	    # 4. run ccdDcsScan.sh
    	    set cmd [cconcat $prefix cdDcsScan.sh]
	    System "$remcmd $cmd $ccdname $envname $ccdlenv" {
	    	if { [string first "FAILURE" "%o"] != -1 } {
		    error "Error running %c: %o."
		}
	    }

    	    # 5. run ccdosScan.sh
    	    set cmd [cconcat $prefix cdosScan.sh]
	    System "$remcmd $cmd $ccdname $envname $ccdlenv" {
	    	if { [string first "FAILURE" "%o"] != -1 } {
		    error "Error running %c: %o."
		}
	    }
	} {
	    error $errorResult $errorCode $errorInfo
	} {
	    if { "$tmpenv" != "" } {
	        SetEnv RTAPENV $tmpenv
	    }
	}
	
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable index   ""
    public variable cfgset  ""
    public variable type    ""
    public variable ccdname ""
    public variable envname ""
    public variable ccdlenv ""
    public variable dbfile  ""
    public variable wcs     ""
    public variable user    ""

    protected common iraceInstalled 0
}


#************************************************************************
#___oOo___

