#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginFILESET.tcl,v 1.105 2005/10/12 19:47:36 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  26/06/01  created.
#

#************************************************************************
#   NAME
#	pkginFILESET - set of files to be installed
# 
#   SYNOPSIS
#	pkginFILESET name
# 
#   DESCRIPTION
#	pkginFILESET is used to install a set of files.
#
#   PARENT
#	ctooOBJ_BASE pkginLOG
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure a file set from a configuration file.
#	    <cfg> should be an instance of type pkginCONFIG.
#	    The following config. FITS keywords are expected
#	    and stored in the public data described below:
#	        INSTALL.FILES<index>.NAME     -> name
#	    	INSTALL.FILES<index>.SRCHOST  -> srchost  (optional)
#	        INSTALL.FILES<index>.SRCDIR   -> srcdir   (optional)
#	        INSTALL.FILES<index>.DESTUSER -> destuser (optional)
#	        INSTALL.FILES<index>.DESTHOST -> desthost (optional)
#		INSTALL.FILES<index>.DESTDIR  -> destdir  (optional)
#		INSTALL.FILES<index>.FILE<i>  -> filelist
#		INSTALL.FILES<index>.PERM<i>  -> filelist (optional)
#		INSTALL.FILES<index>.LINK<i>  -> linklist
#
#	Install
#	    Install the file set.
#	    The files (INSTALL.FILESi.FILEi) and the links 
#	    (INSTALL.FILESi.LINKi) are installed in the 
#	    requested locations.
#
#	Verify
#	    Verify that the files are properly installed
#	    (currently no verification is done).
#
#   PUBLIC DATA
#	name:	    file set name
#	srchost:    the file set is only installed, if this variable
#		    is empty or equal to the local host.
#	srcdir:     File source directory relative to $INTROOT/config.
#		    All <source> locations are relative to this
#		    directory.
#	destuser:   If this variable is empty, then files are installed
#		    under the current user.  Otherwise, the files are
#		    installed under the specified user.
#	desthost:   If this variable is empty, then files are installed
#		    in the local host.  Otherwise, the files are
#		    installed in the specified host.
#	destdir:    File destination directory relative to the home
#		    directory of the destination user.
#		    All <destination> locations are relative to this
#		    directory.
#	filelist:    List of {<source> <destination> [<perm>]} entries.
#		    Each <source> is a filename that shall be copied
#		    to <destination>.  <perm> are the file permissions
#		    of the installed file in chmod format.
#	linklist:   List of {<source> <destination>} entries.
#		    Each <destination> is the location of a link that
#		    shall be installed, pointing to <source>.
#
#   CAUTIONS
#	The local user needs rsh/rcp permission to be able to 
#	install files under user@host.
#
#	No backup of existing files is made before installing
#	the file set.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginFILESET
#------------------------------------------------------------------------

class pkginFILESET {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
	configure -objType FILESET
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
    
    	#*# 1. name
	configure -name [$cfg Get INSTALL.FILES$i.NAME]

	#*# 2. source host
	set txt ""
	catch { set txt [$cfg Get INSTALL.FILES$i.SRCHOST] }
	configure -srchost $txt
	# If srchost does not match, then this file set
	# will not be installed: skip the rest of the config.
	if { $srchost != "" && $srchost != [GetEnv HOST] } {
	    return
	}

	#*# 3. source directory
	#      (INTROOT must be resolved evtl. in the remote host)
	set txt {$INTROOT/config}
	catch { set txt [$cfg Get INSTALL.FILES$i.SRCDIR] }
	configure -srcdir $txt

	#*# 4. destination user
	set txt ""
	catch { set txt [$cfg Get INSTALL.FILES$i.DESTUSER] }
	configure -destuser $txt

	#*# 5. destination host
	set txt ""
	catch { set txt [$cfg Get INSTALL.FILES$i.DESTHOST] }
	configure -desthost $txt

	#*# adjust destuser and desthost
	if { $destuser == "" && $desthost != "" } {
	    configure -destuser [GetEnv USER]
	}
	if { $desthost == "" && $destuser != "" } {
	    configure -desthost [GetEnv HOST]
	}
	if { $destuser == [GetEnv USER] && $desthost == [GetEnv HOST] } {
	    configure -destuser ""
	    configure -desthost ""
	}

	#*# 6. destination directory
	set txt "~"
	catch { set txt [$cfg Get INSTALL.FILES$i.DESTDIR] }
	configure -destdir $txt

	#*# 7. files
	set linklist ""
	set j 1
	set lastj 1
	while { $j < $lastj + 20 } {
	    catch {
	    	set txt [$cfg Get INSTALL.FILES$i.FILE$j]
		if { [llength $txt] != 2 } {
		    error "INSTALL.FILES$i.FILE$j = '$txt' is incorrect."
		}
		catch { lappend txt [$cfg Get INSTALL.FILES$i.PERM$j] }
		lappend filelist $txt
		set lastj $j
	    }
	    incr j
	}

	#*# 8. links
	set linklist ""
	set j 1
	set lastj 1
	while { $j < $lastj + 20 } {
	    catch {
	    	set txt [$cfg Get INSTALL.FILES$i.LINK$j]
		if { [llength $txt] != 2 } {
		    error "INSTALL.FILES$i.LINK$j = '$txt' is incorrect."
		}
		lappend linklist $txt
		set lastj $j
	    }
	    incr j
	}
    }

    #****************************************************************
    #*# method: Install
    #----------------------------------------------------------------

    public method Install {} {

	Info "       Installing file set [format %-7s $name] ... " 0
	Log ""
	try_eval {
	
	    #*# 1. copy files:
	    foreach filedata $filelist {

	    	#*# 2. get source and destination
	    	lassign $filedata src dest perm
		if { [file pathtype $src] != "absolute" } {
		    set src [file join $srcdir $src]
		}
		if { [file pathtype $dest] != "absolute" } {
		    set dest [file join $destdir $dest]
		}

		#*# 3. copy file
		if { $destuser != "" } {
		    System "rcp -r $src $destuser@$desthost:$dest"
		} else {
	            regsub -all {\$INTROOT} $src [GetEnv INTROOT] src
		    if { [file isdirectory $src] == 0 } {
		        file delete $dest
		    }
		    System "cp -R $src $dest"
		}

		#*# 4. set permissions
		if { $perm != "" } {
		    if { $destuser != "" } {
		    	Rsh $destuser $desthost "chmod $perm $dest"
		    } else {
	                Log "       Executing 'chmod 0$perm $dest' ..."
		    	chmod 0$perm $dest
		    }
		}
	    }

	    #*# 5. create symbolic links:
	    foreach linkdata $linklist {

	    	#*# 6. get source and destination
	    	lassign $linkdata src dest
		if { [file pathtype $src] != "absolute" } {
		    set src [file join $srcdir $src]
		}
		if { [file pathtype $dest] != "absolute" } {
		    set dest [file join $destdir $dest]
		}

		#*# 7. create link
		if { $destuser != "" } {
		    Rsh $destuser $desthost "rm -f $dest ; ln -s $src $dest"
		} else {
	            regsub -all {\$INTROOT} $src [GetEnv INTROOT] src
	            Log "       Executing 'ln -s $src $dest' ..."
		    file delete $dest
		    link -sym $src $dest
		}

	    }

	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: Verify
    #----------------------------------------------------------------

    public method Verify {} {

    	#*# 1. init.
	Info "       File set [format %-14s $name]  verifying ...     " 0
	ErrorBegin "ERROR: File set $name: " "FAILURE.\n"
	try_eval {

	    #*# 2. verify files:
            if { "[exec uname]" == "HP-UX" } {
                set rsh "remsh"
            } else {
                set rsh "rsh"
	    }
	    foreach filedata $filelist {

	    	#*# 3. get source and destination
	    	lassign $filedata src dest perm
		if { [file pathtype $src] != "absolute" } {
		    set src [file join $srcdir $src]
		}
		if { [file pathtype $dest] != "absolute" } {
		    set dest [file join $destdir $dest]
		}

		#*# 4. verify file
	        regsub -all {\$INTROOT} $src [GetEnv INTROOT] src
		if { $destuser != "" } {
		    if { [file isdirectory $src] } {
		    	set dest2 [file join $dest [file tail $src]]
		        if { [catch { exec $rsh $desthost -l $destuser /bin/ls $dest2 }] } {
		            ErrorSet "ERROR: Dir. not found: $dest2"
		        }
		    } else {
		        if { [catch { exec $rsh $desthost -l $destuser /bin/ls $dest }] } {
		            ErrorSet "ERROR: File not found: $dest"
		        }
		    }
		} else {
		    if { [file isfile $src] } {
		        if { ![file isfile $dest] } {
		            ErrorSet "ERROR: File not found: $dest"
			}
		    } elseif { [file isdirectory $src] } {
		    	set dest2 [file join $dest [file tail $src]]
		        if { ![file isdirectory $dest2] } {
		            ErrorSet "ERROR: Dir. not found: $dest2"
			}
		    } else {
		        if { ![file exists $dest] } {
		            ErrorSet "ERROR: File not found: $dest"
			}
		    }
		}
	    }

	    #*# 6. verify symbolic links:
	    foreach linkdata $linklist {

	    	#*# 7. get source and destination
	    	lassign $linkdata src dest
		if { [file pathtype $src] != "absolute" } {
		    set src [file join $srcdir $src]
		}
		if { [file pathtype $dest] != "absolute" } {
		    set dest [file join $destdir $dest]
		}

		#*# 8. verify link
		if { $destuser != "" } {
		    if { [catch { exec $rsh $desthost -l $destuser /bin/ls $dest }] } {
		        ErrorSet "ERROR: Link not found: $dest"
		    }
		} else {
	            regsub -all {\$INTROOT} $src [GetEnv INTROOT] src
		    try_eval {
		    	if { [file readlink $dest] != $src } {
			    ErrorSet "ERROR: Link $dest is not pointing to $src."
			}
		    } {
		        ErrorSet "ERROR: Link not found: $dest"
		    }
		}
	    }
	} {
	    ErrorTryEval $errorResult $errorCode $errorInfo
	}
	ErrorEnd
    }

    #****************************************************************
    #*# method: Rsh
    #----------------------------------------------------------------

    protected method Rsh { user host cmd } {

    	#*# 1. build cmdStr
        if { "[exec uname]" == "HP-UX" } {
            set cmdStr "remsh $host -l $user -n '$cmd'"
        } else {
            set cmdStr "rsh $host -l $user -n '$cmd'"
        }

        #*# 2. execute command
        #
        #      (Note: remsh does not propagate the return code
        #       of the called command: errors must be detected
        #       using stderr and stdout).
        #
        System $cmdStr {
            foreach txt { "FAILED" "rror" "ommand not found" "rm: " } {
                if { [string first $txt "%e"] != -1 } {
                    error "Error running %c: %e."
                }
                if { [string first $txt "%c"] != -1 } {
                    error "Error running %c: %c."
                }
            }
        }
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable srchost   ""
    public variable srcdir    ""
    public variable destuser  ""
    public variable desthost  ""
    public variable destdir   ""
    public variable filelist  ""
    public variable linklist  ""
}


#************************************************************************
#___oOo___

