#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginLCUENV.tcl,v 1.105 2005/10/12 19:47:33 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  25/08/04  method Clean added.
# rschmutz  19/01/03  support added to boot LCUs in parallel.
# rschmutz  17/01/03  accept BOOT_ALONE, LOG options.
# rschmutz  06/01/03  Init: pass option -w to pkginEnvInit.
# rschmutz  04/01/03  VLTSW20020254: /etc/services error report updated.
# rschmutz  07/01/01  check DNS.
# rschmutz  23/11/00  /etc/services check fixed.
# rschmutz  06/11/00  INSTALL.LCUENVi.OPTIONS=SLAVE support added.
# rschmutz  11/10/00  issue only warning if logLCU.config is missing.
# rschmutz  30/09/00  /etc/hosts check fixed.
# rschmutz  17/09/00  Verify method added.
# rschmutz  13/05/00  renamed, updated, man page added.
# sskole    25/02/2000 The work-around includes the .bootChange.* files
# sskole    22/02/00  Added temporary code to create and init.
# sskole    20/10/99  bootScript: the boot root is now replaced in init
# rschmutz  11/10/99  m_type = "LCU"
# sskole    03/07/99  created
#

#************************************************************************
#   NAME
#	pkginLCUENV - LCU environment
# 
#   SYNOPSIS
#	pkginLCU name -name <name>
# 
#   DESCRIPTION
#	pkginLCU is used to build and install a LCU environment.
#
#   PARENT
#	ctooOBJ_BASE pkginLOG
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure a software module from a configuration file.
#	    <cfg> should be an instance of type pkginCONFIG.
#	    The following config. FITS keywords are expected
#	    and stored in the public data described below:
#	        INSTALL.LCUENV<index>.NAME      -> name
#	    	INSTALL.LCUENV<index>.TPLSRC    -> tplsrc  (optional)
#	        INSTALL.LCUENV<index>.HOST      -> host    (optional)
#	        INSTALL.LCUENV<index>.BOOTENV   -> bootenv (optional)
#		INSTALL.LCUENV<index>.PROCNAMEi -> procnames (optional)
#		INSTALL.LCUENV<index>.OPTIONS   -> options (optional)
#
#	Create
#	    Create the environment calling vccEnvCreate.
#
#	Init
#	    Initialize the environment calling vccEnvInit.
#
#	Start
#	    Start the environment calling vccEnvStart.
#
#	Stop
#	    Stop the environment calling vccEnvStop.
#
#	Verify
#	    Verify the environment.
#
#   PUBLIC DATA
#	name:	   environment name.
#	tplsrc:    environment template directory (default: none).
#	host:	   LCU where the environment runs (default: <name>).
#	bootenv:   WS environment from where the LCU boots
#		   (default: <INSTALL.RTAPENV1.NAME>).
#	procnames: processes checked by pkginVerify.
#	options:   options: 
#		     SLAVE: only create, do not start/stop/verify.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginLCUENV
#------------------------------------------------------------------------

class pkginLCUENV {
    inherit pkginENV_BASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
	configure -objType LCUENV -type LCU
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
    
	#*# 1. environment name
	configure -name [$cfg Get INSTALL.LCUENV$i.NAME]

	#*# 2. environment template
	set txt ""
	catch { set txt [$cfg Get INSTALL.LCUENV$i.TPLSRC] }
	configure -tplsrc $txt

	#*# 3. host
	set txt ""
	catch { set txt [$cfg Get INSTALL.LCUENV$i.HOST] }
	configure -host $txt

	#*# 4. boot environment
	set txt ""
	catch { set txt [$cfg Get INSTALL.LCUENV$i.BOOTENV] }
	configure -bootenv $txt

	#*# 5. process names
	set txt ""
	set j 1
	while {1} {
	    try_eval {
	    	lappend txt [$cfg Get INSTALL.RTAPENV$i.PROCNAME$j]
		incr j
	    } {
	    	break
	    }
	}
	configure -procnames $txt

	#*# 6. module options
	set txt ""
	catch { set txt [$cfg Get INSTALL.LCUENV.OPTIONS] }
	CheckOptions "INSTALL.LCUENV.OPTIONS" $txt {BOOT_ALONE SLAVE LOG ""}
	catch { append txt " " [$cfg Get INSTALL.LCUENV$i.OPTIONS] }
	CheckOptions "INSTALL.LCUENV$i.OPTIONS" $txt {BOOT_ALONE SLAVE LOG ""}
	configure -options $txt

	#*# 7. call parent config
	pkginENV_BASE::Config $cfg
    }

    #****************************************************************
    #*# method: Create
    #----------------------------------------------------------------

    public method Create {} {

	Info "       LCU environment [format %-7s $name]  creating ...      " 0
	try_eval {
	    set arg "-m template"
	    if { "$bootenv" != "" } {
	    	append arg " -w $bootenv"
	    }
	    if { "$host" != "" } {
	    	append arg " -l $host"
	    }
	    pkginENV_BASE::Create $arg
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: Init
    #----------------------------------------------------------------

    public method Init {} {

    	#*# 1. if SLAVE: skip.
	if { [lcontain $options "SLAVE"] } {
	    return
	}

	#*# 2. initialize
	Info "       LCU environment [format %-7s $name]  initializing ...  " 0
	try_eval {
	    set arg ""
	    if { "$bootenv" != "" } {
	    	append arg " -w $bootenv"
	    }
	    pkginENV_BASE::Init $arg
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start { {init 0} } {

    	#*# 1. if SLAVE: skip.
	if { [lcontain $options "SLAVE"] } {
	    Info "       LCU environment [format %-7s $name]  skipped (slave):  OK."
	    return
	}

	#*# 2. initialize
        if { $init } {
	    Init
	}

	#*# 3. start
	Info "       LCU environment [format %-7s $name]  starting ...      " 0
	set arg ""
	if { [lcontain $options "LOG"] } {
	    set arg " -log"
	} elseif { ![lcontain $options "BOOT_ALONE"] } {
	    set arg "-nowait"
	}
	set lastStartOk 0
	EvalStatus [list pkginENV_BASE::Start $arg]
	set lastStartOk 1
    }

    #****************************************************************
    #*# method: StartWait
    #----------------------------------------------------------------

    public method StartWait {} {

    	#*# 1. if not BOOT_ALONE and not LOG and last start is ok:
	#      wait for boot completion
	if { [lcontain $options "BOOT_ALONE"] == 0 && \
	     [lcontain $options "LOG"] == 0 && $lastStartOk == 1} {
	    Info "       LCU environment [format %-7s $name]  waiting ...       " 0
	    EvalStatus {pkginENV_BASE::EvaluateCmd pkginEnvStart "-onlywait"}
	}
    }

    #****************************************************************
    #*# method: Stop
    #----------------------------------------------------------------

    public method Stop {} {

    	#*# 1. if SLAVE: skip.
	if { [lcontain $options "SLAVE"] } {
	    Info "       LCU environment [format %-7s $name]  skipped (slave):  OK."
	    return
	}

	#*# 2. stop
	set vltdata [GetEnv VLTDATA]
	catch { set vltdata [[[Repository] LookupName VLTDATA ROOT 2] cget -dir] }
    	if {[file exists [file join $vltdata ENVIRONMENTS $name]]} {
	    Info "       LCU environment [format %-7s $name]  stopping ...      " 0
	    EvalStatus {
		# need LCUs to be stopped twice ?
	        pkginENV_BASE::Stop
    	    }
	} else {
	    Info "       LCU environment [format %-7s $name]  already stopped:  OK."
	}
    }

    #****************************************************************
    #*# method: Verify
    #----------------------------------------------------------------

    public method Verify {} {

    	#*# 1. init., get host
	if { [lcontain $options "SLAVE"] } {
	    Info "       LCU environment [format %-7s $name]  verify slave ...  " 0
	} else {
	    Info "       LCU environment [format %-7s $name]  verifying ...     " 0
	}
	ErrorBegin "ERROR: Environment $name: " "FAILURE.\n"
	try_eval {
	    set desthost ""
	    catch { set desthost [AccSelect "SELECT station_name FROM prog_environment WHERE progenv_name = '$name'"] }
	    if { "$desthost" == "" } {
	        error "Env. not registered in ACC (or error accessing ACC)."
	    }
	    set envtype [AccSelect "SELECT env_type FROM prog_environment WHERE progenv_name = '$name'"]
	    if { $envtype != "LCU" } {
	    	error "Wrong env. type $envtype."
	    }
	    set tcpport [AccSelect "SELECT tcp_port FROM prog_environment WHERE progenv_name = '$name'"]
	    set wsenv   [AccSelect "SELECT bootenv_name FROM lcu_progenv WHERE progenv_name = '$name'"]
	    set wstype  [AccSelect "SELECT env_type FROM prog_environment WHERE progenv_name = '$wsenv'"]

	    #*# 2. verify env. directory
	    set file1 [file join {$VLTDATA} ENVIRONMENTS $name]
	    set file2 [file join [GetEnv VLTDATA] ENVIRONMENTS $name]
	    if { ![file isdirectory $file2] } {
	    	error "Env. directory $file1 not found."
	    }

	    #*# 3. verify /etc/hosts (or DNS)
	    set noEtcHosts 0
	    if {![GrepFile "/etc/hosts" "\[ \t\]$desthost\[ \t\]"]} {
	        if {![GrepFile "/etc/hosts" "\[ \t\]$desthost\$"]} {
		    set noEtcHosts 1
		    if { ![Nslookup $desthost] } {
			set noEtcHosts 0
	                ErrorSet "Host $desthost not registered in /etc/hosts or DNS."
		    }
		}
	    }

	    if { ![lcontain $options "SLAVE"] } {

    	        #*# 4. verify /etc/services
	        if {![GrepFile "/etc/services" "^$name\[ \t\]+$tcpport/tcp"]} {
	    	    ErrorSet "Env. entry '$name $tcpport/tcp' not found in /etc/services."
	        }

	        #*# 5. verify RtapEnvList or CcsEnvList
	        if { $wstype == "RTAP" } {
	            set file1 {/etc/$RTAPROOT/RtapEnvList}
	            set file2 "/etc[GetEnv RTAPROOT]/RtapEnvList"
	        } else {
	            set file1 {$VLTDATA/config/CcsEnvList}
	            set file2 [file join [GetEnv VLTDATA] config CcsEnvList]
	        }
	        if {![GrepFile $file2 "^$name\[ \t\]+$desthost\[ \t\]*\$"]} {
	    	    ErrorSet "Env. not registered in $file1."
	        }

	        #*# 6. verify logLCU.config
		set noLogLCUconfig 0
	        set file1 [file join {$VLTDATA} ENVIRONMENTS $wsenv logLCU.config]
	        set file2 [file join [GetEnv VLTDATA] ENVIRONMENTS $wsenv logLCU.config]
	        if { ![file isfile $file2] } {
		    set noLogLCUconfig 1
	        } elseif {![GrepFile $file2 "^$name\[ \t\]+$wsenv\[ \t\]*\$"]} {
	    	    ErrorSet "Env. not registered in $file1."
	        }

	        #*# 7. send PING command to standard servers
	        foreach procname {lccServer msgServer rdbServer} {
	            if {[system "msgSend -v -n $name $procname PING \" \" > /dev/null"]} {
	                ErrorSet "No reply from process $procname (PING)."
		        break
		    }
	        }
    
	        #*# 8. read OLDB attribute
	        if {[system "dbBackup -n '@$name:LCC:STATUS.mode' > /dev/null"]} {
	            ErrorSet "OLDB not readable (read failure)."
	        }

	        #*# 9. send PING command to user processes
	        if { $errorList == "" } {
	            foreach procname $procnames {
	                if {[system "msgSend -v -n $name $procname PING \" \" > /dev/null"]} {
	                    ErrorSet "No reply from process $procname (PING)."
		        }
	            }
	        }

		#*# 10 report warnings
		if { $noEtcHosts } {
		    ErrorSetHeader "WARNING.\n"
	            ErrorInfo "Host $desthost not registered in /etc/hosts."
		}
		if { $noLogLCUconfig } {
		    ErrorSetHeader "WARNING.\n"
	    	    ErrorInfo "File $file1 not found (will be created by step START_LCUENV)."
		}
	    }
	} {
	    ErrorTryEval $errorResult $errorCode $errorInfo
	}
	ErrorEnd
    }

    #****************************************************************
    #*# method: Clean
    #----------------------------------------------------------------

    public method Clean {} {

	#*# 1. clean
	if { $maxbak == 0 } {
	    Info "       LCU ENV:  deleting [format %-7s $name] ... " 0
	} else {
	    Info "       LCU ENV:  renaming [format %-7s $name] ... " 0
	}
        EvalStatus pkginENV_BASE::Backup
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable host    ""
    public variable bootenv ""
    public variable options ""

    protected variable lastStartOk 0
}


#************************************************************************
#___oOo___

