#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginLIBRARY.tcl,v 1.105 2005/10/12 19:47:36 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-04-23  method ShowVersions: introot arg. added.
# rschmutz  2004-10-06  method ShowVersions: -introot support added.
# rschmutz  2004-10-06  method GetIntrootVersion: consider sub-modules.
# rschmutz  2004-06-02  created.
#

#************************************************************************
#   NAME
#	pkginLIBRARY - software library
# 
#   SYNOPSIS
#	pkginLIBRARY name -name <name>
# 
#   DESCRIPTION
#	pkginLIBRARY is used to build and install a software library.
#	A library is a software module that contains a pkgin configuration
#	file with a list of software modules to be built and installed.
#
#   PARENT
#	ctooOBJ_BASE pkginLOG
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure a software module from a configuration file.
#	    <cfg> should be an instance of type pkginCONFIG.
#	    The following config. FITS keywords are expected
#	    and stored in the public data described below:
#	        INSTALL.MODULE<index>.NAME    -> name
#	    	INSTALL.MODULE<index>.VERSION -> version, comment (opt.)
#	        INSTALL.MODULE<index>.SUBPKG  -> subpkg
#		INSTALL.MODULE<index>.OPTIONS -> options (optional)
#		INSTALL.MODULE<index>.MODULES -> modules (optional)
#		INSTALL.CON.MAXBAK	      -> maxbak  (optional)
#
#	    The INSTALL.MODULE<index>.VERSION keyword may contain
#	    the module version and a comment, as follows:
#	        "<version> [<comment>]"
#	    The version may be a version number, or the words
#	    "LAST" or "LOCAL".
#
#	Retrieve [<compress>]
#	    Retrieves the library from Archive.
#	    An existing library is renamed, if it does not have the
#	    correct version (or if it is under modification when
#	    -compress is used).
#
#	Build
#	    Build the library.
#
#	Clean
#	    Clean the library.
#
#	ShowVersions [<introot>] [<update>]
#	    Shows the current, last and configured versions of the
#	    main module.  Shows what module version should be used.
#	    Optionally sets the version to the one that should be
#	    used (usually the last version in the Archive, unless
#	    the current version is "LOCAL" or if comment starts 
#	    with "LOCK" or "FORCE".
#
#   PUBLIC DATA
#	name:	    main module name
#	version:    main module version (default: empty = current or last
#		    module version, depending if the main module is already
#		    retrieved from Archive or not).
#	subpkg:	    subsdirectory where the module should be retrieved
#		    (default: empty = current directory).
#	options:    module options (default: no options).
#		    The following options (separated by spaces) are
#		    recognized:
#		    FULLCCS:  Build only for Full CCS.
#		    CCSLITE:  Build only for CCS-Lite.
#		    VXWORKS:  Build only if VxWorks is available.
#		    SKIP:     Skip the complete library (no build).
#		    CLEAN:    After building the library, clean it.
#	modules:    Software modules installed by the library.
#		    Parameter only used by pkginINTROOT::Verify.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginLIBRARY
#------------------------------------------------------------------------

class pkginLIBRARY {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
	configure -objType LIBRARY
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg i } {
    
	#*# 1. main module name and index
	configure -name [$cfg Get INSTALL.MODULE$i.NAME]
	configure -index $i

	#*# 2. main module version and comments
	#      "<version> [<comment>]"
	set txt "LAST"
	catch { set txt [$cfg Get INSTALL.MODULE$i.VERSION] }
	set txt2 [lvarpop txt]
	configure -cfgvers $txt2
	configure -version $txt2
	configure -comment $txt

	#*# 3. sub-package
	set txt ""
	catch { set txt [$cfg Get INSTALL.MODULE$i.SUBPKG] }
	if { $txt == "" } {
	    error "INSTALL.MODULE$i.SUBPKG is empty: Libraries need to be placed in a sub-directory."
	}
	configure -subpkg $txt

	#*# 4. library options
	set txt ""
	catch { set txt [$cfg Get INSTALL.MODULE.OPTIONS] }
	set txt [intersect $txt {FULLCCS CCSLITE VXWORKS SKIP CLEAN}]
	catch { append txt " " [$cfg Get INSTALL.MODULE$i.OPTIONS] }
	CheckOptions "INSTALL.MODULE$i.OPTIONS" $txt {
	    LIBRARY FULLCCS CCSLITE VXWORKS SKIP CLEAN
	}
	set txt [intersect $txt {FULLCCS CCSLITE VXWORKS SKIP CLEAN}]
	configure -options $txt

	#*# 7. modules (optional)
	set txt ""
	catch { set txt [$cfg Get INSTALL.MODULE$i.MODULES] }
	configure -modules $txt

	#*# 8. maxbak
	set txt F
	catch { set txt [$cfg Get INSTALL.FLAG.DELETEDIR] }
	set txt [expr {2 * (1 - [Logical2Num $txt])}]
	catch { set txt [$cfg Get INSTALL.CON.MAXBAK] }
	configure -maxbak $txt
    }

    #****************************************************************
    #*# method: Retrieve
    #----------------------------------------------------------------

    public method Retrieve { {compress 0} } {

    	try_eval {
	    Info "       Library [format %-7s $name] " 0

	    #*# 1. if no version specified:
	    #      retrieve current or last version
	    set curVer   [GetCurrentVersion]
	    set modified [lindex $curVer 1]
	    set curVer   [lindex $curVer 0]
	    if { "$version" == "LOCAL" } {
	    	if { $compress != 0 } {
	            Info "[format %-10s $version] not allowed:          " 0
		    error "Library $name: local module version not allowed with option -compress."
		}
	        set path [file join $subpkg $name]
	        if {![file isdirectory $path]} {
	            Info "[format %-10s $version] $path not found: " 0
		    error "Library $name: local module version $path not found."
		}
	  	Info "[format %-10s $version] local version [format %-7s $curVer:] OK."
		return
	    } elseif { "$version" == "" || "$version" == "LAST" } {
	    	try_eval {
	            configure -version [lindex [GetLastVersion] 0]
		} {
	  	    Info "[format %-10s $version] may not exist:        " 0
		    error $errorResult $errorCode $errorInfo
	        }
	    }

    	    #*# 2. Only retrieve from the archive when the module is not already 
    	    #      existing in the parent directory, or the version is not correct.
	    #      If -compress, then also retrieve module, if current one is being
	    #      modified.
	    if { $modified == 1 && $compress == 0 } {
	    	set version $curVer
	        Info "[format %-10s "$version+"] is being modified:    " 0
	    } else {

	        #*# 3. if wrong version: rename or delete subpkg dir.
		if { $curVer != $version && [file exists $subpkg] == 1 } {
	            Info "[format %-10s $version] retrieving ...        " 0
                    if {$maxbak != 0 } {
			set dirver ""
		    	if { $curVer != "" } {
		            if { $modified } {
		    	        set dirver "$subpkg-${curVer}+"
		            } else {
		    	        set dirver "$subpkg-$curVer"
		            }
			}
			BackupFile $subpkg $maxbak $dirver
                    } else {
	                file delete -force $subpkg
	            }
	        } else {
	            Info "[format %-10s $version] checking ...          " 0
		}

    	        #*# 4. if subpkg dir. does not exist: create it
	        if { "$subpkg" != "" } {
	            file mkdir $subpkg
	        }

	        #*# 5. pkginBuild <module> -step RETRIEVE
	        set curDir [pwd]
	        cd $subpkg
	        if {[catch {exec pkginBuild $name -ver $version -step RETRIEVE}]} {
	    	    cd $curDir
	            error "Error retrieving library $name (see $subpkg/INSTALL/pkginBuild.log|.err for details)."
	        }
	        cd $curDir
	    }
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: GetLastVersion
    #
    #   returns "<version> <modified_flag>"
    #----------------------------------------------------------------

    public method GetLastVersion {} {

	# get the version of the main trunk:
	# if there are branches, then this is the last line
	if {[catch {exec cmmLast $name | tail -1} result]} {
	    set result [lindex [split $result "\n"] 0]
	    error "Error retrieving version of module $name ($result)."
	}
    
        set lastVer [lindex $result 3]
	if { [lindex $result 4] == "modified" } {
	    set result [list $lastVer 1]
	} else {
	    set result [list $lastVer 0]
	}

	return $result
    }

    #****************************************************************
    #*# method: GetCurrentVersion
    #
    #   retrieve the version of the current module
    #   returns "<version> <modified_flag>"
    #----------------------------------------------------------------

    public method GetCurrentVersion {} {

        #*# 1. search version inside a Makefile
        set result ""
    	set fileList "src/Makefile ws/src/Makefile lcu/src/Makefile test/Makefile"
        foreach file $fileList {
	    set path [file join $subpkg $name $file]
	    if {[file isfile $path]} {
	        if {[catch {set result [exec ident $path]}]} {
		    set result "0 0 0 0"
		}
	        break
	    }
	}
	set curVer [lindex $result 3]

	#*# 2. separate "+": return "<version> <modified_flag>"
	set tmp $curVer
	set tmpCurVer ""
	while {"[set token [ctoken tmp +]]" != ""} {
	    append tmpCurVer $token 
	}
	if { "$curVer" != "$tmpCurVer" } {
	    set result [list $tmpCurVer 1]
	} else {
	    set result [list $curVer 0]
	}

	return $result
    }

    #****************************************************************
    #*# method: GetIntrootVersion
    #----------------------------------------------------------------

    public method GetIntrootVersion {} {

	#*# 1. search INTROOT vltMakeInstall.config file:
	set ret ""
	set filename [file join [GetEnv INTROOT] vltMakeInstall.config]
	if { ![file exists $filename] } {
	    return $ret
	}
	if { $modules == "" } {
	    set checkname $name
	} else {
	    set checkname [lindex $modules 0]
	}
	for_file line $filename {

	    #*# 2. skip comments and empty lines
	    set line [string trim $line]
	    if { $line == "" || [cindex $line 0] == "#" } {
		continue
	    }

	    #*# 3. get module and version
	    set modname [ctoken line " \t"]
	    if { [crange $modname end-3 end] == "-lcu" } {
		set modname [crange $modname 0 end-4]
	    } elseif { [crange $modname end-2 end] == "-ws" } {
		set modname [crange $modname 0 end-3]
	    }

	    #*# 4. if module found: return its version
	    if { $modname == $checkname } {
	        set modvers [ctoken line " \t"]
		if { $modvers == "installed" } {
		    set modvers "0"
		}
	    	set ret $modvers
		break
	    }
	}

	return $ret
    }

    #****************************************************************
    #*# method: Build
    #----------------------------------------------------------------

    public method Build { {installOnly 0} } {

	#*# 1. init: retrieve all options
        set cleanAfterBuild   0
	set envonly ""
	if { $installOnly } {
	    set envonly "-envonly"
	}
        foreach option $options {
	    switch -- $option {
	        "CCSLITE" {
		    if {![ExistsEnv CCSLITE]} {
	    		Info "       Skipping [format %-24s $name:] [format %-30s "built only with CCSLITE. "]"
			return
		    }
		}
	        "FULLCCS" {
		    if {[ExistsEnv CCSLITE]} {
	    		Info "       Skipping [format %-24s $name:] [format %-30s "built only with FULLCCS. "]"
			return
		    }
		}
	        "VXWORKS" {
		    if {![ExistsEnv VXROOT]} {
	    		Info "       Skipping [format %-24s $name:] [format %-30s "built only with VXWORKS. "]"
			return
		    }
		}
	        "CLEAN" {
		    set cleanAfterBuild 1
		}
	        "SKIP" {
	    	    Info "       Skipping [format %-24s $name:] [format %-30s "SKIP option set."] OK."
		    return
		}
	        default  {
	    	    Info "       Skipping [format %-24s $name:] [format %-30s "wrong option '$option': "] FAILURE."
		    set txt "ERROR: Library $name: Wrong build option '$option'.\n       Use: CCSLITE FULLCCS VXWORKS SKIP CLEAN"
		    LogErr $txt
		    error "$txt\n"
		}
	    }
        }

	#*# 5. pkginBuild <module> -step BUILD_MOD
        set txt "Building [format %-24s "$subpkg/$name:"] [format %-30s "pkginBuild ..."] "
	Info "       $txt" 0
        Log  ""
	LogErr $txt
	set curDir [pwd]
	cd $subpkg
	if {[catch {eval exec pkginBuild $name -step BUILD_MOD $envonly}]} {
	    cd $curDir
	    Info "FAILURE."
	    error "Error building library $name (see $subpkg/INSTALL/pkginBuild.log|.err for details)."
	}
	cd $curDir
	Info "OK."

	#*# 6. clean library
	if { $cleanAfterBuild } {
	    Clean
	}
    }

    #****************************************************************
    #*# method: Clean
    #----------------------------------------------------------------

    public method Clean {} {

	#*# 1. clean library
        set txt "Cleaning [format %-24s "$subpkg/$name:"] [format %-18s "pkginClean ..."] "
	Info "       $txt" 0
        Log  ""
	LogErr $txt
	set curDir [pwd]
	cd $subpkg
	if {[catch {exec pkginClean $name}]} {
	    cd $curDir
	    Info "FAILURE."
	    error "Error cleaning library $name (see $subpkg/INSTALL/pkginClean.log|.err for details)."
	}
	cd $curDir
	Info "OK."
    }

    #****************************************************************
    #*# method: Verify
    #----------------------------------------------------------------

    public method Verify {} {
	    Info "       Verifying [format %-24s $name:] OK."
    }

    #****************************************************************
    #*# method: ShowVersions
    #----------------------------------------------------------------

    public method ShowVersions { {introot 0} {update 0} } {

	#*# 1. retrieve config., current, INTROOT and last versions
	if { "$version" == "" } {
	    set vers "LAST"
	} else {
	    set vers $version
	}
	lassign [GetCurrentVersion] currvers currmod
	if { "$currvers" == "" } {
	    set currvers "-"
	}
	lassign [GetIntrootVersion] introotvers introotmod
	if { "$introotvers" == "" } {
	    set introotvers "-"
	}
	if { "$introotmod" == "" } {
	    set introotmod 0
	}
	try_eval {
	    lassign [GetLastVersion] lastvers lastmod
	} {
	    set lastvers "-"
	    set lastmod 0
	}

	#*# 2. new version
	#
	# If the configured version is LOCAL: do not modify it.
	#
	# If compare with INTROOT: then modify the module version
	# if INTROOT and configured versions do not match.
	#
	# Otherwise compare with the Archive version:
	# If the version is not FIXED or LOCAL, and there is a last
	# version in the Archive, then modify the module version;
	# otherwise keep the current version.
	#
	if { [string first "LOCK" $comment] == 0 || \
	     [string first "FORCE" $comment] == 0 } {
	    set isfixed 1
	} else {
	    set isfixed 0
	}
	set updatevers $vers
	if { "$vers" != "LOCAL" } {
	    if { $introot == 0 } {
	        if { $isfixed == 0 && "$lastvers" != "-" } {
	            set updatevers $lastvers
	        }
	    } elseif { "$introotvers" != "-" } {
	        set updatevers $introotvers
	        if { "$vers" == "LAST" && "$introotvers" == "$lastvers" } {
	    	    set updatevers "LAST"
	    	}
	    }
	}

	#*# 3. set comment; show arrow, if version is updated.
	set txt $comment
	set retrieve 0
	if { ("$vers" == "LAST" && "$lastvers" != "$currvers") || \
	     ("$vers" != "LAST" && "$vers" != "LOCAL" && "$vers" != "$currvers") } {
	    if { "$lastvers" != "-" } {
		set retrieve 1
	    }
	}
	if { $introot == 0 && $retrieve == 0 && \
	     (($introotvers != $currvers && $introotvers != $lastvers) || \
	      $introotvers == "-") } {
	    set txt "INSTALL $txt"
	}
	if { $retrieve } {
	    set txt "RETRIEVE $txt"
	}
	if { "$vers" != "$updatevers" } {
	    set arrow "->"
	    set txt "UPDATE $txt"
	} else {
	    set arrow "  "
	}
	if { ("$currvers" == "-" && "$lastvers" == "-") || \
	     ("$vers" == "LAST"  && "$lastvers" == "-") || \
	     ("$vers" == "LOCAL" && "$currvers" == "-") } {
	    set txt "??? $txt"
	}

	#*# 4. show versions
	if { $currmod } {
	    append currvers "+"
	}
	if { $introotmod } {
	    append introotvers "+"
	}
	if { $currvers != $introotvers } {
	    set currvers "$currvers/$introotvers"
	}
	if { $lastmod } {
	    append lastvers "+"
	}
	Info "[format %-8s $name] [format %-16s $currvers] [format %-9s $lastvers] [format %-9s $vers] $arrow [format %-9s $updatevers] $txt"

	#*# 5. update
	if { $update && $index != 0 } {
	    set cfg [[Repository] Contents CONFIG]
	    $cfg Set INSTALL.MODULE$index.VERSION $updatevers
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable cfgvers ""
    public variable version ""
    public variable subpkg  ""
    public variable options ""
    public variable modules ""
    public variable maxbak  2
    public variable comment ""
    public variable index   0
}


#************************************************************************
#___oOo___

