#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginLOG.tcl,v 1.105 2005/10/12 19:47:32 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  2004-08-23  Info: do not print if loglevel = -1.
# rschmutz  2003-11-22  SPR VLTSW20030298: Exit: exit 1 on error.
# rschmutz  05/11/00  set log files group writable during System execution.
# rschmutz  12/05/00  renamed to pkginLOG, updated, man page added.
# sskole    18/11/99  getIntermediateStatus/getStepStatus from pkginCBase
# rschmutz  11/10/99  place log files in directory ./INSTALL.
# sskole    07/10/99  Added timestamp to the top of the full log file.
# sskole    07/10/99  Existing log files are renamed instead of truncated.
# sskole    28/08/99  Implemented intermediateErr and stepErr
# sskole    28/08/99  Added 'newLine' argument to logScr, logErr, logMsg
# sskole    04/07/99  created
#

#************************************************************************
#   NAME
#	pkginLOG - log utilities
# 
#   SYNOPSIS
#	pkginLOG name [-infoHeader <text>]
#		      [-infoWidget <widget>]
#		      [-logname <name>]
#		      [-logdir <directory>]
#		      [-loglevel <level>]
# 
#   DESCRIPTION
#	pkginLOG provides log utilities for the build/install scripts.
#
#   PUBLIC PROCEDURES
#	SetWidget <widget>
#	    Sets the widget where the logs should appear.
#
#   PUBLIC DATA
#	infoHeader: header printed by method Info.
#	infoWidget: widget used by method Info.
#	logname:    logfile name (default: infoHeader).
#	logdir:     directory where log files are placed
#		    (default: "$VLTDATA/tmp").
#	loglevel:   loglevel used by method Debug.
#
#   PROTECTED METHODS
#	Exit [<text>]
#	    Prints the text (if supplied) and terminates the script.
#
#	Debug <text> [<level>]
#	    Prints text using the Info method if the loglevel
#	    is greater or equal level. The default level is 1.
#
#	Info <text>
#	    Prints the text to stdout and to the log file.
#
#	Log <text>
#	    Prints the text to the log file.
#
#	LogErr <text>
#	    Prints the text to the error file.
#
#	System <cmd> [<code>]
#	    Executes <cmd> redirecting stdout and stderr to 
#	    the log and error files.  An error is returned,
#	    if the cmd exitcode is not 0.  If there is an 
#	    error, then the first stderr line is returned 
#	    as the error.
#
#	    If <code> is supplied, then it is executed
#	    after <cmd>.  The following variables may be
#	    used inside the code:
#		%c - <cmd>
#	        %e - stderr text produced by <cmd>
#		%o - stdout text produced by <cmd>
#	        %r - <cmd> return code
#	    The code <code> can be used to detect if the
#	    command <cmd> returned an error (and to raise
#	    a Tcl error in this case).
#
#   FILES
#	$logdir/$logname.log  -  WRITE  -- message log file
#	$logdir/$logname.err  -  WRITE  -- error log file
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginLOG
#------------------------------------------------------------------------

class pkginLOG {

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
	incr objCount
    }

    #****************************************************************
    #*# destructor
    #----------------------------------------------------------------

    destructor {

        #*# 1. close log files
	incr objCount -1
	if { $objCount <= 0 } {
    	    if {$logFileId != ""} {
	        close $logFileId
	        set logFileId ""
            }
            if {$errFileId != ""} {
	        close $errFileId
	        set errFileId ""
            }
	}
    }

    #****************************************************************
    #*# method: SetWidget
    #----------------------------------------------------------------

    public proc SetWidget { widget } {
    	set infoWidget $widget
    }

    #****************************************************************
    #*# method: Exit
    #----------------------------------------------------------------

    protected method Exit { {msg ""} } {
	global cv

	#*# 1. remove duplicate "ERROR:"
	regsub "ERROR: ERROR: " $msg "ERROR: " msg

    	#*# 2. print message
	if { "$msg" != "" } {
	    catch { Info $msg }
	}

	#*# 3. exit or issue an error (if we are in a panel)
	if { ![info exists cv] } {
	    exit 1
	} else {
	    error $msg
	}
    }

    #****************************************************************
    #*# method: Debug
    #----------------------------------------------------------------

    protected method Debug { msg {level 1} } {

    	#*# 1. if loglevel: print message
    	if { $loglevel >= $level } {
	    Info "       $msg"
	}

    }

    #****************************************************************
    #*# method: Info
    #----------------------------------------------------------------

    protected method Info { msg {newline 1} } {

    	#*# 1. check loglevel
    	if { $loglevel == -1 } {
	    return
	}

    	#*# 2. print message
	if { "$infoWidget" == "" } {
	    if { $newline } {
	        puts "$infoHeader$msg"
	    } else {
	        puts -nonewline "$infoHeader$msg"
		flush stdout
	    }
	} else {
	    if { $newline } {
	        $infoWidget addItem end "$infoHeader$msg"
	    } else {
	    	set txt [$infoWidget getItem end]
		append txt $msg
		$infoWidget deleteItem end
	        $infoWidget addItem end $txt
	    }
	    $infoWidget setCursor end
	    update
	}

	#*# 3. log message
	Log $msg $newline
    }

    #****************************************************************
    #*# method: Log
    #----------------------------------------------------------------

    protected method Log { msg {newline 1} } {
	global env

	#*# 1. if no logname: return
	if { "$logname" == "" } {
	    return
	}

	#*# 2. open log files
	OpenLogFile
	OpenErrFile

    	#*# 3. log message
	if { "$logFileId" != "" } {
	    if { $newline } {
	        puts $logFileId "$infoHeader$msg"
	    } else {
	        puts -nonewline $logFileId "$infoHeader$msg"
	    }
	}
    }

    #****************************************************************
    #*# method: LogErr
    #----------------------------------------------------------------

    protected method LogErr { msg } {
	global env

	#*# 1. if no logname: return
	if { "$logname" == "" } {
	    return
	}

	#*# 2. open log files
	OpenLogFile
	OpenErrFile

    	#*# 3. log message
	if { "$errFileId" != "" } {
	    puts $errFileId "$infoHeader$msg"
	}
    }

    #****************************************************************
    #*# method: System
    #----------------------------------------------------------------

    protected method System { cmd {code {}} } {

	#*# 1. if no logname: error
	if { "$logname" == "" } {
	    error "pkginLOG::System: Name for redirection log files is missing."
	}

	#*# 2. open log files
	OpenLogFile
	OpenErrFile

	#*# 3. execute cmd
	Log "\n       Executing '$cmd' ..."
	close $logFileId
	close $errFileId
	set logFileSize1 [file size $logFile]
	set errFileSize1 [file size $errFile]
	chmod 0664 [list $logFile $errFile]
	set retcode [system "$cmd >> $logFile 2>> $errFile"]
	chmod 0644 [list $logFile $errFile]
	set errFileSize2 [file size $errFile]
	set logFileId [open $logFile a+]
	set errFileId [open $errFile a+]
	if { $errFileSize2 - $errFileSize1 > 0 } {
	    LogErr $logSep1
	}

	#*# 4. check retcode and error file size
	if { $retcode == 127 } {
	    error "Script '$cmd' not found."
	}
	if { "$code" != "" } {
	    set errtext [csubstr [read_file -nonewline $errFile] $errFileSize1 len]
	    set logtext [csubstr [read_file -nonewline $logFile] $logFileSize1 len]
	    regsub -all {%c} $code {$cmd} code
	    regsub -all {%r} $code {$retcode} code
	    regsub -all {%o} $code {$logtext} code
	    regsub -all {%e} $code {$errtext} code
	    eval $code
	}
	if { $retcode != 0 && $errFileSize2 - $errFileSize1 > 0 } {
	    set errtext [csubstr [read_file -nonewline $errFile] $errFileSize1 end]
	    set errtext [lindex [split $errtext "\n"] 0]
	    error "Script error: $errtext (retcode $retcode)."
	}
	if { $retcode != 0 } {
	    error "Error running script '$cmd' (retcode $retcode)."
	}
    }

    #****************************************************************
    #*# method OpenLogFile
    #----------------------------------------------------------------

    private method OpenLogFile {} {

	#*# 1. if log file is not open:
	if { "$logFileId" == "" && $logError == 0 } {

    	    #*# 2. if log directory does not exist: create it.
	    if { "$logdir" == "" } {
	        if {[catch { set vltdata $env(VLTDATA) }]} {
	            set logdir "[pwd]"
	        } else {
	    	    set logdir [cconcat $vltdata "/tmp/"]
	        }
	    }
	    file mkdir $logdir
	    set curdir [pwd]
	    cd $logdir
	    set logdir [pwd]
	    cd $curdir

    	    #*# 3. open log file
    	    set logFile [file join $logdir [cconcat $logname ".log"]]
    	    if { [file exists $logFile] } {
	    	file stat $logFile fileStats
		set date [clock format $fileStats(ctime) \
			  -format "%Y-%m-%dT%H:%M:%S"]
	        file rename $logFile $logFile.$date
    	    }
	    if {[catch {
		set logFileId [open $logFile a+]
	    }]} {
		set logError 1
		Exit "ERROR: Wrong unix account."
	    }
	    if { [system "chmod 644 $logFile"] != 0 } {
		set logError 1
		Exit "ERROR: Wrong unix account."
	    }
	}
    }

    #****************************************************************
    #*# method OpenErrFile
    #----------------------------------------------------------------

    private method OpenErrFile {} {

	#*# 1. if error file is not open:
	if { "$errFileId" == "" && $logError == 0 } {

    	    #*# 2. if log directory does not exist: create it.
	    if { "$logdir" == "" } {
	        if {[catch { set vltdata $env(VLTDATA) }]} {
	            set logdir "[pwd]"
	        } else {
	    	    set logdir [cconcat $vltdata "/tmp/"]
	        }
	    }
	    file mkdir $logdir
	    set curdir [pwd]
	    cd $logdir
	    set logdir [pwd]
	    cd $curdir

    	    #*# 3. open error file
    	    set errFile [file join $logdir [cconcat $logname ".err"]]
    	    if { [file exists $errFile] } {
	    	file stat $errFile fileStats
		set date [clock format $fileStats(ctime) \
			  -format "%Y-%m-%dT%H:%M:%S"]
	        file rename $errFile $errFile.$date
    	    }
	    try_eval {
		set errFileId [open $errFile a+]
	    } {
		set logError 1
		Exit "ERROR: Wrong unix account."
	    }
	    if { [system "chmod 644 $errFile"] != 0 } {
		set logError 1
		Exit "ERROR: Wrong unix account."
	    }
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public common loglevel   0
    public common logname    ""
    public common logdir     ""
    public common infoWidget ""
    public common infoHeader ""

    protected common logSep1 [replicate "-" 80]
    protected common logSep2 [replicate "=" 80]

    private common logFile   ""
    private common logFileId ""
    private common logError  0
    private common errFile   ""
    private common errFileId ""
    private common errError  0
    private common objCount  0
}


#
# ___oOo___
