#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginPKGFILE.tcl,v 1.105 2005/10/12 19:47:35 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2004-09-04  initScriptPath added.
# rschmutz  2004-06-03  search the initial script also in the config dir.
# rschmutz  16/01/03  Pack adapted to pack real INTROOT, INS_ROOT and envs.
# rschmutz  14/05/01  register/check root sizes.
# rschmutz  25/03/01  add '+' to filename, if module under modification.
# rschmutz  11/10/00  search init. and Tcl script also in INTROOT.
# rschmutz  23/06/00  deldir replaced with maxbak.
# rschmutz  08/06/00  created.
#

#************************************************************************
#   NAME
#	pkginPKGFILE - package tar file
# 
#   SYNOPSIS
#	pkginPKGFILE name
# 
#   DESCRIPTION
#	pkginPKGFILE contains the INTROOT, INS_ROOT and environments
#	of a complete package built.  The package is built off-line
#	with pkginBuild and installed on the target with pkginUnpack.
#
#	The file is a tar file, with the INTROOT that contains
#	compiled package, plus some special files. The tar file
#	should be unpacked under the target $INTROOT location.
#	The special files are:
#
#	INSTALL/INSTALL.cfg
#	    Install configuration used to build the package.
#
#	INSTALL/INSTALL.script
#	    The file registered in INSTALL.INITIAL.SCRIPT (optional).
#
#	INSTALL/INSTALL.tcl
#	    The file registered in INSTALL.CON.TCLFILE (optional).
#
#	INSTALL/PLUGIN_<name>
#	    Plugin scripts (optional).
#
#	INSTALL/VLTDATA.tar
#	    File that contains all the package environments. This
#	    file has to be unpacked under the directory $VLTDATA.
#
#	INSTALL/INS_ROOT.tar
#	    File that contains the package INS_ROOT. This file
#	    has to be unpacked under the directory $INS_ROOT.
#
#   PARENT
#	ctooOBJ_BASE pkginLOG
#
#   PUBLIC METHODS
#	Pack <dir>
#	    Creates the package file with the files located
#	    under <dir>.  <dir> should be the INTROOT directory
#	    used to build the package.  The special files are
#	    created as follows:
#
#	    INSTALL/INSTALL.cfg
#	    	Created with the current contents of the config.
#		buffer used to build the package.
#	    INSTALL/INSTALL.script
#	        The file registered in INSTALL.INITIAL.SCRIPT.
#	    INSTALL/INSTALL.tcl
#	        The file registered in INSTALL.CON.TCLFILE.
#	    INSTALL/PLUGIN_<name>
#		Plugin scripts registered for the BEFORE_INSTALL hook.
#	    INSTALL/VLTDATA.tar
#		Contents of directory <dir>/config/VLTDATA. The
#		directory is deleted after creating the tar file.
#	    INSTALL/INS_ROOT.tar
#		Contents of directory <dir>/config/INS_ROOT. The
#		directory is deleted after creating the tar file.
#
#	    The package file is given the name:
#	    <name>-<version>.tar, where <name> is the package
#	    name, and <version> the version of <module>, usually
#	    the installation module.
#
#	Unpack
#	    Unpacks the package file to the current
#	    $INTROOT, $INS_ROOT and $VLTDATA directories.
#
#   PUBLIC DATA
#	name:	package name.
#	module: integration module.
#	maxbak:	if 0, then an existing package tar file
#		is deleted, otherwise a backup is made.
#		If there are more than <maxbak> backups, then
#	   	the oldest backup is deleted.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginPKGFILE
#------------------------------------------------------------------------

class pkginPKGFILE {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	configure -objType PKGFILE
    	eval configure $args
    }

    #****************************************************************
    #*# method: Pack
    #----------------------------------------------------------------

    public method Pack {} {

	set curdir [pwd]
	try_eval {
	    #*# 1. get INTROOT, INS_ROOT and environments
            set introot [GetEnv INTROOT]
	    set insroot [GetEnv INS_ROOT]
	    set container [[[Repository] Contents PACKAGE 3] Contents ALL_ENVIRONMENTS]
	    set envs [$container Contents]
	    if { [file pathtype $introot] != "absolute" } {
	    	error "PACK: INTROOT '$introot' must be an absolute path."
	    }
	    if { ![file isdirectory $introot] } {
	    	error "PACK: INTROOT directory '$introot' not found."
	    }
	    if { [file pathtype $insroot] != "absolute" } {
	    	error "PACK: INS_ROOT '$insroot' must be an absolute path."
	    }
	    if { ![file isdirectory $insroot] } {
	    	error "PACK: INS_ROOT directory '$insroot' not found."
	    }
	    set vltdata [GetEnv VLTDATA]
	    if { [glob -nocomplain -directory [file join $insroot SYSTEM DETDATA] "*.fits"] != "" } {
	        Info "WARN : \$INS_ROOT/SYSTEM/DETDATA contains *.fits files."
	        Info "       You may want to delete them before creating the package file."
		Info ""
		sleep 3
	    }
	    set envpaths ""
	    foreach env $envs {
	        set envname [$env cget -name]
	    	set envpath [file join ENVIRONMENTS $envname]
		if { ![file isdirectory [file join $vltdata $envpath]] } {
		    error "PACK: Env. directory $envpath not found."
		}
		if { [$env cget -type] == "RTAP" } {
		    if { [$env IsRunning] } {
		    	error "PACK: WS env. $envname is running.  It must be stopped to pack it."
		    }
		}
		lappend envpaths $envpath
	    }
		
	    #*# 2. create INSTALL/PLUGIN_<name> files
	    file mkdir [file join $introot INSTALL]
	    set showOk 0
	    set container [[Repository] Contents PLUGINS 10]
	    loop i 1 100 {
		set obj [$container LookupName BEFORE_INSTALL_$i]
		if { "$obj" != "" } {
		    if { $i == 1 && "$obj" != "" } {
	                Info "       Plugin scripts:  saving ...                " 0
			set showOk 1
	    		file mkdir [file join $introot INSTALL PLUGINS]
		    }
		    set plugin [file join INSTALL PLUGINS PLUGIN_BEFORE_INSTALL_$i]
		    $obj SaveAs $introot $plugin
		} else {
		    break
		}
	    }
	    if { $showOk } {
	        Info "OK."
	    }

	    #*# 3. register required INS_ROOT, VLTDATA disk space
	    set cfg [[Repository] Contents CONFIG]
	    $cfg Set INSTALL.INSROOT.SIZE [GetDirSize $insroot]
	    set vltdataSize 0
	    foreach envpath $envpaths {
		incr vltdataSize [GetDirSize [file join $vltdata $envpath]]
	    }
	    $cfg Set INSTALL.VLTDATA.SIZE $vltdataSize

	    #*# 4. create INSTALL/INSTALL.script
	    set initScript ""
	    catch { set initScript [$cfg Get INSTALL.INITIAL.SCRIPT] }
	    if { "$initScript" != "" } {
                Info "       Initial script:  saving ...                " 0
		set initScript [SearchFile $initScript "$module/src $module/config $initScriptPath \$INTROOT/bin \$VLTROOT/bin"]
		set initScript [ctoken initScript " \t"]
		set filename [file join $introot INSTALL INSTALL.script]
		file copy -force $initScript $filename
		Info "OK."
	    }

	    #*# 5. create INSTALL/INSTALL.tcl
	    set userTclFile ""
	    catch { set userTclFile [$cfg Get INSTALL.CON.TCLFILE] }
	    if { "$userTclFile" != "" } {
                Info "       TCL file:        saving ...                " 0
		set userTclFile [SearchFile $userTclFile "$module/src \$INTROOT/bin \$VLTROOT/bin"]
		set filename [file join $introot INSTALL INSTALL.tcl]
		file copy -force $userTclFile $filename
		Info "OK."
	    }

	    #*# 6. create INSTALL/VLTDATA.tar
	    if { $envpaths != "" } {
                Info "       VLTDATA (envs.): saving ...                " 0
	        set tarfile [file join $introot INSTALL VLTDATA.tar]
	        cd $vltdata
	        System "tar cf $tarfile $envpaths"
	        cd $curdir
	        Info "OK."
	    }

	    #*# 7. create INSTALL/INS_ROOT.tar
            Info "       INS_ROOT:        saving ...                " 0
	    set tarfile [file join $introot INSTALL INS_ROOT.tar]
	    cd $insroot
	    System "tar cf $tarfile ."
	    cd $curdir
	    Info "OK."

	    #*# 8. register required INTROOT disk space (= tar file size)
	    $cfg Set INSTALL.INTROOT.SIZE [GetDirSize $introot]

	    #*# 9. create INSTALL/INSTALL.cfg
            Info "       Install config.: saving ...                " 0
	    set filename [file join $introot INSTALL INSTALL.cfg]
	    $cfg SaveAs $filename
	    Info "OK."

	    #*# 10. create package file
	    set obj [[Repository] LookupName $module MODULE 10]
	    if { "$obj" == "" } {
	    	error "Module $module not found."
	    }
	    if { [llength $obj] > 1 } {
	        error "More than one module $module registered.  Check configuration."
	    }
	    lassign [$obj GetCurrentVersion] version modified
	    if { $modified } {
	        set filename [cconcat $name "-" $version "+.tar"]
	    } else {
	        set filename [cconcat $name "-" $version ".tar"]
	    }
            Info "       Creating [format %-34s "$filename ... "]" 0
	    set filename [file join $curdir $filename]
	    BackupFile $filename $maxbak
	    cd $introot
	    System "tar cf $filename ."

	    #*# 11. create '<name>-last.tar' link
	    cd $curdir
	    set link [cconcat $name "-last.tar"]
	    file delete -force $link
	    link -sym [file tail $filename] $link
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	} {
	    cd $curdir
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: Unpack
    #----------------------------------------------------------------

    public method Unpack {} {

        set curdir [pwd]
	try_eval {
	    #*# 1. unpack package
            Info "       INTROOT:  unpacking ...                    " 0
	    if { ![file exists $name] } {
	    	error "Package file $name not found."
	    }
	    if { "[file pathtype $name]" != "absolute" } {
	        set tarfile [file join $curdir $name]
	    } else {
	    	set tarfile $name
	    }
	    set introot [GetEnv INTROOT]
	    file mkdir $introot
	    cd $introot
	    # system does not work (broken pipe): use exec.
	    exec pkginUnziptar $tarfile
	    # The extracted INTROOT size should be equal to the
	    # original size.  Allow a 1.0% difference.
	    if { $size != 0 } {
	        set introotSize [GetDirSize $introot]
	        set sizeDiff [expr {abs($introotSize - $size) * 100 / $size}]
	        if { $sizeDiff > 1.0 } {
	    	    error "Extracted INTROOT has [set introotSize]K, but [set size]K was expected."
	        }
	    }
	    Info "OK."
	    Log "       Extracted INTROOT size = [set introotSize]K."

	    #*# 2. unpack environments (VLTDATA)
	    set tarfile [file join $introot INSTALL VLTDATA.tar]
	    if { [file exists $tarfile] } {
                Info "       VLTDATA:  unpacking ...                    " 0
	        cd [GetEnv VLTDATA]
	        System "(umask 0 ; tar xf $tarfile)"
	        Info "OK."
	    }

	    #*# 3. unpack INS_ROOT
            Info "       INS_ROOT: unpacking ...                    " 0
	    set insroot [GetEnv INS_ROOT]
	    file mkdir $insroot
	    cd $insroot
	    set tarfile [file join $introot INSTALL INS_ROOT.tar]
	    System "(umask 0 ; tar xf $tarfile)"
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	} {
	    cd $curdir
	}
	Info "OK."
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public common initScriptPath ""

    public variable module ""
    public variable maxbak 2
    public variable size 0
}


#************************************************************************
#___oOo___

