#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginPLUGIN.tcl,v 1.105 2005/10/12 19:47:33 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2004-08-23  defaultPath added, /bin /usr/bin added to path.
# rschmutz  13/05/00  renamed, updated, man page added.
# sskole    25/02/2000 execute simplified. searchFile used instead of findVLTFile
# sskole    31/01/00  Shell script return values are now considered
# sskole    19/11/99  BEFORE_INSTALL plugins are decomp./executed to/from ./bin
# sskole    18/11/99  When installing plugins are taken from ./config
# sskole    16/11/99  execute: Checks ownership of file before doing chgMode
# rschmutz  31/10/99  before exec: if necessary, chown +x.
# sskole    27/08/99  Implemented that the plugin deletes itself after execution
# sskole    26/08/99  Implemented bundling of plugins for the same hook
# rschmutz  21/08/99  execute code updated.
# sskole    03/07/99  created
#

#************************************************************************
#   NAME
#	pkginPLUGIN - hook to add user scripts
# 
#   SYNOPSIS
#	pkginPLUGIN name -name <name> -plugin <script>
# 
#   DESCRIPTION
#	pkginPLUGIN is a hook that allows a user script to be
#	added to the build and/or installation procedure of
#	pkgin.
#
#   PARENT
#	ctooOBJ_BASE pkginBASE
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure a software module from a configuration file.
#	    <cfg> should be an instance of type pkginCONFIG.
#	    The following config. FITS keywords are expected
#	    and stored in the public data described below:
#	        INSTALL.HOOK<index>.NAME   -> name
#	    	INSTALL.HOOK<index>.PLUGIN -> plugin
#
#	Exec
#	    Executes the plugin.
#
#	SaveAs
#	    Saves the plugin script in file <dest1>[/<dest2>].
#	    The <cfg> config. keyword INSTALL.HOOKi.SCRIPT is
#	    updated accordingly.
#
#   PUBLIC DATA
#	name:	    hook name.
#	path:	    default path to the script.
#	plugin:	    script to be executed at the given hook.
#
#	The recognized hook names are retrieved from the
#	pkginINSTALLATION class (variable hooks).
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginPLUGIN
#------------------------------------------------------------------------

class pkginPLUGIN {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
	configure -objType PLUGIN
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
    
	#*# 1. hook name
	set txt [$cfg Get INSTALL.HOOK$i.NAME]
	set hooks [[[Repository] Contents INSTALL] cget -hooks]
	CheckOption "INSTALL.HOOK$i.NAME" $txt $hooks

	#*# 2. object name
	#
	#      Add an index to allow multiple plugins
	#      for the same hook.
	loop j 1 100 {
	    if { "[[Parent] LookupName ${txt}_$j]" == "" } {
	    	break
	    }
	}
	configure -name ${txt}_$j

	#*# 3. plugin
	configure -plugin [$cfg Get INSTALL.HOOK$i.PLUGIN]

	#*# 4. path
	try_eval {
	    set obj [[[Parent] Parent] Parent] 
	    set txt [file join "." [$obj cget -module] src]
	} {
	    set txt ""
	}
	configure -path $txt

	#*# index
	configure -index $i
    }

    #****************************************************************
    #*# method: Exec
    #----------------------------------------------------------------

    public method Exec {} {

	Info "PLGIN: Plugin [Name]: " 0
	try_eval {

    	    #*# 1. search plugin script
	    #
	    # Installation Phase:
    	    # The hook BEFORE_INSTALL is a bit special in that plugins
    	    # using this hook and residing in $INTROOT are not available 
    	    # at the point where they must be executed. This is because 
    	    # INTROOT hasn't been installed (unpacked) at this stage.
    	    # Instead the plugins are executed from ./INSTALL to where
    	    # they have been unpacked beforehand.
	    #
    	    set buf $plugin
    	    set pluginCmd [ctoken buf " \t"]
	    try_eval {
	        set pluginExpr [SearchFile $plugin "$path $defaultPath \$INTROOT/bin \$VLTROOT/bin /bin /usr/bin"]
	    } {
	        Info "script '$pluginCmd' not found: " 0
		error "Plugin [Name]: script '$pluginCmd' not found."
	    }

	    #*# 2. check that the plugin is executable
	    set buf $pluginExpr
    	    set pluginPath [ctoken buf " \t"]
            if {![file executable $pluginPath]} {
	        if {[file owned $pluginPath]} {
		    chmod "+x" $pluginPath
	        } else {
		    Info "executable attribute not set: " 0
		    error "Plugin [Name]: script '$pluginCmd' is not executable."
	        }
	    }

	    #*# 3. execute plugin
	    Info "'$pluginExpr' ... " 0
	    Log ""
	    System $pluginExpr
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: SaveAs
    #
    #   Save plugin in file <dest1>[/<dest2>].
    #----------------------------------------------------------------

    public method SaveAs { dest1 {dest2 ""} } {

    	#*# 1. search plugin script
    	set buf $plugin
    	set pluginCmd [ctoken buf " \t"]
        try_eval {
	    set pluginExpr [SearchFile $plugin "$path \$INTROOT/bin \$VLTROOT/bin"]
	} {
	    error "Plugin [Name]: script '$pluginCmd' not found."
	}

	#*# 2. copy script to dest
	set buf $pluginExpr
    	set pluginPath [ctoken buf " \t"]
    	file copy -force $pluginPath [file join $dest1 $dest2]

	#*# 3 register new plugin name
	if { "$dest2" == "" } {
	    set dest2 $dest1
	}
	configure -plugin [string trim "$dest2 $buf"]
	if { $index != 0 } {
	    set cfg [[Repository] Contents CONFIG]
	    $cfg Set INSTALL.HOOK$index.PLUGIN $plugin
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public common defaultPath ""

    public variable plugin ""
    public variable path   ""
    public variable index  0
}


#************************************************************************
#___oOo___

