#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginRTAPENV.tcl,v 1.105 2005/10/12 19:47:34 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-10-12  VLTSW20050306: CcsEnvList check adapted to new format.
# rschmutz  2004-10-04  create logLCU.config file.
# rschmutz  2004-08-24  method Clean added.
# rschmutz  26/04/03  differentiate between primary WS env. and others.
# rschmutz  17/01/03  INSTALL.RTAPENVi.USER support added.
# rschmutz  04/01/03  VLTSW20020254: /etc/services error report updated.
# rschmutz  29/10/01  IsLocal method added: check host and IP address.
# rschmutz  14/10/01  if QSEMU: start now with option "-c".
# rschmutz  22/06/01  if QSEMU: start with option "-f 60".
# rschmutz  21/06/01  WS env. verify fixed (link /diska/vltdata).
# rschmutz  14/05/01  INSTALL.RTAPENVi.OPTIONS, ForceSnap method added.
# rschmutz  07/01/01  check DNS.
# rschmutz  23/11/00  /etc/services check fixed.
# rschmutz  30/09/00  /etc/hosts check fixed.
# rschmutz  17/09/00  Verify method added.
# rschmutz  05/07/00  method IsRunning added with support for QSEMU.
# rschmutz  28/05/00  renamed, updated, man page added.
# sskole    22/02/00  Simplified all methods.
# rschmutz  11/10/99  m_type = "RTAP"
# sskole    03/07/99  created
#

#************************************************************************
#   NAME
#	pkginRTAPENV - RTAP environment
# 
#   SYNOPSIS
#	pkginRTAPENV name -name <name>
# 
#   DESCRIPTION
#	pkginRTAPENV is used to build and install a RTAP environment.
#
#   PARENT
#	pkginENV_BASE
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure a software module from a configuration file.
#	    <cfg> should be an instance of type pkginCONFIG.
#	    The following config. FITS keywords are expected
#	    and stored in the public data described below:
#	        INSTALL.RTAPENV<index>.NAME      -> name
#	    	INSTALL.RTAPENV<index>.TPLSRC    -> tplsrc (optional)
#		INSTALL.RTAPENV<index>.PROCNAMEi -> procnames (optional)
#		INSTALL.RTAPENV<index>.OPTIONS   -> options (optional)
#
#	Create
#	    Create the environment calling vccEnvCreate.
#
#	Init
#	    Initialize the environment calling vccEnvInit.
#
#	Start
#	    Start the environment calling vccEnvStart.
#
#	Stop
#	    Stop the environment calling vccEnvStop.
#
#	Verify
#	    Verify the environment.
#
#   PUBLIC DATA
#	name:	   environment name.
#	tplsrc:    environment template directory (default: none).
#	procnames: processes checked by pkginVerify.
#	options:   options: 
#		     RESTART: ForceSnap stops/restarts env. instead
#			      of just taking a snapshot.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginRTAPENV
#------------------------------------------------------------------------

class pkginRTAPENV {
    inherit pkginENV_BASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
	configure -objType RTAPENV -type RTAP
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
    
	#*# 1. environment name
	configure -name [$cfg Get INSTALL.RTAPENV$i.NAME]

	#*# 2. environment template
	set txt ""
	catch { set txt [$cfg Get INSTALL.RTAPENV$i.TPLSRC] }
	configure -tplsrc $txt

	#*# 3. process names
	set txt ""
	set j 1
	while {1} {
	    try_eval {
	    	lappend txt [$cfg Get INSTALL.RTAPENV$i.PROCNAME$j]
		incr j
	    } {
	    	break
	    }
	}
	configure -procnames $txt

	#*# 4. module options
	set txt ""
	catch { set txt [$cfg Get INSTALL.RTAPENV$i.OPTIONS] }
	CheckOptions "INSTALL.RTAPENV$i.OPTIONS" $txt {RESTART ""}
	configure -options $txt

	#*# 5. call parent config
	pkginENV_BASE::Config $cfg

	#*# 6. set user (only if INSTALL.LOGINNAME.USER was not empty)
	if { $user != "" } {
	    if {![catch { set txt [$cfg Get INSTALL.RTAPENV$i.USER] }]} {
	        configure -user $txt
	    }
	}

	#*# 7. check if env. is local (check hostname and IP)
	set desthost ""
	catch { set desthost [AccSelect "SELECT station_name FROM prog_environment WHERE progenv_name = '$name'"] }
	if { "$desthost" == "" || "$desthost" == [GetEnv HOST] } {
	    set local 1
	} else {
	    # check IP address trying to open a local server TCP/IP socket
	    set local 0
	    set ipaddr ""
	    catch { set ipaddr [AccSelect "SELECT ip_address FROM station WHERE station_name = '$desthost'"] }
	    if { $ipaddr != "" } {
		if {[catch {set server [socket -server # -myaddr $ipaddr 0]}]} {
	    	    set local 0
		} else {
		    set local 1
		    close $server
		    unset server
		}
	    }
	}
	# mark non-primary WS environment with local=2
	if { $local == 1 && "$desthost" != "" && "$desthost" != [id host] } {
	    set local 2
	}

    }

    #****************************************************************
    #*# method: Create
    #----------------------------------------------------------------

    public method Create {} {

    	#*# 1. if env. not stopped: stop it.
    	if {[IsRunning]} {
	    Stop
	}

	#*# 2. create
	Info "       WS  environment [format %-7s $name]  creating ...      " 0
	try_eval {
            pkginENV_BASE::Create

	    #*# 3. if logLCU.config file does not exist:
	    set vltdata [GetEnv VLTDATA]
	    catch { set vltdata [[[Repository] LookupName VLTDATA ROOT 2] cget -dir] }
	    set logLCUfile [file join $vltdata ENVIRONMENTS $name logLCU.config]
	    if { ![file exists $logLCUfile] } {

		#*# 4. get all LCU envs. that boot from the WS env.
		set container [[[Parent] Parent] Contents AVAIL_ENVIRONMENTS]
		set objs [$container Contents LCUENV]
		set txt ""
		foreach obj $objs {
		    set bootenv [$obj cget -bootenv]
		    set lcuenv [$obj cget -name]
		    if { $bootenv == "" } {
			set bootenv [AccSelect "SELECT bootenv_name FROM lcu_progenv WHERE progenv_name = '$lcuenv'"]
		    }
		    if { $bootenv == $name } {
			append txt "[format %-7s $lcuenv] $bootenv\n"
		    }
		}

		#*# 5. write logLCU.config file
		set txt [crange $txt 0 end-1]
		write_file $logLCUfile $txt
	    }
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."
    }

    #****************************************************************
    #*# method: Init
    #----------------------------------------------------------------

    public method Init {} {

    	#*# 1. if env. not stopped: stop it.
    	if {[IsRunning]} {
	    Stop
	}

	#*# 2. Init.
	Info "       WS  environment [format %-7s $name]  initializing ...  " 0
        EvalStatus pkginENV_BASE::Init
    }

    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start { {init 0} } {

	#*# 2. Init
	if { $init } {
	    Init
	}

	#*# 3. if env. not running: Start
    	if {![IsRunning]} {
	    Info "       WS  environment [format %-7s $name]  starting ...      " 0
	    set envtype [AccSelect "SELECT env_type FROM prog_environment WHERE progenv_name = '$name'"]
	    if { $envtype == "QSEMU" } {
	        EvalStatus {pkginENV_BASE::Start "-c"}
	    } else {
	        EvalStatus pkginENV_BASE::Start
	    }
	} else {
	    Info "       WS  environment [format %-7s $name]  already running:  OK."
	}

    }

    #****************************************************************
    #*# method: Stop
    #----------------------------------------------------------------

    public method Stop {} {
    	if {[IsRunning]} {
	    Info "       WS  environment [format %-7s $name]  stopping ...      " 0
	    EvalStatus pkginENV_BASE::Stop
	} else {
	    Info "       WS  environment [format %-7s $name]  already stopped:  OK."
	}
    }

    #****************************************************************
    #*# method: ForceSnap
    #----------------------------------------------------------------

    public method ForceSnap {} {
    
    	#*# 1. if not running: skip.
    	if {![IsRunning]} {
	    Info "       WS  environment [format %-7s $name]  stopped: skipped. OK."
	    return
	}

    	#*# Take snapshot or restart
	if { ![lcontain $options "RESTART"] } {
	    Info "       WS  environment [format %-7s $name]  OLDB snapshot ... " 0
    	    try_eval {
	    	System "dbForceSnap -e $name"
	    } {
		LogErr $errorResult
		Info "FAILURE."
		error $errorResult $errorCode $errorInfo
	    }
	    Info "OK."
	} else {
	    Info "       WS  environment [format %-7s $name]  stopping ...      " 0
	    EvalStatus pkginENV_BASE::Stop
	    sleep 10
	    Info "       WS  environment [format %-7s $name]  starting ...      " 0
	    EvalStatus pkginENV_BASE::Start
	}
    }

    #****************************************************************
    #*# method: Verify
    #----------------------------------------------------------------

    public method Verify {} {

    	#*# 1. init., get host
	Info "       WS  environment [format %-7s $name]  verifying ...     " 0
	ErrorBegin "ERROR: Environment $name: " "FAILURE.\n"
	try_eval {
	    set desthost ""
	    catch { set desthost [AccSelect "SELECT station_name FROM prog_environment WHERE progenv_name = '$name'"] }
	    if { "$desthost" == "" } {
	        error "Env. not registered in ACC (or error accessing ACC)."
	    }
	    set envtype [AccSelect "SELECT env_type FROM prog_environment WHERE progenv_name = '$name'"]
	    set tcpport [AccSelect "SELECT tcp_port FROM prog_environment WHERE progenv_name = '$name'"]

	    #*# 2. verify env. directory
	    set vltdata [GetEnv VLTDATA]
	    set file1 [file join {$VLTDATA} ENVIRONMENTS $name]
	    set file2 [file join $vltdata ENVIRONMENTS $name]
	    if { ![file isdirectory $file2] } {
	    	error "Env. directory $file1 not found."
	    }

    	    #*# 3. verify /etc/services
	    if {![GrepFile "/etc/services" "^$name\[ \t\]+$tcpport/tcp"]} {
	    	ErrorSet "Env. entry '$name $tcpport/tcp' not found in /etc/services."
	    }

	    #*# 4. verify /etc/hosts (or DNS)
	    set noEtcHosts 0
	    if {![GrepFile "/etc/hosts" "\[ \t\]$desthost\[ \t\]"]} {
	        if {![GrepFile "/etc/hosts" "\[ \t\]$desthost\$"]} {
	    	    set noEtcHosts 0
		    if { ![Nslookup $desthost] } {
			set noEtcHosts 0
	                ErrorSet "Host $desthost not registered in /etc/hosts or DNS."
		    }
		}
	    }

	    #*# 5. verify RtapEnvList or CcsEnvList
	    if { $envtype == "RTAP" } {
	        set file1 {/etc/$RTAPROOT/RtapEnvList}
	        set file2 "/etc[GetEnv RTAPROOT]/RtapEnvList"
	    } else {
	        set file1 {$VLTDATA/config/CcsEnvList}
	        set file2 [file join $vltdata config CcsEnvList]
	    }
	    set envdir2 ""
	    if { [IsLocal] != 0 } {
	        set envdir [file join $vltdata ENVIRONMENTS $name]
		catch { 
		    set vltdata2 [exec /bin/sh -c "cd $vltdata ; pwd -P"]
	            set envdir2 [file join $vltdata2 ENVIRONMENTS $name]
		}
	    } else {
	        set envdir $desthost
	    }
	    set found [GrepFile $file2 "^$name\[ \t\]+$envdir\[ \t\]*\$"]
	    if { $found == 0 && [IsLocal] != 0 } {
	        set found [GrepFile $file2 "^$name\[ \t\]+$desthost\[ \t\]+$envdir\[ \t\]*\$"]
	    }
	    if { $found == 0 && $envdir2 != "" } {
	        set found [GrepFile $file2 "^$name\[ \t\]+$envdir2\[ \t\]*\$"]
	        if { $found == 0 && [IsLocal] != 0 } {
	            set found [GrepFile $file2 "^$name\[ \t\]+$desthost\[ \t\]+$envdir2\[ \t\]*\$"]
		}
	    }
	    if { $found == 0 } {
	    	ErrorSet "Env. not (or not correctly) registered in $file1."
	    }

	    #*# 6. if not running: stop checks
    	    if {![IsRunning]} {
	    	ErrorSetHeader "STOPPED.\n"
	        if { $noEtcHosts } {
	            ErrorInfo "Host $desthost not registered in /etc/hosts."
	        }
	        error "Env. not running."
	    }

	    #*# 7. send PING command to standard servers
	    foreach procname {cmdManager msgServer logManager} {
	        if {[system "msgSend -v -n $name $procname PING \" \" > /dev/null"]} {
	            ErrorSet "No reply from process $procname (PING)."
		    break
		}
	    }

	    #*# 8. read OLDB attribute
	    if {[system "dbBackup -nb '@$name:ccs_config:class config.names(0:9)' > /dev/null"]} {
	        ErrorSet "OLDB not readable (read failure)."
	    } else {
	    	set reply [exec dbBackup -nb "@$name:ccs_config:class config.names(0:1)"]
		set reply [split $reply "\n"]
		if { [string trim [lindex $reply 2]] != {0 "<base>" 0} } {
	            ErrorSet "OLDB not readable (wrong reply)."
		} elseif { [string trim [lindex $reply 3]] != {100 "BASE_CLASS" 0} } {
	            ErrorSet "OLDB not readable (wrong reply)."
		}
	    }

	    #*# 9. send PING command to user processes
	    if { $errorList == "" } {
	        foreach procname $procnames {
	            if {[system "msgSend -v -n $name $procname PING \" \" > /dev/null"]} {
	                ErrorSet "No reply from process $procname (PING)."
		    }
	        }
	    }

	    #*# 10 report warnings
	    if { $noEtcHosts } {
		ErrorSetHeader "WARNING.\n"
	        ErrorInfo "Host $desthost not registered in /etc/hosts."
	    }
	} {
	    ErrorTryEval $errorResult $errorCode $errorInfo
	}
	ErrorEnd
    }

    #****************************************************************
    #*# method: Clean
    #----------------------------------------------------------------

    public method Clean {} {

    	#*# 1. if env. not stopped: stop it.
    	if {[IsRunning]} {
	    Stop
	}

	#*# 2. clean
	if { $maxbak == 0 } {
	    Info "       WS ENV.:  deleting [format %-7s $name] ... " 0
	} else {
	    Info "       WS ENV.:  renaming [format %-7s $name] ... " 0
	}
        EvalStatus pkginENV_BASE::Backup
    }

    #****************************************************************
    #*# method: IsLocal
    #----------------------------------------------------------------

    public method IsLocal {} {
    	return $local
    }

    #****************************************************************
    #*# method: IsRunning
    #----------------------------------------------------------------

    public method IsRunning {} {

    	#*# 1. get VLTDATA
	set vltdata [GetEnv VLTDATA]
	catch { set vltdata [[[Repository] LookupName VLTDATA ROOT 2] cget -dir] }

	#*# 2. get env. type
	set envtype [AccSelect "SELECT env_type FROM prog_environment WHERE progenv_name = '$name'"]
	if { "$envtype" == "" || ("$envtype" != "RTAP" && "$envtype" != "QSEMU") } {
	    error "Environment $name: Unknown env. type '$envtype'."
	}

	#*# 3. check if env. is running
	if { "$envtype" == "RTAP" } {
	    set lockfile ".RtapEnvLock"
	} else {
	    set lockfile [cconcat "." $name ".lock"]
	}
    	if {[file exists [file join $vltdata ENVIRONMENTS $name $lockfile]]} {
	   return 1
	} else {
	   return 0
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable options ""

    protected variable local 0
}


#************************************************************************
#___oOo___

