#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: pkginSCANLINK.tcl,v 1.105 2005/10/12 19:47:34 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-05-19  SPR VLTSW20050189: reset RTAPENV value.
# rschmutz  2003-11-23  SPR 20030300: support multiple detectors under
#			the same remote environment.
# rschmutz  2003-01-04  index added to scanlink name (to be able to diff.).
# rschmutz  2001-03-25  SPR 20010136: dbRead/Write -> seq_dbRead/WriteSymbolic.
# rschmutz  2001-01-08  single quotes added to -r/-s scanLinks options.
# rschmutz  2001-01-08  clean scan tables only in the first install.
# rschmutz  2000-09-19  Verify method added.
# rschmutz  2000-05-13  created (based in code from pkginCEnvironment.tcl).
#

#************************************************************************
#   NAME
#	pkginSCANLINK - scanlink
# 
#   SYNOPSIS
#	pkginSCANLINK name -remoteenv <env1> -localenv <env2>
#			   -file <linkfile>
# 
#   DESCRIPTION
#	pkginSCANLINK is used to install scanlinks.
#
#   PARENT
#	ctooOBJ_BASE pkginLOG
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure a software module from a configuration file.
#	    <cfg> should be an instance of type pkginCONFIG.
#	    The following config. FITS keywords are expected
#	    and stored in the public data described below:
#	        INSTALL.SCAN<index>.REMOTEENV -> remoteenv
#	    	INSTALL.SCAN<index>.LOCALENV  -> localenv (optional)
#	        INSTALL.SCAN<index>.FILE      -> file     (optional)
#	        INSTALL.SCAN<index>.SRCROOT   -> srcroot  (optional)
#		INSTALL.SCAN<index>.DESTROOT  -> destroot (optional)
#		INSTALL.SCAN<index>.SCRIPT    -> script   (optional)
#		INSTALL.SCAN<index>.SRCATTR   -> srcattr  (optional)
#		INSTALL.SCAN<index>.DESTATTR  -> destattr (optional)
#
#	Install
#	    Install the scanlinks.
#	    1. Runs the specified script.  If the script is "DCS",
#	       then a special script to install the DCS scanlinks
#	       is executed.  A corresponding detector must be
#	       configured in the config. file, if this option is
#	       used.
#	    2. Runs utility scanLinks, if INSTALL.SCANi.FILE is
#	       supplied.
#
#	Verify
#	    Verify the scan links.
#
#   PUBLIC DATA
#	name:	    scanlink name
#	remoteenv:  remote environment from where data is scanned.
#	localenv:   local environment receiving scan data
#		    (default: <INSTALL.RTAPENV1.NAME>).
#	file:	    scan file defining the scan links to be created
#		    (not used, if INSTALL.SCANi.SCRIPT is used).
#	srcroot:    root point for source attributes (default: none).
#	destroot:   root point for dest. attributes (default: none).
#	script:	    script to install the scan links (optional, by
#		    default, pkginBuild itself creates the links).
#	srcattr:    source attribute used for testing the scan links.
#	destattr:   dest. attribute used for testing the scan links.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: pkginSCANLINK
#------------------------------------------------------------------------

class pkginSCANLINK {
    inherit ctooOBJ_BASE pkginBASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
	configure -objType SCANLINK
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
    
	#*# 1. scanlink source environment
	set txt [$cfg Get INSTALL.SCAN$i.REMOTEENV]
	if { "$txt" == "" } {
	    error "INSTALL.SCAN$i.REMOTEENV is empty."
	}
	configure -remoteenv $txt

	#*# 2. destination environment
	set txt ""
	catch { set txt [$cfg Get INSTALL.RTAPENV1.NAME] }
	catch { set txt [$cfg Get INSTALL.SCAN$i.LOCALENV] }
	if { "$txt" == "" } {
	    error "INSTALL.SCAN$i.LOCALENV not known."
	}
	configure -localenv $txt

	#*# 3. scan file
	set txt ""
	catch { set txt [$cfg Get INSTALL.SCAN$i.FILE] }
	configure -file $txt

	#*# 4. source root
	set txt ""
	catch { set txt [$cfg Get INSTALL.SCAN$i.SRCROOT] }
	configure -srcroot $txt

	#*# 5. dest. root
	set txt ""
	catch { set txt [$cfg Get INSTALL.SCAN$i.DESTROOT] }
	configure -destroot $txt

	#*# 5. script
	set txt ""
	catch { set txt [$cfg Get INSTALL.SCAN$i.SCRIPT] }
	configure -script $txt

	#*# 6. check that either file or script have been specified.
	if { "$file" == "" && "$script" == "" } {
	    error "INSTALL.SCAN$i: FILE or SCRIPT are missing."
	}

	#*# 7. source and destination test attribute
	set txt ""
	catch { set txt [$cfg Get INSTALL.SCAN$i.SRCATTR] }
	configure -srcattr $txt
	catch { set txt [$cfg Get INSTALL.SCAN$i.DESTATTR] }
	configure -destattr $txt

	#*# 8. name
	#
	#      Add an index to allow multiple plugins
	#      for the same hook.
	set txt [cconcat $remoteenv "_" $localenv]
	loop j 1 100 {
	    if { "[[Parent] LookupName ${txt}_$j]" == "" } {
	    	break
	    }
	}
	configure -name ${txt}_$j
    }

    #****************************************************************
    #*# method: Install
    #----------------------------------------------------------------

    public method Install {} {

	Info "       Scanlink [format %-7s $remoteenv] to [format %-8s $localenv:] installing ... " 0
	set tmpenv ""
	catch { set tmpenv [GetEnv RTAPENV] }
	try_eval {
	    #*# 1. if script: execute it
	    if { "$script" != "" } {

	    	#*# 1.1 if detector: call DCS InstallScanlinks
	    	if { [lindex $script 0] == "DCS" || "$script" == "DET" } {
		    set container [[[Parent] Parent] Contents DETECTORS]
		    set dcsName [lindex $script 1]
		    if { $dcsName != "" } {
		        set objs [$container LookupName $dcsName]
			if { "$objs" == "" } {
		    	    error "No detector $dcsName found.  Check scanlink configuration $name."
			}
			foreach obj $objs {
			    if { [$obj cget -ccdlenv] != $remoteenv } {
		    	        error "Detector $dcsName CCDLENV entry ([$obj cget -ccdlenv]) does not match scanlink $name remote env. ($remoteenv)."
			    }
			}
		    } else {
		        set objs [$container LookupValue "-ccdlenv" $remoteenv]
		        if { "$objs" == "" } {
		    	    error "No detector found for env. $remoteenv.  Check scanlink configuration $name."
		        }
		    }
		    set clean [expr {![lcontain $cleanedLcus $remoteenv]}]
		    foreach obj $objs {
		        $obj InstallScanLinks $clean
			set clean 0
		    }

		#*# 1.2 run script
		} else {
	    	    System $script
		}

		#*# 1.3 do not clean scan tables from now on
		if { ![lcontain $cleanedLcus $remoteenv] } {
		    lappend cleanedLcus $remoteenv
		}
	    } 

	    #*# 2. if file: call scanLinks, etc.
	    if { "$file" != "" } {
	    	SetEnv RTAPENV $localenv
		# if several files are supplied: clean only first time.
		if { ![lcontain $cleanedLcus $remoteenv] } {
	    	    System "scanOn"
		    System "scanDevOff $remoteenv"
		    System "scanTblClean -e $remoteenv"
		    System "scanDevOn $remoteenv"
		    lappend cleanedLcus $remoteenv
		}
		set path [seq_findFile config/$file]
		set cmd "scanLinks -e $remoteenv -f $path -E"
		if { "$srcroot" != "" } {
		    append cmd " -s '$srcroot'"
		}
		if { "$destroot" != "" } {
		    append cmd " -r '$destroot'"
		}
		System $cmd
	    }
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	} {
	    if { "$tmpenv" != "" } {
	        SetEnv RTAPENV $tmpenv
	    }
	}
	
	Info "OK."
    }

    #****************************************************************
    #*# method: Verify
    #----------------------------------------------------------------

    public method Verify {} {

	#*# 1. check that a test attribute is supplied
	if { $srcattr == "" || $destattr == "" } {
	    Info "       Scanlink [format %-7s $remoteenv] to [format %-8s $localenv:] no test attr. SKIPPED."
	    return	
	}
	    
	Info "       Scanlink [format %-7s $remoteenv] to [format %-8s $localenv:] verifying ... " 0
	catch { seq_ccsInit }
	try_eval {
	    
	    #*# 2. check that the attributes are readable
	    if {[catch { seq_dbReadSymbolic "@$remoteenv:$srcattr" }]} {
	        error "Error reading LCU test attribute (@$remoteenv:$srcattr)."
	    }
	    if {[catch { seq_dbReadSymbolic "@$localenv:$destattr" }]} {
	        error "Error reading WS test attribute (@$localenv:$destattr)."
	    }

	    #*# 3. test scan link
	    set val [seq_dbReadSymbolic "@$remoteenv:$srcattr"]
	    try_eval {
	        seq_dbWriteSymbolic "@$remoteenv:$srcattr" 9999
	        set srcval [seq_dbReadSymbolic "@$remoteenv:$srcattr"]
		loop i 0 5 {
		    sleep 1
		    set destval ""
	    	    catch { set destval [seq_dbReadSymbolic "@$localenv:$destattr"] }
		    if { "$srcval" == "$destval" } {
		    	break
		    }
		}
		if { "$srcval" != "$destval" } {
		    error "Scanlink failed (@$remoteenv:$srcattr -> $localenv:$destattr)."
		}
	    } {
		error $errorResult $errorCode $errorInfo
	    } {
	        seq_dbWriteSymbolic "@$remoteenv:$srcattr" $val
	    }
	} {
	    LogErr $errorResult
	    Info "FAILURE."
	    error $errorResult $errorCode $errorInfo
	}
	Info "OK."
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable remoteenv ""
    public variable localenv  ""
    public variable file      ""
    public variable srcroot   ""
    public variable destroot  ""
    public variable script    ""
    public variable srcattr   ""
    public variable destattr  ""

    protected common cleanedLcus ""
}


#************************************************************************
#___oOo___

