#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: scanei.tcl,v 2.60 2005/07/31 20:47:42 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# tebert    13/03/96  created
# tebert    12/09/96  check for LAN<env> point added
# tebert    28/10/96  added handling of CE
# bgilli    20/05/97  replaced obsolescent tkwait by vwait
# tebert    01/03/99  replaced send with seq_msgSendCommand
# bgilli    06/06/02  updated man page with -l option (SPR 20020043)
#

#************************************************************************
#   NAME
#   scanei - The CCS Scan System Engineering User Interface
#
#   SYNOPSIS
#   scanei [-l <envName] &
#
#   DESCRIPTION
#   The CCS Scan System Engineering User Interface allows a user 
#   to control and check the functionality of the CCS Scan System
#   running in the local environment which is (by default) defined
#   by the RTAPENV environment variable.
#   The -l option allows to redefine the local environment.
#
#   MENU BAR
#   File menu:
#         "Quit"          - exit the panel
#
#   Help menu:
#         "Extended Help" - not implemented yet
#
#   APPLICATION AREA
#   The application area contains the following sections:
#
#   "RtapScanMngr"
#       The status of the RtapScanMngr-process is displayed;
#       possible values are 
#          - ACTIVE     -> RtapScanMngr is running
#          - NOT_ACTIVE -> RtapScanMngr is not active
#
#       "START"
#           The RtapScanMngr is started
#      
#       "STOP"
#           The RtapScanMngr is terminated
#           
#   "CCS Scan Task"
#       The status of the CCS Scan Task (ccsScan) is displayed;
#       possible values are 
#          - ACTIVE     -> ccsScan is running
#          - NOT_ACTIVE -> ccsScan is not active
#          - FAILED     -> RtapScanMngr failed to schedule ccsScan
#
#       "START"
#           The CCS Scan Task is started
#      
#       "STOP"
#           The CCS Scan Task is stopped
#
#   "Scan Shared Memory"
#       The size and usage of the Scan Shared Memory Area (SSMA) is displayed;
#       the SSMA is used to store the data scanned for later (asynchronous) 
#       processing by the calculation engine; 
#       For each attribute to be scanned SSMA-memory is allocated, which is 
#       ONLY freed again by a shutdown of the RtapScanMngr; if the SSMA is 
#       fully occupied, no additional attributes can added to be scanned.
#
#   "Environments"
#       All environments scanned by the local CCS Scan System as well as
#       their status are displayed;
#       Selecting an environment (left mouse button) causes the system to
#       popup a further panel displaying more detailed information about
#       the system with reference to that environment; for more detailed 
#       information see scaneiEnvStatus.
#
#   "Scan Interlock"
#       The status of the scan interlock is displayed; possible values are
#           - ACTIVE     -> scan interlock is set
#           - NOT_ACTIVE -> scan interlock is not set
#       The scan interlock is used by the Scan Manager to coordinate the
#       control calls from multiple application programs; this interlock
#       should normally be NOT_ACTIVE; if it is ACTIVE for a longer period
#       is should be reset; so long as the interlock is set, control requests
#       as done by scanConfig are not possible.
#
#   "Message Queue"
#       The status of the scan task message queue is displayed; possible
#       values are
#           - OK       -> everything OK
#           - OVERFLOW -> messages have been lost by the CCS Scan Task
#       
#   "Reset"
#       The "Scan Interlock" and "Message Queue"-status are reset.
#
#   ACTIONS
#   "Update"                
#       Causes an update of all data displayed in the panel
#
#   "Check Configuration"   
#       Invokes a check of the local Scan System configuration
#
#   ENVIRONMENT
#   RTAPENV         Name of local environment
#   VLTROOT         Path to the global VLT area
#   INTROOT         Path to a public integration area (optional)
#   MODROOT         Path to a private module area (optional)
#
#   RETURN VALUES
#   NONE
#
#   SEE ALSO
#   scaneiEnvStatus(1)
#
#------------------------------------------------------------------------
#
proc scaneiSendClient {} {
    global gvar seq_ccsEnvName

    set del ""
    set cnt [llength $gvar(scaneiChilds)]
    for {set i 0} { $i < $cnt } {incr i} {
        set dest [lindex $gvar(scaneiChilds) $i]
        if { [catch {seq_msgSendCommand $seq_ccsEnvName $dest SCRIPT scaneiRefreshEnvPan} cmdId] } {
            lappend del $dest
            scaneiMessage message ERROR $cmdId
        } else {
            catch {seq_msgRecvReply -all -2000 $cmdId}
        }
        
#       if [ catch {send $dest scaneiRefreshEnvPan} ] {
#           lappend del $dest
#       }
    }

    set cnt [llength $del]
    for {set i 0} { $i < $cnt } { incr i } {
        scaneiDelChildName [lindex $del $i]
    }
}

proc scaneiStartMngr {} {
    global gvar

    if { ![info exists env(RTAPROOT)]} {
        if { [seq_catch { exec msgSchedule "" scanMngr } result] } {
            scaneiMessage message ERROR $result
        } else {
            sleep 5
            scaneiRefreshTop
            scaneiSendClient
        }
    } else {
        if { [seq_catch { exec RtapRunCmd -u -m RtapScanMngr} result] } {
            scaneiMessage message ERROR $result
        } else {
            scaneiRefreshTop
            scaneiSendClient
        }
    }
}

proc scaneiStopMngr {} {
    global gvar

    scaneiMessage message QUESTION \
    "\t\tARE YOU SURE ?? \nYou will stop the Scan System for the whole RTAP\nenvironment, which may concern other users too !!"
    
    if { $gvar(scaneiMessage) == 0 } return

    if { [seq_catch {exec scanOff} result] } {
        scaneiMessage message ERROR $result
    } else {
        if { [seq_catch {exec scanMngrPid} result] } {
            scaneiMessage message ERROR $result
        } else {
           set pid $result
	    if { [seq_catch {exec kill $pid} result] } {
               scaneiMessage message ERROR "$result\nYou have probably not enough privileges" 
           }
        }

        sleep 5
        scaneiRefreshTop
        scaneiSendClient
    }
}

proc scaneiStartScan {} {
    global gvar

    if { [ seq_catch {exec scanOn} result] } {
        scaneiMessage message ERROR $result
    }

    sleep 5
    scaneiRefreshTop
    scaneiSendClient
}

proc scaneiStopScan {} {
    global gvar

    scaneiMessage message QUESTION \
    "\t\tARE YOU SURE ?? \nYou will stop the Scan System for the whole RTAP\nenvironment, which may concern other users too !!"
    
    if { $gvar(scaneiMessage) == 0 } return

    if { [ seq_catch {exec scanOff} result] } {
        scaneiMessage message ERROR $result
    }

    sleep 5
    scaneiRefreshTop
    scaneiSendClient
}

proc scaneiFirstCheck {} {
    if { [seq_catch {seq_dbGetFamilyNames "<alias>scan config" alias} result] } {
        scaneiMessage message ERROR "$result\nIt is not possible to access the Scan System\nDatabase Branch -> check database"
        return 1
    }

    if { [seq_catch {seq_dbGetFamilyNames "<alias>LAN" alias} result] } {
        scaneiMessage message ERROR "$result\nLAN point probably not existing -> check database"
        return 1
    }

    if { [scaneiInitMasks] } {
        scaneiMessage message ERROR "Something is wrong with the attribute\n<alias>LAN.PRBX types -> check database"
        return 1
    }

    return 0
}

proc scaneiCheckConfig {} {
    global gvar
    set errors 0

    #
    #check scan tasks table/communication mode/device type
    #
    if { [seq_catch {seq_dbReadSymbolic "<alias>scan config.scan tasks(0:0)"} result] } {
        scaneiMessage message ERROR "$result -> check database"
        return
    }

    if { [lindex $result 0] != "ccsScan" } {
        scaneiMessage message ASK "!!!!! Configuration ERROR !!!!!\n\"<alias>scan config.scan tasks(0:0,0:0)\"\nshould contain \"ccsScan\"\nThe CCS Scan System cannot work!"
        if { $gvar(scaneiMessage) == "2" } {
            if { [seq_catch {seq_dbWriteSymbolic "<alias>scan config.scan tasks(0:0,0:0)" {TABLE 19 1 1} "ccsScan" } res] } {
                scaneiMessage message ERROR "$res -> check database"
                return
	    }
        } else {
            incr errors
        }
    }

    if { [lindex $result 2] != "ccsScanInit" } {
        scaneiMessage message ASK "!!!!! Configuration ERROR !!!!!\n\"<alias>scan config.scan tasks(0:0,2:2)\"\nshould contain \"ccsScanInit\"\nThe CCS Scan System cannot work!"
        if { $gvar(scaneiMessage) == "2" } {
            if { [seq_catch {seq_dbWriteSymbolic "<alias>scan config.scan tasks(0:0,2:2)" {TABLE 19 1 1} "ccsScanInit" } res] } {
                scaneiMessage message ERROR "$res -> check database"
                return
	    }
        } else {
            incr errors
	}
    }

    if { [seq_catch {seq_dbReadSymbolic "<alias>LAN.communication mode"} result] } {
        scaneiMessage message ERROR "$result -> check database"
        return
    }

    if { $result != "0" } {
        scaneiMessage message ASK "!!!!! Configuration ERROR !!!!!\n\"<alias>LAN.communication mode\"\nshould be \"0\"\nThe CCS Scan System cannot work!"
        if { $gvar(scaneiMessage) == "2" } {
            if { [seq_catch {seq_dbWriteSymbolic "<alias>LAN.communication mode" {SCALAR 1 1 1} 0 } res] } {
                scaneiMessage message ERROR "$res -> check database"
                return
	    }
        } else {
            incr errors
	}
    }
     if { [seq_catch {seq_dbReadSymbolic "<alias>LAN.device type"} result] } {
        scaneiMessage message ERROR "$result -> check database"
        return
    }
    if { $result != "0" } {
        scaneiMessage message ASK "!!!!! Configuration ERROR !!!!!\n\"<alias>LAN.device type\"\nshould be \"0\"\nThe CCS Scan System cannot work!"
        if { $gvar(scaneiMessage) == "2" } {
            if { [seq_catch {seq_dbWriteSymbolic "<alias>LAN.device type" {SCALAR 3 1 1} 0 } res] } {
                scaneiMessage message ERROR "$res -> check database"
                return
	    }
        } else {
            incr errors
	}       
    }

    if { [expr {$errors}] == 0 } {
        scaneiMessage message INFO "Check passed, everything is OK"
    } else {
        scaneiMessage message INFO "Check finished, there are still $errors errors"
    }
}

proc scaneiInitMasks {} {
    global scaneiMasks

    set cnt 1
    for {set i  0} {$i < 32} {incr i} {
        set index [format "%#010x" [expr {$cnt}]]
        set index "(0x)[string range $index 2 end]"
        set scaneiMasks($index) 0
        set cnt [expr {$cnt * 2}]
    }
    
    if { [catch {seq_dbGetAttrInfo "<alias>LAN.PRBX types"} result] } {
        scaneiMessage message ERROR $result
        set size 0
        return 1
    } else {
        set size [lindex $result 2]
    }
   
    if { [expr {$size}] > 32 } {
        scaneiMessage message ERROR "Unexpected size of PRBX table (> 32)"
    }
 
    if { [catch {seq_dbReadSymbolic "<alias>LAN.PRBX types"} result] } {
        scaneiMessage message ERROR $result
        return 1
    }
    
    set cnt 1
    for {set i  0} {$i < [expr {$size}]} {incr i} {
        set day  [lindex [lindex $result $i] 4]
        set hour [lindex [lindex $result $i] 5]
        set min  [lindex [lindex $result $i] 6]
        set sec  [lindex [lindex $result $i] 7]
        set index [format "%#010x" [expr {$cnt}]]
        set index "(0x)[string range $index 2 end]"
        set scaneiMasks($index) [expr {($day * 86400) + ($hour * 3600) + ($min *60) + $sec}]
        set cnt [expr {$cnt * 2}]
    }

    return 0
}

proc scaneiRefreshTop { } {
    global gvar cv

    set res -1

    if { [seq_catch { exec scanQueryState -s "scan config" } result] } {
        scaneiMessage message ERROR $result
        $cv.scanei-scanMngr.cv.startBt configure -state normal
        $cv.scanei-scanMngr.cv.stopBt  configure -state normal
    } else {
        if { [lindex $result 1] == "ENABLED" } {
            set res 1
            $cv.scanei-scanMngr.cv.startBt configure -state disabled
            $cv.scanei-scanMngr.cv.stopBt  configure -state normal
        } else {
            set res 0
            $cv.scanei-scanMngr.cv.startBt configure -state normal
            $cv.scanei-scanMngr.cv.stopBt  configure -state disabled
        }
    }     
    set gvar(scanei-scanMngr_statusVar) $res

    if { $res == 0} {
        set state DISABLED
        $cv.scanei-ccsScan.cv.startBt configure -state disabled
        $cv.scanei-ccsScan.cv.stopBt  configure -state disabled
    } else {
        set res 2
        set state UNKNOWN
        if { [seq_catch { exec scanQueryState -t "LAN" } result] } {
            scaneiMessage message ERROR $result
            $cv.scanei-ccsScan.cv.startBt configure -state normal
            $cv.scanei-ccsScan.cv.stopBt  configure -state normal
        } else {
            if { [lindex $result 1] == "ENABLED" } {
                set res 1
                set state ENABLED
                $cv.scanei-ccsScan.cv.startBt configure -state disabled
                $cv.scanei-ccsScan.cv.stopBt  configure -state normal
            } else {
                set res 0
                set state DISABLED
                $cv.scanei-ccsScan.cv.startBt configure -state normal
                $cv.scanei-ccsScan.cv.stopBt  configure -state disabled
            }
        }
    }
    set gvar(scanei-ccsScan_statusVar) $res

    set size 10000
    set free     0
    if { $gvar(scanei-scanMngr_statusVar) == 1 } {
        if { [seq_catch { exec scanSsma } result] } {
             scaneiMessage message ERROR "$result \nScan System probably not active !"
        } else {
            set size [lindex $result 0]
            set free [lindex $result 1]
        }
    }
    set used [expr {$size - $free}]
    set gvar(scanei-ssma_ssmaSize) $size
    set gvar(scanei-ssma_ssmaUsed) $used
    set gvar(scanei-ssma_ssmaPercent) [expr {100.0*$used/$size}]
    #$cv.scanei-ssma.cv.ssma-bar configure -maxVal $size

    if { [catch {seq_dbReadSymbolic "<alias>scan config.scan interlock"} res] } {
       scaneiMessage message ERROR $res
       set res 0
    }
    set gvar(scanei-sys_interlockVar) $res

    if { [catch {seq_dbReadSymbolic "<alias>LAN.msg queue status"} res] } {
       scaneiMessage message ERROR $res
       set res -1
    }
    set gvar(scanei-sys_queueVar) $res

    $cv.scanei-envs.cv.envs deleteItem 0 end
    if { [catch {seq_dbGetFamilyNames "<alias>LAN" alias} res] } {
        scaneiMessage message ERROR $res        
        set res ""
    }

    set cnt [llength $res]
    for {set i 1} {$i < [expr {$cnt}]} { incr i} {
        set env [string range [lindex $res $i] 7 end]
        if { $state == "UNKNOWN" } {
            set result 111
        } elseif { $state == "DISABLED" } {
            set result  0
        } elseif { [catch {seq_dbReadSymbolic "<alias>$env.scan input status"} result] } {
           scaneiMessage message ERROR $res
           set result 111
        }
        switch -exact $result {
	    "111" { set item [format "%-8s UNKNOWN"  $env]  }
	    "0"  { set item [format "%-8s DISABLED" $env]  }
	    "1"  { set item [format "%-8s ENABLED"  $env]  }
            "2"  { set item [format "%-8s FAILED"   $env]  }
        }
        $cv.scanei-envs.cv.envs addItem end $item
    }
}

proc scaneiResetStates { } {
    if { [catch {seq_dbWriteSymbolic "<alias>scan config.scan interlock" {SCALAR 1 1 1} 0 } res] } {
        scaneiMessage message ERROR $res
    }

    if { [catch {seq_dbWriteSymbolic "<alias>LAN.msg queue status" {SCALAR 3 1 1} 0 } res] } {
        scaneiMessage message ERROR $res
    }    
    scaneiRefreshTop
}

proc scaneiInitTable { env } {
    global gvar cv table tableIndex

    set table($env) ""
    set tableIndex($env) ""
    set tmp ""

    if { [catch {seq_dbGetAttrInfo "<alias>$env.scan input"} result] } {
       scaneiMessage message ERROR $result
       set size 0
    } else {
        set size [lindex $result 2]
    }

    set start 0
    while {$start < $size} {
        set end [expr {$start + 9}]
        if { $end >= $size } {
            set end  [expr {$size - 1}]
        }

        if { [catch {seq_dbReadSymbolic "<alias>$env.scan input($start:$end)"} result] } {
           scaneiMessage message ERROR $result
	} else {
            set tmp [concat $tmp $result]
	}
        incr start 10
    }

    for {set i 0} {$i < [expr {$size}]} {incr i} {
        set item [lindex $tmp $i]
        if { [string length [lindex $item 17]] > 0 } {
            set dirAddr [lindex $item 9]
            if { [catch {seq_dbDirAddrToName $dirAddr alias} result] } {
                scaneiMessage message ERROR $result    
                set xx "$item {???????????????}"   
            } else {
                set xx "$item {$result}"
            }
  
            lappend table($env) $xx
            lappend tableIndex($env) $i
	}
    }        
}

proc scaneiRefreshEnvStatus { env } {
    global gvar cv table bcv

    set state UNKNOWN
    if { [seq_catch { exec scanQueryState -t "LAN" } result] } {
        scaneiMessage message ERROR $result
    } else {
        if { [lindex $result 1] == "ENABLED" } {
            set res 1
            set state ENABLED
        } else {
            set res 0
            set state DISABLED
        }
    }

    if { $state == "UNKNOWN" } {
        set result -1
    } elseif { $state == "DISABLED" } {
        set result  0
    } elseif { [catch {seq_dbReadSymbolic "<alias>$env.scan input status"} result] } {
        scaneiMessage message ERROR $result
        set result -1
    }

    set gvar(scaneiEnvStatus-env_envStat) $result

     if { $result == 1 } {
         $cv.scaneiEnvStatus-env.cv.enable configure -state disabled       
         $cv.scaneiEnvStatus-env.cv.disable configure -state normal       
     } elseif { $result == 0} {
         $cv.scaneiEnvStatus-env.cv.enable configure -state normal
         $cv.scaneiEnvStatus-env.cv.disable configure -state disabled
     } else {
         $cv.scaneiEnvStatus-env.cv.enable configure -state normal       
         $cv.scaneiEnvStatus-env.cv.disable configure -state normal       
     }
    
    $cv.scaneiEnvStatus-table.cv.attr deleteItem 0 end
    for { set i 0 } { $i < [llength $table($env)] } { incr i } {
        set item [lindex [lindex $table($env) $i] 17]
        $cv.scaneiEnvStatus-table.cv.attr addItem end $item
    }


    if { [catch {seq_dbReadSymbolic "<alias>$env.restart flag"} result] } {
        scaneiMessage message ERROR $result
        set result 0
    } 
    set gvar(scaneiEnvStatus-restart_autoStatus) $result

    if { [catch {seq_dbReadSymbolic "<alias>$env.restart delay"} result] } {
        scaneiMessage message ERROR $result
        set gvar(scaneiEnvStatus-restart_delay) 0
    } else {
        set time [split $result :]
        set gvar(scaneiEnvStatus-restart_delay) [round [expr {[lindex $time 0] * 3600 + [lindex $time 1] * 60 + [lindex $time 2]}]]
    }
    
    if { $gvar(scaneiEnvStatus-restart_autoStatus) == 0 } {
         $cv.scaneiEnvStatus-restart.cv.enable configure -state normal
         $cv.scaneiEnvStatus-restart.cv.disable configure -state disabled
    } else {
         $cv.scaneiEnvStatus-restart.cv.enable configure -state disabled
         $cv.scaneiEnvStatus-restart.cv.disable configure -state normal
    }
}

proc scaneiEntryMultiple { env } {
    global cv

    set selection [$cv.scaneiEnvStatus-table.cv.attr getIndex]
    set size [llength $selection]
    if { $size <= 0 } {
        return
    }

    incr size -1
    scaneiEntrySingle $env [lindex $selection $size]
}

proc scaneiEntry { env index } {
    global cv table

    if { $index < 0 || $index > [llength $table($env)] } {
        set index 0
    }

    $cv.scaneiEnvStatus-table.cv.attr setCursor [expr {$index + 1}]
    scaneiEntrySingle $env $index
}

proc scaneiEntrySingle { env index } {
    global gvar cv table scaneiMasks scaneiIndex

    set scaneiIndex $index

    if { [llength $table($env)] > 0 } {
        $cv.scaneiEnvStatus-info.cv.enableCe    configure  -state normal
        $cv.scaneiEnvStatus-info.cv.disableCe   configure  -state normal
        $cv.scaneiEnvStatus-info.cv.enableCEopt configure  -state normal
        set gvar(scaneiEnvStatus-info_attrStat) [lindex [lindex $table($env) $index] 1]
        if { $gvar(scaneiEnvStatus-info_attrStat) == "0" } {
            $cv.scaneiEnvStatus-info.cv.enable configure -state normal
            $cv.scaneiEnvStatus-info.cv.disable configure -state disabled
        } elseif { $gvar(scaneiEnvStatus-info_attrStat) == "1" } {
            $cv.scaneiEnvStatus-info.cv.enable configure -state disabled
            $cv.scaneiEnvStatus-info.cv.disable configure -state normal 
        } else {
            $cv.scaneiEnvStatus-info.cv.enable configure -state normal
            $cv.scaneiEnvStatus-info.cv.disable configure -state normal
        }
        if { [lindex [lindex $table($env) $index] 2] == "" } {
            set gvar(scaneiEnvStatus-info_scanMode) SRBX
            set type [lindex [lindex $table($env) $index] 13]
            set deadband [lindex [lindex $table($env) $index] 16]
            switch $type {
                2 {set gvar(scaneiEnvStatus-info_modeVar1) "NO DEADBAND" }
                1 {set deadband [format "%f" $deadband]
                   set gvar(scaneiEnvStatus-info_modeVar1) "$deadband %" }
                0 {set gvar(scaneiEnvStatus-info_modeVar1) [format "%f" $deadband]   }
            }
        } else {
            set period [lindex [lindex $table($env) $index] 2]
            set gvar(scaneiEnvStatus-info_scanMode) POLL
            set gvar(scaneiEnvStatus-info_modeVar1) [format "%d sec" $scaneiMasks($period)]
        }
 
        set gvar(scaneiEnvStatus-info_dataSize) [lindex [lindex $table($env) $index] 11]
      
        set gvar(scaneiEnvStatus-info_localAttr) [lindex [lindex $table($env) $index] 18]
        set stat 2
        if { [seq_catch {seq_dbGetAttrInfo $gvar(scaneiEnvStatus-info_localAttr)} result] } {
            scaneiMessage message ERROR $result
        } else {
            set type [lindex $result 0]
            if { $type == "SCALAR" } {
                set attr $gvar(scaneiEnvStatus-info_localAttr)
            } else {
                set attr [format "%s(0:0)" $gvar(scaneiEnvStatus-info_localAttr)]
	    }
            if { [seq_catch {seq_dbReadSymbolic $attr stat} result] } {
                scaneiMessage message ERROR $result
            }
        }
        set gvar(scaneiEnvStatus-info_quality) $stat      
        set point [split $gvar(scaneiEnvStatus-info_localAttr) .]
        if { [seq_catch {exec scanGetStatCE \"[lindex $point 0]\" } result] } {
            scaneiMessage message ERROR $result
            set gvar(scaneiEnvStatus-info_ceVar) 4
        } else {
            if { $result == "ENABLED" } {
                set gvar(scaneiEnvStatus-info_ceVar) 1
            } elseif { $result == "ENABLED_OPT" } {
               set gvar(scaneiEnvStatus-info_ceVar) 2 
            } elseif { $result == "DISABLED" } {
                set gvar(scaneiEnvStatus-info_ceVar) 0
            } else {
                set gvar(scaneiEnvStatus-info_ceVar) 4
            }
        }
 
    } else {
        set gvar(scaneiEnvStatus-info_attrStat) -1
        set gvar(scaneiEnvStatus-info_localAttr) "???????????????"
        set gvar(scaneiEnvStatus-info_scanMode) "??????"
        set gvar(scaneiEnvStatus-info_modeVar1) "??????????"        
        set gvar(scaneiEnvStatus-info_quality) 2
        set gvar(scaneiEnvStatus-info_dataSize) 0
        set gvar(scaneiEnvStatus-info_ceVar) 4
        $cv.scaneiEnvStatus-info.cv.enable      configure  -state disabled
        $cv.scaneiEnvStatus-info.cv.disable     configure  -state disabled
        $cv.scaneiEnvStatus-info.cv.enableCe    configure  -state disabled
        $cv.scaneiEnvStatus-info.cv.disableCe   configure  -state disabled
        $cv.scaneiEnvStatus-info.cv.enableCEopt configure  -state disabled
    }
}

proc scaneiEnableCE { } {
    global gvar scaneiIndex

    set point [split $gvar(scaneiEnvStatus-info_localAttr) .]
    if { [llength $point] != 2 } {
        return
    } else {
        if { [seq_catch {exec scanCtrlCE -p \"[lindex $point 0]\" -e } result] } {
            scaneiMessage message ERROR $result
        }

        scaneiEntry $gvar(scaneiEnv) $scaneiIndex
    }    
}

proc scaneiEnableOpt { } {
    global gvar scaneiIndex

    set point [split $gvar(scaneiEnvStatus-info_localAttr) .]
    if { [llength $point] != 2 } {
        return
    } else {
        if { [seq_catch {exec scanCtrlCE -p \"[lindex $point 0]\" -o } result] } {
            scaneiMessage message ERROR $result
        }
        scaneiEntry $gvar(scaneiEnv) $scaneiIndex
    }    
}

proc scaneiDisableCE {} {
    global gvar scaneiIndex

    set point [split $gvar(scaneiEnvStatus-info_localAttr) .]
    if { [llength $point] != 2 } {
        return
    } else {

        scaneiMessage message QUESTION "This will disable the CE for all attributes\nlocated in the same point than the attribute\nselected. Do you want to CONTINUE ?"
        if { $gvar(scaneiMessage) == 0 } return
        if { [seq_catch {exec scanCtrlCE -p \"[lindex $point 0]\" -d } result] } {
            scaneiMessage message ERROR $result
        }
        scaneiEntry $gvar(scaneiEnv) $scaneiIndex
    }
}

proc scaneiSearch { env attr } {
    global table scaneiIndex
    
    set result -1

    if { [expr {$scaneiIndex + 1} ] > [expr {[llength $table($env)]}] } {
        set scaneiIndex -1
    }

    for { set i [expr {$scaneiIndex + 1} ] } {$i < [expr {[llength $table($env)]}] } { incr i } {
        set item [lindex [lindex $table($env) $i] 17]       
        if { $item == $attr } {
            set result $i
            break
        }
    }

    if { $result == -1 } {
        for { set i [expr {$scaneiIndex + 1}] } { $i < [expr {[llength $table($env)]}] } { incr i } {
            set item [lindex [lindex $table($env) $i] 17]
            if { [string first $attr $item] != -1  } {
                set result $i 
                break
            }
        }
    }

    if { $result == -1 } {
        for { set i 0 } { $i <= $scaneiIndex } { incr i } {
            set item [lindex [lindex $table($env) $i] 17]
            if { [string first $attr $item] != -1  } {
                set result $i 
                break
            }
        }
    }
    
    if { $result == -1 || [expr {[llength $table($env)]}] <= [expr {$result}] } {
        set item [lindex [lindex $table($env) $scaneiIndex] 17]
        if { [string first $attr $item] != -1  } {
            set result $scaneiIndex
	} else {
            set result 0
	}
    }

    set scaneiIndex $result
    return $result
}

proc scaneiEnableEnv { penv } {
    global gvar

    if { [seq_catch { exec scanDevOn $penv } result] } {
        scaneiMessage message ERROR $result
    } 
   
    sleep 3
    scaneiInitTable $penv
    scaneiRefreshEnvStatus $penv
    scaneiEntry $penv 0
 
    if { [catch {seq_msgSendCommand $gvar(scaneiParentEnv) $gvar(scaneiParentName) SCRIPT scaneiRefreshTop } cmdId] } {
        scaneiMessage message ERROR $cmdId
    } else {
        catch {seq_msgRecvReply -all -2000 $cmdId}
    }

#    catch {send $gvar(scaneiParent) {scaneiRefreshTop}}
}

proc scaneiDisableEnv { penv } {
    global gvar

    scaneiMessage message QUESTION "ARE YOU SURE ???"
    
    if { $gvar(scaneiMessage) == 0 } return

    if { [seq_catch { exec scanDevOff $penv } result] } {
        scaneiMessage message ERROR $result
    } 

    sleep 3
    scaneiInitTable $penv
    scaneiRefreshEnvStatus $penv
    scaneiEntry $penv 0

    if { [catch {seq_msgSendCommand $gvar(scaneiParentEnv) $gvar(scaneiParentName) SCRIPT scaneiRefreshTop } cmdId] } {
        scaneiMessage message ERROR $cmdId
    } else {
        catch {seq_msgRecvReply -all -2000 $cmdId}
    }

#    catch {send $gvar(scaneiParent) {scaneiRefreshTop}}
}

proc scaneiEnableAttr { env } {
    global gvar cv table tableIndex

    if { $gvar(scaneiEnvStatus-env_envStat) != 1} {
        scaneiMessage message INFO "Scanning for environment disabled/failed !\nNo operation on attributes possible"
        return
    }

    set size  [$cv.scaneiEnvStatus-table.cv.attr getSize]
    set allIndex [$cv.scaneiEnvStatus-table.cv.attr getIndex]
    set len [llength $allIndex]

    if { $size <= 0 } return

    for { set i 0 } { $i < $len } { incr i 10 } {

        set attr ""
        for { set j $i } { $j < $len && [expr {$j - $i}] < 10} { incr j } {
            set index [lindex $allIndex $j]
            lappend attr [format "'%s'" [lindex [lindex $table($env) $index] 18]]
	    }
        
        if { [seq_catch { eval exec scanEnable $attr } result] } {
            scaneiMessage message ERROR $result
	}

        sleep 1

        for { set j $i } { $j < $len && [expr {$j - $i}] < 10 } { incr j } {
            set index [lindex $allIndex $j]       
            set indexTbl [lindex $tableIndex($env) $index]
            if { [catch {seq_dbReadSymbolic "<alias>$env.scan input($indexTbl)"} result] } {
                scaneiMessage message ERROR $result
                scaneiInitTable $env
                scaneiRefreshEnvStatus $env
            } else {
                set tmp "$result [lindex [lindex $table($env) $index] 18]"
                set table($env) [lreplace $table($env) $index $index $tmp]
	    }
        }
    }
    scaneiEntry $env [lindex $allIndex end]
}

proc scaneiDisableAttr { env } {
    global gvar cv table tableIndex

    if { $gvar(scaneiEnvStatus-env_envStat) != 1} {
        scaneiMessage message INFO "Scanning for environment disabled/failed !\nNo operation on attributes possible"
        return
    }    
    set size  [$cv.scaneiEnvStatus-table.cv.attr getSize]
    set allIndex [$cv.scaneiEnvStatus-table.cv.attr getIndex]
    set len [llength $allIndex]

    if { $size <= 0 } return

    for { set i 0 } { $i < $len } { incr i 10 } {
        set attr ""
        for { set j $i } { $j < $len && [expr {$j - $i}] < 10} { incr j } {
            set index [lindex $allIndex $j]
            lappend attr [format "'%s'" [lindex [lindex $table($env) $index] 18]]
	    }

        if { [seq_catch { eval exec scanDisable $attr } result] } {
            scaneiMessage message ERROR $result
	}

        sleep 1

        for { set j $i } { $j < $len && [expr {$j - $i}] < 10 } { incr j } {
            set index [lindex $allIndex $j]       
            set indexTbl [lindex $tableIndex($env) $index]
            if { [catch {seq_dbReadSymbolic "<alias>$env.scan input($indexTbl)"} result] } {
                scaneiMessage message ERROR $result
                scaneiInitTable $env
                scaneiRefreshEnvStatus $env
            } else {
                set tmp "$result [lindex [lindex $table($env) $index] 18]"
                set table($env) [lreplace $table($env) $index $index $tmp]
	    }
        }
    }

    scaneiEntry $env [lindex $allIndex end]
}

proc scaneiRefreshDevPan { env } {
    scaneiInitTable $env
    scaneiRefreshEnvStatus $env    
    scaneiEntry $env 0
}

proc scaneiEnableRestart { environment } {
    global gvar cv
    
    if { [catch {seq_dbWriteSymbolic "<alias>$environment.restart flag" 1} result] } {
       scaneiMessage message ERROR $result
       return
    }

    set gvar(scaneiEnvStatus-restart_autoStatus) 1
    $cv.scaneiEnvStatus-restart.cv.enable configure -state disabled
    $cv.scaneiEnvStatus-restart.cv.disable configure -state normal
}

proc scaneiDisableRestart { environment } {
    global gvar cv
    
    if { [catch {seq_dbWriteSymbolic "<alias>$environment.restart flag" 0} result] } {
       scaneiMessage message ERROR $result
       return
    }

    set gvar(scaneiEnvStatus-restart_autoStatus) 0
    $cv.scaneiEnvStatus-restart.cv.enable configure -state normal
    $cv.scaneiEnvStatus-restart.cv.disable configure -state disabled
}


proc scaneiSetDelay { env delay } {
    if { $delay < 0 } return

    set delay [round $delay]
    set hours [floor [expr {$delay/3600}]]
    set rest [fmod $delay 3600]
    set min [floor [expr {$rest/60}]]
    set rest [fmod $rest 60]
    set res [format "%02d:%02d:%02d" [int $hours] [int $min] [int $rest]]
    
    if { $delay > 86399 } {
        scaneiMessage message ERROR "Time specified greater than 86399 seconds"
    } else {
	if { [catch {seq_dbWriteSymbolic "<alias>$env.restart delay" $res} result]} {
            scaneiMessage message ERROR $result
	}
    }
}

proc scaneiCheckEnv { scanEnv } {
    global gvar seq_ccsEnvName

    set attr(0)  "device poll types"
    set attr(1)  "device PRBX types"
    set attr(2)  "scan input control"
    set attr(3)  "scan input status"
    set attr(4)  "failure threshold"
    set attr(5)  "dev fail threshold"
    set attr(6)  "scan input"
    set attr(7)  "scan output control"
    set attr(8)  "scan output status"
    set attr(9)  "scan output"
    set attr(10) "device stats"
    set attr(11) "restart flag"
    set attr(12) "restart delay"
    set attr(13) "debug mode"
    set attr(14) "scanOut environment"
    set attr(15) "scanOut ProcNum"

    set errors 0

    if { [seq_catch {seq_dbReadSymbolic "<alias>$scanEnv.failure threshold"} result] } {
        scaneiMessage message ERROR "$result -> check database"
        return
    }

    if { $result != "0" } {
        scaneiMessage message ASK "!!!!! Configuration ERROR !!!!!\n\"<alias>$scanEnv.failure threshold\" should contain \"0\"\nThe CCS Scan System cannot work properly!"
        if { $gvar(scaneiMessage) == "2" } {
            if { [seq_catch {seq_dbWriteSymbolic "<alias>$scanEnv.failure threshold" {SCALAR 3 1 1} 0 } res] } {
                scaneiMessage message ERROR "$res -> check database"
                return
	    }
        } else {
            incr errors
        }
    }

    
    if { [seq_catch {exec msgSend -n $scanEnv scanOut PING abc} result] } {
        scaneiMessage message INFO "$result\nscanOut in environment $scanEnv probably not active !\nNo checks on scanOut possible !"
    } elseif { [seq_catch {exec msgSend -n $scanEnv scanOut SCANDES " " 5000} result] } {
        scaneiMessage message ERROR "$result -> check $scanEnv"
        incr errors
    } elseif { [string index $scanEnv 0] == "l" } {
	if { [lindex $result 1] != $seq_ccsEnvName } {
	    scaneiMessage message ERROR "$scanEnv already scanned by environment $result !!!"
	    incr errors
	}
    } else {
	if { [lsearch $result $seq_ccsEnvName] == -1 } {
	    scaneiMessage message ERROR "$scanEnv not scanned by environment $result !!!"
	    incr errors
	}
    }
	    

    if { [seq_catch {seq_dbGetAttrNames "@$scanEnv<alias>$seq_ccsEnvName"} result] } {
        scaneiMessage message ERROR "$result \nCheck database -> $scanEnv<alias>$seq_ccsEnvName is probably not existing"
        return
    }

    for { set i 0 } { $i <= 15 } { incr i } {
        if { [seq_catch {seq_dbGetDirAddr "<alias>$scanEnv.$attr($i)"} result] } {
            scaneiMessage message ERROR "$result -> check database"
            return
        }
	if { [scan $result "@\[%d.%d\]" plin ain] != 2 } {
            scaneiMessage message ERROR "Couldn't parse \"$result\" returned by seq_dbGetDirAddr()"
            incr errors         
	} elseif { $ain != $i } {
            scaneiMessage message ERROR "Order of attributes under <alias>$scanEnv is wrong\nAIN of <alias>$scanEnv.$attr($i) should be $i and not $ain !!!"
            incr errors
	}
    }
   
    if { [expr {$errors}] == 0 } {
        scaneiMessage message INFO "Check passed, everything is OK"
    } else {
        scaneiMessage message INFO "Check finished, there are still $errors errors"
    }
}

proc scaneiCleanTable { environment } {
    global gvar
     
    scaneiMessage message QUESTION "\t\t???? ARE YOU SURE ???\nAll entries in the scan input table will be removed !"

    if { $gvar(scaneiMessage) == 0 } return

    if { [seq_catch {exec scanTblClean -e $environment} result] } {
        scaneiMessage message ERROR "$result\n"
    } else {
        scaneiMessage message INFO "Operation was successful !\nTable is empty now !"
    }

    scaneiInitTable $environment
    scaneiRefreshEnvStatus $environment
    scaneiEntry $environment 0

    if { [catch {seq_msgSendCommand $gvar(scaneiParentEnv) $gvar(scaneiParentName) SCRIPT scaneiRefreshTop } cmdId] } {
        scaneiMessage message ERROR $cmdId
    } else {
        catch {seq_msgRecvReply -all -2000 $cmdId}
    }

#    catch {send $gvar(scaneiParent) {scaneiRefreshTop}}
}

proc scaneiCheckTable { environment } {
    global gvar
 
    set result ""
    if { [catch {exec scanTblClean -e $environment -s} result] } {
        scaneiMessage message ERROR "$result\n"
        return
    }

    if { [llength $result] == 0 } {
        scaneiMessage message INFO "   Scan Input Table checked !\n\tNo error found"
        return
    }

    set repair ""
    for { set i 0} { $i < [llength $result] } {incr i} {
        set entry  [lindex $result $i]
        set number [lindex $entry 0]
        set local  [lindex [lindex $entry 1] 0]
        set remote [lindex [lindex $entry 1] 1]
        set msg    [lindex $entry 2]

        scaneiMessage message ASK "ERROR in record <$number>, found:\n\n    LOCAL ATTRIBUTE: $local\n    REMOTE ATTRIBUTE: $remote\n    ERROR: $msg"
        if { $gvar(scaneiMessage) == 2 } {
            if  { $repair == "" } {
                set repair $number
            } else {
                set repair [format "%s,%d" $repair $number]
	    }
        }
    }

    if { $repair != "" } {
        if { [catch {exec scanTblClean -e $environment -c -r $repair} result] } {
            scaneiMessage message ERROR "$result\n"
	} else {
            scaneiMessage message INFO "\tTable fixed !\nEverthing should be OK now"
	}
    }

    scaneiInitTable $environment
    scaneiRefreshEnvStatus $environment
    scaneiEntry $environment 0

    return
}

proc scaneiMessage { name type message } {
   global gvar

   set c .$name
   set flag -1
   set gvar(scaneiMessage) 0

   toplevel $c

   frame $c.middle         -relief raised -border 1
   frame $c.bot            -relief raised -border 1
   frame $c.middle.center  -relief groove -border 1

   button  $c.bot.ok       -text "Ok" -width 10 -command \
                           "set flag 1; set gvar(scaneiMessage) 1; destroy $c"

   if { $type == "QUESTION" } {
       button  $c.bot.cancel -text "Cancel" -width 10 -command \
                             "set flag 1; destroy $c"
   } elseif { $type == "ASK" } {  
       button  $c.bot.fix -text "Fix" -width 10 -command \
                          "set flag 1; set gvar(scaneiMessage) 2; destroy $c"
   }

   message $c.middle.center.msg  -justify left -text "$message" \
           -font "-Adobe-courier-bold-r-normal--14*" -width 15c 

   if { $type == "ERROR" } {
       label $c.middle.left -bitmap warning
   } elseif { $type == "QUESTION" } {
       label $c.middle.left -bitmap questhead
   } elseif { $type == "ASK" } {
       label $c.middle.left -bitmap question
   } else {
       label $c.middle.left -bitmap info
   }

   $c.middle.left config  -foreground red -borderwidth 5 -height 40 -width 30
 
   pack $c.middle $c.bot -side top -fill x  
   if { $type == "QUESTION" } {
       pack $c.bot.ok $c.bot.cancel -side left -pady 15 -expand yes
   } elseif { $type == "ASK" } {
       pack $c.bot.ok $c.bot.fix -side left -pady 15 -expand yes
   } else {
       pack  $c.bot.ok -side left -pady 15 -expand yes
   }

   pack $c.middle.center $c.middle.left -side right -ipadx 10 -fill x -fill y -fill x -expand yes
   pack $c.middle.center $c.middle.center.msg -side right -ipadx 10 -ipady 10 -expand yes -fill y -fill x

   wm geometry $c +400+400
   wm title $c "CCS Scan System Info"

   tkwait visibility $c
   grab $c   
   vwait flag;      # Stops until variable as been modified
}

proc scaneiGetChildName { name } {
    global gvar
    lappend gvar(scaneiChilds) $name
}

proc scaneiDelChildName { name } {
    global gvar 

    set index [lsearch -exact $gvar(scaneiChilds) $name]
    if { $index != -1 } {
        set gvar(scaneiChilds) [lreplace $gvar(scaneiChilds) $index $index]
    }
}

proc scaneiRefreshEnvPan { } {
    global gvar
    scaneiInitTable $gvar(scaneiEnvStatus-env_envName)
    scaneiRefreshEnvStatus $gvar(scaneiEnvStatus-env_envName)
    scaneiEntry $gvar(scaneiEnvStatus-env_envName) 0
}

#
# ___oOo___
