#***************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: seqCatch.tcl,v 2.97 2005/09/14 09:47:24 vltsccm Exp $" 
# 
# who      when     what
# -------- -------- ------------------
# eallaert 03/22/95 created for cmm
# eallaert 09/06/95 fixed bug - see SPR 950246
# eallaert 28/06/96 fixed CR - see SPR 960330
#
# seqCatch.tcl --
# Create a new command called seq_catch

#***************************************************************************
#   NAME 
#   seq_catch - evaluate a command and return its completion status 
#  
#   SYNOPSIS 
#   seq_catch <command> ?<msg>? ?<sourceId>?
#
#   DESCRIPTION
#   seq_catch has an identical syntax and behaves as the Tcl "catch" command.
#   
#   <command>  : any Sequencer script
#   <msg>      : variable name where to store the result string for <command>
#   <sourceid> : string containing location identifier; remark that only the 
#                first word of this string will be transmitted to the CCS 
#                error logging system
#
#   On top of what "catch" does, seq_catch logs the error (if any) produced 
#   by <command> to the CCS error logging system.  For that purpose it can use
#   the optional <sourceId> argument, which is for the error system known as 
#   the "location identifier".  Typically the caller could set <sourceId> to
#   [info script].  If <sourceId> is not specified, it will be set to the name
#   of the script file which executes this seq_catch command.
#  
#   ENVIRONMENT 
#   no environment nor Tcl variables are read nor set 
# 
#   RETURN VALUES
#   identical to "catch":
#   
#   returns OK, with the result string set to "0" if <command> did not produce 
#   an error.  If <command> returned an error, the result string contains 1 
#   and <msg>, if given, will be set to the error message left by <command>.
#
#   returns ERROR with corresponding message in the result string under the 
#   following conditions:
#   - "wrong # args: should be "seq_catch <command> ?<msg>? ?<sourceId>?"" if 
#     there were more than 2 arguments given to this <command>.
#   
#   EXAMPLES 
#   (For clarity's sake, the example is for an interactive session with 
#   input typed by the user coming after the "seqWish>" prompt, and all
#   replies on a new line) 
#
#   ..... 
#   seqWish>seq_catch {set a $b} msg "MyProcedure"
#   0
#   seqWish>seq_catch {set a $c} msg "MyProcedure"
#   1
#   seqWish>set msg
#   can't read "c": no such variable
#   seqWish>..... 
#
#   SEE ALSO 
#   "Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
#   catch(n), seq_errAdd(n)
# 
#------------------------------------------------------------------------ 
#

# REMARK: comments have been removed and variable names have been kept
#         short for performance reasons.
#    noa: number of optional arguments
#     rv: return value from the Tcl catch command
#    err: eventual error msg contained in the return string for <command>
#    src: see sourceId in seq_errAdd
# The procedure first checks if there are not more than 2 optional args.
# Then it evaluates the <command> wrapped in "catch", using "uplevel" to be
# sure the variables are taken from the right level.  If a <command> took
# the error exit, the error will be logged with a <sourceId> as specified (or
# a default).
# Finally the reply or error message will be stored into "msg" (if specified),
# and the procedure returns the exact value as "catch" would return.

proc seq_catch {cmd args} {
    
    ::internal::seq_initDbg seq_catch "[list $cmd] $args"
    if {[set noa [llength $args]] > 2} {
	set msg {wrong # args: should be "seq_catch <command> ?<msg>? ?<sourceId>?"}
	::internal::seq_returnDbg seq_catch ERR $msg
	error $msg
    }
    if {[set rv [catch {uplevel $cmd} err]]} {
	if {$noa > 1} {
	    set src [lindex [lindex $args 1] 0]
	} else {
	    set src [uplevel info script]
	    switch -- {} $src {
		set src Tcl_Eval
	    } default {
		set ccsLOCID_LEN 80
		if {[string length $src] > $ccsLOCID_LEN} {
		    set src "...[string range $src end-[expr {$ccsLOCID_LEN-4}] end]"
		}
	    }
	}
	seq_errResetStack; seq_errAdd $err $src; seq_errCloseStack
    }
    if {$noa > 0} {
	upvar [lindex $args 0] msg
	set msg $err
    }
    ::internal::seq_returnDbg seq_catch OK $rv
    return $rv
}
