#***************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: seqLogTclErrors.tcl,v 2.97 2005/09/14 09:47:28 vltsccm Exp $" 
# 
# who      when     what
# -------- -------- ------------------
# eallaert 03/22/95 created for cmm
# eallaert 19/09/95 replace addinput by fileevent for Tk4.x
# eallaert 14/02/97 removed tk_version - it can also be used by seqSh
#
# seqLogTclErrors.tcl --
# Create a new command called seq_logTclErrors

#***************************************************************************
#   NAME 
#   seq_logTclErrors - send all output for stderr also to the error log 
#  
#   SYNOPSIS 
#   seq_logTclErrors open|close
#
#   DESCRIPTION
#   seq_logTclErrors allows to send strings that normally go to exclusively
#   to stderr also to the CCS error log.  This feature is enabled by setting
#   the argument to "open", and disabled by setting it to "close".
#   Once active, errors logged will all have the same stack id, until this
#   logging is closed.
#
#   This command relies on a file-event.  Such events are only handled when
#   the Sequencer is idle, i.e. when it is not busy evaluating commands from
#   a script.  In other words, the output to the error log may not be visible
#   immediately, depending on the Sequencer's activity.
#  
#   ENVIRONMENT
#   When this error logging is active, the global Tcl-variables OSE, EPR and 
#   EPW are set and accessed.
#   The global Tcl-variable OSE refers to the original stderr file descriptor.
#   EPR and EPW are the reading resp writing end of a pipe created to divert 
#   the messages sent to stderr.
#
#   RETURN VALUES
#   OK, with and empty result string if no errors 
#   ERROR with corresponding message in the result string under the following 
#   conditions:
#   - "called "seq_logTclErrors" with too many arguments" if there was more
#     than 1 argument given to this command.
#   - "wrong arg: should be "open" or "close"" if argument is not correct
#   - "Tcl error logging already active" if open has been given before (i.e.
#     the global Tcl-variable OSE exists)
#   - "Tcl error logging already disabled" if close has been given before 
#     (i.e. the global Tcl-variable OSE does not exist)
#
#   CAUTIONS
#   When this error logging is active, the Tcl-variables OSE, EPR and EPW are
#   set at a global level (by lack of static variables in Tcl).  They should 
#   not be modified.
#
#   The overhead caused by sending these messages also to the CCS error system
#   is considerable (on top of adding possibly quite a few messages to the 
#   log).  Its use should therefore be restricted to the bare minimum (e.g.
#   for debugging purposes).
#   
#   Stderr must be open when Tcl error logging is started.
#
#   EXAMPLES 
#   (For clarity's sake, the example is for an interactive session with 
#   input typed by the user coming after the "seqWish>" prompt, and all
#   replies on a new line) 
#
#   ..... 
#   seqWish>seq_logTclErrors open
#   seqWish>blablabla
#   Error: invalid command name "blablabla"
#   seqWish>seq_logTclErrors close
#   seqWish>..... 
#
#   SEE ALSO 
#   "Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
#   seq_errResetStack(n), seq_errAdd(n), seq_errCloseStack(n)
# 
#------------------------------------------------------------------------ 
#
# REMARK: comments have been removed and variable names have been kept
#         short for performance reasons.
#    OSE: fileId corresponding to the original stderr file descriptor
#    EPR: fileId of the pipe that was opened for reading
#    EPW: fileId of the pipe that was opened for writing
#
# The procedure will for <mode> open check if it is not yet active.  Then
# it will keep the reference to the original stderr, and create a pipe.  The 
# writing side of this pipe is then duplicated into stderr.  This means that 
# from this point on all output sent to stderr will go to the pipe.  The error
# stack is reset and a file event on the reading side of the pipe is added.  
# When data is available from this pipe, the string will be logged to the 
# error system, and will also be sent to the original stderr.  If stderr is
# closed at the time the "seq_logTclErrors open" command is given, this 
# command will fail, and you will likely see no error messages (for obvious
# reasons....).
#
# For <mode> equal to close, the file event is removed (by closing its fileId),
# the fileIds of the pipe are closed, stderr is restored to its original value
# and the temporary stderr file descriptor is closed.  The global Tcl variables
# are unset and the error stack is closed.
#
# 

proc seq_logTclErrors {mode} {
    global OSE EPR EPW
    if {$mode == "open"} {
	if {[info exists EPR]} {
	    error {Tcl error logging already active}
	}
	set OSE [dup stderr]
	pipe EPR EPW
	dup $EPW stderr
	seq_errResetStack
	fileevent $EPR readable {
	    puts $OSE [set string [gets $EPR]]; flush $OSE
	    seq_errAdd $string "Tcl-error"
	}
    } elseif {$mode == "close"} {
	if {![info exists EPR]} {
	    error {Tcl error logging already disabled}
	}
	dup $OSE stderr
	close $EPW
	close $EPR
	close $OSE
	unset OSE EPR EPW
	seq_errCloseStack
    } else {
	error {wrong arg: should be "open" or "close"}
    }
}
