#***************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: seqMsgFlush.tcl,v 2.97 2005/09/14 09:47:28 vltsccm Exp $" 
#
# 
# who      when     what
# -------- -------- ------------------
# eallaert 03/22/95 created for cmm
#
# seqMsgFlush.tcl --
# Create a new command called seq_msgFlush

#***************************************************************************
#   NAME 
#   seq_msgFlush - flush pending replies 
#  
#   SYNOPSIS 
#   seq_msgFlush ?<option>? <filter>
#
#   DESCRIPTION
#   seq_msgFlush waits a certain time for all replies to flush them as they
#   come in, or optionally flushes replies that are already available.
#
#   <option> can be either:
#	* -nowait : flags not to wait, i.e. flush only replies which are
#		    available at the time the command is given
#       * <nnn> : an integer giving the timeout in ms to wait for replies
#       If no <option> is specified, the wait will be forever.
#
#   <filter> is a list with at least one element; it is either:
#	* the environment, and optionally, the process name for which
#	  we want to flush the replies
#	* the word "all" to indicate replies to all commands
#
#  
#   ENVIRONMENT
#   No Tcl variables are accessed.
#
#   RETURN VALUES
#   OK, with and empty result string if no errors 
#   ERROR with corresponding message in the result string under the following 
#   conditions:
#   - "called "seq_msgFlush" with too many arguments" if there were more
#     than 2 arguments given to this command.
#   - "wrong filter: should be "{<envName> ?<procName>?}"" if <filter> is wrong
#   - "wrong option: should be "-nowait" or "<nnn>"" if <option> is not correct
#   - any error message returned by seq_msgRecvReply
#
#   CAUTIONS
#   The timeout given is applied to all replies separately, i.e. the total,
#   cumulative timeout may be much larger than given by this number
#
#   EXAMPLES 
#   (For clarity's sake, the example is for an interactive session with 
#   input typed by the user coming after the "seqWish>" prompt, and all
#   replies on a new line) 
#
#   .....
#   seqWish>seq_msgList
#   {cmd5 {wte13 OS INIT}} {cmd9 {lte18 ICS RESET}} {cmd27 {wte13 DCS INIT}} 
#   seqWish>seq_msgFlush wte13
#   seqWish>seq_msgList
#   {cmd9 {lte18 ICS RESET }}
#   seqWish>..... 
#
#   SEE ALSO 
#   "Tcl and the Tk toolkit", John K. Ousterhout, ISBN 0-201-6337-X
#   seq_msgRecvReply(n)
# 
#------------------------------------------------------------------------ 
#
# REMARK: comments have been removed and variable names have been kept
#         short for performance reasons.
#         TO: timeout
#
# The procedure will first check the order of the options/arguments.  Then, if
# the filter contains no more than 2 elements, and generate an error if wrong.
# Next it checks if <option> is a number, and if so, convert it to an integer.
# If not, and if <option> is neither "-nowait" nor the default value, an error
# will be produced.
#
# Next it gets the list of commands with pending replies.  For every element
# in this list that meets the filter criteria, a seq_msgRecvReply is given with
# the proper commandId and timeout value.
# 

proc seq_msgFlush {filter {TO ""}} {
    ::internal::seq_initDbg seq_msgFlush "$filter [list $TO]"
    if {[string equal $filter -nowait] || [string is integer -strict $filter]} {
	set x $TO; set TO $filter; set filter $x
    }
    if {[set fl [llength $filter]] > 2} {
	set msg {wrong filter: should be "{<envName> ?<procName>?}"}
	::internal::seq_returnDbg seq_msgFlush ERR $msg
	error $msg
    } elseif {$fl < 1} {
	set msg {wrong # args: should be "seq_msgFlush ?-nowait|<timeout>? <filter>"}
	::internal::seq_returnDbg seq_msgFlush ERR $msg
	error $msg
    }
    set env [lindex $filter 0] 
    if {$fl == 2} {set proc [lindex $filter 1]} 
    
    if {![string is integer -strict $TO] && 
	![string equal $TO -nowait] && 
	([string length $TO] != 0)} {
	set msg {wrong option: should be "-nowait" or "<nnn>"}
	::internal::seq_returnDbg seq_msgFlush ERR $msg
	error $msg
    }
    foreach el [seq_msgList] {
	set cmdId [lindex $el 0]; set cmd [lindex $el 1]
	if {($fl == 1 && ($env == "all" || $env == [lindex $cmd 0])) ||
	($fl == 2 && $env == [lindex $cmd 0] && $proc == [lindex $cmd 1])} {
	    if {[catch {eval seq_msgRecvReply -bin -last $cmdId eNr $TO} msg]} {
		if {($msg == "message queue empty") && ($TO == "-nowait")} {
		    break
		} else {
		    ::internal::seq_returnDbg seq_msgFlush ERR $msg
		    error $msg
		}
	    }
	}
    }
    ::internal::seq_returnDbg seq_msgFlush OK {}
    return
}
