#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooBASE.tcl,v 1.47 2005/08/23 08:02:51 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  06/01/02  package require msqltcl added (SPR 20010719).
# rschmutz  30/04/01  CheckOption added.
# rschmutz  05/01/01  CCSLite support added (AccSelect added).
# rschmutz  23/11/00  MsgCall: better error reporting.
# rschmutz  07/02/00  created (based in os0SCRIPT_BASE.tcl).
# mkiekebu  06/10/99  procheck warnings removed
# rschmutz  08/07/98  issue an error if the logfile can't be opened.
# rschmutz  22/05/98  if we are in a panel: issue an error instead of exit.
# rschmutz  19/05/98  issue error, if the wrong unix user starts the script.
# rschmutz  13/05/98  log info in $VLTDATA/tmp/$infoHeader.log.
# rschmutz  12/05/98  method Sleep added.
# rschmutz  12/11/97  created, based in emoStartEmmiLib by JKN, RRO and PGL.
#

#************************************************************************
#   NAME
#	stooBASE - base for Tcl/Tk scripts
# 
#   SYNOPSIS
#	stooBASE name [-dbPoint <text>]
#		      [-infoHeader <text>]
#		      [-infoWidget <widget>]
#		      [-loglevel <level>]
# 
#   DESCRIPTION
#	stooBASE contains useful methods for scripts.
#	Usually this class is inherited by a class containing
#	the actual script functionality.
#
#   PARENT
#	stooLOG
#
#   PUBLIC METHODS
#	-
#
#   PUBLIC DATA
#	dbPoint:    point used to access OLDB data.
#
#   PROTECTED METHODS
#	IsWsEnvActive [<envName>]
#	    Returns 1 if the RTAP environment is active, 0 otherwise.
#
#	AccSelect <select>
#	    Apply an SQL SELECT instruction on the ACC database.
#	    Return the result set.
#
#	Dialog <text>
#	    Pop ups a dialog to show the text.
#
#	GetOptions optionList options
#	    Reads command line options.
#	    The accepted options are listed in optionList,
#	    e.g.: {-proc <name> -all}, with -all a boolean.
#	    The corresponding public class variables are set,
#	    e.g.: configure proc "<name>", configure all 0|1.
#
#	ExecCmd <name> <execCmd>
#	    Executes the shell command <execCmd>. 
#	    Command <execCmd> is expected to return 0. If this is
#	    not the case, then <name> is printed. ExecCmd returns
#	    the return status of command <execCmd>.
#
#	GetProcessId <name> [<grepFilter>]
#	    Returns the pid of the process <name>.
#
#	DbRead <attr>
#	DbWrite <attr> <value>
#	    Read/write an OLDB attribute. dbPoint is used
#	    as the attribute name prefix.
#
#	DbWait <attr> <value> [<secs>]
#	    Waits <secs> secs. (default: 10) for attribute
#	    <attr> to have value <value>.  Returns 1 if
#	    <attr> equal <value>, 0 otherwise.
#
#	SendCommand <env> <proc> <cmd> <pars> <timeout>
#	MsgCall <env> <proc> <cmd> <pars> ?timeout?
#	    Send a command and wait for all replies.
#	    SendCommand and MsgCall are identical, except for
#	    their return values.
#	    MsgCall returns the last reply, while SendCommand
#	    returns a boolean indicating if the the command
#	    was successful or not.
#
#	Sleep <secs>
#	    Sleep <secs> seconds. The method updates Tk
#	    during the sleep.
#
#	Logical2Num value
#	    Converts 'T'/'F' to 1/0.
#
#
#   FILES
#	$VLTDATA/tmp/$infoHeader.log  -  READ  -- log file
#
#   CAUTIONS
#
#   EXAMPLES
#	See stooINSTRUMENT.
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

package require msqltcl

#************************************************************************
#*# class: stooBASE
#------------------------------------------------------------------------

class stooBASE {
    inherit stooLOG

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
    }

    #****************************************************************
    #*# method: IsWsEnvActive
    #----------------------------------------------------------------

    protected method IsWsEnvActive { {name ""} } {
    	global env

	#*# 1. get env. name
	if { "$name" == "" } {
	    if {[catch { set name $env(RTAPENV) }]} {
	        Exit "Environment variable RTAPENV is not set."
	    }
	}

	#*# 2. get env. type
	set envtype [AccSelect "SELECT env_type FROM prog_environment WHERE progenv_name = '$name'"]
	if { "$envtype" == "" || ("$envtype" != "RTAP" && "$envtype" != "QSEMU") } {
	    Exit "Environment $name: Unknown env. type '$envtype'."
	}

	#*# 3. check if env. is running
	if { "$envtype" == "RTAP" } {
	    set lockfile ".RtapEnvLock"
	} else {
	    set lockfile [cconcat "." $name ".lock"]
	}
    	if {[file exists [file join $env(VLTDATA) ENVIRONMENTS $name $lockfile]]} {
	    set run 1
	    set msg "running"
	} else {
	    set run 0
	    set msg "not running"
	}

	Debug "Environment $name: $msg." 2
	return $run
    }

    #********************************************************************
    #*# method: AccSelect
    #--------------------------------------------------------------------
    
    protected method AccSelect { select } {
        global env

        #*# 1. connect to ACC database
        set ret ""
        set acc [msqlconnect $env(ACC_HOST)]
        msqluse $acc acc
    
        #*# 2. execute select
        set n [msqlsel $acc $select]

        #*# 3. return result set
        loop i 0 $n {
	    lappend ret [msqlnext $acc]
        }
        return $ret
    }

    #****************************************************************
    #*# method: Dialog
    #----------------------------------------------------------------

    protected method Dialog { msg } {

    	#*# 1. show dialog
    	tk_dialog .stooDialog "Dialog Box" $msg info 0 OK
    }

    #****************************************************************
    #*# method: GetOptions
    #----------------------------------------------------------------

    protected method GetOptions { optionList options } {

	#*# 1. parse optionList
	set optionNames {}
	set optionHasValue {}
	set len [llength $optionList]
	set i 0
	set elem [lindex $optionList $i]
	while { $i < $len } {
	    if { "[crange $elem 0 0]" != "-" } {
		Exit "Internal error: syntax error in the option list '$optionList'"
	    }
	    lappend optionNames $elem
	    set hasValue 0
	    incr i
	    set elem [lindex $optionList $i]
	    if { "[crange $elem 0 0]" == "<" } {
		set hasValue 1 
		incr i
		set elem [lindex $optionList $i]
	    }
	    lappend optionHasValue $hasValue
	}

	#*# 2. parse options
	set len [llength $options]
	set i 0
	set elem [lindex $options $i]
	while { $i < $len } {
	    set j [lsearch $optionNames $elem]
	    if { $j == -1 } {
		Exit "Wrong option $elem (options are: $optionList)"
	    }
	    if { [lindex $optionHasValue $j] == 1 } {
		set valueList ""
		incr i
		set value [lindex $options $i]
		while { "$value" != "" && "[crange $value 0 0]" != "-" } {
		    lappend valueList $value
		    incr i
		    set value [lindex $options $i]
		}
		$this configure $elem $valueList
		set elem $value
	    } else {
		$this configure $elem 1
		incr i
		set elem [lindex $options $i]
	    }
	}

    }

    #****************************************************************
    #*# method: ExecCmd
    #----------------------------------------------------------------

    protected method ExecCmd { name execCmd } {

    	Debug "Executing '$execCmd'."
	if {[catch { set status [system $execCmd] } msg]} {
	    Warning "ERROR: Execution problem ($name)."
	    Warning "ERROR: $msg"
	    return 1
	} 
	if { $status != 0 } {
	    Warning "ERROR: $name."
	}

	return $status
    }

    #****************************************************************
    #*# method: GetProcessId
    #----------------------------------------------------------------

    protected method GetProcessId { name { grepFilter "" }} {

    	# 1. get pid
    	set pid 0
	if { "$grepFilter" == "" } {
	    if {! [catch {exec ps -ef | grep $name | egrep -v grep} res]} {
	    	set pid [lindex $res 1]
	    }
	} else {
	    if {! [catch {exec ps -ef | grep $name | grep $grepFilter | egrep -v grep} res]} {
	    	set pid [lindex $res 1]
	    }
    	}

	return $pid
    }

    #****************************************************************
    #*# method: DbRead
    #----------------------------------------------------------------

    protected method DbRead { attr } {

    	#*# 1. read the attribute using dbPoint as prefix
    	set name [cconcat $dbPoint $attr]
	return [seq_dbReadSymbolic $name]
    }
   
    #****************************************************************
    #*# method: DbWrite
    #----------------------------------------------------------------

    protected method DbWrite { attr value } {

    	#*# 1. write the attribute using dbPoint as prefix
    	set name [cconcat $dbPoint $attr]
	seq_dbWriteSymbolic $name $value
    }

    #****************************************************************
    #*# method: DbWait
    #----------------------------------------------------------------

    protected method DbWait { attr value {secs 10} } {

    	#*# 1. wait for attr to have value
	set txt [DbRead $attr]
	while { "$txt" != "$value" && $secs >= 0 } {
	    Sleep 1	
	    set txt [DbRead $attr]
	    incr secs -1
	}

	#*# 2. return 1 if ok, 0 otherwise.
	if { "$txt" != "$value" } {
	    return 0
	}
	return 1
    }

    #****************************************************************
    #*# method: SendCommand
    #----------------------------------------------------------------

    protected method SendCommand { env proc cmd pars {timeout 0} } {

    	#*# 1. message call
	set reply [MsgCall $env $proc $cmd $pars $timeout]

	#*# 2. return ok/not ok (why ?)
	set ok 1
	if { [string first FAILURE $reply] != -1 } {
	    Warning "ERROR: Command reply: $reply"
	    set ok 0
	}
	return $ok
    }
	    
    #****************************************************************
    #*# method: MsgCall
    #----------------------------------------------------------------

    protected method MsgCall { env proc cmd pars {timeout 0} } {

	set errVar 0
	set reply ""
	if { $timeout == 0 } {
	    set timeout $defaultTimeout
	}
	Debug "Sending command: $env/$proc $cmd $pars" 3
	if {[catch {

	    #*# 1. send command
	    set cmdId [eval {seq_msgSendCommand $env $proc $cmd} $pars]

	    #*# 2. wait for replies
	    set lastReply 0
	    while { $lastReply == 0 } {
		set reply [seq_msgRecvReply $cmdId errVar lastReply -$timeout]
		if { $lastReply == 0 } {
		    Debug "Command reply: $reply \[intermediate reply\]" 3
		}
	    }
	} result]} {
	    set reply "FAILURE: $result"
	}
	if { $errVar } {
	    set reply "FAILURE: $reply"
	}
	Debug "Command reply: $reply" 3
	
	return $reply
    }

    #****************************************************************
    #*# method: Sleep
    #----------------------------------------------------------------

    public method Sleep { time } {

	for { set i [expr {2 * $time}] } { $i >= 0 } { incr i -1 } {
	    after 500
	    update
	    update idletasks
	}
    }

    #****************************************************************
    #*# method: Logical2Num
    #----------------------------------------------------------------

    protected method Logical2Num { val } {

    	if { "$val" == "F" } {
	    return 0
	} elseif { "$val" == "T" } {
	    return 1
	}
	error "'$val' is not a boolean value."
    }

    #****************************************************************
    #*# method: CheckOption
    #----------------------------------------------------------------

    protected method CheckOption { key val optList } {

        if { ![lcontain $optList $val] } {
	    error "Wrong value $key = $val.  Expected one of: $optList."
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable dbPoint ""
    public common defaultTimeout 10000
}


#************************************************************************
#___oOo___

