#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooDCS_FIERA.tcl,v 1.47 2005/08/23 08:02:53 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dpopovic  22/11/04  FIERA image directory can be set with OCS.DET$i.IMGDIR.
#                     imgDir added to stooFcdStart and stooRtdStart parameters. 
# rschmutz  08/05/03  StopLoop added.
# rschmutz  21/07/02  RTD name simplified.
# rschmutz  30/04/01  CheckOption added.
# rschmutz  02/02/01  HW_SIM fixed.
# rschmutz  02/02/01  OCS.DETi.DBFILE deleted.
# rschmutz  09/12/00  ignore if DISPLAY env. var. is missing.
# rschmutz  10/10/00  local flag support dropped.
# rschmutz  25/05/00  OCS.DETi.DEFAULT, OCS.DETi.STOP support added.
# rschmutz  07/02/00  created.
#

#************************************************************************
#   NAME
#	stooDCS_FIERA - start/stop detector software
# 
#   SYNOPSIS
#	stooDCS_FIERA dcs
# 
#   DESCRIPTION
#	stooDCS_FIERA allows to start/stop standard detector software.
#
#   PARENT
#	stooAPPLICATION
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure the application.
#	Start
#	    Start the application.
#	Stop
#	    Stop the application.
#	IsActive
#	    Returns 1 if the application is running, 0 otherwise.
#	IsStopped
#	    Returns 1 if no application process is running, 0 otherwise.
#	Standby
#	    Sets the application in state STANDBY.
#	StopLoop
#	    Stop exposure loop.
#
#   FILES
#	xxmcfgINS.cfg - instrument configuration file.
#	    The configuration file is searched in
#	    the following directories:
#	      $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#	      $VLTDATA/config
#	      $VLTROOT/config
#	      $INTROOT/config
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooDCS_FIERA
#------------------------------------------------------------------------

class stooDCS_FIERA {
    inherit stooAPPLICATION

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
        configure -objType DCS
    	eval configure $args
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
        global stooInsMulti env

	#*# 1. name, desc, type, CCD name
	set txt [$cfg Get OCS.DET$i.NAME]
	if { $stooInsMulti } {
	    set rtdName "[$cfg Get INS.CON.ID]_"
	    configure -name [cconcat $rtdName $txt]
	} else {
	    set rtdName ""
	    configure -name $txt
	}
	if { $txt != "DCS" } {
	    append rtdName $txt "_RTD"
	    configure -desc "[$cfg Get INS.CON.ID] DCS $txt"
	} else {
	    append rtdName "RTD"
	    configure -desc "[$cfg Get INS.CON.ID] DCS"
	}
	configure -type [$cfg Get OCS.DET$i.TYPE]
	CheckOption OCS.DET$i.TYPE $type {FIERA}

	#*# 2. process name
	set txt ""
	configure -ccdName [$cfg Get OCS.DET$i.CCDNAME]
	set txt [cconcat "fcdconCI_" $ccdName]
	catch { set txt [$cfg Get OCS.DET$i.PROCNAME] }
	if { "$txt" == "" } {
	    error "Detector $name process name not known."
	}
	configure -procName $txt

	#*# 3. environment name
	set txt [$cfg Get INS.CON.WSENV]
	catch { set txt [$cfg Get OCS.DET$i.ENVNAME] }
	configure -envName $txt

	#*# 4. CCDLENV
	configure -ccdLenv [$cfg Get OCS.DET$i.CCDLENV]

	#*# 4a. VLTSW20040320 set image directory 
	if { $env(INS_USER) != "" } {
	    set txt "$env(INS_ROOT)/$env(INS_USER)/DETDATA"
	} else {
	    set txt "$env(INS_ROOT)/SYSTEM/DETDATA"
	}
	catch { set txt [$cfg Get OCS.DET$i.IMGDIR] }
	configure -imgDir $txt

	#*# 5. access
	set txt "NORMAL"
	catch { set txt [$cfg Get OCS.DET$i.ACCESS] }
	CheckOption OCS.DET$i.ACCESS $txt {NORMAL FORBIDDEN IGNORE} 
	configure -access $txt

	#*# 6. simulation
	set txt "NONE"
	catch { set txt [$cfg Get OCS.DET$i.SWSIM] }
	CheckOption OCS.DET$i.SWSIM $txt {NONE LCU_SIM HW_SIM} 
	configure -simulation $txt
    	if { "$simulation" == "LCU_SIM" } {
    	    configure -wsSimulation 1
	} else {
    	    configure -wsSimulation 0
	}

	#*# 7. stop
	set txt "T"
	catch { set txt [$cfg Get OCS.DET$i.STOP] }
	configure -stop [Logical2Num $txt]

	#*# 8. start application
	set txt "T"
	catch { set txt [$cfg Get OCS.DET$i.DEFAULT] }
	configure -default [Logical2Num $txt]

	#*# 9. start UIF
	set txt "F"
	catch { set txt [$cfg Get OCS.DET$i.STRTUIF] }
	set startUif [Logical2Num $txt]

	#*# 10. start RTD
	set txt "F"
	catch { set txt [$cfg Get OCS.DET$i.STRTRTD] }
	set startRtd [Logical2Num $txt]

	#*# 11. add main process
	#      (only used by method IsActive)
	set obj [stooPROCESS $this.$procName \
		 -name    $procName \
		 -envName $envName \
		 -desc    "[Desc] process $procName"]
	AddObj $obj

	#*# 12. add process 2
	#      (only used by method IsActive)
	set procname2 [cconcat "fcditWs_" $ccdName]
	set obj [stooPROCESS $this.$procname2 \
		 -name    $procname2 \
		 -envName $envName \
		 -desc    "[Desc] process $procname2"]
	AddObj $obj

	#*# 13. add stand-alone panel
	set startScript stooFcdStart
	set execName fcdCtrl
	set lcuenv $ccdLenv
	if { $wsSimulation } {
	    set lcuenv "0"
	}
	set redir ""
	if { $loglevel < 2 } {
	    set redir "> /dev/null"
	}
	set obj [stooPANEL $this.standalone \
		 -name $name \
	    	 -execCmd "$startScript $ccdName $envName $lcuenv $env(INS_ROOT) 0 $imgDir $redir" \
	    	 -execName $execName -grepFilter $ccdName \
	    	 -procName [cconcat "ccdCtrl_" $ccdName] \
	    	 -envName $envName \
	    	 -desc "[Desc] Stand-alone Panel" \
		 -default $startUif]
	AddObj $obj

	#*# 14. add RTD panel
	set display ""
	catch { set display "-display $env(DISPLAY)" }
	set obj [stooPANEL $this.rtd \
		 -name $rtdName \
	    	 -execCmd "stooRtdStart $imgDir -camera $ccdName $display -attach 1" \
	    	 -execName "-noKill" \
	    	 -procName [cconcat "rtd_" $ccdName] \
	    	 -envName $envName \
		 -desc "[Desc] RTD Panel" \
		 -default $startRtd] 
	AddObj $obj
    }

    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start {} {
        global env

        #*# 1. if DCS is active: do nothing
	if {[IsActive]} {
	    Info "INFO : [Desc] already running."
	    return
	}
	if { "$access" != "NORMAL" } {
	    Info "INFO : [Desc] not started.  Access = $access."
	    return
	}
	if { "$wsSimulation" } {
	    Info "START: Starting [Desc] (WS SIMULATION) . . ."
	} else {
	    Info "START: Starting [Desc] . . ."
	}

	#*# 2. get DCS data and build start command
	set startScript fcdDcsStart.sh
	set saveScript fcdDcsDbSave.sh
	set proc1 fcdconCI_$ccdName
	set proc2 fcditWs_$ccdName
	#@@@
	# if <alias>$ccdName.opMode != 3 then check also:
	# fcdcon_$ccdName@wtaccd.
	set opModeAttr "<alias>$ccdName.opMode"
	if { $wsSimulation } {
	    set lcuenv "0"
	    set opModeValue 3
	} else {
	    set lcuenv $ccdLenv
	    if { $simulation == "HW_SIM" } {
	    	set opModeValue 4
	    } else {
	    	set opModeValue 2
	    }
	}
	set redir ""
	if { $loglevel < 3 } {
	    set redir "> /dev/null"
	}
	set saveCmd "$saveScript $ccdName $envName $redir"
	set execCmd "$startScript $ccdName $envName $lcuenv $redir"

	#*# 3. start DCS
	#
	#      Set FIERA env. var FCD_IMGDIR to $imgDir
	#      before starting FIERA software.
	#
	Debug "[Desc]: image directory set to $imgDir" 2
	set env(FCD_IMGDIR) $imgDir

	Debug "[Desc]: save command='$saveCmd'" 2
	seq_dbWriteSymbolic $opModeAttr $opModeValue
	if {[ExecCmd [Desc] $saveCmd]} {
	    Exit
	}
	Debug "[Desc]: start command='$execCmd'" 2
	if {[ExecCmd [Desc] $execCmd]} {
	    Exit
	}

	#*# 4. check startup
	set objList [Contents stooPROCESS]
	set obj [lvarpop objList]
	if { ![$obj IsActive 90000] } {
	    Exit "Problems starting the [Desc]. Process [$obj Name] not started."
	}
	foreach obj $objList {
	    if { ![$obj IsActive 10000] } {
	        Exit "Problems starting the [Desc]. Process [$obj Name] not started."
	    }
	}
	#if { [seq_dbReadSymbolic "<alias>$ccdName.opState"] != 2 } {
	#    Exit "Problem starting the [Desc]. The state is not LOADED."
	#}

	Info "       [Desc] started."
    }

    #****************************************************************
    #*# method: Stop
    #----------------------------------------------------------------

    public method Stop {} {

    	#*# 1. info
	if { [IsStopped] } {
	    Info "STOP : [Desc] seems to be stopped.  Stopping anyway."
	} else {
	    Info "STOP : Stopping [Desc] . . ."
	}

	#*# 2. get DCS data and build stop command
	set stopScript fcdDcsStop.sh
	set lcuenv $ccdLenv
	if { $wsSimulation } {
	    set lcuenv "0"
	}
	set redir ""
	if { $loglevel < 3 } {
	    set redir "> /dev/null"
	}
	set execCmd "$stopScript $ccdName $envName $lcuenv $redir"

	#*# 3. stop DCS
	Debug "[Desc]: stop command='$execCmd'" 2
	if {[ExecCmd [Desc] $execCmd]} {
	    Warning "ERROR: Error stopping [Desc]."
	}
	Info "       [Desc] stopped."
    }

    #****************************************************************
    #*# method: StopLoop
    #----------------------------------------------------------------

    public method StopLoop {} {

    	#*# 1. if not active, not ONLINE, or not looping: return
	if { ! [IsActive] } {
	    return
	}
	if { [DbRead "<alias>$ccdName.opState"] != 4 } {
	    return
	}
	if { ([DbRead "<alias>$ccdName:exposures:exposure_1.expStatus"] & 2048) != 2048 } {
	    return
	}

    	#*# 3. stop looping
	Info "STOP : Stopping [Desc] loops . . ."
    	SendCommand $envName $procName STOP ""
    	SendCommand $envName $procName WAIT "0,Global" 120000
	Debug "[Desc] loops stopped."
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    # common:
    public variable type
    public variable procName
    public variable envName

    # FIERA ONLY:
    public variable imgDir 

    # ACE, FIERA:
    public variable ccdName
    public variable ccdLenv

    # IRACE:
    public variable tcomDev
    public variable tcomSrv
    public variable sdmaDev
    public variable sdmaCmd
    public variable sdmaData
    public variable rtdPort
    public variable rtdCamera
    public variable module
    public variable sysCfg
    public variable detCfg
    public variable options

    public variable simulation {
    	if { "$simulation" == "LCU_SIM" } {
    	    configure -wsSimulation 1
	} else {
    	    configure -wsSimulation 0
	}
    }
    public variable startUif
    public variable startRtd
}


#___oOo___
