#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooENV.tcl,v 1.47 2005/08/23 08:02:49 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  06/01/02  package require msqltcl added (SPR 20010719).
# rschmutz  05/01/01  CCSLite support added (AccSelect added).
# rschmutz  07/02/00  created (based in os0SCRIPT_BASE.tcl).
#

#************************************************************************
#   NAME
#	stooENV - CCS environment
# 
#   SYNOPSIS
#	stooENV name <options>
# 
#   DESCRIPTION
#	stooENV allows to start/stop a CCS environment.
#
#   PARENT
#	ctooOBJ_BASE stooLOG
#
#   PUBLIC METHODS
#	Start
#	    Start a WS environment
#	Stop
#	    Stop a WS environment
#	IsActive
#	    Check if a WS environment is active
#
#   PUBLIC DATA
#	name:	CCS environment name (default: $RTAPENV).
#
#   FILES
#	$VLTDATA/ENVIRONMENTS/<envName>  -- CCS env. directory
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#

package require msqltcl

#************************************************************************
#*# class: stooENV
#------------------------------------------------------------------------

class stooENV {
    inherit ctooOBJ_BASE stooLOG

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
    }

    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start {} {
    	global env

	#*# 1. get env. name
	if { "$name" == "" } {
	    if {[catch { set name $env(RTAPENV) }]} {
	        Exit "ERROR: Environment variable RTAPENV is not set."
	    }
	}
	Debug "Environment $name: starting..."

	#*# 2. check that the environment directory exists
	if {[catch { set vltdata $env(VLTDATA) }]} {
	    Exit "ERROR: Environment variable VLTDATA is not set."
	}
        set fileName "[set vltdata]/ENVIRONMENTS/$name"
	if { [file isdirectory $fileName] == 0 } {
	    Exit "ERROR: Environment directory \$VLTDATA/ENVIRONMENTS/$name is missing."
	}
	Debug "Environment directory \$VLTDATA/ENVIRONMENTS/$name exists." 2

	#*# 3. if the WS environment is not running: start it
	if { ![IsActive] } {
	    Info "START: Environment $name: starting..."
	    exec vccEnvStart -q -e $name
	}

	#*# 4. verify that the environment has started
	set i 0
	while { [IsActive] == 0 && $i < 10 } {
	    incr i
	    Debug "Waiting for the environment to start..." 2
	    sleep 1
	}
	if { $i == 10 } {
	    Exit "ERROR: Trouble starting environment $name."
	}
	Info "INFO : Environment $name: running."

	#*# 5. ccsInit the script
	catch { seq_ccsInit }
	sleep 4
	seq_waitTillActive $name msgServer 10000 $defaultPeriod
	seq_waitTillActive $name cmdManager 10000 $defaultPeriod
	seq_waitTillActive $name logManager 10000 $defaultPeriod
    }

    #****************************************************************
    #*# method: Stop
    #----------------------------------------------------------------

    public method Stop {} {
    	global env

	#*# 1. get env. name
	if { "$name" == "" } {
	    if {[catch { set name $env(RTAPENV) }]} {
	        Exit "Environment variable RTAPENV is not set."
	    }
	}
	Info "STOP : Environment $name: stopping..."

	#*# 2. check that the environment directory exists
	if {[catch { set vltdata $env(VLTDATA) }]} {
	    Exit "Environment variable VLTDATA is not set."
	}
        set fileName "[set vltdata]/ENVIRONMENTS/$name"
	if { [file isdirectory $fileName] == 0 } {
	    Exit "Environment directory \$VLTDATA/ENVIRONMENTS/$name is missing."
	}
	Debug "Environment directory \$VLTDATA/ENVIRONMENTS/$name exists." 2

	#*# 3. delete all PID files
	set execCmd "rm -f $env(VLTDATA)/tmp/[set name].\*.pid"
	catch { system $execCmd }

	#*# 4. if the WS environment is running: stop it
	if { [IsActive] } {
	    Info "       Environment $name: stopped (in 10 secs.)."
	    seq_ccsExit
	    exec vccEnvStop -q -e $name
	}

	#*# 5. verify that the environment has stopped
	set i 0
	while { [IsActive] == 1 && $i < 10 } {
	    incr i
	    Debug "Waiting for the environment to stop..."
	    sleep 1
	}
	if { $i == 10 } {
	    Exit "Trouble stopping environment $name."
	}
	Info "       Environment $name: stopped."
    }

    #****************************************************************
    #*# method: IsActive
    #----------------------------------------------------------------

    public method IsActive {} {
    	global env

	#*# 1. get env. name
	if { "$name" == "" } {
	    if {[catch { set name $env(RTAPENV) }]} {
	        Exit "Environment variable RTAPENV is not set."
	    }
	}

	#*# 2. get env. type
	if { $envtype == "" } {
	    set envtype [AccSelect "SELECT env_type FROM prog_environment WHERE progenv_name = '$name'"]
	    if { "$envtype" == "" || ("$envtype" != "RTAP" && "$envtype" != "QSEMU") } {
	        Exit "Environment $name: Unknown env. type '$envtype'."
	    }
	}

	#*# 3. check if env. is running
	if { "$envtype" == "RTAP" } {
	    set lockfile ".RtapEnvLock"
	} else {
	    set lockfile [cconcat "." $name ".lock"]
	}
    	if {[file exists [file join $env(VLTDATA) ENVIRONMENTS $name $lockfile]]} {
	    set run 1
	    set msg "running"
	} else {
	    set run 0
	    set msg "not running"
	}

	Debug "Environment $name: $msg." 2
	return $run
    }

    #********************************************************************
    #*# method: AccSelect
    #--------------------------------------------------------------------
    
    protected method AccSelect { select } {
        global env

        #*# 1. connect to ACC database
        set ret ""
        set acc [msqlconnect $env(ACC_HOST)]
        msqluse $acc acc
    
        #*# 2. execute select
        set n [msqlsel $acc $select]

        #*# 3. return result set
        loop i 0 $n {
	    lappend ret [msqlnext $acc]
        }
        return $ret
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public common defaultPeriod 100
    protected variable envtype  ""
}


#************************************************************************
#___oOo___

