#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooICS.tcl,v 1.47 2005/08/23 08:02:49 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# dpopovic  05/11/2004  method Start: Start ICS only if access=NORMAL
# dpopovic  29/07/04    ICS simulation accepts NONE and NORMAL
# rschmutz  2004-05-08  sub-instruments: instrument name prefix fixed
# alongino  11/08/03    VLTSW20030303: support multiple panel for the same ICS 
# alongino  01/08/03    VLTSW20030334: support multiple ICSs in the same instrument
# rschmutz  10/05/01    if HW_SIM: send SIMULAT to ICS.
# rschmutz  30/04/01    CheckOption added.
# rschmutz  11/12/00    simulation flag: accept either NONE or NORMAL.
# rschmutz  09/12/00    pass insid to ic0* processes.
# rschmutz  07/06/00    use icbControl, icbSimControl if xxi* processes are not found.
# rschmutz  25/05/00    OCS.INSi.DEFAULT support added.
# rschmutz  07/02/00    created.
#

#************************************************************************
#   NAME
#	stooICS - start/stop ICS
# 
#   SYNOPSIS
#	stooICS ics
# 
#   DESCRIPTION
#	stooICS allows to start/stop a standard ICS.
#
#   PARENT
#	stooAPPLICATION
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure the application.
#	Start
#	    Start the application.
#	Stop
#	    Stop the application.
#	IsActive
#	    Returns 1 if the application is running, 0 otherwise.
#	IsStopped
#	    Returns 1 if no application process is running, 0 otherwise.
#	Standby
#	    Sets the application in state STANDBY.
#
#   FILES
#	xxmcfgINS.cfg - instrument configuration file.
#	    The configuration file is searched in
#	    the following directories:
#	      $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#	      $VLTDATA/config
#	      $VLTROOT/config
#	      $INTROOT/config
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooICS
#------------------------------------------------------------------------

class stooICS {
    inherit stooAPPLICATION

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	configure -objType ICS
    	eval configure $args
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
        global stooInsMulti

	#*# 1. icsName, name, desc, config. set
	#
	# Support for multiple ICSs:
	#  ICS receives the name specified either by:
	#  OCS.INS$i.NAME, "ICS" (first ICS only), INS$i.CON.ID or "ICS$i".
	#
        set cfgcat "INS"
        if { $i > 1 } { set cfgcat "INS$i" }
	if {[catch { set txt [$cfg Get OCS.INS$i.NAME] }]} {
            set txt "ICS"
            if { $cfgcat != "INS" } {
                set txt "ICS$i"
	        catch { set txt [$cfg Get $cfgcat.CON.ID] }
            }
	}
	configure -icsName $txt

	# if multiple instruments are used: prefix the instrument name
	set insid [$cfg Get INS.CON.ID]
	if { $stooInsMulti } {
	    configure -name [cconcat $insid "_" $icsName]
	} else {
	    configure -name $icsName
	}

	configure -desc "$insid $icsName"

        # In order to be able to start the whole instrument with one
        # call to stooStart (e.g. stooStart XXXX), there must be a way 
        # to tell each ICS process to take the config. set associated to that ICS. 
        # In order to achieve this, the name of the configuration set 
        # passed to the ICB processes used for secondary ICSs must correspond 
        # to the value of OCS.INSi.NAME
        set cfgset [$cfg Name]
        if { $i > 1 } { set cfgset $icsName }

	#*# 2. process name
	set prefix [$cfg Get $cfgcat.CON.PREFIX]
	set txt [cconcat $prefix "iControl"]
	catch { set txt [$cfg Get OCS.INS$i.PROCNAME] }
	configure -procName $txt

	#*# 3. environment name
	set txt [$cfg Get INS.CON.WSENV]
	catch { set txt [$cfg Get OCS.INS$i.ENVNAME] }
	configure -envName $txt

	#*# 4. dictionary
	set txt [cconcat [$cfg Get $cfgcat.CON.ID] "_ICS"]
	catch { set txt [$cfg Get OCS.INS$i.DICT] }
	configure -dictionary $txt

	#*# 5. alias
	catch { set txt [$cfg Get OCS.INS$i.ALIAS] }
	configure -alias $txt

	#*# 6. local flag
	set txt "T"
	catch { set txt [$cfg Get OCS.INS$i.LOCAL] }
	configure -local [Logical2Num $txt]

	#*# 7. access
	set txt "NORMAL"
	catch { set txt [$cfg Get OCS.INS$i.ACCESS] }
	CheckOption OCS.INS$i.ACCESS $txt {NORMAL FORBIDDEN IGNORE} 
	configure -access $txt

	#*# 8. simulation
	set txt "NORMAL"
	if {![catch { set txt [$cfg Get $cfgcat.CON.OPMODE] }]} {
            CheckOption $cfgcat.CON.OPMODE $txt {NONE NORMAL LCU_SIM HW_SIM} 
	} else {
	    set txt "NONE"
	    catch { set txt [$cfg Get OCS.INS$i.SWSIM] }
	    CheckOption OCS.INS$i.SWSIM $txt {NONE NORMAL LCU_SIM HW_SIM}
	}
	if { $txt == "NORMAL" } {
	   set txt "NONE"
	}
	configure -simulation $txt
    	if { "$simulation" == "LCU_SIM" } {
    	    configure -wsSimulation 1
	} else {
    	    configure -wsSimulation 0
	}

	#*# 9. start application
	set txt "T"
	catch { set txt [$cfg Get OCS.INS$i.DEFAULT] }
	configure -default [Logical2Num $txt]

	#*# 10. start UIF
	set txt "F"
	catch { set txt [$cfg Get OCS.INS$i.STRTUIF] }
	set startUif [Logical2Num $txt]
	
	#*# 11. add main ICS process
	set execCmd $procName
	if {[catch { seq_findFile "bin/$execCmd" }]} {
	    set execCmd "icbControl $cfgset"
	}
	set obj [stooPROCESS $this.$procName \
		 -name    $procName \
		 -envName $envName  \
		 -execCmd $execCmd  \
		 -desc    "[Desc] control server"]
	AddObj $obj

	#*# 12. add ICS simulation process
	set procName2 [cconcat $prefix "iSimControl"]
	set execCmd2 $procName2
	if {[catch { seq_findFile "bin/$execCmd2" }]} {
	    set execCmd2 "icbSimControl $cfgset"
	}
	set obj [stooPROCESS $this.$procName2 \
		 -name    $procName2 \
		 -envName $envName   \
		 -execCmd $execCmd2  \
		 -desc    "[Desc] simulation server"]
	AddObj $obj

	#*# 13. add stand-alone panel
	set panName [cconcat $prefix "ipanControl"]
	catch { set panName [$cfg Get $cfgcat.UIF.NAME] }
	set obj [stooPANEL $this.standalone \
		 -name    $name \
		 -execCmd $panName \
		 -envName $envName \
		 -desc    "[Desc] Stand-alone Panel" \
		 -default $startUif]
	AddObj $obj
    }

    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start {} {

        #*# 0. Is this ICS available
	if { "$access" != "NORMAL" } {
	    Info "INFO : [Desc] not started.  Access = $access."
	    return
	}

        #*# 1. load CDTs
	if { ![IsActive] } {
	    Debug "Loading [Desc] CDTs . . ."
	    catch { exec cmdSetup -parse ic0lcuServer }
	    catch { exec cmdSetup -parse ic0devServer }
	    Debug "[Desc] CDTs loaded."
	}

	#*# 2. normal start
	stooAPPLICATION::Start

	#*# 3. if HW_SIM: set all ICS devices in simulation
	if { $simulation == "HW_SIM" } {
	    Info "       Setting all [Desc] devices in simulation."
	    set obj [LookupName $procName]
	    if { [$obj SendCommand SIMULAT ""] == 0 } {
		error "ERROR: Problems sending SIMULAT to [$obj Name]."
	    }
	}

    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable icsName
    public variable procName
    public variable envName
    public variable dictionary
    public variable alias
    public variable local
    public variable simulation {
    	if { "$simulation" == "LCU_SIM" } {
    	    configure -wsSimulation 1
	} else {
    	    configure -wsSimulation 0
	}
    }
    public variable startUif
    public variable startWithPanel 1
}


#___oOo___

