#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooINSTRUMENT.tcl,v 1.47 2005/08/23 08:02:49 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# dpopovic  17/08/2005  stooType(ICS) is common to all default stooType(ICSi).
# dpopovic  08/08/2005  method Config: VLTSW20050111 - multiple and different ICS's.
# dpopovic  01/06/2005  method Config: OS configured if OCS.CON.AVAIL==T.
#                       method Standby: STANDBY cmd sent only if OS exists.
# dpopovic  05/11/2004  method Config: load only local sub-instruments 
#                       with access=NORMAL
# rschmutz  2004-05-08  do not load duplicate TCS instances.
# rschmutz  08/05/03    before STANDBY: stop looping CCDs.
# rschmutz  08/11/02    inherit from stooAPPL_SET.
# rschmutz  27/05/02    config. sub-instruments.
# rschmutz  23/05/02    START.[ALARM|BOB|LOGMON].AVAIL support added.
# rschmutz  06/12/00    create DCS according to type.
# rschmutz  10/10/00    OCS.TEL.TCSID replaced with OCS.TEL.NAME.
# rschmutz  25/05/00    START.MIDAS.AVAIL support added.
# rschmutz  19/02/00    created.
#

#************************************************************************
#   NAME
#	stooINSTRUMENT - start/stop instrumentation software
# 
#   SYNOPSIS
#	stooINSTRUMENT ins
# 
#   DESCRIPTION
#	stooINSTRUMENT allows to start/stop instrumentation software.
#
#   PARENT CLASS
#	stooCONTAINER
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure the application.
#
#	Standby
#	    Set OS in state STANDBY.
#
#   SEE ALSO
#	stooAPPLICATION
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooINSTRUMENT
#------------------------------------------------------------------------

class stooINSTRUMENT {
    inherit stooAPPL_SET

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	configure -objType INSTRUMENT
    	eval configure $args
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg } {
        global env stooType stooInsMulti

	#*# 1. name, desc
	configure -name [$cfg Get INS.CON.ID]
	configure -desc "$name Instrumentation Software"
	Debug "Instrument $name:" 5

	#*# 2. check environment variables
	if {[catch { set env(RTAPENV) }]} {
	    error "Env. var. RTAPENV is not set."
	}
	set wsenv [$cfg Get INS.CON.WSENV]
	if { "$env(RTAPENV)" != "$wsenv" } {
	    error "The env. var. RTAPENV is not set correctly to '$wsenv'."
	}
	if {[catch { set env(VLTROOT) }]} {
	    error "Env. var. VLTROOT is not set."
	}
	if {[catch { set env(VLTDATA) }]} {
	    error "Env. var. VLTDATA is not set."
	}
	if {[catch { set env(INS_ROOT) }]} {
	    error "Env. var. INS_ROOT is not set."
	}
	if {[catch { set env(INS_USER) }]} {
	    error "Env. var. INS_USER is not set."
	}

	#*# 3. load BOB
	#      (default type: stooBOB)
	set bobAvail "T"
	catch { set bobAvail [$cfg Get START.BOB.AVAIL] }
	if { $bobAvail == "T" } {
	    set bob [$stooType(BOB) $this.bob]
	    AddObj $bob
	    $bob Config $cfg
	}

	#*# 4. load the OS configuration if exists
	#      (default type: stooOS)
	set osAvail "T"
	catch { set osAvail [$cfg Get OCS.CON.AVAIL] }
	if { $osAvail == "T" } {
	    set os [$stooType(OS) $this.os]
	    AddObj $os
	    $os Config $cfg
	} else {
	    Info "INFO : OS does not exist (OCS.CON.AVAIL == F)."
	    configure -osExists 0
	}

	#*# 5. load MIDAS
	#      (default type: stooMIDAS)
	set midasAvail "F"
	catch { set midasAvail [$cfg Get START.MIDAS.AVAIL] }
	if { $midasAvail == "T"} {
	    set midas [$stooType(MIDAS) $this.midas]
	    AddObj $midas
	    $midas Config $cfg
	}

	#*# 6. load the TCS configuration (if available)
	#      (default type: stooINS_TCS)
	set c1 [catch { set tcsName [$cfg Get OCS.TEL.NAME] }]
	if { $c1 == 0 } {
	    # add TCS if not already present
	    if { [[Repository] LookupName $tcsName TCS 10] == "" } {
	    	set tcs [$stooType(TCS) $this.tcs]
	    	AddObj $tcs
	    	$tcs Config $cfg
	    }
	}

	#*# 7. load the ICS configurations
	#      (default type: stooICS)
	set icsNum [$cfg Get OCS.INS.NUM]
	incr icsNum
	loop i 1 $icsNum {
	    # VLTSW20050111 - multiple and different ICS's

	    # stooType(ICS) should be the default for all ICS's with default behaviour.
	    # By default all ICS's, including stooType(ICS), are of stooICS class type. 
	    # For back compatibility reasons if stooType(ICS) is overloaded, that 
	    # means that all ICS's of default type are also set to the new type.
	    # IMPORTANT: If stooType(ICS) is overloaded and we want e.g. the third
	    #            ICS (ICS3) to be of default type (stooICS), the
	    #            type for this ICS MUST be explicitely specified, i.e.:
	    #
	    #               set stooType(ICS3) stooICS
	    #
	    set icsType $stooType(ICS)
	    catch { set icsType $stooType(ICS$i) }
	    
	    set ics [$icsType $this.ics$i]

	    AddObj $ics
	    $ics Config $cfg $i
	}

	#*# 8. load the DCS configurations
	#      (default type: stooDCS_<type>)
	set dcsNum [$cfg Get OCS.DET.NUM]
	incr dcsNum
	loop i 1 $dcsNum {
	    set dcsType [$cfg Get OCS.DET$i.TYPE]
	    if {[catch { set dcsClass $stooType(DCS_$dcsType) }]} {
	    	error "Unknown DCS type OCS.DET$i.TYPE = '$dcsType'."
	    }
	    set dcs [$dcsClass $this.dcs$i]
	    AddObj $dcs
	    $dcs Config $cfg $i
	}

	#*# 9. load the sub-instrument configurations
	#      The configuration is loaded only if OCS.OS$i.ACCESS==NORMAL
	#
	set insNum [$cfg Get OCS.OS.NUM]
	if { $insNum != 0 } {
	    set insStartup [stooSTARTUP $this.insStartup]
	}
	set saveInsMulti $stooInsMulti
	set stooInsMulti 1
	incr insNum
	loop i 1 $insNum {
	    set insName [$cfg Get OCS.OS$i.NAME]
	    set insEnv  $wsenv
	    catch { set insEnv [$cfg Get OCS.OS$i.ENVNAME] }
	    set access "NORMAL"
	    catch { set access [$cfg Get OCS.OS$i.ACCESS] }
	    # load only local sub-instruments with access=NORMAL
	    if { $insEnv == $wsenv } {
		if { $access == "NORMAL"} {
		    $insStartup SubConfig $this $insName
		} else {
		    Info "INFO : No access to Instrument $insName.  Access = $access."
		}
	    }
	}
	set stooInsMulti $saveInsMulti

	#*# 10. load the alarm display
	#       (default type: stooALARM)
	set alarmAvail "T"
	catch { set alarmAvail [$cfg Get START.ALARM.AVAIL] }
	if { $alarmAvail == "T"} {
	    set alarm [$stooType(ALARM) $this.alarm -envName $wsenv]
	    AddObj $alarm
	    $alarm Config $cfg
	}

	#*# 11. load additional panels
	stooSIMPLE::Config $cfg

	#*# 12. load logMonitor
	set logmonAvail "T"
	catch { set logmonAvail [$cfg Get START.LOGMON.AVAIL] }
	if { $logmonAvail == "T" && $stooInsMulti == 0 } {
	    set txt "F"
	    catch { set txt [$cfg Get START.LOGMON.DEFAULT] }
	    set startLogMonitor [Logical2Num $txt]
	    set obj [stooPANEL $this.logMonitor \
		     -name LOG \
		     -execCmd logMonitor \
		     -envName $wsenv \
		     -desc "Log Monitor" \
		     -default $startLogMonitor]
	    AddObj $obj
	}
    }

    #****************************************************************
    #*# method: Standby
    #----------------------------------------------------------------

    public method Standby {} {

    	#*# 1. first stop any looping CCDs
	CcdStopLoop

    	#*# 2. place OS in state Standby (if exists)
	if { !$osExists } {
	    Info "INFO : OS does not exist. Command STANDBY not sent."
	    return
	}

    	set obj [Contents OS]
	if { "$obj" == "" } {
	    error "No OS found to place it in state Standby."
	}
	$obj Standby

    }

    #****************************************************************
    #*# method: CcdStopLoop
    #----------------------------------------------------------------

    protected method CcdStopLoop {} {

    	#*# 1. search all ACE and FIERA CCDs
	set objs [concat [Contents stooDCS_ACE 10] \
			 [Contents stooDCS_FIERA 10]]

	#*# 2. stop looping CCDs
	foreach obj $objs {
	    $obj StopLoop
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable osExists  1

}


#___oOo___
