#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooLOG.tcl,v 1.47 2005/08/23 08:02:50 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  31/01/02  clear logFile in destructor (when closing file).
# rschmutz  04/01/02  Exit: return exit code 1.
# rschmutz  25/10/01  SPR 20010580: add env. to log name.
# rschmutz  24/03/01  SPR 20010139: better error reporting.
# rschmutz  06/01/01  method Warning added.
# rschmutz  07/02/00  created (based in os0SCRIPT_BASE.tcl).
#

#************************************************************************
#   NAME
#	stooLOG - log utilities
# 
#   SYNOPSIS
#	stooLOG name [-infoHeader <text>]
#		     [-infoWidget <widget>]
#		     [-loglevel <level>]
# 
#   DESCRIPTION
#	stooLOG provides log utilities for scripts.
#
#   PUBLIC METHODS
#	SetWidget <widget>
#	    Sets the widget where the logs should appear.
#
#   PUBLIC DATA
#	infoHeader: header printed by method Info.
#	infoWidget: widget used by method Info.
#	loglevel:   loglevel used by method Debug.
#
#   PROTECTED METHODS
#	Exit [<text>]
#	    Prints the text (if supplied) and terminates the script.
#
#	Debug <text> [<level>]
#	    Prints text using the Info method if the loglevel
#	    is greater or equal level. The default level is 1.
#
#	Info <text>
#	    Prints the text to stdout and to the log file.
#
#	Warning <text>
#	    Prints the text to stdout and to the log file
#	    and sets the public variable warning to 1.
#
#	Log <text>
#	    Prints the text to the log file.
#
#   FILES
#	$VLTDATA/tmp/$infoHeader.log  -  READ  -- log file
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: stooLOG
#------------------------------------------------------------------------

class stooLOG {

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	eval configure $args
    }

    #****************************************************************
    #*# destructor
    #----------------------------------------------------------------

    destructor {

	#*# 1. close log file
	if { "$logFile" != "" } {
	    close $logFile
	    set logFile ""
	}
    }

    #****************************************************************
    #*# method: SetWidget
    #----------------------------------------------------------------

    public proc SetWidget { widget } {
    	set infoWidget $widget
    }

    #****************************************************************
    #*# method: Exit
    #----------------------------------------------------------------

    protected method Exit { {msg ""} } {
	global cv

    	#*# 1. print message
	if { "$msg" != "" } {
	    Info $msg
	}

	#*# 2. exit or issue an error (if we are in a panel)
	if { ![info exists cv] } {
	    exit 1
	} else {
	    error $msg
	}
    }

    #****************************************************************
    #*# method: Debug
    #----------------------------------------------------------------

    protected method Debug { msg {level 1} } {

    	#*# 1. if loglevel: print message
    	if { $loglevel >= $level } {
	    Info "       $msg"
	}

    }

    #****************************************************************
    #*# method: Info
    #----------------------------------------------------------------

    protected method Info { msg } {

    	#*# 1. print message
	if { "$infoWidget" == "" } {
	    puts "$infoHeader> $msg"
	} else {
	    $infoWidget addItem end "$infoHeader> $msg"
	    $infoWidget setCursor end
	    update
	}

	#*# 2. log message
	Log $msg
    }

    #****************************************************************
    #*# method: Warning
    #----------------------------------------------------------------

    protected method Warning { msg } {
	set warning 1
    	Info $msg
    }

    #****************************************************************
    #*# method: Log
    #----------------------------------------------------------------

    protected method Log { msg } {
	global env

	#*# 1. if no infoHeader: return
	if { "$infoHeader" == "-" || "$infoHeader" == "" } {
	    return
	}

	#*# 2. open log file
	if { "$logFile" == "" && $logError == 0 } {
	    set filename "$env(VLTDATA)/tmp/$env(RTAPENV).[set infoHeader].log"
	    if {[catch {
		set logFile [open $filename a+]
	    }]} {
		set logError 1
		Exit "ERROR: Probably wrong unix account (unable to open $filename)."
	    }
	    if { [system "chmod 644 $filename"] != 0 } {
		set logError 1
		Exit "ERROR: Probably wrong unix account ($filename has other owner)."
	    }
	    puts $logFile ""
	}

    	#*# 3. log message
	if { "$logFile" != "" } {
	    puts $logFile "$infoHeader> $msg"
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public common loglevel 0
    public common infoHeader "-"
    public common infoWidget ""
    public common warning 0

    private common logFile ""
    private common logError 0
}


#************************************************************************
#___oOo___

