#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooMIDAS.tcl,v 1.47 2005/08/23 08:02:50 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  15/01/03  SPR VLTSW20030003: Linux support added.
# rschmutz  07/02/00  created.
#

#************************************************************************
#   NAME
#	stooMIDAS - start/stop MIDAS
# 
#   SYNOPSIS
#	stooMIDAS midas
# 
#   DESCRIPTION
#	stooMIDAS allows to start/stop MIDAS.
#
#   PARENT
#	stooAPPLICATION
#
#   PUBLIC METHODS
#	Start
#	    Start the application.
#	Stop
#	    Stop the application.
#	IsActive
#	    Returns 1 if the application is running, 0 otherwise.
#	IsStopped
#	    Returns 1 if no application process is running, 0 otherwise.
#	Standby
#	    Sets the application in state STANDBY.
#
#   FILES
#	xxmcfgINS.cfg - instrument configuration file.
#	    The configuration file is searched in
#	    the following directories:
#	      $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#	      $VLTDATA/config
#	      $VLTROOT/config
#	      $INTROOT/config
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooMIDAS
#------------------------------------------------------------------------

class stooMIDAS {
    inherit stooAPPLICATION

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	#*# 1. configure options
	configure -objType MIDAS -name MIDAS
    	eval configure $args
    }

    #****************************************************************
    #*# destructor
    #----------------------------------------------------------------

    destructor {
    	delete object $pco
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg } {

	#*# 1. set default start
	set txt "F"
	catch { set txt [$cfg Get START.MIDAS.DEFAULT] }
	configure -default [Logical2Num $txt]
	Debug "MIDAS: default    = '$default'" 5

	#*# 2. create pco process
	set pco [stooPROCESS $this.pco -name pco -execCmd pco]
	AddObj $pco
    }

    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start {} {
        global env

	#*# 1. start pco
	Info "START: Starting MIDAS..."
	if { ![$pco IsActive]} {
	    $pco Start
	}

	#*# 2. if MIDAS is already running: return
	if {[IsActive]} {
	    Info "INFO : MIDAS already running."
	    return
	}

	#*# 3. start MIDAS in background
	if { ![IsActive] } {
	    set midasIn [seq_findFile config/stooMidas.in]
	    ExecCmd [Name] "xterm -T [Name] -iconic -e stooMidas $midasIn &"
	    Sleep 5
	    if { ![IsActive] } {
		Warning "ERROR: Problems starting [Name]. Probably address already in use."
		return
	    } 
	    Info "       MIDAS started."
	}

	#*# 4. open a MIDAS display
	if { ![IsActive 1] } {
	    set host [exec hostname]
    	    if {[$pco SendCommand FORWMSG "MIDAS,11,$host,create/display"]} {
	        Info "       MIDAS display started."
	    }
	}
    }

    #****************************************************************
    #*# method: Stop
    #----------------------------------------------------------------

    public method Stop {} {

	#*# 1. MIDAS is normally not stopped
	Info "INFO : MIDAS is normally not stopped."
	return

	#Info "STOP : Stopping MIDAS..."
	#KillProcess "exec/idiserv.exe"
	#KillProcess "monit/prepa.exe"
	#Info "       MIDAS stopped (or was not running)."
    }

    #****************************************************************
    #*# method: IsActive
    #----------------------------------------------------------------

    public method IsActive { {disp 0} } {
	
	#*# 1. check PCO
	if { ![$pco IsActive]} {
	    return 0
	}

	#*# 2. check MIDAS itself
	if { [exec uname] == "Linux" } {
	    set txt "midas"
	} else {
	    set txt "inmidas"
	}
	if { [GetProcessId $txt] == 0 } {
	    return 0
	} 

	#*# 3. check if a display is running
	if { $disp != 0 } {
	    if { [GetProcessId "exec/idiserv.exe"] == 0 } {
		return 0
	    }
	}

	return 1
    }

    #****************************************************************
    #*# method: IsStopped
    #----------------------------------------------------------------

    public method IsStopped {} {

	#*# 1. check PCO
	if {[$pco IsActive]} {
	    return 0
	}

	#*# 2. check MIDAS itself
	if { [exec uname] == "Linux" } {
	    set txt "midas"
	} else {
	    set txt "inmidas"
	}
	if { [GetProcessId $txt] != 0 } {
	    return 0
	} 

	return 1
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    protected variable pco ""
}


#___oOo___
