#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooOS.tcl,v 1.47 2005/08/23 08:02:49 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  30/01/03  SPR 20030026: timeout increased from 30s to 120s
# rschmutz  07/11/02  SPR 20020508: if xxoControl does not exist,
#		       then start bossControl.
# rschmutz  23/05/02  START.*.AVAIL support added.
# rschmutz  23/11/00  BOSS support added.
# rschmutz  25/05/00  OCS.CON.DEFAULT support added.
# rschmutz  07/02/00  created.
#

#************************************************************************
#   NAME
#	stooOS - start/stop OS
# 
#   SYNOPSIS
#	stooOS os
# 
#   DESCRIPTION
#	stooOS allows to start/stop a standard OS.
#
#   PARENT
#	stooAPPLICATION
#
#   PUBLIC METHODS
#	Config <cfg>
#	    Configure the application.
#	Start
#	    Start the application.
#	Stop
#	    Stop the application.
#	IsActive
#	    Returns 1 if the application is running, 0 otherwise.
#	IsStopped
#	    Returns 1 if no application process is running, 0 otherwise.
#	Standby
#	    Sets the application in state STANDBY.
#
#   FILES
#	xxmcfgINS.cfg - instrument configuration file.
#	    The configuration file is searched in
#	    the following directories:
#	      $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#	      $VLTDATA/config
#	      $VLTROOT/config
#	      $INTROOT/config
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooOS
#------------------------------------------------------------------------

class stooOS {
    inherit stooAPPLICATION

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
        configure -objType OS
    	eval configure $args
    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg } {
    	global stooInsMulti

	#*# 1. name, desc
	if { $stooInsMulti } {
	    configure -name [cconcat [$cfg Get INS.CON.ID] "_OS"]
	} else {
	    configure -name "OS"
	}
	configure -desc "[$cfg Get INS.CON.ID] OS"

	#*# 2. environment name
	configure -envName [$cfg Get INS.CON.WSENV]

	#*# 3. dictionary
	set txt [cconcat [$cfg Get INS.CON.ID] "_OS"]
	catch { set txt [$cfg Get OCS.CON.DICT] }
	configure -dictionary $txt

	#*# 4. alias
	catch { set txt [$cfg Get OCS.CON.ALIAS] }
	configure -alias $txt

    	#*# 5. domain (use it to recognize os0 implementation)
	if { ![catch { seq_findFile "bin/os0Control" }] } {
	    configure -domain [cconcat [$cfg Get INS.CON.PREFIX] "Os"]
	} else {
	    configure -domain ""
	}

	#*# 6. start application
	set txt "T"
	catch { set txt [$cfg Get OCS.CON.DEFAULT] }
	configure -default [Logical2Num $txt]

	#*# 7. start control panel
	set txt "F"
	catch { set txt [$cfg Get START.OSCTRL.DEFAULT] }
	set startControlUif [Logical2Num $txt]

	#*# 8. start status panel
	set txt "F"
	catch { set txt [$cfg Get START.OSSTAT.DEFAULT] }
	set startStatusUif [Logical2Num $txt]

	#*# 9. add processes (according to implementation)
	set prefix [cconcat [$cfg Get INS.CON.PREFIX] "o"]
	if { "$domain" == "" } {

	    #*# 9.1 BOSS implementation: add xxoControl
	    set procname [cconcat $prefix "Control"]
	    set execCmd $procname
	    if {[catch { seq_findFile "bin/$execCmd" }]} {
		set cfgset [$cfg Get INS.CON.ID]
	    	set execCmd "bossControl $cfgset"
	    }
	    set obj [stooPROCESS $this.$procname \
		     -name    $procname \
		     -execCmd $execCmd \
		     -envName $envName  \
		     -desc    "[Desc] control server"]
	    AddObj $obj

	    #*# 9.2 add bossArchiver
	    set procname [cconcat "bossArchiver_" $prefix]
	    set obj [stooPROCESS $this.$procname \
		     -name    $procname \
		     -execCmd "bossArchiver [$cfg Name]" \
		     -envName $envName  \
		     -desc    "[Desc] archiver server"]
	    AddObj $obj

	} else {
	    if { $loglevel > 1 } {
	    	set logOption " -loglevel $loglevel"
	    } else {
	        set logOption ""
	    }

	    #*# 9.2 os0 implementation: add os0Control
	    set procname [cconcat "os0Control_" $domain]
	    set obj [stooPROCESS $this.$procname \
		     -name    "os0Control" \
		     -execCmd "os0Control -domain $domain \
		     	       -timeout 20000 $logOption" \
		     -procName $procname \
		     -envName $envName  \
		     -desc    "[Desc] control server" \
		     -startNoCheck 1]
	    AddObj $obj

	    #*# 9.3 add os0ExpoControl
	    set procname [cconcat "os0ExpoControl_" $domain]
	    set obj [stooPROCESS $this.$procname \
		     -name    "os0ExpoControl" \
		     -execCmd "os0ExpoControl -domain $domain \
		     	       -timeout 20000 $logOption" \
		     -procName $procname \
		     -envName $envName  \
		     -desc    "[Desc] exposure server" \
		     -startNoCheck 1]
	    AddObj $obj

	    #*# 9.4 add OSLX server
	    set procname [cconcat "oslxServer_" $domain]
	    set obj [stooPROCESS $this.$procname \
		     -name    "oslxServer" \
		     -execCmd "oslxServer -v -dictionary $dictionary \
		     	       -alias $dictionary -name $procname" \
		     -procName $procname \
		     -envName $envName \
		     -desc    "[Desc] OSLX server"]
	    AddObj $obj

	    #*# 9.5 add etooServer
	    set procname "etooServer"
	    set obj [stooPROCESS $this.$procname \
		     -name    $procname \
		     -execCmd "$procname -d" \
		     -envName $envName \
		     -desc    "[Desc] name server"]
	    AddObj $obj
	}

	#*# 10. add control panel
	set panelAvail "T"
	catch { set panelAvail [$cfg Get START.OSCTRL.AVAIL] }
	if { $panelAvail == "T" } {
	    set panel [stooPANEL $this.control \
		       -name "[Name]_CONTROL" \
		       -execCmd [cconcat $prefix "panControl"] \
		       -envName $envName \
		       -desc "[Desc] Control Panel" \
		       -default $startControlUif]
	    AddObj $panel
	}

	#*# 11. add status panel
	set panelAvail "T"
	catch { set panelAvail [$cfg Get START.OSSTAT.AVAIL] }
	if { $panelAvail == "T" } {
	    set panel [stooPANEL $this.status \
		       -name "[Name]_STATUS" \
		       -execCmd [cconcat $prefix "panStatus"] \
		       -envName $envName \
		       -desc "[Desc] Status Panel" \
		       -default $startStatusUif]
	    AddObj $panel
	}
    }

    #****************************************************************
    #*# method Start
    #----------------------------------------------------------------

    public method Start {} {
        global env

	#*# 1. if BOSS: normal start
	if { "$domain" == "" } {
	    stooAPPLICATION::Start
	    return
	}

        #*# 2. if OS is active: do nothing
	if {[IsOsActive]} {
	    Info "INFO : [Desc] already running."
	    return
	}

	Info "START: Starting [Desc] . . ."

	#*# 4. Start OS name server
	[LookupName etooServer] Start

	#*# 5. Register everything in the OS name server
	Info "       Binding the subsystems to OS."
	set rtEnv $envName
	set osBindList "\
	    {osOslxAlias.$domain   oslxALIAS      @$rtEnv:Appl_data oslxServer_$domain $rtEnv}\
	    {osOslxConfig.$domain  oslxCONFIG     @$rtEnv:Appl_data oslxServer_$domain $rtEnv}\
	    {osOslxDiction.$domain oslxDICTIONARY @$rtEnv:Appl_data oslxServer_$domain $rtEnv}\
	    {osOslxFilter.$domain  oslxFILTER     @$rtEnv:Appl_data oslxServer_$domain $rtEnv}\
	    {osOslxFits.$domain    oslxFITS       @$rtEnv:Appl_data oslxServer_$domain $rtEnv}\
	    {osOslxGen.$domain     oslxGEN        @$rtEnv:Appl_data oslxServer_$domain $rtEnv}\
	    {osOslxOSetup.$domain  oslxOSETUP     @$rtEnv:Appl_data oslxServer_$domain $rtEnv}\
	    {osOslxSetuphd.$domain oslxSETUP_HD   @$rtEnv:Appl_data oslxServer_$domain $rtEnv}\
	    {osOslxShofits.$domain oslxSHORT_FITS @$rtEnv:Appl_data oslxServer_$domain $rtEnv}"
	set allBindList [concat $osBindList $bindList]
	foreach el $allBindList {
	    set etooName	[lindex $el 0]
	    set etooClassId	[EtooClassId [lindex $el 1]]
	    set etooRootPoint	[lindex $el 2]
	    set etooProc	[lindex $el 3]
	    set etooEnv		[lindex $el 4]
	    set execCmd "etooBind $etooName $etooClassId \
	    		 $etooRootPoint $etooProc $etooEnv"
	    Debug "Making OS binding: $etooName"
	    Debug "Making OS binding: $execCmd" 2
	    if {[catch { eval exec $execCmd } msg] } {
	    	Exit "Problems making OS binding ($msg)."
	    }
	}
	Debug "OS bindings made."

	#*# 6. start OSLX server
	[LookupName oslxServer] Start

	#*# 7. start OS exposure server, OS control server
	#      (First both are started, then we check that they are ok).
	set proc1 [LookupName os0ExpoServer]
	set proc2 [LookupName osServer]
  	$proc1 Start 1
	$proc2 Start 1
	if { ![$proc1 IsActive 10000] } {
	    Exit "Problems starting the [$proc1 Desc]."
	}
	if { ![$proc2 IsActive 10000] } {
	    Exit "Problems starting the [$proc2 Desc]."
	}

	Info "       $instrument OS started."
    }

    #****************************************************************
    #*# method: Standby
    #----------------------------------------------------------------

    public method Standby {} {

        #*# 1. check that OS is active
	if { ![IsActive] } {
	    Info "INFO : [Desc] is not active (STANDBY cmd. not sent)."
	    return 0
	}

    	#*# 2. send STANDBY to OS
	Info "INFO : Setting [Desc] in state STANDBY."
	set procname [[lindex [Contents stooPROCESS] 0] cget -procName]
    	if { ![SendCommand $envName $procname STANDBY "" $timeout]} {
	    Warning "ERROR: Problem setting the system in state STANDBY."
	    return 1
	}
	Debug "[Desc] set to state STANDBY."
	return 0
    }

    #****************************************************************
    #*# method: EtooClassId
    #----------------------------------------------------------------

    protected method EtooClassId { name } {

    	#*# 1. translate OS bind type to its corresponding id.
        set ret 0
    	switch -exact $name {
	    oslxALIAS 		{ set ret 170 }
	    oslxCONFIG		{ set ret 171 }
	    oslxDICTIONARY	{ set ret 172 }
	    oslxFILTER		{ set ret 173 }
	    oslxFITS  		{ set ret 174 }
	    oslxGEN   		{ set ret 175 }
	    oslxOSETUP		{ set ret 176 }
	    oslxSETUP_HD	{ set ret 177 }
	    oslxSHORT_FITS	{ set ret 178 }

	    ICS	      		{ set ret 133 }
	    DCS	      		{ set ret 134 }
	    FIERA		{ set ret 134 }
	    TCS	      		{ set ret 135 }

	    default	{ Exit "Unknown OS bind type '$name'." }
	}
	return $ret

    }

    #****************************************************************
    #*# variable
    #----------------------------------------------------------------

    public variable envName
    public variable dictionary
    public variable alias
    public variable domain
    public variable timeout 120000
    public variable startControlUif
    public variable startStatusUif
    public variable startWithPanel 1
}

#___oOo___
