#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooPANEL.tcl,v 1.47 2005/08/23 08:02:50 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# dpopovic  28/10/03  Standard ALARM panel can be stopped.
#                     SPR VLTSW20030369.
# rschmutz  08/11/02  Allow to config. existing panels.
# rschmutz  07/02/00  created (based in os0SCRIPT_BASE.tcl).
#

#************************************************************************
#   NAME
#	stooPANEL - panel
# 
#   SYNOPSIS
#	ctooPANEL name -name <name> -procName <procName>
# 
#   DESCRIPTION
#	stooPANEL allows to start/stop a panel.
#
#   PARENT
#	ctooOBJ_BASE stooLOG
#
#   PUBLIC METHODS
#	Config <cfg> <index>
#	    Configure a panel from a configuration file.
#	    <cfg> should be an instance of type ctooCONFIG.
#	    The following config. FITS keywords are expected
#	    and stored in the public data described below:
#	        START.PANEL<index>.NAME	   -> name
#	    	START.PANEL<index>.EXECMD  -> execCmd
#	        START.PANEL<index>.DESC	   -> desc (optional)
#	        START.PANEL<index>.DIR	   -> dir (optional)
#		START.PANEL<index>.STOP    -> stop (optional)
#		START.PANEL<index>.DEFAULT -> default (optional)
#
#	Start
#	    Start a panel executing <execCmd>
#	    A PID file $VLTDATA/tmp/<envName>.<procName>.pid is 
#	    associated to the panel.
#	    If there is an error, then <desc> is printed along
#	    with the error string.
#
#	Stop
#	    Stops a panel killing the process <execName>.
#	    If <execName> is empty, then <procName> is used to 
#	    search for the panel.
#	    If <execName> is '-nokill', then only the pid in the
#	    associated PID file is used to stop the panel.
#	    If there is an error, then <desc> is printed along
#	    with the error string.
#
#   PUBLIC DATA
#	name:	    panel name
#	execCmd:    command to start the panel
#	desc:	    panel description (default: name)
#	dir:	    directory, where the pacel should be started
#		    (default: current directory)
#	stop:	    stop panel before starting new one (default: 0).
#	default:    start by default (default: 1).
#
#	The following data are provided only for special cases:
#	procName:   part of the *.pid file (default: first word of execCmd)
#	envName:    part of the *.pid file (default: $RTAPENV)
#	execName:   alternative process name, used to stop the panel
#	grepFilter: grep filter used to find the panel to be stopped
#
#	Normally only name, execCmd and desc need to be set.
#
#   FILES
#	$VLTDATA/tmp/<envName>.<procName>.pid  --  lock file
#
#   CAUTIONS
#	If optional options are used, then these must be
#	specified _after_ the option used as default value.
#	E.g. option -desc must be specified _after_ option -name.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: stooPANEL
#------------------------------------------------------------------------

class stooPANEL {
    inherit ctooOBJ_BASE stooLOG

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
        global env

    	#*# 1. configure
    	eval configure $args
	configure -objType PANEL

	#*# 2. get desc, envName, procName, execName
	if { "$desc" == "" } { configure -desc $name }
	if { "$envName" == "" } {
	    if {[catch { set envName $env(RTAPENV) }]} {
	        Exit "ERROR: Environment variable RTAPENV is not set."
	    }
	}
	if { "$procName" == "" } { configure -procName [lindex $execCmd 0] }
	if { "$execName" == "" } { configure -execName [lindex $execCmd 0] }

    }

    #****************************************************************
    #*# method: Config
    #----------------------------------------------------------------

    public method Config { cfg {i ""} } {
        global stooInsMulti
    
	#*# 1. name
	set name1 [$cfg Get START.PANEL$i.NAME]
	if { $stooInsMulti } {
	    configure -name [cconcat [$cfg Get INS.CON.ID] "_" $name1]
	} else {
	    configure -name $name1
	}
	Debug "Panel $i:" 5

	#*# 2. exec cmd.
	configure -execCmd [$cfg Get START.PANEL$i.EXECMD]
	if { [llength $execCmd] == 1 } {
	    configure -procName $execCmd
	} else {
	    configure -procName [string tolower $name1]
	}
	configure -execName [lindex $execCmd 0]
	Debug "Panel $name: execCmd    = '$execCmd'" 5

	#*# 3. description
	set txt $name
	catch { set txt [$cfg Get START.PANEL$i.DESC] }
	configure -desc $txt
	Debug "Panel $name: desc.      = '$txt'" 5

	#*# 4. directory
	set txt ""
	catch { set txt [$cfg Get START.PANEL$i.DIR] }
	configure -dir $txt
	Debug "Panel $name: directory  = '$txt'" 5

	#*# 5. stop
	set txt "F"
	catch { set txt [$cfg Get START.PANEL$i.STOP] }
	configure -stop [Logical2Num $txt]
	Debug "Panel $name: stop       = '$stop'" 5

	#*# 6. default
	set txt "T"
	catch { set txt [$cfg Get START.PANEL$i.DEFAULT] }
	configure -default [Logical2Num $txt]
	Debug "Panel $name: default    = '$default'" 5
    }

    #****************************************************************
    #*# method: ConfigUpdate
    #----------------------------------------------------------------

    public method ConfigUpdate { cfg {i ""} } {

	Debug "Panel $i:" 5

	#*# 1. exec cmd.
	if { ![catch { configure -execCmd [$cfg Get START.PANEL$i.EXECMD] }]} {
	    if { [llength $execCmd] == 1 } {
	        configure -procName $execCmd
	    } else {
		set name1 [$cfg Get START.PANEL$i.NAME]
	        configure -procName [string tolower $name1]
	    }
	    configure -execName [lindex $execCmd 0]
	}
	Debug "Panel $name: execCmd    = '$execCmd'" 5

	#*# 2. description
	catch { configure -desc [$cfg Get START.PANEL$i.DESC] }
	Debug "Panel $name: desc.      = '$desc'" 5

	#*# 3. directory
	catch { configure -dir [$cfg Get START.PANEL$i.DIR] }
	Debug "Panel $name: directory  = '$dir'" 5

	#*# 4. stop
	if { ![catch { set txt [$cfg Get START.PANEL$i.STOP] }]} {
	    configure -stop [Logical2Num $txt]
	}
	Debug "Panel $name: stop       = '$stop'" 5

	#*# 5. default
	if { ![catch { set txt [$cfg Get START.PANEL$i.DEFAULT] }]} {
	    configure -default [Logical2Num $txt]
	}
	Debug "Panel $name: default    = '$default'" 5
    }

    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start {} {
        global env

	#*# 1. check variables
	if { "$name" == "" || "$execCmd" == "" } {
	    error "Panel configuration missing (name or execCmd missing)."
	}

	#*# 2. if required: stop previous copy
	if { $stop } {
	    Stop
	    sleep 2
	}

	#*# 3. change to the required directory
	if { "$dir" != "" } {
	    set pwd [exec pwd]
	    if {[file isdirectory $dir]} {
	    	cd $dir
	    }
	}
	#*# 4. start panel
	Debug "Panel $desc: starting..."
	Debug "Panel $desc: cmd='$execCmd &'." 2
	set pidFile "$env(VLTDATA)/tmp/$envName.$procName.pid"
	if {[catch { eval exec "$execCmd &" } pid]} {
	    Warning "ERROR: Problems starting $desc."
	    Warning "ERROR: $pid"
	    return 1
	} else {
	    file delete -force $pidFile
	    catch { exec echo $pid > $pidFile }
	    catch { exec chmod +w $pidFile }
	    Info "       Panel $desc: started."
	}

	#*# 5. change back to cwd
	if { "$dir" != "" } {
	    cd $pwd
	}

	sleep 1
	return 0
    }

    #****************************************************************
    #*# method: Stop
    #----------------------------------------------------------------

    public method Stop {} {
        global env

	#*# 1. check variables
	if { "$name" == "" || "$execCmd" == "" } {
	    error "Panel configuration missing (name or execCmd missing)."
	}

	#*# 2. stop panel
	Debug "Panel $desc: stopping..."
	set pidFile "$env(VLTDATA)/tmp/$envName.$procName.pid"
	if {[file isfile $pidFile]} {
	    set pid [read_file -nonewline $pidFile]
	    catch {exec kill $pid > /dev/null}
	    file delete -force $pidFile
	}

	# If this is the standard alarm panel (i.e. alrmDisplay), 
	# there are two additional processes that have been started.
	# They have to be killed as well.
	#
	# SPR VLTSW20030369
	#
	if {$procName == "alrmDisplay"} {
	    if {! [catch {exec ps -ef | grep alrm | grep -v seq | grep -v Display | grep $env(USER) | grep $envName} res]} {
		set pid [lindex $res 1]
		catch {exec kill $pid  >& /dev/null} 
	    } 
	    if {! [catch {exec ps -ef | grep alrm  | grep seqWish | grep $env(USER) | grep $envName} res]} {
		set pid [lindex $res 1]
		catch {exec kill $pid  >& /dev/null} 
	    }
	}
	
	if { "$execCmd" != "-nokill" } {
	    loop i 0 5 {
    	    	set pid [GetProcessId]
	    	if { $pid != 0 } {
	            catch { exec kill $pid > /dev/null }
	    	}
	    }
	}
	Info "       Panel $desc: stopped (or was not running)."
	sleep 1
    }

    #****************************************************************
    #*# method: GetProcessId
    #----------------------------------------------------------------

    protected method GetProcessId {} {
        global env

    	# 1. get pid
    	set pid 0
	if { "$grepFilter" == "" } {
	    if {! [catch {exec ps -ef | grep $env(USER) | grep $execName | grep -v grep} res]} {
	    	set pid [lindex $res 1]
	    }
	} else {
	    if {! [catch {exec ps -ef | grep $env(USER) | grep $execName | grep $grepFilter | grep -v grep} res]} {
	    	set pid [lindex $res 1]
	    }
    	}

	return $pid
    }

    #****************************************************************
    #*# method: Logical2Num
    #----------------------------------------------------------------

    protected method Logical2Num { val } {

    	if { "$val" == "F" } {
	    return 0
	} elseif { "$val" == "T" } {
	    return 1
	}
	error "'$val' is not a boolean value."
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable envName ""
    public variable execCmd "" {
	configure -procName [lindex $execCmd 0]
	configure -execName [lindex $execCmd 0]
    }
    public variable procName ""
    public variable execName ""
    public variable grepFilter ""
    public variable dir ""
    public variable stop 0
    public variable default 1
}


#************************************************************************
#___oOo___

