#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooPROCESS.tcl,v 1.47 2005/08/23 08:02:50 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2003-12-03  SPR VLTSW200300261: typo fixed.
# rschmutz  2002-05-26  Start with nocheck: set NO_ERR_DISPLAY,
#		        check msgScheduler result.
# rschmutz  2000-02-07  created (based in os0SCRIPT_BASE.tcl).
#

#************************************************************************
#   NAME
#	stooPROCESS - process
# 
#   SYNOPSIS
#	stooPROCESS name <options>
# 
#   DESCRIPTION
#	stooPROCESS allows to start/stop processes.
#
#   PARENT
#	ctooOBJ_BASE stooLOG
#
#   PUBLIC METHODS
#	Start [<nocheck>]
#	    Start process <name>.
#	    <desc> is only used to print debug information.
#	    The command <execCmd> is assumed to start the process
#	    <procName> in the environment <envName>.
#	    A PID file $VLTDATA/tmp/<envName>.<procName>.pid is 
#	    associated to the process. It contains the PID of the 
#	    process.
#
#	    If the PID file already exists, then an error is issued.
#	    If the PID file does not exist and a <execCmd> is supplied,
#	    then the command is started with seq_waitTillActive.
#	    PING commands are sent to the process until the process
#	    is active or the default timeout (in msecs.) is reached.
#	    If the process does not create the PID file, then the
#	    PID file is created.
#	    If the public variable useXterm is not 0, then the process
#	    is executed in an xterm (for debugging). The xterm is 
#	    positioned according to the xtermPos value (1-4).
#
#	    If <nocheck> is 1, then the method does not send PING
#	    commands to the process to check if it has started
#	    (i.e. this function does not wait, it returns 
#           immediately). IsActive should be called to check
#	    if the process started without problems.
#
#	Stop
#	    Stop process <name>.
#	    The EXIT command is sent to the process.
#	    If the PID file $VLTDATA/tmp/<env>.<procName>.pid exists,
#	    then a 'kill <pid>' signal is sent to the process.
#	    The PID file is deleted.
#	    <desc> is only used to print debug information.
#
#	IsActive [<timeout>] [<period>]
#	    Send PING commands to process <procName> in environment 
#	    <envName> until the process answers or the timeout is 
#	    reached.  The default period of the PINGs is 1 sec.
#	    The method returns 1 if the process answers, 0 otherwise.
#	    <desc> is only used to print debug information.
#
#	SendCommand <cmd> <pars> <timeout>
#	MsgCall <cmd> <pars> ?timeout?
#	    Send a command and wait for all replies.
#	    SendCommand and MsgCall are identical, except for
#	    their return values.
#	    MsgCall returns the last reply, while SendCommand
#	    returns a boolean indicating if the the command
#	    was successful or not.
#
#   PUBLIC DATA
#	name:	    process name
#	execCmd:    command to start the process (default: name)
#	desc:	    process description (default: name)
#
#       procName:   CCS process name (default: name)
#	envName:    CCS environment (default: $RTAPENV)
#	startTimeout: timeout to wait the process to come up during
#		    Start (default: 10000 msec).
#	wsSimOption: if the parent application option -wsSimulation
#		    is true, then the wsSimOption contents is
#		    appended to execCmd to start the process
#		    in WS simulation (default: "").
#	xtermPos:   xterm position (0-4)
#		    (default: 0 = process never started in an xterm)
#	useXterm:   start process in an xterm (if xtermPos = 1..4)
#		    (default: 0).
#
#   FILES
#	$VLTDATA/tmp/<envName>.<procName>.pid  -- lock file
#
#   CAUTIONS
#	If optional options are used, then these must be
#	specified _after_ the option used as default value.
#	E.g. option -desc must be specified _after_ option -name.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: stooPROCESS
#------------------------------------------------------------------------

class stooPROCESS {
    inherit ctooOBJ_BASE stooLOG

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	global env

	#*# 1. configure process
    	configure -objType PROCESS
    	eval configure $args

	#*# 2. get env. name, procName
	if { "$envName" == "" } {
	    if {[catch { set envName $env(RTAPENV) }]} {
	        Exit "ERROR: Environment variable RTAPENV is not set."
	    }
	}
	if { "$execCmd" == "" } { set execCmd $name }
	if { "$procName" == "" } { set procName $name }

    }


    #****************************************************************
    #*# method: Start
    #----------------------------------------------------------------

    public method Start { {nocheck 0} } {
	global env

	#*# 1. if the PID file exists: process already started
	Debug "Process $desc: starting..."
	set pidFile "$env(VLTDATA)/tmp/[set envName].[set procName].pid"
	if {[file isfile $pidFile]} {
	    if {[catch {seq_waitTillActive $envName $procName 300 100}]} {
	        Info "       Process $desc: restarting the process."
	        set pid [exec cat $pidFile]
	        catch {exec kill $pid > /dev/null}
	        file delete -force $pidFile
	    } else {
	        Info "       Process $desc: already running."
		return
	    }
	}

	#*# 3. if simulation: add wsSimOption
	set execCmd1 $execCmd
	catch {
	    if { [[Parent] cget -wsSimulation] } {
	    	append execCmd1 $wsSimOption
	    }
	}

	#*# 4. if useXterm: build xterm execute cmd.
	if { $useXterm != 0 && $xtermPos > 0 } {
	    switch -exact $xtermPos {
	    	1	{ set pos 200 }
		2 	{ set pos 400 }
		3	{ set pos 600 }
		4	{ set pos 800 }
	        default { set pos 200 }
	    }
	    set xtermCmd "xterm -geometry 100x10-10+[set pos] -iconic \
	                  -bg black -fg white -ls -T \"$name\" -e "
	    set execCmd1 "$xtermCmd $execCmd1"
	}

	#*# 5. loading CDT
	Debug "Process $desc: loading CDT." 2
	set i [string last "_" $procName]
	if { $i == -1 } {
	    set cdt $procName
	} else {
	    set cdt [csubstr $procName 0 $i]
	}
	catch { exec cmdSetup -parse $cdt }

	#*# 6. start the process
	Debug "Process $desc: cmd='$execCmd1'." 2
	if { $nocheck } {
	    set saveNoErrDisplay [info exists env(NO_ERR_DISPLAY)]
	    if { $saveNoErrDisplay == 0 } {
	        set env(NO_ERR_DISPLAY) ""
	    }
	    if {[catch { exec msgSchedule $envName "$execCmd1" } msg]} {
	        if { $saveNoErrDisplay == 0 } {
	    	    unset env(NO_ERR_DISPLAY)
	        }
	        Warning "ERROR: Process $desc: problems starting the process."
	        Warning "ERROR: (cmd='$execCmd1')."
	        Warning "ERROR: $msg"
	        Exit ""
	    }
	    if { $saveNoErrDisplay == 0 } {
	    	unset env(NO_ERR_DISPLAY)
	    }
	} else {
	    if {[catch {seq_waitTillActive $envName $procName $startTimeout $defaultPeriod $execCmd1} msg]} {
	        Warning "ERROR: Process $desc: problems starting the process."
	        Warning "ERROR: (cmd='$execCmd1')."
	        Warning "ERROR: $msg"
	        Exit ""
	    } 

	    #*# 7. register the pid
	    if {![file isfile $pidFile]} {
	        catch { exec echo "NO PID" > $pidFile }
	        catch { exec chmod +w $pidFile }
	    }
	}
	Info "       Process $desc: started."
    }

    #****************************************************************
    #*# method: Stop
    #----------------------------------------------------------------

    public method Stop {} {
	global env

	#*# 1. stop the process
	Debug "Process $desc: stopping..."
	catch {seq_waitTillDead "$envName" $procName $defaultTimeout $defaultPeriod EXIT}

	#*# 2. if the PID file exists: kill the process, delete the file.
	set pidFile "$env(VLTDATA)/tmp/[set envName].[set procName].pid"
	if {[file isfile $pidFile]} {
	    set pid [exec cat $pidFile]
	    catch {exec kill $pid > /dev/null}
	    file delete -force $pidFile
	}

	#*# 3. check process is stopped
	if {[catch {seq_waitTillActive $envName $procName 300 100}]} {
	    Info "       Process $desc: stopped (or was not running)."
	} else {
	    Warning "ERROR: Process $desc: problems stopping the process."
	}
    }

    #****************************************************************
    #*# method: IsActive
    #----------------------------------------------------------------

    public method IsActive { {timeout 0} {period 0} } {
	global env

	#*# 1. check process
	set run 0
	Debug "Process $desc: checking..."
	if { $timeout == 0 } {
	    set timeout $defaultTimeout
	}
	if { $period == 0 } {
	    set period $defaultPeriod
	}
	if {[catch {seq_waitTillActive $envName $procName $timeout $period}]} {
	    Debug "Process $desc: not running." 2
	} else {
	    set run 1
	    Debug "Process $desc: ok."
	}
	return $run
    }

    #****************************************************************
    #*# method: SendCommand
    #----------------------------------------------------------------

    public method SendCommand { cmd pars {timeout 0} } {

    	#*# 1. message call
	set reply [MsgCall $cmd $pars $timeout]

	#*# 2. return ok/not ok (why ?)
	set ok 1
	if { [string first FAILURE $reply] != -1 } {
	    Warning "ERROR: Command reply: $reply"
	    set ok 0
	}
	return $ok
    }
	    
    #****************************************************************
    #*# method: MsgCall
    #----------------------------------------------------------------

    public method MsgCall { cmd pars {timeout 0} } {

	set errVar 0
	set reply ""
	if { $timeout == 0 } {
	    set timeout $msgTimeout
	}
	Debug "Sending command: $envName/$procName $cmd $pars" 3
	if {[catch {

	    #*# 1. send command
	    set cmdId [eval {seq_msgSendCommand $envName $procName $cmd} $pars]

	    #*# 2. wait for replies
	    set lastReply 0
	    while { $lastReply == 0 } {
		set reply [seq_msgRecvReply $cmdId errVar lastReply -$timeout]
		if { $lastReply == 0 } {
		    Debug "Command reply: $reply \[intermediate reply\]" 3
		}
	    }
	} result]} {
	    set reply "FAILURE: $result"
	}
	Debug "Command reply: $reply" 3
	
	return $reply
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable execCmd  ""
    public variable envName  ""
    public variable procName ""
    public variable xtermPos ""
    public variable wsSimOption ""
    public variable startTimeout 10000
    public common   useXterm 0
    public common   defaultTimeout 500
    public common   defaultPeriod  100
    public common   msgTimeout 10000
}


#************************************************************************
#___oOo___

