#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooSIMPLE.tcl,v 1.47 2005/08/23 08:02:51 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  08/05/03  ScanRestart added.
# rschmutz  08/11/02  -standby fixed: send Standby to main stooAPPL_SET.
# rschmutz  09/01/01  START.CON.STBYAPP support added.
# rschmutz  08/12/00  START.CON.STARTAPP support added.
# rschmutz  07/02/00  created.
#

#************************************************************************
#   NAME
#	stooSIMPLE - start/stop software
# 
#   SYNOPSIS
#	stooSIMPLE name
# 
#   DESCRIPTION
#	This is the main class responsible for starting/stopping
#	standard software.
#	The startups is performed according to the following steps:
#	1) The configuration is read, creating the corresponding
#	   startup objects (Config)
#	2) The command line options are processed, to define
#	   what applications and panels shall be started/stopped.
#	3) The applications and panels are started/stopped.
#
#   PUBLIC METHODS
#	Config <cfg>
#	    Reads the configuration and stores it in the repository.
#	    This is normally done in a derived class, or loaded in
#	    the file specified with START.CON.TCLFILE.
#	    The supplied method loads the panels specified in the
#	    configuration file.
#
#	Start <options>
#	    Starts the software.
#	    The following options are accepted:
#	    no option:      Call StartAll.
#	    -proc <name>    Start the application <name>.
#	    -panel <name>   Start the panel <name>.
#			    Modifiers:
#	    -restart	    First call StopAll or stop the
#			    application or panel before
#			    restarting it.
#	    -check	    Call CheckAll or check that the
#			    specified application or panel is
#			    running. Report problems.
#
#	Stop <options>
#	    Stops the software.
#	    The following options are accepted:
#	    no option:	    Call StopAll.
#	    -proc <name>    Stop the application <name>.
#	    -panel <name>   Stop the panel <name>.
#	    -standby	    Call StandbyAll or place the
#			    specified application in state
#			    Standby.
#
#   PUBLIC METHODS
#	-
#
#   PROTECTED METHODS
#	StartAll ?restart?
#	    Called by Start when no option is given.
#	    - stops all panels (unless ?restart? is true;
#	      this is used, when StopAll was previously called).
#	    - starts all default applications (in the inverse
#	      order of how they are registered in the repository).
#	    - starts all default panels.
#	    - places all applications directly owned by this object
#	      in state Standby.
#
#	StopAll ?restart?
#	    Called by Stop when no option is given.
#	    - places all applications directly owned by this object
#	      in state Standby.
#	    - stops all panels (if ?restart? is specified,
#	      stop only the default panels).
#	    - stops all applications (with -stop == 1).
#
#	CheckAll
#	    Called by Start when only option -check is given.
#	    - checks all applications.
#
#	StandbyAll
#	    Called by Stop when only option -standby is given.
#	    - places all applications directly owned by this object
#	      in state Standby.
#
#   FILES
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooSIMPLE
#------------------------------------------------------------------------

class stooSIMPLE {
    inherit stooSTARTUP_BASE

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
	configure -objType STARTUP -name SIMPLE
    	eval configure $args
    }

    #****************************************************************
    #*# proc ItclTypes
    #----------------------------------------------------------------

    proc ItclTypes {} {
    	global stooType stooInsMulti

	set stooType(STARTUP) stooSIMPLE
	set stooInsMulti 0
    }

    #****************************************************************
    #*# method Config
    #----------------------------------------------------------------

    public method Config { cfg } {
        global stooInsMulti
    
	#*# 1. load panels
	loop i 1 [expr {$maxPanels + 1}] {
	    if {![catch { set panelName [$cfg Get START.PANEL$i.NAME] }]} {

		# if INS.PANELi.EXECMD is not supplied and 
		# panel does not already exist: error
		set noExecmd [catch { $cfg Get START.PANEL$i.EXECMD }]
		set obj [LookupName $panelName stooPANEL $maxDepth]
		if { $obj == "" && $stooInsMulti == 1 } {
		    set panelName [cconcat [$cfg Get INS.CON.ID] "_" $panelName]
		    set obj [LookupName $panelName stooPANEL $maxDepth]
		}
		if { $obj != "" && [llength $obj] != 1 } {
		    error "Several panels have the same name $panelName."
		}
		if { $noExecmd == 1 && $obj == "" } {
		    error "Config. keyword START.PANEL$i.EXECMD is missing."
		}

		# if old panel exists: update it, 
		# otherwise: create and config. new panel
		if { $obj != "" } {
		    $obj ConfigUpdate $cfg $i
		} else {
	    	    set obj [stooPANEL $this.panel$i]
		    $obj Config $cfg $i
		    AddObj $obj
		}
	    }
	}
    }

    #****************************************************************
    #*# method Start
    #
    #	    Starts the software.
    #
    #	    The following options are accepted:
    #	    no option:      Call StartAll.
    #	    -proc <name>    Start the application <name>.
    #	    -panel <name>   Start the panel <name>.
    #			    Modifiers:
    #	    -restart	    First call StopAll or stop the
    #			    application or panel before
    #			    restarting it.
    #	    -check	    Call CheckAll or check that the
    #			    specified application or panel is
    #			    running. Report problems.
    #----------------------------------------------------------------

    public method Start { {options ""} } {

	#*# 1. get options
	Info "START: [exec date]"
	Info "ARGS : $options."
	set panel ""
	set proc ""
	# -check is not supported.
	#GetOptions {-proc <name> -panel <name> -restart -check} $options
	GetOptions {-proc <name> -panel <name> -restart} $options
	if { $check == 1 && $restart == 1 } {
	    error "Options -check and -restart are not simultaneously allowed."
	}

    	#*# 2. if the environment is not running, skip everything
	if { ![IsWsEnvActive] } {
	    Exit "Environment is not running."
	}

	#*# 3. check if proc is an application set
	set parent ""
	if { $proc != "" } {
	    set parent [LookupName $proc stooAPPL_SET $maxDepth]
	}

	#*# 4. no -panel or -proc supplied: check/restart/start all
	if { $proc == "" && $panel == "" } {
	    if { $check } {
	    	CheckAll
	    } elseif { $restart } {
	    	# stop default panels, default appls.
	    	StopAll "" 1 1
		Info ""
		Sleep 5
		# start default appls., start default panels
		StartAll "" 1
	    } else {
	    	# stop default panels
		StopAll "" 1 0
		# start appls., start default panels
	    	StartAll "" 1
	    }
	} elseif { ($proc == "$infoHeader" || $parent != "") && "$panel" == "" } {
	    if { $check } {
	    	CheckAll $parent
	    } elseif { $restart } {
	    	# stop only default appls.
	    	StopAll $parent 0 1
		Sleep 5
		# start only appls.
		StartAll $parent 0
	    } else {
		# start only appls.
	    	StartAll $parent 0
	    }
	} else {
	    # 5. start specified applications
	    if { $proc != "" } {
	    	Process stooAPPLICATION $proc Start $restart $stbyApp
	    }

	    # 6. start specified panels
	    if { $panel != "" } {
	    	Process stooPANEL $panel Start $restart $stbyApp
	    }
	}

	Info "END  : Start end."
	Info "END  : [exec date]"
    }

    #****************************************************************
    #*# method Stop
    #
    #	    Stops the software.
    #
    #	    The following options are accepted:
    #	    no option:	    Call StopAll.
    #	    -proc <name>    Stop the application <name>.
    #	    -panel <name>   Stop the panel <name>.
    #	    -standby	    Call StandbyAll or place the
    #			    specified application in state
    #			    Standby.
    #----------------------------------------------------------------

    public method Stop { {options ""} } {

    	#*# 1. if the environment is not running, skip everything
	Info "STOP : [exec date]"
	Info "ARGS : $options."
	if { ![IsWsEnvActive] } {
	    Exit "Environment is not running."
	}

	#*# 2. get options
	set panel ""
	set proc ""
	GetOptions {-proc <name> -panel <name> -standby} $options
	if { "$panel" != "" && $standby == 1 } {
	    error "Options -panel and -standby are not simultaneously allowed."
	}

	#*# 3. check if proc is an application set
	set parent ""
	if { $proc != "" } {
	    set parent [LookupName $proc stooAPPL_SET $maxDepth]
	}

	#*# 4. no -panel or -proc supplied: standby/stop all
	if { $proc == "" && $panel == "" } {
	    if { $standby } {
	    	StandbyAll
	    } else {
	    	StopAll "" 2 2
	    }
	} elseif { ($proc == "$infoHeader" || $parent != "") && "$panel" == "" } {
	    if { $standby } {
	    	StandbyAll $parent
	    } else {
	    	StopAll $parent 0 1
	    }
	} else {
	    # 5. standby/stop specified applications
	    if { $proc != "" } {
	    	if { $standby } {
	    	    Process stooAPPLICATION $proc Standby
		} else {
	    	    Process stooAPPLICATION $proc Stop 0 $stbyApp
		}
	    }
	   
	    # 6. stop specified panels
	    if { $panel != "" } {
	    	Process stooPANEL $panel Stop 0 $stbyApp
	    }
	}

	Info "END  : Stop end."
	Info "END  : [exec date]"
    }

    #****************************************************************
    #*# method StartAll
    #
    # 1. if stopPanels: stop all default panels
    # 2. start all default applications
    # 3. if startPanels: start all default panels.
    # 4. place all applications directly owned by this object
    #	 in state Standby.
    #----------------------------------------------------------------

    protected method StartAll { {parent ""} {startPanels 1} } {

    	#*# 1. set parent
	if { $parent == "" } {
	    set parent $this
	}

	#*# 2. start all default applications
	set objList [$parent LookupRValue "-default" 1 stooAPPLICATION $maxDepth]
	foreach obj $objList {
	    $obj Start
	}	

	#*# 3. start all default panels
	if { $startPanels == 1 } {
	    set objList [$parent LookupValue "-default" 1 stooPANEL $maxDepth]
	    foreach obj $objList {
	    	$obj Start
	    }
	}	

	#*# 4. place applications in state Standby
	StandbyAll $parent

	#*# 5. restart scanning
	ScanRestart
    }

    #****************************************************************
    #*# method StopAll
    #
    # 1. place all applications directly owned by this object
    #	 in state Standby.
    # 2. stop panels (?panels? = 1: default panels, 2: all panels)
    # 3. stop applications (?appls? = 1: default appls., 2: all)
    #    (do not stop, if -stop == 0).
    #----------------------------------------------------------------

    protected method StopAll { {parent ""} {panels 2} {appls 2} } {

    	#*# 1. set parent
	if { $parent == "" } {
	    set parent $this
	}

    	#*# 2. place applications in state Standby
	if { $appls != 0 } {
	    StandbyAll $parent
	}

	#*# 3. stop panels
	if { $panels == 2 } {
	    set objList [$parent Contents stooPANEL $maxDepth]
	} elseif { $panels == 1 } {
	    set objList [$parent LookupValue "-default" 1 stooPANEL $maxDepth]
	} else {
	    set objList ""
	}
	foreach obj $objList {
	    $obj Stop
	}

	#*# 4. stop applications (with -stop == 1)
	if { $appls == 2 } {
	    set objList [$parent Contents stooAPPLICATION $maxDepth]
	} elseif { $appls == 1 } {
	    set objList [$parent LookupValue "-default" 1 stooAPPLICATION $maxDepth]
	} else {
	    set objList ""
	}
	foreach obj $objList {
	    if { [$obj cget -stop] == 0 } {
	    	if { [$obj IsActive] } {
	    	    Info "INFO : [$obj Desc] is normally NOT stopped.  Left running."
		} else {
	    	    Info "INFO : [$obj Desc] is normally NOT stopped"
	    	    Info "       (but seems not to be running anyway)."
		}
	    } else {
	        $obj Stop
	    }
	}	
    }

    #****************************************************************
    #*# method CheckAll
    #----------------------------------------------------------------

    protected method CheckAll { {parent ""} } {

    	#*# 1. set parent
	if { $parent == "" } {
	    set parent $this
	}

    	#*# 2. check all applications
	set objList [$parent Contents stooAPPLICATION $maxDepth]
	foreach obj $objList {
	    $obj Check
	}	

    }

    #****************************************************************
    #*# method StandbyAll
    #----------------------------------------------------------------

    protected method StandbyAll { {parent ""} } {

    	#*# 1. set parent
	if { $parent == "" } {
	    set parent $this
	}

	#*# 2. place the main application set(s) in state Standby
	if { [$parent isa stooAPPL_SET] } {
	    $parent Standby
	} else {
	    set objList [$parent Contents stooAPPL_SET]
	    foreach obj $objList {
	        $obj Standby
	    }	
	}
    }

    #****************************************************************
    #*# method ScanRestart
    #----------------------------------------------------------------

    protected method ScanRestart {} {

	#*# 1. re-start scan task
	Info "INFO : Re-starting scanning."
	seq_logData "Re-starting scan task ..."
	ExecCmd SCAN_OFF scanOff
	sleep 2
	ExecCmd SCAN_ON  scanOn
	sleep 2
	seq_logData "Re-starting scan task: done."
	Debug "INFO : Scanning re-started." 2
    }

    #****************************************************************
    #*# method Process
    #
    #	Process applications and panels.
    #   The command <cmd> is applied to all objects <nameList>
    #   of type <type> found in the repository.
    #   If <restart> is true, then first command Stop is called
    #   (the supplied <cmd> is assumed to be Start in this case).
    #   If panels are started, check if the associated application
    #   should also be started.
    #----------------------------------------------------------------

    private method Process { type nameList cmd {restart 0} {standbyApp 1} } {

        #*# 1. lookup all objects
	set objList ""
    	foreach name $nameList {
	    set obj [LookupName $name $type $maxDepth]
	    if { "$obj" == "" } {
	    	if { "$type" == "stooPANEL" } {
		    set typeName "panel"
		} else {
		    set typeName "application"
		}
		set knownList [Contents $type $maxDepth]
		if { "$knownList" == "" } {
		    error "No [set typeName]s known."
		} else {
		    Info "INFO : Known [set typeName]s:"
		    if { $typeName == "application" } {
		    	Info "       [format %-16s $infoHeader]: All applications"
		    }
		    foreach knownObj $knownList {
		    	Info "       [format %-16s [$knownObj Name]]: [$knownObj Desc]"
		    }
	    	    error "Unknown $typeName $name."
		}
	    }
	    set objList [concat $objList $obj]
	}

	#*# 2. if stopping applications: place them in state STANDBY
	if { $standbyApp == 1 && "$type" == "stooAPPLICATION" && \
	     ("$cmd" == "Stop" || $restart == 1) } {
	    foreach obj $objList {
	        $obj Standby
	    }
	}

	#*# 3. if restart: call Stop
	if { $restart } {
	    foreach obj $objList {
	    	$obj Stop
	    }
	}

	#*# 4. foreach object:
	set standbyList ""
	foreach obj $objList {

	    #*# 4.1 if starting panels: check if start application
	    if { $startApp == 1 && "$type" == "stooPANEL" && "$cmd" == "Start" } {
	    	set parent [$obj Parent]
		set parentIsAppl 0
		catch { set parentIsAppl [$parent isa stooAPPLICATION] }
		if { $parentIsAppl } {
		    if { [$parent cget -startWithPanel] == 1 } {
		    	if { ![$parent IsActive] } {
		    	    $parent Start
			    lappend standbyList $parent
			}
		    }
		}
	    }

	    #*# 4.2 if starting applications: make list to send STANDBY
	    if { "$type" == "stooAPPLICATION" && "$cmd" == "Start" } {
		if { ![$obj IsActive] } {
		    lappend standbyList $obj
		}
	    }

	    #*# 4.3 call command
	    $obj $cmd
	}

	#*# 5. send STANDBY to started applications
	if { $standbyApp == 1 } {
	    foreach obj $standbyList {
	        $obj Standby
	    }
	}
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable proc      ""
    public variable panel     ""
    public variable restart   0
    public variable check     0
    public variable standby   0
    public variable startApp  1
    public variable stbyApp   1
    public common   maxDepth  10
    public common   maxPanels 20
}


#___oOo___
