#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: stooSTARTUP.tcl,v 1.47 2005/08/23 08:02:48 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  25/05/02  method SubConfig added.
# rschmutz  06/01/01  report warnings.
# rschmutz  09/12/00  name parameter added.
# rschmutz  15/09/00  adapted to ctoo 1.16: load START.cfg only if it exists.
# rschmutz  06/02/00  created.
#

#************************************************************************
#   NAME
#	stooSTARTUP - start an application
# 
#   SYNOPSIS
#	stooSTARTUP myScript <name> [-loglevel <n>] [-debug]
#	myScript Start
# 
#   DESCRIPTION
#	This class implements the initial step to start and stop 
#	a VLT application.
#
#	The startup is performed as follows:
#
#	1) stooSTARTUP creates a repository (ctooOBJ_REPOSITORY)
#	   and opens the config. file.
#	2) According to the START.CON.TYPE value, the stooType
#	   array is initialized with the class types needed to
#	   start this particular application type.
#       3) If the particular application needs some special 
#	   processing not considered by the standard classes,
#	   then a Tcl script START.CON.TCLFILE can be supplied.
#	   This Tcl script should define alternative class types
#	   and register them in the stooType array.
#	4) A STARTUP object is created and stored in the repository
#          (the STARTUP type is: $stooType(<START.CON.TYPE>))
#       5) The Config method of the STARTUP object is called.
#	   This Config method is expected to read the application
#	   configuration and to store it in the repository.
#	6) The Start resp. Stop method of the STARTUP object is
#	   called.  This method is expected to actually start resp.
#	   stop the application.
#
#   PUBLIC METHODS
#	Start <args>
#	    Start the application as specified above.
#	    The -config option specifies the config. file to use,
#	    all other options are passed to the STARTUP object
#	    in the repository.
#
#	Stop <args>
#	    Stop the application as specified above.
#	    The -config option specifies the config. file to use,
#	    all other options are passed to the STARTUP object
#	    in the repository.
#
#   FILES
#	INS.config - instrument configuration file.
#	    The configuration file is searched in
#	    the following directories:
#	      $INS_ROOT/SYSTEM/COMMON/CONFIGFILES
#	      $VLTDATA/config
#	      $VLTROOT/config
#	      $INTROOT/config
#
#   SEE ALSO
#	stooStart stooStop
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: stooSTARTUP
#------------------------------------------------------------------------

class stooSTARTUP {
    inherit stooBASE

    #****************************************************************
    #*# method Start
    #----------------------------------------------------------------

    public method Start { param } {

    	#*# 1. search -debug flag, call Start
	set i [lsearch $param "-debug"]
	if { $i != -1 } {
	    set debug 1
	    lvarpop param $i
	    Script Start $param
	} else {
            if {[catch { Script Start $param } msg]} {
	    	Exit "ERROR: $msg"
	    }
	}
    }

    #****************************************************************
    #*# method Stop
    #----------------------------------------------------------------

    public method Stop { param } {

    	#*# 1. search -debug flag, call Stop
	set i [lsearch $param "-debug"]
	if { $i != -1 } {
	    set debug 1
	    lvarpop param $i
	    Script Stop $param
	} else {
            if {[catch { Script Stop $param } msg]} {
	    	Exit "ERROR: $msg"
	    }
	}
    }

    #****************************************************************
    #*# method Script
    #
    #  1. get debug level
    #  2. create repository
    #  3. open config. set
    #  4. get application name
    #  5. load application type class types
    #  6. source the user extensions
    #  7. create the STARTUP object
    #  8. call STARTUP::Config
    #  9. process debug options -objList, -objShow
    # 10. call STARTUP::<cmd>
    #----------------------------------------------------------------

    protected method Script { cmd param } {
        global stooType env

    	#*# 1. get debug level
	set i [lsearch $param "-loglevel"]
	if { $i != -1 } {
	    lvarpop param $i
	    configure -loglevel [lvarpop param $i]
	}

    	#*# 2. create repository
	ctooOBJ_REPOSITORY rep -name STOOL

    	#*# 3. open config. set
	set configName [lvarpop param]
	Info "INIT : Reading $configName configuration."
	if {[catch { set cfg [ctooCONFIG $this.cfg $configName] }]} {
	    error "Error loading the configuration set $configName."
	}
	if {[file exists $env(INS_ROOT)/SYSTEM/COMMON/CONFIGFILES/START.cfg]} {
	    $cfg CfgAppend START
	}
	rep AddObj $cfg

	#*# 4. get application name
	set applName ""
	catch { set applName [$cfg Get START.CON.NAME] }
	if { "$applName" == "" } {
	    catch { set applName [$cfg Get INS.CON.ID] }
	}
	configure -infoHeader $applName

        #*# 5. load application type class types
	#
	#      The <applType>::ItclTypes procedure loads
	#      the stooType array with the class types
	#      that can be overloaded by the user extensions.
	#
	#      The <applType> uses this array to create other classes,
	#      E.g.: to create an OS object, <applType> will call:
	#        $stooType(OS) os
	#
	if {[catch { set applType [$cfg Get START.CON.TYPE] }]} {
	    error "Application type START.CON.TYPE is missing."
	}
	set typeClass [cconcat stoo $applType]
	if {[catch { [set typeClass]::ItclTypes }]} {
	    error "Wrong application type $applType."
	}

    	#*# 6. source the user extensions
	#
	#      The user tcl file should usually update the stooType array,
	#      according to the available user types.
	#
	#      E.g.: setting stooType(OS) = myOS
	#            will create an object myOS to start an OS, instead
	#            of the default stooOS.  Of course, the user tcl 
	#	     file should supply this myOS class.
	#
	set userTclFile ""
	catch { set userTclFile [$cfg Get START.CON.TCLFILE] }
	if { "$userTclFile" != "" } {
	    set userTclFile [seq_findFile [cconcat "bin/" $userTclFile]]
	    if { $debug } {
	        uplevel #0 source $userTclFile
	    } else {
	        if {[catch { uplevel #0 source $userTclFile } msg]} {
	    	    error "ERROR loading $userTclFile: $msg."
		}
	    }
	}

    	#*# 7. create the STARTUP object
	if {[catch { set startup [$stooType(STARTUP) $this.startup] }]} {
	    error "Wrong application type class $stooType($applType)."
	}
	rep AddObj $startup

    	#*# 8. call STARTUP::Config
	$startup Config [rep Contents CONFIG]

	#*# 9. process debug options -objList, -objShow
	rep Debug $param 

    	#*# 10. call STARTUP::<cmd>
	$startup $cmd $param

	#*# 11. report warnings
	if { $warning } {
	    error "Warnings (or errors) encountered. Check log."
	}
    }

    #****************************************************************
    #*# method SubConfig
    #
    # 1. save current stooType array
    # 2. open config. set
    # 3. load application type class types
    # 4. source the user extensions
    # 5. create the STARTUP object
    # 6. call STARTUP::Config
    # 7. delete the cfg and STARTUP objects
    # 8. restore stooType array
    #----------------------------------------------------------------

    public method SubConfig { obj configName } {
        global stooType env

	#*# 1. save current stooType array
	set saveStooType [array get stooType]

    	#*# 2. open config. set
	Debug "INIT : Reading $configName configuration."
	if {[catch { set cfg [ctooCONFIG $this.cfg $configName] }]} {
	    error "Error loading the configuration set $configName."
	}
	if {[file exists $env(INS_ROOT)/SYSTEM/COMMON/CONFIGFILES/START.cfg]} {
	    $cfg CfgAppend START
	}

        #*# 3. load application type class types
	#
	#      The <applType>::ItclTypes procedure loads
	#      the stooType array with the class types
	#      that can be overloaded by the user extensions.
	#
	#      The <applType> uses this array to create other classes,
	#      E.g.: to create an OS object, <applType> will call:
	#        $stooType(OS) os
	#
	if {[catch { set applType [$cfg Get START.CON.TYPE] }]} {
	    error "Application type START.CON.TYPE is missing."
	}
	set typeClass [cconcat stoo $applType]
	if {[catch { [set typeClass]::ItclTypes }]} {
	    error "Wrong application type $applType."
	}

    	#*# 4. source the user extensions
	#
	#      The user tcl file should usually update the stooType array,
	#      according to the available user types.
	#
	#      E.g.: setting stooType(OS) = myOS
	#            will create an object myOS to start an OS, instead
	#            of the default stooOS.  Of course, the user tcl 
	#	     file should supply this myOS class.
	#
	set userTclFile ""
	catch { set userTclFile [$cfg Get START.CON.TCLFILE] }
	if { "$userTclFile" != "" } {
	    set userTclFile [seq_findFile [cconcat "bin/" $userTclFile]]
	    if { $debug } {
	        uplevel #0 source $userTclFile
	    } else {
	        if {[catch { uplevel #0 source $userTclFile } msg]} {
	    	    error "ERROR loading $userTclFile: $msg."
		}
	    }
	}

    	#*# 5. create the STARTUP object
	if {[catch { set startup [$stooType(STARTUP) $this.startup#auto] }]} {
	    error "Wrong application type class $stooType($applType)."
	}
	$obj AddObj $startup

    	#*# 6. call STARTUP::Config
	$startup Config $cfg

	#*# 7. delete the cfg and STARTUP objects
	set objs [$startup Contents]
	$obj DeleteObj $startup
	eval $obj AddObj $objs
	delete object $startup
	delete object $cfg

	#*# 8. restore stooType array
	array set stooType $saveStooType
    }

    #****************************************************************
    # variables
    #****************************************************************

    protected common debug 0
}


#___oOo___
