#****************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: timxDevice.tcl,v 1.15 2001/06/22 09:38:04 vltsccm Exp $" 
#
# timxDevice.tcl
# 
#  who      when     what
# -------- -------- ------------------
# imunoz   09/11/95 created for cmm
# imunoz   21/04/96 itcl2.0 port
#

#************************************************************************
#   NAME
#	TimxDevice - Class description for an "tim" device
#
#   SYNOPSIS
#	TimxDevice <object-name> ?<options>?  
#  
#   DESCRIPTION
#       TimxDevice defines the class for a "tim" Time Interfade Module
#	device.  It makes the functionality of the device and the  
#	corresponding "tim" driver available through a graphical user
#	interface.
#	
#	An object of this class is created by: 
#
#		TimxDevice <object-name> -<Option> <Value> ...
#	
#	This will create all sub-objects for user interaction and an
#	object to handle the connection to a server-task in the LCU
#	environment i.e. effectively to the physical device.
#
#   OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<object-name> configure -<Option> <Value> ...
#
#	There are no special device-specific options available.
#	Several options are inherited from InducerDevice and are therefore also
#	available within the scope of this class. See InducerDevice(n) for a 
#	detailed description of those common options.
#
#   METHODS
#	Methods represent procedures on the object and are invoked with:
#
#		<object-name> <Method> <Arguments> ...
#
#	There are no special device-specific methods available.
#
#	Several method are inherited from InducerDevice and overloaded here
#	with device-specific actions. See InducerDevice(n) for a detailed
#	description of those common methods.
#
#   PROCS
#
#   EXAMPLES
#
#   SEE ALSO
#	timxInducer(1),
#	InducerClass(n), InducerDevice(n), InducerLine(n)
#
#------------------------------------------------------------------------
#

#*# class: TimxDevice
class TimxDevice {
    
    inherit InducerDevice

    #--------------------------------------------------------------------
    # Device-specific members of TimxDevice class
    #--------------------------------------------------------------------

    #*# constructor
    constructor { args } { }

    
    #----------------------
    # Protected members
    #----------------------
    protected variable modeList {"UTC" "Local"}
    protected variable online 0
    protected variable afterCmdId ""
    #-----------------------------------------------------
    # Overloaded standard members of InducerDevice class
    #-----------------------------------------------------
    method GetVersion { } { }
    method InitDevice { } { }
    method ExitDevice { } { }
    method ResetDevice { } { }
    method Disconnect { } { }
    method Resume { } { }
    method GetDeviceId { } { }

    #-------------------------
    # Local Methods
    #-------------------------
    method updateTimeDisplay { } { }
    method setTime { } { }
    method getMode { } { }
    method setMode { } { }
    method incTime { inc } { }
    method getTimerStatus { t } { }
    method setTimeFromWs { } { }
    #--------------------------------------------------------------------
    # OPTION PANEL
    #--------------------------------------------------------------------
    method showPanel { t data } { }
}

#--------------------------------------------------------------------
# Device-specific members of TimxDevice class
#--------------------------------------------------------------------

#*# constructor
body TimxDevice::constructor { args } {
    global ::${itk_interior}
    
    set ${itk_interior}(hh)                  "00"
    set ${itk_interior}(mm)                  "00"
    set ${itk_interior}(ss)                  "00"
    set ${itk_interior}(seconds)             "0"
    set ${itk_interior}(setSeconds)          "0"
    set ${itk_interior}(mode)                "N/A"
    set ${itk_interior}(date)                "  /  /  "
    set ${itk_interior}(tzYear)              "   "
    
    $lineW configure -cPeriod 100
    # set timers Owner
    TimxTimer::setOwner $this
    
    #--------------------------
    # Find Bitmaps Files
    #--------------------------
    set syncBitmap [findVLTFile "bitmaps/timxSync.bm"]
    #-------------------------------------------
    #--------------------------
    # Create main Frames
    #--------------------------
    itk_component add F0 {
	frame $itk_interior.f0 -bd 2 -relief groove
    }
    itk_component add F1 {
	frame $itk_interior.f1 -bd 2 -relief groove
    }
    pack $itk_component(F0) $itk_component(F1) -side left -fill x 
    
    #-----------------
    # date + TZ frame
    #-----------------
    itk_component add DateTZ {
	frame $itk_component(F0).dateTz
    }
    pack $itk_component(DateTZ) -side left -anchor w
    itk_component add date {
	UifOutput $itk_component(DateTZ).date -variable ${itk_interior}(date)
    }
    pack $itk_component(date)   -side top -anchor w -pady 10 -padx 5
    
    itk_component add tzYear {
	UifOutput $itk_component(DateTZ).tzYear -variable ${itk_interior}(tzYear)
    }
    pack $itk_component(tzYear) -side top -anchor w -pady 10 -padx 5
    #--------------
    # Time Frame
    #--------------
    itk_component add hms {
	frame $itk_component(F0).hms -bd 2 
    }
    pack $itk_component(hms) -side left

    itk_component add hh {
	iwidgets::spinint $itk_component(hms).hh -range {0 23}\
		-width 3 \
		-labeltext "Hh:" \
		-textvariable ${itk_interior}(hh) \
		-increment [list $this incTime 3600] \
		-decrement [list $this incTime -3600]
    }
    setShortHelp $itk_component(hh) {Set time's  hour value}

    itk_component add mm {
	#entry $itk_component(hms).mm -textvariable ${itk_interior}(mm) \
	#	-width 3 
	iwidgets::spinint $itk_component(hms).mm -range {0 59} \
		-width 3 \
		-labeltext "Mm:" \
		-textvariable ${itk_interior}(mm) \
		-increment [list $this incTime 60] \
		-decrement [list $this incTime -60]
    }
    setShortHelp $itk_component(mm)  {Set time's minute value}


    itk_component add ss {
	#entry $itk_component(hms).ss -textvariable ${itk_interior}(ss) \
	#	-width 3 
	iwidgets::spinint $itk_component(hms).ss -range {0 59} \
		-width 3 \
		-labeltext "Ss:" \
		-textvariable ${itk_interior}(ss) \
		-increment [list $this incTime 1] \
		-decrement [list $this incTime -1]
    }
    setShortHelp $itk_component(ss) {Set time's seconds value}

    pack $itk_component(hh) -side top
    pack $itk_component(mm) -side top
    pack $itk_component(ss) -side top

    ::iwidgets::Labeledwidget::alignlabels $itk_component(hh)\
	    $itk_component(mm) $itk_component(ss)
    #------------------------------------------
    # Pure seconds set and display time Frame
    #------------------------------------------
    itk_component add timeFr {
	frame $itk_component(F0).time -bd 2 
    }
    pack $itk_component(timeFr) -side left -fill both -padx 10 
    # Pure Seconds display Frame
    itk_component add displayFr {
	frame $itk_component(timeFr).displayFr
    }
    pack $itk_component(displayFr) -side top -pady 5 
    
    itk_component add lbl {
	label $itk_component(displayFr).lbl -text "Pure Seconds:"
    }
    itk_component add seconds {
	UifOutput $itk_component(displayFr).seconds -variable ${itk_interior}(seconds)\
		-width 15 
    }
    setShortHelp $itk_component(seconds)  "Current time in seconds since 01/01/1970"
    pack $itk_component(lbl) $itk_component(seconds) -side left 
    
    itk_component add setTime {
	iwidgets::entryfield $itk_component(timeFr).setTime -labeltext "Set Time:" \
		-width 15 -textvariable ${itk_interior}(setSeconds) \
		-command [code $this setTime]
    }
    setShortHelp $itk_component(setTime) \
	    "Type a time value (in seconds) and press <Return> to Set"
    pack $itk_component(setTime) -side right
    
    itk_component add wsSetTime {
	UifButton $itk_component(F0).wsLcu -bitmap @$syncBitmap -width 80\
		-command [code $this setTimeFromWs] \
		-helptext  "Set LCU time to WS time"
    }
    setShortHelp $itk_component(wsSetTime) "Set LCU time to WS time"
    pack $itk_component(wsSetTime) -side right -padx 6
    #------------------------------
    # Second frame components
    # -----------------------------
    # Timers, set Time Mode
    #------------------------------
    
    #---------------------
    # Timer 0-4
    foreach i {0 1} {
	itk_component add F1.r$i {
	    frame $itk_component(F1).r$i
	}
	pack $itk_component(F1.r$i) -side top -anchor w -pady 4 -padx 6
    }
    #timers
    foreach i {0 1 2 3 4} {
	set row [expr {$i/3}]
	set x $itk_component(F1.r$row)
	itk_component add timer$i {
	    UifButton $x.t$i -label " Timer $i " \
		    -command [code $this getTimerStatus $i] \
		    
	}
	setShortHelp $itk_component(timer$i) "Display Status Information of Timer $i" 
	pack $itk_component(timer$i) -side left -padx 10
    }
    #----------------------------------
    # Put Radio buttons to set Tim mode
    #
    itk_component add modeLbl {
	label $itk_component(F1).lbl -text "Set Timer Mode:"
    }
    itk_component add rbLocal {
	UifRadioButton $itk_component(F1).rbLocal -label "Local" \
		-variable ${itk_interior}(mode) -onVal "Local"\
		-command [code $this setMode ]
    }
    itk_component add rbUTC {
	UifRadioButton $itk_component(F1).rbUTC -label "UTC" \
		-variable ${itk_interior}(mode) -onVal "UTC" \
		-command [code $this setMode ]
    }
    pack $itk_component(modeLbl) $itk_component(rbLocal) $itk_component(rbUTC) \
	    -side left -anchor w
    
    eval itk_initialize $args
}

#-----------------------------------------------------
# Overloaded standard members of InducerDevice class
#-----------------------------------------------------
body TimxDevice::GetVersion { } {
    debugPrint
    global timx argv0
    return "[file tail $argv0] - Version $timx(version)"
}

body TimxDevice::InitDevice { } {
    debugPrint
    
    # base behavior 
    set err [InducerDevice::InitDevice] ;# common part to open channel
    if {$err != ""} {return $err}
    
    set online 1
    $this getMode
    updateTimeDisplay
    
    return ""
}

body TimxDevice::ExitDevice { } {
    debugPrint
    after cancel $afterCmdId
    return [InducerDevice::ExitDevice]
}

body TimxDevice::ResetDevice { } {
    debugPrint
    return [$lineW Ioctl RESET 0]
}
body TimxDevice::Disconnect { } {
    global inducer
    set online 0
    set inducer(fOnline) 0
}
body TimxDevice::Resume { } {
    global inducer
    set inducer(fOnline) 1
    set online 1
    updateTimeDisplay
}

body TimxDevice::GetDeviceId { } {
    debugPrint
    return "<unknown>"
}

#-------------------------
# Local Methods
#-------------------------
body TimxDevice::updateTimeDisplay { } {
    global ::${itk_interior}
    global inducer
    
    if {$online==1 } {
	set x 0
	set r [$lineW Ioctl GET_TIME "0;0"]
	
	scan [lindex $r 0] "%ld;%ld" ${itk_interior}(seconds) x
	set date "[fmtclock [set ${itk_interior}(seconds)]]"
	set  ${itk_interior}(date) [lrange $date 0 2]
	set  ${itk_interior}(tzYear) [lrange $date 4 end]
	
	set  time [split [lindex $date 3] :]
	set ${itk_interior}(hh) [lindex $time 0]
	set ${itk_interior}(mm) [lindex $time 1]
	set ${itk_interior}(ss) [lindex $time 2]
    }
    set afterCmdId [after 1000 [code $this updateTimeDisplay]]
}

body TimxDevice::setTime { } {
    global ::${itk_interior}
    if {$online == 1} {
	set r [$lineW Ioctl SET_TIME "[set ${itk_interior}(setSeconds)];0"]
    }
}

body TimxDevice::getMode { } {
    global ::${itk_interior}
    set r [sendServer $itk_option(-vEnv) lccServer TIMGMOD ""]
    set ${itk_interior}(mode) $r
}

body TimxDevice::setMode { } {
    global ::${itk_interior}

    if {$online == 1} {
	set r [sendServer $itk_option(-vEnv)\
		lccServer TIMSMOD [set ${itk_interior}(mode)]]
    }
    getMode
}

body TimxDevice::incTime { inc } {
    global ::${itk_interior}

    if {$online==1} {
	set ${itk_interior}(seconds) \
		[expr [set ${itk_interior}(seconds)] + $inc ]
	$lineW Ioctl SET_TIME "[set ${itk_interior}(seconds)];0"
    }
}

body TimxDevice::getTimerStatus { t } {
    # Get Timer Status
    if {$online ==1 } {
	set r [sendServer $itk_option(-vEnv) lccServer TIMGTST "$t"]
	regsub -all {\"} $r " " s        ; #"
	set st [split $s ,]
	showPanel $t $st
    }
}
body TimxDevice::setTimeFromWs { } {
    global ::${itk_interior}
    set ${itk_interior}(setSeconds) [getclock]
    setTime
}
#--------------------------------------------------------------------
# OPTION PANEL
#--------------------------------------------------------------------
body TimxDevice::showPanel { t data } {
    if {$online == 1} {
	if {[itcl::find objects ::.timer$t] == ""} {
	    TimxTimer .timer$t -number $t
	}
	.timer$t display $data
    } 
}
#
# ___oOo___
