#*********************************************************************************
#
# E.S.O. - VLT project
#
# "@(#) $Id: timxTimer.tcl,v 1.15 2001/06/22 09:38:05 vltsccm Exp $" 
#
# timxTimer.tcl
# 
#  who      when     what
# -------- -------- ------------------
# imunoz   09/11/95 created for cmm
# imunoz   22/04/96 itcl2.0 port
#

#************************************************************************
#   NAME
#	TimxTimer - Class to display status window of a tim's device timer. 
#
#   SYNOPSIS
#	TimxTimer <object-name> ?<options>?  
#  
#   DESCRIPTION
#       TimxTimer defines the class for a tim's timer status display 
#       window.
#
#	An object of this class is created by: 
#
#		TimxTimer <object-name> -<Option> <Value> ...
#	
#	This will create the TimxTimer object and initialize its internal 
#       variables
#
#   OPTIONS
#	Options represent public variables of the object and can be set with:
#
#		<object-name> configure -<Option> <Value> ...
#
#       The following option is publicly available :
#
#       number -  tim's device timer number.
# 
#   METHODS
#	Methods represent procedures on the object and are invoked with:
#
#		<object-name> <Method> <Arguments> ...
#
#       The following method is publicly available :
#
#       display { data } - displays timer's status information as is
#       defined in "data", this is a list with the following elements:
#       status, startStatus, timerMode, time of last interrupt and
#       time of next interrupt.
#	
#   PROCS
#	Procs are invoked with:
#
#		<object-name> <Proc> <Arguments> ...
#
#	The following procs are publicly available:
#
#	setOwner <object-name>	- set the object-name of the "owner"
#               of all the timers, TimxTimer objects invokes its
#               owner's procedure getTimerStatus when asking for an
#		update of the status display.
#
#   CAUTIONS
#	The class uses common variables.
#	All timers must belong to the same device.
#	In this implementation it is not possible to have timers
#	that belong to different devices at the same time.
#
#   EXAMPLES
#
#   SEE ALSO
#	timxDevice(n),
#	InducerClass(n), InducerDevice(n), InducerLine(n)
#
#------------------------------------------------------------------------
#
#*# class Timxtimer 
class TimxTimer {
    inherit itk::Toplevel
    
    constructor { args } { }
    itk_option define -number number Number 0
    common owner ""
    #---------------------
    # Procs
    #---------------------
    proc setOwner { o } { }
    #------------------------------------
    # Methods
    #------------------------------------
    method display { data } {}
} 

body TimxTimer::constructor { args} {
    global ::${itk_interior}
    
    set ${itk_interior}(status)       ""
    set ${itk_interior}(startStatus)  ""
    set ${itk_interior}(mode)         ""
    set ${itk_interior}(lastInt)      ""
    set ${itk_interior}(nextInt)      ""

    eval itk_initialize $args

   # Build short help area
    itk_component add sh {
	UifShortHelp  $itk_interior.sh 
    }  {
	keep -background  -cursor
    }
    pack $itk_component(sh) -side bottom -anchor w -fill x
    
    # status frame
    itk_component add f0 {
	frame $itk_interior.f0
    }
    itk_component add lbl0 {
	label $itk_component(f0).l -text "Timer Status:" -width 15
    }
    itk_component add o0 {
	UifOutput $itk_component(f0).o -variable ${itk_interior}(status)
    }
    pack $itk_component(f0) -side top -anchor w -padx 20
    pack $itk_component(lbl0) $itk_component(o0)\
		-side left -anchor w -fill x -expand yes
    
    # start status frame
    itk_component add f1 {
	frame $itk_interior.f1
    }
    itk_component add lbl1 {
	label $itk_component(f1).l -text "Start Status:" -width 15
    }
    itk_component add o1 {
	UifOutput $itk_component(f1).o -variable ${itk_interior}(startStatus)
    }
    pack $itk_component(f1) -side top -anchor w -padx 20
    pack $itk_component(lbl1) $itk_component(o1)\
	    -side left -anchor w -fill x -expand yes
    
    # mode
    itk_component add f2 {
	frame $itk_interior.f2
    }
    itk_component add lbl2 {
	label $itk_component(f2).l -text "        Mode:" -width 15
    }
    itk_component add o2 {
	UifOutput $itk_component(f2).o -variable ${itk_interior}(mode) -width 15
    }
    pack $itk_component(f2) -side top -anchor w -padx 20
    pack $itk_component(lbl2) $itk_component(o2)\
	    -side left -anchor e -fill x -expand yes
    
    # last int
    itk_component add f3 {
	frame $itk_interior.f3
    }
    itk_component add lbl3 {
	label $itk_component(f3).l -text "Last Interrupt:" -width 15
    }
    itk_component add o3 {
	UifOutput $itk_component(f3).o -variable ${itk_interior}(lastInt) -width 25
    }
    pack $itk_component(f3) -side top -anchor w -padx 20
    pack $itk_component(lbl3) $itk_component(o3)\
	    -side left -anchor w -fill x -expand yes
    # next int
    itk_component add f4 {
	frame $itk_interior.f4
    }
    itk_component add lbl4 {
	label $itk_component(f4).l -text "Next Interrupt:" -width 15
    }
    itk_component add o4 {
	UifOutput $itk_component(f4).o -variable ${itk_interior}(nextInt) -width 25
    }
    pack $itk_component(f4) -side top -anchor w -padx 20
    pack $itk_component(lbl4) $itk_component(o4)\
	    -side left -anchor w -fill x -expand yes
    # Update Button
    itk_component add update {
	UifButton $itk_interior.u -label " Update " \
		-command [code $owner getTimerStatus $itk_option(-number)] \
		-helptext "Update Panel contents"
    }
    pack $itk_component(update) -side bottom -pady 10

    set self [namespace tail $this]
    wm title $self "[wm title .] - Timer $itk_option(-number)"    
}
    
body TimxTimer::setOwner { o } {
    set owner $o
}

#------------------------------------
# Methods
#------------------------------------
body TimxTimer::display { data } {
    
    global ::${itk_interior}
    
 
    # Set data
    set ${itk_interior}(status) [lindex $data 0]
    set ${itk_interior}(startStatus) [lindex $data 1]
    set ${itk_interior}(mode) [lindex $data 2]
    set ${itk_interior}(lastInt) [lindex $data 3]
    set ${itk_interior}(nextInt) [lindex $data 4]

    set self [namespace tail $this]
    raise $self
}