# -*- tcl -*-
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplDCS.tcl,v 1.64 2005/10/07 22:29:46 vltsccm Exp $"
#
# tplDCS.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# alongino  24/10/00  created
#
#************************************************************************
#   NAME
#   	tplDCS - DCS calss interface for templates.
# 
#   SYNOPSIS
#	tplDCS dcs <dbAlias> <subsystem> [<detIndex>] [<imageScale>]
# 
#   DESCRIPTION
#	DCS interface for BOB templates.
#
#   PUBLIC METHODS
#	tplDCS <dbAlias> name subsystem [<detIndex>] [<imageScale>]
#	    dbAlias	- alias of the DCS point
#	    subsystem	- name of OS subsystem for this DCS
#	    detIndex	- value of DETi index (default "" = no index)
#	    imageScale	- value returned by ImageScale (deafult -1)
#	SetupKeys [<filterList>]
#	    Return all INS keywords except keywords present in <filterList>
#	SetupMode [<speed>]
#	    Set the mode according to the supplied parameters.
#           speed = readout speed (default "" = keep it unchanged)
#	SetupFullFrame
#	    Set the read-out for full frame (no window)
#       ExpTime [<dit>] [<ndit>]
#	    Set the exposure time and return the corresponding FITS keywords.
#           dit  Intergation time (default -1 = keep it unchanged)
#           ndit Number of integrations (default -1 = keep it unchanged)
#       Repeat [<nrep>]
#	    Set the FITS keywords definiting the number of repeated exposures
#           nrep number of repeated exposures (default 0 = infinite loop)
#	Mode [<speed>]
#	    Return the mode according to the supplied FITS keywords.
#           speed = readout speed (default "" = keep it unchanged)
#	Modes
#	    Return a list of accepted modes (DET.READ.SPEED values).
#	Filename
#	    Return the image filename.
#	ImageScale
#	    Return the image scale that should be used to display
#	    an image with MIDAS (see tplMIDAS LoadImage).
#	'DET'
#	    Returns 'DETi'.
#	Name
#	    Returns the detector name.
#	Subsystem
#	    Returns the DCS subsystem name, as known to OS.
#	ExpStatus
#	    Returns the current exposure status.
#	IsImageReal
#	    Return 1 if image is read from the chip, otherwise 0
#	IsWsOnlyUsed
#	    Return 1 if only the WS part of DCS is used, otherwise 0
#	IsLoopRunning
#	    Return 1 if exposure loop is running, otherwise 0
#	IsLoopSupported
#	    Return 1 if exposure looping is supported, otherwise 0
#	IsFileNameFixed
#	    Return 1 if the image file name cannot be specified by the user, otherwise 0
#	Wait [<expStatus>]
#	    Waits till the exposure status reaches the specified value (default "-reading")
#           Values supported: "-integrating" 
#                             "-reading" 
#                             "-transfering" 
#                             "-reading&transfering" 
#                             "-finished"
#                             "-loop_running"
#
#   CAUTIONS
#
#   EXAMPLES
#       see xxoseq module
#
#   SEE ALSO
#	tplACE tplFIERA tplIRACE
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#########################################################################
#
class tplDCS {

    #--------------------------------------------------------------------
    # constructor
    #
    constructor { dbAlias subsys {index ""} {scale -1} } {
	global env

	#
	# 1. set the DCS name, index and scale
	#
	set detName $dbAlias
        set subsysName $subsys
        set dbExpStatus "<alias>$detName:exposures:exposure_1.expStatus"    
	set imageScale $scale
	if { $index == 0 } {
	    set det "DET"
	} else {
	    set det "DET$index"
	}

        #
        # 2. set exposure status list
        #
        set expStates { {-integrating 4}          \
	                {-reading 16}             \
	                {-transfering 64}         \
                        {-reading&transfering 80} \
                        {-finished 128}           \
                        {-loop_running 1024}      }
    }


    #--------------------------------------------------------------------
    # method SetupKeys
    # Return all DET.*.* keywords defined except those specified in filterList
    #
    method SetupKeys { {filterList {}} } {
	upvar $det $det

        set buffer ""
        set keyList [array names $det]
        foreach item $keyList {
            set addItem 1
            foreach filter $filterList {            
	        if { [string match $filter $item] } {
                    set addItem 0
                    break
		}
	    }
	    if { $addItem } {
                append buffer "$det.$item [set ${det}($item)] "
	    }
        }
	return "$buffer"
    }


    #--------------------------------------------------------------------
    # method SetupFullFrame
    #
    # Set the read-out for full frame (no window)
    #
    method SetupFullFrame { } {
	upvar $det $det

        set buffer ""
        for { set i 1 } { $i <= 2 } { incr i } {
            set item "WIN$i.ST"
            set ${det}($item) "F"
            append buffer "$det.$item [set ${det}($item)] "
	}
	return "$buffer"
    }
    

    #--------------------------------------------------------------------
    # method SetupMode
    #
    # set the mode according to the supplied FITS keywords.
    #
    method SetupMode { { mode ""} } {
	upvar $det $det

	#
	# 1. get detector mode and return it
	#
	set value [Mode $mode]
	tplLog "$det.READ.CLKIND = $value."  
	return "$det.READ.CLKIND $value "
    }
    

    #--------------------------------------------------------------------
    # method ExpTime
    #
    # set the exposure time and return the corresponding FITS keywords.
    #
    method ExpTime { {expTimeSec -1} {ndit -1} } {
	upvar $det $det
        
        if {$expTimeSec >= 0} { set ${det}(WIN1.UIT1) $expTimeSec }
	return "$det.WIN1.UIT1 [set ${det}(WIN1.UIT1)]"
    }
    

    #--------------------------------------------------------------------
    # method Repeat
    #
    # return the FITS keywords defining a loop
    #
    method Repeat { { n 0 } } {
	upvar $det $det

	return "$det.EXP.NREP $n"
    }
    

    #--------------------------------------------------------------------
    # method Mode
    #
    # return the detector mode according to the supplied FITS keywords.
    #
    method Mode { { mode ""} } {
	upvar $det $det

	#
	# 1. default: get DET(READ.SPEED)
	#
        if { $mode == "" } {
	    set mode [set ${det}(READ.SPEED)]
	}

	#
	# 2. search detector mode
	#
	set index [lsearch $modes $mode]
	if { $index == -1 } {
	    error "$det.FRAM.SPEED $mode not supported."
	}
	set value [lindex $clkinds $index]

	return $value
    }
    

    #--------------------------------------------------------------------
    # method Modes
    #
    method Modes {} {
	return $modes
    }


    #--------------------------------------------------------------------
    # method Filename
    #
    method Filename {} {
	return [seq_dbReadSymbolic "<alias>$detName:exposures:exposure_1:transfer.fileNameUnComp"]
    }


    #--------------------------------------------------------------------
    # method ImageScale
    #
    method ImageScale {} {
	return $imageScale
    }


    #--------------------------------------------------------------------
    # method DET
    #
    method DET {} {
	return $det
    }


    #--------------------------------------------------------------------
    # method Name
    #
    method Name {} {
	return $detName
    }

    #--------------------------------------------------------------------
    # method Subsystem
    #
    method Subsystem {} {
	return $subsysName
    }

    #--------------------------------------------------------------------
    # method ExpStatus
    #
    method ExpStatus {} {
	return [seq_dbReadSymbolic "$dbExpStatus"]
    }

    #--------------------------------------------------------------------
    # method IsImageReal
    #
    method IsImageReal {} {
        set fromchip 1
	set simulation [seq_dbReadSymbolic "<alias>$detName.opMode"]
        if {$simulation != 2} { set fromchip 0 }
	return $fromchip
    }

    #--------------------------------------------------------------------
    # method IsWsOnlyUsed
    #
    method IsWsOnlyUsed {} {
        set wsonly 0
	set simulation [seq_dbReadSymbolic "<alias>$detName.opMode"]
        if {$simulation == 3} { set wsonly 1 }
	return $wsonly
    }

    #--------------------------------------------------------------------
    # method IsLoopRunning
    #
    method IsLoopRunning {} {
        set isRunning 0
	set expStatus [ExpStatus]
        if {$expStatus >= $loopRunning} { set isRunning 1 }
        return $isRunning
    }

    #--------------------------------------------------------------------
    # method IsLoopSupported
    #
    method IsLoopSupported {} {
        return 1
    }

    #--------------------------------------------------------------------
    # method IsFileNameFixed
    #
    method IsFileNameFixed {} {
        return 0
    }

    #--------------------------------------------------------------------
    # method Wait
    #
    method Wait { { status "-reading" } } {
      global env

    set msg [crange $status 1 end ]
    set waitState -1   
    foreach expStatus $expStates {
        set expStatusString [lindex $expStatus 0]
        set expStatusValue [lindex $expStatus 1]
	if { "$status" == "$expStatusString" } {
            set waitState $expStatusValue
        }
	if { "$expStatusString" == "-loop_running" } {
            set expStatusLoop $expStatusValue
        } elseif { "$expStatusString" == "-finished" } {
            set expFinished $expStatusValue
        }
    }
    if {$waitState < 0} { error "Invalid exposure status: $msg" }


    tplLog "DCS Waiting for $msg status....."
    tplLog "______________________________________________________"

    set anExpstatus [ExpStatus]
    while { (($anExpstatus < $waitState) || \
            (($anExpstatus >= $expStatusLoop) && ($waitState == $expFinished)))} {
        sleep 2	   	      
        update
        set anExpstatus [ExpStatus]
        checkAbortFlag         
    }
    tplLog "DCS reached status $msg"
}



    #--------------------------------------------------------------------
    # variables
    #
    protected variable modes {}
    protected variable clkinds {}

    protected variable detName ""
    protected variable subsysName ""
    protected variable det ""
    protected variable imageScale -1
    protected variable dbExpStatus
    protected variable expStates {}
    protected variable loopRunning 1024
    protected variable simulation
}


#########################################################################
# ___oOo___

