# -*- tcl -*-
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplDR.tcl,v 1.64 2005/10/07 22:29:47 vltsccm Exp $"
#
# tplDR.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# dpopovic  31/05/05  VLTSW20050181 - new parameter in method TFocus
# alongino  24/10/00  created
#

#************************************************************************
#   NAME
#   	tplDR - Data Reduction class for templates.
# 
#   SYNOPSIS
#	tplDR dr [<unit>] [<startFlag>]
#
#   DESCRIPTION
#	Data Reduction procedures for BOB templates.
#       This Base Class uses MIDAS
#
#   PUBLIC METHODS
#       tplDR [<unit>] [<startFlag>]
#           unit MIDAS unit to be used. Default 11
#           start a session, if not running yet. Default: 1 (yes)
#       Start
#           Start DR session, if not running yet
#       Stop
#           Stop DR session, if not running yet
#	MeanLevel <file> [<procedure>]
#	    Return the average value of the last exposure from file <file>
#           <procedure> indicates which procedure is used (default tplMeanLevel)
#	LoadImage <file> [<scale>] [<procedure>]
#	    Load an image, contained in <file>, in MIDAS.
#           <scale> = image scale (default -2)
#           <procedure> indicates which procedure is used (default tplLoadImage)
#	TFocus <file> [<procedure>] [<params>]
#	    Run the focus batch and return the best focus value.
#           <procedure> indicates which procedure is used (default tplTFocus)
#           <params>    indicates which input parameters to pass to the procedure
#                       (default "0. 1." to match the default procedure)
#	IsSimulated
#	    Return flag indicating if DR is simulated or not (see DEBUG_MIDAS)
#
#   ENVIRONMENT
#	DEBUG_MIDAS - if set to non-zero, then no commands are
#		      sent to MIDAS. Only logs are printed.
#
#   CAUTIONS
#
#   EXAMPLES
#       see xxoseq module
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#########################################################################
# MIDAS procedures
#
class tplDR {

    #--------------------------------------------------------------------
    # constructor
    #
    constructor { {drUnit 11} {start 1} } {
	global env

	set host "[exec hostname]"
	set envName $env(RTAPENV)
        set insroot "$env(INS_ROOT)"
        set dataPath "$insroot/SYSTEM/DETDATA"
        set unit $drUnit
	catch { set debug $env(DEBUG_MIDAS) }
        #
        # Start now the DR session
        #
        if { $start } {
            Start
        }
    }

    #--------------------------------------------------------------------
    # method Start
    #
    # Start Data Reduction, if not running yet
    #
    method Start { } {

        if { !$debug } {
            if {![catch { set pid [eval "exec ccdMidasBgStart.sh $unit >/dev/null & "] } result]} {
                wait $pid
	    }
	}
    }

    #--------------------------------------------------------------------
    # method Stop
    #
    # Stop Data Reduction, if not running yet
    #
    method Stop { } {

        if { !$debug } {
            if {![catch { set pid [eval "exec ccdMidasBgStop.sh $unit >/dev/null & "] } result]} {
                wait $pid
            }
        }
    }

    #--------------------------------------------------------------------
    # method LocateProc
    #
    # Locate data reduction procedure file in VLT standard path
    #
    method LocateProc { drProc } {
	global env

        set fullPath ""
        set drInsrootPath "$insroot/SYSTEM/COMMON/MIDAS/PROCS"
        set drIntrootPath ""
        set drVltrootPath ""
        set drProcPath "$drInsrootPath/$drProc.prg"
        if { ![file isfile $drProcPath] } {
            if { [info exists env(INTROOT)] } {
                set drIntrootPath "$env(INTROOT)/config/INS_ROOT/SYSTEM/COMMON/MIDAS/PROCS"
                set drProcPath "$drIntrootPath/$drProc.prg"
	    }
            if { ![file isfile $drProcPath] } {
                set drVltrootPath "$env(VLTROOT)/config/INS_ROOT/SYSTEM/COMMON/MIDAS/PROCS"
                set drProcPath "$drVltrootPath/$drProc.prg"
	    }
            if { ![file isfile $drProcPath] } {
	        error "ERROR File $drProc.prg not found in $drInsrootPath $drIntrootPath $drVltrootPath"
	    }
        }

        return "$drProcPath"
    }


    #--------------------------------------------------------------------
    # method MeanLevel
    #
    # get average value of the last exposure
    #
    method MeanLevel { file { drProc tplMeanLevel } } {
	global env

	#
	# 1. call MIDAS
	#
        set drProcPath [LocateProc $drProc]
	set cmd " MIDAS,$unit,$host,\"@@ $drProcPath $dataPath/$file\" "
	tplLog "Sending image to MIDAS (cmd='$cmd')." 
	if { $debug } {
	    tplLog "DEBUG: Would send pco FORWMSG $cmd."
	    set errFlag 0
	} else {
	    set cmdId [seq_msgSendCommand $envName pco FORWMSG $cmd]
	    set reply [seq_msgRecvReply -last -60000 $cmdId errFlag]
	}
	if { $errFlag != 0 } {
	    bell
	    bell
	    bell
	    error "ERROR calling MIDAS ($reply)."
	}
    
	#
	# 2. get MIDAS result
	#
	set cmd	" MIDAS,$unit,$host,outputr,REAL,0,8,1 "
	if { $debug } {
	    tplLog "DEBUG: Would send pco READKEY $cmd."
	    set errFlag 0
	    set reply "0,0.0"
	} else {
	    set cmdId [seq_msgSendCommand $envName pco READKEY $cmd]
	    set reply [seq_msgRecvReply -last -60000 $cmdId errFlag]
	}
	if { $errFlag != 0 } {
	    bell
	    bell
	    bell
	    error "ERROR reading from MIDAS ($reply)."
	}
	tplLog "MIDAS answer: $reply."

	#
	# 3. return the mean level
	#
	scan $reply "%d,%f" a x
	return $x
    }


    #--------------------------------------------------------------------
    # method LoadImage
    #
    # Send an image to MIDAS.
    #
    method LoadImage { file {scale -2} { drProc tplLoadImage } } {
	global env

	#
	# 1. call MIDAS
	#
        set drProcPath [LocateProc $drProc]
	set cmd " MIDAS,$unit,$host,\"@@ $drProcPath $dataPath/$file $scale\" "
	tplLog "Display image in MIDAS (cmd='$cmd')."
	if { $debug } {
	    tplLog "DEBUG: Would send pco FORWMSG $cmd."
	    set errFlag 0
	} else {
	    set cmdId [seq_msgSendCommand $envName pco FORWMSG $cmd]
	    set reply [seq_msgRecvReply -last -100000 $cmdId errFlag]
	}
	if { $errFlag != 0 } {
	    bell
	    bell
	    bell
	    error "ERROR calling MIDAS ($reply)."
	}
    }


    #--------------------------------------------------------------------
    # method TFocus
    #
    # Run the focus batch and return the best focus value.
    #
    method TFocus { file {drProc tplTFocus} {drProcParams "0. 1."} } {
	global env

	#
	# 1. alert the user
	#
	bell
	bell
	tplLog "------------------------------------------------------"
	tplLog "Running MIDAS through focus batch on $file."
	tplLog "Click (after resizing the box - cursor keys) on the image in a sequence."
	tplLog "After the last selection quit the cursor."
	tplLog "You have a maximum of 120 seconds to complete this task."
	tplLog ""

	#
	# 2. call MIDAS
	#
        set drProcPath [LocateProc $drProc]
	# TFocus uses the loaded image
	set cmd " MIDAS,$unit,$host,\"@@ $drProcPath $drProcParams\" "
	if { $debug } {
	    tplLog "DEBUG: Would send pco FORWMSG $cmd."
	    set errFlag 0
	} else {
	    set cmdId [seq_msgSendCommand $envName pco FORWMSG $cmd]
	    set reply [seq_msgRecvReply -last -150000 $cmdId errFlag]
	}
	if { ($errFlag != 0) } {
	    bell
	    bell
	    bell
	    error "ERROR calling MIDAS ($reply)."
	}
    
	#
	# 3. get MIDAS result
	#
	set cmd	" MIDAS,$unit,$host,outputr,REAL,0,1,3 "
	if { $debug } {
	    tplLog "DEBUG: Would send pco READKEY $cmd."
	    set errFlag 0
	    set reply "1,2.0,3.0,4.0"
	} else {
	    set cmdId [seq_msgSendCommand $envName pco READKEY $cmd]
	    set reply [seq_msgRecvReply -last -60000 $cmdId errFlag]
	}
	if { $errFlag != 0 } {
	    bell
	    bell
	    bell
	    error "ERROR reading from MIDAS ($reply)."
	}

	#
	# 4. return the mean level
	#
	scan $reply "%d,%f,%f,%f" a1 x y z
	return "$x $y $z"
    }


    #--------------------------------------------------------------------
    # method IsSimulated
    #
    # Return flag indicating if DR is simulated or not (see DEBUG_MIDAS)
    #
    method  IsSimulated {  } {
        return $debug
    }


    #--------------------------------------------------------------------
    # variables
    #

    private variable host     "unknown"
    private variable envName  ""
    private variable insroot  ""
    private variable dataPath ""
    private variable unit     0
    public  variable debug    0
}


#########################################################################
# ___oOo___
