#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplFITS.tcl,v 1.64 2005/10/07 22:29:48 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-10-07  AddFits,AddFitsCatg: store numbers correctly.
# rschmutz  2005-02-05  created
#

#************************************************************************
#   NAME
#	tplFITS - class to access a FITS file header
# 
#   SYNOPSIS
#	tplFITS <filename> [-dic "<dict1> ... <dictN>"]
#	fits Name
#	fits GetValue <key> [<fits_ext>]
#	fits AddFits <key1> <value1> ... <keyN> <valueN>
#	fits AddFitsCatg <catg1> ... <catgN>
#	fits AddComment <text>
#	fits Save
#	fits Archive
#
#   DESCRIPTION
#     tplFITS allows to read/write to the FITS header of a
#     FITS file.  The file is by default searched in the
#     $INS_ROOT/SYSTEM/DETDATA directory.
#
#     Note that normally OS is responsible of updating the 
#     image FITS headers, therefore this class should not 
#     be used to update normal image files.  This class is
#     provided to process *.fits files created by technical
#     procedures (non-image files).
#
#     tplFITS (constructor)
#	Create a tplFITS instance, reading in the FITS header
#	of file <filename>.  The dictionaries located in 
#	$INS_ROOT, as well as dictionaries TPL, OBS and DPR
#	are loaded by default.  Any other dictionaries have
#	to be specified with the -dic option.
#
#     Name
#	Returns the filename.
#
#     GetValue
#	Returns the value of the FITS keyword <key> located
#	in the main FITS header. If <fits_ext> is specified,
#	then the keyword is searched in the corresponding
#	FITS header extension.
#
#     AddFits
#	Add/change the keywords <keyN> to <valueN>.
#
#     AddFitsCatg
#	Add/change all the keywords of the categories <catgN>.
#	The values are obtained from the Tcl array of the same
#	name, e.g. category OBS from $OBS(ID), $OBS(NAME), etc.
#	Empty array entries are ignored (i.e. no empty values
#	are added to the FITS header).  If the category is TPL,
#	then the following keywords are always ignored:
#	TPL.INSTRUM, TPL.EXECTIME, TPL.REFSUP, TPL.TYPE.
#	
#     AddComment
#	Add a comment to the FITS header.
#
#     Save
#	Save the changes to the FITS file.
#
#     Archive
#	Save and archive the FITS file (with sendFiles).
#	
#   CAUTIONS
#	This class should not be used to modify normal images.
#	Use the OS ADDFITS command instead.
#
#   EXAMPLES
#
#   SEE ALSO
#	tplLogFitsDictionary
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#****************************************************************
# class tplFITS
#
class tplFITS {

    #************************************************************
    # constructor $filename [-dic "<dict1> ... <dictN>"]
    #
    constructor { filename args } {

	#= 1. load arguments
	set name $filename
	eval configure $args

	#= 2. load FITS dictionaries
	incr index
	set oslxId "img$index"
	seq_oslxCmd FITS -loadDictionary $oslxId all
	seq_oslxCmd FITS -loadDictionary $oslxId TPL merge
	seq_oslxCmd FITS -loadDictionary $oslxId OBS merge
	seq_oslxCmd FITS -loadDictionary $oslxId DPR merge
	foreach dicName $dic {
	    seq_oslxCmd FITS -loadDictionary $oslxId $dic merge
	}

	#= 3. load FITS file
	seq_oslxCmd FITS -loadHeader $oslxId $name
	set modified 0
    }

    #************************************************************
    # destructor
    #
    destructor {
	#seq_oslxCmd FITS -clearInstance $oslxId
    }


    #************************************************************
    # method Name - return FITS filename
    #
    method Name {} {
	return $name
    }

    #************************************************************
    # method Save - change filename, save FITS file
    #
    method Save { {filename ""} } {

	#= 1. set new filename
	if { $filename != "" && $filename != $name } {
	    set name $filename
	    set modified 1
	}

	#= 2. save FITS file
	if { $modified } {
	    seq_oslxCmd FITS -save $oslxId $name
	    set modified 0
	}
    }

    #************************************************************
    # method AddFits - add FITS keywords
    #
    method AddFits { args } {
	set modified 1
	foreach {key val} $args {
            if { [string first " " $val] != -1 } {
                seq_oslxCmd FITS -createStoreFits $oslxId $key "\"$val\""
            } else {
                seq_oslxCmd FITS -createStoreFits $oslxId $key $val
            }
	}
    }

    #************************************************************
    # method AddFitsCatg - add a complete FITS category
    #
    method AddFitsCatg { args } {
	eval tplSeeVars $args

        set tplIgnoreParams {INSTRUM EXECTIME REFSUP TYPE}
	foreach catg $args {
	    foreach {param val} [array get $catg] {
		if { $catg == "TPL" } {
		    if { [lcontain $tplIgnoreParams $param] } {
			continue
		    }
		}
		if { $val != "" } {
		    set modified 1
		    set key [cconcat $catg "." $param]
                    if { [string first " " $val] != -1 } {
                        seq_oslxCmd FITS -createStoreFits $oslxId $key "\"$val\""
                    } else {
                        seq_oslxCmd FITS -createStoreFits $oslxId $key $val
                    }
		}
	    }
	}
    }

    #************************************************************
    # method AddComment - add a comment
    #
    method AddComment { txt } {
	seq_oslxCmd FITS -addComment $oslxId "\"$txt\""
    }

    #************************************************************
    # method GetValue - get a FITS value
    #
    method GetValue { key {fits_ext 0} } {
	global seq_errLogging

	set prevErrLogging $seq_errLogging
	set seq_errLogging off
	if { $fits_ext != 0 } {
	    set val [seq_oslxCmd FITS -getValue $oslxId $key fits_ext $fits_ext]
	} else {
	    set val [seq_oslxCmd FITS -getValue $oslxId $key]
	}
	if { "[csubstr $val 0 1]" == "\"" && "[csubstr $val end 1]" == "\"" } {
	    set val [csubstr $val 1 [expr {[clength $val] - 2}]]
	}
	set seq_errLogging $prevErrLogging

	return $val
    }

    #********************************************************************
    # method Archive - archive the FITS file
    #
    method Archive {} {
	global env

	#= 1. save file
	Save

	#= 2. archive file
	set file $name
	if { [file pathtype $file] == "relative" } {
	    set file [file join $env(INS_ROOT) SYSTEM DETDATA $file]
	}
	if { [file isfile $file] == 0 } {
	    error "File $file not found."
	}
	chmod 0444 $file
	set arclink [file join $env(INS_ROOT) SYSTEM ARCDATA [file tail $file]]
	link -sym $file $arclink
    }


    #************************************************************
    # attributes
    #
    public variable dic ""

    protected variable name     ""
    protected variable oslxId   ""
    protected variable modified 0

    private common index 0
}


# ___oOo___
