# -*- tcl -*-
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplICS.tcl,v 1.64 2005/10/07 22:29:47 vltsccm Exp $"
#
# tplICS.tcl
#
# who       when        what
# --------  ----------  ----------------------------------------------
# alongino  24/10/00    created
# dpopovic  2003-11-14  made use of <osDbAlias> optional (SPR 20030456)
# dpopovic  2004-08-06  Addition of IsLcuSimulated proc (VLTSW20040052)
#
#************************************************************************
#   NAME
#   	tplICS - ICS class interface for templates.
# 
#   SYNOPSIS
#	tplICS ics <icsDbPoint> <osDbAlias> <osSubsysName> <icsProcName>
#
#   DESCRIPTION
#	ICS interface for BOB templates.
#	
#   PUBLIC METHODS
#       tplICS <icsDbPoint> <osDbAlias> <osSubsysName> <icsProcName>
#           icsDbPoint ICS OLDB root point in the Instrument Environment
#           osDbAlias  alias of the OS point in the Instrument Environment
#           osSubsysName  name of ICS subsystem as known to OS
#           icsProcName name of the ICS WS front-end process (e.g. xxiControl)
#	IsUsed
#	    Returns 1 if ICS is available, 0 otherwise.
#       IsLcuSimulated
#	    Returns 1 if ICS is in LCU_SIM, 0 otherwise.
#	EnvName
#	    Returns the ICS environment
#	ProcName
#	    Returns the ICS process
#	NoInternalFlatField
#	    Returns 1 if the instrument does not support internal
#	    flat-fields.
#	SetupKeys [<filterList>]
#	    Return all INS keywords except keywords present in <filterList>
#	SetupMode [<mode>]
#	    Set and return INS.MODE
#	SetupLamp
#	    Setup and return the FITS keyword indicating the lamp on.
#
#   CAUTIONS
#
#   EXAMPLES
#       see xxoseq module
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#########################################################################
# ICS procedures
#
class tplICS {

    #--------------------------------------------------------------------
    # constructor
    #
    constructor { icsDbPoint osDbAlias osSubsysName icsProcName } {
	global env

        set icsDbRoot $icsDbPoint
        set osAlias $osDbAlias
        set subsysId $osSubsysName
        set procName $icsProcName

	#
	# check if ICS is used
	#
	if {[string equal $osAlias ""]} {
	    # assume availability
	    set isUsed 1
	} else {
	    set access [seq_dbReadSymbolic "<alias>$osAlias:subsystems:$subsysId.access"]
	    if { $access == 0 } {
		set isUsed 1
	    } else {
		set isUsed 0
		return
	    }
	}

	#
	# get the ICS environment
	#
	set envName $env(RTAPENV)

    }

    #--------------------------------------------------------------------
    # method IsUsed
    #
    method IsUsed {} {
	return $isUsed
    }

    #--------------------------------------------------------------------
    # method IsLcuSimulated
    #
    # VLTSW20040052
    #
    method IsLcuSimulated {} {
	set flag [seq_dbReadSymbolic "$icsDbRoot:PROCESSES:WS:icsControl.opMode"]

	if {$flag == 1} {
	    return 1
	} else {
	    return 0
	}
    }

    #--------------------------------------------------------------------
    # method EnvName
    #
    method EnvName {} {
	return $envName
    }


    #--------------------------------------------------------------------
    # method ProcName
    #
    method ProcName {} {
	return $procName
    }


    #--------------------------------------------------------------------
    # method NoInternalFlatField
    #
    method NoInternalFlatField {} {
	return $ffLampAvailable
    }


    #--------------------------------------------------------------------
    # method SetupKeys
    # Return all INS.*.* keywords defined except those specified in filterList
    #
    method SetupKeys { {filterList {}} } {
	upvar INS INS

        set buffer ""
        set keyList [array names INS]
        foreach item $keyList {
            set addItem 1
            foreach filter $filterList {            
	        if { [string match $filter $item] } {
                    set addItem 0
                    break
		}
	    }
	    if { $addItem } {
                append buffer "INS.$item $INS($item) "
	    }
        }
	return "$buffer"
    }


    #--------------------------------------------------------------------
    # method SetupMode
    #
    method SetupMode { {mode ""} } {
	upvar INS INS

        if {[string length $mode] > 0} { set INS(MODE) $mode }
	return "INS.MODE $INS(MODE)"
    }


    #--------------------------------------------------------------------
    # method SetupLamp
    #
    method SetupLamp { {lamp ""} } {
	upvar INS INS

        if {[string length $lamp] > 0} { set INS(LAMP) $lamp }
	return "INS.LAMP $INS(LAMP)"
    }


    #--------------------------------------------------------------------
    # variables
    #
    private variable icsDbRoot	 ""
    private variable osAlias	 ""
    private variable subsysId	 ""
    private variable isUsed	 0
    private variable isSimulated 0
    private variable envName	 ""
    private variable procName	 ""
    private variable lcuEnvList	 {}
    protected variable ffLampAvailable	 1
}

#########################################################################
# ___oOo___
