# -*- tcl -*-
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplLib.tcl,v 1.64 2005/10/07 22:29:48 vltsccm Exp $"
#
# tplUIF.tcl
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-09-11  VLTSW20050294: tplMsgCall: <log> added;
#			new: tplNew, tplIncr, tplNext*, tplDebug*
# rschmutz  2005-04-11  tplMakeRefFile, tplMsgCall added.
# rschmutz  2005-04-11  ::BobTPL:: removed: moved to global namespace.
# rschmutz  2004-11-26  tplSeeVars: also upvar DPR by default.
# rschmutz  2004-11-22  created
#

#************************************************************************
#   NAME
#	tplMakeRefFile, tplMsgCall, tplSeeVars, tplTry, tplNew,
#	tplIncr, tplNextReset, tplNext, tplDebugLevel,
#	tplDebug - Misc. functions for templates.
# 
#   SYNOPSIS
#	tplMakeRefFile <fileFrom> <fileTo> [<noArchive>]
#	tplMsgCall <env> <proc> <cmd> <args> [<timeout>] [<log>=1]
#	tplSeeVars [<array1> ...]
#	tplTry <code> <finally>
#       tplNew <type> <name>
#       tplIncr <varName> <expr>
#       tplNextReset <listVar> [<index>]
#       tplNext <listVar> [<index>]
#       tplDebugLevel [<level>]
#       tplDebug [<level>] <txt>
#
#   DESCRIPTION
#     Library of misc. functions for Templates.
#
#     tplMakeRefFile <fileFrom> <fileTo> [<noArchive>]
#	Copy a file to the $INS_ROOT/SYSTEM/REFDATA directory
#	and archive it with archivSend (if <noArchive> is 0).
#
#     tplMsgCall <env> <proc> <cmd> <args> [<timeout>] [<log>=1]
#	Send a CCS message to process <proc> in environment <env>
#	Wait and return the reply (default timeout: 60 seconds).
#	If an error reply is returned, then an error is raised.
#	By default, the message exchange is logged in bob,
#	unless <log> is set to zero.
#
#     tplSeeVars [<array1> ...]
#	Helper function similar to seeBobVars: it issues the proper
#	set of "upvar" instructions, within the context of the 
#	calling procedure, to make the specified arrays accessible
#	as local arrays with the same name.  By default, arrays
#	TPL, DPR, SEQ, TEL, INS and DET are "upvar"ed.
#	
#     tplTry <code> <finally>
#	Some templates need to always execute certain commands
#	(turn off lamps, return the telescope to its initial
#	position), regardless if an error is encountered within
#	the template or not.
#
#	The tplTry function is similar to the try_eval command:
#	First the <code> block is executed; then <finally> block
#	is always executed, regardless if an error was encountered
#	during the evaluation of the <code> block or not.
#
#	If an error is encountered in the <code> block, then it is
#	stored in the errorResult variable, accessible from the
#	<finally> block.
#
#	When a template is aborted, bob normally skips all sendCmd
#	commands specified in the ::bob::tplSkipList. To make sure,
#	that sendCmd commands within the <finally> block are always
#	executed, the ::bob:tplSkipList variable is cleared before
#	the execution of the <finally> block (and restored to its
#	original value at the end of the <finally> block).
#	
#     tplNew <type> <name>
#	Instantiates a new object <name> of class <type>.
#	If an object <name> already exists, then it is deleted
#	before creating the new instance.
#
#     tplIncr <varName> <expr>
#	Add <expr> to the floating point variable <varName>,
#	i.e. similar to command 'incr', but for doubles.
#
#     tplNextReset <listVar> [<index>]
#	Reset the index for list <listVar> (or set it to <index>).
#	The next time tplNext is called, it will return the first
#	item of the list (or the item at index position <index>).
#
#     tplNext <listVar> [<index>]
#	Each consecutive call of tplNext returns the next item
#	of list <listVar> (or the item at index position <index>).
#	If there are no more items in the list, then tplNext
#	wraparounds and returns the first item in the list.
#	It is suggested to use this function to retrieve items
#	from FITS keyword lists.
#
#     tplDebugLevel [<level>]
#	Set the debug level to <level> (0..10).
#	0 disables logging.
#
#     tplDebug [<level>] <txt>
#	If the debug level is at least 1 (or <level>), then log
#	the debug message <txt> in bob (with gray color).
#
#	The debug level is by default 0 (no debug messages).
#	It can be set with the TPL_DEBUG env. variable or with
#	the tplDebugLevel function.  If the TPL_DEBUG env.
#	variable exists, but is not a number, then the debug
#	level is set to 1.
#
#   RETURN VALUE
#     tplTry:
#	If an error is encountered within the <finally> block, then
#	<finally> block error is returned, otherwise the status of
#	the <code> block is returned.
#	
#   ENVIRONMENT
#	TPL_DEBUG - debug level: 0..10
#
#   CAUTIONS
#	Within a template, do not use try_eval to make sure that 
#	certain commands are always executed: if the template is
#	aborted, then commands specified in the ::bob::tplSkipList
#	variable will be skipped, even if the sendCmd instruction
#	is located in the try_eval <finally> block.
#
#	If errors encountered in the <finally> block should not
#	affect the return status of the template, then the bob
#	feature bobTPL::setCallback should be used.
#
#	The array variable tplNextIndex is used by functions
#	tplNextReset and tplNext.  Do not access it directly
#	within templates.
#
#   EXAMPLES
#	See the XXXX Template Instrument *.seq files.
#
#   SEE ALSO
#	bobTPL archivSend
#
#   BUGS     
#
#------------------------------------------------------------------------
#


#********************************************************************
# proc tplMakeRefFile - store a new file in REFDATA, archive it.
#
proc tplMakeRefFile { fileFrom fileTo {noArchive 0} } {
    global env

    set fileToPath [file join $env(INS_ROOT) SYSTEM REFDATA $fileTo]
    if { [file exists $fileToPath] } {
	set fileBakDir [file join $env(INS_ROOT) SYSTEM REFDATA OLD]
	set fileBak [cconcat [file rootname $fileTo] "." [seq_isoTime] \
			     [file extension $fileTo]]
	set fileBakPath [file join $fileBakDir $fileBak]
	file mkdir $fileBakDir
	file rename $fileToPath $fileBakPath
    }
    set fileFromPath $fileFrom
    if { [file exists $fileFromPath] == 0 } {
	set fileFromPath [file join $env(INS_ROOT) SYSTEM DETDATA $fileFrom]
	if { [file exists $fileFromPath] == 0 } {
	    error "Source file $fileFrom not found."
	}
    }
    file copy $fileFromPath $fileToPath
    if { $noArchive == 0 } {
	set cwd [pwd]
	try_eval {
	    cd [file join $env(INS_ROOT) SYSTEM REFDATA]
	    exec archivSend $fileTo
	} {
	    error "Error archiving $fileTo ($errorResult)."
	} {
	    cd $cwd
	}
    }
}


#********************************************************************
# proc tplMsgCall - send a CCS message to a process
#
proc tplMsgCall { env proc cmd arg {timeout ""} {log 1} } {
    try_eval {
	if { $timeout == "" } {
	    set timeout 60000
	}
	if { $log } {
	    tplLog "$env $proc: $cmd $arg" blue
	}
	set err ""
	set cmdid [seq_msgSendCommand $env $proc $cmd $arg]
	set reply [seq_msgRecvReply -last -$timeout $cmdid err]
	if { $err != 0 } {
	   error "Error reply from $env $proc: '$cmd $arg': $err"
	}
	if { $log } {
	    tplLog $reply SpringGreen4
	}
	return $reply
    } {
	error "Error sending to $env $proc: '$cmd $arg': $errorResult"
    }
}


#********************************************************************
# proc tplTry
#

#
# The code below is a slightly modified version of
# the try procedure published in the Tcl/Tk Wiki:
# http://wiki.tcl.tk/990/
#
proc tplTry { script1 script2 } {

    # execute 'try' block:
    uplevel 1 { set errorResult "" }
    set status1 [catch { uplevel 1 $script1 } result1]
    if { $status1 == 1 } {
	uplevel 1 [list set errorResult $result1]
	set info1 $::errorInfo
	set code1 $::errorCode
    }

    # execute 'finally' block:
    # (disable skip list during 'finally' block execution)
    #
    set saveList [::bob::tplSkipList]
    ::bob::tplSkipList {}
    set status2 [catch { uplevel 1 $script2 } result2]
    ::bob::tplSkipList $saveList

    # evaluate return status:
    #
    # The status codes are:
    #  0 - TCL_OK
    #  1 - TCL_ERROR
    #  2 - TCL_RETURN
    #  3 - TCL_BREAK
    #  4 - TCL_CONTINUE
    #
    # If 'finally' returns TCL_OK, then the 'try' status is returned,
    # otherwise if 'finally' failed, then the 'finally' status is
    # returned.
    #
    switch -exact -- $status2 {
	0 { # TCL_OK - 'finally' was ok: return 'try' status:
	    switch -exact -- $status1 {
		0 { return $result1 }
		1 { # TCL_ERROR - 'try' failed
		    return -code error -errorcode $code1 \
			   -errorinfo $info1 $result1
		}
		2 { return -code return $result1 }
		3 { return -code break }
		4 { return -code continue }
		default { return -code $code $result1 }
	    }
	}
	1 { # TCL_ERROR -- 'finally' failed
	    set info2 $::errorInfo
	    set code2 $::errorCode
	    append info2 "\n    (\"finally\" block)"
	    return -code error -errorcode $code2 -errorinfo $info2 $result2
	}
	2 { return -code return $result2 }
	3 { return -code break }
	4 { return -code continue }
	default { return -code $code $result2 }
    }
}


#********************************************************************
# proc tplSeeVars: upvar keyword arrays
# (by default: TPL, DPR, SEQ, TEL, INS, DET)
#
proc tplSeeVars { args } {
    if { $args == "" } {
	set args {TPL DPR SEQ TEL INS DET}
    }
    foreach var $args {
	uplevel upvar $var $var
    }
}


#********************************************************************
# proc. tplNew - create a new object (deleting prev. instances)
#
proc tplNew { type name } {
    while { [itcl::find objects $name] != "" } {
	delete object $name
    }
    return [$type $name]
}


#********************************************************************
# proc. tplIncr - increment a double value
#
proc tplIncr { varName expr } {
    upvar $varName n

    set n [eval expr {double($n) + $expr}]

    return $n
}


#********************************************************************
# proc. tplNextReset - reset tplNext index
#
proc tplNextReset { listVar {nextIndex ""} } {
    upvar $listVar list

    #= 1. build index variable name
    set indexVar [cconcat "tplNextIndex(" $listVar ")"]
    upvar $indexVar index

    #= 2. reset index
    if { $nextIndex != "" } {
	set index $nextIndex
	if { $index < 0 || $index >= [llength $list] } {
	    set index 0
	}
    } else {
	set index 0
    }
}


#********************************************************************
# proc. tplNext - get next entry from a list
#
proc tplNext { listVar {nextIndex ""} } {
    upvar $listVar list

    #= 1. build index variable name
    set indexVar [cconcat "tplNextIndex(" $listVar ")"]
    upvar $indexVar index

    #= 2. if index variable does not exist: create it.
    if { [info exists index] == 0 } {
	set index 0
    }

    #= 3. get the next element from the list
    if { $nextIndex != "" } {
	set index $nextIndex
	if { $index < 0 || $index >= [llength $list] } {
	    set index 0
	}
    }
    set elem [lindex $list $index]

    #= 4. incr. index, wrap-around to zero if necessary
    incr index
    if { $index >= [llength $list] } {
	set index 0
    }

    return $elem
}


#********************************************************************
# proc. tplDebugLevel - set/get debug log level
#
# level = 0: disabled
#     1..10: enabled
#
proc tplDebugLevel { {level ""} } {
    global tplDebugLevel

    if { $level != "" } {
	if { $level < 0 || $level > 10 } {
	    error "tplDebugLevel: wrong level '$level' (expected 0..10)."
	}
	set tplDebugLevel $level
    }

    return $tplDebugLevel
}


#********************************************************************
# proc. tplDebug - debug logs
#
# tplDebug [<level>] <txt>
#
proc tplDebug { arg1 {arg2 ""} } {
    global tplDebugLevel

    if { $arg2 == "" } {
	set level 1
	set txt   $arg1
    } else {
	set level $arg1
	set txt   $arg2
    }
    if { $level <= 0 || $level > 10 } {
	error "tplDebug: wrong level '$level' (expected 1..10)."
    }

    if { $level <= $tplDebugLevel } {
	tplLog $txt gray50
    }
}


#********************************************************************
# init. tplDebugLevel
#
# If env. var. TPL_DEBUG is present, but does not
# contain a number, then use tplDebugLevel = 1.
#
global env tplDebugLevel
if { [info exists env(TPL_DEBUG)] } {
    set tplDebugLevel $env(TPL_DEBUG)
    if { $tplDebugLevel == "" } {
	set tplDebugLevel 1
    } elseif { [string is digit $tplDebugLevel] == 0 } {
	set tplDebugLevel 1
    }
} else {
    set tplDebugLevel 0
}


# ___oOo___
