# -*- tcl -*-
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplOBS.tcl,v 1.64 2005/10/07 22:29:46 vltsccm Exp $"
#
# tplOBS.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  2005-09-10  VLTSW20050294: AddFits, ExpoId, InsMode added.
# dpopovic  03/08/05  VLTSW20050224 set PAUSE timeout 
# alongino  10/11/04  VLTSW20040353 added waitArgs to ExposeDark
# mkiekebu 2004-10-10 added wait arguments to Expose
# dpopovic  05/08/04  VLTSW20040083 - COMMENT parameters
# alongino  12/07/04  VLTSW20030481
# alongino  27/01/03  VLTSW20020697
# alongino  28/11/02  VLTSW20020506: new method ImgName
# alongino  18/02/02  catch wait command after ABORT in StopLoop
# alongino  16/06/01  added SetupSave and SetupLookup
# alongino  23/01/01  OCS.SUBSYST replaced with -detId
# alongino  11/10/00  created
#
#************************************************************************
#   NAME
#   	tplOBS - Observation interface class for a template.
# 
#   SYNOPSIS
#	tplOBS obs <dbAlias>
#
#   DESCRIPTION
#	Observation interface for BOB templates.
#
#   PUBLIC METHODS
#	tplOBS <dbAlias>
#            dbAlias alias of the OS OLDB root point
#	Abort [-current]
#	    Abort the observation.
#	Continue
#	    Continue the observation.
#	End
#	    End the observation.
#	Pause
#	    Pause the observation.
#	Setup <buffer> [<expoId>]
#	    Setup the observation. 
#           expoId: value of the optional parameter -expoId
#                   Default: current observation id or 0 if no active observation
#       SetupSubsys <subsystem> <buffer>
#           Forward a setup command to the specified OS subsystem
#	SetupKeys [<filterList>]
#	    Return all OCS keywords except keywords present in <filterList>
#       SetupSave <dicList> <keyVal> <filename>
#           Save in the reference setup file <filename> (no extension) the contents of
#           <keyVal> ("<keyword1> <value1> .. <keywordN> <valueN>").
#           Keywords are checked against all dictionaries defined in <dicList>
#           and must be registered for class Setup
#           Library slxtcl is used and must therefore be linked
#       SetupLookup <dicList> <key> <filename>
#           Look in the reference setup file <filename> (no extension) for the
#           keyword <key> and return its value.
#           Keywords are checked against all dictionaries defined in <dicList>
#           and must be registered for class Setup
#           Library slxtcl is used and must therefore be linked
#       AddFits <buffer> [<detId>] [<expoId>]
#           Add FITS keywords to image header.
#           detId:  value of the optional parameter -detId
#           expoId: value of the optional parameter -expoId
#                   Default: current observation id
#	Start [<detId>] [<expoId>]
#	    Start the observation.
#           detId:  value of the optional parameter -detId
#           expoId: value of the optional parameter -expoId
#                   Default: current observation id
#	Wait [<expoId>] [<options>]
#	    Wait for the end of observation(s).
#           expoId:  value of the optional parameter -expoId.
#                    If "-all" is one of the <options>, <expoId> can have
#                    any value (e.g. 0) since it is not used.
#                    Default: current observation id
#           options: Optional parameter for additional options sent with WAIT command.
#                    They correspond one-to-one to the boss WAIT command parameters.
#                    If "-all" is one of the options, <expoId> is not used.
#                    Therefore, any value of expoId (e.g. 0) can be passed instead.
#                    Examples:
#                       Wait $expoId "-cond CanStartNextObs"
#                       Wait 0       "-all"
#	Timeout [<newTimeout>]
#	    Set the normal timeout and return its value
#	PauseTimeout [<newTimeout>]
#	    Set the PAUSE timeout and return its value
#	SetupTimeout [<newTimeout>]
#	    Set the SETUP timeout and return its value
#	WaitTimeout
#	    Return the WAIT timeout.
#	StopLoop dcs
#	    Stop a loop of exposures possibly running on a dcs subsystem.
#	StartLoop dcs [<detIdFlag>]
#	    Start a loop of exposures directly on a dcs subsystem.
#           detIdFlag: if 1, add option -detId to the START command. Default 0
#	ExpStatus [<n>]
#	    Return the exposure status
#	    n: exposure, if the observation has several exposures.
#		 1: Off
#		 2: Inactive
#		 3: Setup
#		 4: Started
#		 5: Integrating
#		 6: Paused
#		 7: Reading out
#		 8: Processing
#		 9: Transfering
#		10: Storing
#		11: Completed
#		12: Failed
#		13: Aborted
#	Comment
#	    Send Comment command.
#       ImgName [<tplId>] [<dprType>]
#           Build the default value of OCS.DETi.IMGNAME from TPL.ID and DPR.TYPE
#           tplId: template ID. Default: value of TPL.ID
#           dprType: data type. Default: value of DPR.TYPE
#       Expose <tplId> <dcsList> <funcS> <fileS> [<nrep>] [<imgLst>] [<idFlg>] [<waitArgs>]
#	    Perform simple exposures (sequence SETUP+START+WAIT).
#           tplId: template ID (see $TPL(ID))
#           dcsList: list of dcs objects involved in the exposure
#           funcS: -function parameter string for SETUP
#           fileS: -file parameter string for SETUP (if "", no -file specified)
#           nrep: how many times the exposure must be repeated (default 1)
#           imgLst: list of image file names to be used (default: use ImgName method)
#           idFlg: if 1, add option -detId to the START command. Default 0
#           waitArgs: arguments for wait command like -cond CanStartNextObs
#       ExposeDark <tplId> <dcsList> <fileS> [<nrep>]  [<imgLst>] [<idFlg>] [<waitArgs>]
#	    Perform dark exposures (sequence SETUP+START+WAIT).
#           tplId: template ID (see $TPL(ID))
#           dcsList: list of dcs objects involved in the exposure
#           fileS: -file parameter string for SETUP (if "", no -file specified)
#           nrep: how many times the exposure must be repeated (default 1)
#           imgLst: list of image file names to be used (default $TPL(ID).fits)
#           idFlg: if 1, add option -detId to the START command. Default 0
#           waitArgs: arguments for wait command like -cond CanStartNextObs
#       Integrate [<expoId>] [<setupBuffer>] [<detId>]
#           Perform an integration on the specified detector for the specified
#           observation. If a setup buffer is specified, the corresponding
#           setup command is sent before starting the integration.
#           expoId: value of the optional parameter -expoId for the command START
#           setupBuffer: value of the parameters string for the command SETUP
#           detId:  value of the optional parameter -detId for the command START
#       ExpoId
#           Return current exposure identifier (expoId).
#       InsMode
#           Return current INS.MODE value.
#
#   CAUTIONS
#	The first command of an observation must be always Setup.
#	The last command of an observation must be always either
#	Abort or Wait.
#
#   EXAMPLES
#       see xxoseq module and tpl/test/tplTest.seq
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#########################################################################
# Observation procedures
#
class tplOBS {

    #--------------------------------------------------------------------
    # constructor
    #
    constructor { {dbAlias ""} } {
        global env
	set envName $env(RTAPENV)
        set osAlias $dbAlias
    }


    #--------------------------------------------------------------------
    # method Abort
    #
    method Abort { {opt ""} } {
	if { "$opt" == "" } {
	    sendCmd $timeout "ABORT -expoId $id"
	} else {
	    sendCmd $timeout "ABORT"
	}
	set id 0
    }


    #--------------------------------------------------------------------
    # method Continue
    #
    method Continue {} {

	set t $pauseTimeout
	while { [ExpStatus] <= 5 && $t >= 0 } {
	    sleep 1
	    incr t -1
	}
	sendCmd $timeout "CONT -expoId $id"
    }


    #--------------------------------------------------------------------
    # method End
    #
    method End {} {
	sendCmd $timeout "END -expoId $id"
    }


    #--------------------------------------------------------------------
    # method Pause
    #
    method Pause {} {

	set t $pauseTimeout
	set expStatus [ExpStatus]
	while { $expStatus != 5 && $expStatus < 7 && $t >= 0 } {
	    sleep 1
	    incr t -1
	    set expStatus [ExpStatus]
	}
	sendCmd $timeout "PAUSE -expoId $id"
    }


    #--------------------------------------------------------------------
    # method Setup
    #
    method Setup { buffer {expoId 0} } {
        if { $expoId == 0 } { set expoId $id }
	if { $expoId == 0 } {
	    set reply [sendCmd $setupTimeout "SETUP -expoId 0 $buffer"]
            if {"$reply" == ""} { 
                error "ERROR: reply from OS to command SETUP -expoId 0 is empty. It should contain the new expoId"
	    } else { 
                set expoId $reply 
            }
	} else {
	    sendCmd $setupTimeout "SETUP -expoId $expoId $buffer"
	}
        set id $expoId
        return $expoId
    }


    #--------------------------------------------------------------------
    # method SetupSubsys
    #
    method SetupSubsys { subsystem buffer } {
	sendCmd $setupTimeout "FORWARD -subsystem $subsystem -command SETUP -arguments \"$buffer\""
    }


    #--------------------------------------------------------------------
    # method SetupKeys
    # Return all OCS.*.* keywords defined except those specified in filterList
    #
    method SetupKeys { {filterList {}} } {
	upvar OCS OCS

        set buffer ""
        set keyList [array names OCS]
        foreach item $keyList {
            set addItem 1
            foreach filter $filterList {            
	        if { [string match $filter $item] } {
                    set addItem 0
                    break
		}
	    }
	    if { $addItem } {
                append buffer "OCS.$item $OCS($item) "
	    }
        }
	return "$buffer"
    }


    #--------------------------------------------------------------------
    # method SetupSave
    # Save a set of keyword+value in a reference setup file
    #
    method SetupSave { dicList keyVal filename } {
        global env

        set status [slxtclDictionary $dicList]
        if {$status != "FAILURE"} {
            # save backup
            set fullPath $env(INS_ROOT)/SYSTEM/COMMON/SETUPFILES/REF/$filename.ref
	    catch { exec cp $fullPath $fullPath.backup}
            set status [slxtclWriteSetup $keyVal $filename]
	}
        return $status
    }


    #--------------------------------------------------------------------
    # method SetupLookup
    # Look for the value of a keyword in a given setup file
    #
    method SetupLookup { dicList key filename } {
        global env

        set value [slxtclDictionary $dicList]
        if {$value != "FAILURE"} {
            set value [slxtclReadSetup $filename]
            if {$value != "FAILURE"} {
                # look for a keyword in the buffer
                set keyL [split $value ","]; # split string in a list
                set index [lsearch $keyL "$key *"]
                if {$index >= 0} {; # found                    
                    set keyL [lindex $keyL $index]
                    set value [lindex $keyL 1]
	        } else {
                    set value FAILURE
                }
	    }
	}
        return $value
    }


    #--------------------------------------------------------------------
    # method AddFits
    #
    method AddFits { keywords {detId ""} {expoId 0} } {
        set detIdOption ""
        if { "$detId" != "" } { set detIdOption "-detId $detId" }
        if { $expoId == 0 } { set expoId $id }
        sendCmd $timeout ADDFITS "-expoId $expoId $detIdOption -info $keywords"
    }


    #--------------------------------------------------------------------
    # method Start
    #
    method Start { {detId ""} {expoId 0} } {
        set detIdOption ""
	if { "$detId" != "" } { set detIdOption "-detId $detId" }
        if { $expoId == 0 } { set expoId $id }        
	sendCmd $timeout "START -expoId $expoId $detIdOption"
    }


    #--------------------------------------------------------------------
    # method Wait
    #
    method Wait { {expoId 0} {options ""} } {
	# Check if "-all" option is given.
	# If yes, we do not pass expoId to the WAIT command.
	if { [string first "-all" $options] != -1} {
	    sendCmd $waitTimeout "WAIT $options"
	} else {
	    if { $expoId == 0 } {
		set expoId $id 
	    }        
	    
	    sendCmd $waitTimeout "WAIT -expoId $expoId $options"
	}
	set id 0
    }


    #--------------------------------------------------------------------
    # set normal timeout
    #
    method Timeout { {newTimeout "" } } {
	if { "$newTimeout" != "" } { set timeout $newTimeout }
	return $timeout
    }


    #--------------------------------------------------------------------
    # set PAUSE timeout VLTSW20050224
    #
    method PauseTimeout { {newTimeout "" } } {
	if { "$newTimeout" != "" } { set pauseTimeout $newTimeout }
	return $pauseTimeout
    }


    #--------------------------------------------------------------------
    # set SETUP timeout
    #
    method SetupTimeout { {newTimeout "" } } {
	if { "$newTimeout" != "" } { set setupTimeout $newTimeout }
	return $setupTimeout
    }


    #--------------------------------------------------------------------
    # set WAIT timeout
    #
    method WaitTimeout { {newTimeout "" } } {
	if { "$newTimeout" != "" } { set waitTimeout $newTimeout }
	return $waitTimeout
    }


    #--------------------------------------------------------------------
    # method StopLoop
    #
    method StopLoop { dcs } {
        set subsystem [$dcs Subsystem]
        if {[$dcs IsLoopRunning]} {
            tplLog "Stop loop on detector $subsystem ...."
            # Not necessarily the loop has been started by OS. Better to bypass
            # OS and forward commands to DCS
	    sendCmd $timeout "FORWARD -subsystem $subsystem -command STOP"
	    catch { sendCmd $timeout "FORWARD -subsystem $subsystem -command ABORT" }
            if {$loopStartedbyOs} {
                # Note: OS returns error if exposure has been aborted. Need to ignore errors.
                catch { Wait }
                set id 0
            } else {
                $dcs Wait "-finished"
	    }
            tplLog "Loop stopped on detector $subsystem"
        } else {
            tplLog "Exposures loop not running on detector $subsystem"
        }
    }


    #--------------------------------------------------------------------
    # method StartLoop
    #
    method StartLoop { dcs {detIdFlag 0} } {
        set loopStartedbyOs 0
        set subsystem [$dcs Subsystem]
        if {[$dcs IsLoopSupported]} {
            if {[$dcs IsLoopRunning]} {
                tplLog "Exposures loop already running on detector $subsystem"
            } else {
                tplLog "Start loop on detector $subsystem ...."
                Setup "-function [$dcs Repeat]"
                set detIdOption ""
                if {$detIdFlag} { set detIdOption [$dcs Subsystem] }
                Start $detIdOption
                # wait here till OLDB is updated
                set count 5
                while {(![$dcs IsLoopRunning]) && ($count > 0)} {
                    sleep 1
                    incr count -1
		}
                if { $count == 0 } {
                    error "ERROR: DCS OLDB exposure status does not show that loop is running"
		}
                set loopStartedbyOs 1
                tplLog "Loop started on detector $subsystem"
            }
        } else {
           tplLog "Loop not supported for detector $subsystem in the current simulation mode"
        }
    }


    #--------------------------------------------------------------------
    # method ExpStatus
    #
    method ExpStatus { {expNum 0} } {

	#*# search entry for current expo id
	set expoIds [seq_dbReadSymbolic "<alias>$osAlias:exposure.currExposureTbl(0:\$,expoId)"]
	set expoNo [lsearch $expoIds "expo-$id"]
	if { $expoNo == -1 } {
	    return -1
	}

	#*# get status of the requested exposure
	set expSt [seq_dbReadSymbolic "<alias>$osAlias:exposure.currExposureTbl($expoNo,state)"]

	#*# Convert status string to a number: more convenient for checks
        switch $expSt {
	    INACTIVE      { set expStatus  2 } 
	    PENDING       { set expStatus  3 } 
	    INTEGRATING   { set expStatus  5 } 
	    PAUSED        { set expStatus  6 } 
	    "READING OUT" { set expStatus  7 } 
	    PROCESSING    { set expStatus  8 } 
	    TRANSFERRING  { set expStatus  9 } 
	    RUNNING       { set expStatus 10 } 
	    COMPLETED     { set expStatus 11 } 
	    FINISHED      { set expStatus 11 } 
	    FAILED        { set expStatus 12 } 
	    ABORTED       { set expStatus 13 } 
	    default       { set expStatus  1 }
        }
                
	return $expStatus
    }


    #--------------------------------------------------------------------
    # method Comment
    #
    method Comment { buffer } {
	if { $id == 0 } {
	    set id [sendCmd $timeout "COMMENT -expoId 0 -string $buffer"]
	} else {
	    sendCmd $timeout "COMMENT -expoId $id -string \"$buffer\""
	}
    }

    #--------------------------------------------------------------------
    # method ImgName
    #
    method ImgName { {tplId ""} {dprType ""} } {
        upvar DPR DPR
        upvar TPL TPL

        if { [string length $tplId] == 0} {
            set tplId $TPL(ID)
	}
        if { [string length $dprType] == 0} {
            if { [info exists DPR(TYPE)] } {
                set dprType $DPR(TYPE)
	    }
	}

        # VLTSW20020506: rule changed
        #set filename $tplId.fits

        # extract instrument and mode from TPL.ID = "<instr>_<mode>_ ..."
        set tplIdList [split $tplId "_"]
        set instr_mode [string toupper [join [lrange $tplIdList 0 1] "_"]]

        # extract category from DPR.TYPE
        # e.g.:
        #  OBJECT    -> OBS
        #  STD       -> STD
        #  BIAS      -> BIAS
        #  DARK      -> DARK
        #  LAMP,FLAT -> FLAT
        #  LAMP,WAVE -> WAVE
        #
        if { [string length $dprType] > 0} {
            set catgList [split $dprType ","]
            set catg [lindex $catgList 0]
            if { $catg == "LAMP" } { set catg [lindex $catgList 1] }
            if { $catg == "OBJECT" } { set catg "OBS" }
	} else {; # DPR.TYPE not defined: use third element in TPL.ID
            set catg [string toupper [lrange $tplIdList 2 2]]
	}

        # create filename prefix instr_mode_catg
        set filename [cconcat $instr_mode "_" $catg]

        #tplLog "Template $tplId: image name $filename"
        return $filename
    }

    #--------------------------------------------------------------------
    # method Expose
    #
    method Expose { tplId detList funcSetup fileSetup {nrep 1} {imgFileNameList {}} {detIdFlag 0} {waitArgs {}} } {
        upvar DPR DPR

        # tplLog "Template $tplId: Starting $nrep exposures."
        for { set i 1 } { $i <= $nrep } { incr i } {
            set imgNameString ""
            set subsys ""
            foreach dcs $detList {
                set detId [$dcs DET]
                set filename ""
                set index [lsearch -exact $detList $dcs]
                if {[llength $imgFileNameList] >= $index} {
                    set filename [lindex $imgFileNameList $index]
                }
                if {[string length $filename] == 0} {
                    # look for DPR.TYPE in setup buffer
                    set dprType ""
                    if {[string length $funcSetup] > 0} {
                        set funcSetupList [split $funcSetup]
                        set index [lsearch -exact $funcSetupList DPR.TYPE]
                        if {$index >= 0} {; # found
                            while {[string length $dprType] == 0} {; # ignore blanks
                                incr index
                                set dprType [lindex $funcSetupList $index]
			    }
                            set dprType [string trim $dprType \"]
		        }               
		    }
                    set filename [ImgName $tplId $dprType]
                }
                if { [$dcs IsFileNameFixed] == 0 } {
                    set imgNameString "$imgNameString OCS.$detId.IMGNAME $filename"
		}
                append subsys "[$dcs Subsystem] "
            }
            set fileOption ""
            set funcOption ""
            if {[string length $fileSetup] > 0} { set fileOption "-file $fileSetup" }
            if {([string length $funcSetup] > 0) || ([string length $imgNameString] > 0)} { 
                set funcOption "-function " 
                if {[string length $funcSetup] > 0} {
                    append funcOption "$funcSetup " 
		}
                if {[string length $imgNameString] > 0} {
                    append funcOption "$imgNameString " 
		}
            }
            Setup "$fileOption $funcOption"
            set detIdOption ""
            if {$detIdFlag} { set detIdOption "$subsys" }
            Start "$detIdOption" 0
            Wait 0 $waitArgs
            foreach dcs $detList {
                if { [$dcs IsFileNameFixed] } {
                    tplLog "Image not saved in file. Subsystem [$dcs Subsystem] supports only fixed file name"
                } else {
                    tplLog "New image: [$dcs Filename]"
		}
            }
	    checkAbortFlag
        }
        # tplLog "Template $tplId: $nrep exposures finished."
    }


    #--------------------------------------------------------------------
    # method ExposeDark
    #
    method ExposeDark { tplId detList fileSetup {nrep 1} {imgFileNameList {}} {detIdFlag 0} {waitArgs {}} } {
        upvar DPR DPR

        set dcs [lindex $detList 0]
        set det [$dcs DET]
        upvar $det $det
        set expTimeKeyVal [$dcs ExpTime]
        set expTime [lindex $expTimeKeyVal 1]
        if { [cequal $expTime 0] } {
	    set dprType "BIAS"
        } else {
	    set dprType "DARK"
        }

        set funcSetup "DPR.CATG	   CALIB     \
	               DPR.TYPE	   $dprType  \
		       DPR.TECH	   IMAGE     \
		       $expTimeKeyVal"
        Expose $tplId $detList $funcSetup $fileSetup $nrep $imgFileNameList $detIdFlag $waitArgs
    }


    #--------------------------------------------------------------------
    # method Integrate
    #
    method Integrate { {expoId 0} {setupBuffer ""} {detId ""} } {
        if { [string length $setupBuffer] > 0 } {
            Setup $setupBuffer $expoId
        }
        if { $expoId == 0 } { set expoId $id }
        Start $detId $expoId
        Wait $expoId "-cond CanSetupNextObs"
        return $expoId
    }


    #--------------------------------------------------------------------
    # method ExpoId
    #
    method ExpoId {} {
        return $id
    }


    #--------------------------------------------------------------------
    # method InsMode
    #
    method InsMode {} {
        return [seq_dbReadSymbolic "<alias>$osAlias:insModes.insMode"]
    }


    #--------------------------------------------------------------------
    # variables
    #
    private variable timeout	     25000
    private variable setupTimeout   600000
    private variable id                  0
    private variable waitTimeout  10000000
    private variable pauseTimeout        7
    private variable envName            ""
    private variable osAlias            ""
    private variable loopStartedbyOs    0
}


#########################################################################
# ___oOo___
