# -*- tcl -*-
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplTCS.tcl,v 1.64 2005/10/07 22:29:47 vltsccm Exp $"
#
# tplTCS.tcl
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-09-11  CheckVignetting temp. simulation support added.
# rschmutz  2005-07-19  VLTSW20050250: method CheckVignetting added.
# dpopovic  2004-10-21  Addition of '-' in front of timeout in 
#                       calls to seq_msgRecvReply.
#                       Timeout added to Wait method.
# rschmutz  2003-12-08  delete uif before creation.
# alongino  26/11/02    VLTSW20020665 VLTSW20020416
# alongino  24/10/00    created
#
#************************************************************************
#   NAME
#   	tplTCS - TCS class interface for templates.
# 
#   SYNOPSIS
#	tplTCS tcs <tcsDbAlias> <osDbAlias> <osSubsysName> <focusId> <insId>
# 
#   DESCRIPTION
#	TCS interface for BOB templates.
#
#   PUBLIC METHODS
#       tplTCS <tcsDbAlias> <osDbAlias> <osSubsysName> <focusId> <insId>
#           tcsDbAlias alias of the TCS point in the Instrument Environment
#           osDbAlias  alias of the OS point in the Instrument Environment
#           osSubsysName  name of TCS subsystem as known to OS
#           focusId    ID of focus used (e.g. "CA", "NA", "NB")
#           insId      ID of the instrument as known to TCS
#	IsUsed
#	    Returns 1 if TCS is available, 0 otherwise.
#	EnvName
#	    Returns the TCS environment, e.g. 'wt1tcs'.
#	ProcName
#	    Returns the TIF process, e.g. 'tifCA'.
#	GetFocus
#	    Returns the focus ID
#	GetAdapterPos
#	    Returns the current adapter position
#	GetRotatorPos
#	    Returns the current rotator position
#	GetTurnTablePos
#	    Returns the current turntable position
#	GetInsRotAngle
#	    Returns the instrument rotator angle
#	SendFocusOffset <insMode> <detName> <offset>
#	    Send focus offset to TCS (command OFFSFM2).
#           It returns as soon the command to TCS is sent.
#           It returns the ID of the command sent to TCS
#           Method Wait must be called to wait for the last reply from TCS.
#	SendOffsad <raOffset> <decOffset>
#	    Send command OFFSAD to TCS.
#           It returns as soon the command to TCS is sent.
#           It returns the ID of the command sent to TCS
#           Method Wait must be called to wait for the last reply from TCS.
#	SendOffsadg <raOffset> <decOffset>
#	    Send command OFFSADG to TCS.
#           It returns as soon the command to TCS is sent.
#           It returns the ID of the command sent to TCS
#           Method Wait must be called to wait for the last reply from TCS.
#	SendOffsfm2 <how> <offset>
#	    Send command OFFSFM2 to TCS.
#           It returns as soon the command to TCS is sent.
#           It returns the ID of the command sent to TCS
#           Method Wait must be called to wait for the last reply from TCS.
#	SendProbeFocusOffset <how> <offset>
#	    Send command SETFOFF to TCS.
#           It returns as soon the command to TCS is sent.
#           It returns the ID of the command sent to TCS
#           Method Wait must be called to wait for the last reply from TCS.
#	SendSeqAO <flag>
#	    Send command SEQAO to TCS.
#           It returns as soon the command to TCS is sent.
#           It returns the ID of the command sent to TCS
#           Method Wait must be called to wait for the last reply from TCS.
#	SendStopag
#	    Send command STOPAG to TCS.
#           It returns as soon the command to TCS is sent.
#           It returns the ID of the command sent to TCS
#           Method Wait must be called to wait for the last reply from TCS.
#	Wait [<cmdId> <time>]
#	    Wait the reply of command (default cmdId = -1 = last command) 
#           for <time> milliseconds (default time = 920 sec (920000 ms).   
#	SendPark
#	    Park the calibration unit and guide probes
#           It returns as soon the command to TCS is sent.
#           It returns the ID of the command sent to TCS
#           Method WaitPark must be called to wait for the last reply from TCS.
#	WaitPark
#	    Wait until everything is parked.
#	IsAgActive [-wait]
#	    Returns 1 if the autoguider is active, 0 otherwise.
#	    If the -wait option is used, then ask to start
#	    autoguiding and wait.
#	IsAoActive [-wait|-waitRunning]
#	    Returns 1 if active optics is active, 0 otherwise.
#	    If the -wait option is used, then ask to start
#	    active optics and wait until the first analysis
#	    has been completed.	   
#	WaitforAO
#	    Wait for Active Optics active
#	CheckVignetting
#	    Check that the adapter is not vignetting the instrument field.
#	Parallatic
#	    Return parallatic.
#	RotOnSky
#	    Return rotOnSky.
#       SetRotOffset <insMode> <detName> <offset>
#           Send the rotator offset to TCS
#           It returns as soon the command to TCS is sent.
#           It returns the ID of the command sent to TCS
#           Method Wait must be called to wait for the last reply from TCS.
#
#   CAUTIONS
#
#   EXAMPLES
#       see xxoseq module
#
#   SEE ALSO
#
#   BUGS     
#	GetInsRotAngle, SendPark, SendWait are not implemented.
#
#------------------------------------------------------------------------
#

#########################################################################
# TCS procedures
#
class tplTCS {

    #--------------------------------------------------------------------
    # constructor
    #
    constructor { tcsDbAlias osDbAlias osSubsysName focusId insId } {
	global env

	#
	# init. vars.
	#
	catch { delete object uif }
	tplUIF uif
	catch { set debug $env(DEBUG_TEMPLATES) }

        set rtapEnv $env(RTAPENV)
        set tcsAlias $tcsDbAlias
        set osAlias $osDbAlias
        set subsysId $osSubsysName
        set insID $insId
        set focus $focusId

	#
	# check if TCS is used
	#
	set access [seq_dbReadSymbolic "<alias>$osAlias:subsystems:$subsysId.access"]
	if { $access == 0 } {
	    set isUsed 1
	} else {
	    set isUsed 0
	    return
	}

	set envName $env(TCS_ENVNAME)       
        foreach focusEl $focusList {
            set focusName [lindex $focusEl 0]
            set tifName   [lindex $focusEl 1]
            set adaName   [lindex $focusEl 2]
            if { "$focusName" == "$focus" } {
                set procName   $tifName
                set adapter    $adaName
                set adapterLcu "lt$env(TCSID)$adapter"
                break
            }
        }
        if {"$procName" == ""} { error "Invalid focus name: $focus" }

	#
	# init. OLDB locations
	#       
        set ptAgState    "<alias>$tcsAlias:guide.status"       
	set ptAoStep     "<alias>$tcsAlias:ao.currentIndexAvg"
        set ptAom1Corr   "<alias>$tcsAlias:ao.m1Corrected"
        set ptAom2Corr   "<alias>$tcsAlias:ao.m2Corrected"
        set ptAoCmdState "<alias>$tcsAlias:ao.commandState"
        set ptAoTgtSet   "<alias>$tcsAlias:ao.targetSettings"
        
    }


    #--------------------------------------------------------------------
    # destructor
    #
    destructor {
	delete object uif
    }

    #--------------------------------------------------------------------
    # method IsUsed
    #
    method IsUsed {} {
	return $isUsed
    }


    #--------------------------------------------------------------------
    # method EnvName
    #
    method EnvName {} {
	return $envName
    }


    #--------------------------------------------------------------------
    # method ProcName
    #
    method ProcName {} {
	return $procName
    }

    #--------------------------------------------------------------------
    # method GetFocus
    #
    method GetFocus {} {
	return $focus
    }

    #--------------------------------------------------------------------
    # method GetAdapterPos
    #
    method GetAdapterPos {} {
	if {! [IsUsed]} {
	    tplLog "TCS not used. adapter value not read"
	    return
	}
	set adAttr [seq_dbReadSymbolic "@$envName:Appl_data:TCS:LCU:$adapter:adrot:adapter:POSLOOP.pos"]
	return $adAttr
    }
    

    #--------------------------------------------------------------------
    # method GetRotatorPos
    #
    method GetRotatorPos {} {
	if {! [IsUsed]} {
	    tplLog "TCS not used. rotator value not read"
	    return
	}
	set roAttr [seq_dbReadSymbolic "@$envName:Appl_data:TCS:LCU:$adapter:trk.position"]
	return $roAttr
    }


    #--------------------------------------------------------------------
    # method GetTurnTablePos
    #
    method GetTurnTablePos {} {
	if {! [IsUsed]} {
	    tplLog "TCS not used. turntable not read"
	    return
	}
	set ttAttr [seq_dbReadSymbolic "@$envName:Appl_data:TCS:LCU:$adapter:adrot:adttMotor.posUser"]
	return $ttAttr
    }


    #--------------------------------------------------------------------
    # method GetInsRotAngle
    # Not available in BOSS OLDB yet
    method GetInsRotAngle {} {
	#set focus_used [seq_dbReadSymbolic \
        #           "<alias>$osAlias:config.focus(0:\$,used)"]
        #
        #set focus_insRotAngle [seq_dbReadSymbolic \
        #           "<alias>$osAlias:config.focus(0:\$,insRotAngle)"]
        #
	#set focus_index [lsearch -exact $focus_used 1]
        #
        #if { $focus_index != -1 } {
	#  return [lindex $focus_insRotAngle $focus_index]
	#}
        return 0
    }


    #--------------------------------------------------------------------
    # method SendFocusOffset
    #
    # Send focus offset to TCS (command SETINSD).
    #
    method SendFocusOffset { insMode detName offset } {
	#
	# 1. If TCS not used: return
	#
	set cmdName SETINSD
	if {! [IsUsed]} {
	    tplLog "TCS not used. No focus offset sent to TCS."
	    return
	}
                
	set insRotAngle [GetInsRotAngle]
	#
	# 2. Send focus offset to TCS
	#
	tplLog "$cmdName -insId $insID -insMode $insMode -focusStation $focus \
                -offsetRot $insRotAngle -offsetFocus $offset. -detectorLCU $detName"
        # Normally all commands should be sent to OS
	#set cmdId [sendCmd 0 "FORWARD -subsystem TCS -command $cmdName \
        #           -arguments $insID,$insMode,$focus,$insRotAngle,$offset,$detName"]
	set cmdId [seq_msgSendCommand $envName $procName \
		   $cmdName "-insId $insID -insMode $insMode \
                   -focusStation $focus -offsetRot $insRotAngle \
                   -offsetFocus $offset -detectorLCU $detName"]
	sleep 3

	# It seems that the instrument focus offset has to be
	# retrieved with command FGETAO and registered in the
	# ICS tables (not clear yet).
	# Then, during an exposure, the instrument focus offset
	# has to be registered with SETINSD -offsetFocus.
	# This command should probably sleep 2-3 seconds to
	# allow AO to move the mirror accordingly.
        return $cmdId
    }


    #--------------------------------------------------------------------
    # method SendOffsad
    #
    # Send command OFFSAD to TCS.
    #
    method SendOffsad { raOffset decOffset } {
	#
	# 1. If TCS not used: return
	#
	set cmdName OFFSAD
	if {! [IsUsed]} {
	    tplLog "TCS not used. No $cmdName command sent to TCS."
	    return
	}

	#
	# 2. Send focus offset to TCS
	#
	tplLog "$cmdName $raOffset,$decOffset."
	set cmdId [seq_msgSendCommand $envName $procName \
		   $cmdName "$raOffset,$decOffset"]
	return $cmdId
    }


    #--------------------------------------------------------------------
    # method SendOffsadg
    #
    # Send command OFFSADG to TCS.
    #
    method SendOffsadg { raOffset decOffset } {
	#
	# 1. If TCS not used: return
	#
	set cmdName OFFSADG
	if {! [IsUsed]} {
	    tplLog "TCS not used. No $cmdName command sent to TCS."
	    return
	}

	#
	# 2. Send focus offset to TCS
	#
	tplLog "$cmdName $raOffset,$decOffset."
	set cmdId [seq_msgSendCommand $envName $procName \
		   $cmdName "$raOffset,$decOffset"]
	return $cmdId
    }


    #--------------------------------------------------------------------
    # method SendOffsfm2
    #
    # Send command OFFSFM2 to TCS.
    #
    method SendOffsfm2 { how offset } {
	#
	# 1. If TCS not used: return
	#
	set cmdName OFFSFM2
	if {! [IsUsed]} {
	    tplLog "TCS not used. No $cmdName command sent to TCS."
	    return
	}

	#
	# 2. Send focus offset to TCS
	#
	tplLog "$cmdName $how,$offset."
	set cmdId [seq_msgSendCommand $envName $procName \
		   $cmdName "$how,$offset"]
	return $cmdId
    }


    #--------------------------------------------------------------------
    # method SendProbeFocusOffset
    #
    # Send command SETFOFF to TCS.
    #
    method SendProbeFocusOffset { how offset } {
	#
	# 1. If TCS not used: return
	#
	set cmdName OFFSFAD
	if {! [IsUsed]} {
	    tplLog "TCS not used. No $cmdName command sent to TCS."
	    return
	}

	#
	# 2. Send focus offset to TCS
	#
	if { "$how" == "DIFF" } {
	    set how REL
	}
	tplLog "$cmdName $how,$offset."
	set cmdId [seq_msgSendCommand $envName $procName \
		   $cmdName "$how,$offset"]
	return $cmdId
    }


    #--------------------------------------------------------------------
    # method SendSeqAO
    #
    # Send command SEQAO to TCS.
    #
    method SendSeqAO { flag } {
	#
	# 1. If TCS not used: return
	#
	set cmdName SEQAO
	if {! [IsUsed]} {
	    tplLog "TCS not used. No $cmdName command sent to TCS."
	    return
	}

	#
	# 2. Make AO corrections
	#
	tplLog "$cmdName $flag."
	##set cmdId [seq_msgSendCommand $envName actconServer \
		##   $cmdName "$flag"]
	set cmdId [seq_msgSendCommand $envName $procName \
		   $cmdName "$flag"]
	return $cmdId
    }


    #--------------------------------------------------------------------
    # method SendStopag
    #
    # Send command STOPAG to TCS.
    #
    method SendStopag {} {
	#
	# 1. If TCS not used: return
	#
	set cmdName STOPAG
	if {! [IsUsed]} {
	    tplLog "TCS not used. No $cmdName command sent to TCS."
	    return
	}

	#
	# 2. Send stop autoguiding to TCS
	#
	tplLog "$cmdName"
	set cmdId [seq_msgSendCommand $envName $procName $cmdName]
	return $cmdId
    }


    #--------------------------------------------------------------------
    # method Wait
    #
    # wait for TCS reply
    #
    method Wait { {id -1} {time 0} } {
	#
	# 1. If TCS not used: return
	#
	if {! [IsUsed]} {
	    return
	}

	#
	# 2. set the timeout
	#
	if { $time == 0 } {
	    set tout $timeout
	} elseif  { $time > 0 } {
	    set tout $time
	} else {
	    error "ERROR in TCS method Wait. Wrong timeout value ($time). Must be a positive integer."
	}

	#
	# 3. receive command reply
	#
	if { $id == -1 } {
	    set id $cmdId
	    set name $cmdName
	} else {
	    set name ""
	}

	set reply [seq_msgRecvReply -last -$tout $id errFlag]
	if { $errFlag != 0 } {
	    bell
	    bell
	    bell
	    error "ERROR performing TCS command $name ($reply)."
	}
    }


    #--------------------------------------------------------------------
    # method SendPark
    #
    # Park the calibration unit and the guide probe.
    #
    method SendPark {} {
	#
	# 1. If TCS not used: return
	#
	if {! [IsUsed]} {
	    tplLog "TCS not used. Calibration unit and guide probe not parked."
	    return
	}

	#
	# 2. Park guide probe
	#
	set cmdName PRPARK
	tplLog "$cmdName."
	set parkCmdId1 [seq_msgSendCommand $envName $procName $cmdName ""]

	#
	# 3. Park calibration unit
	#
	#set cmdName COUT
	#tplLog "$cmdName."
	#set parkCmdId2 [seq_msgSendCommand $envName $procName $cmdName ""]

        return $parkCmdId1
    }


    #--------------------------------------------------------------------
    # method WaitPark
    #
    # Wait everything to be parked.
    #
    method WaitPark {} {
	#
	# 1. If TCS not used: return
	#
	if {! [IsUsed]} {
	    return
	}

	#
	# 2. receive PRPARK reply
	#
	set reply [seq_msgRecvReply -last -$timeout $parkCmdId1 errFlag]
	if { $errFlag != 0 } {
	    bell
	    bell
	    bell
	    error "ERROR performing TCS command PRPARK ($reply)."
	}
	
    }


    #--------------------------------------------------------------------
    # method IsAgActive
    #
    # return 1 if the autoguider is active, 0 otherwise.
    # if option = '-wait': ask to start autoguiding and wait.
    #
    method IsAgActive { {option ""} } {
	#
	# 1. If TCS not used: return
	#
	if { [IsUsed] == 0 && $debug == 0 } {
	    return 1
	}

	#
	# 2. check option
	#
	set wait 0
	if { "$option" != "" } {
	    if { "$option" == "-wait" } {
		set wait 1
	    } else {
		error "Wrong option $option for tplTCS::IsAgActive."
	    }
	}

	#
	# 3. get autoguider state
	#
	set agActive 0
	if { $debug } {
	    set agState [uif GetValue "Autoguiding State" 122]
	} else {
	    set agState [seq_dbReadSymbolic $ptAgState]
	}
	if { $agState == 122 } {
	    set agActive 1
	    set wait 0
	}

	#
	# 4. wait
	#
	if { $wait } {
	    tplLog "Please start autoguiding, field stabilization."
	    bell
	    bell
	}
	while { $wait } {
	    sleep 2
	    update
	    checkAbortFlag

	    #
	    # 3. get autoguider state
	    #
	    set agActive 0
	    if { $debug } {
		set agState [uif GetValue "Autoguiding State" 122]
	    } else {
		set agState [seq_dbReadSymbolic $ptAgState]
	    }
	    if { $agState == 122 } {
		set agActive 1
		set wait 0
	    }
	}

	return $agActive
    }


    #--------------------------------------------------------------------
    # method IsAoActive
    #
    # return 1 if active optics is active, 0 otherwise.
    #
    # if option = '-wait': indicate that the template should  wait 
    # indefinetly for AO  corrections, otherwise if found any error 
    # return 0    
    # 
    # correction: indicate the number of correction wished before 
    # go on with the template. Default is 1.
    #
    method IsAoActive { {option "" } {correction 1} } {
	#
	# 1. If TCS not used: return
	#
	if { [IsUsed] == 0 && $debug == 0 } {
	    return 1
	}
	
	#
	# 3. get active optics attributes
	# TargetSetting, m1correction, m2correction and CmdState
	#
	set aoActive 0
        set loop 1

	if { $debug } {	   
	    set aoTgtSet [uif GetValue "Active Optics Target" 1]
	    set aom1Corr [uif GetValue "Active Optics Correction to M1" 1] 
	    set aom2Corr [uif GetValue "Active Optics Correction to M2" 1] 
	    set aoCmdState [uif GetValue "Active Optics correction state" 207]
	} else {
	    set aoTgtSet [seq_dbReadSymbolic $ptAoTgtSet]
            set aom1Corr [seq_dbReadSymbolic $ptAom1Corr]
	    set aom2Corr [seq_dbReadSymbolic $ptAom2Corr]
            set aoCmdState [seq_dbReadSymbolic $ptAoCmdState]            
	}

        if { $aoTgtSet == "0" } {	          
             tplLog "#### Active Optics is configured for not make corrections"
             tplLog "#### Please Configure it correctly "    
	     if { "$option" == "-wait" } {
                sleep 5
	        checkAbortFlag
	    } else { 
		set aoActive 0
	        return $aoActive
	    }
	}
  
        if { ! $debug } {
	   set aoTgtSet [seq_dbReadSymbolic $ptAoTgtSet]
        } 

    
       if { $aoTgtSet == "0" } {
    	    tplLog "Active Optics was not configured for make corrections"     
            set aoActive 0
    	    return $aoActive
        }
       
        while { $loop } {
          #- This state indicate that is doing a correction -#
	  if { $aoCmdState >= "207"} {
	      if { $aoTgtSet == "1" } {
		  #- Check number of correction of M2-#
		  if { $aom2Corr >= $correction } {
		      tplLog "Correction reached"
		      set aoActive 1
		      return $aoActive
		  }
	      } elseif { $aoTgtSet == "2" } {
		  #- Check number of correction M1 y M2-#
		  if { (($aom2Corr >= $correction) && ($aom1Corr >= $correction ))} {
		      tplLog "Correction reached"
		      set aoActive 1
	              return $aoActive
		  }
	      }
	  } elseif { $aoCmdState == "200" } {; #- State IDLE -#
	      tplLog " Please start active optics. !!"
	      bell
	      bell
	      sleep 5      
	      checkAbortFlag
	  }         
	  sleep 2
          update
          checkAbortFlag

	  #- Check if parameter was wait, if it is so then enter in a loop -#
          if { "$option" != "-wait" } { 
	      set loop 0 
	  } else {
	        tplLog "Waiting for $correction correction(s)"
	        #- Update values -#
	        if { $debug } {	   	           
	           set aom1Corr [uif GetValue "Active Optics Correction to M1" 1] 
	           set aom2Corr [uif GetValue "Active Optics Correction to M2" 1] 
	           set aoCmdState [uif GetValue "Active Optics correction state" 207]
	        } else {	          
	           set aom1Corr [seq_dbReadSymbolic $ptAom1Corr]
	           set aom2Corr [seq_dbReadSymbolic $ptAom2Corr]
                   set aoCmdState [seq_dbReadSymbolic $ptAoCmdState]
	           set aoTgtSet [seq_dbReadSymbolic $ptAoTgtSet]
	       }
	   }
       }	
    	     
    }


    #--------------------------------------------------------------------
    # method WaitForAO
    #
    # wait for active optics to be active and 
    # to execute one complete analysis loop.
    #
    # One loop is complete, when the loop counter value resets to 1.
    #
    method WaitForAO {} {
	#
	# 1. If TCS not used: return
	#
	if { [IsUsed] == 0 && $debug == 0 } {
	    return 0
	}

	#
	# 2. if AO is already running, then assume everything is ok
	#
	if {[IsAoActive]} {
	    return 1
	}

	#
	# 3. wait for active optics to be active
	#
	set aoActive [IsAoActive -waitRunning]
	if { $aoActive == 0 } {
	    return 0
	}

	#
	# 4. wait for the step counter to start a new loop,
	#    i.e. to complete the first analysis.
	#
	tplLog "Waiting for active optics to complete first analysis."
	set prevStep -1
	set aoStep 0
	while { $prevStep < $aoStep && $aoActive == 1 } {
	    if { $aoStep >= 1 } {
	        tplLog "Active optics loop step $aoStep."
	    }
	    set prevStep $aoStep
	    while { $prevStep == $aoStep } {
		sleep 2
		update
		checkAbortFlag

		# check that active optics is still active
		set aoActive [IsAoActive]
		if { $aoActive == 0 } {
		    tplLog "Active optics stopped."
		    set aoActive [IsAoActive -waitRunning]
		    set prevStep -1
		    set aoStep 0
		    continue
		}

		# get the active optics step
		if { $debug } {
		    set aoStep [uif GetValue "Active Optics Step" [expr {$prevStep + 1}]]
		} else {
		    set aoStep [seq_dbReadSymbolic $ptAoStep]
		}
	    }
	}
	if { $aoActive } {
	    tplLog "First active optics analysis completed."
	}

	return $aoActive
    }

    #--------------------------------------------------------------------
    # method Parallatic
    #
    method Parallatic {} {
      if {! [IsUsed]} {
	    return 0
      }
      return [seq_dbReadSymbolic "@$rtapEnv<alias>$tcsAlias:coord.prltic"]
       
    }


    #--------------------------------------------------------------------
    # method RotOnSky
    #
    method RotOnSky {} {
	if {! [IsUsed]} {
	    return 0
	}
       return [seq_dbReadSymbolic "@$rtapEnv<alias>$tcsAlias:track.adaPosAngle"]	
    }

    #--------------------------------------------------------------------
    # method SetRotOffset
    #
    method SetRotOffset { insMode detName offset } {
        #
	# 1. If TCS not used: return
	#

	set cmdName SETINSD

	if {! [IsUsed]} {
	    tplLog "TCS not used. No focus offset sent to TCS."
	    return
	}
       
	#
	# 2. Send rotator offset to TCS
	#
	tplLog "$cmdName -insId $insID insMode $insMode -focusStation $focus \
                 -offsetRot $offset -detectorLCU $detName"

	set cmdId [seq_msgSendCommand $envName $procName \
		   $cmdName "-insId $insID -insMode $insMode \
		   -focusStation $focus \
                   -offsetRot $offset -detectorLCU $detName"]
	sleep 3

	return $cmdId
    }


    #--------------------------------------------------------------------
    # method CheckVignetting
    #
    method CheckVignetting {} {
        #
	# 1. If TCS not used: return
	#
	if {! [IsUsed]} {
	    return
	}

	#
	# 2. check vignetting
	#
	#    Temporary: Until a public interface is provided by tif,
	#    a private TCS attr. is accessed.  This attribute does
	#    not exist in tcssim, therefore as workaround, if the
	#    OLDB read fails, we will assume that there is no 
	#    vignetting ... ejem ...
	#
	set vignetting 0
	if {[catch { set vignetting [seq_dbReadSymbolic \
	    "@$envName:Appl_data:TCS:LCU:adc:adrot:admain.gpVignetting"] }]} {
	    seq_errResetStack
	}
	if { $vignetting == 1 } {
	    tplLog "The guide probe is vignetting: check the guide probe GUI." red
	    tplLog "Cancel the abort to continue." red
	    bell
	    bell
	    bell
	    ::bob::bobAbort
	}
    }
    
      
    #--------------------------------------------------------------------
    # variables
    #
    public  variable osAlias	""
    public  variable subsysId	""
    public  variable tcsAlias	""
    private variable focus      ""
    private variable focusList  { {"CA" "tifCA" "adc"} {"NA" "tifNA" "ada"} {"NB" "tifNB" "adb"} }
    private variable isUsed	0
    private variable adapter	""
    private variable adapterLcu ""    
    private variable envName	""
    private variable rtapEnv    ""
    private variable procName	""
    private variable insID      ""
    private variable timeout	920000

    private variable cmdName	""
    private variable cmdId	0
    private variable parkCmdId1 0
    private variable parkCmdId2 0
    private variable uif
    private variable debug      0

    private variable ptAgState ""
    private variable ptAoErrMsg ""
    private variable ptAoStep   ""
    private variable ptAom1Corr "" 
    private variable ptAom2Corr ""
    private variable ptAoCmdState ""
    private variable ptAoTgtSet ""
}


#########################################################################
# ___oOo___
