# -*- tcl -*-
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplUIF.tcl,v 1.64 2005/10/07 22:29:47 vltsccm Exp $"
#
# tplUIF.tcl
#
# who       when      what
# --------  --------  ----------------------------------------------
# dpopovic  30/05/05  Improved Sleep method (VLTSW20050126)
# alongino  08/06/04  added support for tat tests (VLTSW20040156)
# alongino  30/01/02  added optional log parameter to Sleep (VLTSW20020036)
# alongino  24/10/00  created
#

#************************************************************************
#   NAME
#   	tplUIF - UIF class for templates.
# 
#   SYNOPSIS
#	tplUIF uif
#
#   DESCRIPTION
#	UIF methods for BOB templates.
#
#   PUBLIC METHODS
#	Confirm <title> [<labelYes>] [<labelNo>]
#	    Pop ups a confirmation dialog.
#	    Returns 0 or 1.
#	ConfirmOrRetry <title>
#	    Pop ups a confirmation dialog with three buttons:
#	    Yes, No, Retry. Returns 0, 1 or 2.
#	GetExpTimeAndContinue <title> <raOffset> <decOffset> <expTime>
#	    Display a dialog window to modify the exptime
#	    and ask to quit or continue interactive pointing.
#	    Returns a list with the continue flag and with 
#	    the new exposure time.
#	GetValue <title> <value>
#	    Display a dialog window to request a value to the user.
#           Returns the value entered by the user.
#	Sleep <time> [<doLog>]
#	    Sleep for <time> seconds, while updating the UIFs.
#           If doLog = 1 (default) a tplLog call is done.
#
#   CAUTIONS
#
#   EXAMPLES
#       see xxoseq module
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#########################################################################
# UIF procedures
#
class tplUIF {

    #--------------------------------------------------------------------
    # constructor
    #
    constructor { } {
	global env

        if { [info exist env(TAT_TEST)] } {
            set interactive 0
        }
    }

    #--------------------------------------------------------------------
    # method Confirm
    #
    # confirm that you accept result $x described by $title
    #
    method Confirm { title {lblYes "Yes"} {lblNo "No"}} {
	global ::putton

	bell
	
	#
	# 1. create the top level window and divide it into top
	#    and bottom parts.
	#
	toplevel .k -class Dialog
	wm title .k $title
	wm iconname .k Dialog

	frame .k.top -relief raised -bd 1
	pack .k.top -side top -fill both

	frame .k.bot -relief raised -bd 1
	pack .k.bot -side bottom -fill both
	
	#
	# 2. fill the top part with message.
	#
	message .k.top.msg -width 5i -text " $title " \
		-font -Adobe-Courier-Medium-R-Normal-*-180-*
	pack .k.top.msg -side right -expand 1 -fill both\
		-padx 3m -pady 3m
	
	#
	# 3. create a row of buttons at the bottom of the dialog.
	#
	set default 1
	set args { Yes No }
	button .k.bot.button0 -text $lblYes -command {set putton 0; destroy .k}
	button .k.bot.button1 -text $lblNo  -command {set putton 1; destroy .k}
	set i 0
	foreach but $args {
	    if {$i == $default} {
	        frame .k.bot.default -relief sunken -bd 1
	        raise .k.bot.button$i
	        pack .k.bot.default -side left -expand 1\
	        	-padx 3m -pady 2m
	        pack .k.bot.button$i -in .k.bot.default\
	        	-side left -padx 2m -pady 2m\
	        	-ipadx 2m -ipady 1m
	    } else {
	        pack .k.bot.button$i -side left -expand 1\
	        	-padx 3m -pady 3m -ipadx 3m -ipady 1m
	    }
	    incr i
	}
	
	#
	# 4. set up a binding for <Return> if there is a default,
	#    set a grab and claim the focus too.
	#
#	if {$default >= 0} {
#	    bind .k <Return> ".k.bot.button$default flash; \
#	    	set putton $default"
#	}
#	set oldFocus [focus]
#	grab set .k
#	focus .k
	
	#
	# 5. Wait for the user to respond, then restore the focus 
	#    and return the index of the selected button.
	#
	bell
	
        if { $interactive } {
	    tkwait window .k
        } else {
	    tplLog "TAT_TEST active: simulating user interaction"
            set putton 0
            destroy .k          
        }

	return $putton
    }


    #--------------------------------------------------------------------
    # method ConfirmOrRetry
    #
    # Confirm that you accept result $x described by $title.
    # 3 buttons: Yes, No, Retry.
    #
    method ConfirmOrRetry { title } {
	global ::putton
	
	bell
	
	# 1. create the top level window and divide it into top
	# and bottom parts.
	
	toplevel .k -class Dialog
	wm title .k $title
	wm iconname .k Dialog

	frame .k.top -relief raised -bd 1
	pack .k.top -side top -fill both

	frame .k.bot -relief raised -bd 1
	pack .k.bot -side bottom -fill both
	
	# 2. fill the top part with message.
	
	message .k.top.msg -width 5i -text " $title " \
		-font -Adobe-Courier-Medium-R-Normal-*-180-*
		
	pack .k.top.msg -side right -expand 1 -fill both\
		-padx 3m -pady 3m
	
	# 3 create a row of buttons at the bottom of the dialog.
	set default 1
	set args { Yes No Retry}

	button .k.bot.button0 -text Yes -command {set putton 0; destroy .k}
	button .k.bot.button1 -text No  -command {set putton 1; destroy .k}	
	button .k.bot.button2 -text Retry  -command {set putton 2; destroy .k}
	set i 0
	foreach but $args {
	    if {$i == $default} {
	        frame .k.bot.default -relief sunken -bd 1
	        raise .k.bot.button$i
	        pack .k.bot.default -side left -expand 1\
	        	-padx 3m -pady 2m
	        pack .k.bot.button$i -in .k.bot.default\
	        	-side left -padx 2m -pady 2m\
	        	-ipadx 2m -ipady 1m
	    } else {
	        pack .k.bot.button$i -side left -expand 1\
	        	-padx 3m -pady 3m -ipadx 3m -ipady 1m
	    }
	    incr i
	}
	
	# 4. set up a binding for <Return> if there is a default,
	#    set a grab and clainm the focus too
	
#	if {$default >= 0} {
#	    bind .k <Return> ".k.bot.button$default flash; \
#	    	set putton $default"
#	}
	#set oldFocus [focus]
	#grab set .k
	#focus .k
	
	# 5. Wait for the user to respond, then restore the focus 
	#    and return the index of the selected button.
	
	bell
	
        if { $interactive } {
	    tkwait window .k
	} else {
	    tplLog "TAT_TEST active: simulating user interaction"
            set putton 0
            destroy .k          
        }

	return $putton
    }


    #--------------------------------------------------------------------
    # method GetExpTimeAndContinue (based on exptimeAT13)
    #
    # display a dialog window to modify the exptime
    # and ask to quit or continue interactive pointing
    #
    method GetExpTimeAndContinue { title x y t } {
        global ::putton 
        global ::tempTime
        set tempTime $t

        bell
        
        # 1. create the top level window and divide it into top
        # and bottom parts.
        
        toplevel .k -class Dialog
        wm title .k $title
        wm iconname .k Dialog
        frame .k.top -relief raised -bd 1
        pack .k.top -side top -fill both
        frame .k.mid -relief raised -bd 1
        pack .k.mid -side top -fill both
        frame .k.bot -relief raised -bd 1
        pack .k.bot -side bottom -fill both
        
        # 2. fill the top part with message.
        
        message .k.top.msg -width 5i -text " Set next Exposure time , \
                \n then click Quit or Continue. \n \n \
                If continuing : \
                    \n    the telescope will apply  \n \
                \n RA  offset: $x \n DEC offset: $y" \
                -font -Adobe-Courier-Medium-R-Normal-*-180-*
        pack .k.top.msg -side right -expand 1 -fill both\
                -padx 3m -pady 3m
        
        # 3.1 create an entry box for exposure time
        
        entry .k.mid.entry \
                -width 20 -relief sunken -bd 2 -textvariable tempTime
        pack .k.mid.entry -side top -pady 2m
#        scale .k.mid.scale -label "               Next  Exposure  Time"  \
#                -from 1 -to 100 \
#                -length 10c -orient horizontal  -variable tempTime \
#                -font -Adobe-Times-Medium-R-Normal-*-180-* 
#                
#        pack .k.mid.scale -side top 
                
        # 3.2 create a row of buttons at the bottom of the dialog.
        set default 0
        set args { Quit Continue Offset Retry }

        button .k.bot.button0 -text Quit -command {set putton 0; destroy .k}
        
        button .k.bot.button1 -text Offset+Repeat -command {set putton 1; destroy .k}
        button .k.bot.button2 -text Offset+Quit -command {set putton 2; destroy .k}
        button .k.bot.button3 -text ReselectOnSame -command {set putton 3; destroy .k}
        set i 0
        foreach but $args {
            if {$i == $default} {
                frame .k.bot.default -relief sunken -bd 1
                raise .k.bot.button$i
                pack .k.bot.default -side left -expand 1\
                        -padx 3m -pady 2m
                pack .k.bot.button$i -in .k.bot.default\
                        -side left -padx 2m -pady 2m\
                        -ipadx 2m -ipady 1m
            } else {
                pack .k.bot.button$i -side left -expand 1\
                        -padx 3m -pady 3m -ipadx 3m -ipady 1m
            }
            incr i
        }
        
        # 4. set up a binding for <Return> if there is a default,
        #    set a grab and clainm the focus too
        
#       if {$default >= 0} {
#           bind .k <Return> ".k.bot.button$default flash; \
#               set putton $default"
#       }
        #set oldFocus [focus]
        #grab set .k
        #focus .k
        
        # 5. Wait for the user to respond, then restore the focus 
        #    and return the index of the selected button.
        
        bell
        
        if { $interactive } {
	    tkwait window .k
	} else {
	    tplLog "TAT_TEST active: simulating user interaction"
            set putton 0
            destroy .k          
        }

        return [list $putton $tempTime]
    }


    #--------------------------------------------------------------------
    # method GetValue
    #
    # display a dialog window to enter a value
    #
    method GetValue { title value } {
        global ::putton 
        global ::tempValue
        set tempValue $value

        bell
        
        # 1. create the top level window and divide it into top
        # and bottom parts.
        
        toplevel .k -class Dialog
        wm title .k "Value Entry"
        wm iconname .k Dialog
        frame .k.top -relief raised -bd 1
        pack .k.top -side top -fill both
        frame .k.mid -relief raised -bd 1
        pack .k.mid -side top -fill both
        frame .k.bot -relief raised -bd 1
        pack .k.bot -side bottom -fill both
        
        # 2. fill the top part with message.
        
        message .k.top.msg -width 5i -text "$title" \
                -font -Adobe-Courier-Medium-R-Normal-*-180-*
        pack .k.top.msg -side right -expand 1 -fill both\
                -padx 3m -pady 3m
        
        # 3.1 create an entry box for the value
        entry .k.mid.entry \
                -width 20 -relief sunken -bd 2 -textvariable tempValue
        pack .k.mid.entry -side top -pady 2m
                
        # 3.2 create a row of buttons at the bottom of the dialog.
        set default 0
        set args { Ok Cancel }
        button .k.bot.button0 -text Ok -command {set putton 0; destroy .k}
        button .k.bot.button1 -text Cancel -command {set putton 1; destroy .k}
        set i 0
        foreach but $args {
            if {$i == $default} {
                frame .k.bot.default -relief sunken -bd 1
                raise .k.bot.button$i
                pack .k.bot.default -side left -expand 1\
                        -padx 3m -pady 2m
                pack .k.bot.button$i -in .k.bot.default\
                        -side left -padx 2m -pady 2m\
                        -ipadx 2m -ipady 1m
            } else {
                pack .k.bot.button$i -side left -expand 1\
                        -padx 3m -pady 3m -ipadx 3m -ipady 1m
            }
            incr i
        }
        
        # 4. set up a binding for <Return> if there is a default,
        #    set a grab and clainm the focus too
        
#       if {$default >= 0} {
#           bind .k <Return> ".k.bot.button$default flash; \
#               set putton $default"
#       }
        #set oldFocus [focus]
        #grab set .k
        #focus .k
        
        # 5. Wait for the user to respond, then restore the focus 
        #    and return the index of the selected button.
        
        bell
        
        if { $interactive } {
	    tkwait window .k
	} else {
	    tplLog "TAT_TEST active: simulating user interaction"
            set putton 0
            destroy .k          
        }
	if { $putton } {
	    error "Template aborted by the user."
	}
        return $tempValue
    }


    #--------------------------------------------------------------------
    # method Sleep 
    #
    #	wait time in seconds
    #
    method Sleep { time { doLog 1 } } {
        if { $doLog } {
	    tplLog "Sleeping for $time seconds."
	}

	# VLTSW20050126
	set timeoutVar ${this}_SleepTimeout
	set $timeoutVar 0; # can be set to anything
	after [expr {int($time*1000)}] unset $timeoutVar
	vwait $timeoutVar; # wait till timeoutVar gets modified/unset
    }
    
    #--------------------------------------------------------------------
    # variables
    #
    protected variable interactive 1
}


#########################################################################
# ___oOo___
