#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplBASE.tcl,v 1.15 2005/09/12 02:25:19 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-05-08  created
#

#************************************************************************
#   NAME
#       tplBASE - base class to allow children classes to 'see' its parent
#
#   SYNOPSIS
#	tplBASE obj
#
#   DESCRIPTION
#	Tcl classes may create child classes with e.g. names:
#	$this.tcs, $this.ics, etc.
#	This class may be inherited by a child class, to allow it
#	to access its parent or siblings.
#
#   PROTECTED METHODS
#	Parent
#	  Return the name of the parent object, i.e. $this without
#	  the last element (elements being separated by dots).
#
#	Contents
#	  Returns all child object names, i.e. all objects named
#	  "$this.<name>".
#
#   HaveChild name
#     Check if child name exists.  Returns 0 or 1.
#
#	Lookup name
#	  Search for a child, sibling or 'uncle' object with the
#	  given name.  E.g. if the current object is ::os.os1.ics,
#	  then a lookup for "tcs" will search for objects
#	  "::os.os1.ics.tcs", "::os.os1.tcs" and "::os.tcs".
#
#   CAUTION
#	Using method Parent on the top level object (e.g. ::os)
#	returns an empty string.
#
#	When a tplBASE object is deleted, its destructor takes
#	care of deleting all child objects.
#
#   EXAMPLES
#	See tplOS, mactplMACAO_BASE.
#
#   SEE ALSO
#	tplOS
#
#   BUGS
#
#------------------------------------------------------------------------
#


class tplBASE {

    #----------------------------------------------------------------
    # constructor
    #
    constructor {} {
	set parent [join [lrange [split $this "."] 0 end-1] "."]
    }


    #----------------------------------------------------------------
    # destructor - destroy all child objects
    #
    destructor {
	foreach child [Contents] {
	    catch { delete object $child }
	}
    }


    #----------------------------------------------------------------
    # method Parent - return parent object name
    #
    protected method Parent {} {
	return $parent
    }


    #----------------------------------------------------------------
    # method Contents - return child object names
    #
    protected method Contents {} {

	#= 1. get all objects "$this.*"
	set objList [itcl::find objects $this.*]

	#= 2. extract only the childs
	set len [llength [split $this "."]]
	incr len
	set objs ""
	foreach obj $objList {
	    if { [llength [split $obj "."]] == $len } {
		lappend objs $obj
	    }
	}

	return $objs
    }

    #----------------------------------------------------------------
    # method HaveChild - check if child exists
    #
    protected method HaveChild { name } {
	if { [itcl::find objects $this.$name] != "" } {
	    return 1
	} else {
	    return 0
	}
    }

    #----------------------------------------------------------------
    # method Lookup - search for a child, sibling or 'uncle' object
    #
    protected method Lookup { name } {

	set obj [itcl::find objects $this.$name]
	if { $obj != "" } {
	    return $obj
	}
	if { $parent != "" } {
	    set obj [itcl::find objects $parent.$name]
	    if { $obj != "" } {
		return $obj
	    }
	    set gparent [$parent Parent]
	    if { $gparent != "" } {
		set obj [itcl::find objects $gparent.$name]
		if { $obj != "" } {
		    return $obj
		}
	    }
	}

	error "Internal error: object $name not found."
    }

    #================================================================
    # variables
    #
    protected variable parent
}


#___oOo___

