#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tplOS.tcl,v 1.15 2005/09/12 02:25:19 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-05-08  created
#

#************************************************************************
#   NAME
#       tplOS - OS class interface for templates
#
#   SYNOPSIS
#       tplOS os
#
#   DESCRIPTION
#       OS interface for BOB templates.
#
#       The OS class allows to create several child objects to
#       obtain interfaces to the OS subsystems (e.g. TCS, ICS).
#       To create a child object, the class type must be passed
#       with one of the following options (see example below):
#
#	-uifClass, -obsClass, -tcsClass, -icsClass, -ics1Class,
#	-ics2Class, -ics3Class, -ics4Class, -dcsClass, -dcs1Class,
#	-dcs2Class, -dcs3Class, -dcs4Class, -drClass -macaoClass
#
#	If the -*Class value contains a list, then the first entry
#	is interpreted as the class name, the rest as arguments to
#	be passed to the child object constructor.
#
#       If option -uifClass is not present, then an object of
#       type tplUIF is created.
#
#       Public access methods are provided, to access the child
#       objects from outside the tplOS object (e.g. method Tcs
#       for accessing object $this.tcs, Ics for $this.ics, etc.).
#
#   PARENT CLASS
#       tplBASE
#
#   PUBLIC METHODS
#       Uif, Obs, Tcs, Ics, Ics1, Ics2, Ics3, Ics4, Dcs, Dcs1,
#       Dcs2, Dcs3, Dcs4, Os1, Os2, Os3, Os4, Dr, Macao
#         Methods that return the name of the corresponding
#         child object.
#
#       Uif <method> <args> ...
#       Obs <method> <args> ...
#       Tcs <method> <args> ...
#       ...
#       Macao <method> <args> ...
#         Access methods, that allow to call methods of the
#         child objects, e.g. to call the Sleep method of
#         the uif object:  os Uif Sleep 30.
#
#     Begin [<reserved>]
#       Begin a new observation.
#	The first argument is reserved for future use.
#	This method should be called at the beginning of
#	a template.
#     InsMode
#	Return the current instrument mode (INS.MODE).
#     ImgName [<dprType>]
#       Return image name according to current TPL.ID and DPR.TYPE.
#     Setup <buffer>
#       Setup an exposure.
#	Options -expoId and -noExposure should not be used.
#	They are generated automatically as/when needed.
#     AddFits <buffer> [<detId>]
#       Add FITS header keyword to the current exposure.
#     Start [<detId>]
#       Start an exposure.
#     Wait [<options>]
#       Wait for the end of exposure.
#     End
#       End the observation.
#	Note that this method is automatically called at the
#	end of the template execution (see CAUTION below).
#
#   PROTECTED METHODS
#
#   CAUTION
#       The access methods return the child object name,
#       regardless if the object really exists or not.
#
#	It is assumed, that only a single instance of this
#	class is instantiated within a template.  When this
#	class is instantiated, it registers a callback (with
#       BOBs setCallBack function) to automatically call its
#       End method at the end of the template execution.
#
#   EXAMPLES
#       class snoseqOS {
#           inherit tplOS
#
#           constructor {} {
#               chain -obsClass "tplOBS sno" \
#                     -tcsClass "tplTCS TCS sno tcs CA SINFONI" \
#                     -icsClass snoseqICS \
#                     -dcsClass "tplIRACE iracq IRDCS 1" \
#                     -macaoClass snoseqMACAO
#           } {}
#
#           ...
#       }
#
#   SEE ALSO
#	tplBASE
#
#   BUGS
#
#------------------------------------------------------------------------
#

#
# Note: Booking/resource handling is not implemented.
#
#       This will be only useful for instruments that allow the
#       execution of templates in parallel (e.g. FLAMES), where
#       we must make sure, that templates do not interfere with
#       each-other, i.e. use the same instrument resources.
#       For the time being this is not implemented, and the
#       operators are responsible of not making something wrong.
#
#     Begin [<resources>]
#       Begin a new observation.
#       Unless specific resources are specified, all instrument
#       resources are assigned to the observation.  This method
#	should be called at the beginning of a template.
#     End
#       End the observation.
#       Unlock all assigned resources.
#	Note that this method is automatically called at the
#	end of the template execution (see CAUTION below).
#
#     Lock <resources> ...
#       Lock resources.
#     Unlock <resources> ...
#       Unlock resources.
#


#--------------------------------------------------------------------
# proc tplOsEnd
#
# This procedure is registered as a callback, to make sure
# that the observation is always ended when the template
# completes.
#
proc tplOsEnd {} {
    global tplOsInstance

    if { $tplOsInstance != "" } {
	$tplOsInstance End
    }
}


#--------------------------------------------------------------------
# class tplOS
#
class tplOS {
    inherit tplBASE

    #----------------------------------------------------------------
    # constructor
    #
    constructor { args } {
	global tplOsInstance

	#= 1. read options
	eval configure $args

	#= 2. create child objects
	#
	# The following looks somewhat strange, because
	# the object creation is done within a loop
	# (to make the code shorter), and not secuentially
	# as normally done.
	#
	foreach child {uif obs tcs ics ics1 ics2 ics3 ics4 \
		       dcs dcs1 dcs2 dcs3 dcs4 dr macao} {

	    # construct option name "-<child>Class"
	    # if it is not empty, then ...
	    set typeVar [cconcat $child "Class"]
	    if { [set $typeVar] != "" } {

		# ... separate class name and constructor args ...
		set arg [set $typeVar]
		set class [lvarpop arg]

		# ... and create the child object
		eval $class $this.$child $arg
	    }
	}

	#= 3. register tplOsEnd callback
	set tplOsInstance $this
	setCallBack tplOsEnd

	#= 4. create booking object
	tplBOOKING $this.booking
    }


    #----------------------------------------------------------------
    # destructor
    #
    destructor {
	#= 1. unregister from tplOsEnd callback
	set tplOsInstance ""
    }


    #----------------------------------------------------------------
    # Access methods
    #
    # Here we also use a loop to make the code shorter
    # (otherwise we have to duplicate the code 15 times ...).
    #
    # For each child object:
    foreach obj {{uif tplUIF} obs tcs ics ics1 ics2 ics3 ics4 \
		 dcs dcs1 dcs2 dcs3 dcs4 dr macao} {

	# extract child name and default class
	lassign $obj child default

	# create the <child>Class option variable
	public variable [cconcat $child "Class"] $default

	# create the access method
	#
	# The body is created with 'subst' to substitute the
	# child name. Other variables have to be escaped, to
	# save them from substitution.
	#
	# If no arguments are passed to the access method,
	# then the child name is returned.
	# If arguments are passed, then the corresponding
	# child object method is called, and its result
	# returned.
	# uplevel is used, to make sure that any upvar's
	# in the called method will work correctly.
	#
	method [string totitle $child] { args } [subst -nocommands {
	    if { \$args == "" } {
		return \$this.$child
	    } else {
		return [uplevel 1 \$this.$child \$args]
	    }
	}]
    }


    #================================================================
    # Booking methods
    #
    method Begin { {resources ""} } {
	$this.booking Begin
	set observing 1
    }
    method End {} {
	$this.booking End
	set observing 0
    }
    method Lock { args } {
	eval $this.booking Lock $args
    }
    method Unlock { args } {
	eval $this.booking Unlock $args
    }

    #================================================================
    # Observation related methods
    #
    method InsMode
    method ImgName
    method Setup
    method AddFits
    method Start
    method Wait

    #================================================================
    # TCS related methods
    #
    method Offset
    method OffsetReturn


    #================================================================
    # variables
    #
    protected variable observing 0
}


#___oOo___

