#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tpltooOBD.tcl,v 1.15 2005/09/12 02:25:16 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  10/07/01  OBS.ID=-1.
# rschmutz  08/06/01  created (based in calobOBD.tcl).
#

#************************************************************************
#   NAME
#	tpltooOBD - OBD interface
# 
#   SYNOPSIS
#	tpltooOBS obs
# 
#   DESCRIPTION
#	Itcl interface to an OBD.
#
#   PARENT
#	tpltooPAF
#
#   PUBLIC METHODS
#	See tpltooPAF.
#
#	Add <obs> [<rmdup>]
#	    Add an observation (tpltooOBS) to the OBD.
#	    If <rmdup> is 1, then the observation is not added,
#	    if another identical observation is already present.
#	    Returns 1 if the observation was added, 0 otherwise.
#
#	Contains <obs>
#	    Returns 1, if <obs> is contained in the OBD,
#	    0 otherwise.
#
#	Sort
#	    Sort observations according to their sortkey.
#
#   ENVIRONMENT
#	INS_ROOT	Instrument root area
#
#   CAUTION
#
#   EXAMPLES
#
#   SEE ALSO
#	tpltooPAF(8)
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# class: tpltooOBS
#------------------------------------------------------------------------

class tpltooOBD {
    inherit tpltooPAF

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
    	global env

        tpltooPAF::constructor -objType OBD
	eval configure $args

	#*# 1. set file directory
	if { ![file exists $file] && [file tail $file] == $file } {
	    configure -file "$env(INS_ROOT)/SYSTEM/COMMON/TEMPLATES/OBD/$file"
	}

	#*# 2. load init. reference
	if { $init != "" } {
	    RefPAF INIT $init
	}

	#*# 3. add PAF entries
	set now [clock format [clock seconds] -format "%Y-%m-%dT%H:%M:%S.00"]
	Set PAF.HDR.START	""
	Set PAF.TYPE            "ObsBlockDescription"
	Set PAF.ID              ""
	Set PAF.NAME            ""
	Set PAF.DESC            ""
	Set PAF.CRTE.NAME       "tpltooBuild"
	Set PAF.CRTE.DAYTIM     "$now"
	Set PAF.LCHG.NAME       ""
	Set PAF.LCHG.DAYTIM     "$now"
	Set PAF.CHCK.NAME       ""
	Set PAF.CHCK.DAYTIM     ""
	Set PAF.CHCK.CHECKSUM   ""
	Set PAF.HDR.END         ""

	Set OBS.ID              "-1"
	Set OBS.NAME            "Maintenance"
	Set OBS.GRP             "0"
	Set OBS.PROG.ID         "Maintenance"
	Set OBS.PI-COI.ID       "0"
	Set OBS.PI-COI.NAME     "Condor"
	catch { Set OBS.ID          [Get OBS.ID.TEST.VALUE] }
	catch { Set OBS.NAME        [Get OBS.NAME.TEST.VALUE] }
	catch { Set OBS.GRP         [Get OBS.GRP.TEST.VALUE] }
	catch { Set OBS.PROG.ID     [Get OBS.PROG.ID.TEST.VALUE] }
	catch { Set OBS.PI-COI.ID   [Get OBS.PI-COI.ID.TEST.VALUE] }
	catch { Set OBS.PI-COI.NAME [Get OBS.PI-COI.NAME.TEST.VALUE] }
    }

    #****************************************************************
    #*# method: Load
    #----------------------------------------------------------------

    public method Load { {fileName ""} {rmdup 0} } {
        global env

    	#*# 1. get filename
    	if { "$fileName" != "" } {
    	    configure -file $fileName
	}
	set fileName $file
	if { "$file" != "" } {
	    if {![file exists $file] && "[file tail $file]" == "$file" } {
	        configure -file "$env(INS_ROOT)/SYSTEM/COMMON/TEMPLATES/OBD/$file"
	    }
	}
	if { "$file" == "" } {
	    return
	}
	if {![file exists $file]} {
	    error "File $fileName not found."
	}

	#*# 2. load file
	set newObs 0
	set inTpl 0
	set obs ""
	for_file txt $file {
	    #*# 2.1 parse line
	    set line [LoadLine $txt]
	    if { "$line" == "" } {
	    	continue
	    }
	    lassign [LoadLine $txt] param val

	    #*# 2.2 save, create observations
	    if { [crange $param 0 3] == "TPL." } {
	    	if { $inTpl == 0 } {
	    	    set newObs 1
		}
		set inTpl 1
	    } else {
	    	set inTpl 0
	    }
	    if { $newObs } {
	    	#*# save current observation
	    	if { "$obs" != "" } {
		    if {![Add $obs $rmdup]} {
		    	delete object $obs
		    }
		}

		#*# create new observation
		set obs [tpltooOBS $this.obs$oi -name OBS$oi]
		incr oi
		set newObs 0
		Debug "New observation:" 4
	    }

	    #*# 2.3 save keyword
	    if { "$obs" == "" } {
		Debug "Saving obs. param. $param='$val'." 4
	    	Set $param $val
	    } else {
		Debug "Saving header param. $param='$val'." 4
		$obs Set $param $val
	    }
	}

	#*# 2.4 save last observation
	if { "$obs" != "" } {
	    if {![Add $obs $rmdup]} {
		delete object $obs
	    }
	}
    }

    #****************************************************************
    #*# method: SaveAs
    #----------------------------------------------------------------

    public method SaveAs { fileName } {
    	global env

	#*# 1. get filename
	if { [file tail $fileName] == $fileName } {
	    set fileName "$env(INS_ROOT)/SYSTEM/COMMON/TEMPLATES/OBD/$fileName"
	}

    	#*# 2. check file is not read-only
	if {[file exists $fileName]} {
    	    if {![file writable $fileName]} {
	        error "File $fileName is read-only.  Unable to save."
	    }
	}

	#*# 3. save file
	write_file $fileName [Puts]
	return $fileName
    }

    #****************************************************************
    #*# method: Puts
    #----------------------------------------------------------------

    public method Puts {} {
        global env

    	#*# 1. set PAF header
	set user  $env(USER)
	set today [clock format [clock seconds] -format "%Y-%m-%d"]
	set now   [clock format [clock seconds] -format "%Y-%m-%dT%H:%M:%S.00"]
    	set txt \
"#*******************************************************************************
# E.S.O. - VLT project
#
# \"@(#) \$Id\$\"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# [format %-8s $user]  $today  created.
#

PAF.HDR.START;
PAF.TYPE                [format %-26s "\"[Get1 PAF.TYPE]\";"	   ] # Type of PAF
PAF.ID                  [format %-26s "\"[Get1 PAF.ID]\";"	   ] # ID of PAF
PAF.NAME                [format %-26s "\"[Get1 PAF.NAME]\";"	   ] # Name of PAF
PAF.DESC                [format %-26s "\"[Get1 PAF.DESC]\";"	   ] # Short description of PAF
PAF.CRTE.NAME           [format %-26s "\"[Get1 PAF.CRTE.NAME]\";"  ] # Name of creator
PAF.CRTE.DAYTIM         [format %-26s "\"[Get1 PAF.CRTE.DAYTIM]\";"] # Civil time for creation
PAF.LCHG.NAME           [format %-26s "\"[set user]\";"		   ] # Name of person/appl. chg.
PAF.LCHG.DAYTIM         [format %-26s "\"[set now]\";"		   ] # Timestamp of last change
PAF.CHCK.NAME           [format %-26s "\"[Get1 PAF.CHCK.NAME]\";"  ] # Name of appl. checking
PAF.CHCK.DAYTIM         [format %-26s "\"[Get1 PAF.CHCK.DAYTIM]\";"] # Time for checking
PAF.CHCK.CHECKSUM       [format %-26s "\"[Get1 PAF.CHCK.CHECKSUM]\";"] # Checksum for the PAF
PAF.HDR.END;

# --------------------------------------------------------------------
OBS.ID                  [format %-26s "\"[Get OBS.ID]\";"	  ] # Observation block ID
OBS.NAME                [format %-26s "\"[Get OBS.NAME]\";"	  ] # OB name
OBS.GRP                 [format %-26s "\"[Get OBS.GRP]\";"	  ] # linked blocks
OBS.PROG.ID             [format %-26s "\"[Get OBS.PROG.ID]\";"	  ] # ESO program identification
OBS.PI-COI.ID           [format %-26s "\"[Get OBS.PI-COI.ID]\";"  ] # ESO internal PI-COI ID
OBS.PI-COI.NAME         [format %-26s "\"[Get OBS.PI-COI.NAME]\";"] # PI-COI name

"
	#*# 2. add each observation
	foreach obj [Contents] {
	    append txt [$obj Puts]
	}

	#*# 3. trailer
	append txt "\n#___oOo___\n"

	return $txt
    }

    #****************************************************************
    #*# method: Add
    #----------------------------------------------------------------

    public method Add { obj {rmdup 0} } {

    	#*# 1. if duplicates are allowed: add the object.
    	if { !$rmdup } {
    	    AddObj $obj
	    return 1
	}

	# 2. if object not contained: add object
	set add 0
	if {![Contains $obj]} {
    	    AddObj $obj
	    set add 1
	}

	return $add
    }

    #****************************************************************
    #*# method: Contains
    #----------------------------------------------------------------

    public method Contains { obs } {

	set ret 0
	foreach obj [Contents] {
	    if {[tpltooPAF::IsEqual $obj $obs]} {
	    	set ret 1
		break
	    }
	}

	return $ret
    }

    #****************************************************************
    #*# method: Sort
    #----------------------------------------------------------------

    public method Sort {} {
    	set ctooObjects [lsort -command "tpltooOBD::SortCmp" $ctooObjects]
    }

    #****************************************************************
    #*# proc: SortCmp
    #----------------------------------------------------------------

    public proc SortCmp { obj1 obj2 } {

    	#*# 1. compare sort keys
    	set val1 [$obj1 cget -sortkey]
	set val2 [$obj2 cget -sortkey]
	set cmp [string compare $val1 $val2]

	#*# 2. if sort keys are the same, sort by objId
	if { $cmp == 0 } {
	    set val1 [$obj1 cget -objId]
	    set val2 [$obj2 cget -objId]
	    set cmp [expr {$val1 - $val2}]
	}

	return $cmp
    }


    #****************************************************************
    #*# method: Merge
    #----------------------------------------------------------------

    public method Merge {} {

    	#*# 1. process each observation:
    	set prevObs   ""
	set prevTplId ""
	set prevList  ""
	set newObjs ""
	foreach obs $ctooObjects {

	    #*# 2. if observation uses a different TPL.ID, or
	    #*#    if observations are not equal: continue
	    if { $prevObs == "" || [$obs Get TPL.ID] != $prevTplId || 
	    	 [tpltooPAF::IsEqual $prevObs $obs "$prevList SEQ.NEXPO"] == 0 } {
	    	lappend newObjs $obs
		catch {
		    set prevList  [$obs Get CALIB.TPL.LIST]
		    set prevTplId [$obs Get TPL.ID]
		    set prevObs   $obs
		}
		continue
	    }

	    #*# 3. merge observations
	    $prevObs Merge $obs $prevList
	    delete object $obs
	}

	#*# 4. set new observations list
	set ctooObjects $newObjs
    }

    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable init ""

    protected variable oi 1
}

#
# ___oOo___
