#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tpltooPAF.tcl,v 1.15 2005/09/12 02:25:17 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# rschmutz  2005-08-14  ResolveVar: env. variables lookup fixed.
# rschmutz  26/11/01    place TEL.AG.* before TEL.GS* keywords.
# rschmutz  07/06/01    created (based in calobPAF.tcl).
#

#************************************************************************
#   NAME
#	tpltooPAF - configuration file interface
# 
#   SYNOPSIS
#	tpltooPAF paf name
# 
#   DESCRIPTION
#	Itcl interface to the configuration stored either in:
#	the INS_ROOT, INTROOT or VLTROOT.
#
#   PARENT
#	ctooOBJ_BASE
#
#   PUBLIC METHODS
#	tpltooPAF paf [-dict <dictionaries>] [-file <filename>]
#	    Constructor.  Loads dictionaries.
#	    To load the PAF file, call Load.
#
#	Clear
#	    Clear object.
#
#	Load [<filename>]
#	    Load the PAF file <filename>
#
#	Get <parameter>
#	    Returns the value of a parameter.
#
#	Set <parameter> <value>
#	    Set a parameter value.
#
#	GetKeywords
#	    Returns the list of parameters.
#
#	Save
#	    Saves the parameters.
#
#	SaveAs <name>
#	    Saves the file <name>.
#
#	Puts [<exclist>]
#	    Returns the list of parameters in 
#	    short FITS format (ready to be saved).
#	    <sys>.* keywords are not included, if
#	    <sys> is listed in <exclist>.
#
#	RefPAF <name> <obj>
#	    Register a reference PAF file or image.
#	    (of type tpltooPAF or tpltooIMAGE).
#
#	ResolveAll
#	    Resolve all variables and expressions.
#
#   ENVIRONMENT
#	INS_ROOT	Instrument root area
#       INTROOT         Integration area
#       VLTROOT         VLT area
#
#   CAUTION
#
#   EXAMPLES
#
#   SEE ALSO
#	tpltooBuild(1) tpltooTSF(8) tpltooOBD(8) tpltooOBS(8)
#
#   BUGS
#
#------------------------------------------------------------------------
#


#************************************************************************
#*# proc tpltooRegisterCmd
#------------------------------------------------------------------------

proc tpltooRegisterCmd { name cmd } {
    global tpltooCmd

    set tpltooCmd($name) $cmd
}


#************************************************************************
#*# class: tpltooPAF
#------------------------------------------------------------------------

class tpltooPAF {
    inherit ctooOBJ_CONTAINER tpltooLOG

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
        ctooOBJ_BASE::constructor -objType PAF
	eval configure $args

	#*# 1. check name
	if { "$name" == "" || "[cindex $name 0]" == "-" } {
	    error "Wrong argument $name."
	}
    }

    #****************************************************************
    #*# method: Clear
    #----------------------------------------------------------------

    public method Clear {} {
    	set params ""
	set values ""
    }

    #****************************************************************
    #*# method: LoadLine
    #----------------------------------------------------------------

    protected method LoadLine { txt {multiline 0} } {

	#*# 1. ignore empty lines and comments
	set txt [string trim $txt]
	if { $txt == "" || [cindex $txt 0] == "#" } {
	    return ""
	}

	#*# 2. multiline support:
	#
	#      if inside a multiline (inLineTxt != ""), then
	#	   append the current line to inLineTxt.
	#      If the line finishes now (line contains a single
	#      double quote), then assign everthing to txt
	#      (simulating that everything was received in a 
	#      single line).
	#
	#	   If not inside a multiline, then check if we are
	#      starting one (line contains a single double quote).
	#      In this case, save the line in inLineTxt and return.
	#
	if { $multiline } {
	    # retrieve number of double quotes
	    set txt2 ""
	    regsub -all {[^"]} $txt "" txt2
	    #"
	    set txt2len [clength $txt2]

	    # if inside a multiline: continue it or finish it.
	    if { $inLineTxt != "" } {
		append inLineTxt " $txt"
		if { $txt2len == 1 } {
		    set txt $inLineTxt
		    set inLineTxt ""
		} else {
		    return ""
		}

	    # if not inside a multiline: check if we are starting one.
	    } else {
		if { $txt2len == 1 } {
		    set inLineTxt $txt
		    return ""
		}
	    }
	}

	#*# 3. get parameter name
	set param [ctoken txt " ;\t\n\r"]
    
	#*# 4. get parameter value
	set val ""
	set txt [string trim $txt]
	if { [cindex $txt 0] != ";" && $txt != "" } {
	    if {![regexp {"(.*)";?[ \t]*(#.*)?} $txt dummy val]} {
		if { $multiline } {
	            error "Syntax error loading $file: '$param $txt'\n       (a double quote may be missing in any line above)."
		} else {
	            error "Syntax error loading $file: '$param $txt'."
		}
	    }
	}

	return [list $param $val]
    }

    #****************************************************************
    #*# method: Load
    #----------------------------------------------------------------

    public method Load { {fileName ""} {multiline 0} } {

    	#*# 1. get filename
    	if { "$fileName" != "" } {
    	    configure -file $fileName
	}
	if { "$file" == "" } {
	    return
	}
	if {![file exists $file]} {
	    error "File $file not found."
	}

	#*# 2. load file
	set inLineTxt ""
	for_file txt $file {
	    set line [LoadLine $txt $multiline]
	    if { "$line" != "" } {
	        lassign $line param val
	        Set $param $val
	    }
	}
    }

    #****************************************************************
    #*# method: Get1
    #----------------------------------------------------------------

    protected method Get1 { param } {
    	set i [lsearch $params $param]
	if { $i == -1 } {
	    error "Parameter $param not found."
	}
    	set val [lindex $values $i]
    }

    #****************************************************************
    #*# method: Get
    #----------------------------------------------------------------

    public method Get { param } {
    	set c1 [cindex $param 0]
        if { $c1 == "\[" || $c1 == "\$" } {
            return [ResolveString "$param"]
        } else {
            return [ResolveString "\$$param"]
        }
    }


    #****************************************************************
    #*# method: Set
    #----------------------------------------------------------------

    public method Set { param val } {
	if { [csubstr $val 0 1] == "\"" && [csubstr $val end 1] == "\"" } {
	    set val [csubstr $val 1 [expr {[clength $val] - 2}]]
	}
    	set i [lsearch $params $param]
	if { $i != -1 } {
	    set values [lreplace $values $i $i $val]
	} else {
	    lappend params $param
	    lappend values $val
	}
    }

    #****************************************************************
    #*# method: GetKeywords
    #----------------------------------------------------------------

    public method GetKeywords {} {
	return $params
    }

    #****************************************************************
    #*# method: Save
    #----------------------------------------------------------------

    public method Save {} {
    	if { $file == "" } {
	    error "Can't save PAF file $name: filename not known."
	}
    	SaveAs $file
    }

    #****************************************************************
    #*# method: SaveAs
    #----------------------------------------------------------------

    public method SaveAs { fileName } {

    	#*# 1. check file is not read-only
	if {[file exists $fileName]} {
    	    if {![file writable $fileName]} {
	        error "File $fileName is read-only.  Unable to save."
	    }
	}

	#*# 2. save file
	write_file $fileName [Puts]
	return $fileName
    }

    #****************************************************************
    #*# method: Puts
    #----------------------------------------------------------------

    public method Puts { {exclist ""} } {

    	#*# 1. if no PAF: remove from parameter list
	set paramlist [GetKeywords]
	if { $exclist != "" } {
	    set paramlist1 $paramlist
	    set paramlist ""
	    foreach param $paramlist1 {
	    	if {![lcontain $exclist [csubstr $param 0 [string first "." $param]]]} {
		    lappend paramlist $param
		}
	    }
	}

	#*# 2. sort keywords
	set paramlist1 [SortKeywords $paramlist]

    	#*# 3. process all keywords
	set txt ""
	foreach param $paramlist1 {
	    append txt "[format %-23s $param] \"[Get1 $param]\";\n"
	}

	append txt "\n"
	return $txt
    }

    #****************************************************************
    #*# method: SortKeywords
    #----------------------------------------------------------------

    public method SortKeywords { paramlist } {

    	#*# 1. set category order
        set catOrder "TPL DET1 INS1 DET2 INS2 DET3 INS3 DET4 INS4 DET5 DET6 DET SEQ TEL INS"
	catch { set catOrder [Get TPL.SORT.CATEGORY] }
	if { ![lcontain $catOrder "TPL"] } {
	    set catOrder "TPL $catOrder"
	}

	#*# 2. set parameter order
	#      (if no DET or TEL orders are specified, then use default orders)
	set paramOrder ""
	catch { set paramOrder [Get TPL.SORT.PARAMS] }
	if { [lmatch $paramOrder "DET*"] == "" } {
	    set detParamOrder "DET*.EXP.TYPE DET*.READ.SPEED DET*.DIT DET*.NDIT \
	    	    DET*.WIN1.UIT* DET*.WIN1.ST DET*.WIN1.STR* DET*.WIN1.N* DET*.WIN1.* \
	    	    DET*.WIN2.UIT* DET*.WIN2.ST DET*.WIN2.STR* DET*.WIN2.N* DET*.WIN2.*"
	    set paramOrder "$detParamOrder $paramOrder"
	}
	if { [lmatch $paramOrder "TEL.*"] == "" } {
	    set telParamOrder "TEL.TARG.TYPE TEL.TARG.ALPHA TEL.TARG.DELTA \
	    	    TEL.TARG.EQUINOX TEL.TARG.OFFSET* TEL.ROT.* \
		    TEL.TARG.ADDVEL* TEL.TARG.PM* TEL.TARG.* TEL.AG* TEL.GS*"
	    set paramOrder "$paramOrder $telParamOrder"
	}

	# 3. order keywords
	set paramlist1 ""
	foreach cat $catOrder {
	    append paramlist1 " [SortParam $paramOrder [lmatch $paramlist "$cat.*"]]"
	}
	append paramlist1 " [SortParam $paramOrder [lindex [intersect3 $paramlist $paramlist1] 0]]"

	return $paramlist1
    }

    #****************************************************************
    #*# method: SortParam
    #----------------------------------------------------------------

    protected method SortParam { paramOrder paramList } {

    	#*# 1. if list is empty: return
    	if { $paramList == "" } {
	    return ""
	}

	#*# 2. extract parameters in paramOrder
	set paramlist1 ""
	foreach param $paramOrder {
	    set param1 [lsort [lmatch $paramList $param]]
	    if { $param1 != "" } {
		append paramlist1 " $param1"
		set paramList [lindex [intersect3 $paramList $paramlist1] 0]
	    }
	}

	#*# 3. append the missing parameters
	append paramlist1 " [lsort $paramList]"

	return $paramlist1
    }

    #****************************************************************
    #*# method: RefPAF
    #----------------------------------------------------------------

    public method RefPAF { refname obj } {
    	lappend refnames $refname
	lappend refobjs  $obj
	if { $refname == "ISF" } {
	    set refisf 1
	}
    }

    #****************************************************************
    #*# method: ResolveAll
    #----------------------------------------------------------------

    public method ResolveAll {} {
        foreach param [GetKeywords] {
	    Set $param [Get $param]
	}
    }

    #****************************************************************
    #*# method: ResolveString
    #----------------------------------------------------------------

    protected method ResolveString { txt } {

    	#*# 1. if ISF: accept "ISF <key>" references
	if { $refisf == 1 && [string first "ISF " $txt] == 0 } {
	    set txt [cconcat {$ISF:} [crange $txt 4 end]]
	}

        #*# 2. Resolve variables
        while { [string first "\$" $txt ] != -1 } {
	    # extract text before var., var. name and after var.
	    if {[regexp {([^$]*)\$([A-Za-z0-9_\.:\-]+)(.*)} $txt dummy txt1 varname txt3]} {
		# if found: search config. var. or env. var.
		set txt2 [ResolveVar $varname]
	        set txt [cconcat $txt1 $txt2 $txt3]
	    } elseif {[regexp {([^$]*)\$\(([A-Za-z0-9_\.:\-]+)\)(.*)} $txt dummy txt1 varname txt3]} {
		set txt2 [ResolveVar $varname]
	        set txt [cconcat $txt1 $txt2 $txt3]
	    } else {
		break
	    }
	    if { $refisf == 1 && [string first "ISF " $txt] == 0 } {
	        set txt [cconcat {$ISF:} [crange $txt 4 end]]
	    }
        }
        
	#*# 3. recursive resolve '[...]'
	while {[regexp {(.*)\[([^][]+)\](.*)} $txt dummy txt1 expr txt3]} {
	    set txt2 [ResolveExpr $expr]
	    set txt [cconcat $txt1 $txt2 $txt3]
	}

        #*# 4. Resolve shell variables  
        if { [string first "~" $txt ] != -1 } {
	    catch {exec csh -cft "echo $txt"} txt
        } 

        return [string trim $txt " "]
    }

    #****************************************************************
    #*# method: ResolveVar
    #----------------------------------------------------------------

    protected method ResolveVar { varname } {
        global env

	#*# 1. default: return original text
    	set txt "#NOT_FOUND#"

	#*# 2. search reference PAF
	Debug "Varname='$varname'." 4
	if {[regexp {([^:]+):(.+)} $varname dummy txt1 txt2]} {
	    set i [lsearch $refnames $txt1]
	    if { $i == -1 || [string first ":" $txt2] != -1 } {
	        #*# 2.1 if reference not found: search PAF variable
	        try_eval {
		    Debug "Regexp = RefNotFound1: '$txt1'." 4
	            set txt [Get1 $txt1]
		    set txt [cconcat $txt ":" $txt2]
	        } {
	    	    #*# 2.2 else: search env. variable
		    Debug "Regexp = RefNotFound2: '$txt1'." 4
	            catch {
		    	set txt $env($txt1)
		        set txt [cconcat $txt ":" $txt2]
		    }
	        }
	    } else {
	    	# 2.3 get referenced variable
		Debug "Regexp = RefFound2: '$txt1' '$txt2'." 4
	        set obj [lindex $refobjs $i]
	        catch { set txt [$obj Get $txt2] }
	    }

	} else {
	    #*# 3. else: search PAF variable
	    Debug "Local1 = Loc '$varname'." 4
	    catch { set txt [Get1 $varname] }

	    #*# 4. search in all reference PAF
	    if { $txt == "#NOT_FOUND#" } {
		Debug "Local2 = Ref '$varname'." 4
		foreach obj $refobjs {
		    catch {
		    	set txt [$obj Get $varname]
			break
		    }
		}
	    }

	    #*# 5. search user variable
	    if { $txt == "#NOT_FOUND#" } {
		Debug "Local3 = Usr '$varname'." 4
		if { "$uservars" != "" } {
		    catch { set txt [$uservars Get $varname] }
		}
	    }

	    #*# 6. search env. variable
	    if { $txt == "#NOT_FOUND#" } {
		Debug "Local4 = Env '$varname'." 4
	        catch { set txt $env($varname) }
	    }
	}

	if { $txt == "#NOT_FOUND#" } {
	    error "Variable $varname not found."
	}
	Debug "Return  = '$txt'.\n" 4
	return $txt
    }

    #****************************************************************
    #*# method: ResolveExpr
    #----------------------------------------------------------------

    protected method ResolveExpr { txt } {
    	global tpltooCmd

    	#*# 1. search cmd.
	Debug "Expr = '$txt'" 4
	set cmd [ctooStrFirst $txt]
	set arg [ctooStrTail $txt]
	switch $cmd {
	    TABLE { return [ResolveTable $arg] }
	    EXPR  {
		if { [csubstr $arg 0 1] == "\"" && [csubstr $arg end 1] == "\"" } {
	    	    set arg [csubstr $arg 1 [expr {[clength $arg] - 2}]]
		}
	    	set val [expr $arg]
		return [expr {round(100.0*$val)/100.0}]
	    }
	    LENGTH { return [llength $arg] }
	    TEXT { return [ResolveString $arg] }
	    PARAMFILE {
	    	if { [llength $arg] != 1 } {
		    error "PARAMFILE $arg: Wrong number of arguments (PARAMFILE <file>)."
		}
		if { ! [file exists $arg] } {
        	    error "File $arg not found."
		}
		set val [read_file -nonewline $arg]
		regsub -all "\"" $val "\\\"" val
		regsub -all "\n" $val "\\n" val
		return $val
	    }
	    default {
	    	try_eval {
	    	    set usercmd $tpltooCmd($cmd)
		} {
	    	    error "Unknown command $cmd."
		}
		eval $usercmd $arg
	    }
	}
    	
    }

    #****************************************************************
    #*# method: ResolveTable
    #----------------------------------------------------------------

    protected method ResolveTable { txt } {

    	#*# 1. get table and args.
	set table [ctoken txt " \t\n\r"]
	set arg [join $txt " "]
	if { "$tables" != "" } {
	    return [$tables Get $table $arg]
	} else {
	    error "Table '$table' for '\[TABLE $table $arg\]' not found."
	}
    }

    #****************************************************************
    #*# proc: IsEqual
    #----------------------------------------------------------------

    public proc IsEqual { paf1 paf2 {exclist {}} } {

    	#*# 1. retrieve keyword lists
    	set paramlist1 [lsort [$paf1 GetKeywords]]
	set paramlist ""
	foreach param $paramlist1 {
	    if { [crange $param 0 3] != "PAF." } {
	    	lappend paramlist $param
	    }
	}
	set paramlist1 $paramlist

    	set paramlist2 [lsort [$paf2 GetKeywords]]
	set paramlist ""
	foreach param $paramlist2 {
	    if { [crange $param 0 3] != "PAF." } {
	    	lappend paramlist $param
	    }
	}
	set paramlist2 $paramlist
    	
	#*# 2. if lists are not identical: return
	if { $paramlist1 != $paramlist2 } {
	    return 0
	}

	#*# 3. compare keywords
	set ret 1
	foreach param $paramlist1 {
	    if {![lcontain $exclist $param]} {
	        if { "[$paf1 Get $param]" != "[$paf2 Get $param]" } {
	    	    set ret 0
		    break
		}
	    }
	}

	return $ret
    }


    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable file        ""
    public variable tables      ""
    public variable uservars    ""

    protected variable params   ""
    protected variable values   ""
    protected variable refnames ""
    protected variable refobjs  ""
    protected variable refisf   0

    private variable inLineTxt  ""
}

#
# ___oOo___
