#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: tpltooSCRIPTS.tcl,v 1.15 2005/09/12 02:25:17 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mkiekebu  2005-06-22  used package name for tsf report output file.
# rschmutz  2005-05-19  SPR VLTSW20050185: ObsBuildAll: allow dup. tsfs.
# mkiekebu  2005-05-08  added method TsfReport
# rschmutz  2004-09-04  SPR 20030499: use tsf VALUE instead of TEST.VALUE.
# rschmutz  20/06/01    *.obdv support added.
# rschmutz  07/06/01    created.
#

#************************************************************************
#   NAME
#	tpltooSCRIPTS - tpltoo script code
# 
#   SYNOPSIS
#	tpltooSCRIPTS name
# 
#   DESCRIPTION
#	This class contains the main code for the tpltoo utilities.
#
#   PARENT CLASS
#	ctooOBJ_REPOSITORY tpltooLOG
#
#   PUBLIC METHODS
#	ObdBuild <tsfList>
#	    Build a test *.obd file for each *.tsf file in tsfList.
#
#	ObdBuildAll <tsfList> <fileName>
#	    Build a test *.obd file to execute all *.tsf in tsfList.
#
#	TsfKeySort <tsfList>
#	    Sort the keywords in the *.tsf files in tsfList.
#
#   FILES
#	*.tsf:	template signature files
#	*.isf:	instrument signature file
#	*.obdv: observation block values file
#	*.obd:	observation block
#
#   SEE ALSO
#	tpltooCalob
#
#   BUGS
#
#------------------------------------------------------------------------
#

#************************************************************************
#*# class: tpltooSCRIPTS
#------------------------------------------------------------------------

class tpltooSCRIPTS {
    inherit ctooOBJ_REPOSITORY tpltooLOG

    #****************************************************************
    #*# constructor
    #----------------------------------------------------------------

    constructor { args } {
        global env

	#*# 1. process arguments
	configure -name tpltooSCRIPTS
    	eval configure $args

	#*# 2. read ISF file
	if { $isfFile != "" } {
	    tpltooISF $this.isf -name [file tail $isfFile] -file $isfFile
	    AddObj $this.isf
	}

	#*# 3. set obdDir
	if { $obdDir == "" } {
	    set obdDir "$env(INS_ROOT)/SYSTEM/COMMON/TEMPLATES/OBD"
	}

	#*# 4. check tsfDir
	if { $tsfDir != "" } {
	    if { ![file isdirectory $tsfDir] } {
	        error "Destination directory $tsfDir does not exist."
	    }
	}
    }

    #****************************************************************
    #*# method ObdBuild
    #----------------------------------------------------------------

    public method ObdBuild { tsfList } {

    	#*# 0. Init.
	Debug $debug
	if { ![file isdirectory $obdDir] } {
	    error "Destination directory $obdDir does not exist."
	}

	#*# 1. process each TSF file
	foreach tsfFile $tsfList {
	    Info "Processing [file tail $tsfFile] ..."

	    #*# 2. load TSF file
	    set rootname [file rootname [file tail $tsfFile]]
	    tpltooTSF $this.tsf -name $rootname -file $tsfFile
	    $this.tsf RefPAF ISF $this.isf

	    #*# 3. load OBDV file
	    set obdvFile [cconcat [file rootname $tsfFile] ".obdv"]
	    if { [file exists $obdvFile] } {
	    	tpltooOBDV $this.obdv -name [cconcat $rootname ".obdv"] -file $obdvFile
		$this.tsf RefPAF OBDV $this.obdv
	    }

	    #*# 4. create observation
	    tpltooOBS $this.$rootname -name $rootname
	    $this.$rootname RefPAF ISF $this.isf
	    $this.$rootname Set TPL.ID   [$this.tsf Get PAF.NAME]
	    $this.$rootname Set TPL.NAME [$this.tsf Get PAF.DESC]

	    #*# 5. process all keywords
	    set keywList [lmatch [$this.tsf GetKeywords] {*.TYPE}]
	    set keywList [lindex [intersect3 $keywList {PAF.TYPE TPL.TYPE}] 0]
	    foreach keywType $keywList {
	    	regsub {.TYPE$} $keywType "" keyw
		set val "NODEFAULT"
		catch { set val [$this.tsf  Get $keyw.DEFAULT] }
		catch { set val [$this.tsf  Get $keyw.TEST.VALUE] }
		catch { set val [$this.tsf  Get $keyw.VALUE] }
		catch { set val [$this.obdv Get $keyw] }
		$this.$rootname Set $keyw $val
	    }

	    #*# 6. save OBD file
	    set obdFile [file join $obdDir [cconcat $rootname ".obd"]]
	    tpltooOBD $this.obd -name OBD -file $obdFile -init $this.isf
	    AddObj $this.obd
	    $this.obd Add $this.$rootname
	    $this.obd Save

	    #*# 7. delete processed objects
	    DeleteObj $this.obd
	    delete object $this.obd
	    delete object $this.$rootname
	    catch { delete object $this.obdv }
	    delete object $this.tsf
	}
    }

    #****************************************************************
    #*# method ObdBuildAll
    #----------------------------------------------------------------

    public method ObdBuildAll { tsfList fileName } {

    	#*# 0. Debug
	Debug $debug

	#*# 1. get filename
	if { [file tail $fileName] == $fileName } {
	    if { ![file isdirectory $obdDir] } {
	        error "Destination directory $obdDir does not exist."
	    }
	    set fileName [file join $obdDir $fileName]
	}

    	#*# 2. check file is not read-only
	if {[file exists $fileName]} {
    	    if {![file writable $fileName]} {
	        error "File $fileName is read-only.  Unable to save."
	    }
	}

        #*# 3. create OBD object
	tpltooOBD $this.obd -name OBD -file $fileName -init $this.isf
	AddObj $this.obd

	#*# 4. process each TSF file
	set obsList ""
	set index 0
	foreach tsfFile $tsfList {
	    Info "Processing [file tail $tsfFile] ..."
	    incr index

	    #*# 5. load TSF file
	    set rootname [file rootname [file tail $tsfFile]]
	    tpltooTSF $this.tsf -name $rootname -file $tsfFile
	    $this.tsf RefPAF ISF $this.isf

	    #*# 6. load OBDV file
	    set obdvFile [cconcat [file rootname $tsfFile] ".obdv"]
	    if { [file exists $obdvFile] } {
	    	tpltooOBDV $this.obdv -name [cconcat $rootname ".obdv"] -file $obdvFile
		$this.tsf RefPAF OBDV $this.obdv
	    }

	    #*# 7. create observation
	    tpltooOBS $this.obs$index -name $rootname
	    $this.obs$index RefPAF ISF $this.isf
	    $this.obs$index Set TPL.ID   [$this.tsf Get PAF.NAME]
	    $this.obs$index Set TPL.NAME [$this.tsf Get PAF.DESC]

	    #*# 8. process all keywords
	    set keywList [lmatch [$this.tsf GetKeywords] {*.TYPE}]
	    set keywList [lindex [intersect3 $keywList {PAF.TYPE TPL.TYPE}] 0]
	    foreach keywType $keywList {
	    	regsub {.TYPE$} $keywType "" keyw
		set val "NODEFAULT"
		catch { set val [$this.tsf Get $keyw.DEFAULT] }
		catch { set val [$this.tsf Get $keyw.TEST.VALUE] }
		catch { set val [$this.tsf Get $keyw.VALUE] }
		catch { set val [$this.obdv Get $keyw] }
		$this.obs$index Set $keyw $val
	    }

	    #*# 9. add observation to OBD
	    $this.obd Add $this.obs$index

	    #*# 10. delete TSF object
	    lappend obsList $this.obs$index
	    catch { delete object $this.obdv }
	    delete object $this.tsf

	}

	#*# 11. save OBD file
	Info "Saving $fileName."
	$this.obd Save

	#*# 12. delete processed objects
	DeleteObj $this.obd
	delete object $this.obd
	foreach obsObj $obsList {
	    delete object $obsObj
	}
    }

    #****************************************************************
    #*# method TsfKeySort
    #----------------------------------------------------------------

    public method TsfKeySort { tsfList } {

    	#*# 0. Debug
	Debug $debug

	#*# 1. process each TSF file
	foreach tsfFile $tsfList {
	    Info "Sorting [file tail $tsfFile] ..."

	    #*# 2. load TSF file
	    set rootname [file rootname [file tail $tsfFile]]
	    tpltooTSF $this.tsf -name $rootname -file $tsfFile
	    $this.tsf RefPAF ISF $this.isf

	    #*# 3. save TSF file (this will sort the keywords)
	    if { $tsfDir != "" } {
	        set newTsfFile [file join $tsfDir [cconcat $rootname ".tsf"]]
	    } else {
	    	set newTsfFile $tsfFile
	    }
	    $this.tsf SaveAs $newTsfFile

	    #*# 4. delete TSF object
	    delete object $this.tsf
	}
    }

    public method TsfReport { tsfList {format "txt"} {outDir {}}} {
	Debug $debug

	set fmt [string toupper $format]
	tpltooFMT_$fmt $this.fmt -dir $outDir -repName [file rootname [$this.isf cget -name]]
	$this.fmt addHeader [file rootname [$this.isf cget -name]] [$this.isf Get VERSION]

	foreach tsfFile $tsfList {
	    Info "Processing [file tail $tsfFile] ..."	    
	    
	    set rootname [file rootname [file tail $tsfFile]]
	    tpltooTSF $this.tsf -name $rootname -file $tsfFile
	    $this.tsf RefPAF ISF $this.isf

	    
	    set keywList [lmatch [$this.tsf GetKeywords] {*.TYPE}]
	    set keywList [lindex [intersect3 $keywList {PAF.TYPE TPL.TYPE}] 0]
	   
	    foreach keywType $keywList {
		
	    	regsub {.TYPE$} $keywType "" keyw
		set value "NODEFAULT"
		set range ""
		set default "NODEFAULT"
		set label ""
		set hide "no"
		catch { set default [$this.tsf Get $keyw.DEFAULT] }		
		catch { set value   [$this.tsf Get $keyw.VALUE] }			       
		catch { set range   [$this.tsf Get $keyw.RANGE] }
		catch { set label   [$this.tsf Get $keyw.LABEL] }
		catch { set hide    [$this.tsf Get $keyw.HIDE] }

		if {"$hide" == "OHS"} {
		    set hide "yes"
		}

		if {"$value" == "NODEFAULT"} {
		    $this.fmt addTableRowVar $keyw $hide $range $default $label		    
		} else {
		    $this.fmt addTableRowFix $keyw $hide $value $label
		}
	    }
	    $this.fmt createTable [file tail $tsfFile]
	    delete object $this.tsf
	}
	
	$this.fmt addFooter "ss"
	delete object $this.fmt
    }


    #****************************************************************
    #*# variables
    #----------------------------------------------------------------

    public variable isfFile ""
    public variable obdDir  ""
    public variable tsfDir  ""
    public variable debug   ""
}


#___oOo___



