#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccCompletion.tcl,v 2.116 2005/10/21 14:15:11 vltsccm Exp $" 
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 04.10.1995 Created from ccseiComplLib.
# imunoz     29.03.1995 Ported to itcl2.0

#************************************************************************
#   NAME
#	VccCompletion - abstract class for completion operations
#
#   SYNOPSIS
#	inherit VccCompletion
#
#   DESCRIPTION
#	The VccCompletion command creates a new object, used for
#	completion of words based on search-lists.
#
#	Several modes - including completion on both ends of a word - can 
#	be selected.
#
#   INHERITANCE
#	Internal debugging support from VccDebug(n).
#
#   PUBLIC OPTIONS
#	The following options are specific to objects of this class:
#
#	-completionMode
#		Can be a combination of:
#		    enabled	- no operation if not set
#		    ignorecase	- ignore case for completion
#		    leading	- complete at beginning of item
#		    trailing	- complete at end of item
#
#		Default is: {enabled ignorecase leading trailing}
#
#	-completionIgnore
#		is a list of items to be ignored in the argument list given
#		to the operations `new' and `add'.
#		Default is: {""}	(i.e. empty strings are ignored)
#
#   METHODS
#	The VccSelector command creates a new command whose name is <pathName>.
#	This command may be used to invoke the `Completion' method:
#
#	Completion <Operation> <Item> ?<Mode>?
#
#	Operation: specifies one of the following operations:
#
#	  new	Create new completion list consisting of the word(s) `Item', 
#		overwrite the list if already existing.
#
#	  add	Add the word(s) in `Item' to the completion list.
#
#	  all	Determine all possible completions for word `Item' from the
#		completion list, and return them as a list.
#
#	  get	Return a unique completion for word `Item' from the
#		completion list, or an empty string if there is none.
#
#	  best	Return the best possible completion for word `Item' from the
#		completion list. Normally this will be the longest common
#		part of all matches, or simply the given word itself.
#		An empty string is returned if there is no completion at all.
#
#	  info	Return information on internal state as a text string.
#
#	Item:	single search-item or list, depends on Operation, see there
#
#	Mode:	to override temporarily the default `completionMode',
#		except for the `enabled' keyword, which cannot be overridden.
#
#	Return value depends on Operation, see there.
#
#   COMMON PROCEDURES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#	VccDebug(n)
#
#   TO DO
#
#------------------------------------------------------------------------

class VccCompletion {

    inherit VccDebug
    
    constructor { args } {}
    destructor {}

    public method Completion { Operation Item {Mode ""} }  

    public variable completionMode {enabled ignorecase leading trailing}
    public variable completionIgnore {""} ;# list of items to be ignored
    protected variable completionList {}  ;# list of valid completions
}

body VccCompletion::Completion { Operation Item {Mode ""} } {
    debugEnter
    
    # first check if completion is at all enabled:
    if {[lsearch $completionMode enabled] == -1} {
	debugPrint "completion not enabled. Operation changed to `none'."
	set Operation none ;# to fall through
    }

    if {$Mode == ""} {
	set Mode $completionMode
    }

    # set flags for easier use:
    #   for backward compatibility also allowed:
    #   atfirst == leading, atlast == trailing
    set m_ignorecase [llength [lmatch -regexp $Mode ignorecase      ]]
    set m_leading    [llength [lmatch -regexp $Mode leading|atfirst ]]
    set m_trailing   [llength [lmatch -regexp $Mode trailing|atlast ]]

    # fork depending on Operation:
    switch -- [string tolower $Operation] {
	new {
	    # fill completionList with all in Item except completionIgnore:
	    set completionList \
		    [lindex [intersect3 $Item $completionIgnore] 0]
	    set returnValue $completionList
	}
	add {
	    # extend completionList with Item except completionIgnore:
	    set completionList [lindex [intersect3 \
		    [concat $completionList $Item] $completionIgnore] 0]
	    set returnValue $completionList
	}
	all {
	    set completions {}
	    set item [lindex $Item 0]
	    set pattern $item
	    if {$m_ignorecase} {set pattern [string tolower $pattern]}
	    if {$m_leading}    {set pattern "*$pattern"}
	    if {$m_trailing }    {set pattern "$pattern*"}
	    set clist $completionList
	    set wlist $clist
	    if {$m_ignorecase} {set wlist [string tolower $wlist]}
	    #->
	    set i 0 ;# pos of item in sub-list
	    set j 0 ;# start-pos of sub-list in original list
	    while {[set i [lsearch -glob $wlist $pattern]] >= 0} {
		lappend completions [lindex $clist [expr {$j+$i}]]
		set wlist [lrange $wlist [incr i] end] ;#remaining sublist
		incr j $i
	    }
	    set returnValue $completions
	    #<- NOTE: `lmatch' isn't possible, no ignore case supported:
	    #set returnValue [lmatch -glob $wlist $pattern]
	}
	get {
	    set completions [Completion all $Item]
	    if {[llength $completions] > 1} {
		set returnValue {}	;# not unique ==> empty string
	    } else {
		set returnValue [lindex $completions 0]
	    }
	}
	best {
	    set completions [Completion all $Item]	;# get all possibilites
	    if {[llength $completions] > 1} {
		# don't use leading completion mode for further tries:
		set nmode [intersect $Mode {trailing ignorecase}]
		set completions [Completion all $Item $nmode]
		# abort if there are no completion with this new mode:
		if {[lempty $completions]} {
		    # no sense for further tries:
		    set returnValue $Item	;# return item unchanged
		} else {
		    # trailing completion ==> find longest common part,
		    # shorten a possible completion step by step until
		    # the completions for it are the same as for original:
		    set try [lindex $completions 0]	;# try with any match
		    set i1  [string length $try]	;# begin with match
		    set i2  [string length $Item]	;# end with original
		    for {set i $i1} {$i >= $i2} {incr i -1} {
			set try [string range $try 0 [expr {$i - 1}]]
			# break when found completions are the same:
			if {[Completion all $try $nmode] == $completions} {
			    break
			}
		    }
		    set returnValue $try	;# return longest common part
		}
	    } else {
		set returnValue [lindex $completions 0]
	    }
	}
	info {
	    set returnValue\
		    "($this):\n\
		    completionMode: $completionMode\n\
		    completionList: $completionList\n\
		    ([llength $completionList] items)\n"
	}
	none {
	    set returnValue {}
	}
	default {
	    error "invalid completion operation ($Operation)"
	}
    } 
    # switch ends here
    
    debugReturn $returnValue
}
#___oOo___
