#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccDebug.tcl,v 2.116 2005/10/21 14:15:09 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# ssandroc   06.09.1995 created
# imunoz     28.03.1996 ported to itcl2.0
#

#************************************************************************
#   NAME
#	VccDebug - Abstract class with debugging support
# 
#   SYNOPSIS
#	inherit VccDebug
#
#	method abc { args } {
#	    debugEnter
#		...
#	    debugPrint "irgenwas"
#		...
#	    debugReturn [expr cos(1)]
#	}
#
#	VccDebug::Enable
#
#   DESCRIPTION
#
#   INHERITANCE
#	None.
#
#   METHODS
#	All methods are intended to be called from inside a method of
#	an object of a derived class.
#
#	debugEnter
#		To be called at the beginning of a method.
#		Prints "(<objName>) ... <methodName args>: <String>" to stdout
#		if debugging is enabled.
#
#	debugPrint ?<string>?
#		To be called in the middle of a method.
#		Prints "(<objName>) ... <String>" to stdout if debugging is 
#		enabled.
#
#	debugReturn ?<value>?
#		To be called at the end of a method.
#		Prints "(<objName>) ... ==> <Value>" to stdout if debugging is 
#		enabled, and then returns with <value> from the calling method.
#
#	"..." indicates a string of dots with a length corresponding to the
#	call-stack level.
#
#   PROCS
#	Enable	- enables debugging outputs
#	Disable	- disables debugging outputs
#	Control	<OnOff> - dto. depending on given argument
#
#	The initial state is taken from the global variable `vcc(debug)'.
#	The enable-state applies to ALL derived objects of this class.
#
#   ENVIRONMENT
#	vcc(debug)	global variable that enables debug outputs.
#			It is set to 0 if not already defined before.
#
#   FILES
#
#   CAUTIONS
#	Usage of the methods contained herein do not make the calling methods
#	faster - of course. They should not be used for time-critical tasks.
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------

#

::vcc	;# make sure to source the global variable definitions before

class VccDebug {
    #inherit itk::Widget
    #====================================================================
    # Debugging Support
    #====================================================================

    # flag for debugging outputs:
    ;#common to ALL derived objects of this class
    constructor {} {}

    protected common debugFlag 0 ;# re-initialized at end of file

    # enables debugging outputs:
    public proc Enable { } {
	set debugFlag 1
    }

    # disables debugging outputs:
    proc Disable { } {
	set debugFlag 0
    }

    # enables or disables debugging outputs:
    proc Control { OnOff } {
	set debugFlag $OnOff
    }
    
    # prints "(<objName>) <methodName args>: <String>" if debugFlag is on:
    public method debugEnter { } 
    # prints "(<objName>) <String>" if debugFlag is on:
    public method debugPrint { {String ""} }
    # prints "(<objName>) ==> <Value>" if debugFlag is on, and returns <Value>:
    public method debugReturn { {Value ""} }
}

# prints "(<objName>) <methodName args>: <String>" if debugFlag is on:
body VccDebug::debugEnter { } {
    if {$debugFlag} {
	puts "($this) [replicate . [info level]] [info level -1]"
    }
}

# prints "(<objName>) <String>" if debugFlag is on:
body VccDebug::debugPrint { {String ""} } {
    if {$debugFlag} {
	set n [lindex [info level -1] 0]
	puts "($this) [replicate . [info level]] $n: $String"
    }
}
# prints "(<objName>) ==> <Value>" if debugFlag is on, and returns <Value>:
body VccDebug::debugReturn { {Value ""} } {
    if {$debugFlag} {
	set n [lindex [info level -1] 0]
	puts "($this) [replicate . [info level]] $n ==> $Value"
    }
    uplevel [list return $Value]
}


# initialize common debugFlag from global default:
VccDebug::Control $vcc(debug)

# ___oOo___
