#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccFindLib.tcl,v 2.116 2005/10/21 14:15:09 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 31.08.1995 created
#

#************************************************************************
#   NAME
#	vccFindLib,
#	vccFindFile, vccFindConfigFile, vccFindRcFile, 
#	vccFindBitmapFile, vccFindPixmapFile
#	- VLT Central Configuration Find File Library
# 
#   SYNOPSIS
#	vccFindFile <RelativePath>	==> <AbsolutePath> | ""
#	vccFindConfigFile ?<Name> ...?	==> <AbsolutePath> | ""
#	vccFindRcFile ?<Name> ...?	==> <AbsolutePath> | ""
#	vccFindBitmapFile ?<Name> ...?	==> <AbsolutePath> | ""
#	vccFindPixmapFile ?<Name> ...?	==> <AbsolutePath> | ""
#
#   DESCRIPTION
#     The functions contained herein locate an explicitly for implicitly
#     given file in the VLT standard areas. See also vccSourceLib(n) for 
#     functions that locate and immediately source scripts.
#
#     vccFindFile		- Locate a file by its relative path
#
#	This function should be used to locate a file at run time.
#	The file given as relative path is searched with the following order:
#
#	1. in the current module area:	"../<RelativePath>"
#	2. in INTDATA (if defined):	"$INTDATA/<RelativePath>"
#	3. in VLTDATA:			"$VLTDATA/<RelativePath>"
#	4. in INTROOT (if defined):	"$INTROOT/<RelativePath>"
#	5. in VLTROOT:			"$VLTROOT/<RelativePath>"
#
#
#     vccFindConfigFile		- Loacte a config file
#
#	This function should be used to locate a config file at run time.
#	<Name> can be given as filename without path, e.g. "abc.config".
#	Multiple names are possible which are then searched in order.
#	If no argument is given, then the name is automatically derived from
#	the current program name, i.e. "[file tail $argv0]".
#	The file is first searched by its plain name, and then by
#	appending the extension ".config".
#	It is searched with the same priority order as for `vccFindFile'
#	in the "config" sub-directory.
#
#
#     vccFindRcFile		- Locate a user-config file
#
#	This function should be used to locate a user-config file at run time.
#	<Name> can be given as filename without path, e.g. "abcProg".
#	Multiple names are possible which are then searched in order.
#	If no argument is given, then the name is automatically derived from
#	the current program name, i.e. "[file tail $argv0]".
#	It is searched in the following priority order:
#
#	1. "~/.<Name>"
#	2. "~/.<Name>" where <Name> is converted to all lower case
#	3. "~/.<Name>Rc"
#	4. "~/.<Name>rc" where <Name> is converted to all lower case
#
#	The following files would be found for a program "abcProg":
#	"~/.abcProg", "~/.abcprog", "~/.abcProgRc", "~/.abcprogrc".
#
#     vccFindBitmapFile		- Locate a bitmap file
#     vccFindPixmapFile		- Locate a pixmap file
#
#	This function can be used to locate a bit/pixmap file at run time.
#	If <Name> is given then it must be without extension.
#	Similar rules as for vccFindConfigFile are applied.
#
#
#   RETURN VALUES
#	The absolute path of the first found file is returned,
#	or an empty string if none was found.
#
#	The file must be readable for the calling user, otherwise it will
#	be ignored even if it exists.
#
#   FILES
#	The explicitly or implicitly specified files are accessed by the
#	procs in read-onlv mode. They must be readable by the calling user.
#
#   ENVIRONMENT
#	VLTROOT - path to VLT code area
#	INTROOT - path to intgration code area (optional)
#	VLTDATA - path to VLT data area
#	INTDATA - path to intgration data area (optional)
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#	vcc(n), vccSourceLib(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

vcc	;# make sure to source the global variable definitions before


proc vccFindLib { } {
    # dummy proc to force auto-loading
}


#
# local proc to determine default args:
#
proc _vccFindDefArgs { Arglist } {
    global argv0

    # if args are actually given, return them unchanged:
    if {! [lempty $Arglist] && [lindex $Arglist 0] != ""} {
	return $Arglist
    }

    # determine default names derived from the following:
    set progName   [file tail $argv0]			;# program name
    set scriptName [file tail [info script]]		;# script name
    set modName1 ""
    set modName2 ""
    regexp {^([A-Z]?[a-z0-9]+)} $progName   m modName1	;# module by program
    regexp {^([A-Z]?[a-z0-9]+)} $scriptName m modName2	;# module by script
    return [list $progName $scriptName $modName1 $modName2]
}


#
#    vccFindFile <RelativePath>		==> <AbsolutePath> | ""
#
proc vccFindFile { RelativePath } {
    global env vcc_n

    # search in current module area:
    set absPath [pwd]/../$RelativePath
    if {[file readable $absPath]} {
	return $absPath				;# found in current module area
    }

    # search in VLT standard areas:
    foreach area $vcc_n(areaList) {
	if {[info exists env($area)]} {
	    set absPath $env($area)/$RelativePath
	    if {[file readable $absPath]} {
		return $absPath			;# found in standard area
	    }
	}
    }

    return ""					;# nothing found
}

#
#    vccFindConfigFile ?<Name> ...?		==> <AbsolutePath> | ""
#
proc vccFindConfigFile { args } {
    global argv0 vcc_n

    # take the default derived from the current program/script/module name:
    set args [_vccFindDefArgs $args]
    ##if [lempty $args] {	set args [_vccFindDefArgs]    }

    # repeat for all given names:
    foreach name $args {
	if {[lempty $name]} continue

	# locate file by its plain name and return absolute path:
	set absPath [vccFindFile $vcc_n(configDir)/$name]
	if {$absPath != ""} {
	    return $absPath
	}

	# append config extension to Name and try again:
	set absPath [vccFindFile $vcc_n(configDir)/$name$vcc_n(configExt)]
	if {$absPath != ""} {
	    return $absPath
	}
    }

    return ""					;# nothing found
}

#
#    vccFindRcFile ?<Name> ...?		==> <AbsolutePath> | ""
#
proc vccFindRcFile { args } {
    global argv0

    # take the default derived from the current program/script/module name:
    set args [_vccFindDefArgs $args]

    # repeat for all given names:
    foreach name $args {
	if {[lempty $name]} continue

	# build possible names:
	lappend nameList ~/.$name
	lappend nameList ~/.[string tolower $name]
	lappend nameList ~/.${name}Rc
	lappend nameList ~/.[string tolower $name]rc

	# search for the files in that order:
	foreach name $nameList {
	    if {[file readable $name]} {
		return [glob $name]
	    }
	}
    }

    return ""					;# nothing found
}


#
#    vccFindBitmapFile ?<Name> ...?		==> <AbsolutePath> | ""
#
proc vccFindBitmapFile { args } {
    global argv0 vcc_n

    # take the default derived from the current program/script/module name:
    set args [_vccFindDefArgs $args]

    # repeat for all given names:
    foreach name $args {
	if {[lempty $name]} continue

	# locate file by its plain name and return absolute path:
	set absPath [vccFindFile $vcc_n(bitmapsDir)/$name$vcc_n(bitmapsExt)]
	if {$absPath != ""} {
	    return $absPath
	}
    }

    return ""					;# nothing found
}

#
#    vccFindPixmapFile ?<Name> ...?		==> <AbsolutePath> | ""
#
proc vccFindPixmapFile { args } {
    global argv0 vcc_n

    # take the default derived from the current program/script/module name:
    set args [_vccFindDefArgs $args]

    # repeat for all given names:
    foreach name $args {
	if {[lempty $name]} continue

	# locate file by its plain name and return absolute path:
	set absPath [vccFindFile $vcc_n(pixmapsDir)/$name$vcc_n(pixmapsExt)]
	if {$absPath != ""} {
	    return $absPath
	}
    }

    return ""					;# nothing found
}

# ___oOo___
