#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccInfo.tcl,v 2.116 2005/10/21 14:15:08 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 15.05.1995 created
# S.Sandrock 06.09.1995 100% revised
# S.Sandrock 18.12.1995 added support for ACC database
# imunoz     03.04.1996 ported to itcl2.0 -- not too much testing --
# imunoz     23.01.97   added subnet_name and routerIpAddr to a_acc
# ssandroc   10.09.1997 removed all obsolete Rtap-based VCCDB stuff.
# eallaert   25.02.1999 made compatible with both Tcl 7.6 and Tcl 8.0
# psivera    23.05.2001 added ACSROOT and ACE_ROOT_VW
# psivera    16.11.2001 added backplane_name variable
# psivera    11.12.2001 added a "package require msqltcl" instruction 
#                       SPR VLTSW20010719
# eallaert   2002-07-15 replaced file access via "exec" by "file" cmds
#

#************************************************************************
#   NAME
#	VccInfo - query information from the configuration database
# 
#   SYNOPSIS
#	VccInfo <object-name>		- create object with explicit name
#	set <var> [VccInfo #auto]	- create object with implicit name
# 
#   DESCRIPTION
#	VccInfo is a class to query information for configuration
#	panels. The intended use is mainly internal to those panels.
#	Normally a configuration database (ACCDB) is queried, but it
#	is also possible in degraded modes to filter the requested
#	information from derived configuration files.
#
#	The proc `vccInit' automatically creates an object of this class with
#	the name `vccInfo', see vccLib(n).
#
#   INHERITANCE
#	Internal debugging support from VccDebug(n).
#
#   PUBLIC OPTIONS
#	Options are set at object creation time, or later with:
#
#		<object-name> configure <option> <value>
#
#	The following public options are available:
#
#	-queryModes		Set query modes and their priority
#
#		This is a list which can contain the following modes:
#
#		"fromaccdb" - query from the ACC database (mSQL)
#		"fromfiles" - query from configuration files, i.e. files in
#			the VLT areas and standard OS files in "/etc".
#		"dynamic" - query by asking servers etc.
#
#		The order of these modes in the list determine their priority.
#		The operation stops after the first successful query.
#		usually it is: {fromaccdb fromfiles dynamic}
#
#	-complain		Specifies behaviour for unsuccessful query
#
#		If this is off then VccInfo returns only an empty string
#		when no valid result could be found for a query.
#		If it is on, then a Tcl error is generated.
#		The default if 0 (off).
#
#   METHODS
#	Methods are called with:
#
#		(a) vccInfo <method> <args>
#		(b) <object-name> <method> <args>
#
#	If the object has been created with `vccInit' then (a) is used.
#	For other object-names the general way of (b) is used.
#
#	Some methods can be called with either one item or a list of items as
#	argument and return correspondingly one result or a list of results.
#
#	Host-based query:
#	-----------------
#
#	GetHosts ?-all? ?-attic? ?<Filter> ...?
#
#		Returns the list of hosts matching one of the filters.
#		Default-filter is `*' for "all".
#		By default all items with an `attic' flag are not returned.
#		The `-all' switch includes also the attic.
#		The `-attic' switch returns only the attic. 
#
#	GetByHost <Host> <Attrs>
#
#		Returns the database value of Attrs for Host.
#		Attr: ipAddr, cpuName, hostType, ...
#
#
#	Environment-based query:
#	------------------------
#
#	GetEnvs	?-all? ?-attic? ?<Filter> ...?
#
#		Returns list of environments matching one of the filters.
#		Default-filter is `*' for "all".
#		By default all items with an `attic' flag are not returned.
#		The `-all' switch includes also the attic.
#		The `-attic' switch returns only the attic. 
#
#	GetByEnv <Env> <Attrs>
#
#		Returns the database value of Attrs for Env.
#		Attr: hostName, hostName0/1/2, tcpPort, processList, ...
#
#	GetBootEnvByEnv	<Env>	Returns assigned WS boot-environment
#	GetProcsByEnv	<Env>	Returns process-list of environment
#	GetIPaddrByEnv	<Env>	Returns IP-address of host of environment
#
#
#	<Filters>	glob-style filter or list of, e.g. "w*qs" {w* l*}
#	<Host>		host-name
#	<Env>		environment-name
#	<Attrs>		attribute-name or list of attribute-names
#
#   PROCS
#	None.
#
#   ENVIRONMENT
#	Two optional environment variables control the database access:
#
#	ACC_HOST - shell variable that defines the ACC database server host;
#		   the local host is assumed - resulting in faster access -
#		   if this equals to either:
#		   - the name returned by the command `hostname'
#		   - the string "localhost"
#		   - or an empty string.
#		   Note that an IP address in dot-notation also works,
#		   but will not be checked for the local host, and therefore
#		   will always result in a somewhat slower remote connection.
#
#	If ACC_HOST is undefined or unaccessible, then 
#	VccInfo tries to provide a limited set of functions based on 
#	degraded query methods. `fromaccdb' will be 
#	removed from the `-queryModes' option in that case.
#
#   FILES
#	None if ACC_HOST is defined and accessible.
#	Otherwise file-based querying is performed on various Unix and
#	VLT standard files.
#
#   CAUTIONS
#
#   EXAMPLES
#	vccInit ;# creates object `vccInfo', may enter local CCS environment
#	set wsEnvs  [vccInfo GetEnvs -all w*] ;# query all WS-environments
#	set lcuEnvs [vccInfo GetEnvs l*]      ;# query LCU-environments
#	vccExit ;# leave vcc and exit CCS environment
#
#   SEE ALSO
#	vcc(n), vccLib(n), VccDebug(n)
#
#   BUGS     
#
#------------------------------------------------------------------------
#

::vcc	;# make sure to source the global variable definitions before

package require msqltcl

class VccInfo {

    inherit VccDebug	;# includes several debugging support
    constructor { args } { }
    destructor  {}    
    
    private method evalQuery { args } {}
    private method filterList { ItemList FilterList {RegexpPattern ""} } {}

    public  method GetHosts { args } {}
    private method _GetHosts_fromfiles { args } {}
    private method _GetHosts_fromaccdb { args } {}

    public  method GetByHost { Host Attrs } {}
    private method _GetByHost_fromaccdb { Host Attrs } {}


    public  method GetEnvs { args } {}
    private method _GetEnvs_fromaccdb { args } {}
    private method _GetEnvs_fromfiles { args } {}

    public  method GetByEnv { Env Attrs } {}
    private method _GetByEnv_fromaccdb { Env Attrs } {}
    #method _GetByEnv_fromfiles { } {not yet available}
    #method _GetByEnv_dynamic { } {not yet available}
    
    public method GetIPaddrByEnv { Env } {}

    public  method GetBootEnvByEnv { Env } {}
    private method _GetBootEnvByEnv_fromaccdb { Env } {}
    #method _GetBootEnvByEnv_fromfiles { } {not yet available}
    #method _GetBootEnvByEnv_dynamic { } {not yet available}

    public  method GetProcsByEnv { Env } {}
    private method _GetProcsByEnv_fromaccdb { Env } { }
    private method _GetProcsByEnv_fromfiles { Env } { }

    public method GetRouterIpBySubnet { subnet} { }


    #====================================================================
    # Common Constants
    #====================================================================
    
    # attribute names in database:
    common a_Attic		Attic		;# vector of hidden names
    common a_backplaneName	backplaneName	
    common a_location		Location	
    common a_stationType	stationType	
    common a_envName		envName
    common a_envType		envType
    common a_hostName		hostName
    common a_hostName0		hostName0
    common a_hostName1		hostName1
    common a_hostName2		hostName2
    common a_ipAddr		ipAddr
    common a_tcpPort		tcpPort
    common a_processList	processList
    common a_bootEnv		bootEnv		;# LCU envs only
    common a_cpuName		cpuName
    common a_bspName		bspName
    common a_hostType		hostType
    common a_procName		procName
    common a_subnetName         subnetName
    common a_routerIpAddr       routerIpAddr    ;# in ACC's subnet db

    # file names and env-vars for "fromfiles" query:
    common e_AREAS	{MODROOT INTROOT VLTDATA VLTROOT ACSROOT ACE_ROOT_VW}
    common f_ENVS	ENVIRONMENTS
    common f_PROCS	PROCESSES
    common f_hosts	/etc/hosts
    common f_services	/etc/services
    common f_exports	/etc/exports
    

    #====================================================================
    # ACCDB related Constants
    #====================================================================
    
    private common n_accdb	acc	;# ACC database name
    
    # table names in ACC database::
    private common t_envs	prog_environment
    private common t_hosts	station
    private common t_procs	proc_progenv_comp
    private common t_bootenvs	lcu_progenv
    private common t_subnets    subnet
    
    # attribute names in ACC database:
    private common a_acc
    set a_acc($a_Attic)		attic
    set a_acc($a_backplaneName)	backplane_name
    set a_acc($a_location)	location
    set a_acc($a_stationType)	station_type
    set a_acc($a_envName)	progenv_name
    set a_acc($a_hostName)	station_name
    set a_acc($a_hostName0)	station_name0
    set a_acc($a_hostName1)	station_name1
    set a_acc($a_hostName2)	station_name2
    set a_acc($a_subnetName)	subnet_name
    set a_acc($a_routerIpAddr)  router_ip_addr
    set a_acc($a_ipAddr)	ip_address
    set a_acc($a_tcpPort)	tcp_port
    set a_acc($a_cpuName)	station_cpu
    set a_acc($a_bspName)	station_bsp
    set a_acc($a_hostType)	station_arch
    set a_acc($a_bootEnv)	bootenv_name
    set a_acc($a_procName)	proc_name
    set a_acc($a_envType)	env_type
    
    #====================================================================
    # Protected Variables
    #====================================================================
    
    protected variable accHost	{}	;# ACC database host, set from env(ACC_HOST)
    protected variable accId	{}	;# connection handle to ACC database server
    
    #====================================================================
    # Public Option Variables
    #====================================================================
    public variable queryModes {fromaccdb fromfiles dynamic}
    public variable complain	0	;# flag unsuccessful query as Tcl error    
}

body VccInfo::constructor { args } {
    global env
    set text ""
    
    # determine name of ACC database host and connect to it:
    if {[info exists env(ACC_HOST)]} {
	set accHost $env(ACC_HOST)
	set thisHost {};
	if {[info exists env(HOSTNAME)]} {
	    set thisHost $env(HOSTNAME)
	} elseif {[info exists env(HOST)]} {
	    set thisHost $env(HOST)
	}
	if {[string equal $accHost ""] || \
		[string equal $accHost localhost]	|| \
		[string equal $accHost $thisHost] } {
	    set cmd [list msqlconnect]; set str "local host"
	} else {
	    set cmd [list msqlconnect $accHost]; set str $accHost
	}
	if {[catch {set accId [eval $cmd]; msqluse $accId $n_accdb} err]} {
	    puts stderr [set text "vcc: failed to connect to $n_accdb database\
		    on $str:\n\t$err\n\t(continue without it ...)"]
	} else {
	    debugPrint [set text "connected to $n_accdb database\
		    on $str ($accId)"]
	}
	if {[namespace tail $this] == "vccInfo"} {
	    debugPrint "vccInfo: $text"
	}
    } else {
	debugPrint [set text "info: ACC_HOST undefined, no ACC database"]
    }
    
    # inhibit ACCDB access if not available:
    if {[lempty $accId]} {
	# remove "fromaccdb" query-mode:
	if {[set i [lsearch -exact $queryModes "fromaccdb"]] >= 0} {
	    set queryModes [lreplace $queryModes $i $i]
	    debugPrint "queryModes reduced to: $queryModes"
	}
    }
    
    eval configure $args
}

body VccInfo::destructor {} {
    if {![lempty $accId]} {
	msqlclose $accId
	debugPrint "connection to $n_accdb database closed ($accId)"
    }
}

#====================================================================
# Local methods:
#====================================================================

#--------------------------------------------------------------------
# evalQuery - perform query for calling method with modes in `queryModes'
#
# `evalQuery' calls indirectly mode-specific sub-methods.
# The querying stops at the first sub-method returning a non-empty string.
# The indirectly called sub-methods must be named "_${baseMethod}_${mode}",
# where `baseMethod' is the name of the calling method and `mode' are the 
# single elements of `queryModes'. 
# All calls are catch'ed, so that a sub-method need not necessarily exist.
# `args' are passed to each called sub-method.
# And: yes, this is a tricky function.
#--------------------------------------------------------------------
body VccInfo::evalQuery { args } {
    # get name of calling method:
    set baseMethod [lindex [info level -1] 0]
    # call sub-method for each of the query-modes, until valid result:
    foreach mode $queryModes {
	set subMethod _${baseMethod}_${mode}
	if {[eval catch \{$subMethod $args\} result]} {
	    debugPrint "$subMethod failed or not existing"
	} elseif {$result != ""} {
	    return $result
	}
    }
    # means no valid result at all
    if {$complain} {
	error "VccInfo: no valid result from query: $baseMethod $args"
    } else {
	return "" ;# silently accept no result
    }
}

#--------------------------------------------------------------------
# filterList - Filter matching items from a list
#
# ItemList      - list of items to filter
# FilterList    - list of filters, at least one must match
# RegexpPattern - regular expression that an item must match (default: all)
#--------------------------------------------------------------------
body VccInfo::filterList { ItemList FilterList {RegexpPattern ""} } {
    set matchList {}
    foreach item $ItemList {
	foreach pattern $FilterList {
	    if {[string match $pattern $item] && \
		    [regexp $RegexpPattern $item]} {
		lappend matchList $item
		break
	    }
	}
    }
    return $matchList
}


#====================================================================
# Host-based public methods:
#
# (the methods starting with "_" are called indirectly via evalQuery)
#====================================================================

#--------------------------------------------------------------------
# GetHosts
#--------------------------------------------------------------------

body VccInfo::GetHosts { args } {
    debugEnter
    lassign [intersect3 $args {-all -attic}] filters ;# pop switches
    if {[lempty $filters]} {set filters *}
    # query the whole list of hosts:
    set allHosts [evalQuery $args]
    # filter matching hosts:
    debugReturn [filterList $allHosts $filters {^[^_]}]
}

body VccInfo::_GetHosts_fromaccdb { args } {
    debugEnter
    if {[lempty $accId]} {debugReturn ""}
    switch -glob -- $args {
	*-all*   {set cond ""}
	*-attic* {set cond "WHERE $a_acc($a_Attic) = 1"}
	default  {set cond "WHERE $a_acc($a_Attic) = 0"}
    }
    set order "ORDER BY $a_acc($a_hostName)"
    msqlsel $accId "SELECT $a_acc($a_hostName) FROM $t_hosts $cond $order"
    msqlmap $accId x {lappend hosts $x}
    debugReturn $hosts
}

body VccInfo::_GetHosts_fromfiles { args } {
    debugEnter
    if {![file readable $f_hosts]} {
	debugPrint "\"$f_hosts\" is not readable!"
	debugReturn {}
    }
    set h {}
    for_file z $f_hosts {
	if {[cindex $z 0] != "#"} {
	    lappend h [lindex $z 1]
	}
    }
    switch -glob -- $args {
	*-all*   {}
	*-attic* {set h {} ;# attic is not implemented here}
	default  {}
    }
    debugReturn $h
}

#method _GetHosts_dynamic { } {not yet available}

#--------------------------------------------------------------------
# GetByHost - query given `Attrs' for `Host'
#--------------------------------------------------------------------

body VccInfo::GetByHost { Host Attrs } {
    debugEnter
    debugReturn [evalQuery $Host $Attrs]
}

body VccInfo::_GetByHost_fromaccdb { Host Attrs } {
    debugEnter
    foreach attr $Attrs {lappend alist $a_acc($attr)}
    set cond "$a_acc($a_hostName) = '$Host'"
    msqlsel $accId "SELECT [join $alist ,] FROM $t_hosts WHERE $cond"
    debugReturn [msqlnext $accId]
}

#body VccInfo::_GetByHost_fromfiles { } {not yet available}
#body VccInfo::_GetByHost_dynamic { } {not yet available}


#====================================================================
# Environment-based public methods:
#
# (the methods starting with "_" are called indirectly via evalQuery)
#====================================================================

#--------------------------------------------------------------------
# GetEnvs
#--------------------------------------------------------------------

body VccInfo::GetEnvs { args } {
    debugEnter
    lassign [intersect3 $args {-all -attic}] filters ;# pop switches
    if {[lempty $filters]} {set filters *}
    # query the whole list of envs:
    set allEnvs [evalQuery $args]
    # filter matching envs:
    debugReturn [filterList $allEnvs $filters {^[^_]}]
}

body VccInfo::_GetEnvs_fromaccdb { args } {
    debugEnter
    if {[lempty $accId]} {debugReturn ""}
    switch -glob -- $args {
	*-all*   {set cond ""}
	*-attic* {set cond "WHERE $a_acc($a_Attic) = 1"}
	default  {set cond "WHERE $a_acc($a_Attic) = 0"}
    }
    set order "ORDER BY $a_acc($a_envName)"
    msqlsel $accId "SELECT $a_acc($a_envName) FROM $t_envs $cond $order"
    msqlmap $accId x {lappend envs $x}
    debugReturn $envs
}

body VccInfo::_GetEnvs_fromfiles { args } {
    debugEnter
    ::global env

    set envlist {}
    set pwd [pwd]
    foreach area $e_AREAS {
	if {![info exists env($area)]} continue

	set dir $env($area)
	# `ls' the dedicated env-directory to get the env-list:
	cd $dir/$f_ENVS
	lvarcat envlist [glob -nocomplain *]
	cd $pwd
    }
    # remove duplicates and invalid entries:
    set envlist [lrmdups $envlist]
    set envlist [lmatch -regexp $envlist {l|w.*[^~#]}]
    switch -glob -- $args {
	*-all*   {}
	*-attic* {set envlist {} ;# attic is not implemented here}
	default  {}
    }
    debugReturn $envlist
}

#body VccInfo::_GetEnvs_dynamic { } {not yet available}

#--------------------------------------------------------------------
# GetByEnv - query given `Attrs' for `Env'
#--------------------------------------------------------------------

body VccInfo::GetByEnv { Env Attrs } {
    debugEnter
    debugReturn [evalQuery $Env $Attrs]
}

body VccInfo::_GetByEnv_fromaccdb { Env Attrs } {
    debugEnter
    foreach attr $Attrs {lappend alist $a_acc($attr)}
    set cond "$a_acc($a_envName) = '$Env'"
    msqlsel $accId "SELECT [join $alist ,] FROM $t_envs WHERE $cond"
    debugReturn [msqlnext $accId]
}

#body VccInfo::_GetByEnv_fromfiles { } {not yet available}
#body VccInfo::_GetByEnv_dynamic { } {not yet available}

#--------------------------------------------------------------------
# GetIPaddrByEnv (==> GetHostByEnv + GetByHost)
#--------------------------------------------------------------------

body VccInfo::GetIPaddrByEnv { Env } {
    debugEnter
    debugReturn [GetByHost [GetByEnv $Env $a_hostName] $a_ipAddr]
}

#--------------------------------------------------------------------
# GetBootEnvByEnv
#--------------------------------------------------------------------

body VccInfo::GetBootEnvByEnv { Env } {
    debugEnter
    debugReturn [evalQuery $Env]
}

body VccInfo::_GetBootEnvByEnv_fromaccdb { Env } {
    debugEnter
    set cond "$a_acc($a_envName) = '$Env'"
    msqlsel $accId "SELECT $a_acc($a_bootEnv) FROM $t_bootenvs WHERE $cond"
    debugReturn [lindex [msqlnext $accId] 0]
}

#body VccInfo::_GetBootEnvByEnv_fromfiles { } {not yet available}
#body VccInfo::_GetBootEnvByEnv_dynamic { } {not yet available}


#--------------------------------------------------------------------
# GetProcsByEnv
#--------------------------------------------------------------------

body VccInfo::GetProcsByEnv { Env } {
    debugEnter
    debugReturn [evalQuery $Env]
}

body VccInfo::_GetProcsByEnv_fromaccdb { Env } {
    debugEnter
    set cond "$a_acc($a_envName) = '$Env'"
    msqlsel $accId "SELECT $a_acc($a_procName) FROM $t_procs WHERE $cond"
    msqlmap $accId x {lappend procs $x}
    debugReturn $procs
}

body VccInfo::_GetProcsByEnv_fromfiles { Env } {
    debugEnter
    ::global env

    set envprolist {}
    foreach environment $Env {
	set prolist {}
	foreach area $e_AREAS {
	    if {![info exists env($area)]} continue

	    set dir $env($area)
	    set tmplist {}
	    set filename "$dir/$f_ENVS/$environment"
	    if {[file isfile $filename]} {
		# it's a normal file containing the processes
	    } elseif {[file isdirectory $filename]} {
		# it's a directory; then look for a process-file therein:
		set filename "$filename/$f_PROCS"
	    } else {
		continue
	    }

	    # read the processes-file:
	    if {[file readable $filename]} {
		debugPrint "reading \"$filename\""
		set tmplist {}
		for_file line $filename {
		    if {[cindex $line 0] != "#"} {
			lappend tmplist [lindex $line 0]
		    }
		}
	    } else {
		debugPrint "\"$filename\" is not readable!"
	    }
	    lvarcat prolist $tmplist ;# extend the process-list
	}
	# remove duplicate process-names and append:
	debugPrint "found: $prolist"
	lappend envprolist [lrmdups $prolist]
    }

    # collapse a list if it si for only one environment:
    if {[llength $envprolist] <= 1} {
	set envprolist [lindex $envprolist 0]
    }
    debugReturn $envprolist
}

#body VccInfo::_GetProcsByEnv_dynamic { } {not yet available}

body VccInfo::GetRouterIpBySubnet { subnet } { 
    debugEnter
    set cond "$a_acc($a_subnetName) = '$subnet'"
    msqlsel $accId "SELECT $a_acc($a_routerIpAddr) FROM $t_subnets WHERE $cond"
    debugReturn [lindex [msqlnext $accId] 0]
}
# ___oOo___
