#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccRemLib.tcl,v 2.116 2005/10/21 14:15:14 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 09.06.1995 created
# S.Sandrock 31.08.1995 revised
# fcarbogn   24.08.1999 Modified vccRemPath to allow offline generation 
#			also when target machine is down (SPR 990195)
# tebert     24.02.2000 PING for LINUX
# psivera    01.07.2000 introduced VCC_REM_PATH_LOCAL variable (SPR 20000192)
# psivera    25.01.2002 introduced support for dual-network hosts (SPR 20020003)
# psivera    12.02.2002 added VCC_LOCAL initialization
# psivera    13.02.2002 fixed bug in VCC_LOCAL initialization
# eallaert   2002-07-15 retrieve hostname without [exec hostname]
# psivera    2002-07-16 modified ping command to work on RH7.2
# psivera    2002-07-23 modified ping output variable to work on RH7.3
# psivera    2003-07-01 catched output of remote command (SPR 20030137)
# psivera    2003-07-08 ported to RH 9 
# eallaert   2005-09-28 replaced sleep by vccWait
#

#************************************************************************
#   NAME
#	vccRemLib,
#	vccRemOpen, vccRemClose,
#	vccRemPath, vccRemExec,
#	vccRemLogin
#	- VLT Central Configuration Remote Host Operations Library
# 
#   SYNOPSIS
#	vccRemOpen	<Host> <File> ?<Mode>? ?<Options>?	==> <fd>
#	vccRemClose	<Fd>					==> ""
#	vccRemPath	<Host> <Path>				==> <dir> | ""
#	vccRemExec	<Host> <Command>			==> <stdout>
#	vccRemLogin	<Host>					==> <fd>
#
#   DESCRIPTION
#	vccRemOpen -	opens a remote file on `Host' with pathname `File'
#
#		`Host' denotes the host on which `File' resides.
#		The host can also be included in the form "<host>:<file>"
#		in `File', which then takes precedence over `Host'.
#		If no host specification is given in either form, 
#		then a local access is assumed.
#		For local accesses the `File' is opened directly.
#		For remote accesses the concatenation "<host>:<file>" 
#		is passed to rcp(1) in order to make a local copy of the file
#		in a tmp directory (see FILES). This file is then opened. 
#		The access-mode `Mode' is as in open(n): r, r+, w, w+, a, a+.
#		"r" is the default.
#		`Options' can be "-dialog" to ask for confirmation to
#		overwrite a file if it already exists, otherwise it is
#		replaced without asking.
#		Returns a file-identifier to be used in subsequent
#		file commands like gets(n), puts(n), and `vccRemClose'.
#
#	vccRemClose -	closes a file previously opened by `vccRemOpen'
#
#		If the file was not opened remotely in read-only mode, then  
#		the local copy is written back with rcp(1) to it's original 
#		place. The local copy is then deleted.
#		Returns an empty string.
#
#	vccRemPath -	gets a remote directory-path in resolved notation
#
#		The value of `Path' must denote the name of a directory on
#		`Host' and may contain environment-variables to be resolved.
#		Returns the expanded physical path-name of the directory via 
#		pwd(1), or an empty string if the directory does not exist.
#
#	vccRemExec -	executes a shell-command on a remote host
#
#		`Command' is executed as a shell-command on `Host'.
#		Returns the standard-output of the command.
#
#	vccRemLogin -	rlogin to a remote host
#
#		Opens a `rlogin' pipe to a remote `Host'.
#		Returns a file-identifier for the pipe that is open for
#		writing. If the rlogin was not successful, a dialog-box
#		is opened for information, as an empty string is returned.
#		The rlogin session will be closed by a `close <fd>',
#		which may yield a Tcl error due to some stderr writing.
#		See also CAUTIONS.
#
#   FILES
#	Temporary files are used for local copies of remote files.
#	The variable `vcc_n(tmpDir)' defines the target directory,
#	which is "/tmp" by default.
#
#   ENVIRONMENT
#	Note that environment-variables on remote hosts should be defined
#	e.g. in "$HOME/.cshrc" resp. "$HOME/.cshrc.local" for VLT.
#	Otherwise they may be undefined when accessed through a remote
#	operation like `remsh'.
#
#   RETURN VALUES
#
#   CAUTIONS
#	All procs shall only be used with a windowing shell, eg. seqWish.
#
#	Note that I/O operation are usually buffered. In particular, an
#	explicit `flush' is necessary after each `puts' to a pipe opened
#	with `vccRemLogin'.
#
#   EXAMPLES
#	set fd [vccRemOpen te1 /vlt/config/xyz]	;# open file on te1 for read
#	set x [read $fd]			;# read it's contents
#	vccRemClose $fd				;# close the file
#
#	set v [vccRemVar te16 HOME]		;# get variable HOME on te16
#	set w [vccRemPath te49 \$VLTROOT]	;# get path to VLTROOT on te49
#
#   SEE ALSO
#	vccLib(n),
#	open(n), close(n), puts(n), gets(n), read(n),
#	rcp(1), remsh(1), rsh(1), cd(1), pwd(1), echo(1)
#
#   BUGS
#	vccRemOpen: do not specify "-dialog" for read modes.
#
#------------------------------------------------------------------------
#

vcc	;# make sure to source the global variable definitions before


proc vccRemLib { } {
    # dummy proc to force auto-loading
}


global vccRem_n			;# array to store global names

# where local copies of remote files are temporarily stored:
vccSetDef vccRem_n(tmpDir) $vcc_n(tmpDir)

# set correct local POSIX shell command for HP and SUN:
set vccRem_n(posixsh) $vcc_n(posixsh)

# set correct remote shell command for HP and SUN:
set vccRem_n(remsh) $vcc_n(remsh)

# determine local hostname (empty if not available):
set vccRem_n(hostname) {};
if {[info exists env(HOSTNAME)]} {
    set vccRem_n(hostname) $env(HOSTNAME)
} elseif {[info exists env(HOST)]} {
    set vccRem_n(hostname) $env(HOST)
}

# array of hosts that have responded to ping:
global vccRemHosts

proc vccRemPing { Host } {
    global vccRemHosts vcc_n

    if {! [lempty $Host] && ! [info exists vccRemHosts($Host)]} {
        if {$vcc_n(isLINUX)} {
	    set msg [exec $vcc_n(ping) -c1 -s8 -q -U $Host | grep loss]	;# if error ==> host unknown
        } else {
	    set msg [exec $vcc_n(ping) $Host 8 1]	;# if error ==> host unknown
        }

	# note: on SUN, ping may give an error, on HP msg has to be checked;
	# if OK: on SUN matches "alive", on HP check for 0% (note the blank!)

	set msgPing "*0%*loss*"

	if {! [string match *alive* $msg] && ! [string match $msgPing $msg] } {
	    error "vccRemPing: no answer from $Host"	;# host unreachable
	}
	set vccRemHosts($Host) {}		;# mark host OK
    }
    return {}
}


# array of opened remote files:
global vccRemFiles
	# internally used by `vccRemOpen' and `vccRemClose' to keep track of
	# remotely opened files.
	# index of array: file-descriptor `fd'
	# each value is a list with the elements:
	#   0: name of local copy 
	#      (or "" if no `rm <local>' shall be done on close)
	#   1: name of remote file 
	#      (or "" if no `rcp <local> <remote>' shall be done on close)


# open remote file:
proc vccRemOpen { Host File {Mode r} args } {
    global argv0
    global vccRem_n
    global vccRemFiles ;# record of remotely opened files

    # host-specifier possibly included in the filename has priority:
    regexp {^(.*):(.*)$} $File m Host File

    vccPuts "$Host:$File $Mode $args"	;# debug

    # check if local or remote:
    set local [expr {$Host == "" || $Host == $vccRem_n(hostname)}]

    if {$local} {
	# for explicit local operation:
	set localFile $File			;# direct name of local file
    } else {
	# for remote operation:
	set remoteFile $Host:$File		;# name of remote file for rcp
						;# name of local copy:
	set localFile $vccRem_n(tmpDir)/[pid]_$Host:[file tail $File]

	# copy file from remote host and perform mode-specific error-handling:
	set rcpErr [catch {exec rcp $remoteFile $localFile} rcpMsg]
	switch -glob $Mode {
	    r* {
		if {$rcpErr} {
		    return -code $rcpErr "couldn't copy remote file\
			    \"$remoteFile\" to local \"$localFile\".\n$rcpMsg"
		}
	    }
	}
    }

    # check if file already exists and ask for confirmation to overwrite:
    if {[lsearch $args "-dialog"] >= 0 &&  [file exists $localFile] } {
	if {[tk_dialog .vccRemOpen "[file tail $argv0]: Confirm" \
		"\"$Host:$File\" already exist! Overwrite?" \
		question 1 Continue Cancel]} {
	    if {! $local} {
		file delete -force -- $localFile	;# remove local copy again
	    }
	    return ""				;# return NO filedescriptor
	}
    }

    set fd [open $localFile $Mode]	;# open local file resp. local copy

    # record remotely opened files for vccRemClose:
    if {! $local} {
	set vccRemFiles($fd) $localFile	;# save local name for later deletion
	# save remote name for later back-copying (unnecessary if read-only):
	if {$Mode != "r"} {lappend vccRemFiles($fd) $remoteFile}
    }

    return $fd					;# return filedescriptor
}


# close remote file:
proc vccRemClose { Fd } {
    global vccRemFiles
    set s ""
    close $Fd					;# close local copy
    # if it was a remotely opened file:
    if {[info exists vccRemFiles($Fd)]} {
	set s $vccRemFiles($Fd)
	set localFile  [lindex $s 0]		;# get saved local filename
	set remoteFile [lindex $s 1]		;# get saved remote filename
	# the following operation depends on the open-mode etc.:
	if {$localFile != ""} {
	    if {$remoteFile != ""} {
		exec rcp $localFile $remoteFile	;# copy file back to remote
	    }
	    file delete -force -- $localFile	;# remove local copy
	}
    }
    vccPuts "$Fd ($s)"	;# debug
    return ""
}


# get remote path in expanded format:
proc vccRemPath { Host Path } {
    global env vccRem_n VCC_LOCAL

if { [catch {set VCC_LOCAL}] } {
set VCC_LOCAL dummy
}

# If VCC_PATH_LOCAL is defined then do not ask target machine for the path.
# Check host, if host unreachable, set local host by default and path will be 
# resolved locally. This will allow offline generation also when target machine
# is down
    if {[info exists env(VCC_REM_PATH_LOCAL)] || [string match $VCC_LOCAL "yes"]} {
        set Host ""
	if {[info exists env(VCC_REM_PATH_LOCAL)]} {
            puts "vccRemPath: VCC_REM_PATH_LOCAL defined:  " 
            puts "            $Host ignored, resolve $Path locally"
	}
    } elseif {[catch {vccRemPing $Host}]} {
        puts "vccRemPath: $Host inaccessible - resolve $Path locally"
        set Host ""
    }

    set full ""
    # NOTE: the "cd $<Path>;pwd" is done to get physical path without links
    if {$Host == "" || $Host == $vccRem_n(hostname)} {
	# local operation (sh flags no error if variable is undefined!):
	catch {set full [exec $vccRem_n(posixsh) -c "echo $Path"]}
	if {! [lempty $full]} {
	    catch {set full [exec $vccRem_n(posixsh) -c "cd $full; pwd"]}
	}
    } else {
	# remote operation (-n redirects stdin to /dev/null):
	catch {set full [exec $vccRem_n(remsh) $Host -n "cd ${Path};pwd"]}
    }
    vccPuts "$Host $Path ==> $full"	;# debug
    return $full
}


# execute shell command on remote host:
proc vccRemExec { Host Command } {
    global vccRem_n

    vccRemPing $Host ;# check host

    if {$Host == "" || $Host == $vccRem_n(hostname)} {
	# local operation:
	set out [exec $vccRem_n(posixsh) -c $Command]
    } else {

	# remote operation:

	# SPR 20030137: commented out the following line
	# set out [exec $vccRem_n(remsh) $Host -n $Command]
	# and changed with the following one:
	catch {[exec $vccRem_n(remsh) $Host -n $Command]} out
	# end SPR 20030137
    }
    vccPuts "$Host $Command ==> $out"	;# debug
    return $out
}


# rlogin to another (LCU) host:
proc vccRemLogin { Host } {

    vccRemPing $Host ;# check host

    # open connection:
    set pipe [open "|rlogin $Host" w]; flush $pipe

    # test the connection:
    vccWait 2000 ;# how long? This time is like shooting straight in the dark!

    if {[catch {puts $pipe "" ; flush $pipe} msg]} {
	tk_dialog .dialog "[wm title .]: Confirm" \
		"Cannot rlogin to \"$Host\".\
		Possibly the hostname is wrong\
		or there is another rlogin session that locks the shell.\
		\nRun-time error is:\n$msg" \
		error 0 OK
	catch {close $pipe}
	return ""
    }

    return $pipe	;# the fileId to write to
}

# ___oOo___
