#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccSelectButton.tcl,v 2.116 2005/10/21 14:15:09 vltsccm Exp $" 
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 22.08.1995 Created from ccseiMenuLib.tcl
# imunoz     02.04.1996 Ported to itcl2.0
# imunoz     08.04.1996 Fixed problem with configuration of menu 
#                       components. 
# imunoz     22.04.1996 fix problem whe -variable is empty
#

#************************************************************************
#   NAME
#	VccSelectButton - class for a menubutton widget with special properties
#
#   SYNOPSIS
#	VccSelectButton <pathName> ?-<option> <value> ...?
#	<pathName> configure ?-<option> <value> ...?
#	<pathName> <method> ?<arg> <arg> ...?
#
#   DESCRIPTION
#	The VccSelectButton command creates a new window (given by the pathName
#	argument) and makes it into a corresponding widget. Additional options,
#	described below, may be specified on the command line or in the option
#	database to configure aspects of the widget.
#
#	The VccSelectButton command returns its pathName argument. At the time 
#	this command is invoked, there must not exist a window named pathName,
#	but pathName's parent must exist.
#
#	The VccSelectButton widget consists of a menubutton and a connected
#	menu. The menubutton has by default the width 1 and the label `+'.
#	The menu can be generated from the elements of a list representation.
#	The text of the menu-items correspond to the elements of the list.
#	Leaf-menu-items invoke a callback-script when selected, the text
#	of the menu-items is passed to this script as argument.
#	If the number of entries exceed a maximum then a continuation-menu
#	is automatically appended ("more...").
#
#   INHERITANCE
#	All options of menubutton(n) are inherited.
#	Deviating default values are:
#
#	-width		Default: 1
#	-text		Default: +
#	-relief		Default: raised
#
#   PUBLIC OPTIONS
#	The following options are specific to objects of this class:
#
#	-menuOption	Default: ""
#
#		Specifies how the menu shall be created from `menuItems':
#
#		"-nopropagate"	inhibits the propagation of callback-arguments
#				to the child-entries, i.e. only the label of
#				the leaf item is used as argument, but not the
#				concatenation with all its menu parents.
#
#		If no option is given, then the propagation is enabled
#		The menu is regenerated whenever this option is changed.
#
#	-menuItems	Default: {}
#
#		Contains a list of elements to make the menu from.
#		It will consist of entries of the type specified in `menuType',
#		except when one of the following is given:
#
#		- an empty string (except as the first list element) generates
#		  a separator
#
#		- if an element is itself a list, and the first element is
#		  an empty string, then the second element of this sub-list 
#		  denotes the label of a cascade-entry, while the rest of the 
#		  list denotes the elements of the corresponding sub-menu.
#		  (This somewhat complicated construction makes it possible
#		   to differentiate a list from a string containing spaces;
#		   therefore menu labels with spaces are permitted.)
#
#		The menu is regenerated whenever this option is changed.
#
#	-menuType	Default: "radiobutton"
#
#		Specifies the type of leaf menu entries. All leafs can only
#		have the same type, which must be "radiobutton" or "command".
#		The menu is regenerated whenever this option is changed.
#
#	-menuMaxLen	Default: 25
#
#		Specifies the maximum number of menu-entries on one level.
#		If that count is exceeded, then a "more..." sub-menu is 
#		automatically created, where the further entries will go in.
#		The specified count is excluding the "more..." itself.
#		The menu is regenerated whenever this option is changed.
#
#	-menuAcceleratorToken	Default: ""
#
#		If an element in `menuItems' contains this character, then
#		the text after it is regarded as accelerator. It can therefore
#		not be part of the label or accelerator itself. Empty string
#		means that no special handling of accelerators is done.
#		The menu is regenerated whenever this option is changed.
#
#	-command	Default: none
#
#		is a script (usually a procedure or method) to be called when
#		a leaf-item of the menu is selected. The text of the item is
#		passed as parameter to the script.
#		If "-nopropagate" is specified in `menuOption' then the
#		only the label of the leaf menu entry is passed as argument,
#		otherwise the concatenation of with all labels of the cascaded
#		parents of the leaf.
#		Cascade-entries itself do not have callbacks.
#		No command is performed is this option is an empty string.
#
#	-variable	Default: derived from widget name ($this)
#
#		Name of a variable connected to the menu items.
#
#	-state		Default: normal
#
#		This option is propagated to menubutton(n), see there.
#
#   METHODS
#	The VccSelectButton command creates a new command <pathName>.
#	This command may be used to invoke various operations on the widget.
#	It has the following general form:
#
#		<pathName> <method> ?<arg> <arg> ...?
#
#	<method> and the <arg>s determine the exact behavior of the command.
#	The following commands are possible for VccSelectButton widgets:
#
#
#	<pathName> MakeMenu <List> ?<Option>? ?<Menu>?
#
#	Assembles the menu connected to the button from a user-defined list.
#	The menubutton is activated to indicate that the menu has changed.
#	It returns the widget name of the first-level menu.
#
#	List	see PUBLIC OPTION `menuItems'.
#
#	Option	is prefixed to the parameter of the Callback. This is useful
#		for cascaded menus and normally only necessary for recursive
#		calls but not for user calls, except for the
#		special value `-nopropagate', which inhibits the propagation of
#		callback-parameters to the child-entries.
#		The argument is optional, default is an empty string.
#		See also PUBLIC OPTION `menuOption'.
#
#	Menu	is only used for recursion and specifies the widget-name 
#		of the target menu.
#		All entries of this menu - if any - are deleted before.
#		Any cascaded sub-menus are automatically created.
#
#	All entries of the top-level menu - if any - are deleted.
#	Any already existing sub-menu is destroyed.
#	Menus having sub-menu should be `destroy'ed before re-generating them,
#	otherwise unused sub-menus may occupy memory.
#	Sub-menus are built by internal recursive calls of this procedure.
#	Note that the `List' is stored inside the object only for the first
#	(non-recursive) call, i.e. when `Menu' is empty.
#
#   COMMON PROCEDURES
#	None.
#
#   DEFAULT BINDINGS
#	All default bindings are inherited from menubutton(n) and menu(n).
#
#   FILES
#
#   ENVIRONMENT
#
#   EXAMPLES
#	set menuList {a {} { {} "b b" d e} "c c c" d|ACC}
#	VccSelectButton .a -width 10 -text Attribute
#	.a configure -menuItems $menuList -menuAcceleratorToken "|"
#
#	This would produce the following cascaded menu:
#
#		+------------+
#		| * a        |
#		|  --------- |+-----+
#		|   b b     >|| * d |
#		|            || * e |
#		| * c c c    |+-----+
#		| * d     ACC|
#		+------------+
#
#   BUGS
#
#   SEE ALSO
#	menubutton(n), menu(n), options(n)
#
#------------------------------------------------------------------------
#

usual VccSelectButton {
    keep -background -foreground -font -relief -width -selectcolor
}

option add *VccSelectButton.relief raised                   widgetDefault
option add *VccSelectButton.width  1                        widgetDefault
option add *VccSelectButton.selectColor red                 widgetDefault
#NOTE: is a problem with selectColor handling!

class VccSelectButton {
    
    inherit itk::Widget VccDebug

    itk_option define -menuItems   menuItems  MenuItems  {}    
    itk_option define -menuOption  menuOption MenuOption {} 
    itk_option define -menuType    menuType   MenuType   radiobutton
    itk_option define -variable    variable   Variable   {}   
    itk_option define -command     command    Command    {}
    itk_option define -selectcolor selectColor Background red
    itk_option define -state       state       State      normal

    itk_option define -menuMaxLen  menuMaxLen MenuMaxLen 25
    itk_option define -menuAcceleratorToken  menuAcceleratorToken\
	    MenuAcceleratorToken {}   
    
    constructor { args } {}
    
    destructor {}
    public method MakeMenu { List {Option ""} {Menu ""} }
    private method invoke { arg }
    #------------------------------------------------------
    # Public Variables
    # NOTE: propagate all default values to the man-page!
    #------------------------------------------------------
    protected variable myMenu
}

body VccSelectButton::constructor {args} {
    debugEnter

    #---------------------
    # components
    #---------------------
    itk_component add mb {
	menubutton $itk_interior.mb\
		-relief raised -width 1 -text +
    } { 
	usual
    }
    pack $itk_component(mb)
    
    itk_component add menu {
	menu $itk_interior.mb.menu
    } {
	usual
	keep -selectcolor
    }
    
    $itk_component(mb) configure -menu $itk_component(menu)
    set myMenu $itk_component(menu)
    
    eval itk_initialize $args

    debugReturn
}

body VccSelectButton::MakeMenu { List {Option ""} {Menu ""} } {
    debugEnter
    
    # `Option' is either a special string or a prefix used for recursion:
    set o_nopropagate [cequal $Option -nopropagate]
    set o_none        [expr {!$o_nopropagate}]
    set prefix $Option
    
    # `Menu' is only non-empty for recursive calls:
    if {$Menu == ""} {
	set Menu $myMenu ;# starting point for first non-recursive call
	    set $itk_option(-menuItems) $List ;# save the given item-list
    }
    
    # working variables:
    set wMenu $Menu	;# widget-name of the current working menu
    set xMenu $Menu	;# dto. to save the name during last loop
    set nSubs  0	;# number of sub-menus on the current level
    set nItems 0	;# number of entries in the current menu
    #set nLeafs 0	;# total number of leafs in the hierarchy
   
    $wMenu delete 0 last
    foreach item $List {

	if {$nItems == 0} {
	    # On the top create a "more"-entry for a continuation-sub-menu.
	    # This entry is later removed if a more-menu is not needed.
	    set moreMenu $wMenu.more	;# new widget-name
	    
	    catch {destroy $moreMenu}	;# destroy if already existing
	    itk_component add menu_$wMenu.more { 
		menu $moreMenu			;# and (re-)create it
	    } {
		usual
	    }
	    $wMenu add cascade -label more... -menu $moreMenu 
	}
	
	# create cascade or leaf item:
	if { [llength $item] > 1 && [lindex $item 0] == {} } {
	    # create a sub-menu and connect it to the current entry:
	    set topLabel [lindex $item 1]	;# 2nd element is the label
	    set subList [lrange $item 2 end] ;# the rest forms the sub-menu
	    set subMenu $wMenu.[incr nSubs]	;# new widget-name
	    catch {destroy $subMenu}	;# destroy if already existing
	    itk_component add menu_$subMenu { 
		menu $subMenu			;# and (re-)create it
	    } {
		usual
	    }
	    $wMenu add cascade -label $topLabel -menu $subMenu
	    # call myself recursively for the sub-menu:
	    set subPrefix $prefix
	    if {! $o_nopropagate} {append subPrefix $topLabel}
	    MakeMenu $subList $subPrefix $subMenu
	} else {
	    if { $item == {} } {
		# empty string means a separator:
		$wMenu add separator
	    } else {
		# add a normal leaf-entry:
		set itext [ctoken item $itk_option(-menuAcceleratorToken)]
		set accel [ctoken item $itk_option(-menuAcceleratorToken)]
		set param [expr {$o_none ? "$prefix$itext" : $itext}]
		$wMenu add $itk_option(-menuType) \
			-label $itext \
			-accelerator $accel \
			-command [code $this invoke $param]
		# special handling of certain entry types:
		switch $itk_option(-menuType) {
		    radiobutton {
			$wMenu entryconfigure last \
				-variable $itk_option(-variable) -value $param\
				-selectcolor red ;# no other way than explicit?
		    }
		}
		#incr nLeafs
	    }
	}
	
	set xMenu $wMenu	;# save the name of the current menu, otherwise
				;# it's not possible to delete the "more..."
				;# entry when `menuMaxLen' is exactly matching.
	if {[incr nItems] >= $itk_option(-menuMaxLen)} {
	    # continue with the "more"-menu for the further entries:
	    set wMenu $moreMenu ;# from now on use this sub-menu instead
	    set nItems 0	;# re-start counting the entries
	}
    }
    # was 0 in Tk3.6 versions
    $xMenu delete 1	;# remove the preventively created "more" entry again
    catch {destroy menu_$xMenu.more}
    # activate menubutton to indicate that the menu has changed:
    #$this configure -state active
    
    debugReturn $myMenu
}

body VccSelectButton::invoke { arg } {
    debugEnter
    #global $itk_option(-variable);# NOTE that this doesn't work for array-vars
    set v $itk_option(-variable)  ;# ... therefore use a trick:
    # prevents to get a global ::          ===> ERROR!!
    if {![lempty $v]} {
	global ::[ctoken v \(]	  ;# for arrays only use basename before "(..)"
        set $itk_option(-variable) $arg
    }

    # call configured command:
    if {![cequal $itk_option(-command) {}]} {
	eval $itk_option(-command) {$arg}
    }
    debugReturn
}

#-------------------------------------
# Configuration options
#-------------------------------------

#-------------------------------------
# option -menuItems
#-------------------------------------
configbody VccSelectButton::menuItems {
    MakeMenu $itk_option(-menuItems) $itk_option(-menuOption)
}

#-------------------------------------
# option -menuOption
#-------------------------------------
configbody VccSelectButton::menuOption {
    MakeMenu $itk_option(-menuItems) $itk_option(-menuOption)
}

#-------------------------------------
# option -menuMaxLen
#-------------------------------------
# max.number of menu-entries on one level
# above that a "more..." sub-menu is created
configbody VccSelectButton::menuMaxLen {
    MakeMenu $itk_option(-menuItems) $itk_option(-menuOption)
}
#-------------------------------------
# option -menuType
#-------------------------------------
configbody VccSelectButton::menuType {
    MakeMenu $itk_option(-menuItems) $itk_option(-menuOption)
}

#-------------------------------------
# option -menuAcceleratorToken
#-------------------------------------
configbody VccSelectButton::menuAcceleratorToken {
    MakeMenu $itk_option(-menuItems) $itk_option(-menuOption)
}

#-------------------------------------
# option -variable
#-------------------------------------
configbody VccSelectButton::variable {
    if {[lempty $itk_option(-variable)]} {
	set $itk_option(-variable) $itk_interior
    }
    MakeMenu $itk_option(-menuItems) $itk_option(-menuOption)
}
    
#-------------------------------------
# option -command
#-------------------------------------
configbody VccSelectButton::command {}

#-------------------------------------
# option -state
#-------------------------------------
configbody VccSelectButton::state {
    $itk_component(mb) configure -state $itk_option(-state)
}

# ___oOo___
