#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccSelector.tcl,v 2.116 2005/10/21 14:15:08 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 15.05.1995 created
# S.Sandrock 05.10.1995 added completion function
# imunoz     02.04.1995 ported to itcl2.0
# imunoz     08.04.1996 fixed keep list of widget's components
# eallaert   1999-02-25 made compatible with both Tcl 7.6 and Tcl 8.0
# eallaert   2002-07-15 replaced file access via "exec" by "file" cmds
#

#************************************************************************
#   NAME
#	VccSelector - class for a selection widget
#
#   SYNOPSIS
#	VccSelector <pathName> ?-<option> <value> ...?
#	<pathName> configure ?-<option> <value> ...?
#	<pathName> <method> ?<arg> <arg> ...?
#
#   DESCRIPTION
#	The VccSelector command creates a new window (given by the pathName
#	argument) and makes it into a VccSelector widget. Additional options,
#	described below, may be specified on the command line or in the option
#	database to configure aspects of the widget.
#
#	The VccSelector command returns its pathName argument. At the time 
#	this command is invoked, there must not exist a window named pathName,
#	but pathName's parent must exist.
#
#	A VccSelector widget consists of a label, an input entry, a menubutton
#	with connected menu, and build-in properties to modify them.
#
#	The menubutton is of class VccSelectButton.
#	The button has always the width 1 and is labelled with `+'. 
#	The contents of the menu can be configured using a provided method.
#
#   INHERITANCE
#	VccSelector inherits completion functions from VccCompletion(n).
#	All properties of UifWidget(n) are inherited.
#
#   PUBLIC OPTIONS
#	The following options are specific to objects of this class:
#
#	-labelText	String displayed before the entry.
#			Default: "Label:"
#
#	-labelWidth	Width of the label (anchor is "e").
#			Default: (autosize)
#
#	-entryWidth	Width of the input entry.
#			Default: 10
#
#	-entryState	State of the input entry (normal | disabled)
#			Default: normal
#
#	-variable	Name of a variable connected to the input entry.
#			It will be set _after_ `command' has been evaluated.
#			Default: none
#
#	-command	Script to be executed when a menu item is
#			selected or <Return> is pressed in the entry.
#			The selected/input string is passed as argument to
#			this script. Note that if a `variable' is configured,
#			it will have still the _old_ value at this time!
#			Default: none
#
#	-userMenuEnable	If this is set to 1 then items entered via the
#			entry can be appended to the menu upon <Return>.
#			This requires that `command' is set, and that the
#			the return value of the executed script contains
#			the value to be appended (usually the same as the
#			input argument). If an empty string is returned,
#			then nothing is appended, allowing to sort out
#			invalid user inputs.
#			Default: 0
#
#	-menuMaxLen	See VccSelectButton(n). Default: 25
#
#	-menuType	See VccSelectButton(n). Default: radiobutton
#
#	-menuOption	See VccSelectButton(n). Default: ""
#
#	-state		Propagated to entry(n) and menubutton(n), see there.
#			Default: normal
#
#	The next options are to define events that invoke special functions.
#	Each can also be a list of keys etc. that trigger the same action.
#	An empty string means that no event is connected to the action.
#	NOTE: the focus events require Tk4 or higher!
#
#	-evtInvoke	to invoke the command	Default: <Return>
#	-evtComplete	to complete the entry	Default: {<Escape> <Tab>}
#	-evtFocusNext	to move focus forward	Default: {} ==> no event
#	-evtFocusPrev	to move focus backward	Default: {} ==> no event
#
#   METHODS
#	The VccSelector command creates a new command whose name is <pathName>.
#	This command may be used to invoke various operations on the widget.
#	It has the following general form:
#
#		<pathName> <method> ?<arg> <arg> ...?
#
#	<method> and the <arg>s determine the exact behavior of the command.
#	The following commands are possible for VccSelector widgets:
#
#
#	<pathName> MakeMenu <itemList> ?<option>?
#
#		Assembles the menu connected to the button from a user-defined
#		list. If the option `userMenuEnable' is set then the entries
#		of the user-defined menu are appended to <itemList>.
#		It maps into the corresponding method of `VccSelectButton'.
#		See VccSelectButton(n) for details.
#
#   COMMON PROCEDURES
#
#   DEFAULT BINDINGS
#	<Return>	to invoke the command
#	<Tab>		to complete the entry
#	<Escape>	to complete the entry
#
#	See PUBLIC OPTIONS to change event bindings.
#
#   FILES
#
#   ENVIRONMENT
#
#   EXAMPLES
#	VccSelector $cv.wLcuEnv -command vccEvhSelectLcuEnv -userMenuEnable 1 \
#		-variable gvar(vLcuEnv) -entryWidth 8 -labelWidth 15 \
#		-labelText {LCU Environment:} \
#		-helptext {LCU Environment which shall be configured} \
#		-evtInvoke {<Return> <FocusOut>} \
#		-evtComplete <Tab>
#	$cv.wLcuEnv MakeMenu {lte1 lte2 lte3}
#
#   CAUTIONS
#
#   BUGS
#
#   SEE ALSO
#	VccSelectButton(n), VccCompletion(n),
#	frame(n), label(n), entry(n), options(n)
#
#------------------------------------------------------------------------
#

option add *VccSelector.labelText      Label:                widgetDefault
option add *VccSelector.labelWidth     0                     widgetDefault
option add *VccSelector.entryWidth     10                    widgetDefault
option add *VccSelector.entryState     normal                widgetDefault
option add *VccSelector.menuType       radiobutton           widgetDefault
option add *VccSelector.menuMaxLen     25                    widgetDefault
option add *VccSelector.userMenuEnable 0                     widgetDefault


class VccSelector {

    inherit UifWidget VccCompletion

    # public options
    itk_option define -labelText  labelText  LabelText  {Label:}
    itk_option define -labelWidth labelWidth LabelWidth 0
    itk_option define -entryWidth entryWidth EntryWidth 10
    itk_option define -entryState entryState EntryState normal
    itk_option define -variable   variable   Variable   {}
    itk_option define -command    command    Command    {}
    itk_option define -state      state      State      normal
    itk_option define -menuType   menuType   MenuType   radiobutton
    itk_option define -menuMaxLen menuMaxLen MenuMaxLen 25
    itk_option define -menuOption menuOption MenuOption {} 
    itk_option define -userMenuEnable userMenuEnable UserMenuEnable 0
    itk_option define -evtInvoke    evtInvoke    EvtInvoke    <Return>
    itk_option define -evtComplete  evtComplete  EvtComplete  {<Tab> <Escape>}
    itk_option define -evtFocusNext evtFocusNext EvtFocusNext {}
    itk_option define -evtFocusPrev evtFocusPrev EvtFocusPrev {}
    
    constructor { args } {}
    destructor {}
    
    public  method MakeMenu { ItemList {Option ""} } 
    public  method childsite {}    

    protected   method invoke { {arg ""} } 
    protected   method complete { } 
    private     method setBindings { } 

    #------------------------------------------------------
    # Public Variables
    # NOTE: propagate all default values to the man-page!
    #------------------------------------------------------

    # none

    #------------------------------------------------------
    # Protected Variables
    #------------------------------------------------------

    protected variable menuItems {}	;# list of "official" menu items
    protected variable userMenuItems {}	;# list of user-added menu items
    protected variable lastCompletions {} ;# used in `complete' method
}

body VccSelector::constructor { args } {
    debugEnter
    
    #---------------------
    # components
    #---------------------
    itk_component add l {
	label $itk_interior.label -anchor e
    } {
	keep   -font -width -background -foreground
	rename -width -labelWidth labelWidth LabelWidth
	rename -text  -labelText labelText LabelText
    }
    
    # use normal Tk entry instead:
    itk_component add e {
	entry $itk_interior.entry -width 10
    } {
	keep -foreground -font -background
	rename -width -entryWidth entryWidth EntryWidth
	rename -state -entryState entryState EntryState
    }
    
    itk_component add b {
	VccSelectButton $itk_interior.b
    } {
	keep -font -foreground -background
    }
    
    itk_component add vccselchildsite {
	frame $itk_interior.vccselchildsite
    } {
	keep -background -cursor
    }
    set itk_interior $itk_component(vccselchildsite)

    # integrate options 
    eval itk_initialize $args

    # configure button widget
    $itk_component(b) configure -command [code $this invoke] \
	    -variable ${itk_interior}.b

    
    pack $itk_component(l) $itk_component(e) $itk_component(b) -side left
    pack $itk_component(e) -fill x -expand 1
    setBindings		;# set bindings for entry

    debugReturn
}

body VccSelector::MakeMenu { ItemList {Option ""} } {
    debugEnter
    set menuItems $ItemList
    set items $ItemList
    if {[lempty $menuItems]} {
	# only user-menu:
	set items $userMenuItems
    } elseif {![lempty $userMenuItems]} {
	# menu / separator / user-menu:
	set items [concat $ItemList {""} $userMenuItems]
    }
    Completion new [concat $ItemList $userMenuItems]
    debugReturn [$itk_component(b) MakeMenu $items $Option]
}

body VccSelector::childsite { } {
    return $itk_component(vccselchildsite)
}

#------------------------------------------------------
# Local Methods
#------------------------------------------------------

#
# method invoke: 
# process command with arg resp. the current entry contents
#
body VccSelector::invoke { {arg ""} } {
    debugEnter
    
    if { $arg == {} } {
	# <Return> pressed in entry:
	set arg [$itk_component(e) get]
	debugPrint "processing input from entry: \"$arg\""
	if { $itk_option(-command) != {} } {
	    set userItem [eval $itk_option(-command) {$arg}]
	    if { $itk_option(-userMenuEnable) && $userItem != {} && \
		    [lsearch -exact $menuItems $userItem] < 0 } {
		# append new item, sort list and remove any duplicates:
		set userMenuItems \
			[lrmdups [lappend userMenuItems $userItem]]
		MakeMenu $menuItems
	    }
	}
	# update variable connected to SelectButton
	global ::${itk_interior}.b
	set ${itk_interior}.b $arg
    } else {
	# item selected from menu:
	debugPrint "processing selection from menu"
	# execute the configured command (with entry/variable unchanged!):
	if { $itk_option(-command) != {} } {
	    eval $itk_option(-command) {$arg}
	}
	# then update the entry/variable with the selected item:
	$itk_component(e) delete 0 end
	$itk_component(e) insert 0 $arg
    }
    debugReturn
}

#
# method complete: perform completion of the current entry contents
#
body VccSelector::complete { } {
    debugEnter
    # get all possible completions for the current entry contents:
    set completions [Completion all [$itk_component(e) get]]
    if {[llength $completions] == 1} {
	# there is only one completions ==> insert it:
	$itk_component(e) delete 0 end
	$itk_component(e) insert 0 [lindex $completions 0]
    } else {
	if {[llength $completions] > 1} {
	    # there are many possible completions ==> insert the best:
	    set best [Completion best [$itk_component(e) get]]
	    $itk_component(e) delete 0 end
	    $itk_component(e) insert 0 $best
	    #for repeated attempt pop-up window with possible completions:
	    if {$completions == $lastCompletions} {
		vccWinBusy [namespace tail $this] \
		    "Completions:\n$completions" {sleep 2}
	    }
	    set lastCompletions $completions
	}
	# give an audio feedback:
	::bell
    }
    debugReturn $completions
}

#
# method setBindings: bind the configured events
#
body VccSelector::setBindings { } {
    debugEnter
    set entry $itk_component(e)

    foreach evt $itk_option(-evtInvoke) {
	debugPrint "bind $evt to invoke"
	::bind $entry $evt "[code $this invoke] ; break"
    }
    foreach evt $itk_option(-evtComplete) {
	debugPrint "bind $evt to complete"
	::bind $entry $evt "[code $this complete] ; break"
    }
    foreach evt $itk_option(-evtFocusNext) {
	debugPrint "bind $evt to tk_focusNext"
	::bind $entry $evt {tk_focusNext %q ; break} ;# >= Tk 4 only!
    }
    foreach evt $itk_option(-evtFocusPrev) {
	debugPrint "bind $evt to tk_focusPrev"
	::bind $entry $evt {tk_focusPrev %q ; break} ;# >= Tk 4 only!
    }

    ::bind $entry <Delete>  {%q delete 0 end}

    # the cursor key bindings are the same as in UifEntry:
    #::bind $entry <Left> \
    #    {%q icursor [expr [%q index insert] -1]; tk_entrySeeCaret %q}
    #::bind $entry <Right> \
    #	 {%q icursor [expr [%q index insert] +1]; tk_entrySeeCaret %q}

    debugReturn
}

#----------------------------------------
# OPTIONS
#----------------------------------------
configbody VccSelector::labelText {
    $itk_component(l) configure -text $itk_option(-labelText)
}

configbody VccSelector::labelWidth {
    $itk_component(l) configure -width $itk_option(-labelWidth)
}

configbody VccSelector::entryState {
    $itk_component(e) configure -state $itk_option(-entryState)
}

configbody VccSelector::entryWidth {
    $itk_component(e) configure -width $itk_option(-entryWidth)
}

configbody VccSelector::variable {
    if {[lempty $itk_option(-variable)]} {
	set itk_option(-variable) $itk_interior
    }
    $itk_component(e) configure -textvariable $itk_option(-variable)

    ##REMOVED: better not the same var for the menubutton!:
    ##$itk_component(b) configure -variable $itk_option(-variable)
}
    
configbody VccSelector::userMenuEnable {
    set userMenuItems {}	;# clear contents of user menu
}

configbody VccSelector::command {}		;# executed by invoke

configbody VccSelector::state  {
    $itk_component(e) configure -state $itk_option(-state)
    $itk_component(b) configure -state $itk_option(-state)
}

configbody VccSelector::menuMaxLen {
    $itk_component(b) configure -menuMaxLen $itk_option(-menuMaxLen)
}

configbody VccSelector::menuType  {
    $itk_component(b) configure -menuType $itk_option(-menuType)
}

configbody VccSelector::menuOption  {
    $itk_component(b) configure -menuOption $itk_option(-menuOption)
}

# ___oOo___
