#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccSelectorAll.tcl,v 2.116 2005/10/21 14:15:10 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 06.09.1995 created
# S.Sandrock 19.11.1995 added non-private VccInfo and validate selection
# imunoz     03.04.1996 ported to itcl2.0
# mcomin     19/08/99   Method VccSelectorAll::menuFilter modified in order
#                       to treat the menuFilter as a list 
#

#************************************************************************
#   NAME
#	VccSelectorAll - base-class for VccInfo-based selection widget
#
#   SYNOPSIS
#	inherit VccSelectorAll
#	constructor { args } {
#		set infoMethod ...
#		eval VccSelectorAll::constructor $args
#		...
#	}
#
#   DESCRIPTION
#	VccSelectorAll is a generic VccInfo-based selector widget.
#
#   INHERITANCE
#	It has the same properties as the VccSelector base class, except for
#	some specific deviations listed below.
#
#   STANDARD OPTIONS
#
#   PUBLIC OPTIONS
#	The following options are different from VccSelector:
#
#	-vccInfoObj	Name of a VccInfo object to be used for VCCDB queries.
#			If an empty string is given, then a private VccInfo
#			object will be created, and deleted automatically
#			when the selector widget is destroyed.
#			Otherwise the explicitly given object, which must
#			already exist, is used; it will not be deleted.
#			Default: vccInfo (i.e. the global VccInfo object)
#
#	-menuFilter	A glob-style filter expression that all menu entries
#			must match. It can also be a list of filters, which 
#			are then or-combined.
#			Default: *
#
#	-queryModes	Maps into VccInfo::queryModes.
#
#	-validate	Specifies whether a selected item shall be checked
#			for validity with the `validate' method.
#			A Tcl error with the appropriate message is
#			generated if the validation fails.
#			Default: 0 (off)
#
#   METHODS
#	validate {Item}	This method must be overloaded by the derived class
#			in order to check `Item' for validity.
#			It is called internally by the `invoke' method
#			if the option `validate' is enabled.
#			Return value must be an empty string on success,
#			or a non-empty string describing the fault.
#			(The generic method here returns always OK.)
#
#   COMMON PROCEDURES
#
#   DEFAULT BINDINGS
#
#   FILES
#
#   ENVIRONMENT
#
#   EXAMPLES
#
#   CAUTIONS
#
#   BUGS
#
#   SEE ALSO
#	VccSelector(n), VccInfo(n)
#
#------------------------------------------------------------------------
#

class VccSelectorAll {
    
    inherit VccSelector
    
    constructor { args } {}
    
    itk_option define -validate   validate   Validate   0
    itk_option define -menuFilter menuFilter MenuFilter *	    
    itk_option define -queryModes queryModes QueryModes {}
    itk_option define -vccInfoObj vccInfoObj VccInfoObj  ::vccInfo
    
    public     method invoke { {arg ""} } { }
    protected method validate { Selection } {}
    
    protected variable myVccInfo  {}	;# name of VccInfo object to be used
    protected variable infoMethod {}	;# VccInfo method to be used
}

body VccSelectorAll::constructor { args } {
    debugEnter
    debugPrint $args
    eval itk_initialize $args
    
    # connect to global or private VccInfo object:
    if {$itk_option(-vccInfoObj) == {}} {
	# create a VccInfo object for private use:
	set myVccInfo [VccInfo $this.vccInfo]
    } else {
	# use global VccInfo object:
	::vccInit
	set myVccInfo $itk_option(-vccInfoObj)
    }
    
    # use the default setting of queryModes, unless explicitly defined:
    if {$itk_option(-queryModes) == {}} {
	set itk_option(-queryModes) [$myVccInfo cget -queryModes]
    } else {
	configure -queryModes $itk_option(-queryModes)
    }
    # assemble the menu:
    if {$infoMethod != ""} {
    	configure -menuFilter $itk_option(-menuFilter)
    }
   
    # BUG (11425.4): the bindings should already be done in the inherited
    # class, but it has no effect! Therefore it's repeated:
    #setBindings		;# set bindings for entry
    if {$itk_option(-vccInfoObj) == {}} {
	# delete private VccInfo object when this widget is destroyed:
	::bind $this <Destroy> +[code $myVccInfo delete]
    }
    debugReturn
}

#
# invoke: called when an item is selected
# (overloads the VccSelector::invoke)
#
body VccSelectorAll::invoke { {arg ""} } {
    debugEnter

    if {$itk_option(-validate)} {
	if { $arg == {} } {
	    # <Return> pressed in entry:
	    set argVal [$itk_component(e) get]
	} else {
	    # selected from menu:
	    set argVal $arg
	}

	# call class-specific `validate' method to check selected item:
	if {[set msg [$this validate $argVal]] != ""} {
	    debugPrint "validate FAILED: $msg"
	    error $msg
	} else {
	    debugPrint "validate OK"
	}
    }

    # continue with method of the base-class:
    debugReturn [VccSelector::invoke $arg]
}

#
# validate: generic method to check a selected item for validity
# (must be overloaded by derived class with specific operations)
#
body VccSelectorAll::validate { Selection } {
    debugEnter
    debugReturn ""	;# "" ==> OK always
}


configbody VccSelectorAll::validate {}

configbody VccSelectorAll::menuFilter {
    debugEnter
    if {$myVccInfo != {} && $infoMethod != {}} {

       # Some methods have 'args' as parameters and other 

    
       switch $infoMethod {
           GetEnvs  {
              # These methods have 'args' as parameters therefore they need
              # to be evalueated ... 
              set filterList [lrmdups [split $itk_option(-menuFilter) " "]]
              MakeMenu [eval $myVccInfo $infoMethod $filterList]
           }
           default {
              MakeMenu [$myVccInfo $infoMethod $itk_option(-menuFilter)]
           }
       }
    }
    debugReturn
}

configbody VccSelectorAll::queryModes {
    debugEnter
    if {$myVccInfo != {}} {
	$myVccInfo configure -queryModes $itk_option(-queryModes)
    }
    debugReturn
}

# name of explicitly assigned VccInfo object
# if empty, then use private object

configbody VccSelectorAll::vccInfoObj {}

# ___oOo___
