#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: vccConfigLcu.tcl,v 2.116 2005/10/21 14:15:07 vltsccm Exp $"
#
# who        when       what
# ---------- ---------- ----------------------------------------------
# S.Sandrock 21.06.1995 created
# S.Sandrock 06.11.1995 adapted to split ROOT/DATA directory branches
# S.Sandrock 15.03.1996 modified to use vccConfigLcuLib and vccEnv
# fcarbogn   26.03.1999 Set default VLT_VCCBOOTTIME=500s, VLT_VCCTIMEOUT=50s
# psivera    21/01/01   changed lcubootAutoLoad to lcubootAutoLoadNoAbort (SPR 20000566)
# psivera    23/05/01   added ACSROOT and ACE_ROOT_VW
# eallaert   2002-07-15 replaced file access via "exec" by "file" cmds
# eallaert   2005-07-21 fixed problems with gvar(vDevCount) (SPR20000357)
# eallaert   2005-09-26 replaced sleep by vccWait
#

#************************************************************************
#   NAME
#	vccConfigLcu - Configuration Panel for LCU Boot Files
# 
#   SYNOPSIS
#	vccConfigLcu [<environment-name>] &
# 
#   DESCRIPTION
#	This panel allows to generate the most important files needed
#	for an LCU environment, in particular:
#
#		- the boot-script executed by the LCU upon start-up
#		- the processes-file used by engineering interfaces
#
#	If an explicit <environment-name> is given at the command-line,
#	then it is immediately selected and the panel's contents is updated
#	from the current configuration files, when present.
#	Otherwise it can be selected interactively.
#
#	The following sections describe the usage of each corresponding
#	section in the panel.
#
#   MENU BAR
#	File menu:
#		"Quit"		- exit the panel
#
#	Help menu:
#		"Extended Help"	- display this man-page
#
#   ENVIRONMENTS SELECTION
#	The first action must always be the selection of the LCU environment
#	that shall be modified. This is done by selection from the menu
#	at the top left in the panel. The derived values like node-name,
#	IP-address and associated boot-host are queried from the central
#	configuration database - see VccInfo(n) - provided it is active.
#	Otherwise these values have to be typed in.
#
#	The "Target LCU" parameters refer to the environment as a whole
#	and to the booting interface of it. In case of slave CPU systems
#	this may not be obvious; see MULTIPLE CPU SYSTEMS for more.
#
#	The "Boot WS" environment can be arbitrarily selected.
#	Remote file operations are performed if the "Boot Host" is not
#	identical to the workstation where this panel is running on.
#	Note that these remote accesses are much slower that local accesses.
#
#	"Target LCU"	- values related to the LCU to be configured
#	"Boot WS"	- values related to the WS the LCUs boots from
#
#	"Environment"	- name of the environment, as "lt1alt", "wt1tcs"
#	"Host"		- host where the environment is located
#	"IP Address"	- IP address of that host, eg. "11.22.33.44"
#	"TCP Port"	- assigned TCP port number of the environment
#	"CPU"		- CPU name, eg. "MC68040"; must match "vw/bin/<cpu>"
#	"Host Type"	- architecture of the host, eg. "68k" (for info only)
#
#	When a new "Target LCU Environment" is selected, the corresponding
#	values will be queried from the configuration database as far as
#	available. If defined, also the "Boot WS" is automatically selected.
#
#	Similarly, when a new "Boot WS Environment" is selected, the values 
#	are queried, and the available modules are updated as if the "Reset"
#	button in the MODULES CONFIGURATION section was pressed (see there).
#
#	"Boothome"	- directory where the environment resides
#	"VXROOT"	- VxWorks target root directory
#	"BSP"		- VxWorks board-support-package name, eg. "mv167".
#			  Values as "mv167-p0" indicate master/slave versions.
#			  See MULTIPLE CPU SYSTEMS for special issues.
#
#	BOOTHOME is derived from VLTDATA, meaning that the -->TARGET FILES
#	will be read from resp. written to that area, and that the LCU will
#	finally boot with the boot-script located there.
#
#	The type of system is indicated with the radio-buttons:
#
#	"Single"	- for a normal system with only one CPU
#	"Master"	- for the master in the system
#	"Slave"		- for slave #1 in the system (> 1 not supported)
#
#	The selection has only indirect effect on the BSP value,
#	which in turn results in the dedicated VxWorks kernel being loaded.
#	See MULTIPLE CPU SYSTEMS for more details.
#
#   ROOT CONFIGURATION
#	The variables BOOTHOME, VLTROOT, and optionally INTROOT must exist
#	in the LCU environment. The contents of the entries is by default
#	set to the same values of the corresponding shell variables on the
#	selected "Boot WS" host. They may be edited by the user.
#
#	VLTROOT and INTROOT may reside on a different host than BOOTHOME.
#	Implicit NFS mounts will be done on the LCU as appropriate.
#
#	Note that path names must be given as seen from the LCU. This might
#	in some cases be different from the path names on the WS, eg. when
#	there are links in the path. Only the physical path is allowed.
#	Lateron, however, the LCU NFS-mount may use different local names.
#
#	"VLTROOT" - host name and physical path to the VLTROOT area
#	"INTROOT" - host name and physical path to the INTROOT area
#
#	The "IP" field must contain the IP address in dot-notation of 
#	the selected host, if it is different from the "Boot WS" host.
#
#	When a new host is selected, the available modules are updated as if
#	the "Reset" button in the MODULES CONFIGURATION section was pressed.
#
#   NETWORK CONFIGURATION
#	The defaults in these entries will fit in most cases:
#
#	"Boot User"	specifies under which user-name the LCU is accessing
#			the boot-script. It is usually `vx'.
#	"Password"	specifies the password of the "Boot User".
#			It is usually empty, so that remote shell access is 
#			used; otherwise FTP is applied.
#
#	"NFS User"	specifies under which user ID the LCU is accessing
#			files via NFS. It is usually `vx'.
#			The first entry with the user name is only for 
#			convenience. The second entry is the important one,
#			and it must contain the corresponding numeric ID.
#	"NFS Group"	specifies under which group ID the LCU is accessing
#			files via NFS. It is usually the group of user `vx'.
#			It applies the same as for "NFS User"
#
#	"Main Host(IP)"	specifies the parameters of the on-board CPU network 
#			interface; this is usually the same as the booting
#			interface specified above, except for slave CPUs
#			which are booting over shared memory backplane.
#	"2nd Host (IP)"	specifies the parameters of an optional network 
#			interface, eg. additional ethernet board.
#	"3rd Host (IP)"	dto. for further interface.
#
#	"Subnet-Mask"	must be set to the network's inherent value.
#	"Gateway IP"	is the gateway IP address to the "Boot WS" host.
#			It should be empty when both host are on the same net.
#			This value is discarded in MULTIPLE CPU SYSTEMS.
#	"Backplane IP"	is the IP address assigned to the backplane interface.
#			This value is discarded in MULTIPLE CPU SYSTEMS.
#
#	The user-ID and group-ID for NFS accesses is mandatory and must be
#	set corresponding to a valid user on the boot-host. All valid user
#	and groups are permitted, by default the IDs of user "vx" are taken.
#
#	REMARK: the VLTROOT/INTROOT/MODROOT areas can be located on different 
#	hosts. The proc vccConfigLcuLib wil take care of checking whether 
#	the subnet is the same as the lcu subnet or not. If not, a route 
#	to the new subnet will be added, using the command "routeAdd" in 
#	the bootScript. 
#	
#	CAUTION: At the moment, ACSROOT and ACE_ROOT_VW are considered 
#	always located on the same host as VLTROOT. Eventually this should 
#	be generalised to whatever location in the future.
#
#
#   MODULES CONFIGURATION
#	The names of modules to be loaded must be specified in the two boxes.
#	Modules in the "System" box are normally always needed and their
#	order should not be changed. Modules in the "User" box usually
#	require all the system-modules and are not always needed, depending
#	on the LCU hardware etc.
#
#	Individual changes to the lists can be done by selecting an entry and
#	using the buttons on the right of the boxes.
#
#	For each module listed, there must exist a module-boot-script named
#	"<module>.boot" in one of the directories listed in -->FILES,
#	which should contain the auto-loading and initialization of the
#	module, using the functions of lcuboot(1).
#
#	"Add below"	- add the module from the entry below the selection
#	"Remove"	- remove the selected module
#	"Config..."	- open panel for individual module configuration
#	"Help..."	- display help for selected module (if available)
#	"Reset..."	- fill listboxes with all available modules. This
#			  implies that the available module-boot-scripts on
#			  the selected hosts (Boot WS, VLTROOT, INTROOT)
#			  are scanned to resolve module dependencies.
#			  For remote accesses this can take a while.
#
#   DEVICES CONFIGURATION
#	All hardware-devices on the LCU can be installed automatically,
#	using the facilities of lcuboot(1). The devices configuration therefore
#	consists only in checking that the required number of devices have
#	actually been installed by that procedure.
#
#	An explicit number can be given, or the special value `?' resp. `-1',
#	meaning that no explicit check shall be done. This is useful when
#	the board configuration in a LCU often changes. No modification in
#	the boot-script would be necessary then.
#
#	"Count"	 - the number of expected devices, or `-1' for auto-config
#	"Remove" - remove the selected device family from the listbox
#	"Reset"  - reset the listbox to the default
#
#   PROCESSES
#	This box contains the names of the LCU processes that shall be
#	accessible by commands via the message-system. The list is normally
#	only used by engineering interfaces (lccei, ccsei) to provide user-
#	menus, but not to restrict any access.
#
#	"Add"    - add the process from the entry to the listbox
#	"Remove" - remove the selected process from the listbox
#	"Reset"  - reset the listbox to the default
#
#   TARGET FILES
#	This box contains the names of files that shall be processed when
#	the action-button for "Read Files" or "Write Files" is pressed
#	(see -->ACTIONS).
#
#	"Reset"   - reset the listbox to the default
#	"Remove"  - omit the selected target file from any processing
#	"Edit..." - open an editor for the selected target file
#
#   ACTIONS
#	All action buttons are disabled until an LCU environment is selected.
#
#	"Create Env"	create a new set of LCU environment files under 
#			VLTDATA on the selected boot-host. The operation 
#			fails if the environment to be created already exists.
#
#	"Read Files"	read the files listed in the "Target Files" box and
#			parses their contents, so that the sections in the
#			panel are updated accordingly.
#			Note that the behaviour for reading files not produced
#			with this tool is undefined!
#
#	"Write Files"	write the files listed in the "Target Files" box
#			according to selections in the panel's sections.
#			A confirmation is requested before overwriting any
#			existing files.
#
#	"Configure LCU"	write the boot-line according to the selections in the
#			panel, i.e. host-names, IP-addresses etc.;
#			it uses the VxWorks `bootChange' command.
#			See -->DIAGNOSTICS for potential failures.
#
#	"Reboot LCU"	restart the LCU and execute the selected boot-script.
#			A transient window appears displaying the elapsed
#			boot time. The default waiting time before closing
#			the connection can be adjusted with the scale.
#			- NOTE THAT THE DEFAULT TIME MUST BE HIGHER THAN THE
#			  ACTUAL BOOTING TIME, THERE IS NO SYNCHRONIZATION -
#			The "Close Connection" button aborts the connection
#			explicitly, eg. when the booting has completed.
#			The output during start-up is redirected to the 
#			terminal where the panel was started from.
#			See -->DIAGNOSTICS for potential failures.
#
#   MULTIPLE NETWORK INTERFACES
#	An LCU may contain additional "passive" network interface boards.
#	Each of them must be assigned with a hostname and IP address.
#
#	In such case, there must be a list of hostnames assigned to this
#	environment in the configuration database, where normally is only 
#	one single hostname. The first in the list is always the on-board
#	main interface. The IP addresses are then resolved from the names.
#
#	Under the VxWorks shell there will be special variables defined,
#	which can be used by the network driver to setup the interfaces.
#	These are: LOCALHOST_n and LOCALIPADDRESS_n, with n = 1,2.
#	Up to two additional interfaces are supported.
#
#   MULTIPLE CPU SYSTEMS
#	The special case of multiple CPU systems is discussed here.
#
#	The master CPU will be configured to boot over its ethernet port,
#	as for a normal single CPU system. A specilized VxWorks kernel is
#	required in order to support booting of the slaves via the master
#	as gateway, see below.
#
#	The slave CPUs will be configured to boot over the backplane;
#	note that the hostname and IP addresses in the ENVIRONMENTS SELECTION
#	have to reflect this! The booting interface, and not the on-board
#	ethernet port, must be specified there.
#
#	By VLT convention multi-CPU systems shall use Proxy-ARP.
#	This implies different VxWorks kernels for master and slave CPUs,
#	where the following kernel options must be set:
#
#	- master CPU with proxy-server:
#	  INCLUDE_SM_NET		include backplane net interface
#	  INCLUDE_PROXY_SERVER		proxy arp server (Master Board)
#	  INCLUDE_PROXY_DEFAULT_ADDR	ethernet addr to generate bp addrs
#	  INCLUDE_SM_SEQ_ADDR		SM network auto address setup
#
#	- slave CPU with proxy-client:
#	  INCLUDE_SM_NET		include backplane net interface
#	  INCLUDE_PROXY_CLIENT		proxy arp client (Slave Board)
#	  INCLUDE_SM_SEQ_ADDR		SM network auto address setup
#
#	The VCC configuration distinguishes between master and slave CPUs
#	based on the value of the BSP field. If it has the form:
#
#		<basename>-p<procnum>-s<memsize>
#
#	then <memsize> indicates the CPU's decimal memory-size in MB and
#	<procnum> (0..15) indicates the processor-number, ie. zero
#	for the master and 1..15 for the slaves.
#
#	The corresponding VxWorks kernel must then exist under the 
#	location derived from the full BSP value, see FILES.
#
#	Slave CPUs will be configured to boot over shared memory located
#	in the master with the SM anchor set according to the memory-size,
#	namely: anchor = 2 * memorysize + 0x600.
#	This requires corresponding and consistent configuration in 
#	sysLib.c in the struct sysPhysMemDesc.
#
#	Due to the above mentioned kernel options, inet addresses for 
#	backplane of master and slaves, as well as gateway of slaves will 
#	be determined automatically by incrementing the masters ethernet
#	inet address. Explicit assignment of those is not supported.
#
#	When mv167 boards are booted over backplane, the on-board ethernet
#	interface is NOT automatically attached (although documented so).
#	Therefore explicit instructions are written into the bootScript
#	NETWORK section to do this. Note however that the ethernet i/f
#	will remain detached until the bootScript is executed.
#
#   ENVIRONMENT
#	VLTDATA		path to VLT data area
#	VLTROOT		path to VLT code area
#	INTROOT		path to private integration code area (optional)
#	VXROOT		path to VxWorks files
#
#	VLT_VCCENV	name of configuration database environment (optional)
#	VLT_VCCBOOTTIME	time in seconds after which reboot session is closed
#			(optional, default is 500 seconds)
#	VLT_VCCTIMEOUT	timeout in seconds for LCU reboot with VxWorks only
#			(optional, default is 50 seconds)
#
#	Other variables see VccInfo(n).
#
#   FILES
#	Under VLTDATA the following files may be read and/or written:
#
#	ENVIRONMENTS/<env>/bootScript	- boot-script for <env>
#	ENVIRONMENTS/<env>/userScript	- user-script for <env>
#	ENVIRONMENTS/<env>/devicesFile	- LCC software-devices for <env>
#	ENVIRONMENTS/<env>/PROCESSES	- process-file for <env>
#	ENVIRONMENTS/<env>/*.boot	- module-boot-scripts
#
#	Under VLTROOT or INTROOT the following files may be read only:
#
#	vw/bin/*.boot			- module-boot-scripts
#	vw/bin/<cpuName>/*.boot		- module-boot-scripts
#
#	Under VXROOT the following files may be read only:
#
#	config/<bsp>/vxWorks		- VxWorks kernel
#
#	See also VccInfo(n) and lcuboot(5) for accessed files.
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   DIAGNOSTICS
#	The -->ACTIONS "Configure LCU" and "Reboot LCU" require a `rlogin'
#	to the target LCU. The panel might be blocked when this remote login
#	fails, e.g. when another session is active on the same LCU.
#	To recover from this situation, the user must manually "kill" all
#	active rlogin processes to this LCU node.
#	This may also happen when the LCU has "crashed".
#
#   BUGS     
#	The action-button "Reboot LCU" does an `rlogin' to the target-LCU.
#	This remote-login is terminated after the time set with 
#	VLT_VCCBOOTTIME and is not synchronized with the boot-script execution.
#	A message appears on stdout when the connection is finally closed.
#
#   SEE ALSO
#	vccEnv(1), vccEnvCreate(1),
#	bootScript(5), userScript(5), module.boot(5),
#	VccInfo(n),
#	lcuboot(1)
#
#------------------------------------------------------------------------
#

#
# Called from panel-editor proc vccConfigLcuInit
#
proc vccConfigLcu { } {
    global cv env gvar vcc_n

    # Init VLT Central Configuration:
    if {[catch vccInit msg]} {puts $msg ; exit}

    # Init root vars (take physical names without links!):
    set gvar(VXROOT)  [vccRemPath "" \$VXROOT]
    set gvar(BOOTHOME) [vccRemPath "" \$VLTDATA] ;# will be extended later
    set host {};
    if {[info exists env(HOSTNAME)]} {
	set host $env(HOSTNAME)
    } elseif {[info exists env(HOST)]} {
	set host $env(HOST)
    }
    vccEvhSelectVltHost [set gvar(vVltHost) $host]
    vccEvhSelectIntHost [set gvar(vIntHost) $host]

    # Init network vars:
    set gvar(bootUser) vx
    set gvar(vPassword) ""
    set gvar(vGateway) ""
    set gvar(vSubnet) $vcc_n(netmask)

    set gvar(nfsUser) $gvar(bootUser)
    catch {id convert user $gvar(nfsUser)} gvar(nfsUserId)
    set gvar(nfsGroup) [id group]
    catch {id convert group $gvar(nfsGroup)} gvar(nfsGroupId)

    # set some bindings:
    update

    # reset targets on any change in critical entries:
    bind $cv.wBOOTHOME.uifEntry <FocusOut> vccSetTargets

    # bind <Return> in NFS-User/Group/ID entries:
    bind $cv.wNfsUser.uifEntry <Return> \
	    {set gvar(nfsUserId) [id convert user $gvar(nfsUser)]}
    bind $cv.wNfsUserId.uifEntry <Return> \
	    {set gvar(nfsUser) [id convert userid $gvar(nfsUserId)]}
    bind $cv.wNfsGroup.uifEntry <Return> \
	    {set gvar(nfsGroupId) [id convert group $gvar(nfsGroup)]}
    bind $cv.wNfsGroupId.uifEntry <Return> \
	    {set gvar(nfsGroup) [id convert groupid $gvar(nfsGroupId)]}

    # bind selection in System Modules listbox:
    $cv.wSysmodlist setBind <B1-ButtonRelease> {
	set gvar(vMod) [$cv.wSysmodlist getSelection]
	set gvar(wModlist) $cv.wSysmodlist
    }

    # bind selection in User Modules listbox:
    $cv.wUsrmodlist setBind <B1-ButtonRelease> {
	set gvar(vMod) [$cv.wUsrmodlist getSelection]
	set gvar(wModlist) $cv.wUsrmodlist
    }

    # bind selection in Devices listbox:
    $cv.wDevlist setBind <B1-ButtonRelease> {
	set gvar(vDev) [lindex [$cv.wDevlist getSelection] 0]
	# Watch out not to assign a non-numeric value to gvar(vDevCount), as there
	# is a trace on it via the slider widget; see also SPR 20000357
	if {[string equal [lindex $gvar(vDev) 0] "?"]} {
	    set gvar(vDevCount) -1
	} else {
	    set gvar(vDevCount) [lindex $gvar(vDev) 0]
	}
	set gvar(vDevName)  [lindex $gvar(vDev) 1]
	$cv.wDevCount setValue $gvar(vDevCount)
    }

    # bind selction of device-count:
    $cv.wDevCount.uifScale configure -command vccEvhSelectDevCount

    # execute standard init part:
    vccStdMain

    # reset Devices listbox:
    vccEvhDevicesReset

    # reset Processes listbox:
    vccEvhProcessesReset

    # set input focus to first entry:
    focus $cv.wLcuEnv.entry

    # if explicit environment given at command-line, then select it:
    global argv
    if {! [lempty $argv]} {
	vccEvhSelectLcuEnv [lindex $argv 0]
	catch vccActReadFiles ;# catch error if files are not present
    }
}


#
# Called when `Env' is selected from option-button "LCU Environment"
# - queries corresponding info from VCC database, calls `vccEvhSelectBootEnv'
#
proc vccEvhSelectLcuEnv { Env } {                                           
    global cv bcv gvar

    set gvar(vLcuEnv) $Env

    # execute lengthy command with busy cursor:
    vccWinBusy {
	# query vcc info for selected LCU-environment:
	vccConfigLcuGet gvar $gvar(vLcuEnv) ;# set gvar elements
	$cv.wProlist configure -itemList $gvar(vProlist)
	vccEvhSelectBootEnv $gvar(vBwsEnv)

	# query vcc info for selected LCU-environment:
	# (REPLACED)
	#lassign [vccInfo GetByEnv $Env {hostName tcpPort}] \
	#	gvar(vLcuHost) gvar(vLcuTCPport)
	#lassign [vccInfo GetByHost $gvar(vLcuHost) \
	#	{ipAddr cpuName hostType bspName}] \
	#	gvar(vLcuIPaddr) gvar(vLcuCpu) gvar(vLcuType) gvar(vLcuBsp)
	#$cv.wProlist configure -itemList [vccInfo GetProcsByEnv $Env]

	# make entries editable only if info was not available:
	# (REMOVED)
	#$cv.wLcuHost configure -state \
	#	[expr {$gvar(vLcuHost) == "" ? "normal" : "disabled"}]
	#$cv.wLcuIPaddr configure -state \
	#	[expr {$gvar(vLcuIPaddr) == "" ? "normal" : "disabled"}]
	#$cv.wLcuTCPport configure -state \
	#	[expr {$gvar(vLcuTCPport) == "" ? "normal" : "disabled"}]
	#$cv.wLcuCpu configure -state \
	#	[expr {$gvar(vLcuCpu) == "" ? "normal" : "disabled"}]
	#$cv.wLcuType configure -state \
	#	[expr {$gvar(vLcuType) == "" ? "normal" : "disabled"}]
	#$cv.wLcuBsp configure -state \
	#	[expr {$gvar(vLcuBsp) == "" ? "normal" : "disabled"}]

	# set the vcc-defined boot-env as default:
	# (REPLACED)
	#vccEvhSelectBootEnv  [vccInfo GetBootEnvByEnv $Env]

	# update names of target-files:
	vccSetTargets
	# now the action-buttons can be enabled:
	$bcv.wActCreateEnv	configure -state normal
	$bcv.wActReadFiles	configure -state normal
	$bcv.wActWriteFiles	configure -state normal
	$bcv.wActConfigureLcu	configure -state normal
	$bcv.wActRebootLcu	configure -state normal

	# set the sysType buttons from the BSP value:
	lassign [vccConfigLcuEvalBspName $gvar(vLcuBsp)] dummy gvar(sysType)
    }

    return $Env
}


#
# Called when `Env' is selected from option-button "Boot Environment"
# - queries corresponding info from VCC database
#
proc vccEvhSelectBootEnv { Env } {                                            
    global cv gvar vcc_n

    set lastEnv $gvar(vBwsEnv)
    set gvar(vBwsEnv) $Env

    # execute lengthy command with busy cursor:
    vccWinBusy {
	# query vcc info for selected boot-environment:
	vccConfigLcuGet gvar $gvar(vLcuEnv) $gvar(vBwsEnv) ;# set gvar elements

	# (REPLACED)
	#lassign [vccInfo GetByEnv $Env {hostName tcpPort}] \
	#	gvar(vBwsHost) gvar(vBwsTCPport)
	#lassign [vccInfo GetByHost $gvar(vBwsHost) {ipAddr cpuName hostType}]
	#	gvar(vBwsIPaddr) gvar(vBwsCpu) gvar(vBwsType)

	# make entries editable only if info was not available:
	# (REMOVED)
	#$cv.wBwsHost configure -state \
	#	[expr {$gvar(vBwsHost) == "" ? "normal" : "disabled"}]
	#$cv.wBwsIPaddr configure -state \
	#	[expr {$gvar(vBwsIPaddr) == "" ? "normal" : "disabled"}]
	#$cv.wBwsTCPport configure -state \
	#	[expr {$gvar(vBwsTCPport) == "" ? "normal" : "disabled"}]

	# query the root-variables on that host:
	# (REPLACED)
	#set gvar(VXROOT)  [vccRemPath $gvar(vBwsHost) \$VXROOT]
	#set vltdata [vccRemPath $gvar(vBwsHost) \$VLTDATA]
	#set gvar(BOOTHOME) $vltdata/$vcc_n(envDir)/$gvar(vLcuEnv)

	# update names of target-files:
	vccSetTargets

	# reset modules list if host has changed:
	# (REMOVED)
	#if {$Env != $lastEnv} {
	#    after 1 vccEvhModulesReset
	#    # Note: the pop-up busy-window appears only with "after 1" (!??)
	#}
    }

    return $Env
}


#
# Called when `Host' is selected from "VLTROOT host" selector
# - queries corresponding info from VCC database
#
proc vccEvhSelectVltHost { Host } {
    global cv gvar

    set lastHost $gvar(vVltHost)
    set gvar(vVltHost) $Host

    # execute lengthy command with busy cursor:
    vccWinBusy {
	# set gvar elements:
	vccConfigLcuGet gvar $gvar(vLcuEnv) $gvar(vBwsEnv) \
		$gvar(vVltHost) $gvar(vIntHost)

	# (REPLACED)
	#set gvar(VLTROOT) [vccRemPath $gvar(vVltHost) \$VLTROOT]
	#set gvar(vVltIPaddr) [vccInfo GetByHost $Host ipAddr]
	# make entry editable only if info was not available:
	# (REMOVED)
	#$cv.wVltIPaddr configure -state \
	#	[expr {$gvar(vVltIPaddr) == "" ? "normal" : "disabled"}]

	# reset modules list if host has changed:
	if {$Host != $lastHost} {
	    after 1 vccEvhModulesReset
	    # Note: the pop-up busy-window appears only with "after 1" (!??)
	}
    }

    return $Host
}


#
# Called when `Host' is selected from "INTROOT host" selector
# - queries corresponding info from VCC database
#
proc vccEvhSelectIntHost { Host } {
    global cv gvar

    set lastHost $gvar(vIntHost)
    set gvar(vIntHost) $Host

    # execute lengthy command with busy cursor:
    vccWinBusy {
	# set gvar elements:
	vccConfigLcuGet gvar $gvar(vLcuEnv) $gvar(vBwsEnv) \
		$gvar(vVltHost) $gvar(vIntHost)

	# (REPLACED)
	#set gvar(INTROOT) [vccRemPath $gvar(vIntHost) \$INTROOT]
	#set gvar(vIntIPaddr) [vccInfo GetByHost $Host ipAddr]
	# make entry editable only if info was not available:
	# (REMOVED)
	#$cv.wIntIPaddr configure -state \
	#	[expr {$gvar(vIntIPaddr) == "" ? "normal" : "disabled"}]

	# reset modules list if host has changed:
	if {$Host != $lastHost} {
	    after 1 vccEvhModulesReset
	    # Note: the pop-up busy-window appears only with "after 1" (!??)
	}
    }

    return $Host
}


#
# Called when "Reset" button in the MODULES section is pressed
# - searches for installed module-boot-script and puts them into the listboxes
#
proc vccEvhModulesReset { } {
    global cv gvar vcc_n vccConfigLcu_c

    # assemble list of search hosts and directories:
    set searchList {}
    if {$gvar(vBwsHost) != "" && $gvar(BOOTHOME) != ""} {
	lappend searchList \
		[list $gvar(vBwsHost) [list \
		$gvar(BOOTHOME) $gvar(BOOTHOME)/../../config ]]
    }
    if {$gvar(vVltHost) != "" && $gvar(VLTROOT) != ""} {
	 lappend searchList \
		 [list $gvar(vVltHost) [list \
		 $gvar(VLTROOT)/$vcc_n(bootDir)/$gvar(vLcuCpu) \
		 $gvar(VLTROOT)/$vcc_n(bootDir) ]]
    }
    if {$gvar(vIntHost) != "" && $gvar(INTROOT) != ""} {
	 lappend searchList \
		 [list $gvar(vIntHost) [list \
		 $gvar(INTROOT)/$vcc_n(bootDir)/$gvar(vLcuCpu) \
		 $gvar(INTROOT)/$vcc_n(bootDir) ]]
    }
    if {$gvar(vVltHost) != "" && $gvar(ACSROOT) != "" && $gvar(ACE_ROOT_VW) != ""} {
	 lappend searchList \
		 [list $gvar(vVltHost) [list \
		 $gvar(ACSROOT)/$vcc_n(bootDir)/$gvar(vLcuCpu) \
		 $gvar(ACE_ROOT_VW)/bin \
		 $gvar(ACSROOT)/$vcc_n(bootDir) ]]
    }

    set mods {}
    foreach searchItem $searchList {
      set host [lindex $searchItem 0]
      set searchDirs [lindex $searchItem 1]
      set comment "Scanning module-boot-scripts"
      if {$host != "" && $host != [exec hostname]} {
          append comment " (lengthy remote access!)"
      }

      # execute lengthy command with busy cursor:
      vccWinBusy {

	# get the available module-boot-scripts by searching in the dirs:
	foreach dir $searchDirs {
	    # display a busy comment in the short-help:
	    global shArea
	    $shArea displayText "$comment: $host:$dir"
	    update idletasks

	    set remCmd "ls ${dir}/*$vcc_n(bootExt)"
	    vccPuts $remCmd
	    if [catch {vccRemExec $host $remCmd} files] {
		continue
	    }
	    set files [split $files \n]

	    # filter the files, check each for dependencies:
	    foreach f $files {
		if {[file extension $f] == $vcc_n(bootExt)} {
		    set modname [file rootname [file tail $f]]
		    set bootfile($modname) $f
		    if {[lsearch $mods $modname] < 0} {
			lappend mods $modname	;# append if not already there
			# check for dependencies:
			vccPuts "checking dependencies in $host:$f"
			#next line is without pre-filtering:
			#set fcontents [vccRemExec $host "cat $f"]
			#next line is with pre-filtering:
			if [catch {vccRemExec $host \
				"grep lcubootAutoLoadNoAbort $f"} fcontents] {
			    set fcontents ""
			}
			set dep($modname) {}
			foreach line [split $fcontents \n] {
			    if [regexp {^lcubootAutoLoadNoAbort[^"]+"([^"]+)"} $line \
				    m dmod] {
				lappend dep($modname) $dmod
			    }
			}
			vccPuts "dep($modname) ==> $dep($modname)"
		    }
		}
	    }
	}
      }
    }

    if {[llength $mods] == 0} {
	vccPuts "no modules found!"
	global shArea
	$shArea displayText "no modules found!"
	return
    }

    # execute lengthy command with busy cursor:
    global shArea
    $shArea displayText "Resolving module dependencies ..."
    vccWinBusy {

	##sleep 1 ;# just to make the busy window more visible

	# sort out the system-modules in correct order, 
	# the remainder are user-mods:
	set sysMods {}
	set usrMods $mods
	foreach m $vccConfigLcu_c(sysMods) {
	    if {[set pos [lsearch -exact $usrMods $m]] >= 0} {
		lappend sysMods $m ;# add to system-modules
		set usrMods [lreplace $usrMods $pos $pos] ;# rm from user-mods
	    }
	}

	# resolve dependencies in user modules:
	set mods $usrMods
	set usrMods {}
	foreach m $mods {
	    foreach n $dep($m) {
		if {[lsearch -exact $usrMods $n] < 0 && \
		    [lsearch -exact $sysMods $n] < 0} {
		    vccPuts "prepend $n to base-module $m"
		    lappend usrMods $n
		}
	    }
	}

    }

    vccPuts "SYSTEM: $sysMods"
    vccPuts "USER: $usrMods"

    # put the modules into the boxes:
    $cv.wSysmodlist deleteItem 0 end
    $cv.wUsrmodlist deleteItem 0 end
    $cv.wSysmodlist addItemList end $sysMods
    $cv.wUsrmodlist addItemList end $usrMods
}


#
# Called when the "Count" scale in the DEVICES section is set
# - updated the associated listbox to save the selected value
#
proc vccEvhSelectDevCount { Count } {     
    global cv gvar
    if {[set i [$cv.wDevlist getIndex]] == ""} {
	set i end
    } else {
	$cv.wDevlist deleteItem $i
    }
    if {[set gvar(vDevCount) $Count] == -1} {
	$cv.wDevlist addItem $i "? $gvar(vDevName)"
    } else {
	$cv.wDevlist addItem $i "$gvar(vDevCount) $gvar(vDevName)"
    }
    $cv.wDevlist setCursor $i
}

#
# Called when the "Reset" button in the DEVICES section is pressed
#
proc vccEvhDevicesReset { } {
    global cv vccConfigLcu_c
    $cv.wDevlist configure -itemList $vccConfigLcu_c(devicesDefault)
}

#
# Called when the "Reset" button in the PROCESSES section is pressed
#
proc vccEvhProcessesReset { } {
    global cv vccConfigLcu_c
    $cv.wProlist configure -itemList $vccConfigLcu_c(processesDefault)
}


#
# Called when the action-button "Create Env" is pressed
# - calls `lccCreateEnv' to do the work
#
proc vccActCreateEnv { } {    
    global env cv gvar vcc_n

    vccSetTargets

    # execute lengthy command with busy cursor:
    vccWinBusy "Creating $gvar(vLcuEnv) on $gvar(vBwsHost) from template" {
	vccRemExec $gvar(vBwsHost) \
		"envsCreate -e $gvar(vLcuEnv) -t LCU -d $gvar(BOOTHOME)"
    }

    # create default database:
    vccWinBusy "Creating default database for $gvar(vLcuEnv)" {
	set remCmd "cd $gvar(BOOTHOME)/dbl; make db"
	if [catch {vccRemExec $gvar(vBwsHost) $remCmd} out] {
	    puts $out
	    error "Generation of default database with `make db' failed.\
		    Detailed message on stdout."
	}

	# make DB branch world writable;
	catch {chmod 0777 [list $gvar(BOOTHOME)/DB [recursive_glob $gvar(BOOTHOME)/DB *]]}
    }

    # overwrite template files:
    vccWinBusy "Now generating fully configured files" {
	vccActWriteFiles
    }
}


#
# Called when the action-button "Read Files" is pressed
# - interprets the target-files in BOOTHOME and displays their contents
#
proc vccActReadFiles { } {                                            
    vccSetTargets
    # execute lengthy command with busy cursor:
    vccWinBusy {
	global cv vcc_n
	set i 0
	foreach target [$cv.wTargets getList] {
	    $cv.wTargets setCursor $i; incr i; update idletasks
	    switch -regexp $target {
		bootScript\$	vccReadBootScript
		PROCESSES	vccReadProcessFile
	    }
	}
	selection clear $cv.wTargets	;# switch listbox-cursor off again
    }
}


#
# Called when the action-button "Write Files" is pressed
# - generates the target-files in BOOTHOME from the panel contents
#
proc vccActWriteFiles { } {                                            
    vccSetTargets
    # execute lengthy command with busy cursor:
    vccWinBusy {
	global cv vcc_n
	set i 0
	foreach target [$cv.wTargets getList] {
	    $cv.wTargets setCursor $i; incr i; update idletasks
	    switch -regexp $target {
		bootScript\$	vccWriteBootScript
		PROCESSES	vccWriteProcessFile
	    }
	}
	selection clear $cv.wTargets	;# switch listbox-cursor off again
    }
}


#
# Called when the action-button "Configure LCU" is pressed
# - does rlogin to LCU and executes `bootChange'
#
proc vccActConfigureLcu { } {                
    global gvar

    set bootChange [vccConfigLcuGenerateBootChange gvar $gvar(vBootscript)]

    if {[set pipe [vccRemLogin $gvar(vLcuHost)]] == ""} {return}

    foreach i [split $bootChange \n] {
	puts $pipe $i; flush $pipe
	after 250
    }
    vccWait 2000
    catch {close $pipe}
}


#
# Called when the action-button "Reboot LCU" is pressed
# - does rlogin to LCU
# - executes `bootChange' to temporarily disable the boot-script
# - issues `reboot'
# - re-installs the boot-script
# - sources the boot-script with redirected output
#
proc vccActRebootLcu { } {                
    global env gvar

    if {[set pipe [vccRemLogin $gvar(vLcuHost)]] == ""} {return}

    # first remove the start-up script, to see later its output:
    puts "\nuninstalling start-up script ..."
    set bootChange "bootChange\n\n\n\n\n\n\n\n\n\n\n\n\n.\n"
    foreach i [split $bootChange \n] {
	puts $pipe $i; flush $pipe
	after 250
    }
    vccWait 1000
    catch {close $pipe}
    puts \n

    # determine the configured reboot connection timeout:
    if {[info exists env(VLT_VCCBOOTTIME)]} {
	set bootTime $env(VLT_VCCBOOTTIME)
	puts "reboot timeout set to VLT_VCCBOOTTIME = $bootTime seconds"
    } else {
	set bootTime 500		;# default boot-time in seconds
	set env(VLT_VCCBOOTTIME) $bootTime
	puts "reboot timeout defaults to $bootTime seconds\
		(changeable with VLT_VCCBOOTTIME)."
    }
    vccWait 1000

    # determine the configured reconnection timeout:
    if {[info exists env(VLT_VCCTIMEOUT)]} {
	set rcnTime $env(VLT_VCCTIMEOUT)
	puts "reconnection timeout set to VLT_VCCTIMEOUT = $rcnTime seconds"
    } else {
	set rcnTime 50		;# default reconnection time in seconds
	set env(VLT_VCCTIMEOUT) $rcnTime
	puts "reconnection timeout defaults to $rcnTime seconds\
		(changeable with VLT_VCCTIMEOUT)."
    }

    # reboot VxWorks and login again:
    puts "\nrebooting VxWorks and waiting for reconnection ..."
    if {[catch {exec vccRemExpect $gvar(vLcuHost) reboot} msg]} {
	if {[string match "*\[Tt\]imeout*" $msg]} {
	    # on timeout: double the reconnection timeout, dialog to the user:
	    puts $msg
	    set env(VLT_VCCTIMEOUT) [expr {2 * $env(VLT_VCCTIMEOUT)}]
	    error "Reconnection timed out!\
		    \nI will increase the timeout to $env(VLT_VCCTIMEOUT) sec.\
		    You may then try again.\
		    If it fails permanently, check your LCU,\
		    and kill any pending rlogin processes to it.\
		    Note that the start-up script will remain uninstalled now."
	    return
	} else {
	    # on any other error: dialog to the user:
	    error "Reconnection failed:\n$msg\n\
		    Note that the start-up script will remain uninstalled now."
	    return
	}
    }
    if {[set pipe [vccRemLogin $gvar(vLcuHost)]] == ""} {return}
    #set pipe [open "|rlogin $gvar(vLcuHost)" w]; flush $pipe

    # reset the start-up script in the boot-line:
    puts "\nreinstalling start-up script ..."
    set bootChange "bootChange\n\n\n\n\n\n\n\n\n\n\n\n\n$gvar(vBootscript)\n"
    foreach i [split $bootChange \n] {
	puts $pipe $i; flush $pipe
	after 250
    }
    vccWait 1000

    # execute the start-up script directly from the shell:
    puts "\nexecuting bootScript ...\n"
    puts $pipe "< $gvar(vBootscript)"; flush $pipe

    # pop up a window to view the time and provide an abort button:
    set W .$gvar(vLcuEnv)
    vccStdToplevel [toplevel $W]
    pack [label $W.h -text "Rebooting $gvar(vLcuHost) ..."]
    pack [UifScale $W.s -label "Default Connection Time (seconds):" \
	    -minVal 30 -maxVal 1000 -length 10c \
	    -command [list set env(VLT_VCCBOOTTIME)] ] \
	    -fill x -expand 1
    $W.s setValue $bootTime
    pack [UifBarChart $W.a -label "Elapsed Time (seconds)" -length 10c \
	    -minVal 0 -maxVal $bootTime] \
	    -fill x -expand 1
    pack [UifActionButton $W.b -label "Cancel Connection" -width 20 \
	    -command [list destroy $W] ] \
	    -fill x -expand 1
    vccActRebootLcuView $W 0 $pipe ;# little barchart update proc

    # close connection after the timeout (just for safety):
    #after [expr $bootTime*1000] [list catch [list close $pipe]]
}

# little barchart update proc:
#	W: widget-name of barchart
#	T: elapsed time in seconds
proc vccActRebootLcuView { W T Pipe } {
    global env
    set tmax $env(VLT_VCCBOOTTIME)

    if {$T < $tmax && [winfo exists $W]} {
	$W.a configure -maxVal $tmax
	$W.a setValue $T ;# show elapsed time in barchart
	# reinvoke after 1 second:
	after 1000 [list vccActRebootLcuView $W [incr T] $Pipe]
    } else {
	if {[winfo exists $W]} {
	    destroy $W
	}
	catch {close $Pipe}
	puts "vccConfigLcu: connection closed ($Pipe)."
    }
}


#
# Interprets the boot-script and displays its contents in the panel
#
proc vccReadBootScript { } {
    global argv0 cv gvar

    # open the remote file:
    set fd [vccRemOpen $gvar(vBwsHost) $gvar(vBootscript)]

    # read and parse the contents:
    vccConfigLcuParseBootScript gvar [read $fd]
    vccRemClose $fd	;# close the remote file

    # update widgets from contents:
    $cv.wSysmodlist deleteItem 0 end
    $cv.wUsrmodlist deleteItem 0 end
    $cv.wDevlist deleteItem 0 end

    $cv.wSysmodlist addItemList end $gvar(vSysmodlist)
    $cv.wUsrmodlist addItemList end $gvar(vUsrmodlist)
    $cv.wDevlist    addItemList end $gvar(vDevlist)

    # set the sysType buttons from the BSP value:
    lassign [vccConfigLcuEvalBspName $gvar(vLcuBsp)] dummy gvar(sysType)
}


#
# Generates the boot-script from the contents of the panel
#
proc vccWriteBootScript { } {
    global argv0 cv gvar

    # open the remote file:
    set fd [vccRemOpen $gvar(vBwsHost) $gvar(vBootscript) w -dialog]
    if {$fd == ""} return

    # update vars from widgets:
    set gvar(vSysmodlist) [$cv.wSysmodlist getList]
    set gvar(vUsrmodlist) [$cv.wUsrmodlist getList]
    set gvar(vDevlist)    [$cv.wDevlist getList]
    set gvar(vProlist)    [$cv.wProlist getList]

    # write generated contents:
    puts $fd [vccConfigLcuGenerateBootScript gvar \
	    $gvar(vLcubootSource) $gvar(vUserScript)]

    vccRemClose $fd	;# close the remote file
}


#
# Reads the process-file and displays it in the panel
#
proc vccReadProcessFile { } {                                            
    global cv gvar

    # open the remote file and put it's contents into the process listbox:
    set fd [vccRemOpen $gvar(vBwsHost) $gvar(vProcessfile)]
    $cv.wProlist configure -itemList [split [read -nonewline $fd] \n]
    vccRemClose $fd	;# close the remote file
}


#
# Writes the process-file from the contents in the panel
#
proc vccWriteProcessFile { } {                                            
    global cv gvar

    # open the remote file and write the contents of the process listbox to it:
    set fd [vccRemOpen $gvar(vBwsHost) $gvar(vProcessfile) w -dialog]
    if {$fd == ""} return
    puts $fd [join [$cv.wProlist getList] \n]
    vccRemClose $fd	;# close the remote file
}


#
# Determines the names of target-files and displays them in the panel
#
proc vccSetTargets { } {   
    global cv gvar vcc_n

    set gvar(vBootscript)  $gvar(BOOTHOME)/bootScript
    set gvar(vUserscript)  $gvar(BOOTHOME)/userScript
    set gvar(vDevicesfile) $gvar(BOOTHOME)/devicesFile
    set gvar(vProcessfile) $gvar(BOOTHOME)/$vcc_n(proFile)

    $cv.wTargets deleteItem 0 end
    $cv.wTargets addItemList end [list \
	    $gvar(vBwsHost):$gvar(vBootscript) \
	    $gvar(vBwsHost):$gvar(vUserscript) \
	    $gvar(vBwsHost):$gvar(vDevicesfile) \
	    $gvar(vBwsHost):$gvar(vProcessfile) ]
}


# ___oOo___
