/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tacBlock.tpl,v 1.55 2005/10/03 17:38:02 vltsccm Exp $"
*
* who      when      what
* -------  --------  ----------------------------------------------
* bbauvir  21/03/01  Created
*/

/************************************************************************
*   NAME
*     tacTplBlock - Brief description of the function block
* 
*   DESCRIPTION
*     Block type: Tpl
*     Parameters: paramNb
*                 <List of parameters>
*     Inputs: inputNb
*             <List of input signals>
*     Outputs: outputNb
*              <List of output signals>
*
*     Thorough description of the block IO and function.
*
*   FILES
*
*   ENVIRONMENT
*     TAC
*
*   RETURN VALUES 
*
*   CAUTIONS 
*
*   EXAMPLES
*     TAC.BLOCK9.TYPE    Tpl
*     TAC.BLOCK9.NAME    Tpl
*     TAC.BLOCK9.PARAM   <value0>, <value1>
*
*   SEE ALSO
*
*   BUGS   
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: tacBlock.tpl,v 1.55 2005/10/03 17:38:02 vltsccm Exp $"; 
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* 
 * System Headers
 */

#include <stdio.h>     /* printf ... */
#include <stdlib.h>    /* malloc ... */
#include <string.h>    /* memcpy ... */
#include <math.h>

/* 
 * Local Headers
 */

#include "tacStdBlock.h"

/*
 * Compilation directives
 */

/*
 * Note: The following compilation directives are used to specify if, and which of, the
 *       optional function block hooks are provided with this block implementation.
 * 
 *       #define USE_REPORT_HOOK     to provide block instance parameter reporting.
 *       #define USE_PARAMETER_HOOK  to allow mofification of the block parameters 
 *                                   at runtime.
 *       #define USE_RESET_HOOK      to reset the block at runtime (e.g. output or history
 *                                   table).
 *       #define USE_STOP_HOOK       to provide stop behaviour for the block at runtime.
 *       #define USE_DESTRUCTOR_HOOK if this block implementation needs to perform
 *                                   specific operation before the block is removed from
 *                                   the memory (e.g. disable TIM board interrupt).
 *       #define USE_SHOW_HOOK       to allow to display some block information on the
 *                                   VxWorks shell when the tacRtcTaskShow function is 
 *                                   called.
 */

#undef USE_REPORT_HOOK
#undef USE_PARAMETER_HOOK
#undef USE_RESET_HOOK
#undef USE_STOP_HOOK
#undef USE_DESTRUCTOR_HOOK
#undef USE_SHOW_HOOK

/*
 * Constants
 */

/*
 * Note: tacTPL_EXPECTED_PARAM defines the number of parameters expected to be received
 *       by the block constructor.
 */

#define tacTPL_EXPECTED_PARAM paramNb

/*
 * Note: tacTPL_PARAM_NUMBER defines the number of parameter attributes constituting
 *       the specific 'Tpl' function block. It may be different from the number of 
 *       parameters passed at construction time (tacTPL_EXPECTED_PARAM).
 *
 * Example: Implementation of an Integration block will not receive construction parameters
 *          but needs to store locally 2 double values for its algorithmic purpose, one for 
 *          half the sampling frequency, the other to store the past value of its output
 *          signal.
 */

#define tacTPL_PARAM_NUMBER  paramNb    /* Size of parameter array attribute */

/*
 * Note: The two following constants define the block number of input and output signals.
 *
 * Example: Implementation of a summation block will require 2 inputs and produce 1 output.
 *
 *            #define tacSUM_INPUT_NUMBER  2
 *            #define tacSUM_OUTPUT_NUMBER 1
 */

#define tacTPL_INPUT_NUMBER  inputNb
#define tacTPL_OUTPUT_NUMBER outputNb

/*
 * Note: The local parameters are private to the block instance. The array is allocated as
 *       if the parameters are locally stored as double values but this array is untyped by
 *       design. The block implementation can manage/manipulate the local parameters in any
 *       suitable way (strings, shorts, etc.).
 */

#define tacTPL_PARAM_ARRAY_SIZE  tacTPL_PARAM_NUMBER*sizeof(double)

/*
 * Note: Function blocks have to provide TAC with arrays to identify the provenance of their
 *       input signals and to store the value of their output signals. These arrays are
 *       manipulated by TAC when interconnection links are defined. Therefore, the input
 *       attribute of the block instance is implemented as an array of addresses and output 
 *       as an array of doubles.
 * 
 * Example: A Gain block applies a transformation on 1 signal i.e. it receives 1 input and 
 *          produces 1 output. When a Gain block is created, it must provide TAC with an
 *          attribute to identify the input and another to store the output.
 *
 *          In an algorithm configuration file ...
 *
 *            TAC.BLOCK1.TYPE Gain
 *            TAC.BLOCK1.NAME myGain
 *            TAC.BLOCK1.PARAM 0.5
 *
 *          When the Gain block input is attached ...
 *
 *            TAC.LINK1.NAME Signal
 *            TAC.LINK1.SOURCE.NAME mySignal
 *            TAC.LINK1.SOURCE.PORT 1
 *            TAC.LINK1.DEST.NAME myGain
 *            TAC.LINK1.DEST.PORT 1
 *
 *          ... TAC writes the address of the first output value of mySignal block into the
 *          first element of myGain input array. 
 */

#define tacTPL_INPUT_ARRAY_SIZE  tacTPL_INPUT_NUMBER*sizeof(double*)
#define tacTPL_OUTPUT_ARRAY_SIZE tacTPL_OUTPUT_NUMBER*sizeof(double)

/* 
 * Types
 */

/*
 * Local function declaration
 */

STATUS tacTplBlockCheckParameter(tacSTDBLOCK* pSelf, tacSTDBLOCK_PARAM* parameter, tacERROR* error);

/*
 * Note: Routines called from within the algorithm must be declared with
 *       static __inline__ <prototype>
 */

/*
 * Global variables
 */

/*
 * Function definition
 */

/*
 * Note: Most function block types will not require many modifications of the 
 *       code provided in the following routines. 
 *
 *       The part of the code that is required by TAC and should not be modified
 *       will be identified by the following marks ...
 *       
 *         /-***************************************************-/
 *         /-* The following code is common to all block types *-/ 
 *         /-*      Do not modify without any good reasons     *-/
 *         /-***************************************************-/
 *
 *         /-*****************************-/
 *         /-* End of common code region *-/
 *         /-*****************************-/
 *
 *       The rest of the code is just there as example and may/should be modified 
 *       to fit the block-specific requirements.
 */

/*
 * Constructor Hooks - Called when a block of this type is instanciated (CONFIG or ADDBLCK commands)
 */

STATUS tacTplBlockConstructor (tacSTDBLOCK* pSelf,              /* Reference to the block instance   */
                               tacSTDBLOCK_PARAM* parameter,    /* Parameters of the ADDBLCK command */ 
                               tacERROR* error)                 /* Error structure - not a stack     */
{
  double* pRealParam = NULL;
  double* pSelfParam = NULL;

  /***************************************************/
  /* The following code is common to all block types */
  /*     Do not modify without any good reasons      */
  /***************************************************/

  /* Check that the parameters are valid */
  if (tacTplBlockCheckParameter(pSelf, parameter, error) == ERROR)
    {
      return ERROR;
    }

  /* Allocate and initialize block-specific arrays */
  if (tacStdBlockAllocateArrays(pSelf, tacTPL_PARAM_NUMBER, tacTPL_INPUT_NUMBER, tacTPL_OUTPUT_NUMBER, error) == ERROR)
    {
      return ERROR;
    }

  /*****************************/
  /* End of common code region */
  /*****************************/

  /* Store parameters */
  pRealParam = (double*) parameter->buffer;
  pSelfParam = (double*) pSelf->parameter;

  memcpy(pSelfParam, pRealParam, tacTPL_PARAM_ARRAY_SIZE);

  /* Perform other block initialization operations */

  return OK;
}

/*
 * Report parameters Hooks - Called to retrieve the current block parameters (REPORT command)
 */

#ifdef USE_REPORT_HOOK
STATUS tacTplBlockGetParameter (tacSTDBLOCK* pSelf,              /* Reference to the block instance    */
                                tacSTDBLOCK_PARAM* parameter,    /* Structure receiving the parameters */
                                tacERROR* error)                 /* Error structure - not a stack      */
{
  double* pRealParam = (double*) parameter->buffer;
  double* pSelfParam = (double*) pSelf->parameter;

  /***************************************************/
  /* The following code is common to all block types */
  /*     Do not modify without any good reasons      */
  /***************************************************/

  /* Send back the expected number of parameters */
  parameter->number = tacTPL_EXPECTED_PARAM;

  /*****************************/
  /* End of common code region */
  /*****************************/

  /* Report current parameters */
  memcpy(pRealParam, pSelfParam, tacTPL_PARAM_ARRAY_SIZE);

  return OK;
}
#endif

/*
 * Change parameters Hooks - Called when modifying the block parameters (MODBLCK command)
 */

#ifdef USE_PARAMETER_HOOK
STATUS tacTplBlockSetParameter (tacSTDBLOCK* pSelf,              /* Reference to the block instance   */
                                tacSTDBLOCK_PARAM* parameter,    /* Parameters of the MODBLCK command */ 
                                tacERROR* error)                 /* Error structure - not a stack     */
{
  double* pRealParam = (double*) parameter->buffer;
  double* pSelfParam = (double*) pSelf->parameter;

  /***************************************************/
  /* The following code is common to all block types */
  /*     Do not modify without any good reasons      */
  /***************************************************/

  /* Check that the parameters are valid */
  if (tacTplBlockCheckParameter(pSelf, parameter, error) == ERROR)
    {
      return ERROR;
    }

  /*****************************/
  /* End of common code region */
  /*****************************/

  /* Store parameters */
  memcpy(pSelfParam, pRealParam, tacTPL_PARAM_ARRAY_SIZE);

  return OK;
}
#endif

/* 
 * Algorithm Hooks - Called at each time the TAC algorithm is evaluated (ONLINE state)
 */

/*
 * Note: The input signals are accessed by reference, pSelf->input is an array of
 *       pointers, the value of which is defined by TAC when the associated link
 *       is identified (ADDLINK command). The output signals are accessed by value,
 *       pSelf->output is an array of doubles. 
 *
 * Example: The value of the input signals are retrieved using ...
 *            
 *            value = *(pSelf->input[inputIndex]);              /-* Direct access *-/
 *            value = tacStdBlockGetInput(pSelf,inputIndex);    /-* Just a macro  *-/
 *
 *          The output signal is updated using ...
 *
 *            pSelf->output[outputIndex] = value;               /-* Direct access *-/
 *            tacStdBlockSetOutput(pSelf,outputIndex,value);    /-* Just a macro  *-/
 */

void tacTplBlockAlgorithm (tacSTDBLOCK* pSelf)    /* Reference to the block instance */
{
  double* pSelfParam = (double*) pSelf->parameter;
  double input = tacStdBlockGetInput(pSelf, 0);

  /* Update output array */
#if tacTPL_OUTPUT_NUMBER
  if (input == 0)
      tacStdBlockSetOutput(pSelf, 0, 1);
  else
      tacStdBlockSetOutput(pSelf, 0, 0);
#endif
}

/* 
 * Reset Hooks - Called to reset the block (RESET command or going STANDBY state)
 */

#ifdef USE_RESET_HOOK
void tacTplBlockReset (tacSTDBLOCK* pSelf)    /* Reference to the block instance */
{
  double* pSelfParam = (double*) pSelf->parameter;

  /* Perform block reset operations (e.g. flush buffer of signal history) */

  return;
}
#endif

/* 
 * Stop Hooks - Called to react on a stop request (STOP command)
 */

#ifdef USE_STOP_HOOK
void tacTplBlockStop (tacSTDBLOCK* pSelf)    /* Reference to the block instance */
{
  double* pSelfParam = (double*) pSelf->parameter;

  /* Perform block stop operations */

  return;
}
#endif

/*
 * Destructor Hooks - Called when the block is removed from the algorithm (DELBLCK or new CONFIG commands)
 */

#ifdef USE_DESTRUCTOR_HOOK
void tacTplBlockDestructor (tacSTDBLOCK* pSelf)    /* Reference to the block instance */
{
  /* Perform unload operations */

  return;
}
#endif

/*
 * Show Hooks - Called when the tacRtcTaskShow routine is executed
 */

#ifdef USE_SHOW_HOOK
void tacTplBlockShow (tacSTDBLOCK* pSelf)    /* Reference to the block instance */
{
  double* pSelfParam = (double*) pSelf->parameter;

  /* Print parameter values */
#if tacTPL_PARAM_NUMBER
  printf("   Parameter: %g\n", pSelfParam[0]);
#endif
  return;
}
#endif
 
/***************************************************/
/* The following code is common to all block types */
/*     Do not modify without any good reasons      */
/***************************************************/

/*
 * Installation function
 */

tacSTDBLOCK_TYPE tacTplTypeInfo = {
  "Tpl",
  &tacTplBlockConstructor,

#ifdef USE_REPORT_HOOK
  &tacTplBlockGetParameter,    /* Optional hook */
#else
  NULL,                        /* Optional hook */
#endif

  &tacTplBlockAlgorithm,

  /* Optional hooks */

#ifdef USE_PARAMETER_HOOK
  &tacTplBlockSetParameter,
#else
  NULL,
#endif

#ifdef USE_RESET_HOOK
  &tacTplBlockReset,
#else
  NULL,
#endif

#ifdef USE_STOP_HOOK
  &tacTplBlockStop,
#else
  NULL,
#endif

#ifdef USE_DESTRUCTOR_HOOK
  &tacTplBlockDestructor,
#else
  NULL,
#endif

#ifdef USE_SHOW_HOOK
  &tacTplBlockShow,
#else
  NULL,
#endif
};

/*****************************/
/* End of common code region */
/*****************************/

STATUS tacTplBlockInitAll (void)
{
  /***************************************************/
  /* The following code is common to all block types */
  /*     Do not modify without any good reasons      */
  /***************************************************/

  /* Install Tpl Block Hooks */
  tacStdBlockInstallNewType(&tacTplTypeInfo);

  /*****************************/
  /* End of common code region */
  /*****************************/

  /* Perform other library initialization operations */

  return OK;
}

/*
 * Local functions
 */

STATUS tacTplBlockCheckParameter (tacSTDBLOCK* pSelf,              /* Reference to the block instance   */
                                  tacSTDBLOCK_PARAM* parameter,    /* Parameters of the ADDBLCK command */ 
                                  tacERROR* error)                 /* Error structure - not a stack     */
{
  double* pRealParam = (double*) parameter->buffer;

  /***************************************************/
  /* The following code is common to all block types */
  /*     Do not modify without any good reasons      */
  /***************************************************/

  /* Check that the expected number of parameters has been received */
  if (parameter->number < tacTPL_EXPECTED_PARAM)
    {
      char errorParam[40];

      sprintf(errorParam, "Extected %d, received %d", tacTPL_EXPECTED_PARAM, parameter->number);
      tacRTC_ERR(tacERR_PARAM_NUMBER, errorParam);
      return ERROR;
    }

  /*****************************/
  /* End of common code region */
  /*****************************/

  /* Check that the parameters are valid */
#if tacTPL_EXPECTED_PARAM
  if (parameter->type[0] != tacPARAM_DOUBLE)
    {
      tacRTC_ERR(tacERR_PARAM_INVALID, "Example - Must be a real value");
      return ERROR;
    }

  if (pRealParam[0] < 0.0)
    {
      tacRTC_ERR(tacERR_PARAM_INVALID, "Example - Must be higher than 0.0");
      return ERROR;
    }
#endif
  return OK;
}

/*
 * Note: Routines called from within the algorithm must be declared with
 *       static __inline__ <prototype>
 */


/*___oOo___*/
