#ifndef CANSTACK_H
#define CANSTACK_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: canstack.h,v 0.12 2005/10/10 13:09:50 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* vltuser5 2005-05-03 added new function argument to canstackSlaveCreateNode,
*                     added canstackDEV_NODE_INFO
*                     modifications to use more than 4 PDOs 
* vltuser5 2005-02-03 added initialisation routines of ESD canOpen layer,
*                     removed guarding functions, done by CANopen manager now
* mba(4DE) 2004-04-02 semaphore semGuarding and semEmergency added
* bgustafs 2002-11-14 added flag inUse to canOpenDEVICE
* bgustafs 2002-07-04 modified for standalone library
* vltuser5  17/01/02  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */	 
 
#include <semLib.h>
#include "canstackErrors.h"
#include "scanopen.h"

/******************** Define useful constants *********************************/

/* 
 * SYNC ID must differ for each node, SYNC ID for node 0 is  0x6C0, 
 * for node 1 ID is 0x6C1 etc.
 */ 
#define canstackSYNC_BASE				0x6C0
/* 
 * node id of the local slave must differ from the 
 * counterpart I/O device node:
 */                    
#define canstackOFFSET_SLAVE_NODE       (64)

/* 
 * directory index containing analog input data
 */                    
#define canstackINDEX_ANALOG_INPUT		(0x6401)


/* 
 * number of maximal CAN nets that are supported
 */                    
#define canstackMAX_NET       			(10)

/* 
 * number of bytes of a PDO 
 */                    
#define canstackPDO_NUM_BYTES           (8)

/* 
 * PDOs are asynchron + response only at a RTR   
 */                    
#define canstackPDO_ASYNCHRON           (255)

/* 
 * PDOs are cyclic + synchron + response at each SYNC 
 */                    
#define canstackPDO_CYCLIC_SYNC_1       (1)


#define canstackSTATE_PREOPERATIONAL    1  /* SDO possible, PDO not allowed */
#define canstackSTART                   2  /* transition to OPERATIONAL state */
#define canstackSTOP                    3  /* transition to STOPPED state */
#define canstackRESET_NODE              4  /* reboot node */
#define canstackRESET_COMMUNICATION     5
#define canstackSTATE_OPERATIONAL       6  /* SDO and PDO messages allowed */
#define canstackSTATE_STOPPED           7  /* error state */

#define canstackGUARD_TIME              10
#define canstackLIFE_TIME               5

#define canstackCOB_NMT                     0x00  /* COB-ID for NMT message */
#define canstackCOB_SYNC                    0x080 /* Sync-Identifier */
#define canstackCOB_EMERGENCY_BASE          0x080 /* Basic COB-ID for emergency messages */
#define canstackCOB_GUARDING_BASE           0x700 /* Basic COB-ID for guarding messages */
#define canstackCOB_SDO_REQUEST_BASE        0x600 /* Basic COB-ID for SDO request */
#define canstackCOB_SDO_RESPONSE_BASE       0x580 /* Basic COB-ID for SDO request */
#define canstackINDEX_NUMBER_OF_PDOS        0x1004
#define canstackINDEX_SYNC_IDENTIFIER       0x1005
#define canstackINDEX_RX_PDO_PARAMETER      0x1400
#define canstackINDEX_RX_PDO_MAPPING        0x1600
#define canstackINDEX_TX_PDO_PARAMETER      0x1800
#define canstackINDEX_TX_PDO_MAPPING        0x1A00
#define canstackINDEX_BUS_REGISTER          0x4500
#define canstackINDEX_TERMINAL_REGISTER     0x4501
#define canstackINDEX_PDO_ACTIVATION        0x5500
#define canstackINDEX_DIGITAL_INPUTS        0x6000
#define canstackINDEX_DIGITAL_OUTPUTS       0x6200
#define canstackINDEX_ANALOG_INPUTS         0x6401
#define canstackINDEX_ANALOG_OUTPUTS        0x6411

#define canstackSUBINDEX_COB_ID             0x01
#define canstackSUBINDEX_TRANSMISSION_TYPE  0x02

#define canstackUPLOAD_REQUEST              0x40   
#define canstackUPLOAD_RESPONSE_1BYTE       0x4F
#define canstackUPLOAD_RESPONSE_2BYTE       0x4B
#define canstackUPLOAD_RESPONSE_3BYTE       0x47
#define canstackUPLOAD_RESPONSE_4BYTE       0x43
#define canstackDOWNLOAD_REQUEST_1BYTE      0x2F
#define canstackDOWNLOAD_REQUEST_2BYTE      0x2B
#define canstackDOWNLOAD_REQUEST_3BYTE      0x27
#define canstackDOWNLOAD_REQUEST_4BYTE      0x23
#define canstackDOWNLOAD_RESPONSE           0x60
#define canstackERROR_RESPONSE              0x80
#define canstackRTR_FRAME                   0x101

/******************** Predefined dictionary mapping types**********************/
    
#define OD_INT8            0x1
#define OD_INT16           0x2
#define OD_INT32           0x3
#define OD_UINT8           0x4
#define OD_UINT16          0x5
#define OD_UINT32          0x6

/******************* Type definitions *****************************************/

/* struct describing the node parameters */
typedef struct
    {
    UINT16 devType;  /* device type (401, 402, ...) */
    UINT8  addInfo;  /* additional information of the device */
    } canstackDEV_NODE_INFO;

typedef canstackDEV_NODE_INFO  *canstackDEV_NODE_INFO_PTR;


/* enum describing the communication type of an I/O PDO of a CAN device */
typedef enum
	{
	SYNCHRON_RXPDO,	 	/* RxPDOs of local slave is synchron, means 
	                       received data is handed over to DataEventHandler */
	ASYNCHRON_RXPDO,  	/* RxPDOs of local slave is asynchron, means 
	                       received data is delivered to waiting Read() funct. */
    TXPDO,				/* TxPDO of local slave	*/ 

    UNUSED_PDO
    
	} canstackTYPE_PDO;

/* struct describing an I/O PDO of a CAN device */
typedef struct 
    {
    UINT16 				identifier;   /* COB-ID */
    UINT16 				mapping;      /* first mapped address */
    UINT16 				numElements;  /* number of mapped addresses in th PDO   */
    UINT16 				dataType;     /* dictionary mapping type */
    UINT16 				firstElement; /* First Subelement of an entry */
	canstackTYPE_PDO 	type;
    } canstackPDO;

typedef canstackPDO            *canstackPDO_PTR;


/* struct storing the I/O PDOs of a CAN device  */
typedef struct 
    {  
    canstackPDO_PTR  recvPDO;
    canstackPDO_PTR  sendPDO;
    UINT8            numRxPdo;
    UINT8            numTxPdo;
	int 			 (* eventHandler)(int,int,int,int,void*);  /* Data Event Handler,
	                                                              controlled by canio, not by canstack 	*/
    } canstackDEV_SETTING;

typedef canstackDEV_SETTING    *canstackDEV_SETTING_PTR;
 
typedef HPDO                   *canstackHPDO_PRT;

/* struct storing the PDO handles of a local slave  */
typedef struct 
    {  
    canstackHPDO_PRT   recvPdoHpdo; 
    canstackHPDO_PRT   sendPdoHpdo;
    HNODE  hNode;
    } canstackDEV_HANDLE;

typedef canstackDEV_HANDLE    *canstackDEV_HANDLE_PTR;

typedef struct 
    {
    unsigned short node;
    unsigned short net;
    int            card;
    int            chan;
    int            state;
    SEM_ID         semLock;
    SEM_ID         semGuarding;
    SEM_ID         semEmergency;
    BOOL           guarding;
    UINT8          errorInfo[8];
    BOOL           inUse;
    BOOL           sdoVerbose; /* If enables the SDO buffers are printed */
    BOOL           pdoVerbose; /* If enables the PDO buffers are printed */
    } canstackDEVICE;



/*
 * public canstack interface
 */

void   canstackPRINTF          (char *format, ... );
STATUS canstackInit            ( );
STATUS canstackManagerStart    (char * cfgFile);
STATUS canstackSlaveCreateNode (unsigned short net, unsigned short node,
                               canstackDEV_SETTING_PTR device, 
                               canstackDEV_HANDLE_PTR handle,
                               canstackDEV_NODE_INFO_PTR nodeInfo);
STATUS canstackSlaveDeleteNode (unsigned short net, unsigned short node, HNODE hNode);

/*
 * private canstack interface !!!!!!
 */
STATUS canstackResolveError        (int errorCode);
STATUS canstackSendDownloadRequest (canstackDEVICE *device, int objIndex, 
                                   int objSubIndex, INT8 *data, int numDataBytes);
STATUS canstackSendUploadRequest   (canstackDEVICE *device, INT32 objIndex, 
                                   INT32 objSubIndex, INT8 *data, INT32 *numDataBytes);
STATUS canstackSlaveSendSync	   (unsigned short net, unsigned short node);
					   
#endif /*!CANSTACK_H*/
