/******************************************************************************
*  E.S.O. - VLT project
* 
*  "@(#) $Id: ic0senCom.h,v 1.51 2005/08/15 13:36:27 vltsccm Exp $"
* 
*  who        when      what
*  ---------- --------  ----------------------------------------------
*  rschmutz   04/01/99  created.
*
*/

/*************************************************************************
*   NAME
*	ic0senCom - functions to access serial ports.
*
*   SYNOPSIS
*	#include "ic0senCom.h"
*
*	ccsCOMPL_STAT ic0senComOpen(
*	    IN    dbSYMADDRESS	point,
*	    OUT   vltINT32*	fd,
*	    IN    vltLOGICAL	simulation,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senComClose(
*	    IN    vltINT32	fd,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senComClean(
*	    IN    vltINT32	fd,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senComRead(
*	    IN    vltINT32	fd,
*	    OUT   char*		buf,
*	    INOUT vltINT32*	size,
*	    IN    char		endchar,
*	    IN    vltINT32	lines,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senComRead1(
*	    IN    vltINT32	fd,
*	    OUT   char*		buf,
*	    INOUT vltINT32*	size,
*	    IN    char*		endchars,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senComRead2(
*	    IN    vltINT32	fd,
*	    OUT   char*		buf,
*	    INOUT vltINT32*	size,
*	    IN    char		midchar,
*	    IN    char		endchar,
*	    IN    char		nackchar,
*	    IN    vltINT32	expLines,
*	    OUT   ccsERROR*	error)
*
*	ccsCOMPL_STAT ic0senComWrite(
*	    IN    vltINT32	fd,
*	    IN	  char*		buf,
*	    IN    vltINT32	size,
*	    OUT   ccsERROR*	error)
*
*	void ic0senComDelEndchars(
*	    IN	  char*		buf,
*	    INOUT vltINT32*	size,
*	    IN    char*		chars)
*
*	void ic0senComDelCr(
*	    IN	  char*		buf,
*	    INOUT vltINT32*	size)
*
*	void ic0senComDelCrLf(
*	    IN	  char*		buf,
*	    INOUT vltINT32*	size)
*
*	char* ic0senComInt2Hex(
*	    OUT	  char*		buf,
*	    IN	  vltINT32	i,
*	    IN	  vltINT32	precision)
*
*	vltINT32 ic0senComHex2Int(
*	    IN	  char*		buf,
*	    IN	  vltINT32	start,
*	    IN	  vltINT32	size)
*
*   DESCRIPTION
*	The ic0senCom*() functions can be used to access the
*	serial ports of a VLT LCU and/or the serial lines of an
*	iser board.  The serial port configuration must be stored
*	in an ic0senCOM OLDB point.
*
*	ccsCOMPL_STAT ic0senComOpen(point,fd,simulation,error)
*	    Opens a serial port and flushes the input buffers.
*	    If simulation, then no port is opened and fd returns -1.
*
*	    point	<IN>	OLDB point of type ic0senCOM where
*				the port configuration is stored.
*	    fd		<OUT>	returned port handle
*
*	ccsCOMPL_STAT ic0senComClose(fd,error)
*	    Closes a serial port.
*
*	ccsCOMPL_STAT ic0senComClean(fd,error)
*	    Flushes the input buffers.
*
*	ccsCOMPL_STAT ic0senComRead(fd,buf,size,endchar,lines,error)
*	    Reads from the serial port until the character <endchar>
*	    is found <lines> times.
*
*	    fd		<IN>	port handle
*	    buf		<IN>	character buffer
*	    size	<INOUT>	size of the buffer,
*				returns the number of char. read.
*	    endchar	<IN>	end character.
*	    lines	<IN>	number of times endchar should be found.
*
*	ccsCOMPL_STAT ic0senComRead1(fd,buf,size,endchars,error)
*	    Reads from the serial port one byte at a time until
*	    one of the <endchars> characters is found.
*
*	    fd		<IN>	port handle
*	    buf		<IN>	character buffer
*	    size	<INOUT>	size of the buffer,
*				returns the number of char. read.
*	    endchars	<IN>	end characters.
*
*	ccsCOMPL_STAT ic0senComRead2(fd,buf,size,midchar,endchar,
*				     nackchar,expLines,error)
*	    Reads from the serial port until <lines> lines have been found.
*	    Each line contains either a <midchar> followed later by <endchar>,
*	    or of the character <nackchar>.
*
*	ccsCOMPL_STAT ic0senComWrite(fd, buf, size, error)
*	    Write '\0' terminated buffer to the serial port.
*
*	    fd		<IN>	port handle
*	    buf		<IN>	character buffer
*	    size	<IN>	size of the buffer (or 0).
*
*	void ic0senComDelCr(buf, size)
*	    Deletes the last char. of the buffer, if it is a CR.
*
*	    buf		<INOUT>	character buffer
*	    size	<INOUT>	size of the buffer,
*				returns the new size.
*
*	void ic0senComDelCrLf(buf, size)
*	    As ic0senComDelCr(), but deletes CR and LFs.
*
*	char* ic0senComInt2Hex(buf,i,precision)
*	    Returns the hexadecimal representation of the number i
*	    in buf.
*
*	vltINT32 ic0senComInt2Hex(buf,start,size)
*	    Reads buf from position <start> (to start+size),
*	    expecting to find the hexadecimal representation of
*	    an integer.  Returns the integer (or 0).
*
*   RETURN VALUES
*	If the function prototype allows, methods return ccsCOMPL_STAT.
*
*   CAUTIONS
*	If no characters are received from a serial port, then
*	first write at least one character to it before reading
*	(this seems to be a bug of the VxWorks drivers).
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS
* 
*-------------------------------------------------------------------------
*/

#ifndef IC0SEN_COM_H
#define IC0SEN_COM_H

#include "egen.h"
#include "ic0senErrors.h"



ccsCOMPL_STAT ic0senComOpen(
    IN    dbSYMADDRESS	point,
    OUT   vltINT32*	fd,
    IN    vltLOGICAL	simulation,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senComClose(
    IN    vltINT32	fd,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senComClean(
    IN    vltINT32	fd,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senComRead(
    IN    vltINT32	fd,
    OUT   char*		buf,
    INOUT vltINT32*	size,
    IN    char		endchar,
    IN    vltINT32	lines,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senComRead1(
    IN    vltINT32	fd,
    OUT   char*		buf,
    INOUT vltINT32*	size,
    IN    char*		endchars,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senComRead2(
    IN    vltINT32	fd,
    OUT   char*		buf,
    INOUT vltINT32*	size,
    IN    char		midchar,
    IN    char		endchar,
    IN    char		nackchar,
    IN    vltINT32	expLines,
    OUT   ccsERROR*	error);

ccsCOMPL_STAT ic0senComWrite(
    IN    vltINT32	fd,
    IN	  char*		buf,
    IN    vltINT32	size,
    OUT   ccsERROR*	error);

void ic0senComDelEndchars(
    IN	  char*		buf,
    INOUT vltINT32*	size,
    IN    char*		chars);

char* ic0senComInt2Hex(
    OUT	  char*		buf,
    IN	  vltINT32	i,
    IN	  vltINT32	precision);

vltINT32 ic0senComHex2Int(
    IN	  char*		buf,
    IN	  vltINT32	start,
    IN	  vltINT32	size);

#define ic0senComDelCr(buf,size)   ic0senComDelEndchars((buf),(size),"\r")
#define ic0senComDelCrLf(buf,size) ic0senComDelEndchars((buf),(size),"\r\n")

#define ic0senErrComTimeout(p1) \
	{ \
	lcctooLOG_0("ic0senERR_COM_TIMEOUT: Serial device timeout (fd=%d).",(p1)); \
	errAdd(error,"ic0sen",ic0senERR_COM_TIMEOUT,__FILE_LINE__,"%d",(p1)); \
	}
#define ic0senErrComRead(p1) \
	{ \
	lcctooLOG_0("ic0senERR_COM_READ: Error reading from serial device (fd=%d).",(p1)); \
	errAdd(error,"ic0sen",ic0senERR_COM_READ,__FILE_LINE__,"%d",(p1)); \
	}
#define ic0senErrComWrite(p1) \
	{ \
	lcctooLOG_0("ic0senERR_COM_WRITE: Error writing to serial device (fd=%d).",(p1)); \
	errAdd(error,"ic0sen",ic0senERR_COM_WRITE,__FILE_LINE__,"%d",(p1)); \
	}


#endif // !IC0SEN_COOLING_H

/************************************************************************/
/* ___oOo___ */
