#ifndef IRVME_TASK_H
#define IRVME_TASK_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: irvmeTask.h,v 1.7 2005/09/12 08:08:39 vltsccm Exp $" 
*
* irvmeTask.h - Task control routines for irvme interface
*
* who        when       what
* ---------  ---------- ----------------------------------------------
* jstegmei   04/01/01   created
*/

#include "irvme.h"  

/*
 * Verbose Mode
 */
#define irvmeLOG_LOG     0
#define irvmeLOG_ERROR   1
#define vbLog(format, arg...) irvmeSrvLog(irvmeLOG_LOG, format, ##arg)
#define vbErr(format, arg...) irvmeSrvLog(irvmeLOG_ERROR, format, ##arg)

/*
 * Maximum number of data transfer tasks
 */
#define irvmeMAX_DT         2

/*
 * Maximum number of registered processing tasks
 */
#define irvmeMAX_TASK       64

/*
 * Maximum number of data sets
 */
#define irvmeMAX_DATA_SETS   4
/*
 * Supported parameter types
 */
#define irvmePTYPE_INT   1
#define irvmePTYPE_FLOAT 2

/*
 * Supported data types
 */
#define irvmeDTYPE_BYTE          0
#define irvmeDTYPE_REAL32        1
#define irvmeDTYPE_REAL64        2
#define irvmeDTYPE_INT16         3
#define irvmeDTYPE_INT32         4
#define irvmeDTYPE_INT64         5

/*
 * Transfer modes
 */
#define irvmeTMODE_NOTRANS    0    /* no transfer      */
#define irvmeTMODE_SCIENCE    1    /* science transfer */
#define irvmeTMODE_VIDEO      2    /* video transfer   */

/*
 * Timeout definitions (in seconds)
 */
#define irvmeTIMEOUT_SYNC      5   /* synchronization timeout          */
#define irvmeTIMEOUT_ERR_XFER  5   /* error-stack transmission timeout */
#define irvmeTIMEOUT_ERR_ACC   2   /* error-stack access timeout       */
#define irvmeTIMEOUT_ERR       (irvmeTIMEOUT_ERR_XFER+irvmeTIMEOUT_ERR_ACC)

/*
 * Parameter structure
 */
typedef struct
{
  char name[64];  /* parameter name            */
  int  value;     /* parameter (default) value */
  int  type;      /* parameter type            */
} irvmePARAM;

/*
 * Task status
 */
#define irvmeSTAT_IDLE        0
#define irvmeSTAT_WAITING     1
#define irvmeSTAT_PROCESSING  2
#define irvmeSTAT_READY       3
#define irvmeSTAT_FAILURE     4

/*
 * Frame request type
 */
typedef struct
{
  int  type;           /* requested frame type                         */
  int  blocking;       /* block until frame is ready                   */
  int  retransmit;     /* transfer last buffer again (video mode only) */
  int  start_x;        /* window start-x                               */
  int  start_y;        /* window start-y                               */
  int  nx;             /* window nx                                    */
  int  ny;             /* window ny                                    */
} irvmeDATA_REQ;

/*
 * Frame request acknowledge type
 */
typedef struct
{
  int      transfer;        /* transfer flag (1-transfer, 0-skip)  */
  int      start_x;         /* window start-x                      */
  int      start_y;         /* window start-y                      */
  int      nx;              /* window nx                           */
  int      ny;              /* window ny                           */
} irvmeDATA_ACK;

/*
 * Frame header type
 */
typedef struct
{
  int   dtype;               /* data type                          */
  int   ftype;               /* frame type                         */
  int   start_x;             /* window start-x                     */
  int   start_y;             /* window start-y                     */
  int   nx;                  /* window nx                          */
  int   ny;                  /* window ny                          */
  int   scal;                /* scale factor                       */
  int   cnt;                 /* frame counter                      */
  int   setupId;             /* setup-id                           */
  int   err;                 /* error-id                           */
  int   overrun;             /* overrun flag                       */
  int   frames;              /* available frame types              */
  int   tx;                  /* track point x                      */
  int   ty;                  /* track point y                      */
} irvmeFRAME_H;

/*
 * Frame type
 */
typedef struct
{
  irvmeFRAME_H   h;           /* header structure */
  char          *fbuf;        /* frame buffer     */
  char          *fbufOut;     /* frame buffer     */
} irvmeFRAME;

/*
 * Data connection object
 */
typedef struct
{
  int           id;             /* own ID                           */
  int           taskId;         /* VxWorks task id                  */
  SEM_ID        semData;        /* data object protection semaphore */
  SEM_ID        semDataT;       /* data trigger semaphore           */
  SEM_ID        semDataF;       /* data frame protection semaphore  */
  irvmeFRAME   *frame;          /* data frame                       */
  int           portNum;        /* port number of connection        */
  int           sockSrv;        /* server socket                    */
  int           sockClient;     /* client socket                    */
  char          clientName[64]; /* client id string                 */
  int           processId;      /* process-id of connected client   */
  int           tmode;          /* transfer mode                    */
  BOOL          connected;      /* connected flag                   */
  BOOL          aborted;        /* abort flag                       */
  BOOL          active;         /* active flag                      */
} irvmeDT_CON;

/*
 * Data transfer object
 */
typedef struct
{
  SEM_ID       semData;          /* data object protection semaphore */
  int          numCon;           /* number of connections            */
  irvmeDT_CON  con[irvmeMAX_DT]; /* conection object                 */
} irvmeDT;


/*
 * Data sets
 */
#define irvmeTASK_MEM_NO_CHANGE  -1
#define irvmeTASK_MEM_FREE        0
#define irvmeTASK_MEM_ALL        -1
typedef struct
{
  int   id;
  int   size;
  BOOL  valid;
  char *buf;
} irvmeDS;

typedef struct
{
  int           id;          /* registration id                */
  char          name[64];    /* registered task name           */
  FUNCPTR       entryPt;     /* VxWorks function entry point   */
  int           options;     /* VxWorks task option            */
  int           taskId;      /* VxWorks task id                */
  int           stackSize;   /* VxWorks stack size             */
  int           numParam;    /* number of parameters           */
  irvmePARAM   *param;       /* parameter structure            */
  SEM_ID        semAccess;   /* protection semaphore           */
  SEM_ID        semTask1;    /* synchronization semaphore 1    */
  SEM_ID        semTask2;    /* synchronization semaphore 2    */
  int           timeout;     /* protection timeout             */
  void         *argIn;       /* points to setup arguments      */
  void         *argOut;      /* pointe to arguments in use     */
  int           noff1;       /* frame offset                   */
  int           noff2;       /* frame offset                   */
  BOOL          abort;       /* abort flag                     */
  BOOL          running1;    /* running flag                   */
  BOOL          running2;    /* running flag                   */
  BOOL          end;         /* exposure end flag              */
  BOOL          rstCnt;      /* exposure reset flag            */
  BOOL          pause;       /* exposure pause flag            */
  int           status;      /* task status                    */
  int           hdr;         /* device header on GIGA-Bus      */
  SEM_ID        semData[irvmeMAX_DT];  /* data task semaphores */
  SEM_ID        semFrame[irvmeMAX_DT]; /* data task semaphores */
  irvmeDT      *dt;          /* data transfer object           */
  irvmeFRAME    frame;       /* output frame                   */
} irvmeTASK;

#ifdef __cplusplus
extern "C" {
#endif


/*
 * Control server
 */
STATUS irvmeServer(int cmdPort, int dataPort);
int    irvmeSrvCmdPort(void);
int    irvmeSrvDataPort(void);
int    irvmeSrvErrorPort(void);
void   irvmeErrStackAdd(const char *msg);
void   irvmeSrvAbort(void);
void   irvmeSrvLog(int logType, const char *str, ...);
void   irvmeSrvLogDisable(void);
void   irvmeSrvLogEnable(void);
irvmeDT *irvmeSrvData(void);

/*
 * Data tasks
 */
int    irvmeOpenData(irvmeDT *dataTask, int numDt, char *erms);
void   irvmeCloseData(irvmeDT *dataTask);
int    irvmeDataService(int sock, irvmeDT *dt, int service, int processId,
			int tmode, char *erms);
  
/*
 * Task execution
 */
int    irvmeTaskInit(void);
int    irvmeTaskRegister(const char *name, FUNCPTR entryPt, int options,
			 int stackSize, int numParam, irvmePARAM *param,
			 int protectionTimoeut, char *erms);
int    irvmeTaskId(const char *name);
void   irvmeTaskFree(int id);
void   irvmeTaskFreeAll(void);
int    irvmeTaskSpawn(int id, int timeout);
int    irvmeTaskVerify(int id);
int    irvmeTaskAbort(int id, int timeout);
int    irvmeTaskWait(int id, int timeout);
int    irvmeTaskSetParam(int id, const char *pname, const char *pval);
int    irvmeTaskGetParam(int id, const char *pname, char *pval);
int    irvmeTaskParamSetup(int id, irvmePARAM *param, int *numParam);
int    irvmeTaskParamSync(int id);
int    irvmeTaskMemValidate(int memId, BOOL valid);

int    irvmeTaskHdr(int id);
BOOL   irvmeTaskStart(int id);
BOOL   irvmeTaskCont(int id, BOOL brk);
void  *irvmeTaskParam(int id);
void   irvmeTaskError(int id, const char *erms);
BOOL   irvmeTaskReqOut(irvmeFRAME *frame, int id);
int    irvmeTaskArgSet(const char *arg);
char **irvmeTaskArgGet(void);
void   irvmeTaskFlags(int id, BOOL *resetFlag, BOOL *endFlag);
void  *irvmeTaskMem(int memId, int size, BOOL *valid);
int    irvmeTaskMemSize(int memId);

int  irvmeSp(FUNCPTR entryPt);

/*
 * Global task list
 */
extern irvmeTASK *irvmeTask[irvmeMAX_TASK];

/*
 * Verbose mode
 */
extern BOOL irvmeSrvVerbose;

#ifdef __cplusplus
}
#endif

#endif /* !IRVME_TASK_H */

/* __oOo__ */
