#ifndef LCUDRV_CHECK_DEV_H
#define LCUDRV_CHECK_DEV_H
/******************************************************************************
*
* E.S.O. - VLT project
*
* "@(#) $Id: lcudrvCheckDev.h,v 1.35 2005/10/11 09:15:26 vltsccm Exp $" 
*
* who        when       what
* ---------  ---------- ----------------------------------------------
* S.Sandrock 16.03.1995 New
* bgustafs   2002-01-18 added lcudrvCheckDevBaseAddrA32 for A32 addresses
*/

/*************************************************************************
*   NAME
*       lcudrvCheckDev,
*	lcudrvCheckDevDriver,
*	lcudrvCheckDevName,
*	lcudrvCheckDevBaseAddrA16,
*	lcudrvCheckDevBaseAddrA24,
*	lcudrvCheckDevProbe,
*	lcudrvCheckDevReturnOK
*	- Inline functions to check device installation parameters
*
*   SYNOPSIS
*	#define DEV_PREFIX <device-name-prefix>
*	#define FNAME <device-install-function-name>
*       #include "lcudrvCheckDev.h"
*	int lcudrvCheckDevDriver(int driverNumber)
*	int lcudrvCheckDevName(const char *devName, int maxUnits, int *unit)
*	int lcudrvCheckDevBaseAddrA16(void *baseAddrA16, int simBytes,
*				      char **localAddrA16)
*	int lcudrvCheckDevBaseAddrA24(void *baseAddrA24, int simBytes,
*				      char **localAddrA24)
*	int lcudrvCheckDevProbe(void *probeAddr)
*	int lcudrvCheckDevReturnOK(const char *devName)
* 
*   DESCRIPTION
*	All these function shall be used from inside a device-specific 
*	installation function (xxxDevCreate). They are declared as 
*	"static __inline__" so that the code will be incorporated into the
*	caller. They log any errors using the functions of the lculog module.
*
*	lcudrvCheckDevDriver - checks if a driver is installed for that device.
*		The `driverNumber' is as returned by `iosDrvInstall'.
*
*	lcudrvCheckDevName - checks the device-name and returns the unit
*		number in `unit', e.g. `1' for "/xxx1". It must not be greater
*		or equal to `maxUnits'.
*
*	lcudrvCheckDevBaseAddrA16 - converts a VMEbus-address in the short
*		address space into a local address as seen from the CPU.
*		If `baseAddrA16' is 0xffffffff, then `simBytes' are allocated
*		in main memory rather than using the real device space.
*
*	lcudrvCheckDevBaseAddrA24 - converts a VMEbus-address in the standard
*		address space into a local address as seen from the CPU.
*		If `baseAddrA24' is 0xffffffff, then `simBytes' are allocated
*		in main memory rather than using the real device space.
*
*	lcudrvCheckDevBaseAddrA32 - converts a VMEbus-address in the extended
*		address space into a local address as seen from the CPU.
*		If `baseAddrA32' is 0xffffffff, then `simBytes' are allocated
*		in main memory rather than using the real device space.
*
*	lcudrvCheckDevProbe - checks if the given address is valid.
*
*	lcudrvCheckDevReturnOK - logs that device installation is OK and
*		always returns lcudrvOK.
*
*   RETURN VALUES
*	All functions return lcudrvOK on success, or the appropriate error
*	code of the detected failure condition (lcudrvERROR_...).
*
*   ENVIRONMENT
*
*   CAUTIONS
*	As shown in SYNOPSIS, the macros `DEV_PREFIX' and `FNAME' must be
*	defined before including this header.
*
*	Unlike other files in `lcudrv', this requires the module `lculog',
*	not for loading, but for compilation of the drivers that use it.
*	The main purpose is to avoid code duplication in the device-specific
*	device installation functions.
*
*   EXAMPLE
*    #define DEV_PREFIX "/acro"
*    #define FNAME "acroDevCreate"
*    #include "lcudrvCheckDev.h"
*
*    int status;
*    
*    status = lcudrvCheckDevName(devName, acroMaxDevices, &unit);
*    if (status != lcudrvOK)
*	return status;
*    
*    status = lcudrvCheckDevBaseAddrA16(baseAddrA16, ... , &localBase);
*    if (status != lcudrvOK)
*	return status;
*
*    status = lcudrvCheckDevProbe(localBase);
*    if (status != lcudrvOK)
*	return status;
*
*--------------------------------------------------------------------------
*/

#include "lculog.h"
#include "sysLib.h"
#include "vme.h"


/*
 * Check if driver is installed for that device
 */
static __inline__
int lcudrvCheckDevDriver(int driverNumber)
{
    if (driverNumber < 0)
	{
	LCU_LOG_error("%s: driver not installed.", (int)FNAME, 0,0,0,0,0);
	return lcudrvERROR_NO_DRIVER;
	}
    return lcudrvOK;
}


/*
 * Check device name and get unit number
 */
static __inline__
int lcudrvCheckDevName(const char *devName, int maxUnits, int *unit)
{
    if (devName == NULL ||
	strncmp(devName, DEV_PREFIX, strlen(DEV_PREFIX)) != 0 ||
	sscanf(devName, DEV_PREFIX "%d", unit) != 1 ||
	*unit < 0 || *unit >= maxUnits)
	{
	LCU_LOG_error("%s: invalid device-name: \"%s\". "
		      "Must be \"%s0\" to \"%s%d\"." ,
		      (int)FNAME, (int)devName, 
		      (int)DEV_PREFIX, (int)DEV_PREFIX, maxUnits-1, 0);
	return lcudrvERROR_INVALID_DEVICE;
	}
    return lcudrvOK;
}


/*
 * Convert given VMEbus-address to local address,
 * simulate board memory if special base-address given:
 */
static __inline__
int lcudrvCheckDevBaseAddrA16(void *baseAddrA16, int simBytes,
			      char **localAddrA16)
{
    if (baseAddrA16 == (void *)0xffffffff)
	{
	LCU_LOG_warning("%s: Allocate A16 space in memory for simulation.", 
			(int)FNAME, 0,0,0,0,0);
	*localAddrA16 = malloc((size_t)simBytes);
	if (*localAddrA16 == NULL)
	    return lcudrvERROR_NO_MEMORY;
	}
    else
	{
	/* for baseAddrA16: ignore bits 31..16 and convert to CPU address: */
	baseAddrA16 = (void *)((int)baseAddrA16 & 0x0000ffff);
	if (sysBusToLocalAdrs(VME_AM_SUP_SHORT_IO, baseAddrA16, localAddrA16)
	    == ERROR)
	    {
	    LCU_LOG_error("%s: could not convert baseAddrA16 to CPU addr: %p", 
			  (int)FNAME, (int)baseAddrA16, 0,0,0,0);
	    return lcudrvERROR_INVALID_ARGUMENT;
	    }
	}
    return lcudrvOK;
}


/*
 * Convert given VMEbus-address to local address,
 * simulate board memory if special base-address given:
 */
static __inline__
int lcudrvCheckDevBaseAddrA24(void *baseAddrA24, int simBytes,
			      char **localAddrA24)
{
    if (baseAddrA24 == (void *)0xffffffff)
	{
	LCU_LOG_warning("%s: Allocate A24 space in memory for simulation.", 
			(int)FNAME, 0,0,0,0,0);
	*localAddrA24 = malloc((size_t)simBytes);
	if (*localAddrA24 == NULL)
	    return lcudrvERROR_NO_MEMORY;
	}
    else
	{
	/* for baseAddrA24: ignore bits 31..24 and convert to CPU address: */
	baseAddrA24 = (void *)((int)baseAddrA24 & 0x00ffffff);
	if (sysBusToLocalAdrs(VME_AM_STD_SUP_DATA, baseAddrA24, localAddrA24)
	    == ERROR)
	    {
	    LCU_LOG_error("%s: could not convert baseAddrA24 to CPU addr: %p", 
			  (int)FNAME, (int)baseAddrA24, 0,0,0,0);
	    return lcudrvERROR_INVALID_ARGUMENT;
	    }
	}
    return lcudrvOK;
}


/*
 * Convert given VMEbus-address to local address,
 * simulate board memory if special base-address given:
 */
static __inline__
int lcudrvCheckDevBaseAddrA32(void *baseAddrA32, int simBytes,
			      char **localAddrA32)
{
    if (baseAddrA32 == (void *)0xffffffff)
	{
	LCU_LOG_warning("%s: Allocate A32 space in memory for simulation.", 
			(int)FNAME, 0,0,0,0,0);
	*localAddrA32 = malloc((size_t)simBytes);
	if (*localAddrA32 == NULL)
	    return lcudrvERROR_NO_MEMORY;
	}
    else
	{
	/* for baseAddrA32: convert to CPU address: */
	if (sysBusToLocalAdrs(VME_AM_EXT_SUP_DATA, baseAddrA32, localAddrA32)
	    == ERROR)
	    {
	    LCU_LOG_error("%s: could not convert baseAddrA32 to CPU addr: %p", 
			  (int)FNAME, (int)baseAddrA32, 0,0,0,0);
	    return lcudrvERROR_INVALID_ARGUMENT;
	    }
	}
    return lcudrvOK;
}


/*
 * Check if board is actually present, exit if not.
 */
static __inline__
int lcudrvCheckDevProbe(void *probeAddr)
{
    if (lcudrvProbeAddress(probeAddr) != lcudrvOK)
	{
	LCU_LOG_error("%s: nothing installed at CPU address 0x%x.", 
		      (int)FNAME, (int)probeAddr, 0,0,0,0);
	return lcudrvERROR_INVALID_ARGUMENT;
	}
    return lcudrvOK;
}


/*
 * Check if board is actually present, exit if not.
 */
static __inline__
int lcudrvCheckDevReturnOK(const char *devName)
{
    LCU_LOG_log ("%s: %s: Device installation OK.",
                 (int)FNAME, (int)devName, 0, 0, 0, 0);
    return lcudrvOK;  
}


#endif  /* !LCUDRV_CHECK_DEV_H */
