#ifndef tooInterface_H
#define tooInterface_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: tooInterface.h,v 1.45 2004/11/16 13:36:44 vltsccm Exp $" 
*
* who       when      what
* --------  --------  ----------------------------------------------
* bgustafs  05/07/99  made it C++ compliant
* S.Sandrock 30.10.1996 created in module asm
*/

/************************************************************************
*   NAME
*	tooINTERFACE - Macros to build LCC-CI interface functions
*
*   SYNOPSIS
*	tooINTERFACE_NOARG(function)
*	tooINTERFACE_ARG(function, argtype, argfmt)
*	tooINTERFACE_ARG_ARG(function, argtype1, argtype2, argfmt)
*	tooINTERFACE_STR(function)
*	tooINTERFACE_STR_STR(function)
*	tooINTERFACE_STR_ARG(function, argtype, argfmt)
*	tooINTERFACE_STR_STR_ARG(function, argtype, argfmt)
*	tooINTERFACE_STR_ARG_ARG(function, argtype1, argtype2, argfmt)
*
*	tooINTERFACE_REPLY_RETURN(format, args...)
*
*   DESCRIPTION
*	These macros produce an interface-function for a command that is 
*	compliant with the LCC command-interpreter. Depending on the CDT
*	parameter-list for the command, one of the macros must be used:
*
*	    NOARG	for no paramters
*	    ARG		for one scalar parameter
*	    ARG_ARG	for two scalar parameters
*	    STR		for one string parameter
*	    STR_STR	for two string parameters
*	    STR_ARG	for one string and one scalar parameter
*	    STR_STR_ARG	for two strings and one scalar parameter
*	    STR_ARG_ARG	for one string and two scalar parameters
*
*	The macro arguments are:
*
*	    function	Name of the function to be interfaced
*	    argtype	Type of a scalar parameter, eg. vltUINT32
*	    argfmt	Format-string corresponding to argtype, eg. "%d"
*
*       The generated interface-function has a standard prototype:
*
*		ccsCOMPL_STAT I<function>(
*			char         *parameters,
*			msgMSG       *msg,              
*			char         **replyBuffer,     
*			msgLENGTH    *replyBufferSize,  
*			ccsERROR     *error)
*
*		parameters      <IN>  Formatted input parameters
*		msg             <IN>  Command message received
*		replyBuffer     <OUT> Output buffer for reply message
*		replyBufferSize <OUT> Output buffer size
*		error           <OUT> Returned error structure
*
*	The packed parameters are then resolved and passed to the
*	underlying function with the proper types in order, plus
*	pointers to the message and error-stack.
*
*	The prototype of the original function must therefore be:
*
*		ccsCOMPL_STAT <function>(
*			<type1>		<param1>,
*			...		...,
*			msgMSG		*msg,
*			ccsERROR	*error)
*
*	tooReplyf may be used in the function body to send intermediate
*	replies, where msg is used to extract the message parameters.
*
*	The default last reply to be sent on SUCCESS is "OK".
*
*	tooINTERFACE_REPLY_RETURN() may alternatively be specified at the
*	end of the function body to format a string that will be used as
*	last reply, and return with SUCCESS.
*	The identifier `msg' must be defined as pointer to the current message!
*	If `msg' is NULL then the reply is printed to stdout with trailing 
*	new-line.
*
*   EXAMPLES
*	The handler function, here with one string parameter:
*
*	    ccsCOMPL_STAT asmCmdSimulat(const char *devName, 
*					msgMSG *msg, ccsERROR *error)
*	    {
*		return lccDevEnterSim(devName, error);
*	    }
*
*	And the interface function will then be generated by the macro:
*
*	    tooINTERFACE_STR(asmCmdSimulat)
*
*	The last reply is "OK" unless explicitly specified as in:
*
*	    ccsCOMPL_STAT asmCmdVersion(msgMSG *msg, ccsERROR *error)
*	    {
*		... (assemble str) ...
*		tooINTERFACE_REPLY_RETURN(str);
*	    }
*
*	    tooINTERFACE_NOARG(asmCmdVersion)
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES
*
*   TO DO
*	More parameter type combinations than the above may be added.
*	I have only implemented what I needed for the ASM.
*
*   SEE ALSO
*	tooReplyf(3)
*
*------------------------------------------------------------------------
*/
#ifdef __cplusplus
extern "C" {
#endif

#include "too.h" /* for tooReplyf() */

/* format last reply and return */
#define tooINTERFACE_REPLY_RETURN(format, args...)\
{\
    if (msg != NULL) sprintf(msg->body, format, ##args);\
    else printf(format, ##args), printf("\n");\
    return SUCCESS;\
}


/* send verbose reply if global verbose-mode is on */
#define tooINTERFACE_VERBOSE(format, args...)	\
	tooReplyfVerbose(msg, format, ##args)



/* standard interface function */
#define tooINTERFACE(function, argfmt, args...)				\
ccsCOMPL_STAT I ## function (						\
    IN  const char *parameters,		/* Formatted input parameters	*/\
    IN  msgMSG     *msg,		/* Message body used for reply	*/\
    OUT char       **replyBuffer,	/* Output buffer		*/\
    OUT msgLENGTH  *replySize,		/* Output buffer size		*/\
    OUT ccsERROR   *error		/* Returned error structure	*/\
    )									\
{									\
    tooINTERFACE_VERBOSE("ENTER: " #function " " argfmt " ", ##args);	\
    strcpy(msg->body, "OK"); /* default reply */			\
    if (function (##args) == FAILURE)					\
	{								\
	tooINTERFACE_VERBOSE("EXIT: " #function " ==> FAILURE");	\
	return FAILURE;							\
	}								\
    else								\
	{								\
	*replyBuffer = msg->body;					\
	*replySize = strlen(msg->body) + 1;				\
	tooINTERFACE_VERBOSE("EXIT: " #function				\
			     " ==> SUCCESS, %d chars in reply",		\
			     *replySize);				\
	return SUCCESS;							\
	}								\
}

/* interface function with no arguments */
#define tooINTERFACE_NOARG(function)				\
tooINTERFACE(function, "", msg, error);

/* interface function with one scalar argument */
#define tooINTERFACE_ARG(function, argtype, argfmt)		\
tooINTERFACE(function, argfmt, *(argtype *)parameters, msg, error);

/* interface function with two scalar arguments */
#define tooINTERFACE_ARG_ARG(function, argtype1, argtype2, argfmt)	\
tooINTERFACE(function, argfmt,						\
	     *(argtype1 *)parameters,					\
	     *(argtype2 *)(parameters + sizeof(argtype1)),		\
	     msg, error);

/* interface function with one string argument */
#define tooINTERFACE_STR(function)				\
tooINTERFACE(function,"\"%s\"", parameters, msg, error);

/* interface function with two string arguments */
#define tooINTERFACE_STR_STR(function)				\
tooINTERFACE(function,"\"%s\",\"%s\"",				\
	     parameters,					\
	     parameters + strlen(parameters) + 1,		\
	     msg, error);

/* interface function with one string and one scalar argument */
#define tooINTERFACE_STR_ARG(function, argtype, argfmt)		\
tooINTERFACE(function,"\"%s\"," argfmt, parameters,		\
	     *(argtype *)(parameters + strlen(parameters) + 1),	\
	     msg, error);

/* interface function with two strings and one scalar argument */
#define tooINTERFACE_STR_STR_ARG(function, argtype, argfmt)	\
tooINTERFACE(function,"\"%s\",\"%s\"," argfmt,			\
	     parameters,					\
	     parameters + strlen(parameters) + 1,		\
	     *(argtype *)(parameters + strlen(parameters) + 1	\
			  + strlen(parameters + strlen(parameters) + 1) + 1),\
	     msg, error);

/* interface function with one string and two scalar arguments */
#define tooINTERFACE_STR_ARG_ARG(function, argtype1, argtype2, argfmt)	\
tooINTERFACE(function,"\"%s\"," argfmt, parameters,			\
	     *(argtype1 *)(parameters + strlen(parameters) + 1),	\
	     *(argtype2 *)(parameters + strlen(parameters) + 1 +	\
			   sizeof(argtype1)),				\
	     msg, error); 

/*
 * Command Interpreter prototypes
 */
#define tooLCU_INTERFACE(fct) \
ccsCOMPL_STAT Ilcu ## fct ( \
    IN  const char   *parameters,       /* Formatted input parameters   */ \
    IN  msgMSG       *msg,              /* Message body used for reply  */ \
    OUT char         **replyBuffer,     /* Output buffer                */ \
    OUT msgLENGTH    *replyBufferSize,  /* Output buffer size           */ \
    OUT ccsERROR     *error )           /* Returned error structure     */

tooLCU_INTERFACE(SetVerbose);
tooLCU_INTERFACE(GetVerbose);
tooLCU_INTERFACE(SetDebug);
tooLCU_INTERFACE(GetDebug);
tooLCU_INTERFACE(SetTrace);
tooLCU_INTERFACE(GetTrace);

#ifdef __cplusplus
}
#endif

#endif 	/* !tooInterface_H */
