!
! "@(#) $Id: ron.prg,v 2.83 2005/09/30 08:26:50 vltsccm Exp $"
!
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.NAME:
! MIDAS procedure RON.PRG
!
!.PURPOSE: 
! Obtain crude measure of CCD read noise; plot noise histogram
!
!.APPLICABILITY: 
! Any pair of overlapping bias frames 
!
!.METHOD:
! The two bias frames are subtracted from one another. The region where 
! they overlap is devided into subx x suby subframes of identical size. 
! Only 1 every skipx (in x) and skipy (in y) subframes is analysed. In each 
! such subframe, the rms is computed (quantity 1). Furthermore, in both 
! input frames the mean is computed for each subframe region (quantities 2 
! and 3). The results are written to table RON. After kappa*sigma clipping
! on the rms values, mean and standard deviation are computed for each of 
! the 3 quantities named. They are output to the MIDAS session terminal. 
! The difference frame is displayed with +-5 sigma cuts; the local RON values
! appear as labels. The histogram of the difference frame is plotted; a 
! gaussian with the same FWHM, position, and peak height is overplotted 
! (dotted) for comparison. 
!
! (The purpose of this overhead is to eliminate the effect of particle
! hits without resorting to conventional filter techniques.) 
!
!.ASSUMPTIONS:  
! Frames overlap in a region of at least (5 x subx) x (5 x suby) pixels. An 
! error message is issued if this not the case. 
!
!.NOTES: 
! a) The plotted histogram is too broad by a factor of sqrt(2) because it 
!    is produced from the difference diagram. However, the results output
!    to the terminal have been corrected for this. 
! b) Because of the extensive usage of the STAT/IMA command, the procedure
!    is quite slow (up to a few minutes), especially for big frames!
!
!.RESTRICTIONS: 
! None known at time of writing (except for required frame overlap)
!
!.KNOWN ANOMALIES:
! For relatively noisy histograms, the computed Gaussian (overplotted as 
! a dotted line) can be substantially off in one or more of center position, 
! FWHM, and peak height. This is due to an instability in command 
! CENTER/GAUSS. With large frames this does not happen (good statistics)
! and in any case AFFECTS THE GRAPHICAL OUTPUT ONLY. THE NUMBERS OUTPUT TO 
! THE SESSION TERMINAL ARE CORRECT. 
!
!.TESTS MADE:
! Procedure was tested with synthetic data and EMMI/SUSI and ESO 1.5m
! + Boller & Chivens bias frames. 
!
!.CALLING SEQUENCE:
! @@ ron file1 file1 [subx,suby] [skipx,skipy]
! where:
! file1 and file 2 are the names of the two bias frames to be analysed.
! subx, suby are the number of subframes in x and y, respectively 
! skipx,skipy are the skipping factors in x,y, i.e. only 1 subframe 
! every skipx (skipy) subframes is analysed. 
!
!.INPUT 
! Names of two bias frames, number of subframes (in x and y), 
! skipping factors in x and y. 
! File size is taken from descriptors of difference file. 
!
!.OUTPUT
! To MIDAS session window. 
!
!.Author:
! D. Baade, ESO
!
!.VERSIONS:
!  D. Baade    1.0    05-Jun-1995:   created
!  D. Baade    1.1    24-Jun-1995:   improved subdivision scheme+cosmetics
!  D. Baade    1.2    25-Jun-1995:   fixed bug in partition scheme 
!
!----------------------------------------------------------------------------
!
defi/para p1 ? C "Enter name of first bias frame: "
defi/para p2 ? C "Enter name of second bias frame: "
defi/para p3 7,7 N "Enter number of subdivisions in x,y: " 1,100
defi/para p4 1,1 N "Enter skipping factors in x,y: " 1,100
!
! Save input parameters in local keywords 
defi/loc file1/c/1/20 {p1}
defi/loc file2/c/1/20 {p2}
defi/loc sub/i/1/2 {p3} 
defi/loc skip/i/1/2 {p4} 
!
! check parameters further
if {skip(1)} .ge. {sub(1)} then
   write/out "Input error: Number of partitions to be skipped in X too large."
   goto exit
endif
if {skip(2)} .ge. {sub(2)} then
   write/out "Input error: Number of partitions to be skipped in Y too large." 
   goto exit
endif
!
! Define remaining variables
defi/loc i/i/1/1 0
defi/loc j/i/1/1 0
defi/loc xgrid/i/1/1 0
defi/loc ygrid/i/1/1 0
defi/loc xrange/i/1/2 1,0
defi/loc yrange/i/1/2 1,0
defi/loc mean/r/1/1 0
defi/loc mean1/r/1/1 0
defi/loc smean1/r/1/1 0
defi/loc mean2/r/1/1 0
defi/loc smean2/r/1/1 0
defi/loc sigma/r/1/1 0
defi/loc ssigma/r/1/1 0
defi/loc cut/r/1/2 0,0
defi/loc row/i/1/1 0
defi/loc peak/r/1/1 0
defi/loc limit/r/1/1 0
defi/loc select/i/1/1 0
defi/loc xy/r/1/3 0,0,0
defi/loc frac/r/1/1 0
defi/loc compl/i/1/1 20
defi/loc nx/i/1/1 0
defi/loc ny/i/1/1 0
defi/loc total/i/1/1 0
defi/loc icount/i/1/1 0
defi/loc jcount/i/1/1 0
defi/loc half/i/1/1 0
defi/loc x/r/1/1 0
defi/loc y/r/1/1 0
defi/loc iscale/r/1/2 0,0
defi/loc end/r/1/2 0,0
!
write/out " "
write/out "  >>> This procedure is very I/O (and CPU) intensive: have a coffee! <<<"
write/out " "
!
! Messages off
log(4) = 1
!
set/format i4 f12.2
crea/grap            ! in case graphics window doesn't exist yet
set/grap bin=on
!
! Create table to store results
nx = (sub(1)-1)/skip(1)+1
ny = (sub(2)-1)/skip(2)+1
total = nx*ny
log(4) = 0
write/out "Will examine {total} subframes:"
write/out " "
log(4) = 1
create/tab ron 5 {total} null
create/col ron :mean1  ! mean bias in frame1
create/col ron :mean2  ! mean bias in frame2
create/col ron :sigma  ! mean ron
create/col ron :x      ! x position of subframes
create/col ron :y      ! y position of subframes
!
! Compute difference of frames, extract overlap region from input frames
comp/ima &z = {file1}-{file2}
end(1) = {&z,start(1)}+{&z,step(1)}*({&z,npix(1)}-1)
end(2) = {&z,start(2)}+{&z,step(2)}*({&z,npix(2)}-1)
extra/ima &v = {file1}[{&z,start(1)},{&z,start(2)}:{end(1)},{end(2)}]
extra/ima &w = {file2}[{&z,start(1)},{&z,start(2)}:{end(1)},{end(2)}]
!
file1 = "&v"
file2 = "&w" 
!
! Define grid of sub(1) x sub(2) equally-sized subframes
! Check that size is sufficient for minimal statistics
xgrid = {&z,npix(1)}
xgrid = xgrid/sub(1)
if {xgrid} .le. 5 then
   log(4) = 0
   write/out "Overlap of frames too small in X."
   log(4) = 1
   goto exit 
endif
ygrid = {&z,npix(2)}
ygrid = ygrid/sub(2)
if {ygrid} .le. 5 then
   log(4) = 0
   write/out "Overlap of frames too small in Y."
   log(4) = 1
   goto exit 
endif
!
xrange(1) = 1
xrange(2) = xrange(1)-1+xgrid
yrange(1) = 1
yrange(2) = yrange(1)-1+ygrid
!
! do statistics on subframes, store results in table 
do i = 1 {sub(1)} {skip(1)}
  icount = icount+1
  yrange(1) = 1
  yrange(2) = yrange(1)-1+ygrid
  do j = 1 {sub(2)} {skip(2)}
     jcount = jcount+1
     row = (icount-1)*ny+jcount     
     x = ({xrange(1)}+{xrange(2)})/2.
     y = ({yrange(1)}+{yrange(2)})/2.
     ron,:x,{row} = {x}
     ron,:y,{row} = {y}
     stat/ima &z [@{xrange(1)},@{yrange(1)}:@{xrange(2)},@{yrange(2)}] ? ? 2
     ron,:sigma,{row} = {outputr(4)}
     stat/ima {file1} [@{xrange(1)},@{yrange(1)}:@{xrange(2)},@{yrange(2)}] ? ? 2
     ron,:mean1,{row} = {outputr(3)}
     stat/ima {file2} [@{xrange(1)},@{yrange(1)}:@{xrange(2)},@{yrange(2)}] ? ? 2
     ron,:mean2,{row} = {outputr(3)}
     yrange(1) = yrange(1)+skip(2)*ygrid
     yrange(2) = yrange(2)+skip(2)*ygrid
!
! progress messages
     comp/key frac = row*99/total
     if {frac} .gt. {compl} then
        set/format i2
        comp/key outputc = m$time()
        log(4) = 0
        write/out "{outputc(19:26)}    {compl}% of statistics completed."
        log(4) = 1
        compl = compl+20
        set/format i4
     endif
  enddo
  jcount = 0
  xrange(1) = xrange(1)+skip(1)*xgrid
  xrange(2) = xrange(2)+skip(1)*xgrid 
enddo
!
outputc = m$time()
log(4) = 0
write/out "{outputc(19:26)}   100% of statistics completed; now analyzing results."
log(4) = 1
!  
! do statistics on table with results for subframes,
! apply kappa*sigma clipping. 
! note that in the first iteration the mean is computed 
! of the 50% lowest subframes only.
comp/tab ron :sigma = :sigma/(sqrt(2)) 
comp/tab ron :orig_seq = seq
sort/tab ron :sigma
half = total/2. 
sele/tab ron seq.le.{half}
do i = 1 3
   stat/tab ron :sigma
   sigma = outputr(3)
   ssigma = outputr(4)
   limit = sigma+3*ssigma
   sele/tab ron :sigma.lt.{limit}
enddo
stat/tab ron :mean1
select = outputi(2)
mean1 = outputr(3)
smean1 = outputr(4)
stat/tab ron :mean2
mean2 = outputr(3)
smean2 = outputr(4)
!
! compute and plot full histogram
mean = mean1-mean2
cut(1) = mean-5*sigma
cut(2) = mean+5*sigma
stat/ima &z ? 1 {cut(1)},{cut(2)} FN
copy/dima &z histogram &y 
outputr(1) = cut(1)-1 
write/desc &y start/d/1/1 {outputr(1)}
center/gauss &y 
outputr(12) = outputr(12)/2.3548   ! convert from fwhm to sigma 
peak = outputr(14)
set/grap ltype=1
outputr(14) = outputr(14)*1.2
cut &y 0,{outputr(14)}
plot/axes {cut(1)},{cut(2)},5,1 0,{outputr(14)},100000,20000 ? "Pixel value" "Number of pixels" 
overplot &y
crea/ima &x 1,{&y,npix(1)} {&y,start(1)},{&y,step(1)} gauss {outputr(5)},{outputr(12)}
comp/ima &x = &x*10000
stat/ima &x 
comp/ima &x = &x*{peak}/{outputr(2)}
set/grap ltype=2
over/row &x 
xy(1) = cut(1) +0.2*sigma
xy(2) = peak*1.13
xy(3) = cut(1)+0.85*sigma
over/line 1 {xy(1)},{xy(2)} {xy(3)},{xy(2)} 
xy(1) = xy(3)+0.5*sigma
label/grap "Histogram of difference frame" {xy(1)},{xy(2)} ? 0.8 1
xy(1) = cut(1)+0.2*sigma
xy(2) = peak*1.07
xy(3) = cut(1)+0.85*sigma
over/line 2 {xy(1)},{xy(2)} {xy(3)},{xy(2)} 
xy(1) = xy(3)+0.5*sigma
label/grap "Gaussian of same FWHM, position, and height" {xy(1)},{xy(2)} ? 0.8 1 
!
! display difference frame:
clea/ch over
iscale(1) = {&z,npix(1)}/ididev(2)
iscale(2) = {&z,npix(2)}/ididev(3)
load/ima &z cuts={cut(1)},{cut(2)} scale=-{iscale(1)},-{iscale(2)}
clea/ch over
comp/tab ron :x = :x+{&z,start(1)}-1
comp/tab ron :y = :y+{&z,start(2)}-1
sele/tab ron :sigma.lt.{limit}
load/tab ron :x :y :sigma
xy(1) = (ididev(2)-350)/2
xy(2) = ididev(3)/10
label/disp "Difference between the two bias frames" {xy(2)},{xy(1)} ? white 1
!
! output results to terminal
log(4) = 0
write/out " "
write/out "Mean bias level (1st frame): {mean1}  (+- {smean1}) ADU's."
write/out "Mean bias level (2nd frame): {mean2}  (+- {smean2}) ADU's."
write/out "Mean read noise:               {sigma}  (+- {ssigma}) ADU's."
set/format i3
write/out "(Statistics based on {select} of {total} subframes analyzed.)"
write/out " "
!
exit:
set/grap ltype=1
set/format
log(4) = 0


